
/* associated_documents_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "associated_comp_documents_control.h"
#include "associated_comp_documents_model.h"
#include "util.h"
#include "parameter_manager.h"
#include "control_util.h"
#include "company.h"
#include "company_model.h"
#include "document_model.h"

#define C_LEGAL_ID_COLUMN    0
#define C_NAME_COLUMN 1

static GtkWidget *associated_companies=NULL;
static GladeXML *xml = NULL;
static gchar dialog_file[]="/src/interface/glade/documents_view.glade";
static gchar widget_name[]="asociados-documentos";

static Document *document_loaded_associated = NULL;
static Company  **companies_loaded = NULL;
static void on_associated_companies_button_aceptar_clicked(GtkButton *button,gpointer data);
static void on_associated_companies_button_cancel_clicked(GtkButton *button,gpointer data);
static void close_associated_companies_window(GtkWidget *widget,gpointer user_data);

static void clean_screen(void);
static void set_all_companies(void);
static void set_associated_companies(Document *document);
static void fill_associated_companies_list(GtkCList *list, Company **companies);


GtkWidget *c_associated_companies_open (GtkWidget *app1, GtkWidget *vbox, Document *document) {

    gchar *directory = g_strconcat(AA_DATA_DIR,dialog_file,NULL);
    gchar *name      = d_document_get_name (document);
    GtkWidget *dialog;

    xml = glade_xml_new (directory, widget_name);
    dialog = glade_xml_get_widget (xml, widget_name);

    associated_companies = dialog;

    glade_xml_signal_connect (xml, "on_button-aceptar-asociados_clicked",
			      GTK_SIGNAL_FUNC (on_associated_companies_button_aceptar_clicked));
    glade_xml_signal_connect (xml, "on_button-cancelar-asociados_clicked",
			      GTK_SIGNAL_FUNC (on_associated_companies_button_cancel_clicked));
    glade_xml_signal_connect (xml, "on_asociados-documentos_destroy_event",
			      GTK_SIGNAL_FUNC (close_associated_companies_window));
    clean_screen();                            

    document_loaded_associated = d_document_duplicate (document);
    if (name!=NULL) {
        g_strstrip(name);
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry-documento-asociados")), name);
	g_free(name);

    }

    set_all_companies();
    set_associated_companies(document);

    g_free(directory);
    return associated_companies;

}

static void set_all_companies() {
    GtkCList *list      = GTK_CLIST(glade_xml_get_widget(xml,"clist-compaia-asociados"));
    if (companies_loaded == NULL) {
	companies_loaded = m_company_find_by_code (ULONG_MAX);
    }

    fill_associated_companies_list(list, companies_loaded);
}

static void set_associated_companies(Document *document) {
    Company **associated = NULL;
    GtkCList *list       = GTK_CLIST(glade_xml_get_widget(xml,"clist-compaia-asociados"));
    gint    index = 0;
    gint    associated_index = 0;
    gulong  code, associated_code;
    GtkWidget * window;
    GdkCursor * cursor;

    window = glade_xml_get_widget (xml, widget_name);
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    associated = m_document_find_associated_companies (document);

    if ((companies_loaded!=NULL) && (associated!=NULL)) {
	while((companies_loaded[index]!=NULL)&&(associated[associated_index]!=NULL)) {
            code = d_participant_get_code (d_company_get_participant (companies_loaded[index]));
	    associated_code =
		d_participant_get_code (d_company_get_participant (associated[associated_index]));

	    if(code==associated_code) {
                gtk_clist_select_row(list,index,0);
		associated_index++;
                index  = 0;
	    } else {
		index++;
	    }
	}
    }

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    m_company_clear_array (associated);
    associated = NULL;
}

static void close_associated_companies_window(GtkWidget *widget,gpointer user_data) {
    clean_screen();
    gtk_widget_destroy(associated_companies);
}

static void fill_associated_companies_list(GtkCList *list, Company **companies) {
    gint i=0;
    gchar * temp;

    gchar *empty_row[]={"",""};


    while(companies[i]!=NULL) {
	if (i==30) gtk_clist_freeze ((GtkCList *) list);

	gtk_clist_append ((GtkCList *) list,empty_row);

	temp = d_participant_get_legal_id (d_company_get_participant (companies[i]));
	gtk_clist_set_text((GtkCList *) list, i, C_LEGAL_ID_COLUMN, temp);
	g_free(temp);

	temp = d_participant_get_nick (d_company_get_participant (companies[i]));
	gtk_clist_set_text((GtkCList *) list, i, C_NAME_COLUMN, temp);
	g_free(temp);
        i++;
    }

    gtk_clist_thaw ((GtkCList *) list);


}

static void clean_screen() {

    GtkWidget *vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    GtkWidget *list = glade_xml_get_widget(xml,"clist-compaia-asociados");

    gtk_clist_clear ((GtkCList *) list);

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
    d_document_clear (document_loaded_associated);
    document_loaded_associated = NULL;

}

static void obtain_company_ids(GArray *selected, Company **companies, gulong *ids) {

    gint index=0;
    gint company_index=0;

    while (index<selected->len) {
	company_index = g_array_index(selected,gint,index);

	ids[index] = d_participant_get_code (d_company_get_participant (companies[company_index]));
	index++;
    }

    ids[index] = ULONG_MAX;

}

static void on_associated_companies_button_aceptar_clicked(GtkButton *button, gpointer data) {
    GtkCList  *list = GTK_CLIST(glade_xml_get_widget(xml,"clist-compaia-asociados"));
    GArray    *selected = c_util_get_selected_rows_from_clist (list);
    gulong    *company_ids = NULL;
    gint       rows;

    rows = selected->len;
    company_ids = (gulong *)g_malloc(sizeof(gulong *)*(rows+1));
    obtain_company_ids(selected, companies_loaded, company_ids);

    m_document_update_associated_companies (document_loaded_associated, company_ids);

    close_associated_companies_window(associated_companies, data);

    g_array_free(selected, FALSE);
    g_free(company_ids);
    company_ids = NULL;
}

static void on_associated_companies_button_cancel_clicked(GtkButton *button, gpointer data) {
    clean_screen();
    close_associated_companies_window(associated_companies, data);
}
            
