
/* charge_invoice_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#define C_DELIVERY_PAYMENT_COLUMN 0
#define C_ORDER_CODE_COLUMN 1
#define C_EUROCODE_COLUMN 2
#define C_DESCRIPTION_COLUMN 3
#define C_QUANTITY_COLUMN 4
#define C_VAT_COLUMN 5
#define C_PRICE_COLUMN 6

#include "parameter_manager.h"

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"


#include "piece.h"
#include "supplier.h"
#include "supplier_model.h"
#include "accounting_period_model.h"
#include "shop_model.h"
#include "enterprise_model.h"
#include "control_util.h"
#include "charge_invoice_line.h"
#include "charge_invoice_control.h"
#include "charge_invoice_line_model.h"
#include "charge_invoice.h"
#include "find_order_control.h"
#include "payment_control.h"
#include "insert_cash_operation_control.h"
#include "insert_charge_invoice_line.h"

static GtkWidget *charge_invoices=NULL;
static gboolean maintain = TRUE;
static GladeXML *xml;
static gchar widget_name[]="vbox_factura_cargo";
static gchar url[]="/src/interface/glade/invoice_view.glade";
static ChargeInvoice * charge_invoice = NULL;
static Payment * payment_to_invoice=NULL;
static GList *charge_lines = NULL;

static void clean_screen(void);
static void on_clean_charge_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_modify_charge_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_save_charge_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_manage_charge_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_dissasociate_charge_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_delete_charge_invoice_button_clicked(GtkButton *button, gpointer data);
static void on_shop_combo_entry_changed(GtkEntry *entry, gpointer user_data);
static void on_payment_checkbutton_toggled(GtkToggleButton *togglebutton,
					gpointer user_data);
static void on_vat_checkbutom_toggled(GtkToggleButton *togglebutton,
						     gpointer user_data);
static void fill_supplier_combo(void);
static void fill_shop_combo(void);
static void delete_invoice(gint reply,
			   gpointer user_data);
static void save_charge_invoice(void);
static gint create_invoice_from_screen(gboolean modify);
static void fill_charge_invoice_data(ChargeInvoice * charge);
static void fill_combos_with_names(gchar * shopName,gchar * supplierName);
static void fill_entries_with_charge_invoice(gchar* invoice_number,time_t * fecha,gdouble total,gchar * concepto);
static void set_type_payment_toggle_button_state(gboolean associated_payment,gboolean paid,gboolean wihtout_vat);
static void inicialize_sensitive(gboolean note,gboolean new_case);
static void inicialize_shop_combo(void);
static void cancel_payment(gpointer value);
static void on_add_charge_line_button_clicked(GtkButton *button, gpointer data);
static void on_delete_charge_line_button_clicked(GtkButton *button, gpointer data);
static void on_modify_charge_line_button_clicked(GtkButton *button, gpointer data);
static void on_add_order_line_button_clicked(GtkButton *button, gpointer data);
static void update_charge_invoice(gint reply,
			       gpointer user_data);
static void check_charge_invoice(ChargeInvoice * charge_invoice,ChargeInvoiceLine ** charge_invoice_lines);
GtkWidget *c_charge_invoice_open (GtkWidget *app1, GtkWidget *vbox)
{

    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (charge_invoices==NULL)
    {
      GtkWidget *dialog;
      gchar *s_name = NULL;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_button_limpiar_factura_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_clean_charge_invoice_button_clicked));
	glade_xml_signal_connect (xml, "on_button_guardar_factura_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_save_charge_invoice_button_clicked));
	glade_xml_signal_connect (xml, "on_button_gestionar_factura_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_manage_charge_invoice_button_clicked));
	glade_xml_signal_connect (xml, "on_button_desasociar_factura_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_dissasociate_charge_invoice_button_clicked));
	glade_xml_signal_connect (xml, "on_combo_entry_tienda_factura_cargo_changed",
				  GTK_SIGNAL_FUNC (on_shop_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_checkbutton_pago_toggled",
				  GTK_SIGNAL_FUNC (on_payment_checkbutton_toggled));
	glade_xml_signal_connect (xml, "on_checkbutton_iva_factura_cargo_toggled",
				  GTK_SIGNAL_FUNC (on_vat_checkbutom_toggled));
	glade_xml_signal_connect (xml, "on_button_modificar_factura_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_modify_charge_invoice_button_clicked));
	glade_xml_signal_connect (xml, "on_anadir_linea_cargo_button_clicked",
				  GTK_SIGNAL_FUNC (on_add_charge_line_button_clicked));
	glade_xml_signal_connect (xml, "on_eliminar_button_clicked",
				  GTK_SIGNAL_FUNC (on_delete_charge_line_button_clicked));
	glade_xml_signal_connect (xml, "on_modificar_button_clicked",
				  GTK_SIGNAL_FUNC (on_modify_charge_line_button_clicked));
	glade_xml_signal_connect (xml, "on_anadir_linea_pedido_button_clicked",
				  GTK_SIGNAL_FUNC (on_add_order_line_button_clicked));
	glade_xml_signal_connect (xml, "on_button_borrar_factura_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_delete_charge_invoice_button_clicked));


#ifdef IS_SHOP
	gtk_widget_hide(glade_xml_get_widget (xml, "anadir_linea_pedido_button"));
	gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo_tienda_factura_cargo"),FALSE);
	gtk_widget_set_sensitive(glade_xml_get_widget (xml, "entry_empresa_factura_cargo"),FALSE);
#endif

	fill_shop_combo();

	s_name = u_parameter_manager_get_parameter ("tienda");
	if (s_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_tienda_factura_cargo")), s_name);
	}
	g_free(s_name);

	fill_supplier_combo();
	charge_invoices=dialog;

    } 

//    inicialize_sensitive(FALSE,TRUE);


    return charge_invoices;

}

void load_charge_invoice(ChargeInvoiceId * chargeInvoiceId,gboolean with_vat) {
    Supplier *supplier = NULL; 
    gulong supplier_code_gulong = ULONG_MAX; 
    gchar *invoice_number = NULL;
    gchar *supplier_code = NULL;
    ChargeInvoiceLine **charge_invoice_lines = NULL;

    if (charge_invoice!=NULL) {
      d_charge_invoice_clear (charge_invoice);
      charge_lines = NULL;
      charge_invoice=NULL;
      payment_to_invoice=NULL;
    } else {
      if (payment_to_invoice!=NULL) {
	d_payment_clear (payment_to_invoice);
	payment_to_invoice=NULL;
      }
    }

    maintain =FALSE;
    clean_screen();
    maintain = TRUE;
    inicialize_shop_combo();

    charge_invoice = m_charge_invoice_find_by_code (chargeInvoiceId,with_vat);

    

    d_charge_invoice_set_vat (charge_invoice,with_vat);   
    inicialize_sensitive(!with_vat,FALSE);

    fill_charge_invoice_data(charge_invoice);

    supplier = d_charge_invoice_get_supplier (charge_invoice);
    if (supplier != NULL) {
      supplier_code_gulong = d_participant_get_code (d_supplier_get_participant (supplier));
      invoice_number = g_strdup_printf("%lu",d_charge_invoice_get_code (charge_invoice));
      
      if (supplier_code_gulong != ULONG_MAX) supplier_code = g_strdup_printf("%lu", supplier_code_gulong);

      charge_invoice_lines = m_charge_invoice_line_find_by_invoice (supplier_code, invoice_number);
    
      d_charge_invoice_set_charge_invoice_lines (charge_invoice, charge_invoice_lines); 


      m_charge_invoice_line_convert_to_glist (&charge_lines, charge_invoice_lines);
      
      c_charge_invoice_load_charge_lines_from_list ();


      if (supplier_code != NULL) g_free(supplier_code);

      g_free(invoice_number);

      check_charge_invoice(charge_invoice,charge_invoice_lines);

    }

}

static void check_charge_invoice(ChargeInvoice * charge_invoice,ChargeInvoiceLine ** charge_invoice_lines) {
    gdouble total_lines;
    gchar * total_lines_char;
    gchar * invoice_total_char;

    total_lines = d_charge_invoice_line_calculate_total_by_vat (charge_invoice_lines);

    total_lines_char = g_strdup_printf("%.2f",total_lines);
    g_print(_("Lines total: %s.\n"),total_lines_char);
    invoice_total_char = g_strdup_printf("%.2f",d_charge_invoice_get_total (charge_invoice));

    if (strcmp(total_lines_char,invoice_total_char)) {
	gpointer total_to_question;
	gchar * message = g_strconcat(_("Total obtained from lines don't aggre"),
				      _("with total invoice\n       * Total by lines = "),total_lines_char,
				      _("\n       * Total invoice = "),invoice_total_char,
				      _("\nDo you want to update total invoice ?"),NULL);
				      
	total_to_question = (gpointer) &total_lines;
	c_util_open_question_dialog_with_param (message, (*update_charge_invoice),total_to_question);
	g_free(message);
    }
    g_free(total_lines_char);
    g_free(invoice_total_char);

}
static void update_charge_invoice(gint reply,
				  gpointer user_data) {
    gdouble total = *(gdouble *)user_data;
    if (!reply) {
	GtkWidget * wid = glade_xml_get_widget(xml,"entry_total_factura_cargo");
	gchar * total_char=g_strdup_printf("%.2f",total);
        gboolean no_error;
	d_charge_invoice_set_total (charge_invoice,total);
	no_error = m_charge_invoice_update (charge_invoice,FALSE);
	gtk_entry_set_text(GTK_ENTRY(wid),total_char);
	g_free(total_char);
	if (no_error==FALSE)
	    c_util_open_error_window (_("Data not modified"));
    }

}

static void fill_charge_invoice_data(ChargeInvoice * charge) {
    gchar * shopName = d_participant_get_name (d_shop_get_participant (d_charge_invoice_get_shop (charge)));
    gchar * supplierName =d_participant_get_nick (d_supplier_get_participant (d_charge_invoice_get_supplier (charge)));
    time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
    gchar * concepto;
    gchar * temp_invoice;
    gboolean associated_payment;
    gboolean paid;
    gboolean wihtout_vat;

    *time_aux = d_charge_invoice_get_date (charge);
    charge_invoice = charge;
    concepto = d_charge_invoice_get_section (charge);
    fill_combos_with_names(shopName,supplierName);

    temp_invoice = d_charge_invoice_get_invoice_number (charge);
    fill_entries_with_charge_invoice(temp_invoice,time_aux,
				     d_charge_invoice_get_total (charge),
				     concepto);

    wihtout_vat = (!d_charge_invoice_get_vat (charge));
    if (wihtout_vat==FALSE) {
	associated_payment=FALSE;
	maintain=FALSE;
	paid = (d_payment_get_achieved_payments (d_charge_invoice_get_payment (charge))==
		  d_payment_get_number_expiries (d_charge_invoice_get_payment (charge)));
	if ((d_payment_get_payment_code (d_charge_invoice_get_payment (charge))!=ULONG_MAX) &&
	    (d_payment_get_payment_code (d_charge_invoice_get_payment (charge))!=0)) {
	    associated_payment=TRUE;
	    maintain=TRUE;
	} else {
	    associated_payment=FALSE;
	    maintain=FALSE;
	}
    } else {
	associated_payment=FALSE;
	maintain=FALSE;
	paid = TRUE;
    }
    set_type_payment_toggle_button_state(associated_payment,paid,wihtout_vat);


    g_free(time_aux);
    g_free(concepto);
    g_free(shopName);
    g_free(supplierName);
}

static void fill_combos_with_names(gchar * shop_name,gchar * supplier_name) {
    GtkWidget * entry_shop = glade_xml_get_widget(xml,"combo_entry_tienda_factura_cargo");
    GtkWidget * entry_supplier =glade_xml_get_widget(xml,"combo_entry_proveedor_factura_cargo");
    gchar * shop_aux = g_strdup(shop_name);
    gchar * supplier_aux = g_strdup(supplier_name);
    gtk_entry_set_text(GTK_ENTRY(entry_shop),shop_aux);
    gtk_entry_set_text(GTK_ENTRY(entry_supplier),supplier_aux);
    g_free(shop_aux);
    g_free(supplier_aux);
}

static void set_type_payment_toggle_button_state(gboolean associated_payment,gboolean paid,gboolean without_vat) {
    GtkWidget * payment_checkbutton = glade_xml_get_widget(xml,"checkbutton_pago");
    GtkWidget * disassociate_button = glade_xml_get_widget(xml,"button_desasociar_factura_cargo");
    GtkWidget * paid_button = glade_xml_get_widget(xml,"checkbutton_estado_factura_cargo");
    GtkWidget * vat_button = glade_xml_get_widget(xml,"checkbutton_iva_factura_cargo");

    if (associated_payment!=FALSE) {
	c_charge_invoice_type_payment_toggle_button_change_state (TRUE);
	if (GTK_WIDGET_SENSITIVE(payment_checkbutton)) gtk_widget_set_sensitive(payment_checkbutton,FALSE);
	if (GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,FALSE);
    } else {
	if (without_vat!=FALSE) {
	    if (GTK_WIDGET_SENSITIVE(payment_checkbutton)) gtk_widget_set_sensitive(payment_checkbutton,FALSE);
	    if (GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,FALSE);
	    c_charge_invoice_type_payment_toggle_button_change_state (FALSE);
	} else {
	    if (!GTK_WIDGET_SENSITIVE(payment_checkbutton)) gtk_widget_set_sensitive(payment_checkbutton,TRUE);
	    if (!GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,TRUE);
	}
    }
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(paid_button),paid);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(vat_button),without_vat);
}

static void fill_entries_with_charge_invoice(gchar * invoice_number,time_t * fecha,gdouble total,gchar * concepto) {
    GtkWidget * spinbutton_numero = glade_xml_get_widget(xml,"spinbutton_numero_factura_factura_cargo");
    GtkWidget * codigo = glade_xml_get_widget(xml,"entry_codigo_factura");
    GtkWidget * dateedit_fecha=glade_xml_get_widget(xml,"dateedit_fecha_factura_cargo");
    GtkWidget * total_charge_invoce_entry = glade_xml_get_widget(xml,"entry_total_factura_cargo");
    GtkWidget * text_concepto =glade_xml_get_widget(xml,"text_concepto_factura_cargo");
    gint *position = (gint *) g_malloc0(sizeof(gint));
    gint len=0;
    gchar *total_char = g_strdup_printf("%.2f",total);
    gchar *invoice_number_char = g_strdup(invoice_number);
    gchar *codigo_factura_char = g_strdup_printf("%lu",d_charge_invoice_get_code (charge_invoice));

    *position = 0;
    if (concepto!=NULL) {
	len = strlen(concepto);
	gtk_editable_insert_text (GTK_EDITABLE(text_concepto), concepto, len, position);
    }
    g_free(position);

    gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_fecha),(*fecha));
    gtk_entry_set_text(GTK_ENTRY(spinbutton_numero),invoice_number_char);
    gtk_entry_set_text(GTK_ENTRY(total_charge_invoce_entry),total_char);
    gtk_entry_set_text(GTK_ENTRY(codigo),codigo_factura_char);
    g_free(total_char);
    g_free(invoice_number_char);
}

static void fill_supplier_combo() {
    GtkWidget * combo;
    Supplier ** suppliers;

    combo = glade_xml_get_widget (xml, "combo_proveedor_factura_cargo");

    suppliers = m_supplier_find_all ();

    c_util_fill_combo_with_suppliers (combo,suppliers);

}

static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;

    combo = glade_xml_get_widget (xml, "combo_tienda_factura_cargo");

    shops = m_shop_find_all ();

    c_util_fill_combo_with_shops (combo,shops);

}

static void on_clean_charge_invoice_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * button_aceptar = glade_xml_get_widget(xml,"button_guardar_factura_cargo");
    if (charge_invoice!=NULL) {
	d_charge_invoice_clear (charge_invoice);
	charge_lines = NULL;
	charge_invoice=NULL;
	payment_to_invoice=NULL;
    } else {
	if (payment_to_invoice!=NULL) {
	    d_payment_clear (payment_to_invoice);
	    payment_to_invoice=NULL;
	    charge_invoice=NULL;
	}
    }

    clean_screen();
    inicialize_sensitive(FALSE,TRUE);
    gtk_widget_set_sensitive(button_aceptar,TRUE);
    c_charge_invoice_type_payment_toggle_button_change_state (FALSE);
    inicialize_shop_combo();

}

static void on_save_charge_invoice_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * open_cash_operation=NULL;
    GtkWidget * payment_checkbutton=glade_xml_get_widget(xml,"checkbutton_pago");
    GtkWidget * checkbutton_iva=glade_xml_get_widget(xml,"checkbutton_iva_factura_cargo");
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    gboolean associated_payment=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(payment_checkbutton));
    gboolean save_without_vat=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(checkbutton_iva));    
    
    if ((associated_payment!=FALSE) && (save_without_vat==FALSE)) {
	save_charge_invoice();
//	c_order_search();
	if (charge_invoice!=NULL) {
	    if (d_charge_invoice_get_vat (charge_invoice)==FALSE) {
		open_cash_operation = c_cash_operation_open_window(app1,NULL,FALSE);
		load_payment_cash_operation(d_charge_invoice_get_payment (charge_invoice),d_charge_invoice_get_total (charge_invoice));
		hide_cancelar_button_on_cash_operation();
		if (open_cash_operation!=NULL) {
		    gtk_window_set_modal(GTK_WINDOW(open_cash_operation),1);
		    gtk_window_set_position(GTK_WINDOW(open_cash_operation),GTK_WIN_POS_CENTER);
		    gtk_window_set_transient_for(GTK_WINDOW(open_cash_operation),
						 GTK_WINDOW(app1));
		    gtk_widget_show(GTK_WIDGET(open_cash_operation));
		}
	    }
	}
    } else if (save_without_vat!=FALSE) {
	save_charge_invoice();
//	c_order_search();
	if (charge_invoice!=NULL) {
	    if (d_charge_invoice_get_vat (charge_invoice)==FALSE) {
		open_cash_operation = c_cash_operation_open_window(app1,NULL,FALSE);
		load_payment_cash_operation(d_charge_invoice_get_payment (charge_invoice),d_charge_invoice_get_total (charge_invoice));
		hide_cancelar_button_on_cash_operation();
		if (open_cash_operation!=NULL) {
		    gtk_window_set_modal(GTK_WINDOW(open_cash_operation),1);
		    gtk_window_set_position(GTK_WINDOW(open_cash_operation),GTK_WIN_POS_CENTER);
		    gtk_window_set_transient_for(GTK_WINDOW(open_cash_operation),
						 GTK_WINDOW(app1));
		    gtk_widget_show(GTK_WIDGET(open_cash_operation));
		}
	    }
	}
    } else {
	c_util_open_info_window (_("Associate a payment type to invoice is mandatory."));
    }

}

static void clean_screen() {

    GtkWidget *vbox;
    gchar *s_name = NULL;

    maintain = FALSE;
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
    s_name = u_parameter_manager_get_parameter ("tienda");
    if (s_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo_entry_tienda_factura_cargo")), s_name);
    }
    g_free(s_name);


}

static void on_shop_combo_entry_changed(GtkEntry *entry, gpointer user_data) {
    gulong codeCombo;
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_tienda_factura_cargo");
    GtkWidget * entry_enter= glade_xml_get_widget(xml,"entry_empresa_factura_cargo");
    Enterprise * enterprise;
    gchar *enterpriseName;
    codeCombo = c_util_get_code_from_combo (combo);
    if (codeCombo>0) {
	enterprise = m_enterprise_find_by_shop (codeCombo);
	if (enterprise !=NULL) {
	    enterpriseName = d_participant_get_name (d_enterprise_get_participant (enterprise));
	    gtk_entry_set_text(GTK_ENTRY(entry_enter),enterpriseName);
	    g_free(enterpriseName);
	    d_enterprise_clear (enterprise);
	    enterprise = NULL;
	}

    }
}

static gint create_invoice_from_screen(gboolean modify) {
    GtkWidget * combo_shop= glade_xml_get_widget(xml,"combo_tienda_factura_cargo");
    GtkWidget * combo_provider= glade_xml_get_widget(xml,"combo_proveedor_factura_cargo");
    GtkWidget * spin_invoice_number= glade_xml_get_widget(xml,"spinbutton_numero_factura_factura_cargo");
    GtkWidget * dateedit_date_invoice= glade_xml_get_widget(xml,"dateedit_fecha_factura_cargo");
    GtkWidget * total_invoice_entry= glade_xml_get_widget(xml,"entry_total_factura_cargo");
    GtkWidget * code_entry= glade_xml_get_widget(xml,"entry_codigo_factura");
    GtkWidget * text_concept = glade_xml_get_widget(xml,"text_concepto_factura_cargo");
    GtkWidget * vat_checkbutton = glade_xml_get_widget(xml,"checkbutton_iva_factura_cargo");
    gulong shop_code = c_util_get_code_from_combo (combo_shop);
    Enterprise * enterprise;
    Shop * shop;
    Supplier * supplier;
    gulong supplier_code = c_util_get_code_from_combo (combo_provider);
    gchar * invoice_number_char;
    gchar * invoice_code_char;
    gulong  invoice_code_gulong;
    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
    time_t date;
    gdouble total;
    gchar * total_char;
    gchar * concept;
    gint length;
    gint error=0;
    gboolean with_vat = (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(vat_checkbutton)));


    invoice_code_char = g_strdup(gtk_entry_get_text(GTK_ENTRY(code_entry)));
    invoice_code_gulong = strtoul(invoice_code_char,stop,10);
    g_free(stop);

    invoice_number_char = g_strdup(gtk_entry_get_text(GTK_ENTRY(spin_invoice_number)));
    if (strlen(invoice_number_char)==0) {
	error = 1;
	if (d_charge_invoice_get_vat (charge_invoice)!=FALSE) {
	    c_util_open_error_window (_("Incorrect invoice number"));
	} else {
	    c_util_open_error_window (_("Incorrect payment note number"));
	}
    }

    if (error==0) {
	total_char=g_strdup(gtk_entry_get_text(GTK_ENTRY(total_invoice_entry)));
	u_points_to_commas (total_char);
	total = atof(total_char);

	if (total==0) {
	    error = 1;
	    c_util_open_error_window (_("Incorrect total value"));
	}

	enterprise=m_enterprise_find_by_shop (shop_code);
	
	date = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_date_invoice));

	if (!m_accounting_get_date_in_closed (d_participant_get_code (d_enterprise_get_participant (enterprise)),
					      date)) {
	  if (error ==0) {
	    Payment *local_payment = NULL;
	                             
	    ChargeInvoiceLine **charge_invoice_lines = NULL; 

	    if (charge_invoice!=NULL)
	    {
		local_payment = d_payment_duplicate (d_charge_invoice_get_payment (charge_invoice));
	    } else {
		local_payment = payment_to_invoice;
	    }
	    	    
	    charge_invoice_lines = m_charge_invoice_line_convert_glist_on_array (charge_lines);
	    shop=m_shop_find_by_shop_code (shop_code);

	    length = gtk_text_get_length(GTK_TEXT(text_concept));
	    
	    if (length<300) {
	      concept = gtk_editable_get_chars(GTK_EDITABLE(text_concept),0,length);
	    } else {
	      concept = gtk_editable_get_chars(GTK_EDITABLE(text_concept),0,299);
	    }

	    supplier = m_supplier_find_by_code (supplier_code);
	    
	    if (modify!=FALSE) {
	      if (charge_invoice!=NULL) {
		d_charge_invoice_clear (charge_invoice);
		charge_invoice = NULL;
		payment_to_invoice = NULL;
	      }
	    }

	    charge_invoice = d_charge_invoice_create_empty ();
	    
	    d_charge_invoice_set_payment (charge_invoice,local_payment);
	    d_charge_invoice_set_shop (charge_invoice,shop);
	    d_charge_invoice_set_enterprise (charge_invoice,enterprise);
	    d_charge_invoice_set_supplier (charge_invoice,supplier);
	    d_charge_invoice_set_vat (charge_invoice,with_vat);
	    d_charge_invoice_set_total (charge_invoice,total);
	    d_charge_invoice_set_date (charge_invoice,date);
	    d_charge_invoice_set_section (charge_invoice,concept);
	    d_charge_invoice_set_code (charge_invoice,invoice_code_gulong);
	    d_charge_invoice_set_invoice_number (charge_invoice,invoice_number_char);	  	  
	    d_charge_invoice_set_charge_invoice_lines (charge_invoice, charge_invoice_lines);
	    
	    
	    g_free(concept);
	    
	    shop=NULL;
	    enterprise = NULL;
	    supplier = NULL;	    
	  }
	} else {
	  time_t * good_date = NULL;	 

	  good_date = m_accounting_get_day_after_to_last_closed (d_participant_get_code (d_enterprise_get_participant (enterprise)), &date);

	  gnome_date_edit_set_time(GNOME_DATE_EDIT(dateedit_date_invoice),*good_date);
 
	  d_enterprise_clear (enterprise);
	  
	  error = 1;

	  c_util_open_error_window (N_("The date is previous to the last period of closed accountability\nIt has been used the first date available\nSave again if you actually want to keep that date in the invoice or change it"));
	}
    }

    return error;
}



static void save_charge_invoice(){
    GtkWidget * accept_button = glade_xml_get_widget(xml,"button_guardar_factura_cargo");
    GtkWidget * check_pago = glade_xml_get_widget(xml,"checkbutton_pago");
    GtkWidget * disassociate_button= glade_xml_get_widget(xml,"button_desasociar_factura_cargo");
    GtkWidget * vat_button = glade_xml_get_widget(xml,"checkbutton_iva_factura_cargo");
    GtkWidget * spinbutton_invoice_number= glade_xml_get_widget(xml,"spinbutton_numero_factura_factura_cargo");
    gchar * result=NULL;
    gchar * char_invoice = gtk_entry_get_text(GTK_ENTRY(spinbutton_invoice_number));
    gint error=0;
    if (strcmp(char_invoice,"")) {
	if (charge_invoice == NULL) {
	    error = create_invoice_from_screen(FALSE);
	    if (error == 0) {
		if (payment_to_invoice!=NULL) {
		    Enterprise * enter = d_enterprise_duplicate (d_charge_invoice_get_enterprise (charge_invoice));
		    Shop * shop = d_shop_duplicate (d_charge_invoice_get_shop (charge_invoice));
		    d_payment_set_shop (payment_to_invoice,shop);
		    d_payment_set_enterprise (payment_to_invoice,enter);
		    d_charge_invoice_set_payment (charge_invoice,payment_to_invoice);

		} else {
		    Enterprise * enter = d_enterprise_duplicate (d_charge_invoice_get_enterprise (charge_invoice));
		    Shop * shop = d_shop_duplicate (d_charge_invoice_get_shop (charge_invoice));
		    payment_to_invoice = d_payment_create_empty ();
		    d_payment_set_shop (payment_to_invoice,shop);
		    d_payment_set_enterprise (payment_to_invoice,enter);
		    d_payment_set_achieved_payments (payment_to_invoice,1);
		    d_payment_set_achieved_date (payment_to_invoice,time(NULL));
		    d_payment_set_entry_percentage (payment_to_invoice,100.0);
		    d_payment_set_number_expiries (payment_to_invoice,1);
		    d_payment_set_init_date (payment_to_invoice,time(NULL));
		    d_payment_set_is_paid_cash (payment_to_invoice,TRUE);
		    d_charge_invoice_set_payment (charge_invoice,payment_to_invoice);

		}
	    }
	    if (error==0) {
		gint state ;

		state = (d_payment_get_number_expiries (d_charge_invoice_get_payment (charge_invoice))==
			 d_payment_get_achieved_payments (d_charge_invoice_get_payment (charge_invoice)));
		if (state>0) state=1;
		if (d_charge_invoice_get_vat (charge_invoice)==0) {
		    d_charge_invoice_set_state (charge_invoice,1);
		} else {
		    d_charge_invoice_set_state (charge_invoice,state);
		}
		result = m_charge_invoice_insert (charge_invoice);
	    }
	} else {
	    result = m_charge_invoice_insert (charge_invoice);
	}

	if (error == 0) {
	    if (result == NULL) {
		if (d_charge_invoice_get_vat (charge_invoice)!=FALSE) {
		    c_util_open_error_window (_("Insert invoice error"));
		} else {
		    c_util_open_error_window (_("Payment note insert error"));
		}
		if (charge_invoice!=NULL)
		    d_charge_invoice_clear (charge_invoice);
		charge_lines = NULL;
		charge_invoice = NULL;
		payment_to_invoice=NULL;
		maintain = FALSE;
		c_charge_invoice_type_payment_toggle_button_change_state (FALSE);
	    } else if (!strcmp(result,"")) {
		if (d_charge_invoice_get_vat (charge_invoice)!=FALSE) {
		    c_util_open_error_window (_("Nonexistent invoice"));
		} else {
		    c_util_open_error_window (_("Nonexistent payment note"));
		}
		if (charge_invoice!=NULL)
		    d_charge_invoice_clear (charge_invoice);
		charge_lines = NULL;
		charge_invoice = NULL;
		payment_to_invoice=NULL;
		maintain = FALSE;
		c_charge_invoice_type_payment_toggle_button_change_state (FALSE);
	    } else {
		GtkWidget * entry_codigo = glade_xml_get_widget(xml,"entry_codigo_factura");
                gchar * codigo=g_strdup_printf("%lu",d_charge_invoice_get_code (charge_invoice));
		m_charge_invoice_line_set_new (&charge_lines, charge_invoice);
		gtk_widget_set_sensitive(check_pago,FALSE);
		gtk_widget_set_sensitive(disassociate_button,FALSE);
		gtk_widget_set_sensitive(accept_button,FALSE);
		gtk_widget_set_sensitive(vat_button,FALSE);
		gtk_entry_set_text(GTK_ENTRY(entry_codigo),codigo);
                g_free(codigo);
              
	    }
	    maintain = TRUE;
	}
    } else {
	c_util_open_error_window (_("The invoice has no number"));
    }

}

static void on_payment_checkbutton_toggled(GtkToggleButton *togglebutton,
					gpointer user_data) {
    Payment * my_payment=NULL ;
    GtkWidget * open_payment_control;
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(togglebutton), "app1");

    if (gtk_toggle_button_get_active(togglebutton)!=FALSE) {
	maintain = TRUE;
	if (charge_invoice==NULL) {
	    if (payment_to_invoice==NULL) {
		gulong providerCode=c_util_get_code_from_combo (glade_xml_get_widget(xml,"combo_proveedor_factura_cargo"));
		PaymentId * payment_id =m_charge_invoice_find_last_from_supplier (providerCode);

		if (payment_id == NULL) {
		    my_payment = d_payment_create_empty ();
		} else {
		    my_payment = m_payment_find_by_id (payment_id);
		    d_enterprise_clear (d_payment_get_enterprise (my_payment));
		    d_payment_set_enterprise (my_payment,NULL);
		    d_shop_clear (d_payment_get_shop (my_payment));
		    d_payment_set_shop (my_payment,NULL);
                    d_payment_set_payment_code (my_payment,ULONG_MAX);
                    d_payment_set_achieved_payments (my_payment,0);
		}
		open_payment_control = c_payment_open_window(app1,my_payment,cancel_payment,TRUE,FALSE);
		payment_to_invoice = my_payment;
                  
		if (open_payment_control!=NULL) {
		    gtk_window_set_modal(GTK_WINDOW(open_payment_control),1);
		    gtk_window_set_position(GTK_WINDOW(open_payment_control),GTK_WIN_POS_CENTER);
		    gtk_window_set_transient_for(GTK_WINDOW(open_payment_control),
						 GTK_WINDOW(app1));
		    gtk_widget_show(GTK_WIDGET(open_payment_control));
		}
	    }
	}
    } else if (maintain==TRUE){
	gtk_toggle_button_set_active((togglebutton),TRUE);
    }

}

void c_charge_invoice_type_payment_toggle_button_change_state (gboolean new_state) {
    GtkWidget * button = glade_xml_get_widget(xml,"checkbutton_pago");
    maintain = FALSE;
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),new_state);
}

void c_charge_invoice_is_paid_toggle_button_change_state (gboolean new_state) {
    GtkWidget * button = glade_xml_get_widget(xml,"checkbutton_estado_factura_cargo");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button),new_state);
    if (new_state==TRUE) {
	m_charge_invoice_update_state (charge_invoice,D_CHARGE_INVOICE_PAID);
    }
}

static void on_manage_charge_invoice_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * open_payment_control=NULL;
    GtkWidget * open_cash_operation=NULL;
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    gint error =0 ;

    if (charge_invoice!=NULL) {
	if (payment_to_invoice!=NULL) {
	    open_payment_control = c_payment_open_window(app1,payment_to_invoice,cancel_payment,FALSE,TRUE);
	} else {
	    if (d_charge_invoice_get_payment (charge_invoice)!=NULL) {
		if (d_charge_invoice_get_vat (charge_invoice)!=FALSE) {
		    open_payment_control = c_payment_open_window(app1,d_charge_invoice_get_payment (charge_invoice),cancel_payment,FALSE,TRUE);
		    payment_to_invoice = NULL;
		    if (open_payment_control!=NULL) {
			gtk_window_set_modal(GTK_WINDOW(open_payment_control),1);
			gtk_window_set_position(GTK_WINDOW(open_payment_control),GTK_WIN_POS_CENTER);
			gtk_window_set_transient_for(GTK_WINDOW(open_payment_control),
						     GTK_WINDOW(app1));
			gtk_widget_show(GTK_WIDGET(open_payment_control));

		    }
		} else {
		    open_cash_operation = c_cash_operation_open_window(app1,NULL,FALSE);
		    load_payment_cash_operation(d_charge_invoice_get_payment (charge_invoice),d_charge_invoice_get_total (charge_invoice));
		    if (open_cash_operation!=NULL) {
			gtk_window_set_modal(GTK_WINDOW(open_cash_operation),1);
			gtk_window_set_position(GTK_WINDOW(open_cash_operation),GTK_WIN_POS_CENTER);
			gtk_window_set_transient_for(GTK_WINDOW(open_cash_operation),
						     GTK_WINDOW(app1));
			gtk_widget_show(GTK_WIDGET(open_cash_operation));
		    }
		}
	    } else {
		error = 1;
		c_util_open_info_window (_("It is necessary to link a payment previously"));
	    }
	}
	if (error == 0) {
	}
    } else {
	c_util_open_error_window (_("The invoice should be saved"));
    }
}

static void on_dissasociate_charge_invoice_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * payment_button = glade_xml_get_widget(xml,"checkbutton_pago");
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(payment_button))!=FALSE) {
	if (payment_to_invoice!=NULL) {
	    d_payment_clear (payment_to_invoice);
	    payment_to_invoice=NULL;
	}
	c_charge_invoice_type_payment_toggle_button_change_state (FALSE);
    }
}

static void inicialize_sensitive(gboolean note,gboolean new_case) {
    GtkWidget * spin_invoice_number= glade_xml_get_widget(xml,"spinbutton_numero_factura_factura_cargo");
    GtkWidget * label_invoice_number= glade_xml_get_widget(xml,"label_numero_factura_cargo");
    GtkWidget * manage_button = glade_xml_get_widget(xml,"button_gestionar_factura_cargo");
    GtkWidget * paid_checkbutton = glade_xml_get_widget(xml,"checkbutton_estado_factura_cargo");
    GtkWidget * without_vat_checkbutton = glade_xml_get_widget(xml,"checkbutton_iva_factura_cargo");
    GtkWidget * payment_checkbutton = glade_xml_get_widget(xml,"checkbutton_pago");
    GtkWidget * disassociate_button = glade_xml_get_widget(xml,"button_desasociar_factura_cargo");
    GtkWidget * save_button = glade_xml_get_widget(xml,"button_guardar_factura_cargo");
    GtkWidget * scrolled_window = glade_xml_get_widget(xml,"lineas_scrolledwindow");
    GtkWidget * frame = glade_xml_get_widget(xml,"lineas_frame");

    if (new_case==FALSE) {
	if (GTK_WIDGET_SENSITIVE(spin_invoice_number)) gtk_widget_set_sensitive(spin_invoice_number,FALSE);
	if (GTK_WIDGET_SENSITIVE(paid_checkbutton)) gtk_widget_set_sensitive(paid_checkbutton,FALSE);
	if (GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,FALSE);

	if (note != FALSE) {
	    gtk_label_set_text(GTK_LABEL(label_invoice_number),_("Payment note number"));
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(without_vat_checkbutton),TRUE);
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(paid_checkbutton),FALSE);
	    c_charge_invoice_type_payment_toggle_button_change_state (FALSE);
	    if (GTK_WIDGET_SENSITIVE(manage_button)) gtk_widget_set_sensitive(manage_button,FALSE);
	    if (GTK_WIDGET_SENSITIVE(save_button)) gtk_widget_set_sensitive(save_button,FALSE);
	    if (GTK_WIDGET_SENSITIVE(scrolled_window)) gtk_widget_set_sensitive(scrolled_window,FALSE);
	    if (GTK_WIDGET_SENSITIVE(frame)) gtk_widget_set_sensitive(frame,FALSE);
	} else {
	    gtk_label_set_text(GTK_LABEL(label_invoice_number),_("Invoice number"));
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(without_vat_checkbutton),FALSE);
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(paid_checkbutton),FALSE);
	    if (GTK_WIDGET_SENSITIVE(save_button)) gtk_widget_set_sensitive(save_button,FALSE);
	    if (GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,TRUE);
	    if (GTK_WIDGET_SENSITIVE(scrolled_window)) gtk_widget_set_sensitive(scrolled_window,TRUE);
	}
	if (GTK_WIDGET_SENSITIVE(without_vat_checkbutton)) gtk_widget_set_sensitive(without_vat_checkbutton,FALSE);


    } else {
	if (!GTK_WIDGET_SENSITIVE(spin_invoice_number)) gtk_widget_set_sensitive(spin_invoice_number,TRUE);
	if (!GTK_WIDGET_SENSITIVE(without_vat_checkbutton)) gtk_widget_set_sensitive(without_vat_checkbutton,TRUE);
	if (GTK_WIDGET_SENSITIVE(paid_checkbutton)) gtk_widget_set_sensitive(paid_checkbutton,FALSE);
	if (!GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,TRUE);
	if (!GTK_WIDGET_SENSITIVE(manage_button)) gtk_widget_set_sensitive(manage_button,TRUE);
	if (!GTK_WIDGET_SENSITIVE(payment_checkbutton)) gtk_widget_set_sensitive(payment_checkbutton,TRUE);
	if (!GTK_WIDGET_SENSITIVE(save_button)) gtk_widget_set_sensitive(save_button,TRUE);
 	if (GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,TRUE);
	if (GTK_WIDGET_SENSITIVE(scrolled_window)) gtk_widget_set_sensitive(scrolled_window,TRUE);
	gtk_label_set_text(GTK_LABEL(label_invoice_number),_("Invoice number"));
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(without_vat_checkbutton),FALSE);

    }
}

static void on_vat_checkbutom_toggled(GtkToggleButton *togglebutton,
						     gpointer user_data) {
    GtkWidget * manage_button = glade_xml_get_widget(xml,"button_gestionar_factura_cargo");
    GtkWidget * payment_checkbutton = glade_xml_get_widget(xml,"checkbutton_pago");
    GtkWidget * paid_checkbutton = glade_xml_get_widget(xml,"checkbutton_estado_factura_cargo");
    GtkWidget * disassociate_button = glade_xml_get_widget(xml,"button_desasociar_factura_cargo");
    GtkWidget * label_number = glade_xml_get_widget(xml,"label_numero_factura_cargo");
    GtkWidget * scrolled_window = glade_xml_get_widget(xml,"lineas_scrolledwindow");
    GtkWidget * frame = glade_xml_get_widget(xml,"lineas_frame");

    if (gtk_toggle_button_get_active(togglebutton)!=FALSE) {
	if (GTK_WIDGET_SENSITIVE(manage_button)) gtk_widget_set_sensitive(manage_button,FALSE);
	if (GTK_WIDGET_SENSITIVE(payment_checkbutton)) gtk_widget_set_sensitive(payment_checkbutton,FALSE);
	if (GTK_WIDGET_SENSITIVE(paid_checkbutton)) gtk_widget_set_sensitive(paid_checkbutton,FALSE);
	if (GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,FALSE);
	if (GTK_WIDGET_SENSITIVE(scrolled_window)) gtk_widget_set_sensitive(scrolled_window,FALSE);
	if (GTK_WIDGET_SENSITIVE(frame)) gtk_widget_set_sensitive(frame,FALSE);
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(paid_checkbutton),TRUE);
	gtk_label_set_text(GTK_LABEL(label_number),_("Payment note number"));

	if (payment_to_invoice!=NULL) {
	    d_payment_clear (payment_to_invoice);
	    payment_to_invoice=NULL;
	    maintain=FALSE;
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(payment_checkbutton),FALSE);
	    maintain=TRUE;

	}
    } else {
	gtk_label_set_text(GTK_LABEL(label_number),_("Invoice number"));
	if (!GTK_WIDGET_SENSITIVE(manage_button)) gtk_widget_set_sensitive(manage_button,TRUE);
	if (!GTK_WIDGET_SENSITIVE(payment_checkbutton)) gtk_widget_set_sensitive(payment_checkbutton,TRUE);
	if (!GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,TRUE);
	if (!GTK_WIDGET_SENSITIVE(paid_checkbutton)) gtk_widget_set_sensitive(paid_checkbutton,FALSE);
 	if (!GTK_WIDGET_SENSITIVE(disassociate_button)) gtk_widget_set_sensitive(disassociate_button,TRUE);
	if (!GTK_WIDGET_SENSITIVE(scrolled_window)) gtk_widget_set_sensitive(scrolled_window,TRUE);
	if (!GTK_WIDGET_SENSITIVE(frame)) gtk_widget_set_sensitive(frame,TRUE);

	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(paid_checkbutton),FALSE);
    }

}



static void on_modify_charge_invoice_button_clicked(GtkButton *button, gpointer data) {
    if (charge_invoice!=NULL) {
	if (d_charge_invoice_get_vat (charge_invoice)) {
	    if (!create_invoice_from_screen(TRUE)) {
		check_charge_invoice(charge_invoice,d_charge_invoice_get_charge_invoice_lines (charge_invoice));
		m_charge_invoice_update (charge_invoice,FALSE);
		m_charge_invoice_line_set_new (&charge_lines, charge_invoice);
	  }
	} else {
	    if (!create_invoice_from_screen(TRUE)) {
		m_charge_invoice_update (charge_invoice,TRUE);
	    }
	}
    } else {
	c_util_open_error_window (_("It has to be saved in the DB"));
    }
}

static void inicialize_shop_combo() {
    GtkWidget * shop_combo_entry = glade_xml_get_widget(xml,"combo_entry_tienda_factura_cargo");
    gchar * aux;

    aux = g_strdup(gtk_entry_get_text(GTK_ENTRY(shop_combo_entry)));
    gtk_entry_set_text(GTK_ENTRY(shop_combo_entry),"");
    gtk_entry_set_text(GTK_ENTRY(shop_combo_entry),aux);
    g_free(aux);

}

static void cancel_payment(gpointer value) {
    GtkWidget * payment_button = glade_xml_get_widget(xml,"checkbutton_pago");
    gboolean * from_managing = (gboolean *) value;
    if (*from_managing!=TRUE) {
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(payment_button))!=FALSE) {
	    if (payment_to_invoice!=NULL) {
		d_payment_clear (payment_to_invoice);
		payment_to_invoice=NULL;
	    }
	    maintain =FALSE;
	    c_charge_invoice_type_payment_toggle_button_change_state (FALSE);
	    maintain = TRUE;
	}
    } else {

    }

}

static void on_add_charge_line_button_clicked(GtkButton *button, gpointer data) {
    gboolean error = FALSE;
    if (charge_invoice!=NULL) {
	if (d_charge_invoice_get_state (charge_invoice)!=D_CHARGE_INVOICE_NOT_PAID)
            error = TRUE;
    }

    if (error==FALSE) {
	gboolean *insert = (gboolean *) g_malloc0(sizeof(gboolean));
	GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
	GtkWidget * insert_line = NULL;

	*insert = TRUE;


	insert_line = c_charge_invoice_line_open_window(app1,(gpointer) insert);

	gtk_window_set_modal(GTK_WINDOW(insert_line),1);
	gtk_window_set_position(GTK_WINDOW(insert_line),GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(insert_line), GTK_WINDOW(app1));

	gtk_widget_show(insert_line);

    } else {
        c_util_open_error_window (_("You can't add invoice lines to payed invoices"));
    }
}

static void on_modify_charge_line_button_clicked(GtkButton *button, gpointer data) {

  gboolean *insert = (gboolean *) g_malloc0(sizeof(gboolean));
  GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
  GtkWidget * insert_line = NULL;
  GtkWidget * list = glade_xml_get_widget (xml, "clist_detalle_factura_cargo");
  GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
  gint line = selected->len-1;
  gpointer cl = NULL;

  if (line != -1) {
    gint nindex = g_array_index(selected,gint,line);
    cl = g_list_nth_data (charge_lines, nindex);

    *insert = FALSE;
    
    insert_line = c_charge_invoice_line_open_window(app1,(gpointer) insert);
    
    c_charge_invoice_line_load((ChargeInvoiceLine *) cl);
    
    gtk_window_set_modal(GTK_WINDOW(insert_line),1);
    gtk_window_set_position(GTK_WINDOW(insert_line),GTK_WIN_POS_CENTER);
    gtk_window_set_transient_for(GTK_WINDOW(insert_line), GTK_WINDOW(app1));
    
    gtk_widget_show(insert_line);
    
  }

}

static void on_delete_charge_line_button_clicked(GtkButton *button, gpointer data) {
    gboolean error = FALSE;
    if (charge_invoice!=NULL) {
	if (d_charge_invoice_get_state (charge_invoice)!=D_CHARGE_INVOICE_NOT_PAID) {
            error = TRUE;
	}
    }

    if (charge_lines!=NULL) {

	if (error==FALSE) {

	    GtkWidget * list = glade_xml_get_widget (xml, "clist_detalle_factura_cargo");
	    GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
	    gint line = selected->len-1;
	    gchar ** stop=(gchar **)g_malloc0(sizeof(gchar *));
	    gchar * shop_name = u_parameter_manager_get_parameter ("tienda");
	    gchar * enter_name = u_parameter_manager_get_parameter ("empresa");
	    gulong enter_code_gulong= m_enterprise_get_code_from_name (enter_name);
	    gulong shop_code_gulong=m_shop_get_code_from_name (shop_name);

	    while (line != -1) {
		gint nindex = g_array_index(selected,gint,line);
		gpointer data = g_list_nth_data (charge_lines, nindex);
		m_charge_invoice_line_delete ((ChargeInvoiceLine *) data, enter_code_gulong, shop_code_gulong);

		gtk_clist_remove(GTK_CLIST(list),nindex);

		charge_lines = g_list_remove (charge_lines, data);

		line--;
	    }

	    if ((selected->len-1) != -1){
		c_charge_invoice_load_charge_lines_from_list ();
	    }

	    if (charge_invoice!=NULL) {
		check_charge_invoice(charge_invoice,d_charge_invoice_get_charge_invoice_lines (charge_invoice));
	    }

	    g_array_free(selected, FALSE);

	    g_free(enter_name);
	    g_free(shop_name);

	    g_free(stop);
	} else {
	    c_util_open_error_window (_("You can't remove invoice lines from a paid invoice"));
	}
    } else {
	c_util_open_error_window (_("There is no line"));
    }
}

static void on_add_order_line_button_clicked(GtkButton *button, gpointer data) {
    gboolean error = FALSE;
    if (charge_invoice!=NULL) {
	if (d_charge_invoice_get_state (charge_invoice)!=D_CHARGE_INVOICE_NOT_PAID)
	    error = TRUE;
    }
    if (error==FALSE) {

	GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
	GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");

	c_find_order_open_window(app1, vbox, NULL);

	i_shortcut_change_dialog (GTK_WIDGET(button),2,4);
    } else {
	c_util_open_error_window (_("You can't add invoice lines to paid invoices"));

    }

}

void c_charge_invoice_add_charge_line (ChargeInvoiceLine *charge_invoice_line) {
 
    charge_lines = g_list_append(charge_lines, charge_invoice_line);
    if (charge_invoice!=NULL) {
	d_charge_invoice_line_set_invoice_number (charge_invoice_line,d_charge_invoice_get_code (charge_invoice));
	d_charge_invoice_line_set_supplier_code (charge_invoice_line,d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoice))));
    }
    c_charge_invoice_load_charge_lines_from_list ();
}

void c_charge_invoice_load_charge_lines_from_list () {
  
  if (charge_lines!=NULL) {
    GtkWidget * list = glade_xml_get_widget (xml, "clist_detalle_factura_cargo");
    gchar *empty_row[]={"","","","","","",""};
    GList * element = g_list_first(charge_lines);
    gint index = 0;

    gtk_clist_clear(GTK_CLIST(list));
    
    while (element != NULL) {
      ChargeInvoiceLine *line =(ChargeInvoiceLine *) element->data; 
      gchar * description = d_charge_invoice_line_get_description (line);
      gdouble vat_gdouble = d_charge_invoice_line_get_vat_type (line);
      gdouble total_gdouble = d_charge_invoice_line_get_total (line);
      OrderLine *order_line = d_charge_invoice_line_get_order_line (line);

      if (index==30)
	gtk_clist_freeze ((GtkCList *) list);
      
      gtk_clist_append ((GtkCList *) list,empty_row);
	
      if (order_line != NULL) {
	gulong order_line_number_gulong = d_order_line_get_code (order_line);
	Piece *piece = d_order_line_get_piece (order_line);
	gdouble quantity_gulong = u_round_to_decimal (d_order_line_get_quantity (order_line),3);
	gchar *dev_payment_number = d_order_line_get_delivery_payment (order_line);

	if (order_line_number_gulong != ULONG_MAX) {
	  gchar *order_line_number =  g_strdup_printf("%lu",order_line_number_gulong);
	  
	  gtk_clist_set_text((GtkCList *) list, index, C_ORDER_CODE_COLUMN, order_line_number);
	    g_free(order_line_number);
	}
	
	if (quantity_gulong != G_MINDOUBLE) {
	  gchar *quantity =  g_strdup_printf("%.3f",quantity_gulong);
	  gtk_clist_set_text((GtkCList *) list, index, C_QUANTITY_COLUMN, quantity);
	  g_free(quantity);
	}	

	if (dev_payment_number != NULL) {
	  gtk_clist_set_text((GtkCList *) list, index, C_DELIVERY_PAYMENT_COLUMN, dev_payment_number); 
	  g_free(dev_payment_number); 
	}

	if (piece != NULL) {
	  gchar *eurocode = d_piece_get_euroCode (piece);

	  if (eurocode!=NULL) {
	    gtk_clist_set_text((GtkCList *) list, index, C_EUROCODE_COLUMN, eurocode);
	    g_free(eurocode);
	  }
	}
      }
      
      if (description!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_DESCRIPTION_COLUMN, description);
	g_free(description);
      }

      if (total_gdouble!= G_MINDOUBLE) {
	gchar *total =  g_strdup_printf("%.2f",total_gdouble);
	gtk_clist_set_text((GtkCList *) list, index, C_PRICE_COLUMN, total);
	g_free(total);
      }	

      if (vat_gdouble!=-1.0) {
	gchar *vat =  g_strdup_printf("%.2f",vat_gdouble);
	gtk_clist_set_text((GtkCList *) list, index, C_VAT_COLUMN, vat);
	g_free(vat);
      }	
      
      element = g_list_next(element);
      index++;
    }
    
    gtk_clist_thaw ((GtkCList *) list);
  }
  
}

void c_charge_invoice_load_order_lines (GList *order_lines) {

  ChargeInvoiceLine **charge_invoice_lines = NULL;
  ChargeInvoiceLine **lines = NULL;

  if (order_lines != NULL) {
    GList *element = g_list_last(order_lines);
    gint number_elements = g_list_position(order_lines, element);

    guint line;
    
    charge_invoice_lines = (ChargeInvoiceLine **) g_malloc0(sizeof(ChargeInvoiceLine*)*(number_elements+2));
    
    charge_invoice_lines[number_elements+1] = NULL;
    
    for (line = 0; line <= number_elements; line ++) {
      OrderLine * order_line = g_list_nth_data(order_lines,line);
      Product product = d_piece_get_product (d_order_line_get_piece (order_line));

      charge_invoice_lines[line] = d_charge_invoice_line_create_empty ();
      if (charge_invoice!=NULL) {
	  d_charge_invoice_line_set_invoice_number (charge_invoice_lines[line],d_charge_invoice_get_code (charge_invoice));
	  d_charge_invoice_line_set_supplier_code (charge_invoice_lines[line],d_participant_get_code (d_supplier_get_participant (d_charge_invoice_get_supplier (charge_invoice))));
      }
      d_charge_invoice_line_set_description (charge_invoice_lines[line],d_product_get_description (&product));
      d_charge_invoice_line_set_order_line (charge_invoice_lines[line],order_line);
      if (d_piece_get_cost_price (d_order_line_get_piece (order_line))!=G_MINDOUBLE)
	  d_charge_invoice_line_set_total (charge_invoice_lines[line],d_piece_get_cost_price (d_order_line_get_piece (order_line))*d_order_line_get_quantity (order_line));
      if (d_piece_get_cost_price (d_order_line_get_piece (order_line))>=0)
      d_charge_invoice_line_set_vat_type(charge_invoice_lines[line],d_product_get_vat_type (&product));
    }

    lines = d_charge_invoice_line_duplicate (charge_invoice_lines);

    m_charge_invoice_line_convert_to_glist (&charge_lines,lines);
    c_charge_invoice_load_charge_lines_from_list ();

    g_free(charge_invoice_lines);
    
  }
  
}

static void delete_invoice(gint reply,
			   gpointer user_data) {
    if (!reply) {
	if (charge_invoice!=NULL) {
		gchar * message = m_charge_invoice_delete (charge_invoice);
		if (message == NULL) {
		    c_util_open_info_window (_("Deleted invoice"));

		    m_charge_invoice_line_delete_glist (&charge_lines);

		    d_charge_invoice_clear (charge_invoice);
		    charge_invoice=NULL;
		    payment_to_invoice=NULL;
		    charge_lines = NULL;

		    clean_screen();

		    maintain =FALSE;
		    clean_screen();
		    maintain = TRUE;
		    inicialize_shop_combo();

		} else {
		    c_util_open_error_window (message);
		}
		if (message!=NULL)
		    g_free(message);
	}
    } else {
	c_util_open_info_window (_("Cancelled command"));
    }
}

static void on_delete_charge_invoice_button_clicked(GtkButton *button, gpointer data) {
    if (charge_invoice!=NULL) {
	gulong invoice_code= d_charge_invoice_get_code (charge_invoice);
	if (invoice_code!=ULONG_MAX) {
	    gchar * invoice_number_char=d_charge_invoice_get_invoice_number (charge_invoice);
	    gchar * my_message = g_strconcat(_("Do you really want to remove THE INVOICE:"),invoice_number_char, "?",
					     _("\nYou should consider that:\n       * Just unpaid invoices can be removed.\n       * When the invoice is removed the automatically unrecoverable data will be lost"),NULL);
	    g_free(invoice_number_char);
	    c_util_open_question_dialog(my_message,
				 (*delete_invoice));
	    g_free(my_message);
	} else {
	    c_util_open_error_window (_("Nonexistent invoices can't be removed"));
	}
    } else {
	c_util_open_error_window (_("Nonexistent invoices can't be removed"));
    }
}


