
/* clients_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "repairs_control.h"
#include "a-shortcut.h"
#include "clients_control.h"
#include "edit_clients_control.h"
#include "insert_clients_control.h"
#include "order_control.h"
#include "find_order_control.h"
#include "control_util.h"
#include "support.h"
#include "particular_object.h"
#include "client_model.h"

#define C_LEGAL_ID_COLUMN 0
#define C_NAME_COLUMN 1
#define C_PHONE1_COLUMN 2
#define C_PHONE2_COLUMN 3
#define C_EMAIL_COLUMN 4

static GtkWidget *clients=NULL;
static GladeXML *xml;
static Participant **clients_data = NULL;

static gchar url[]="/src/interface/glade/clients_view.glade";
static gchar widget_name[]="vbox_lista_clientes";
static gboolean only_supplier = FALSE;
static void (*function_to_call)(GtkWidget *app1, GtkWidget *vbox, Participant *participant);
static gint group_num;
static gint item_num;


static void fill_person_combo(void);
static void show_particular(void);
static void show_enterprise(void);
static void show_nicks(void);
static void on_find_persons_button_clicked(GtkButton *button,gpointer user_data);
static void on_person_combo_entry_changed(GtkEntry *entry, gpointer user_data);
static void open_insert_client(GtkWidget *widget,gpointer user_data);
static void open_edit_client(GtkWidget *widget,gpointer user_data);
static void clean_screen(GtkButton *button,gpointer user_data);
static void fill_person_list(GtkCList * list, Participant **participants);
static void find_persons_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void on_person_list_select_row (GtkCList *list, gint row, gint column, GdkEvent *event, gpointer user_data);
static void on_person_listing_destroy(GtkObject *object, gpointer user_data);
static void on_repair_button_clicked(GtkButton *button,gpointer user_data);


GtkWidget *c_clients_control_open (GtkWidget *app1, GtkWidget *vbox)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (clients==NULL)
     {
	 GtkWidget *dialog;
	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);  

	glade_xml_signal_connect (xml, "on_person_entry_find_person_changed",
				  GTK_SIGNAL_FUNC (on_person_combo_entry_changed));

	glade_xml_signal_connect (xml, "on_inserta_button_clicked",
				  GTK_SIGNAL_FUNC (open_insert_client));

	glade_xml_signal_connect (xml, "on_modify_button_clicked",
				  GTK_SIGNAL_FUNC (open_edit_client));

	glade_xml_signal_connect (xml, "on_borrar_button_clicked",
				  GTK_SIGNAL_FUNC (clean_screen));

	glade_xml_signal_connect (xml, "on_find_button_clicked",
				  GTK_SIGNAL_FUNC (on_find_persons_button_clicked));

	glade_xml_signal_connect (xml, "on_cif_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_cifnif_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_apellido1_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_nombre_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_dni_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_apellido2_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_razonsocial_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_nick_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_calle_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_numero_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_piso_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_cp_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_municipio_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_provincia_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_telefono1_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_telefono2_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));
	
	glade_xml_signal_connect (xml, "on_fax_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_email_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_dto_entry_key_press_event",
				  GTK_SIGNAL_FUNC (find_persons_key_pressed));

	glade_xml_signal_connect (xml, "on_person_clist_select_row",
				  GTK_SIGNAL_FUNC (on_person_list_select_row));

	glade_xml_signal_connect (xml, "on_listado_persona_destroy",
				  GTK_SIGNAL_FUNC (on_person_listing_destroy));

	glade_xml_signal_connect (xml, "on_repair_button_clicked",
				  GTK_SIGNAL_FUNC (on_repair_button_clicked));

	fill_person_combo();

	clients=dialog;
	
     }

   g_free(directory);
   return clients;
   
}

GtkWidget *c_clients_control_open_supplier (GtkWidget *app1, GtkWidget *vbox, gpointer f_to_call,
			gint group_number, gint item_number) {

    c_clients_control_open (app1, vbox);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "person_entry_find_person")), _(C_SUPPLIER_MESSAGE));
    only_supplier = TRUE;
    function_to_call = f_to_call;
    group_num = group_number;
    item_num  = item_number;

    return clients;
}

static void fill_person_combo() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "person_combo_find_clients");

    c_util_set_code_and_name_on_combo (M_CLIENT_PERSON_CODE, _(C_PERSON_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_ENTERPRISE_CODE,_(C_ENTERPRISE_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_SUPPLIER_CODE,_(C_SUPPLIER_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_COMPANY_CODE,_(C_COMPANY_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_CONSORTIUM_CODE,_(C_CONSORTIUM_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_GARAGE_CODE,_(C_GARAGE_MESSAGE),GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (M_CLIENT_ALL_PARTICIPANT_CODE,_(C_ALL_PARTICIPANTS_MESSAGE),GTK_COMBO(combo));

}

static void on_person_combo_entry_changed(GtkEntry *entry,gpointer user_data){
  GtkWidget * combo;
  gulong combo_code;

  combo = glade_xml_get_widget (xml,"person_combo_find_clients");

  combo_code = c_util_get_code_from_combo (combo);

  switch (combo_code)
    {
    case M_CLIENT_PERSON_CODE: show_particular();break;
    case M_CLIENT_ENTERPRISE_CODE: show_enterprise();break;
    case M_CLIENT_SUPPLIER_CODE: show_enterprise();break;
    case M_CLIENT_COMPANY_CODE: show_enterprise();break;
    case M_CLIENT_CONSORTIUM_CODE: show_enterprise();break;
    case M_CLIENT_GARAGE_CODE: show_enterprise();break;
    case M_CLIENT_ALL_PARTICIPANT_CODE: show_nicks();break;
    }
}

static void show_particular(){
  GtkWidget *name_label = glade_xml_get_widget(xml, "nombre_label");
  GtkWidget *surname1_label = glade_xml_get_widget(xml, "apellido1_label");
  GtkWidget *surname2_label = glade_xml_get_widget(xml, "apellido2_label");
  GtkWidget *name_entry = glade_xml_get_widget(xml, "nombre_entry");
  GtkWidget *surname1_entry = glade_xml_get_widget(xml, "apellido1_entry");
  GtkWidget *surname2_entry = glade_xml_get_widget(xml, "apellido2_entry");
  GtkWidget *personal_legal_id_label = glade_xml_get_widget(xml, "dni_label");
  GtkWidget *personal_legal_id_entry = glade_xml_get_widget(xml, "dni_entry");
  GtkWidget *trade_name_label = glade_xml_get_widget(xml, "razonsocial_label");
  GtkWidget *trade_name_entry = glade_xml_get_widget(xml, "razonsocial_entry");
  GtkWidget *enterprise_legal_id_label = glade_xml_get_widget(xml, "cif_label");
  GtkWidget *enterprise_legal_id_entry = glade_xml_get_widget(xml, "cif_entry");
  GtkWidget *legal_id_label = glade_xml_get_widget(xml, "cifnif_label");
  GtkWidget *legal_id_entry = glade_xml_get_widget(xml, "cifnif_entry");
  GtkWidget *nick_label = glade_xml_get_widget(xml, "nick_label");
  GtkWidget *nick_entry = glade_xml_get_widget(xml, "nick_entry");

  gtk_widget_show(name_label);
  gtk_widget_show(name_entry);

  gtk_widget_show(surname1_label);
  gtk_widget_show(surname1_entry);

  gtk_widget_show(surname2_label);
  gtk_widget_show(surname2_entry);

  gtk_widget_show(personal_legal_id_label);
  gtk_widget_show(personal_legal_id_entry);
  
  gtk_widget_hide(trade_name_label);
  gtk_widget_hide(trade_name_entry);

  gtk_widget_hide(enterprise_legal_id_label);
  gtk_widget_hide(enterprise_legal_id_entry);

  gtk_widget_hide(legal_id_label);
  gtk_widget_hide(legal_id_entry);

  gtk_widget_hide(nick_label);
  gtk_widget_hide(nick_entry);

}

static void show_enterprise(){
  GtkWidget *name_label = glade_xml_get_widget(xml, "nombre_label");
  GtkWidget *surname1_label = glade_xml_get_widget(xml, "apellido1_label");
  GtkWidget *surname2_label = glade_xml_get_widget(xml, "apellido2_label");
  GtkWidget *name_entry = glade_xml_get_widget(xml, "nombre_entry");
  GtkWidget *surname1_entry = glade_xml_get_widget(xml, "apellido1_entry");
  GtkWidget *surname2_entry = glade_xml_get_widget(xml, "apellido2_entry");
  GtkWidget *personal_legal_id_label = glade_xml_get_widget(xml, "dni_label");
  GtkWidget *personal_legal_id_entry = glade_xml_get_widget(xml, "dni_entry");
  GtkWidget *trade_name_label = glade_xml_get_widget(xml, "razonsocial_label");
  GtkWidget *trade_name_entry = glade_xml_get_widget(xml, "razonsocial_entry");
  GtkWidget *enterprise_legal_id_label = glade_xml_get_widget(xml, "cif_label");
  GtkWidget *enterprise_legal_id_entry = glade_xml_get_widget(xml, "cif_entry");
  GtkWidget *legal_id_label = glade_xml_get_widget(xml, "cifnif_label");
  GtkWidget *legal_id_entry = glade_xml_get_widget(xml, "cifnif_entry");
  GtkWidget *nick_label = glade_xml_get_widget(xml, "nick_label");
  GtkWidget *nick_entry = glade_xml_get_widget(xml, "nick_entry");


  gtk_widget_hide(name_label);
  gtk_widget_hide(name_entry);

  gtk_widget_hide(surname1_label);
  gtk_widget_hide(surname1_entry);

  gtk_widget_hide(surname2_label);
  gtk_widget_hide(surname2_entry);

  gtk_widget_hide(personal_legal_id_label);
  gtk_widget_hide(personal_legal_id_entry);
  
  gtk_widget_show(trade_name_label);
  gtk_widget_show(trade_name_entry);

  gtk_widget_show(enterprise_legal_id_label);
  gtk_widget_show(enterprise_legal_id_entry);

  gtk_widget_hide(legal_id_label);
  gtk_widget_hide(legal_id_entry);

  gtk_widget_hide(nick_label);
  gtk_widget_hide(nick_entry);

}

static void show_nicks(){
  GtkWidget *name_label = glade_xml_get_widget(xml, "nombre_label");
  GtkWidget *surname1_label = glade_xml_get_widget(xml, "apellido1_label");
  GtkWidget *surname2_label = glade_xml_get_widget(xml, "apellido2_label");
  GtkWidget *name_entry = glade_xml_get_widget(xml, "nombre_entry");
  GtkWidget *surname1_entry = glade_xml_get_widget(xml, "apellido1_entry");
  GtkWidget *surname2_entry = glade_xml_get_widget(xml, "apellido2_entry");
  GtkWidget *personal_legal_id_label = glade_xml_get_widget(xml, "dni_label");
  GtkWidget *personal_legal_id_entry = glade_xml_get_widget(xml, "dni_entry");
  GtkWidget *trade_name_label = glade_xml_get_widget(xml, "razonsocial_label");
  GtkWidget *trade_name_entry = glade_xml_get_widget(xml, "razonsocial_entry");
  GtkWidget *enterprise_legal_id_label = glade_xml_get_widget(xml, "cif_label");
  GtkWidget *enterprise_legal_id_entry = glade_xml_get_widget(xml, "cif_entry");
  GtkWidget *legal_id_label = glade_xml_get_widget(xml, "cifnif_label");
  GtkWidget *legal_id_entry = glade_xml_get_widget(xml, "cifnif_entry");
  GtkWidget *nick_label = glade_xml_get_widget(xml, "nick_label");
  GtkWidget *nick_entry = glade_xml_get_widget(xml, "nick_entry");

  gtk_widget_hide(name_label);
  gtk_widget_hide(name_entry);

  gtk_widget_hide(surname1_label);
  gtk_widget_hide(surname1_entry);

  gtk_widget_hide(surname2_label);
  gtk_widget_hide(surname2_entry);

  gtk_widget_hide(personal_legal_id_label);
  gtk_widget_hide(personal_legal_id_entry);
  
  gtk_widget_hide(trade_name_label);
  gtk_widget_hide(trade_name_entry);

  gtk_widget_hide(enterprise_legal_id_label);
  gtk_widget_hide(enterprise_legal_id_entry);

  gtk_widget_show(legal_id_label);
  gtk_widget_show(legal_id_entry);

  gtk_widget_show(nick_label);
  gtk_widget_show(nick_entry);

}

static void open_insert_client(GtkWidget *widget, gpointer user_data){

  GtkWidget * app1 = lookup_widget(GTK_WIDGET(widget), "app1");
  GtkWidget * client_insertion = NULL;
  
  client_insertion = c_clients_open_insertion_window(app1);

  gtk_window_set_modal(GTK_WINDOW(client_insertion),1);
  gtk_window_set_position(GTK_WINDOW(client_insertion),GTK_WIN_POS_CENTER);
  gtk_window_set_transient_for(GTK_WINDOW(client_insertion),
			       GTK_WINDOW(app1));

  gtk_widget_show(client_insertion);
  
}

static void open_edit_client(GtkWidget *widget, gpointer user_data){

  GtkWidget * app1 = lookup_widget(GTK_WIDGET(widget), "app1");
  GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "person_clist"));
  GtkWidget * client_edition = NULL;
  GArray *selected = c_util_get_selected_rows_from_clist (list);
  if (selected->len>0){
    gint row = g_array_index(selected,gint,0);
    
#ifdef IS_SHOP
    gulong code_gulong = d_participant_get_code (clients_data[row]);
    gint type = m_client_get_type (code_gulong);
    
    if ((type != M_CLIENT_SUPPLIER_CODE)&&(type != M_CLIENT_COMPANY_CODE)&&(type != M_CLIENT_CONSORTIUM_CODE)) {
#endif

    client_edition = c_clients_open_edition_window(app1, NULL);

    c_edit_clients_load_person(clients_data[row]);
    
    g_array_free(selected, TRUE);
    
    gtk_window_set_modal(GTK_WINDOW(client_edition),1);
    gtk_window_set_position(GTK_WINDOW(client_edition),GTK_WIN_POS_CENTER);
    gtk_window_set_transient_for(GTK_WINDOW(client_edition),
				 GTK_WINDOW(app1));
    gtk_widget_show(client_edition);
#ifdef IS_SHOP
  }
#endif  

  }
}

static void find_persons_key_pressed(GtkWidget *widget, GdkEventKey *event, gpointer user_data){

  if (event->keyval==GDK_Return)
    c_clients_control_find_persons ();
}

static void on_find_persons_button_clicked(GtkButton *button,gpointer user_data){

    c_clients_control_find_persons ();
}

void c_clients_control_find_persons (){
  GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "person_clist"));
  gpointer data = NULL;
  Participant *participant = d_participant_create_empty ();
  GtkWidget *combo = glade_xml_get_widget (xml, "person_combo_find_clients");
  gulong type = c_util_get_code_from_combo (combo);
  gchar *street = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "calle_entry")),0,-1);
  gchar *number = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "numero_entry")),0,-1);
  gchar *floor = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "piso_entry")),0,-1);
  gchar *cp = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cp_entry")),0,-1);
  gchar *town = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "municipio_entry")),0,-1);
  gchar *province = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "provincia_entry")),0,-1);
  gchar *phone1 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "telefono1_entry")),0,-1);
  gchar *phone2 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "telefono2_entry")),0,-1);
  gchar *fax = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "fax_entry")),0,-1);
  gchar *email = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "email_entry")),0,-1);
  Participant **participants = NULL;
  GdkCursor * cursor;
  GtkWidget * window;

  window = lookup_widget(GTK_WIDGET(list),"app1");
  cursor = gdk_cursor_new(GDK_WATCH);
  gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
  gdk_cursor_destroy(cursor);

  while (gtk_events_pending())
      gtk_main_iteration();

  if (g_strcasecmp(email,""))
    d_participant_set_email (participant, email);
  if (g_strcasecmp(phone1,""))
    d_participant_set_phone (participant, phone1);
  if (g_strcasecmp(phone2,""))
    d_participant_set_phone2 (participant,phone2);
  if (g_strcasecmp(fax,""))
    d_participant_set_fax (participant,fax);
  if (g_strcasecmp(street,""))
    d_participant_set_street (participant,street);
  if (g_strcasecmp(number,""))
    d_participant_set_number (participant,number);
  if (g_strcasecmp(floor,""))
    d_participant_set_floor (participant, floor);
  if (g_strcasecmp(cp,""))
    d_participant_set_postal_code (participant, atol(cp));
  if (g_strcasecmp(town,""))
    d_participant_set_town (participant, town);
  if (g_strcasecmp(province,""))
    d_participant_set_province (participant, province);

  switch (type)
    {
    case M_CLIENT_PERSON_CODE:{
      gchar *personal_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dni_entry")),0,-1);
      gchar *name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nombre_entry")),0,-1);
      gchar *surname1 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "apellido1_entry")),0,-1);
      gchar *surname2 = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "apellido2_entry")),0,-1);      
  
      data = d_particular_create_empty ();
      
      if (g_strcasecmp(personal_legal_id,""))
	d_participant_set_legal_id (participant, personal_legal_id);
      if (g_strcasecmp(name,"")) 
	 d_participant_set_name (participant, name); 
      if (g_strcasecmp(surname1,"")) 
	 d_particular_set_first_surname ((ParticularObject *) data, surname1); 
      if (g_strcasecmp(surname2,"")) 
	 d_particular_set_second_surname ((ParticularObject *) data, surname2); 


      g_free(personal_legal_id);
      g_free(name);
      g_free(surname1);
      g_free(surname2);

      break;}
    case M_CLIENT_ENTERPRISE_CODE:
    case M_CLIENT_SUPPLIER_CODE:
    case M_CLIENT_COMPANY_CODE:
    case M_CLIENT_CONSORTIUM_CODE:
    case M_CLIENT_GARAGE_CODE:{
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cif_entry")),0,-1);
      gchar *name = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "razonsocial_entry")),0,-1);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(name,""))
	d_participant_set_name (participant, name);

      g_free(enterprise_legal_id);
      g_free(name);

      break;}
    case M_CLIENT_ALL_PARTICIPANT_CODE:{
      gchar *enterprise_legal_id = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "cifnif_entry")),0,-1);
      gchar *nick = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "nick_entry")),0,-1);

      if (g_strcasecmp(enterprise_legal_id,""))
	d_participant_set_legal_id (participant, enterprise_legal_id);
      if (g_strcasecmp(nick,""))
	d_participant_set_nick (participant, nick);

      g_free(enterprise_legal_id);
      g_free(nick);

      break;
    }
    }
  
  g_free(street);
  g_free(number);
  g_free(email);
  g_free(fax);
  g_free(phone2);
  g_free(phone1);
  g_free(province);
  g_free(town);
  g_free(cp);
  g_free(floor);

  
  participants = m_client_find_participants (participant, type, data);

  fill_person_list(list,participants);

  d_participant_clear (participant);

  if (clients_data!=NULL)
      m_client_clear_participant (clients_data);
  clients_data = participants;

/*   m_client_clear_participant (participants);  */

  switch (type)
    {
    case M_CLIENT_PERSON_CODE:{ d_particular_clear ((ParticularObject *) data);
      break;}
    }
/*     case M_CLIENT_ENTERPRISE_CODE: */
/*     case M_CLIENT_SUPPLIER_CODE: */
/*     case M_CLIENT_COMPANY_CODE: */
/*     case M_CLIENT_CONSORTIUM_CODE: */
/*     case M_CLIENT_GARAGE_CODE: */
  cursor = gdk_cursor_new(GDK_LEFT_PTR);
  gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
  gdk_cursor_destroy(cursor);

  cursor = gdk_cursor_new(GDK_LEFT_PTR);
  gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
  gdk_cursor_destroy(cursor);

  g_print(_("People search\n"));

}

static void clean_screen(GtkButton *button,gpointer user_data){

  GtkWidget *container = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

  if (clients_data!=NULL)
    m_client_clear_participant (clients_data);

  clients_data=NULL;

  gtk_container_foreach(GTK_CONTAINER(container),&c_util_clean_widget,NULL);

}

static void fill_person_list(GtkCList * list, Participant **participants) {

    gint index=0;
    gchar *empty_row[]={"","","","",""};

    gtk_clist_clear(GTK_CLIST(list));

    if (participants!=NULL) {
        while (participants[index]!=NULL) {
	  gchar *phone1 = d_participant_get_phone (participants[index]);
	  gchar *phone2 = d_participant_get_phone2 (participants[index]);
	  gchar *email = d_participant_get_email (participants[index]);
	  gchar *enterprise_legal_id = d_participant_get_legal_id (participants[index]);
	  gchar *nick = d_participant_get_nick (participants[index]);	  
	  
            if (index==30) gtk_clist_freeze ((GtkCList *) list);

            gtk_clist_append ((GtkCList *) list,empty_row);

            if (enterprise_legal_id!=NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_LEGAL_ID_COLUMN, enterprise_legal_id);
            }

            if (nick!=NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_NAME_COLUMN, nick);
            }

            if (phone1!=NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_PHONE1_COLUMN, phone1);
            }

            if (phone2!=NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_PHONE2_COLUMN, phone2);
            }

            if (email!=NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_EMAIL_COLUMN, email);
            }	    

            index++;
	    g_free(phone1);
	    g_free(phone2);
	    g_free(email);
	    g_free(enterprise_legal_id);
	    g_free(nick);	    

        }
    }
    gtk_clist_thaw ((GtkCList *) list);
}

static void on_person_list_select_row (GtkCList *list, gint row, gint column, GdkEvent *event, gpointer user_data) {

  if (event!=NULL)
    if (event->type == GDK_2BUTTON_PRESS)
      {
	open_edit_client((GtkWidget *) list, user_data);
      }
  
}

static void on_person_listing_destroy(GtkObject *object, gpointer user_data){

  
    m_client_clear_participant (clients_data);
    clients_data = NULL;

}

static void on_repair_button_clicked(GtkButton *button,gpointer user_data){

  GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
  GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");
  GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "person_clist"));
  GArray *selected = c_util_get_selected_rows_from_clist (list);
  GtkWidget *combo = glade_xml_get_widget (xml, "person_combo_find_clients");
  gulong type_combo = c_util_get_code_from_combo (combo);

  if (selected->len>0){
    gint row = g_array_index(selected,gint,0);
    Participant *participant = d_participant_duplicate (clients_data[row]);
    gulong code_gulong = d_participant_get_code (participant);
    Client *client = m_client_get_by_code (code_gulong);

    if (client!=NULL)
      d_client_set_participant (client, participant);

    if (only_supplier == FALSE) {

	c_repair_open_window(app1, vbox);
	
	c_repair_load_client(client, TRUE);

	i_shortcut_change_dialog (GTK_WIDGET(button),0,1);
    } else {
	if (type_combo==M_CLIENT_SUPPLIER_CODE) {
	    function_to_call(app1, vbox, participant);
	    only_supplier = FALSE;

	    i_shortcut_change_dialog (GTK_WIDGET(button), group_num, item_num);
	} else {
            c_util_open_error_window (_("You can't insert clients that are not suppliers at least in one order"));
	}
    }
    if (only_supplier!=TRUE)
	function_to_call = NULL;
    
  }
    g_array_free(selected, TRUE);
  
}

void c_clients_control_fill_window (Participant ** participants) {
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "person_clist"));
    GtkWidget *entry_type = glade_xml_get_widget (xml, "person_entry_find_person");

    gtk_entry_set_text(GTK_ENTRY(entry_type),_(C_ALL_PARTICIPANTS_MESSAGE));
    if (clients_data!=NULL)
	m_client_clear_participant (clients_data);
    clients_data = participants;
    fill_person_list(list,clients_data);

}


