/* control_util.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#include "config.h"
#include "util.h"
#include "control_util.h"
#include "support.h"
#include "listing_window_control.h"

#include <../interface/printing/listing-print-wrapper.h>

void c_util_clean_widget (GtkWidget * wid,gpointer data) {

    GtkType type ;

    if (strcmp(gtk_type_name(GTK_WIDGET_TYPE(wid)),"GtkCombo"))
    {
	type = gtk_type_from_name("GtkContainer");
	if (GTK_IS_CONTAINER(wid)) {
	    gtk_container_foreach(GTK_CONTAINER(wid),&c_util_clean_widget,NULL);
	}
	if (GNOME_IS_DATE_EDIT(wid)) {
            time_t time_aux=time(NULL);
	    gnome_date_edit_set_time(GNOME_DATE_EDIT(wid),time_aux);
	}
	if (GTK_IS_EDITABLE(wid)) {
	    gtk_editable_delete_text(GTK_EDITABLE(wid),0,-1);
	} else if (!strcmp(gtk_type_name(GTK_WIDGET_TYPE(wid)),"GtkCList")) {
	    gtk_clist_clear(GTK_CLIST(wid));
	} else if (GTK_IS_TOGGLE_BUTTON(wid)) {
	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid),FALSE);
	}
    }
}

void c_util_set_actual_date (GtkEntry * wid) {
    time_t timer;
    gchar *timer_char;
    size_t stored;

    timer_char=(gchar *) g_malloc(sizeof(gchar)*100);
    timer=time(NULL);
    stored=strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d",localtime(&timer));

    gtk_entry_set_text(wid,timer_char);

    g_free(timer_char);

}

gint c_util_days_between_dates_in_month (time_t * time_ant,time_t * time_pos) {
    gchar * day_ant=(gchar *) g_malloc0(sizeof(gchar)*3);
    gchar * day_pos=(gchar *) g_malloc0(sizeof(gchar)*3);
    gchar * month_ant=(gchar *) g_malloc0(sizeof(gchar)*3);
    gchar * month_pos=(gchar *) g_malloc0(sizeof(gchar)*3);
    size_t stored_ant;
    size_t stored_pos;
    gint gint_day_ant;
    gint gint_day_pos;
    gint gint_month_ant;
    gint gint_month_pos;
    gint days;

    stored_ant=strftime(day_ant,sizeof(gchar)*3,"%d",localtime(time_ant));
    stored_pos=strftime(day_pos,sizeof(gchar)*3,"%d",localtime(time_pos));

    stored_ant=strftime(month_ant,sizeof(gchar)*3,"%m",localtime(time_ant));
    stored_pos=strftime(month_pos,sizeof(gchar)*3,"%m",localtime(time_pos));

    gint_month_ant = atoi(month_ant);
    gint_month_pos = atoi(month_pos);
    gint_day_ant = atoi(day_ant);
    gint_day_pos = atoi(day_pos);
    if (gint_month_pos-gint_month_ant==0) {
	days = gint_day_pos-gint_day_ant;
    } else {
        days = 0;
    }
    return days;
}
void c_util_set_code_and_name_on_combo (gulong code,gchar * name,GtkCombo * combo) {

    if (name!=NULL)
    {
	GtkWidget *item;
	GtkWidget *label1,*label2;


	GtkWidget *hbox;

	gchar * auxName;
	gchar * auxCode;

	auxName = g_strdup(name);

	auxCode = g_strdup_printf("%lu",code);

	item = gtk_list_item_new();

	gtk_widget_show(item);
	label1= gtk_label_new (auxCode);
	gtk_widget_set_name(label1,"code");
	label2= gtk_label_new (auxName);
	gtk_widget_set_name(label2,"name");
	gtk_widget_show (label2);


	hbox = gtk_hbox_new (FALSE, 3);
	gtk_widget_set_name(hbox,"hbox");
	gtk_container_add (GTK_CONTAINER (item), hbox);
	gtk_widget_show (hbox);


	gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
	gtk_combo_set_item_string(combo,GTK_ITEM(item),auxName);


	gtk_container_add (GTK_CONTAINER (GTK_COMBO (combo)->list), item);
	g_free(auxName);
	g_free(auxCode);
    }
}


void c_util_add_days(time_t *time, int days) {

    struct tm * time_tm = localtime(time);

    time_tm->tm_mday = time_tm->tm_mday + days;       
    
    *time = mktime(time_tm);
}


void c_util_set_codechar_and_name_on_combo (gchar *code,gchar * name,GtkCombo * combo) {

    if (name!=NULL)
    {
	GtkWidget *item;
	GtkWidget *label1,*label2;


	GtkWidget *hbox;

	gchar * aux_name;
	gchar * aux_code;

	aux_name = g_strdup(name);

	aux_code = g_strdup(code);

	item = gtk_list_item_new();

	gtk_widget_show(item);
	label1= gtk_label_new (aux_code);

	gtk_widget_set_name(label1,"code");
	label2= gtk_label_new (aux_name);
	gtk_widget_set_name(label2,"name");
	gtk_widget_show (label2);


	hbox = gtk_hbox_new (FALSE, 3);
	gtk_widget_set_name(hbox,"hbox");
	gtk_container_add (GTK_CONTAINER (item), hbox);
	gtk_widget_show (hbox);


	gtk_box_pack_start (GTK_BOX (hbox), label2, FALSE, FALSE, 0);
	gtk_box_pack_start (GTK_BOX (hbox), label1, FALSE, FALSE, 0);
	gtk_combo_set_item_string(combo,GTK_ITEM(item),aux_name);


	gtk_container_add (GTK_CONTAINER (GTK_COMBO (combo)->list), item);
/* 	g_free(auxName); */
/* 	g_free(auxCode); */
    }
}

gulong c_util_get_code_from_combo (GtkWidget * combo) {
    gulong code = ULONG_MAX;
    GList * selected = GTK_LIST((GTK_COMBO(combo)->list))->selection;
    GtkItem * item;
    GList * list;
    GList * aux_list;
    GtkWidget * hbox;
    GtkWidget * label;
    gchar * name;
    gchar **gchar_code;

    gchar_code = (gchar **) g_malloc0(sizeof(gchar *));
    if (selected!=NULL) {
	item = GTK_ITEM(selected->data);
	list= gtk_container_children(GTK_CONTAINER(item));
	hbox = (GtkWidget *) list -> data;
	list= gtk_container_children(GTK_CONTAINER(hbox));

	aux_list = g_list_first(list);
	/*        first_element=aux_list;

	do {
	if (aux_list!= first_element)


	} while (aux_list!=g_list_last(list));

	*/
	while (aux_list!=g_list_last(list)) {
	    label = (GtkWidget *) aux_list->data;
	    name = gtk_widget_get_name(label);
	    if (!g_strcasecmp(name,"code")) {
		gchar ** stop;

		stop = (gchar **) g_malloc0(sizeof(gchar *));

		gtk_label_get(GTK_LABEL(label),gchar_code);
//		code = atoi(*gchar_code);
		code = strtoul(*gchar_code,stop,10);

		g_free(stop);

	    }
            //g_free(name);
	    aux_list = g_list_next(aux_list);
	}
	label = (GtkWidget *) aux_list->data;
	name = gtk_widget_get_name(label);
	if (!g_strcasecmp(name,"code")) {
	  gchar ** stop;

	  stop = (gchar **) g_malloc0(sizeof(gchar *));

	  gtk_label_get(GTK_LABEL(label),gchar_code);
	  code = strtoul(*gchar_code,stop,10);

	  g_free(stop);
	}
     //   g_free(name);
	g_free(gchar_code);
    }


    return code;
}

gchar *c_util_get_name_of_codechar_from_combo (GtkWidget * combo, gchar *code) {
    GList * list = GTK_LIST((GTK_COMBO(combo)->list))->children;
    GList * combo_list = NULL;
    GList * aux_combo_list = NULL;
    GtkItem * item = NULL;
    GList * aux_list = NULL;
    GtkWidget * hbox = NULL;
    GtkWidget * label = NULL;
    gchar * name = NULL;
    gchar **gchar_code = (gchar **) g_malloc0(sizeof(gchar *));
    gchar **gchar_name = (gchar **) g_malloc0(sizeof(gchar *));


    if (list!=NULL) {
	aux_combo_list = g_list_first(list);

	*gchar_code="";

	while(aux_combo_list!=g_list_last(list) && (g_strcasecmp((*gchar_code),code))) {
            item = GTK_ITEM(aux_combo_list->data);
	    combo_list= gtk_container_children(GTK_CONTAINER(item));
	    hbox = (GtkWidget *) combo_list -> data;
	    combo_list= gtk_container_children(GTK_CONTAINER(hbox));

	    aux_list = g_list_first(combo_list);

            while (aux_list!=g_list_last(combo_list) && (g_strcasecmp((*gchar_code),code))) {
		label = (GtkWidget *) aux_list->data;
		name = gtk_widget_get_name(label);
		if (!g_strcasecmp(name,"name")) {
		    gtk_label_get(GTK_LABEL(label),gchar_name);
		}
		if (!g_strcasecmp(name,"code")) {
		    gtk_label_get(GTK_LABEL(label),gchar_code);
		}
		aux_list = g_list_next(aux_list);
	    }

	    if (g_strcasecmp((*gchar_code),code)) {
		label = (GtkWidget *) aux_list->data;
		name = gtk_widget_get_name(label);
		if (!g_strcasecmp(name,"name")) {
		    gtk_label_get(GTK_LABEL(label),gchar_name);
		}
		if (!g_strcasecmp(name,"code")) {
		    gtk_label_get(GTK_LABEL(label),gchar_code);
		}
	    }

	    aux_combo_list = g_list_next(aux_combo_list);
	}

	if (g_strcasecmp((*gchar_code),code)) {
	    item = GTK_ITEM(aux_combo_list->data);
	    combo_list= gtk_container_children(GTK_CONTAINER(item));
	    hbox = (GtkWidget *) combo_list -> data;
	    combo_list= gtk_container_children(GTK_CONTAINER(hbox));

	    aux_list = g_list_first(combo_list);

	    while (aux_list!=g_list_last(combo_list) && (g_strcasecmp((*gchar_code),code))) {
		label = (GtkWidget *) aux_list->data;
		name = gtk_widget_get_name(label);
		if (!g_strcasecmp(name,"name")) {
		    gtk_label_get(GTK_LABEL(label),gchar_name);
		}
		if (!g_strcasecmp(name,"code")) {
		    gtk_label_get(GTK_LABEL(label),gchar_code);
		}
		aux_list = g_list_next(aux_list);
	    }

	    if (g_strcasecmp((*gchar_code),code)) {
		label = (GtkWidget *) aux_list->data;
		name = gtk_widget_get_name(label);
		if (!g_strcasecmp(name,"name")) {
		    gtk_label_get(GTK_LABEL(label),gchar_name);
		}
		if (!g_strcasecmp(name,"code")) {
		    gtk_label_get(GTK_LABEL(label),gchar_code);
		}
	    }
	}
    }
    return (*gchar_name);
}

gchar *c_util_get_code_from_combo_as_gchar (GtkWidget * combo) {
    GList * selected = GTK_LIST((GTK_COMBO(combo)->list))->selection;
    GtkItem * item;
    GList * list;
    GList * aux_list;
    GtkWidget * hbox;
    GtkWidget * label;
    gchar * name;
    gchar **gchar_code;

    gchar_code = (gchar **) g_malloc(sizeof(gchar *));
    if (selected!=NULL) {
	item = GTK_ITEM(selected->data);
	list= gtk_container_children(GTK_CONTAINER(item));
	hbox = (GtkWidget *) list -> data;
	list= gtk_container_children(GTK_CONTAINER(hbox));

	aux_list = g_list_first(list);

	while (aux_list!=g_list_last(list)) {
	    label = (GtkWidget *) aux_list->data;
	    name = gtk_widget_get_name(label);
	    if (!g_strcasecmp(name,"code")) {
		gtk_label_get(GTK_LABEL(label),gchar_code);
	    }
	    aux_list = g_list_next(aux_list);
	}
	label = (GtkWidget *) aux_list->data;
	name = gtk_widget_get_name(label);
	if (!g_strcasecmp(name,"code")) {
	    gtk_label_get(GTK_LABEL(label),gchar_code);
	}
    }

    return (*gchar_code);
}

void c_util_fill_combo_with_shops (GtkWidget * combo, Shop ** shops) {
    gchar * aux;
    gint index;

    if (shops!=NULL) {
	index = 0;
	while (shops[index]!=NULL) {
	    aux=d_participant_get_name (d_shop_get_participant (shops[index]));
	    c_util_set_code_and_name_on_combo (d_participant_get_code (d_shop_get_participant (shops[index])),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}

    }

}

void c_util_fill_combo_with_families (GtkWidget * combo, Family ** families) {
    gchar * code;
    gchar * description;
    gint index;

    if (families!=NULL) {
	index = 0;
	while (families[index]!=NULL) {
	    code=d_family_get_code (families[index]);
	    description=d_family_get_description (families[index]);
	    c_util_set_codechar_and_name_on_combo (code, description, GTK_COMBO(combo));
	    g_free(code);
	    g_free(description);
	    index++;
	}

    }

}

void c_util_fill_combo_with_policies (GtkWidget * combo,Policy ** policies) {
    gchar * aux;
    gint index;

    if (policies!=NULL) {
	index = 0;
	while (policies[index]!=NULL) {
	    aux=d_policy_get_policy_number (policies[index]);
	    c_util_set_code_and_name_on_combo (d_policy_get_holder_code (policies[index]),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_fill_combo_with_receipts (GtkWidget * combo,Receipt ** receipts) {
    gchar * aux;
    gint index;

    if (receipts!=NULL) {
	index = 0;
	while (receipts[index]!=NULL) {
	    aux=d_receipt_get_receipt_number (receipts[index]);
	    c_util_set_code_and_name_on_combo (d_receipt_get_holder_code (receipts[index]),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_fill_combo_with_vehicles (GtkWidget * combo,Vehicle ** vehicles) {
    gchar * matricula = NULL;
    gulong codigo = ULONG_MAX;
    gint index;

    if (vehicles!=NULL) {
	index = 0;
	while (vehicles[index]!=NULL) {
	    matricula = d_vehicle_get_license_number (vehicles[index]);
	    codigo = d_vehicle_get_code (vehicles[index]);

	    c_util_set_code_and_name_on_combo (codigo,matricula,GTK_COMBO(combo));

	    g_free(matricula);
	    index++;
	}
    }
}

void c_util_fill_combo_with_enterprises (GtkWidget * combo,Enterprise ** enterprises) {
    gchar * aux;
    gint index;

    if (enterprises!=NULL) {
	index = 0;
	while (enterprises[index]!=NULL) {
	    aux=d_participant_get_name (d_enterprise_get_participant (enterprises[index]));
	    c_util_set_code_and_name_on_combo (d_participant_get_code (d_enterprise_get_participant (enterprises[index])),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_fill_combo_with_warehouses(GtkWidget * combo, Warehouse ** warehouses) {
    gchar * aux;
    gint index;

    if (warehouses!=NULL) {
	index = 0;
	while (warehouses[index]!=NULL) {
	    aux=d_warehouse_get_nick (warehouses[index]);
	    c_util_set_code_and_name_on_combo (d_warehouse_get_code (warehouses[index]),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_fill_combo_with_consortiums (GtkWidget * combo,Consortium ** consortiums) {
    gchar * aux;
    gint index=0;

    if (consortiums!=NULL) {
	while (consortiums[index]!=NULL) {
	    aux=d_participant_get_name (d_consortium_get_participant (consortiums[index]));
	    c_util_set_code_and_name_on_combo (d_participant_get_code (d_consortium_get_participant (consortiums[index])),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_fill_combo_with_provinces (GtkWidget * combo,Province ** provinces) {
    gchar * aux;
    gint index=0;

    if (provinces!=NULL) {
	while (provinces[index]!=NULL) {
	    aux=d_province_get_name (provinces[index]);
	    c_util_set_code_and_name_on_combo (d_province_get_province_code (provinces[index]),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_fill_combo_with_companies (GtkWidget * combo,Company ** companies) {
    gchar * aux;
    gint index=0;

    if (companies!=NULL) {
	while (companies[index]!=NULL) {
	    aux=d_participant_get_name (d_company_get_participant (companies[index]));
	    c_util_set_code_and_name_on_combo (d_participant_get_code (d_company_get_participant (companies[index])),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_fill_combo_with_suppliers (GtkWidget * combo,Supplier ** suppliers) {
    gchar * aux;
    gint index=0;

    if (suppliers!=NULL) {
	while (suppliers[index]!=NULL) {
	    aux=d_participant_get_nick (d_supplier_get_participant (suppliers[index]));
	    c_util_set_code_and_name_on_combo (d_participant_get_code (d_supplier_get_participant (suppliers[index])),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

void c_util_clean_code_and_name_combo (GtkWidget * combo) {
    GtkListItem * item;
    GList * first_item;
    GtkList * items =GTK_LIST((GTK_COMBO(combo)->list));
    GList * list;
    GList * aux_list;
    GtkWidget * hbox;
    GtkWidget * label;


    if (items!=NULL) {
	first_item = gtk_container_children(GTK_CONTAINER(items));
	first_item = g_list_first(first_item);
	while (first_item!=g_list_last(first_item)) {
	    item = GTK_LIST_ITEM(first_item->data);
	    list= gtk_container_children(GTK_CONTAINER(item));

	    hbox = (GtkWidget *) list -> data;
	    list= gtk_container_children(GTK_CONTAINER(hbox));

	    aux_list = g_list_first(list);
	    while (aux_list!=g_list_last(list)) {
		label = (GtkWidget *) aux_list->data;
		aux_list = g_list_next(aux_list);
		gtk_widget_destroy(label);
		label=NULL;
	    }
	    label = (GtkWidget *) aux_list->data;
	    gtk_widget_destroy(label);
	    label=NULL;
	    gtk_widget_destroy(hbox);
	    hbox=NULL;
	    first_item = g_list_next(first_item);
	    gtk_container_remove(GTK_CONTAINER(items),GTK_WIDGET(item));
	    //        gtk_widget_destroy(GTK_WIDGET(item));

	}

	if (first_item!=NULL) {
	    item = GTK_LIST_ITEM(first_item->data);
	    list= gtk_container_children(GTK_CONTAINER(item));
	    hbox = (GtkWidget *) list -> data;

	    list= gtk_container_children(GTK_CONTAINER(hbox));

	    aux_list = g_list_first(list);
	    while (aux_list!=g_list_last(list)) {
		label = (GtkWidget *) aux_list->data;
		aux_list = g_list_next(aux_list);
		gtk_widget_destroy(label);
		label=NULL;

	    }


	    label = (GtkWidget *) aux_list->data;
	    gtk_widget_destroy(label);

	    label=NULL;

	    if (hbox!=NULL)
		gtk_widget_destroy(hbox);


	    gtk_container_remove(GTK_CONTAINER(items),GTK_WIDGET(item));

	    //    gtk_widget_destroy(GTK_WIDGET(item));

	    hbox=NULL;

	}
    }

    item=NULL;
    first_item = NULL;
    items=NULL;


}

GArray * c_util_get_selected_rows_from_clist (GtkCList * list) {
    GList * actual ;
    gint is_selected = 0;
    gint index_actual=0;
    GArray *selected=g_array_new(FALSE, FALSE, sizeof(gint));

    actual = g_list_first(list->row_list);
    while (actual!=NULL) {

	if ((GTK_CLIST_ROW(actual)->state)==GTK_STATE_SELECTED) {
	    is_selected = 1;
	    g_array_append_val(selected,index_actual);
	}
	actual = g_list_next(actual);
	index_actual++;

    }
    return selected;
}


GArray * c_util_get_rows_from_clist (GtkCList * list) {
    GList * actual ;
    gint index_actual=0;
    GArray *selected=g_array_new(FALSE, FALSE, sizeof(gint));

    actual = g_list_first(list->row_list);
    while (actual!=NULL) {
	g_array_append_val(selected,index_actual);
	actual = g_list_next(actual);
	index_actual++;
    }
    return selected;
}


void c_util_open_listing_window (Listing * listado, char *message, gpointer callback) {
	
	GtkWidget * window;
	//gint cols = listado->n_columns;
	gint rows = listado->n_rows;
	
	window = c_listing_open_window(message, callback);
  	c_listing_fill_generic_list(listado, rows);
	
        gtk_window_set_modal(GTK_WINDOW(window),1);
	
	gtk_widget_show(window);
}



void c_util_open_question_dialog(const gchar *question, GnomeReplyCallback callback){

  GtkWidget *question_window = gnome_question_dialog (question, callback, NULL);

  if (question_window != NULL) {
    gtk_window_set_modal(GTK_WINDOW(question_window),1);
    gtk_window_set_position(GTK_WINDOW(question_window),GTK_WIN_POS_CENTER);
    gnome_dialog_run_and_close(GNOME_DIALOG(question_window));
  }

}

void c_util_open_question_dialog_with_param (const gchar *question, GnomeReplyCallback callback,gpointer user_data){

  GtkWidget *question_window = gnome_question_dialog (question, callback, user_data);

  if (question_window != NULL) {
    gtk_window_set_modal(GTK_WINDOW(question_window),1);
    gtk_window_set_position(GTK_WINDOW(question_window),GTK_WIN_POS_CENTER);
    gnome_dialog_run_and_close(GNOME_DIALOG(question_window));
  }

}

void c_util_open_error_window (const gchar * message) {

    GtkWidget * modal_window = gnome_message_box_new (message,
					  GNOME_MESSAGE_BOX_ERROR,
					  _("Ok"), NULL);

    if (modal_window !=NULL) {
	gtk_window_set_modal(GTK_WINDOW(modal_window),1);
	gtk_window_set_position(GTK_WINDOW(modal_window),GTK_WIN_POS_CENTER);
	gnome_dialog_run_and_close(GNOME_DIALOG(modal_window));

    }

}

void c_util_open_info_window (const gchar * message) {

    GtkWidget * modal_window = gnome_message_box_new (message,
						      GNOME_MESSAGE_BOX_INFO,
					  _("Ok"), NULL);

    if (modal_window !=NULL) {
	gtk_window_set_modal(GTK_WINDOW(modal_window),1);
	gtk_window_set_position(GTK_WINDOW(modal_window),GTK_WIN_POS_CENTER);
	gnome_dialog_run_and_close(GNOME_DIALOG(modal_window));

    }
}

/*time_t * u_parse_date (gchar * date) {
    time_t * time_of_gchar= (time_t *) g_malloc0(sizeof(time_t));
    gchar * delimiters ="/ :";
    gchar new_delimiter = '-';
    gchar * date_to_convert;
    struct  tm * time_tm = (struct tm *) g_malloc0(sizeof(struct tm));
    gchar ** tokens = (gchar**) g_malloc0(sizeof(gchar *));

    date_to_convert = g_strdup(date);
    g_strdelimit(date_to_convert,delimiters,new_delimiter);

    tokens = g_strsplit(date_to_convert,"-",6);
    time_tm ->  tm_sec = atoi(tokens[5]);
    time_tm ->  tm_min = atoi(tokens[4]);
    time_tm ->  tm_hour = atoi(tokens[3]);
    time_tm ->  tm_mon = atoi(tokens[0])-1;
    time_tm ->  tm_year = atoi(tokens[2])-1900;
    time_tm ->  tm_mday = atoi(tokens[1]);

    *time_of_gchar = mktime(time_tm);
    g_free(date_to_convert);
    g_strfreev(tokens);
    g_free(time_tm);

    return time_of_gchar;
}
  */
gint c_util_get_days_to_last_day_of_month (time_t * actual_time){
    struct tm * temp;
    gint day;
    gint month;
    gint year;
    GDateMonth g_date_month;
    GDateYear g_date_year;
    gint lastday ;


    temp = localtime(actual_time);
    day = temp->tm_mday;
    month = temp->tm_mon;
    year = temp->tm_year;
    g_date_month=temp->tm_mon+1;
    g_date_year=temp->tm_year;
    lastday =  g_date_days_in_month(g_date_month,
				    g_date_year);
    return (lastday-day);
}

gchar * c_util_get_card_number (gint first_number,gint second_number,gint third_number,gint fourth_number) {

    gchar * one_number;
    gchar * number_aux;
    gchar * number;
    if (first_number < 10) {
	gchar * temp = g_strdup_printf("%d",first_number);
	one_number = g_strconcat("000",temp,NULL);
	g_free(temp);
    } else if ((first_number>10) && (first_number<100)){
	gchar * temp = g_strdup_printf("%d",first_number);
	one_number = g_strconcat("00",temp,NULL);
	g_free(temp);
    } else if ((first_number>100) && (first_number<1000)){
	gchar * temp = g_strdup_printf("%d",first_number);
	one_number = g_strconcat("0",temp,NULL);
	g_free(temp);
    } else {
	one_number = g_strdup_printf("%d",first_number);
    }
    g_strstrip(one_number);
    number = g_strconcat(one_number,NULL);
    g_free(one_number);
    if (second_number < 10) {
	gchar * temp = g_strdup_printf("%d",second_number);
	one_number = g_strconcat("000",temp,NULL);
	g_free(temp);
    } else if ((second_number>10) && (second_number<100)){
	gchar * temp = g_strdup_printf("%d",second_number);
	one_number = g_strconcat("00",temp,NULL);
	g_free(temp);
    } else if ((second_number>100) && (second_number<1000)){
	gchar * temp = g_strdup_printf("%d",second_number);
	one_number = g_strconcat("0",temp,NULL);
	g_free(temp);
    } else {
	one_number = g_strdup_printf("%d",second_number);
    }
    g_strstrip(one_number);
    number_aux = g_strconcat(number," ",one_number,NULL);
    g_free(number);
    g_free(one_number);
    number = number_aux;
    number_aux = NULL;
    if (third_number < 10) {
	gchar * temp = g_strdup_printf("%d",third_number);
	one_number = g_strconcat("000",temp,NULL);
	g_free(temp);
    } else if ((third_number>10) && (third_number<100)){
	gchar * temp = g_strdup_printf("%d",third_number);
	one_number = g_strconcat("00",temp,NULL);
	g_free(temp);
    } else if ((third_number>100) && (third_number<1000)){
	gchar * temp = g_strdup_printf("%d",third_number);
	one_number = g_strconcat("0",temp,NULL);
	g_free(temp);
    } else {
	one_number = g_strdup_printf("%d",third_number);
    }
    g_strstrip(one_number);
    number_aux = g_strconcat(number," ",one_number,NULL);
    g_free(number);
    g_free(one_number);
    number = number_aux;
    number_aux = NULL;

    if (fourth_number < 10) {
	gchar * temp = g_strdup_printf("%d",fourth_number);
	one_number = g_strconcat("000",temp,NULL);
	g_free(temp);
    } else if ((fourth_number>10) && (fourth_number<100)){
	gchar * temp = g_strdup_printf("%d",fourth_number);
	one_number = g_strconcat("00",temp,NULL);
	g_free(temp);
    } else if ((fourth_number>100) && (fourth_number<1000)){
	gchar * temp = g_strdup_printf("%d",fourth_number);
	one_number = g_strconcat("0",temp,NULL);
	g_free(temp);
    } else {
	one_number = g_strdup_printf("%d",fourth_number);
    }
    g_strstrip(one_number);
    number_aux = g_strconcat(number," ",one_number,NULL);
    g_free(number);
    g_free(one_number);
    number = number_aux;
    number_aux = NULL;


    return number;

}

gchar * c_util_get_cheque_number (gint first_number,gint second_number,gint third_number,gint fourth_number) {

    gchar * one_number;
    gchar * number_aux;
    gchar * number;

    one_number = g_strdup_printf("%d",first_number);
    number = g_strconcat(one_number,NULL);
    g_free(one_number);
    if (second_number < 10) {
	gchar * temp = g_strdup_printf("%d",second_number);
	one_number = g_strconcat("00",temp,NULL);
	g_free(temp);
    } else if ((second_number>10) && (second_number<100)){
	gchar * temp = g_strdup_printf("%d",second_number);
	one_number = g_strconcat("0",temp,NULL);
	g_free(temp);
    }else {
	one_number = g_strdup_printf("%d",second_number);
    }
    g_strstrip(one_number);
    number_aux = g_strconcat(number," ",one_number,NULL);
    g_free(number);
    g_free(one_number);
    number = number_aux;
    number_aux = NULL;
    if (third_number < 10) {
	gchar * temp = g_strdup_printf("%d",third_number);
	one_number = g_strconcat("00",temp,NULL);
	g_free(temp);
    } else if ((third_number>10) && (third_number<100)){
	gchar * temp = g_strdup_printf("%d",third_number);
	one_number = g_strconcat("0",temp,NULL);
	g_free(temp);
    } else {
	one_number = g_strdup_printf("%d",third_number);
    }
    g_strstrip(one_number);
    number_aux = g_strconcat(number," ",one_number,NULL);
    g_free(number);
    g_free(one_number);
    number = number_aux;
    number_aux = NULL;

    one_number = g_strdup_printf("%d",fourth_number);

    g_strstrip(one_number);
    number_aux = g_strconcat(number," ",one_number,NULL);
    g_free(number);
    g_free(one_number);
    number = number_aux;
    number_aux = NULL;


    return number;

}

/* THAT FUNCTION IS VALID TO CREDIT CARD NUMBERS AND CHEQUE NUMBERS*/
gchar ** c_util_get_card_numbers (gchar * card_number) {
    gchar * number_to_convert;
    gchar ** tokens =NULL;//= (gchar**) g_malloc0(sizeof(gchar *));

    number_to_convert = g_strdup(card_number);

    tokens = g_strsplit(number_to_convert," ",4);

    g_free(number_to_convert);

    return tokens;

}

void c_util_fill_combo_with_medidas (GtkWidget * combo,Measure ** measures) {
    gchar * aux;
    gint index;

    if (measures!=NULL) {
	index = 0;
	while (measures[index]!=NULL) {
	    aux=d_measure_get_name (measures[index]);
	    c_util_set_code_and_name_on_combo (d_measure_get_measure_code (measures[index]),aux,GTK_COMBO(combo));
	    g_free(aux);
	    index++;
	}
    }
}

    
void c_util_list_select_rows (GtkCList *clist, GArray *rows) {

	gint i = 0;	
	while (i < rows->len) {
	
		gint row = g_array_index(rows,gint,i);
		gtk_clist_select_row(clist, row, 0);		
		
		i++;
	}
}	
			 
