
/* util.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _UTIL_CONTROL_
#define _UTIL_CONTROL_

#include "config.h"
#include <gnome.h>
#include <gtk/gtk.h>

#include "family.h"
#include "shop.h"
#include "supplier.h"
#include "policy.h"
#include "receipt.h"
#include "province.h"
#include "measure.h"
#include "vehicle.h"
#include "enterprise.h"
#include "consortium.h"
#include "warehouse.h"
#include "time.h"
#include "listing_window_control.h"


#include <../interface/printing/listing-print-wrapper.h>

/**
 * c_util_clean_widget:
 * @wid: Widget que se desea limpiar.
 * @data: Puntero que puede ser utilizado por el desarrollador para pasar datos de cualquier tipo.
 *
 * Limpia un #Widget, realizando diferentes operaciones segn el tipo de #Widget.
 * 
 */
void   c_util_clean_widget (GtkWidget * wid,gpointer data);
/**
 * c_util_set_actual_date:
 * @wid: Entrada de texto en la cual se escribir la fecha.
 *
 * Introduce la fecha actual en una entrada de texto con el formato aaaa-mm-dd.
 *
 */
void   c_util_set_actual_date (GtkEntry * wid);
/**
 * c_util_set_code_and_name_on_combo:
 * @codigo: Cdigo que se desea asociar un nombre en un combo.
 * @name: Nombre que se desea que aparezca en un combo.
 * @combo: Combo en el que se insertar el par (cdigo,nombre).
 *
 * Introduce en un #combo una entrada con el par (cdigo,nombre) donde solo se mostrar el
 * nombre pero quedar asociado al cdigo.
 *
 */
void   c_util_set_code_and_name_on_combo (gulong codigo,gchar * name,GtkCombo * combo);
/**
 * c_util_add_days:
 * @time: fecha actual.
 * @days: Numero de dias a aadir.
 * 
 * Aade el numero de dias especificado por parmetro, a una fecha determinada.
 */
void   c_util_add_days (time_t *time, int days);
/**
 * c_util_set_codechar_and_name_on_combo:
 * @codigo: Cdigo en texto que se desea asociar un nombre en un combo.
 * @name: Nombre que se desea que aparezca en un combo.
 * @combo: Combo en el que se insertar el par (cdigo,nombre).
 *
 * Introduce en un #combo una entrada con el par (cdigo,nombre) donde solo se mostrar el
 * nombre pero quedar asociado al cdigo.
 *
 */
void   c_util_set_codechar_and_name_on_combo (gchar *codigo,gchar * name,GtkCombo * combo);
/**
 * c_util_get_code_from_combo:
 * @combo: #Combo del que se desea obtener el cdigo asociado al texto que se est mostrando actualmente en el
 * mismo.
 *
 * Obtiene el cdigo asociado al nombre que aparece en el #combo.
 *
 * Returns:
 * El cdigo asociado.
 */
gulong   c_util_get_code_from_combo (GtkWidget * combo);
/**
 * c_util_get_code_from_combo_as_gchar:
 * @combo: #Combo del que se desea obtener el cdigo asociado al texto que se est mostrando actualmente en el
 * mismo.
 *
 * Obtiene el cdigo asociado al nombre que aparece en el #combo.
 *
 * Returns:
 * El cdigo asociado en formato texto.
 */
gchar *c_util_get_code_from_combo_as_gchar (GtkWidget * combo);
/**
 * c_util_get_name_of_codechar_from_combo:
 * @combo: #Combo del que se desea obtener el texto asociado al cdigo.
 * @code: Cdigo por el que se desea obtener el texto.
 *
 * Devuelve el texto del combo que est asociado a un cdigo. Si existen dos nombres asociados al
 * mismo cdigo devuelve el que est en primer lugar.
 * 
 * Returns:
 * Texto asociado al cdigo.
 * 
 */
gchar *c_util_get_name_of_codechar_from_combo (GtkWidget * combo, gchar *code);
/**
 * c_util_fill_combo_with_families:
 * @combo: #Combo que se desea rellenar.
 * @families: Lista de familias que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las familias de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_families (GtkWidget * combo, Family ** families);
/**
 * c_util_fill_combo_with_shops:
 * @combo: #Combo que se desea rellenar.
 * @shops: Lista de tiendas que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las tiendas de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_shops (GtkWidget * combo,Shop ** shops);
/**
 * c_util_fill_combo_with_enterprises:
 * @combo: #Combo que se desea rellenar.
 * @enterprises: Lista de empresas que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las empresas de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_enterprises (GtkWidget * combo,Enterprise ** enterprises);
/**
 * c_util_fill_combo_with_warehouses:
 * @combo: #Combo que se desea rellenar.
 * @warehouses: Lista de almacenes que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las almacenes de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_warehouses(GtkWidget * combo, Warehouse ** warehouses);
/**
 * c_util_fill_combo_with_consortiums:
 * @combo: #Combo que se desea rellenar.
 * @consortiums: Lista de consorcios que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las consorcios de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_consortiums (GtkWidget * combo,Consortium ** consortiums);
/**
 * c_util_fill_combo_with_companies:
 * @combo: #Combo que se desea rellenar.
 * @companies: Lista de compaas de seguros que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las compaas de seguros de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_companies (GtkWidget * combo,Company ** companies);
/**
 * c_util_fill_combo_with_vehicles:
 * @combo: #Combo que se desea rellenar.
 * @vehicles: Lista de vehculos que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de los vehculos de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_vehicles (GtkWidget * combo,Vehicle ** vehicles);
/**
 * c_util_fill_combo_with_suppliers:
 * @combo: #Combo que se desea rellenar.
 * @suppliers: Lista de proveedores que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de los proveedores de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_suppliers (GtkWidget * combo,Supplier ** suppliers);
/**
 * c_util_clean_code_and_name_combo:
 * @combo: #Combo del que se desean vaciar los pares (cdigo,nombre).
 *
 * Libera la memoria utilizada por la estructura de pares (cdigo,nombre) en el #combo.
 *
 */
void   c_util_clean_code_and_name_combo (GtkWidget * combo);
/**
 * c_util_get_selected_rows_from_clist:
 * @list: Lista mostrada en pantalla de la que se desean extraer las filas seleccionadas por el usuario.
 *
 * Obtiene las filas de la lista que estn seleccionadas por el usuario.
 *
 * Returns:
 * Un array con el ndice de las filas seleccionadas.
 *
 */
GArray *c_util_get_selected_rows_from_clist (GtkCList * list);
/**
 * c_util_get_rows_from_clist:
 * @list: Lista mostrada en pantalla de la que se desean extraer las filas.
 *
 * Obtiene las filas de una lista.
 *
 * Returns:
 * Un array con el ndice de las filas solicitadas.
 * 
 */
GArray * c_util_get_rows_from_clist (GtkCList * list);
/**
 * c_util_open_error_window:
 * @message: Mesaje que se desea mostrar en la ventana de error.
 *
 * Abre una ventana de error en la que se muestra un mensaje indicando el error que
 * se produjo. La ventana constar de un botn "Aceptar" para indicar que el mensaje ha sido ledo.
 *
 */
void c_util_open_error_window (const gchar * message);
/**
 * c_util_open_listing_window:
 * @listado: Datos a mostrar en la clist.
 * @message: Mesaje que se desea mostrar en la ventana de error.
 * @callback: Funcin que se llamar al cerrar el dilogo.
 *
 * Abre una ventana de error en la que se muestra un mensaje indicando el error que
 * se produjo. La ventana constar de un botn "Aceptar" para indicar que el mensaje ha sido ledo.
 *
 */
void c_util_open_listing_window (Listing * listado, char *message, gpointer callback);


/**
 * c_util_open_question_dialog:
 * @question: Mensaje en el que se realizar una pregunta a contestar por el usuario.
 * @callback: Funcin que se llamar al cerrar el dilogo.
 *
 * Abre un dilogo en el que se realizar una pregunta a contestar por el usuario. Al responder
 * est pregunta pulsando "Si" o "No" se llamar a la funcin #callback, que segn la respuesta
 * actuar de un modo u otro.
 *
 */
void c_util_open_question_dialog(const gchar *question, GnomeReplyCallback callback);

/**
 * c_util_open_question_dialog_with_param:
 * @question: Mensaje en el que se realizar una pregunta a contestar por el usuario.
 * @callback: Funcin que se llamar al cerrar el dilogo.
 * @user_data: Parmetros para la callback;
 *
 * Abre un dilogo en el que se realizar una pregunta a contestar por el usuario. Al responder
 * est pregunta pulsando "Si" o "No" se llamar a la funcin #callback, que segn la respuesta
 * actuar de un modo u otro.
 *
 */
void c_util_open_question_dialog_with_param (const gchar *question, GnomeReplyCallback callback,gpointer user_data);
/**
 * c_util_open_info_window:
 * @message: Mesaje que se desea mostrar en la ventana de informacin.
 *
 * Abre una ventana de informacin en la que se muestra un mensaje informativo.
 * La ventana constar de un botn "Aceptar" para indicar que el mensaje ha sido ledo.
 * 
 */
void c_util_open_info_window (const gchar * message);
/**
 * c_util_days_between_dates_in_month:
 * @time_ant: Fecha desde la cual se calcularn los das.
 * @time_pos: Fecha hasta la cual se calcularn los das.
 *
 * Obtiene el nmero de das que hay entre dos das del mes.
 * 
 * Returns:
 * El nmero de das o cero si se trata de diferentes meses.
 * 
 */
gint c_util_days_between_dates_in_month (time_t * time_ant,time_t * time_pos);
/**
 * c_util_get_days_to_last_day_of_month:
 * @actual_time: Fecha a partir de la cual se calcularn los das hasta fin de mes.
 *
 * Calcula los das que restan hasta fin de mes desde la fecha pasa por parmetro.
 *
 * Returns:
 * Das que faltan para llegar al fin del mes de la fecha pasada por parmetro, a partir de esa fecha.
 * 
 */
gint c_util_get_days_to_last_day_of_month (time_t * actual_time);
/**
 * c_util_get_card_number:
 * @first_number: Primer nmero de la tarjeta.
 * @second_number: Segundo nmero de la tarjeta.
 * @third_number: Tercer nmero de la tarjeta.
 * @fourth_number: Cuarto nmero de la tarjeta.
 *
 * Calcula el nmero completo de una tarjeta de crdito a partir de los nmeros en entero.
 *
 * Returns:
 * La cadena de caracteres que contiene el nmero completo de la tarjeta.
 */
gchar * c_util_get_card_number (gint first_number,gint second_number,gint third_number,gint fourth_number);
/**
 * c_util_get_cheque_number:
 * @first_number: Primer nmero del taln.
 * @second_number: Segundo nmero del taln.
 * @third_number: Tercer nmero del taln.
 * @fourth_number: Cuarto nmero del taln.
 *
 * Calcula el nmero completo de un taln a partir de los nmeros en entero.
 *
 * Returns:
 * La cadena de caracteres que contiene el nmero completo del taln.
 */
gchar * c_util_get_cheque_number (gint first_number,gint second_number,gint third_number,gint fourth_number);
/**
 * c_util_get_card_numbers:
 * @card_number: Nmero completo de una tarjera.
 *
 * Obtiene los nmeros de una tarjeta a partir del nmero completo.
 * 
 * Returns:
 * Los nmeros en un array de cadenas de caracteres.
 */
gchar ** c_util_get_card_numbers (gchar * card_number);
/**
 * c_util_fill_combo_with_policies:
 * @combo: #Combo que se desea rellenar.
 * @policies: Lista de plizas de seguro que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las plizas de seguro de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_policies (GtkWidget * combo,Policy ** policies);
/**
 * c_util_fill_combo_with_receipts:
 * @combo: #Combo que se desea rellenar.
 * @receipts: Lista de recibos que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de los recibos de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_receipts (GtkWidget * combo,Receipt ** receipts);
/**
 * c_util_fill_combo_with_provinces:
 * @combo: #Combo que se desea rellenar.
 * @provinces: Lista de provincias que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las provincias de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_provinces (GtkWidget * combo,Province ** provinces);
/**
 * c_util_fill_combo_with_medidas:
 * @combo: #Combo que se desea rellenar.
 * @measures: Lista de medidas que se quieren mostrar en el #combo.
 *
 * Rellena el combo con el cdigo y el nombre de las medidas de la lista pasada por parmetro.
 *
 */
void c_util_fill_combo_with_medidas (GtkWidget * combo,Measure ** measures);
/**
 * c_util_list_select_rows:
 * @clist: #Lista para aplicar la seleccion de filas.
 * @rows: Filas que se queieren seleccionar.
 *
 * Selecciona la filas indicadas el la lista GtkCList.
 *
 */
void c_util_list_select_rows (GtkCList *clist, GArray *rows);

#endif


