/* emited_payment_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "payment.h"
#include "invoice.h"
#include "account.h"
#include "emited_payment_control.h"
#include "account_model.h"
#include "invoice_control.h"
#include "insert_operation_control.h"
#include "insert_cash_operation_control.h"

static GtkWidget *emited_payments=NULL;
static gint previous_money_order_days=0;
static Payment ** emited_payment_to_manage;
static gboolean handle_events=FALSE;
static gulong participant_code;
static gboolean modify=FALSE;
static GladeXML *xml;
static gchar widget_name[]="gestion_pagos";
static gchar url[]="/src/interface/glade/invoice_view.glade";

static void on_cancel_management_cash_button_clicked(GtkButton *button, gpointer user_data);
static void on_accept_management_cash_button_clicked(GtkButton *button,  gpointer user_data);
static void on_pay_button_clicked(GtkButton *button,gpointer user_data);
static void on_exit_management_cash_button_clicked(GtkButton *button,gpointer user_data);
static void emited_payment_mode_combo_init(void);
static void emited_payment_type_combo_init(void);
static void on_payment_mode_combo_entry_changed(GtkEntry *entry,
					     gpointer user_data);
static void on_payment_environment_combo_entry_changed(GtkEntry *entry,
					      gpointer user_data);
static void on_money_order_days_entry_changed(GtkEntry *entry,
				 gpointer user_data);

static void clean_screen(void);
static void fill_account_numbers_combo(Payment * emited_payment,gulong client_code);
static void fill_emited_payment_on_screen(Payment * payment);
static void set_dates_on_emited_payment(void);
static gboolean set_emited_payment_from_screen(Payment ** emited_payment_to_fill);
static void on_management_cash_destroy(GtkWidget * widget,gpointer user_data);
static void restore_to_initial_purchase(gboolean no_sensitive);
static void set_button_visible(void);
static void generate_emited_operation(void);
static void pay(gpointer value);
static void get_code_from_payment(gpointer value);

GtkWidget *c_emited_payment_open_window(GtkWidget *app1,Payment ** new_emited_payment,gboolean modificar,gulong client_code) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (emited_payments==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_button_cancelar_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_management_cash_button_clicked));
	glade_xml_signal_connect (xml, "on_button_aceptar_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_accept_management_cash_button_clicked));
	glade_xml_signal_connect (xml, "on_button_pagar_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_pay_button_clicked));
	glade_xml_signal_connect (xml, "on_button_salir_gestion_pagos_clicked",
				  GTK_SIGNAL_FUNC (on_exit_management_cash_button_clicked));
	glade_xml_signal_connect (xml, "on_combo_entry_modo_pago_gestion_pagos_changed",
				  GTK_SIGNAL_FUNC (on_payment_mode_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_combo_entry_medio_pago_gestion_pagos_changed",
				  GTK_SIGNAL_FUNC (on_payment_environment_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_dias_giro_gestion_pagos_changed",
				  GTK_SIGNAL_FUNC (on_money_order_days_entry_changed));
	glade_xml_signal_connect (xml, "on_gestion_pagos_destroy",
				  GTK_SIGNAL_FUNC (on_management_cash_destroy));

	emited_payment_mode_combo_init();
	emited_payment_type_combo_init();
	emited_payments=dialog;
    } else {
	if (emited_payment_to_manage!=NULL)
	    /* DOESN'T FREE THE PAYMENT*/
	    emited_payment_to_manage = NULL;
    }

    clean_screen();
    //	restore_to_initial_purchase(TRUE);

    modify = modificar;
    emited_payment_to_manage=(Payment **) g_malloc0(sizeof(sizeof(Payment *)));
    emited_payment_to_manage[0] = new_emited_payment[0];
    participant_code = client_code;

    fill_account_numbers_combo(new_emited_payment[0],participant_code);

    if (emited_payment_to_manage!=NULL) {
	fill_emited_payment_on_screen(emited_payment_to_manage[0]);
    }

    g_free(directory);
    return emited_payments;
}


static void on_cancel_management_cash_button_clicked(GtkButton *button,gpointer user_data) {
    gtk_widget_destroy(emited_payments);
    emited_payments = NULL;
    previous_money_order_days=0;
}

static void on_management_cash_destroy(GtkWidget * widget,gpointer user_data) {
    gtk_widget_destroy(emited_payments);
    emited_payments = NULL;
    previous_money_order_days=0;
}


static void on_accept_management_cash_button_clicked(GtkButton *button,gpointer user_data) {
    if (modify!=FALSE) {
	gboolean done = FALSE;
	gulong code;
	done = set_emited_payment_from_screen(emited_payment_to_manage);

	if (done ==TRUE) {
	    if (d_payment_get_payment_code (emited_payment_to_manage[0])!=0) {
		code = m_payment_update (emited_payment_to_manage[0]);
		d_payment_set_payment_code (emited_payment_to_manage[0],code);
		if (code!=ULONG_MAX) {
                    c_invoice_repaint_payment();
		    c_util_open_info_window (_("The data has been modified correctly"));
		}
	    } 
	    gtk_widget_destroy(emited_payments);
	    emited_payments = NULL;
	    previous_money_order_days=0;

	}
    }
}

static void pay(gpointer value) {
    if (modify==FALSE) {
	GtkWidget * spin_payments_done = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
	GtkWidget * next_done_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
	GtkWidget * interval_spin = glade_xml_get_widget(xml,"spinbutton_intervalos_gestion_pagos");
	GtkWidget * button = glade_xml_get_widget(xml,"button_pagado_gestion_pagos");
	gint payments_done;
	gint interval_days;
	gint expiries;
	struct tm * time_tm = (struct tm *) g_malloc0(sizeof(struct tm));
	time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));


	interval_days = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(interval_spin));
	payments_done = d_payment_get_achieved_payments (emited_payment_to_manage[0]);
	expiries = d_payment_get_number_expiries (emited_payment_to_manage[0]);
	if (expiries>payments_done) {
	    if (expiries == 1) {
		if (payments_done == 0) {
                    gboolean error = 0;
		    payments_done++;

		    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_payments_done),1.0*payments_done);
		    gtk_spin_button_update(GTK_SPIN_BUTTON(spin_payments_done));
		    gtk_widget_set_sensitive(GTK_WIDGET(button),FALSE);
		    d_payment_set_interval_days (emited_payment_to_manage[0],interval_days);
		    c_invoice_set_state_paid((gpointer) &error);
		}
	    } else {
		if (payments_done==(expiries-1)) {
                    gboolean error = 0;
		    payments_done++;
		    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_payments_done),1.0*payments_done);
		    gtk_spin_button_update(GTK_SPIN_BUTTON(spin_payments_done));
		    gtk_widget_set_sensitive(GTK_WIDGET(button),FALSE);
		    c_invoice_set_state_paid((gpointer) &error);

		} else if (payments_done<expiries){
		    GDate * date;
		    gint days;
		    *time_aux = gnome_date_edit_get_date(GNOME_DATE_EDIT(next_done_dateedit));
		    payments_done++;
		    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_payments_done),1.0*payments_done);
		    gtk_spin_button_update(GTK_SPIN_BUTTON(spin_payments_done));

		    time_tm = localtime(time_aux);
		    date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
		    days = d_payment_get_interval_days (emited_payment_to_manage[0]);
		    g_date_add_days(date,days);

		    time_tm ->tm_mday = g_date_day(date);
		    time_tm ->tm_mon= g_date_month(date)-1;
		    time_tm ->tm_year = g_date_year(date)-1900;
		    *time_aux = mktime(time_tm);
		    gnome_date_edit_set_time(GNOME_DATE_EDIT(next_done_dateedit),*time_aux);
		    d_payment_set_init_date (emited_payment_to_manage[0],*time_aux);

		}
	    }

	    d_payment_set_achieved_payments (emited_payment_to_manage[0],payments_done);
	    m_payment_update (emited_payment_to_manage[0]);
	} else {
	    c_util_open_info_window (_("Paid invoice"));
	}
	gtk_widget_destroy(emited_payments);
	emited_payments=NULL;
	previous_money_order_days=0;

    }
}

static void on_pay_button_clicked(GtkButton *button,gpointer user_data) {
    if (modify==FALSE) {
	generate_emited_operation();
    } else {
        c_util_open_error_window (_("This command is only allowed on payment"));
    }
}


static void emited_payment_mode_combo_init() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "combo_modo_pago_gestion_pagos");

    c_util_set_code_and_name_on_combo (D_PAYMENT_AT_MOMENT,D_PAYMENT_MESSAGE_AT_MOMENT,GTK_COMBO(combo));
    handle_events=FALSE;
    c_util_set_code_and_name_on_combo (D_PAYMENT_HIRE_PURCHASE,D_PAYMENT_MESSAGE_HIRE_PURCHASE,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH,D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_MONEY_ORDER,D_PAYMENT_MESSAGE_MONEY_ORDER,GTK_COMBO(combo));
    handle_events=TRUE;
}


static void emited_payment_type_combo_init() {
    GtkWidget * combo;

    combo = glade_xml_get_widget (xml, "combo_medio_pago_gestion_pagos");

    c_util_set_code_and_name_on_combo (D_PAYMENT_IN_CASH,D_PAYMENT_MESSAGE_IN_CASH,GTK_COMBO(combo));
    handle_events=FALSE;
    c_util_set_code_and_name_on_combo (D_PAYMENT_TRANSFER_CODE,D_PAYMENT_MESSAGE_TRANSFER,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_CHEQUE,D_PAYMENT_MESSAGE_CHEQUE,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_CREDIT_CARD,D_PAYMENT_MESSAGE_CREDIT_CARD,GTK_COMBO(combo));
    c_util_set_code_and_name_on_combo (D_PAYMENT_DIRECT_DEBIT_CODE,D_PAYMENT_MESSAGE_DIRECT_DEBIT,GTK_COMBO(combo));
    handle_events=TRUE;
}


static void on_payment_environment_combo_entry_changed(GtkEntry *entry,
					 gpointer user_data) {

    if (handle_events!=FALSE) {

	GtkWidget * combo = glade_xml_get_widget(xml,"combo_medio_pago_gestion_pagos");
	gint code_combo = c_util_get_code_from_combo (combo);
	GtkWidget * valid_card_label = glade_xml_get_widget(xml,"label_valida_hasta_gestion_pagos");
	GtkWidget * valid_date_edit = glade_xml_get_widget(xml,"dateedit_valida_hasta_gestion_pagos");
	GtkWidget * card_number_label = glade_xml_get_widget(xml,"label_numero_gestion_pagos");
	GtkWidget * spin;
	GtkWidget * combo_parent_company = glade_xml_get_widget(xml,"parent_company_accounts_combo");
	GtkAdjustment * adjustment;
	GtkWidget * account_number_label = NULL;


	if (GTK_WIDGET_VISIBLE(valid_card_label)) gtk_widget_hide(valid_card_label);
	if (GTK_WIDGET_VISIBLE(valid_date_edit)) gtk_widget_hide(valid_date_edit);
	if (GTK_WIDGET_VISIBLE(card_number_label)) gtk_widget_hide(card_number_label);

	if (code_combo == D_PAYMENT_TRANSFER_CODE) {

	    spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);
	    account_number_label = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(account_number_label)) gtk_widget_hide(account_number_label);

	} else if (code_combo == D_PAYMENT_IN_CASH) {

	    spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);
	    account_number_label = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(account_number_label)) gtk_widget_hide(account_number_label);

	} else if (code_combo == D_PAYMENT_CREDIT_CARD) {
	    if (!GTK_WIDGET_VISIBLE(card_number_label)) gtk_widget_show(card_number_label);
	    if (GTK_WIDGET_VISIBLE(valid_card_label)==FALSE) gtk_widget_show(valid_card_label);
	    if (GTK_WIDGET_VISIBLE(valid_date_edit)==FALSE) gtk_widget_show(valid_date_edit);

	    account_number_label = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(account_number_label)) gtk_widget_hide(account_number_label);
	    if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);

	    gtk_label_set_text(GTK_LABEL(card_number_label),_("Card number"));

	    spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=9999;
	    gtk_adjustment_changed(adjustment);

	    spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=9999;
	    gtk_adjustment_changed(adjustment);

	    spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=9999;
	    gtk_adjustment_changed(adjustment);

	    spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=9999;
	    gtk_adjustment_changed(adjustment);


	} else if (code_combo == D_PAYMENT_CHEQUE) {
	    GtkWidget * cheque_number_label = glade_xml_get_widget(xml,"label_numero_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(cheque_number_label)) gtk_widget_show(cheque_number_label);
	    gtk_label_set_text(GTK_LABEL(card_number_label),_("Check number"));

	    account_number_label = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(account_number_label)) gtk_widget_hide(account_number_label);
	    if (GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_hide(combo_parent_company);

	    spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=9;
	    gtk_adjustment_changed(adjustment);

	    spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=999;
	    gtk_adjustment_changed(adjustment);

	    spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=999;
	    gtk_adjustment_changed(adjustment);

	    spin = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(spin)) gtk_widget_show(spin);
	    adjustment = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(spin));
	    adjustment->upper=9;
	    gtk_adjustment_changed(adjustment);


	} else if (code_combo == D_PAYMENT_DIRECT_DEBIT_CODE) {
	    GtkWidget * account_number_label = glade_xml_get_widget(xml,"label_numero_gestion_pagos");

	    spin = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
	    if (!GTK_WIDGET_VISIBLE(account_number_label)) gtk_widget_show(account_number_label);
	    gtk_label_set_text(GTK_LABEL(account_number_label),_("Current account"));

	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    spin = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);

	    spin = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(spin)) gtk_widget_hide(spin);
	    if (!GTK_WIDGET_VISIBLE(combo_parent_company)) gtk_widget_show(combo_parent_company);

	    account_number_label = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
	    if (GTK_WIDGET_VISIBLE(account_number_label)) gtk_widget_hide(account_number_label);

	} else {
	    g_print(_("Not yet implemented"));
	}
    }
}


static void on_payment_mode_combo_entry_changed(GtkEntry *entry,
							   gpointer user_data) {


    if (handle_events!=FALSE) {
	GtkWidget * combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
	GtkWidget * money_order_days_label = glade_xml_get_widget(xml,"label_dias_giro_gestion_pagos");
	GtkWidget * spin_button = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
	GtkWidget * payment_code_entry = glade_xml_get_widget(xml,"entry_codigo_pago_gestion_pagos");
	gint code_combo = c_util_get_code_from_combo (combo);
	gboolean no_sensitive = TRUE;

	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_button),(gfloat) 1);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin_button));

	if (code_combo == D_PAYMENT_AT_MOMENT) {
	    gtk_widget_set_sensitive(GTK_WIDGET(spin_button),FALSE);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spin_button));
	    gtk_label_set_text(GTK_LABEL(money_order_days_label),"");

	} else if (code_combo == D_PAYMENT_MONEY_ORDER) {

	    gtk_widget_set_sensitive(GTK_WIDGET(spin_button),TRUE);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spin_button));
	    gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Draft days"));

	} else if (code_combo == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {

	    gtk_widget_set_sensitive(GTK_WIDGET(spin_button),FALSE);
	    gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Deferment days"));

	} else if (code_combo == D_PAYMENT_HIRE_PURCHASE) {

	    no_sensitive = FALSE;
	    gtk_widget_set_sensitive(GTK_WIDGET(spin_button),TRUE);
	    gtk_spin_button_update(GTK_SPIN_BUTTON(spin_button));
	    gtk_label_set_text(GTK_LABEL(money_order_days_label),_("Number of instalments"));

	}

	restore_to_initial_purchase(no_sensitive);
	if (strcmp(gtk_entry_get_text(GTK_ENTRY(payment_code_entry)),"")) {
	    set_button_visible();
	}
	set_dates_on_emited_payment();
    }
}

static void set_button_visible() {
    GtkWidget * entry_mode = glade_xml_get_widget(xml,"combo_entry_modo_pago_gestion_pagos");
    GtkWidget * several_instalments_button = glade_xml_get_widget(xml,"button_siguiente_pago_gestion_pagos");
    GtkWidget * accept_button = glade_xml_get_widget(xml,"button_aceptar_gestion_pagos");
    GtkWidget * one_instalment_button = glade_xml_get_widget(xml,"button_pagado_gestion_pagos");
    gchar * value;

    if (modify==FALSE) {
	if (GTK_WIDGET_VISIBLE(accept_button)) gtk_widget_hide(accept_button);
	value = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_mode)));
	if (!strcmp(value,D_PAYMENT_MESSAGE_HIRE_PURCHASE)) {
	    if (!GTK_WIDGET_VISIBLE(several_instalments_button)) gtk_widget_show(several_instalments_button);
	    if (GTK_WIDGET_VISIBLE(one_instalment_button)) gtk_widget_hide(one_instalment_button);
	} else {
	    if (GTK_WIDGET_VISIBLE(several_instalments_button)) gtk_widget_hide(several_instalments_button);
	    if (!GTK_WIDGET_VISIBLE(one_instalment_button)) gtk_widget_show(one_instalment_button);
	}
    } else {
	if (GTK_WIDGET_VISIBLE(one_instalment_button)) gtk_widget_hide(one_instalment_button);
	if (GTK_WIDGET_VISIBLE(several_instalments_button)) gtk_widget_hide(several_instalments_button);
    }

}


static void restore_to_initial_purchase(gboolean no_sensitive) {
    GtkWidget * spin1 = glade_xml_get_widget(xml,"spinbutton_entrada_gestion_pagos");
    GtkWidget * spin2 = glade_xml_get_widget(xml,"spinbutton_intervalos_gestion_pagos");
    GtkWidget * date_edit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    GtkWidget * spin3 = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
    GtkWidget * spin4 = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");

    handle_events=FALSE;

    gtk_widget_set_sensitive(spin3,FALSE);
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin3),0);
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin3));
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin4),1);
    gtk_spin_button_update(GTK_SPIN_BUTTON(spin4));

    if (no_sensitive) {
	gtk_widget_set_sensitive(spin2,FALSE);
	gtk_widget_set_sensitive(spin1,FALSE);
	gtk_widget_set_sensitive(date_edit,FALSE);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin1),(gfloat) 100);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin1));
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin2),(gfloat) 1);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin2));
    } else {
	gtk_widget_set_sensitive(spin2,TRUE);
	gtk_widget_set_sensitive(spin1,TRUE);
	gtk_widget_set_sensitive(date_edit,TRUE);
    }

    handle_events=TRUE;
}


static void set_dates_on_emited_payment() {
    struct tm * time_tm = (struct tm *) g_malloc0(sizeof(struct tm));
    time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
    GtkWidget * next_done_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * spin_button = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    gint code_combo = c_util_get_code_from_combo (combo);

    *time_aux = time(NULL);

    if (code_combo == D_PAYMENT_MONEY_ORDER) {
	    GDate * date;
	    gint days;
	    if (modify==FALSE) {
		time_tm = localtime(time_aux);
		date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
		days = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin_button));
		g_date_add_days(date,days);
		time_tm ->tm_mday = g_date_day(date);
		time_tm ->tm_mon= g_date_month(date)-1;
		time_tm ->tm_year = g_date_year(date)-1900;
		*time_aux = mktime(time_tm);
	    } else {
                *time_aux = d_payment_get_init_date (emited_payment_to_manage[0]);
		time_tm = localtime(time_aux);
		date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
		days = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin_button));
		g_date_add_days(date,(days-previous_money_order_days));
		time_tm ->tm_mday = g_date_day(date);
		time_tm ->tm_mon= g_date_month(date)-1;
		time_tm ->tm_year = g_date_year(date)-1900;
		*time_aux = mktime(time_tm);
	    }
    } else if (code_combo == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
	gint days;
	time_tm = localtime(time_aux);
	days = c_util_get_days_to_last_day_of_month (time_aux);
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin_button),1.0*days);
	gtk_spin_button_update(GTK_SPIN_BUTTON(spin_button));
	time_tm->tm_mday = time_tm->tm_mday + days;
	*time_aux = mktime(time_tm);
    } else if (code_combo == D_PAYMENT_HIRE_PURCHASE) {
        GDate * date;
	gint days;
        *time_aux = d_payment_get_init_date (emited_payment_to_manage[0]);
	time_tm = localtime(time_aux);
	date = g_date_new_dmy(time_tm->tm_mday,time_tm->tm_mon+1,time_tm->tm_year+1900);
	days = d_payment_get_interval_days (emited_payment_to_manage[0]);
	g_date_add_days(date,days);
	time_tm ->tm_mday = g_date_day(date);
	time_tm ->tm_mon= g_date_month(date)-1;
	time_tm ->tm_year = g_date_year(date)-1900;
	*time_aux = mktime(time_tm);
    }

    gtk_calendar_display_options(GTK_CALENDAR(GNOME_DATE_EDIT(next_done_dateedit)->calendar),GTK_CALENDAR_SHOW_DAY_NAMES);
    gnome_date_edit_set_time(GNOME_DATE_EDIT(next_done_dateedit),*time_aux);
}

static void on_money_order_days_entry_changed(GtkEntry *entry,
				 gpointer user_data) {

    if (strcmp(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"combo_entry_modo_pago_gestion_pagos"))),D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH))
	set_dates_on_emited_payment();
}

static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);


}

static void fill_emited_payment_on_screen(Payment * payment) {
    GtkWidget * payment_mode_combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * payment_mean_combo = glade_xml_get_widget(xml,"combo_medio_pago_gestion_pagos");
    GtkWidget * payment_mean_entry = glade_xml_get_widget(xml,"combo_entry_medio_pago_gestion_pagos");
    GtkWidget * payment_mode_entry = glade_xml_get_widget(xml,"combo_entry_modo_pago_gestion_pagos");
    GtkWidget * payment_code_entry = glade_xml_get_widget(xml,"entry_codigo_pago_gestion_pagos");
    GtkWidget * spin_realizados_pago = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
    GtkWidget * next_date_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    GtkWidget * money_order_days_spinbutton = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    GtkWidget * first_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
    GtkWidget * second_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
    GtkWidget * third_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
    GtkWidget * fourth_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
    GtkWidget * account_number_label = glade_xml_get_widget(xml,"entry_numero_cuenta_gestion_pagos");
    GtkWidget * expiry_date_dateedit= glade_xml_get_widget(xml,"dateedit_valida_hasta_gestion_pagos");
    GtkWidget * entry_spinbutton=glade_xml_get_widget(xml,"spinbutton_entrada_gestion_pagos");
    GtkWidget * interval_spinbutton=glade_xml_get_widget(xml,"spinbutton_intervalos_gestion_pagos");
    GtkWidget * expiries_spinbutton=glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    GtkWidget * pay_all_button=glade_xml_get_widget(xml,"button_siguiente_pago_gestion_pagos");
    GtkWidget * pay_instalment_button=glade_xml_get_widget(xml,"button_pagado_gestion_pagos");
    GtkWidget * accept_button=glade_xml_get_widget(xml,"button_aceptar_gestion_pagos");
    GtkWidget * cancel_button=glade_xml_get_widget(xml,"button_cancelar_gestion_pagos");
    GtkWidget * exit_button=glade_xml_get_widget(xml,"button_salir_gestion_pagos");
    gchar * mean_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(payment_mean_entry)));
    gchar * modo_temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(payment_mode_entry)));
    gint insensible = 0;

    if (payment!=NULL) {
	gchar * money_order_days=NULL;
	time_t * time_aux = (time_t *) g_malloc0(sizeof(time_t));
	gchar * codigo_pago = g_strdup_printf("%lu",d_payment_get_payment_code (payment));
	gchar * payments_done =  g_strdup_printf("%lu",d_payment_get_achieved_payments (payment));
	*time_aux = d_payment_get_init_date (payment);
	money_order_days =  g_strdup_printf("%d",d_payment_get_interval_days (payment));

	if (d_payment_get_payment_code (payment)!=ULONG_MAX) {
	    set_button_visible();
	    if (strcmp(codigo_pago,"0")) {
		gtk_entry_set_text(GTK_ENTRY(payment_code_entry),codigo_pago);
	    }
	}


	g_free(codigo_pago);
	if (d_payment_get_interval_days (payment)==0) {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    gchar * intervalo = g_strdup_printf("%d",d_payment_get_interval_days (payment));
	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_AT_MOMENT)) gtk_entry_set_text(GTK_ENTRY(payment_mode_entry),D_PAYMENT_MESSAGE_AT_MOMENT);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(expiries_spinbutton),expiries);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),intervalo);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(intervalo);


	} else if (d_payment_get_number_expiries (payment)>1) {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    gchar * intervalo = g_strdup_printf("%d",d_payment_get_interval_days (payment));
	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_HIRE_PURCHASE)) gtk_entry_set_text(GTK_ENTRY(payment_mode_entry),D_PAYMENT_MESSAGE_HIRE_PURCHASE);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(expiries_spinbutton),expiries);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),intervalo);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(intervalo);

	} else if ((c_util_get_days_to_last_day_of_month (time_aux))==d_payment_get_interval_days (payment)) {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    gchar * intervalo = g_strdup_printf("%d",d_payment_get_interval_days (payment));
            gchar * temp_vista=g_strconcat(D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH,"\0",NULL);

            previous_money_order_days = d_payment_get_interval_days (payment);
	    if (strcmp(modo_temp,temp_vista))
	                    	    gtk_entry_set_text(GTK_ENTRY(payment_mode_entry),temp_vista);

            g_free(temp_vista);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(expiries_spinbutton),expiries);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),intervalo);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(intervalo);



	} else {
	    gchar * expiries = g_strdup_printf("%d",d_payment_get_number_expiries (payment));
	    gchar * percentaje = g_strdup_printf("%.2f",d_payment_get_entry_percentage (payment));
	    gchar * intervalo = g_strdup_printf("%d",d_payment_get_interval_days (payment));
	    if (strcmp(modo_temp,D_PAYMENT_MESSAGE_MONEY_ORDER)) gtk_entry_set_text(GTK_ENTRY(payment_mode_entry),D_PAYMENT_MESSAGE_MONEY_ORDER);
            previous_money_order_days = d_payment_get_interval_days (payment);
	    gtk_entry_set_text(GTK_ENTRY(entry_spinbutton),percentaje);
	    gtk_entry_set_text(GTK_ENTRY(expiries_spinbutton),intervalo);
	    gtk_entry_set_text(GTK_ENTRY(interval_spinbutton),expiries);
	    g_free(expiries);
	    g_free(percentaje);
	    g_free(intervalo);
	}

	gtk_spin_button_update(GTK_SPIN_BUTTON(entry_spinbutton));
	gtk_spin_button_update(GTK_SPIN_BUTTON(expiries_spinbutton));
	gtk_spin_button_update(GTK_SPIN_BUTTON(interval_spinbutton));

	if (d_payment_get_is_transfer (payment)) {
	    if (strcmp(mean_temp,D_PAYMENT_MESSAGE_TRANSFER)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_TRANSFER);

	} else if (d_payment_get_is_cheque (payment)) {
	    gchar * number = d_payment_c_util_get_cheque_number (payment);
	    gchar ** numbers = c_util_get_card_numbers (number);
	    if (strcmp(mean_temp,D_PAYMENT_MESSAGE_CHEQUE)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_CHEQUE);
	    gtk_entry_set_text(GTK_ENTRY(first_number_spinbutton),numbers[0]);
	    gtk_entry_set_text(GTK_ENTRY(second_number_spinbutton),numbers[1]);
	    gtk_entry_set_text(GTK_ENTRY(third_number_spinbutton),numbers[2]);
	    gtk_entry_set_text(GTK_ENTRY(fourth_number_spinbutton),numbers[3]);
	    g_free(number);
	    g_strfreev(numbers);


	} else if (d_payment_get_is_direct_debit (payment)) {
	    if (strcmp(mean_temp,D_PAYMENT_MESSAGE_DIRECT_DEBIT)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_DIRECT_DEBIT);
	} else if (d_payment_get_is_credit_card (payment)) {
	    gchar * number = d_payment_get_credit_card_number (payment);
	    gchar ** numbers = c_util_get_card_numbers (number);
	    time_t * expiry_date = (time_t *) g_malloc0(sizeof(time_t));
	    *expiry_date = d_payment_get_expiry_date (payment);
	    if (strcmp(mean_temp,D_PAYMENT_MESSAGE_CREDIT_CARD)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_CREDIT_CARD);
	    gtk_entry_set_text(GTK_ENTRY(first_number_spinbutton),numbers[0]);
	    gtk_entry_set_text(GTK_ENTRY(second_number_spinbutton),numbers[1]);
	    gtk_entry_set_text(GTK_ENTRY(third_number_spinbutton),numbers[2]);
	    gtk_entry_set_text(GTK_ENTRY(fourth_number_spinbutton),numbers[3]);
	    gnome_date_edit_set_time(GNOME_DATE_EDIT(expiry_date_dateedit),*expiry_date);
	    g_free(number);
	    g_strfreev(numbers);
	    g_free(expiry_date);
	} else {
	    if (strcmp(mean_temp,D_PAYMENT_MESSAGE_IN_CASH)) gtk_entry_set_text(GTK_ENTRY(payment_mean_entry),D_PAYMENT_MESSAGE_IN_CASH);

	}
	if (GTK_WIDGET_VISIBLE(exit_button)) gtk_widget_hide(exit_button);
	if (d_payment_get_payment_code (payment)!=ULONG_MAX) {
	    if ((d_payment_get_payment_code (payment)!=0) && (d_payment_get_payment_code (payment)!=ULONG_MAX) &&
		(d_payment_get_is_hire_purchase (payment))) {
		insensible=1;
	    }
	    if ((d_payment_get_achieved_payments (payment))==(d_payment_get_number_expiries (payment))) {
		insensible=1;
		if (GTK_WIDGET_SENSITIVE(pay_all_button)) gtk_widget_set_sensitive(pay_all_button,FALSE);
		if (GTK_WIDGET_SENSITIVE(pay_instalment_button)) gtk_widget_set_sensitive(pay_instalment_button,FALSE);
		if (GTK_WIDGET_SENSITIVE(accept_button)) gtk_widget_set_sensitive(accept_button,FALSE);
		if (GTK_WIDGET_SENSITIVE(cancel_button)) gtk_widget_set_sensitive(cancel_button,FALSE);
		if (!GTK_WIDGET_VISIBLE(exit_button)) gtk_widget_show(exit_button);
	    } else {
		if (!GTK_WIDGET_SENSITIVE(pay_all_button)) gtk_widget_set_sensitive(pay_all_button,TRUE);
		if (!GTK_WIDGET_SENSITIVE(pay_instalment_button)) gtk_widget_set_sensitive(pay_instalment_button,TRUE);
		if (!GTK_WIDGET_SENSITIVE(accept_button)) gtk_widget_set_sensitive(accept_button,TRUE);
		if (!GTK_WIDGET_SENSITIVE(cancel_button)) gtk_widget_set_sensitive(cancel_button,TRUE);
	    }
	    if (modify==FALSE) {

		if (insensible==1) {
		    if (GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,FALSE);
		    if (GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,FALSE);
		    if (GTK_WIDGET_SENSITIVE(next_date_dateedit)) gtk_widget_set_sensitive(next_date_dateedit,FALSE);
		    if (GTK_WIDGET_SENSITIVE(money_order_days_spinbutton)) gtk_widget_set_sensitive(money_order_days_spinbutton,FALSE);
		    if (GTK_WIDGET_SENSITIVE(first_number_spinbutton)) gtk_widget_set_sensitive(first_number_spinbutton,FALSE);
		    if (GTK_WIDGET_SENSITIVE(second_number_spinbutton)) gtk_widget_set_sensitive(second_number_spinbutton,FALSE);
		    if (GTK_WIDGET_SENSITIVE(third_number_spinbutton)) gtk_widget_set_sensitive(third_number_spinbutton,FALSE);
		    if (GTK_WIDGET_SENSITIVE(fourth_number_spinbutton)) gtk_widget_set_sensitive(fourth_number_spinbutton,FALSE);
		    if (GTK_WIDGET_SENSITIVE(account_number_label)) gtk_widget_set_sensitive(account_number_label,FALSE);
		    if (GTK_WIDGET_SENSITIVE(expiry_date_dateedit)) gtk_widget_set_sensitive(expiry_date_dateedit,FALSE);
		    if (GTK_WIDGET_SENSITIVE(expiries_spinbutton)) gtk_widget_set_sensitive(expiries_spinbutton,FALSE);
		    if (GTK_WIDGET_SENSITIVE(entry_spinbutton)) gtk_widget_set_sensitive(entry_spinbutton,FALSE);

		} else {
		    if (!GTK_WIDGET_SENSITIVE(payment_mode_combo)) gtk_widget_set_sensitive(payment_mode_combo,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(payment_mean_combo)) gtk_widget_set_sensitive(payment_mean_combo,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(next_date_dateedit)) gtk_widget_set_sensitive(next_date_dateedit,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(money_order_days_spinbutton)) gtk_widget_set_sensitive(money_order_days_spinbutton,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(first_number_spinbutton)) gtk_widget_set_sensitive(first_number_spinbutton,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(second_number_spinbutton)) gtk_widget_set_sensitive(second_number_spinbutton,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(third_number_spinbutton)) gtk_widget_set_sensitive(third_number_spinbutton,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(fourth_number_spinbutton)) gtk_widget_set_sensitive(fourth_number_spinbutton,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(account_number_label)) gtk_widget_set_sensitive(account_number_label,TRUE);
		    if (!GTK_WIDGET_SENSITIVE(expiry_date_dateedit)) gtk_widget_set_sensitive(expiry_date_dateedit,TRUE);
		}
	    }
	} else {
	    if (GTK_WIDGET_VISIBLE(pay_instalment_button)) gtk_widget_hide(pay_instalment_button);
	    if (GTK_WIDGET_VISIBLE(pay_all_button)) gtk_widget_hide(pay_all_button);

	}
	gtk_entry_set_text(GTK_ENTRY(spin_realizados_pago),payments_done);

	gnome_date_edit_set_time(GNOME_DATE_EDIT(next_date_dateedit),*time_aux);

	g_free(time_aux);
	g_free(money_order_days);
    }

    if (d_payment_get_payment_code (payment)==ULONG_MAX) {
        set_dates_on_emited_payment();
    } else if (d_payment_get_is_hire_purchase (payment)!=FALSE) {
	if (modify==FALSE) {
	    set_dates_on_emited_payment();
	}
    }
    g_free(mean_temp);
    g_free(modo_temp);

}

static gboolean set_emited_payment_from_screen(Payment ** emited_payment_to_fill) {

    GtkWidget * payment_mean_combo = glade_xml_get_widget(xml,"combo_medio_pago_gestion_pagos");
    GtkWidget * payment_mode_combo = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * entry_spinbutton = glade_xml_get_widget(xml,"spinbutton_entrada_gestion_pagos");
    GtkWidget * interval_spinbutton = glade_xml_get_widget(xml,"spinbutton_intervalos_gestion_pagos");
    GtkWidget * payment_code_entry = glade_xml_get_widget(xml,"entry_codigo_pago_gestion_pagos");
    GtkWidget * money_order_days_spinbutton = glade_xml_get_widget(xml,"dias_giro_gestion_pagos");
    GtkWidget * done_spinbutton = glade_xml_get_widget(xml,"spinbutton_realizados_gestion_pagos");
    GtkWidget * first_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_primero_gestion_pagos");
    GtkWidget * second_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_segundo_gestion_pagos");
    GtkWidget * third_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_tercero_gestion_pagos");
    GtkWidget * fourth_number_spinbutton = glade_xml_get_widget(xml,"spinbutton_cuarto_gestion_pagos");
    GtkWidget * expiry_date_dateedit = glade_xml_get_widget(xml,"dateedit_valida_hasta_gestion_pagos");
    GtkWidget * next_date_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_siguiente_gestion_pagos");
    gboolean is_instalment_payment=FALSE;
    gshort expiries_number=1;
    gdouble entry_percentaje=100;
    time_t next_date=time(NULL);
    time_t done_date=time(NULL);
    gshort interval_days=0;
    gboolean esTransferencia=FALSE;
    gboolean is_cheque=FALSE;
    gchar * cheque_number=NULL;
    gboolean esTarjeta=FALSE;
    gboolean is_direct_debit=FALSE;
    gint bank_code=-1;
    gint office_code=-1;
    gchar *  account_number=NULL;
    gchar * card_number=NULL;
    time_t expiry_date=time(NULL);
    gboolean is_cash_payment=FALSE;
    gint error = 0;
    gulong payment_code=ULONG_MAX;
    gchar ** stop =  (gchar **) g_malloc0(sizeof(gchar *));
    gchar * payment_code_char = g_strdup(gtk_entry_get_text(GTK_ENTRY(payment_code_entry)));
    gint payment_mean_combo_code = c_util_get_code_from_combo (payment_mean_combo);
    gint payment_mode_combo_code = c_util_get_code_from_combo (payment_mode_combo);

    if (strcmp(payment_code_char,"")) {

	payment_code = strtoul(payment_code_char,stop,10);
	g_free(stop);
	g_free(payment_code_char);

	if (emited_payment_to_fill[0]==NULL) emited_payment_to_fill[0]=d_payment_create_empty ();



	if (payment_mode_combo_code == D_PAYMENT_AT_MOMENT) {
	    is_instalment_payment=FALSE;
	    expiries_number=1;
	    entry_percentaje=100;
	    next_date=time(NULL);
	    interval_days=0;
	    is_cash_payment = TRUE;
	    d_payment_set_achieved_payments (emited_payment_to_fill[0],gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));
	    done_date = next_date;
	} else if (payment_mode_combo_code == D_PAYMENT_MONEY_ORDER) {
	    is_instalment_payment=FALSE;
	    expiries_number=1;
	    entry_percentaje=100;
	    gtk_spin_button_update(GTK_SPIN_BUTTON(money_order_days_spinbutton));
	    interval_days=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(money_order_days_spinbutton));
	    next_date= gnome_date_edit_get_date(GNOME_DATE_EDIT(next_date_dateedit));
	    is_cash_payment = FALSE;
	    d_payment_set_achieved_payments (emited_payment_to_fill[0],gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));
	    done_date = next_date;

	} else if (payment_mode_combo_code == D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH) {
	    time_t * date = (time_t *) g_malloc0(sizeof(time_t));
	    is_instalment_payment=FALSE;
	    expiries_number=1;
	    entry_percentaje=100;
	    next_date=gnome_date_edit_get_date(GNOME_DATE_EDIT(next_date_dateedit));
	    *date = time(NULL);
	    interval_days= c_util_get_days_to_last_day_of_month (date);
	    g_free(date);
	    is_cash_payment = FALSE;
	    d_payment_set_achieved_payments (emited_payment_to_fill[0],gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));

	    done_date = next_date;
	} else if (payment_mode_combo_code == D_PAYMENT_HIRE_PURCHASE) {
	    is_instalment_payment=TRUE;
	    gtk_spin_button_update(GTK_SPIN_BUTTON(money_order_days_spinbutton));
	    expiries_number=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(money_order_days_spinbutton));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(entry_spinbutton));
	    entry_percentaje=gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(entry_spinbutton));;
	    next_date=gnome_date_edit_get_date(GNOME_DATE_EDIT(next_date_dateedit));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(interval_spinbutton));
	    interval_days=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(interval_spinbutton));
	    is_cash_payment = FALSE;
	    gtk_spin_button_update(GTK_SPIN_BUTTON(done_spinbutton));
	    d_payment_set_achieved_payments (emited_payment_to_fill[0],gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(done_spinbutton)));

	    done_date = d_payment_get_achieved_date (emited_payment_to_fill[0]);
	} else {
	    g_print(_("Not yet implemented"));
	}

	if (payment_mean_combo_code == D_PAYMENT_TRANSFER_CODE) {
	    esTransferencia=TRUE;
	} else if (payment_mean_combo_code == D_PAYMENT_IN_CASH) {
	} else if (payment_mean_combo_code == D_PAYMENT_CREDIT_CARD) {
	    gint value1;
	    gint value2;
	    gint value3;
	    gint value4;

	    gtk_spin_button_update(GTK_SPIN_BUTTON(first_number_spinbutton));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(second_number_spinbutton));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(third_number_spinbutton));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(fourth_number_spinbutton));

	    value1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(first_number_spinbutton));
	    value2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(second_number_spinbutton));
	    value3 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(third_number_spinbutton));
	    value4 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fourth_number_spinbutton));

	    expiry_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(expiry_date_dateedit));

		card_number = c_util_get_card_number (value1,value2,value3,value4);

		esTarjeta=TRUE;
	} else if (payment_mean_combo_code == D_PAYMENT_CHEQUE) {
	    gint value1 ;
	    gint value2 ;
	    gint value3 ;
	    gint value4 ;

	    gtk_spin_button_update(GTK_SPIN_BUTTON(first_number_spinbutton));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(second_number_spinbutton));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(third_number_spinbutton));
	    gtk_spin_button_update(GTK_SPIN_BUTTON(fourth_number_spinbutton));

	    value1 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(first_number_spinbutton));
	    value2 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(second_number_spinbutton));
	    value3 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(third_number_spinbutton));
	    value4 = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(fourth_number_spinbutton));
	    is_cheque=TRUE;
	    cheque_number = c_util_get_cheque_number (value1,value2,value3,value4);

	} else if (payment_mean_combo_code == D_PAYMENT_DIRECT_DEBIT_CODE) {
	    gchar * complete_account_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"parent_company_accounts_entry"))));
	    gchar ** tokens = (gchar **) g_malloc0(sizeof(gchar *));

	    if (strcmp(complete_account_number,"")) {
		tokens = g_strsplit(complete_account_number," ",4);


		bank_code=atoi(tokens[0]);
		office_code=atoi(tokens[1]);
		account_number=g_strdup(tokens[3]);
		g_strfreev(tokens);
		is_direct_debit = TRUE;
		} else {
		c_util_open_error_window (_("There is not any current account"));
		error = 1;
		}
	    }
	if (error==0) {
	    d_payment_set_payment_code (emited_payment_to_fill[0],payment_code);
	    d_payment_set_is_hire_purchase (emited_payment_to_fill[0],is_instalment_payment);
	    d_payment_set_number_expiries (emited_payment_to_fill[0],expiries_number);
	    d_payment_set_entry_percentage (emited_payment_to_fill[0],entry_percentaje);
	    d_payment_set_init_date (emited_payment_to_fill[0],next_date);
	    d_payment_set_achieved_date (emited_payment_to_fill[0],done_date);
	    d_payment_set_interval_days (emited_payment_to_fill[0],interval_days);
	    d_payment_set_is_paid_cash (emited_payment_to_fill[0],is_cash_payment);
	    d_payment_set_is_hire_purchase (emited_payment_to_fill[0],is_instalment_payment);
	    d_payment_set_is_transfer (emited_payment_to_fill[0],esTransferencia);
	    d_payment_set_is_cheque (emited_payment_to_fill[0],is_cheque);
	    d_payment_set_cheque_number (emited_payment_to_fill[0],cheque_number);
	    d_payment_set_is_credit_card (emited_payment_to_fill[0],esTarjeta);
	    d_payment_set_is_direct_debit (emited_payment_to_fill[0],is_direct_debit);
	    d_payment_set_credit_card_number (emited_payment_to_fill[0],card_number);
	    d_payment_set_expiry_date (emited_payment_to_fill[0],expiry_date);
	    d_payment_set_is_paid_cash (emited_payment_to_fill[0],is_cash_payment);
	    d_payment_set_bank_code (emited_payment_to_fill[0],bank_code);
	    d_payment_set_office_code (emited_payment_to_fill[0],office_code);
	    d_payment_set_account_number (emited_payment_to_fill[0],account_number);
	    if (cheque_number!=NULL)  g_free(cheque_number);
	    if (account_number!=NULL)  g_free(account_number);
	    if (card_number!=NULL)  g_free(card_number);
	    return TRUE;
	} else {
	    c_util_open_error_window (_("Select another payment option or cancel the command"));
	    return FALSE;
	}
    } else {
	c_util_open_error_window (_("Cancel the command"));
	return FALSE;

    }


return TRUE;
}



static void generate_emited_operation(void) {
    GtkWidget * wid = glade_xml_get_widget(xml,"combo_modo_pago_gestion_pagos");
    GtkWidget * app1 = lookup_widget(wid, "app1");
    GtkWidget * open_insert;
    gdouble total=G_MINDOUBLE;
    PaymentId * emited_paymentId = (PaymentId *) g_malloc0(sizeof(PaymentId));

    emited_paymentId ->payment_code = g_strdup_printf("%lu",d_payment_get_payment_code (emited_payment_to_manage[0]));
    emited_paymentId ->enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (emited_payment_to_manage[0]))));
    emited_paymentId ->shop_code = g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_payment_get_shop (emited_payment_to_manage[0]))));

    if (d_payment_get_is_paid_in_cash(emited_payment_to_manage[0])!=FALSE) {
	total = (m_payment_calculate_next_total (emited_paymentId,FALSE));

	open_insert = c_cash_operation_open_window(app1,pay,FALSE);
	load_payment_cash_operation(emited_payment_to_manage[0],total);

	if (open_insert!=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
	    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
					 GTK_WINDOW(app1));
	    gtk_widget_show(GTK_WIDGET(open_insert));
	}

    } else {
	total = (m_payment_calculate_next_total (emited_paymentId,FALSE));

	open_insert = c_insert_operation_open_window(app1,get_code_from_payment,pay,total);
	if (open_insert!=NULL) {
	    gtk_window_set_modal(GTK_WINDOW(open_insert),1);
	    gtk_window_set_position(GTK_WINDOW(open_insert),GTK_WIN_POS_CENTER);
	    gtk_window_set_transient_for(GTK_WINDOW(open_insert),
					 GTK_WINDOW(app1));
	    gtk_widget_show(GTK_WIDGET(open_insert));
	}
	}

}


static void on_exit_management_cash_button_clicked(GtkButton *button,gpointer user_data) {
    gtk_widget_destroy(emited_payments);
    emited_payments = NULL;
    previous_money_order_days=0;

}

static void fill_account_numbers_combo(Payment * emited_payment,gulong client_code) {
    GtkWidget * combo;
    GList * accounts;
    GList * list;


    accounts = m_account_find_by_participant_code (client_code);

    combo = glade_xml_get_widget (xml, "parent_company_accounts_combo");

    c_util_clean_code_and_name_combo (combo);

    if (accounts!=NULL) {
	gint index_account=0;
        list = g_list_first(accounts);
	while (list!=NULL) {
	    gchar * complete_account_number=NULL;
	    Account * account = (Account *) list->data;

            complete_account_number = d_account_get_complete_number(account);
	    c_util_set_code_and_name_on_combo (index_account,complete_account_number,GTK_COMBO(combo));
	    g_free(complete_account_number);
            list = g_list_next(list);
	    index_account++;
	}

	if ((d_payment_get_payment_code (emited_payment) != ULONG_MAX) && (d_payment_get_is_direct_debit (emited_payment))) {
	    GtkWidget * entry = glade_xml_get_widget(xml,"parent_company_accounts_entry");
	    Account * account = d_account_create_empty ();
	    Office * office = d_office_create_empty ();
	    gchar * complete_account_number=NULL;
	    gchar * numero_aux=NULL;
	    d_office_set_bank_code (office,d_payment_get_bank_code (emited_payment));
	    d_office_set_office_code (office,d_payment_get_office_code (emited_payment));
	    numero_aux = d_payment_get_account_number (emited_payment);
	    d_account_set_office (account,office);
	    d_account_set_account_number (account,numero_aux);
	    g_free(numero_aux);
	    complete_account_number = d_account_get_complete_number (account);
	    gtk_entry_set_text(GTK_ENTRY(entry),complete_account_number);
	    g_free(complete_account_number);
	    d_account_clear (account);
	    gtk_widget_set_sensitive(combo,FALSE);
	} else {
	    gtk_widget_set_sensitive(combo,TRUE);

	}
	if (accounts!=NULL) {
	    g_list_free(accounts);
	    accounts =NULL;
	}
    }

}

static void get_code_from_payment(gpointer value) {
    gulong payment_code;
    if (emited_payment_to_manage[0]!=NULL) {
        payment_code = d_payment_get_payment_code (emited_payment_to_manage[0]);
	*((gulong *) value) =payment_code;
    }
}
