
/* exchange_pieces_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "util.h"
#include "parameter_manager.h"
#include "exchange_pieces_control.h"
#include "find_stocked_pieces_model.h"
#include "find_pieces_model.h"
#include "warehouse_model.h"
#include "enterprise_model.h"

#define C_EUROCODE_COLUMN          0
#define C_FAMILY_COLUMN           1
#define C_DESCRIPTION_COLUMN       2
#define C_WIDTH_COLUMN             3
#define C_LONG_COLUMN             4
#define C_ALL_ENTERPRISES           0
#define C_ALL_WAREHOUSES_OF_ENTERPRISE   0


static GtkWidget *exchange_pieces=NULL;
static GladeXML *xml = NULL;

static gchar url[]="/src/interface/glade/store_view.glade";
static gchar widget_name[]="vbox-intercambio";

GArray *selected;
Piece **pieces_array;
gulong  pieces_array_len;

static void on_eurocode_filtrering_entry_key_press_event(GtkWidget *widget,
								   GdkEventKey *event,
								   gpointer user_data);
static void on_description_filtering__key_press_event(GtkWidget *widget,
								      GdkEventKey *event,
								      gpointer user_data);
static void on_press_key(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void obtain_pieces(void);
static void fill_CList(GtkCList * list, gulong size);
static gint obtain_pieces_ids(GArray * selected, Piece **pieces, gulong *pieces_ids);
static void on_accept_exchange_button_clicked(GtkButton *button, gpointer data);
static void on_serch_exchange_button_clicked(GtkButton *button, gpointer data);
static void on_cancel_exchange_button_clicked(GtkButton *button, gpointer data);
static void clean_screen(void);

static void on_source_enterprise_entry_changed(void);
static void on_destination_enterprise_entry_changed(void);
static void fill_source_enterprise_combo(void);
static void fill_destination_enterprise_combo(void);
static void fill_source_warehouse_combo(void);
static void fill_destination_warehouse_combo	(void);
static void clean_source_enterprise_combo(void);
static void clean_destination_enterprise_combo(void);
static void clean_source_warehouse_combo(void);
static void clean_destination_warehouse_combo(void);
static void clean_combos(void);
static gboolean can_search(void);
static gboolean can_insert(void);

GtkWidget *c_exchange_piece_open_window(GtkWidget *app1, GtkWidget *vbox) {

    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    gchar *eo_name = NULL;
    gchar *ed_name = NULL;
    gchar *wo_name = NULL;
    gchar *wd_name = NULL;

    selected=g_array_new(0,0,sizeof(gint));

    if (exchange_pieces==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_entry-filtrado-eurocode-intercambio_key_press_event",
				  GTK_SIGNAL_FUNC (on_eurocode_filtrering_entry_key_press_event));
	glade_xml_signal_connect (xml, "on_entry-filtrado-descripcion-intercambio_key_press_event",
				  GTK_SIGNAL_FUNC (on_description_filtering__key_press_event));
	glade_xml_signal_connect (xml, "on_combo-entry-empresaorigen-intercambio_changed",
				  GTK_SIGNAL_FUNC (on_source_enterprise_entry_changed));
	glade_xml_signal_connect (xml, "on_combo-entry-empresadestino-intercambio_changed",
				  GTK_SIGNAL_FUNC (on_destination_enterprise_entry_changed));
	glade_xml_signal_connect (xml, "on_boton-cancelar-intercambio_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_exchange_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-aceptar-intercambio_clicked",
				  GTK_SIGNAL_FUNC (on_accept_exchange_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-buscar-intercambio_clicked",
				  GTK_SIGNAL_FUNC (on_serch_exchange_button_clicked));

	fill_source_enterprise_combo();
	fill_destination_enterprise_combo();
	exchange_pieces =dialog;
    }
#ifdef IS_SHOP
    gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo-entry-almacendestino-intercambio"),FALSE);
    gtk_widget_set_sensitive(glade_xml_get_widget (xml, "combo-entry-empresadestino-intercambio"),FALSE);

#endif
    ed_name = u_parameter_manager_get_parameter ("empresa");
    if (ed_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresadestino-intercambio")), ed_name);
    }
    wd_name = u_parameter_manager_get_parameter ("almacen");
    if (wd_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacendestino-intercambio")), wd_name);
    }
    eo_name = u_parameter_manager_get_parameter ("empresa");
    if (eo_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresaorigen-intercambio")), eo_name);
    }
    wo_name = u_parameter_manager_get_parameter ("almacen");
    if (wo_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacenorigen-intercambio")), wo_name);
    }

    g_free(eo_name);
    g_free(ed_name);
    g_free(wo_name);
    g_free(wd_name);

    return exchange_pieces;

}


static void on_eurocode_filtrering_entry_key_press_event(GtkWidget *widget,
								   GdkEventKey *event,
								   gpointer user_data) {
    on_press_key(widget, event, user_data);
}

static void on_description_filtering__key_press_event(GtkWidget *widget,
								      GdkEventKey *event,
								      gpointer user_data) {
    on_press_key(widget, event, user_data);
}

static void on_press_key(GtkWidget *widget, GdkEventKey *event,
			 gpointer user_data) {
    GtkWidget * wid;
    GtkContainer * container;
    GList * list=NULL;
    GList * lastList;
    GList * firstFocusableOfList;

    if (event->keyval==GDK_Return)  {
	on_serch_exchange_button_clicked(NULL, user_data);
    } else if (event->keyval==GDK_Tab) {

	container = (GtkContainer *) widget->parent;
	list = gtk_container_children(container);
	wid=GTK_WIDGET((list)->data);

	while (((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
	       ((GTK_WIDGET_VISIBLE(wid))==0) ||
	       ((GTK_WIDGET_SENSITIVE(wid))==0)) {
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}

	firstFocusableOfList = list;

	lastList = (GList *) g_list_last(list);
	while (lastList!=list) {
	    wid=GTK_WIDGET((list)->data);
	    if (GTK_WIDGET_HAS_FOCUS(wid)==1) {
		break;
	    }
	    list=g_list_next(list);

	}
	list=g_list_next(list);
	wid=GTK_WIDGET((list)->data);
	while ((((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
		((GTK_WIDGET_VISIBLE(wid))==0) ||
		((GTK_WIDGET_SENSITIVE(wid))==0)) && (lastList!=list)){
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}
	if (lastList==list) {
	    list=firstFocusableOfList;
	    wid=GTK_WIDGET((list)->data);
	}

	gtk_signal_emit_stop_by_name(GTK_OBJECT (widget),"key_press_event");

	gtk_widget_grab_focus(wid);

    }
}

static void on_serch_exchange_button_clicked(GtkButton *button,
						gpointer data) {
    if (can_search()){
	obtain_pieces();
    }
}

void obtain_pieces() {
    GtkWidget * list;
    ConditionFindPieces con;
    gchar * content;
    GdkCursor * cursor;
    GtkWidget * window;
    gulong pieces_array_len = 0;

    list = glade_xml_get_widget(xml,"clist-listadopiezas-intercambio");
    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    content=gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-filtrado-eurocode-intercambio")),0,-1);
    if (!strcmp(content,"")) {
	con.eurocode=NULL;
    } else {
	con.eurocode   = (gchar *) g_malloc (strlen(content)+1);
	strcpy(con.eurocode, content);

    }
    g_free(content);

    content=gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-filtrado-descripcion-intercambio")),0,-1);
    if (!strcmp(content,"")) {
	con.description=NULL;
    } else {
	con.description   = (gchar *) g_malloc (strlen(content)+1);
	strcpy(con.description, content);
    }
    g_free(content);

    gtk_clist_clear ((GtkCList *) list);
    m_piece_clear_array (pieces_array);
    pieces_array=NULL;

    pieces_array = m_piece_find_by_condition (&con,&pieces_array_len);

    if ((pieces_array!=NULL) && (pieces_array_len>0)) {
	fill_CList(GTK_CLIST(list), pieces_array_len);
    } else {
	c_util_open_info_window (_("It hasn't been found any piece"));
    }

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
}


static void fill_CList(GtkCList * list, gulong size) {

    gint i;
    gchar * temp;

    gchar *empty_row[]={"","","","","",""};


    for (i=0;i<size;i++) {
	if (i==30) gtk_clist_freeze ((GtkCList *) list);

	gtk_clist_append ((GtkCList *) list,empty_row);

	if (d_product_get_is_piece ((Product *)pieces_array[i])) {
	    gtk_clist_set_text((GtkCList *) list, i, C_EUROCODE_COLUMN, d_piece_get_euroCode (pieces_array[i]));

	    temp = (gchar *) g_malloc(16*sizeof(d_piece_get_width (pieces_array[i])));
	    snprintf(temp,16,"%d",d_piece_get_width (pieces_array[i]));
	    gtk_clist_set_text((GtkCList *) list, i, C_WIDTH_COLUMN, temp);
	    g_free(temp);

	    temp = (gchar *) g_malloc(16*sizeof(d_piece_get_length (pieces_array[i])));
	    snprintf(temp,16,"%d",d_piece_get_length (pieces_array[i]));
	    gtk_clist_set_text((GtkCList *) list, i, C_LONG_COLUMN, temp);
	    g_free(temp);

	}

	gtk_clist_set_text((GtkCList *) list, i, C_FAMILY_COLUMN, d_product_get_family_code ((Product *)pieces_array[i]));
	gtk_clist_set_text((GtkCList *) list, i, C_DESCRIPTION_COLUMN, d_product_get_description ((Product *)pieces_array[i]));

    }

    gtk_clist_thaw ((GtkCList *) list);

}

static gint obtain_pieces_ids(GArray * selected, Piece **pieces, gulong *pieces_ids) {
    gint index=0;
    gint indexOfPiece=0;

    while (index<selected->len) {
	indexOfPiece = g_array_index(selected,gint,index);

	pieces_ids[index] =
	    d_product_get_product_code ((Product *)pieces[indexOfPiece]);
	index++;
    }
    pieces_ids[index]=-1;
    return index;
}

static void on_accept_exchange_button_clicked(GtkButton *button,
						 gpointer data) {
    GtkSpinButton *spinbutton =  NULL;
    GtkWidget *check       = NULL;
    GtkWidget *list        = glade_xml_get_widget (xml, "clist-listadopiezas-intercambio");
    GArray    *selected    = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
    Exchange  *exchange    = NULL;
    gdouble    unidades       = G_MINDOUBLE;
    gdouble    unidades_stock_in = G_MINDOUBLE;
    gdouble    unidades_stock_out = G_MINDOUBLE;
    gulong  n              =  2;
    gulong  *pieces_ids = NULL;
    Stock **stocked_pieces_in = NULL;
    Stock **stocked_pieces_out = NULL;
    gboolean movimiento_salida = FALSE;
    gboolean correct = FALSE;
    gulong code_origin_enterprise =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-empresaorigen-intercambio")));
    gulong code_origin_warehouse =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-almacenorigen-intercambio")));
    gulong code_destination_enterprise =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-empresadestino-intercambio")));
    gulong code_destination_warehouse =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-almacendestino-intercambio")));

    if (can_insert()) {
	if ((selected->len)==1) {
	    pieces_ids = (gulong *) g_malloc(sizeof(gulong)*(1+1));

	    obtain_pieces_ids(selected, pieces_array, pieces_ids);

	    // Only load the first one. (Really, there is only one)
	    if (pieces_ids[0]!=-1) {
		spinbutton =  GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "spinbutton-unidades-intercambio"));
		gtk_spin_button_update(spinbutton);
		unidades = (gdouble)gtk_spin_button_get_value_as_float(spinbutton);

		check = glade_xml_get_widget(xml, "radiobutton-salida-intercambio");
                movimiento_salida = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check))==TRUE);


		stocked_pieces_out =
		    m_stock_find_pieces_by_product_code (pieces_ids,
								code_origin_enterprise,
								code_origin_warehouse,
								&n);

		stocked_pieces_in =
		    m_stock_find_pieces_by_product_code (pieces_ids,
								code_destination_enterprise,
								code_destination_warehouse,
								&n);

		if ((d_stock_get_quantity (stocked_pieces_out[0])-unidades)>=0) {
		    Product product = d_piece_get_product (d_stock_get_piece (stocked_pieces_out[0]));

		    if (d_stock_get_quantity (stocked_pieces_in[0])!=G_MINDOUBLE)
			unidades_stock_in = d_stock_get_quantity (stocked_pieces_in[0]) + unidades;
		    else
			unidades_stock_in = d_stock_get_quantity (stocked_pieces_in[0]);

		    unidades_stock_out = d_stock_get_quantity (stocked_pieces_out[0]) - unidades;

		    d_stock_set_quantity (stocked_pieces_in[0], unidades_stock_in);

		    d_stock_set_quantity (stocked_pieces_out[0], unidades_stock_out);

		    exchange = d_exchange_create_empty ();

		    d_exchange_set_enterprise_code (exchange, code_origin_enterprise);
		    d_exchange_set_warehouse_code (exchange, code_origin_warehouse);
		    d_exchange_set_quantity (exchange, unidades);
		    d_exchange_set_consignee_enterprise_code (exchange, code_destination_enterprise);
		    d_exchange_set_consignee_warehouse_code (exchange, code_destination_warehouse);
		    d_exchange_set_product_code (exchange, d_product_get_product_code (&product));

		    correct = m_exchange_insert (exchange, stocked_pieces_in[0],stocked_pieces_out[0]);
		    if(!correct) {
			c_util_open_error_window (_("The exchange couldn't be done correctly. There has been a problem with the database.\n Try it again.\n"));
		    }
		} else {
		    c_util_open_error_window (_("There isn't enough stock to exchange"));
		}
	    }
	    clean_screen();
	    g_free(pieces_ids);
	    m_stock_clear_array (stocked_pieces_in);
	    m_stock_clear_array (stocked_pieces_out);
	    stocked_pieces_in = NULL;
	    stocked_pieces_out = NULL;
	} else {
	    if ((selected->len)==0) {
		c_util_open_error_window (_("You should select a piece the exchange"));
	    } else {
		c_util_open_error_window (_("Select just a piece to exchange"));
	    }
	}
    }
}

static void clean_screen() {

    GtkWidget *vbox;
    GtkWidget *list;

    list = glade_xml_get_widget(xml,"clist-listadopiezas");
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    if (pieces_array!=NULL) {
	if (list!=NULL) {
	    gtk_clist_clear ((GtkCList *) list);
	}
	m_piece_clear_array (pieces_array);
	pieces_array=NULL;
    }

}

static void on_cancel_exchange_button_clicked(GtkButton *button,
						  gpointer data) {
    clean_screen();
}

static void on_source_enterprise_entry_changed() {
    fill_source_warehouse_combo();
}

static void on_destination_enterprise_entry_changed() {
    fill_destination_warehouse_combo();
}


static void fill_source_warehouse_combo() {
    GtkWidget * warehouse_combo;
    GtkWidget * enterprise_combo;
    Warehouse ** warehouses;
    gint code_combo;

    enterprise_combo = glade_xml_get_widget (xml, "combo-empresaorigen-intercambio");
    warehouse_combo = glade_xml_get_widget (xml, "combo-almacenorigen-intercambio");
    code_combo = c_util_get_code_from_combo (enterprise_combo);


    warehouses = m_warehouse_find_by_enterprise_code (code_combo);

    clean_source_warehouse_combo();

    c_util_set_code_and_name_on_combo (C_ALL_WAREHOUSES_OF_ENTERPRISE,g_strdup(_("All the warehouses")),GTK_COMBO(warehouse_combo));

    c_util_fill_combo_with_warehouses(warehouse_combo,warehouses);

    m_warehouse_clear_array (warehouses);
    warehouses = NULL;

}

static void fill_destination_warehouse_combo	() {
    GtkWidget * warehouse_combo;
    GtkWidget * enterprise_combo;
    Warehouse ** warehouses;
    gint code_combo;

    enterprise_combo = glade_xml_get_widget (xml, "combo-empresadestino-intercambio");
    warehouse_combo = glade_xml_get_widget (xml, "combo-almacendestino-intercambio");
    code_combo = c_util_get_code_from_combo (enterprise_combo);

    warehouses = m_warehouse_find_by_enterprise_code (code_combo);

    clean_destination_warehouse_combo();

    c_util_set_code_and_name_on_combo (C_ALL_WAREHOUSES_OF_ENTERPRISE,g_strdup(_("All the warehouses")),GTK_COMBO(warehouse_combo));
    c_util_fill_combo_with_warehouses(warehouse_combo,warehouses);

    m_warehouse_clear_array (warehouses);
    warehouses = NULL;

}

static void fill_source_enterprise_combo() {
    GtkWidget * combo;
    Enterprise ** enterprises;

    combo = glade_xml_get_widget (xml, "combo-empresaorigen-intercambio");
    c_util_set_code_and_name_on_combo (C_ALL_ENTERPRISES,g_strdup(_("All the enterprises")),GTK_COMBO(combo));
    enterprises = m_enterprise_find_all ();

    c_util_fill_combo_with_enterprises (combo,enterprises);

}

static void fill_destination_enterprise_combo() {
    GtkWidget * combo;
    Enterprise ** enterprises;

    combo = glade_xml_get_widget (xml, "combo-empresadestino-intercambio");

    c_util_set_code_and_name_on_combo (C_ALL_ENTERPRISES,g_strdup(_("All the enterprises")),GTK_COMBO(combo));
    enterprises = m_enterprise_find_all ();

    c_util_fill_combo_with_enterprises (combo,enterprises);

}

static void clean_source_enterprise_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-empresaorigen-intercambio");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-empresaorigen-intercambio");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_destination_enterprise_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-empresadestino-intercambio");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-empresadestino-intercambio");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_source_warehouse_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-almacenorigen-intercambio");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-almacenorigen-intercambio");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_destination_warehouse_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-almacendestino-intercambio");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-almacendestino-intercambio");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_combos() {
    gchar *eo_name = NULL;
    gchar *ed_name = NULL;
    gchar *wo_name = NULL;
    gchar *wd_name = NULL;


    clean_source_enterprise_combo();
    clean_source_warehouse_combo();
    clean_destination_enterprise_combo();
    clean_destination_warehouse_combo();

    fill_source_enterprise_combo();
    fill_destination_enterprise_combo();

    ed_name = u_parameter_manager_get_parameter ("empresa");
    if (ed_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresadestino-intercambio")), ed_name);
    }
    wd_name = u_parameter_manager_get_parameter ("almacen");
    if (wd_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacendestino-intercambio")), wd_name);
    }
    eo_name = u_parameter_manager_get_parameter ("empresa");
    if (eo_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-empresaorigen-intercambio")), eo_name);
    }
    wo_name = u_parameter_manager_get_parameter ("almacen");
    if (wo_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-almacenorigen-intercambio")), wo_name);
    }
}

static gboolean can_search() {
    gulong long_entry_eurocode =
	strlen(g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-filtrado-eurocode-intercambio")),0,-1)));
    gulong long_entry_description =
	strlen(g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-filtrado-descripcion-intercambio")),0,-1)));

    if ((long_entry_eurocode>0) || (long_entry_description>0)) {
	return TRUE;
    } else {
	c_util_open_error_window (_("Fill the fields eurocode/description to search"));
    }
    return FALSE;
}

static gboolean can_insert() {
    gboolean result = FALSE;
    gulong long_entry_eurocode =
	strlen(g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-filtrado-eurocode-intercambio")),0,-1)));
    gulong long_entry_description =
	strlen(g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-filtrado-descripcion-intercambio")),0,-1)));
    gulong code_origin_enterprise =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-empresaorigen-intercambio")));
    gulong code_origin_warehouse =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-almacenorigen-intercambio")));
    gulong code_destination_enterprise =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-empresadestino-intercambio")));
    gulong code_destination_warehouse =
	c_util_get_code_from_combo (GTK_WIDGET(glade_xml_get_widget (xml, "combo-almacendestino-intercambio")));

    if ((long_entry_eurocode>0) || (long_entry_description>0)) {
	if ((code_origin_enterprise!=C_ALL_ENTERPRISES)
	    && (code_origin_warehouse!=C_ALL_WAREHOUSES_OF_ENTERPRISE)) {
	    if ((code_destination_enterprise!=C_ALL_ENTERPRISES)
		&& (code_destination_warehouse!=C_ALL_WAREHOUSES_OF_ENTERPRISE)) {

		if ((code_destination_enterprise!=code_origin_enterprise)
		    || (code_destination_warehouse!=code_origin_warehouse)) {
		    result = TRUE;
		} else {
		    c_util_open_error_window (_("Source and destination have to be different"));
		}
	    } else {
		c_util_open_error_window (_("Select the destionation of the piece"));
	    }
	} else {
	    c_util_open_error_window (_("Select the source of the piece"));
	}
    } else {
	c_util_open_error_window (_("Fill the fields eurocode/description to search"));
    }
    return result;
}
