/* insert_cash_operation_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "util.h"
#include "parameter_manager.h"
#include "control_util.h"
#include "find_cash_movement_control.h"
#include "insert_account_control.h"
#include "account_model.h"
#include "cash_operation_model.h"
#include "cash_model.h"
#include "shop_model.h"
#include "enterprise_model.h"
#include "participant.h"
#include "enterprise.h"

static GtkWidget *find_cash_movement=NULL;
static GladeXML *xml;
static gulong shop_code=ULONG_MAX;
static gulong enterprise_code=ULONG_MAX;

static CashOperation **list_of_cash_operations = NULL;
 
static gchar widget_name[]="vbox_consulta_de_movimientos_caja";
static gchar url[]="/src/interface/glade/cash_view.glade";
static void on_consult_button_clicked(GtkButton *button, gpointer data);
static void on_clean_button_clicked(GtkButton *button, gpointer data);
static void on_delete_button_clicked(GtkButton *button, gpointer data);
static void on_window_destroy(GtkWidget * widget,gpointer data);
static void on_shop_combo_entry_changed(GtkWidget * widget,gpointer data);
static void fill_shop_combo(void);
static void fill_enterprise_entry(void);
static void fill_balance_entry(void);
static void fill_cash_operation_list(GtkCList * list, CashOperation ** array);
static void clean_screen(void);
static void obtain_cash_movement(GArray * );
static void on_ok_delete_cash_movements(gint reply, gpointer user_data);


GtkWidget *c_find_cash_movement_open_window(GtkWidget *app1,GtkWidget *vbox)
{
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    gchar *s_name = NULL;
   
  if (find_cash_movement==NULL)
    {
      GtkWidget *dialog;

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);

      gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

      glade_xml_signal_connect (xml, "on_button_consultar_movimientos_caja_clicked",
				GTK_SIGNAL_FUNC (on_consult_button_clicked));
      glade_xml_signal_connect (xml, "on_button_limpiar_consulta_movimientos_caja_clicked",
				GTK_SIGNAL_FUNC (on_clean_button_clicked));
      glade_xml_signal_connect (xml, "on_button_borrar_movimientos_caja_clicked",
				GTK_SIGNAL_FUNC (on_delete_button_clicked));
      glade_xml_signal_connect (xml, "on_consulta_de_movimientos_caja_destroy",
				GTK_SIGNAL_FUNC (on_window_destroy));
      glade_xml_signal_connect (xml, "on_combo_entry_tienda_consulta_movimientos_caja_changed",
				GTK_SIGNAL_FUNC (on_shop_combo_entry_changed));
  
      find_cash_movement=dialog;
      
      fill_shop_combo();
      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry-tienda-movimientos-caja")), s_name);
#ifndef IS_SHOP
	  gtk_widget_set_sensitive(glade_xml_get_widget(xml, "combo-tienda-consulta-movimientos-caja"),FALSE);
#endif
      }
      g_free(s_name);

    }

  fill_balance_entry();
  clean_screen();
  g_free(directory);
  return find_cash_movement; 
}

static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;
    combo = glade_xml_get_widget (xml, "combo-tienda-consulta-movimientos-caja");
    shops = m_shop_find_all ();
    c_util_fill_combo_with_shops (combo, shops);

#ifdef IS_SHOP

    gtk_widget_set_sensitive(combo,FALSE);

#endif    
}

static void fill_enterprise_entry() {
  GtkWidget * entry;
  Enterprise * enterprise;
  gchar* enterprise_name;
  shop_code=c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo-tienda-consulta-movimientos-caja"));
  enterprise = m_enterprise_find_by_shop (shop_code);
  entry = glade_xml_get_widget (xml,"entry-empresa-consulta-movimientos-caja");
  enterprise_name=d_participant_get_name (d_enterprise_get_participant (enterprise));
  enterprise_code=d_participant_get_code (d_enterprise_get_participant (enterprise));
  gtk_entry_set_text(GTK_ENTRY(entry),enterprise_name);
  d_enterprise_clear (enterprise);
}

static void fill_balance_entry() {
  GtkWidget * entry = glade_xml_get_widget (xml, "entry-saldo-consulta-movimientos-caja");
  Cash *cash=NULL;
  gdouble balance=0.00;
  gchar *balance_char=NULL;
  cash = m_cash_find_by_shop (enterprise_code,shop_code);
  balance=d_cash_get_current_balance (cash);
  balance_char=g_strdup_printf("%.2f",balance);
  gtk_entry_set_text(GTK_ENTRY(entry),balance_char);
  g_free(balance_char);
}


static void on_consult_button_clicked(GtkButton *button, gpointer data) {

	obtain_cash_movement(NULL);  
}


static void on_delete_button_clicked(GtkButton *button, gpointer data) {

   	GtkWidget * clist = glade_xml_get_widget(xml,"clist-movimientos-caja");
	GArray * selected;
	
	c_util_open_question_dialog(_("Are you sure you want to remove the selected movements?"), *on_ok_delete_cash_movements);

        selected = c_util_get_selected_rows_from_clist (GTK_CLIST(clist));
	if (selected->len > 0) {
		
		c_util_open_info_window (_("Marked lines can't be removed, they already have a linked payment. !!! \nJust unlinked payments can be removed"));
		
		/*tk_clist_unselect_all(GTK_CLIST(clist));*/
		g_array_free(selected, TRUE);
	}
}	


static void on_ok_delete_cash_movements(gint reply, gpointer user_data) {
     
   if (!reply) {
    	
   	GtkWidget * clist = glade_xml_get_widget(xml,"clist-movimientos-caja");      
	GArray * selected = c_util_get_selected_rows_from_clist (GTK_CLIST(clist));
        gboolean result =FALSE;
   
   	result = m_cash_operation_delete (selected, list_of_cash_operations, shop_code, enterprise_code);

	if (result !=FALSE) {
	    if (selected->len == 0) {
		g_array_free(selected, TRUE);
		selected = NULL;
	    }

	} else {
            c_util_open_error_window (_("They couln't be deleted because there has been a problem in the database"));
	}
	obtain_cash_movement(selected);

   }
}


static void obtain_cash_movement(GArray * selected) {

	time_t init_date, end_date;
	Cash *cash_movement = NULL;
   	GtkWidget * clist = glade_xml_get_widget(xml,"clist-movimientos-caja");
	GtkWidget * init_date_dateedit = glade_xml_get_widget(xml,"dateedit-fechainicio-movimientos-caja");
	GtkWidget * end_date_dateedit = glade_xml_get_widget(xml,"dateedit-fechafin-movimientos-caja");	   

	if (list_of_cash_operations != NULL) {
		
		m_cash_operation_clear_array (list_of_cash_operations);
		list_of_cash_operations = NULL;
	}
	 
	init_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(init_date_dateedit));
	end_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(end_date_dateedit));
	   
	cash_movement = m_cash_find_by_shop (enterprise_code,shop_code);
        list_of_cash_operations = m_cash_find_by_period (init_date,end_date,cash_movement);	 

  	c_util_clean_widget (clist, NULL);
   	fill_cash_operation_list(GTK_CLIST(clist),list_of_cash_operations);
	
	if (selected != NULL) c_util_list_select_rows (GTK_CLIST(clist), selected);
	 
	d_cash_clear_cash (cash_movement);	
}
	   
	

static void on_clean_button_clicked(GtkButton *button, gpointer data) {
  clean_screen();
}


static void on_window_destroy(GtkWidget * widget,gpointer user_data) {

    gtk_widget_destroy(find_cash_movement);
    find_cash_movement = NULL;
}


static void on_shop_combo_entry_changed(GtkWidget * widget,gpointer data) {
    fill_enterprise_entry();
#ifndef IS_SHOP
    fill_balance_entry();
#endif
}


static void fill_cash_operation_list(GtkCList * list, CashOperation ** array) {
    gchar * content=NULL;
    gint index=0;
    gchar *empty_row[]={"","",""};
    time_t *date;
    struct tm *datetm;

   gtk_clist_clear((GtkCList *) list);
   if (array!=NULL) {
	while (array[index]!=NULL) {
	    if (index==30) gtk_clist_freeze ((GtkCList *) list);
	    gtk_clist_append ((GtkCList *) list,empty_row);
	    date = (time_t *) g_malloc(sizeof(time_t));
	    //datetm = (struct tm *) g_malloc(sizeof(struct tm));
	    *date=d_cash_operation_get_date (array[index]);
	    datetm=localtime(date);
	    content = g_strdup_printf("%i/%i/%i",datetm->tm_mday,(datetm->tm_mon)+1,(datetm->tm_year)+1900);
	    gtk_clist_set_text((GtkCList *) list, index, 0, content);
	    g_free(content);
    	    g_free(date);
	    //g_free(datetm);

	    content = g_strdup_printf("%.2f",1.0*(d_cash_operation_get_amount (array[index])));
	    gtk_clist_set_text((GtkCList *) list, index, 1, content);
	    g_free(content);

	    gtk_clist_set_text((GtkCList *) list, index, 2,d_cash_operation_get_section (array[index]));
	    //d_cash_operation_clear (array[index]);
	    index++;
	}
	//g_free(array);
   }
   else {
     c_util_open_info_window (_("There is not any cash movement"));
   }
   gtk_clist_thaw ((GtkCList *) list);

}

static void clean_screen() {
    GtkCList * list = GTK_CLIST(glade_xml_get_widget(xml,"clist-movimientos-caja"));
    gchar *s_name = NULL;
    gtk_clist_clear((GtkCList *) list);
    s_name = u_parameter_manager_get_parameter ("tienda");
    if (s_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry-tienda-movimientos-caja")), s_name);
    }
    g_free(s_name);
}

