/* find_charge_delivery_payment_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>
#include <stdlib.h>

#include "a-shortcut.h"
#include "control_util.h"
#include "support.h"

#include "parameter_manager.h"

#include "find_charge_delivery_payment_control.h"
#include "clients_control.h"
#include "order_control.h"
#include "update_entry_control.h"

#include "warehouse_model.h"
#include "supplier_model.h"
#include "enterprise_model.h"
#include "order_model.h"
#include "order_line_model.h"

#define C_EUROCODE_COLUMN     2
#define C_FAMILY_COLUMN      3
#define C_DESCRIPTION_COLUMN  4
#define C_QUANTITY_COLUMN     5
#define C_ORDER_CODE_COLUMN 0
#define C_ORDER_LINE_NUMBER_COLUMN 1
#define C_INVOICE_NUMBER_COLUMN 6

#define C_ALL_SUPPLIERS                0
#define C_ALL_ENTERPRISES               0
#define C_ALL_WAREHOUSES_OF_ENTERPRISE 0

static GtkWidget *charge_delivery_payment=NULL;
static GladeXML *xml = NULL;

static gchar        url[]="/src/interface/glade/store_view.glade";
static gchar        widget_name[]="vbox1_busqueda_albaranes_cargo";
static Order      **order_array = NULL;

static void fill_lines_list(GtkCList * list, gulong order_array_len);


static void on_find_button_clicked(GtkButton *button,gpointer data);
static void on_delete_button_clicked(GtkButton *button,gpointer data);
static void on_enterprise_entry_changed(void);
static void on_delivery_note_number_entry_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void fill_supplier_combo(void);
static void fill_warehouse_combo(void);
static void fill_enterprise_combo(void);
static void clean_warehouse_combo(void);
static void clean_combos(void);
static void clean_screen(void);


GtkWidget *c_find_charge_delivery_payment_open_window(GtkWidget *app1, GtkWidget *vbox) {

    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    gchar *e_name = NULL;
    gchar *w_name = NULL;

    if (charge_delivery_payment==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);
                                        
	glade_xml_signal_connect (xml, "on_button_buscar_busqueda_albaranes_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_find_button_clicked));
	glade_xml_signal_connect (xml, "on_button_borrar_busqueda_albaranes_cargo_clicked",
				  GTK_SIGNAL_FUNC (on_delete_button_clicked));
	glade_xml_signal_connect (xml, "on_entry_empresa_busqueda_albaranes_cargo_changed",
				  GTK_SIGNAL_FUNC (on_enterprise_entry_changed));
	glade_xml_signal_connect (xml, "on_entry_numero_albaran_busqueda_albaranes_cargo_key_press_event",
				  GTK_SIGNAL_FUNC (on_delivery_note_number_entry_key_press_event));

	clean_screen();
	e_name = u_parameter_manager_get_parameter ("empresa");
	if (e_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_empresa_busqueda_albaranes_cargo")), e_name);
	}

	w_name = u_parameter_manager_get_parameter ("almacen");
	if (w_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_almacen_busqueda_albaranes_cargo")), w_name);
	}

	g_free(e_name);
        g_free(w_name);
	fill_supplier_combo();
//        fill_enterprise_combo();
	charge_delivery_payment =dialog;
    }


    g_free(directory);

    return charge_delivery_payment;

}


static void on_find_button_clicked(GtkButton *button, gpointer data) {
    ConditionFindOrder condition;
    GtkWidget * lines_list;
    GtkWidget * entry_albaran;
    GdkCursor * cursor;
    GtkWidget * window;
    gulong  order_array_len = 0;
    gulong  code;
    gchar * payment_note_number=NULL;

    lines_list    = glade_xml_get_widget(xml,"lista_detalle_busqueda_albaranes_cargo");
    window = lookup_widget(GTK_WIDGET(lines_list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
    entry_albaran = glade_xml_get_widget (xml, "entry_numero_albaran_busqueda_albaranes_cargo");
    gtk_widget_set_sensitive(entry_albaran,FALSE);
    //We don't use this line yet.
    condition.order_code = ULONG_MAX;

    code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo_empresa_busqueda_albaranes_cargo"));
    condition.enterprise_code = code;

    code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo_almacen_busqueda_albaranes_cargo"));
    condition.warehouse_code = code;

    code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo_proveedor_busqueda_albaranes_cargo"));
    condition.supplier_code = code;

    payment_note_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_albaran)));

    gtk_clist_clear ((GtkCList *) lines_list);

    m_order_clear_array (order_array);
    order_array=NULL;

    order_array = m_order_find_by_condition_and_delivery_payment (&condition, payment_note_number,&order_array_len);

    g_free(payment_note_number);

    if ((order_array!=NULL) && (order_array_len>0)) {
	fill_lines_list(GTK_CLIST(lines_list), order_array_len);
    } else {
	c_util_open_info_window (_("Theres is not any order"));
    }

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
    gtk_widget_set_sensitive(entry_albaran,TRUE);

}




static void on_delete_button_clicked(GtkButton *button, gpointer data) {
    clean_screen();
}



static void on_enterprise_entry_changed() {
    fill_warehouse_combo();
}



static void fill_lines_list(GtkCList * list, gulong size) {

    gchar * temp;
    gint   index=0;
    gint   index_show=0;
    gchar *empty_row[]={"","","","","","",""};
    gint i;
    gchar * payment_note_number =g_strdup(gtk_entry_get_text(GTK_ENTRY(
    		glade_xml_get_widget (xml, "entry_numero_albaran_busqueda_albaranes_cargo"))));
 
 
    for (i=0;i<size;i++) {
	OrderLine ** lines = d_order_get_order_lines (order_array[i]);
        gchar * order_code = g_strdup_printf("%lu",d_order_get_code (order_array[i]));

	if (lines!=NULL) {
	    while (lines[index]!=NULL) {
		gchar * payment_note_line = d_order_line_get_delivery_payment (lines[index]);
		if (payment_note_line!=NULL) {
		    if (!strcmp(payment_note_line,payment_note_number)) {
			gtk_clist_append ((GtkCList *) list,empty_row);

			temp = g_strdup(order_code);
			gtk_clist_set_text((GtkCList *) list, index_show, C_ORDER_CODE_COLUMN, temp);
			g_free(temp);

			temp = g_strdup_printf("%lu", d_order_line_get_line_number (lines[index_show]));
			gtk_clist_set_text((GtkCList *) list, index_show, C_ORDER_LINE_NUMBER_COLUMN, temp);
			g_free(temp);

			temp = d_piece_get_euroCode (d_order_line_get_piece (lines[index_show]));
			gtk_clist_set_text((GtkCList *) list, index_show, C_EUROCODE_COLUMN, temp);
			g_free(temp);

			temp = d_product_get_description ((Product *)d_order_line_get_piece (lines[index_show]));
			gtk_clist_set_text((GtkCList *) list, index_show, C_DESCRIPTION_COLUMN, temp);
			g_free(temp);

			temp = d_product_get_family_code ((Product *)d_order_line_get_piece (lines[index_show]));
			gtk_clist_set_text((GtkCList *) list, index_show, C_FAMILY_COLUMN, temp);
			g_free(temp);


                        if (d_order_line_get_quantity (lines[index_show])!=G_MINDOUBLE)
			    temp = g_strdup_printf("%.3f", d_order_line_get_quantity (lines[index_show]));
			else
			    temp = g_strdup("");

			gtk_clist_set_text((GtkCList *) list, index_show, C_QUANTITY_COLUMN, temp);
			g_free(temp);

			if (d_order_line_get_charge_invoice_number (lines[index_show])!=ULONG_MAX) {
			    temp = g_strdup_printf("%lu",d_order_line_get_charge_invoice_number (lines[index_show]));
			    if (temp!=NULL) {
				gtk_clist_set_text((GtkCList *) list, index_show, C_INVOICE_NUMBER_COLUMN, temp);
				g_free(temp);
			    }
			}
			gtk_clist_thaw ((GtkCList *) list);
			index_show++;

		    }
		    g_free(payment_note_line);
		}
		index++;
	    }
	}
        g_free(order_code);
    }
    g_free(payment_note_number);
}


static void fill_warehouse_combo() {
    GtkWidget * warehouse_combo;
    GtkWidget * enterprise_combo;
    Warehouse ** warehouses;
    gint codeCombo;

    enterprise_combo = glade_xml_get_widget (xml, "combo_empresa_busqueda_albaranes_cargo");
    warehouse_combo = glade_xml_get_widget (xml, "combo_almacen_busqueda_albaranes_cargo");
    codeCombo = c_util_get_code_from_combo (enterprise_combo);


    warehouses = m_warehouse_find_by_enterprise_code (codeCombo);

    clean_warehouse_combo();

    c_util_set_code_and_name_on_combo (C_ALL_WAREHOUSES_OF_ENTERPRISE,g_strdup(_("All the warehouses")),GTK_COMBO(warehouse_combo));

    c_util_fill_combo_with_warehouses(warehouse_combo,warehouses);

    m_warehouse_clear_array (warehouses);
    warehouses = NULL;

}

static void fill_supplier_combo() {
    GtkWidget * combo;
    Supplier ** suppliers;

    combo = glade_xml_get_widget (xml, "combo_proveedor_busqueda_albaranes_cargo");

    c_util_set_code_and_name_on_combo (C_ALL_SUPPLIERS,g_strdup(_("All the suppliers")),GTK_COMBO(combo));
    suppliers = m_supplier_find_all ();

    c_util_fill_combo_with_suppliers (combo,suppliers);
}

static void fill_enterprise_combo() {
    GtkWidget * combo;
    Enterprise ** enterprises;

    combo = glade_xml_get_widget (xml, "combo_empresa_busqueda_albaranes_cargo");

    c_util_set_code_and_name_on_combo (C_ALL_ENTERPRISES,g_strdup(_("All the enterprises")),GTK_COMBO(combo));
    enterprises = m_enterprise_find_all ();

    c_util_fill_combo_with_enterprises (combo,enterprises);
}

static void clean_warehouse_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "entry_almacen_busqueda_albaranes_cargo");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo_almacen_busqueda_albaranes_cargo");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_enterprise_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "entry_empresa_busqueda_albaranes_cargo");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo_empresa_busqueda_albaranes_cargo");
    c_util_clean_code_and_name_combo (combo);

}

static void clean_combos() {
    gchar *e_name = NULL;
    gchar *w_name = NULL;
    clean_enterprise_combo();
    clean_warehouse_combo();
    fill_enterprise_combo();

    e_name = u_parameter_manager_get_parameter ("empresa");
    if (e_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_empresa_busqueda_albaranes_cargo")), e_name);
    }

    w_name = u_parameter_manager_get_parameter ("almacen");
    if (w_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry_almacen_busqueda_albaranes_cargo")), w_name);
    }
}

static void clean_screen() {
    GtkWidget *vbox;
    GtkWidget * lines_list;

    lines_list = glade_xml_get_widget(xml,"lista_detalle_busqueda_albaranes_cargo");
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    if (order_array!=NULL) {
	gtk_clist_clear ((GtkCList *) lines_list);
	m_order_clear_array (order_array);
	order_array=NULL;
    }

    clean_combos();
}

static void on_delivery_note_number_entry_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data) {
  GtkWidget *button =glade_xml_get_widget (xml, "button_buscar_busqueda_albaranes_cargo");

  if (event->keyval==GDK_Return)
    on_find_button_clicked(GTK_BUTTON(button), user_data);
}
