
/* find_manodeobra.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "find_manodeobra.h"
#include "control_util.h"
#include "manodeobra_model.h"
#include "repairs_control.h"
#include "participant.h"
#include "manodeobra.h"

#define C_DESCRIPTION_COLUMN 0
#define C_PRICE_COLUMN 1

static GtkWidget *find_man_power=NULL;
static GladeXML *xml;
static ManPower **man_power_array = NULL;
static Company *company = NULL;

static gchar url[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="insecion_manodeobra";

static void on_find_button_clicked(GtkButton *button,gpointer user_data);
static void on_accept_button_clicked(GtkButton *button,gpointer user_data);
static void on_cancel_button_clicked(GtkButton *button,gpointer user_data);
static void on_window_destroy(GtkWidget *widget,gpointer user_data);
static void on_description_entry_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void load_man_power_list(void);

GtkWidget *c_find_man_power_open_window(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (find_man_power==NULL)
     {
	GtkWidget *window;
	GtkWidget *toolbar;

	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	toolbar = glade_xml_get_widget (xml, "find_toolbar");

	glade_xml_signal_connect (xml, "on_find_button_clicked",
				  GTK_SIGNAL_FUNC (on_find_button_clicked));
	glade_xml_signal_connect (xml, "on_accept_button_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_cancel_button_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_insecion_manodeobra_destroy",
				  GTK_SIGNAL_FUNC (on_window_destroy));
	glade_xml_signal_connect (xml, "on_descripcion_entry_key_press_event",
				  GTK_SIGNAL_FUNC (on_description_entry_key_press_event));

	gtk_toolbar_set_style(GTK_TOOLBAR(toolbar), GTK_TOOLBAR_ICONS);

	find_man_power = window;
	
     }

   g_free(directory);

   return find_man_power;
   
}

void c_find_man_power_insuranced_repair(Company *local_company) {
  
  company = local_company;
}

static void load_man_power_list(){

    gint index=0;
    gchar *empty_row[]={"",""};
    GtkWidget *list =glade_xml_get_widget (xml, "manodeobra_clist");
    
    gtk_clist_clear(GTK_CLIST(list));

    if (man_power_array!=NULL) {
        while (man_power_array[index]!=NULL) {
	  gchar *description = d_product_get_description ((Product *) man_power_array[index]);
	  gdouble price_gdouble = d_product_get_price ((Product *) man_power_array[index]);

            if (index==30) gtk_clist_freeze (GTK_CLIST(list));

            gtk_clist_append ((GtkCList *) list,empty_row);

            if (description != NULL) {
                gtk_clist_set_text((GtkCList *) list, index, C_DESCRIPTION_COLUMN, description);
            }

            if (price_gdouble != G_MINDOUBLE) {
	      gchar *price =  g_strdup_printf("%.2f",price_gdouble);

	      gtk_clist_set_text((GtkCList *) list, index, C_PRICE_COLUMN, price);
	      
	      g_free(price);	    
            }

            index++;
	    g_free(description);

        }
    }

    gtk_clist_thaw ((GtkCList *) list);
}

static void on_find_button_clicked(GtkButton *button,gpointer user_data){

  gchar *nombre = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "descripcion_entry")),0,-1);
  gulong company_code_gulong = ULONG_MAX;
  gchar *company_code = NULL;

  if (company != NULL) {
    company_code_gulong = d_participant_get_code (d_company_get_participant (company));
    
    if (company_code_gulong!=ULONG_MAX)
      company_code = g_strdup_printf("%lu",company_code_gulong);
    else company_code = g_strdup("NULL");
  }
  
  if (man_power_array != NULL){
    m_man_power_clear_array (man_power_array);
    man_power_array = NULL;
  }

  if (!g_strcasecmp(nombre,""))
    man_power_array = m_man_power_find_by_description (NULL, company_code);
  else man_power_array = m_man_power_find_by_description (nombre, company_code);
  
  load_man_power_list();

  g_free(nombre);

}

static void on_accept_button_clicked(GtkButton *button,gpointer user_data){

  GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "manodeobra_clist"));

  GArray *selected = c_util_get_selected_rows_from_clist (list);

  if (selected->len>0){
    gint row = g_array_index(selected,gint,0);
    Repair_line *repair_line = d_repair_line_create_empty ();
    
    d_repair_line_set_product (repair_line, (Product *) d_man_power_duplicate (man_power_array[row]));
    d_repair_line_set_quantity (repair_line, 1.0);

    c_repair_add_line(repair_line);

    repair_line = NULL;    
  }

  g_array_free(selected, TRUE);    

  gtk_widget_destroy(find_man_power);

}

static void on_cancel_button_clicked(GtkButton *button,gpointer user_data){

  gtk_widget_destroy(find_man_power);
}

static void on_window_destroy(GtkWidget *widget,gpointer user_data){

  m_man_power_clear_array (man_power_array);
  d_company_clear (company);

  company = NULL;
  man_power_array=NULL;
  find_man_power = NULL;
  xml = NULL;
  
}

static void on_description_entry_key_press_event(GtkWidget *widget, GdkEventKey *event, gpointer user_data){

  GtkWidget *button =glade_xml_get_widget (xml, "buscar_button");

  if (event->keyval==GDK_Return)
    on_find_button_clicked(GTK_BUTTON(button), user_data);
}
