
/* find_order_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>
#include <stdlib.h>

#include "a-shortcut.h"
#include "control_util.h"
#include "support.h"

#include "parameter_manager.h"

#include "find_order_control.h"
#include "charge_invoice_control.h"
#include "clients_control.h"
#include "order_control.h"
#include "update_entry_control.h"

#include "warehouse_model.h"
#include "supplier_model.h"
#include "enterprise_model.h"
#include "order_model.h"
#include "order_line_model.h"

#define C_EUROCODE_COLUMN     0
#define C_FAMILY_COLUMN      1
#define C_DESCRIPTION_COLUMN  2
#define C_QUANTITY_COLUMN     3
#define C_DELIVERY_PAYMENT_COLUMN    4
#define C_INVOICE_NUMBER_COLUMN   5
#define C_LINE_NUMBER_COLUMN    6

#define C_ORDER_COLUMN       0
#define C_DATE_COLUMN        1
#define C_ENTERPRISE_COLUMN      2
#define C_WAREHOUSE_COLUMN      3
#define C_SUPPLIER_COLUMN    4

#define C_ALL_ENTERPRISES              0
#define C_ALL_WAREHOUSES_OF_ENTERPRISE 0

static GtkWidget *order=NULL;
static GladeXML *xml = NULL;

static gchar        url[]="/src/interface/glade/store_view.glade";
static gchar        widget_name[]="vbox-busqueda-pedidos";
static Participant *supplier_loaded = NULL;
static Order      **order_array = NULL;
static gint         selected_row = -1;

static void load_supplier_order(Participant *participant);
static void fill_order_list(GtkCList * list, gulong size);
static void fill_lines_list(GtkCList * list, Order *order);

void set_delivery_note_number(gpointer payment_note_number_value);

static void on_search_button_clicked(GtkButton *button,
						      gpointer data);
static void on_manage_button_clicked(GtkButton *button,
							 gpointer data);
static void on_clean_button_clicked(GtkButton *button,
						       gpointer data);
static void on_modify_button_clicked(GtkButton *button,
							 gpointer data);
static void on_filter_supplier_button_clicked(GtkButton *button,
								       gpointer data);
static void on_filter_enterprise_combo_entry_changed(void);
static void on_order_list_select_row(GtkCList *list,
							       gint      row,
							       gint      column,
							       GdkEvent *event,
							       gpointer  user_data);
static void on_order_list_unselect_row(GtkCList *list,
								 gint      row,
								 gint      column,
								 GdkEvent *event,
								 gpointer  user_data);
static gint on_order_list_search_button_press_event(GtkWidget *widget,
									     GdkEventButton *event, gpointer user_data);

static void fill_enterprise_combo(void);
static void fill_warehouse_combo(void);
static void clean_warehouse_combo(void);
static void clean_combos(void);
static void clean_screen(void);
static void on_payment_note_activate(GtkMenuItem *menuitem,
				gpointer user_data);
static void on_select_activate(GtkMenuItem *menuitem,
				gpointer user_data);
static void on_unselect_activate(GtkMenuItem *menuitem,
				gpointer user_data);

GtkWidget *c_find_order_open_window(GtkWidget *app1, GtkWidget *vbox, Participant *supplier_param) {

    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    gchar *e_name = NULL;
    gchar *w_name = NULL;

    if (order==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);
                                        
	glade_xml_signal_connect (xml, "on_combo-entry-filtro-empresa-busqueda-pedidos_changed",
				  GTK_SIGNAL_FUNC (on_filter_enterprise_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_button-filtros-buscarproveedor-busqueda-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_filter_supplier_button_clicked));
	glade_xml_signal_connect (xml, "on_button-buscar-busqueda-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_search_button_clicked));
	glade_xml_signal_connect (xml, "on_button-gestionar-busqueda-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_manage_button_clicked));
	glade_xml_signal_connect (xml, "on_button-limpiar-busqueda-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_clean_button_clicked));
	glade_xml_signal_connect (xml, "on_button-modificar-busqueda-pedidos_clicked",
				  GTK_SIGNAL_FUNC (on_modify_button_clicked));
	glade_xml_signal_connect (xml, "on_clist-lista-pedidos-busqueda-pedidos_select_row",
				  GTK_SIGNAL_FUNC (on_order_list_select_row));
	glade_xml_signal_connect (xml, "on_clist-lista-pedidos-busqueda-pedidos_unselect_row",
				  GTK_SIGNAL_FUNC (on_order_list_unselect_row));
	glade_xml_signal_connect (xml, "on_clist_busqueda_pedidos_button_press_event",
				  GTK_SIGNAL_FUNC (on_order_list_search_button_press_event));

	clean_screen();
	e_name = u_parameter_manager_get_parameter ("empresa");
	if (e_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-filtro-empresa-busqueda-pedidos")), e_name);
	}

	w_name = u_parameter_manager_get_parameter ("almacen");
	if (w_name!=NULL) {
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-filtro-almacen-busqueda-pedidos")), w_name);
	}

	g_free(e_name);
        g_free(w_name);

	order =dialog;
    }

    if (supplier_param!=NULL) {
	load_supplier_order(supplier_param);
    }

    g_free(directory);

    return order;

}

static void load_supplier_order(Participant *participant) {

    if (participant != NULL) {
	gchar *nick = d_participant_get_nick (participant);
	if (supplier_loaded != NULL){
	    d_participant_clear (supplier_loaded);
	    supplier_loaded = NULL;
	}

	if (nick != NULL) {
	    GtkWidget *nick_entry =glade_xml_get_widget (xml, "entry-filtros-proveedor-busqueda-pedidos");
	    gtk_entry_set_text (GTK_ENTRY(nick_entry),g_strdup(nick));
	    g_free(nick);
	}
	supplier_loaded = participant;
    }
}

void c_order_search(void) {
    GtkWidget * orders_list;
    GtkWidget * lines_list;
    GtkWidget * date_dateedit;
    gulong  order_array_len = 0;
    gulong  code;
    ConditionFindOrder condition;


    while (gtk_events_pending())
	gtk_main_iteration();


    if (order!=NULL) {
    
    	orders_list   = glade_xml_get_widget(xml,"clist-lista-pedidos-busqueda-pedidos");
    	lines_list    = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");
    	date_dateedit = glade_xml_get_widget(xml,"dateedit-filtros-fecha-busqueda-pedidos");

	//We don't use this line yet.
	condition.order_code = ULONG_MAX;

	condition.placed = FALSE;
	code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo-filtro-empresa-busqueda-pedidos"));
	condition.enterprise_code = code;

	code = c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo-filtro-almacen-busqueda-pedidos"));
	condition.warehouse_code = code;

	if (supplier_loaded!=NULL) {
	    condition.supplier_code = d_participant_get_code (supplier_loaded);
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry-filtros-proveedor-busqueda-pedidos")),"");
	    d_participant_clear (supplier_loaded);
	} else {
	    condition.supplier_code = ULONG_MAX;
	}
	supplier_loaded = NULL;

	//    if (supplier_loaded!=NULL) {
	//	condition.supplier_code = d_participant_get_code (supplier_loaded);
	//    } else {
	//    }
	/*ATA AQUI*/

	condition.date = gnome_date_edit_get_date(GNOME_DATE_EDIT(date_dateedit));

    	m_order_clear_array (order_array);
    	order_array=NULL;

	order_array = m_order_find_by_condition (&condition, &order_array_len);

	if ((order_array!=NULL) && (order_array_len>0)) {
	    fill_order_list(GTK_CLIST(orders_list), order_array_len);
	} else {
	    c_util_open_info_window (_("There is not any order"));
	}


	if ((order_array!=NULL) && (order_array_len>0)) {
		fill_order_list(GTK_CLIST(orders_list), order_array_len);
    	} else {
		c_util_open_info_window (_("There is not any order"));
    	}
    }

}



static void on_search_button_clicked(GtkButton *button,
						      gpointer data) {
    GdkCursor * cursor;
    GtkWidget * list_pedidos;
    GtkWidget * window;

    list_pedidos   = glade_xml_get_widget(xml,"clist-lista-pedidos-busqueda-pedidos");

    window = lookup_widget(GTK_WIDGET(list_pedidos),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    c_order_search();

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
}

static void fill_order_list(GtkCList * list, gulong size) {

    gint i=0;
    gchar * temp;
    time_t date;

    gchar *empty_row[]={"","","","",""};

    gtk_clist_clear ((GtkCList *) list);

    while (order_array[i]!=NULL) {
	size_t stored;
	gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);
 
	if (i==30) gtk_clist_freeze ((GtkCList *) list);

	gtk_clist_append ((GtkCList *) list,empty_row);

	temp = g_strdup_printf("%lu",d_order_get_code (order_array[i]));
	gtk_clist_set_text((GtkCList *) list, i, C_ORDER_COLUMN, temp);
	g_free(temp);


	date =  d_order_get_date (order_array[i]);
	stored=strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));
	gtk_clist_set_text((GtkCList *) list, i, C_DATE_COLUMN, timer_char);

	temp = m_warehouse_get_name_from_code (d_order_get_warehouse_code (order_array[i]));
	gtk_clist_set_text((GtkCList *) list, i, C_WAREHOUSE_COLUMN, temp);
	g_free(temp);

	temp = m_enterprise_get_name_from_code (d_order_get_enterprise_code (order_array[i]));
	gtk_clist_set_text((GtkCList *) list, i, C_ENTERPRISE_COLUMN, temp);
	g_free(temp);

	temp = m_supplier_get_name_from_code (d_order_get_supplier_code (order_array[i]));

	gtk_clist_set_text((GtkCList *) list, i, C_SUPPLIER_COLUMN, temp);
	g_free(temp);
        g_free(timer_char);

        i++;
    }
    gtk_clist_thaw ((GtkCList *) list);

}


static void on_manage_button_clicked(GtkButton *button,
							 gpointer data) {
    GtkWidget * lines_list = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");
    GtkWidget * orders_list = glade_xml_get_widget(xml,"clist-lista-pedidos-busqueda-pedidos");
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button),"app1");
    GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");

    GArray * selected_order = c_util_get_selected_rows_from_clist (GTK_CLIST(orders_list));
    GArray * selected = c_util_get_selected_rows_from_clist (GTK_CLIST(lines_list));
    GList * order_line_list = NULL;
    gboolean associated = FALSE;
    if (selected_order->len>0) {
	gint line = 0;
	gint index_order = g_array_index(selected_order,gint,0);
	OrderLine ** order_lines = d_order_get_order_lines (order_array[index_order]);

	if (selected->len>0){
	    while (line != (selected->len)) {
		gint nindex = g_array_index(selected,gint,line);
		if (d_order_line_get_charge_invoice_number(order_lines[nindex])==ULONG_MAX) {
		    order_line_list = g_list_append(order_line_list,(gpointer) order_lines[nindex]);
		} else {
		    associated = TRUE;
		}
		line++;
	    }
	}
	if (associated == FALSE) {
	    c_charge_invoice_open (app1,vbox);
	    if (selected->len>0){
		c_charge_invoice_load_order_lines (order_line_list);
	    }
	    i_shortcut_change_dialog (GTK_WIDGET(button),1,3);
	} else {
	    c_util_open_info_window(_("You can't link an order line to more than one invoices"));
	}

	g_list_free(order_line_list);
    } else {
	c_charge_invoice_open (app1,vbox);
	i_shortcut_change_dialog (GTK_WIDGET(button),1,3);
    }

}

static void on_clean_button_clicked(GtkButton *button,
						       gpointer data) {
    clean_screen();
}

static void on_modify_button_clicked(GtkButton *button,
							 gpointer data) {
    GtkWidget * list = glade_xml_get_widget(xml,"clist-lista-pedidos-busqueda-pedidos");
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(list),"app1");
    GtkWidget * vbox = lookup_widget(GTK_WIDGET(button), "main_container");

    GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));

    gint lines = selected->len;
    Order *order = NULL;

    if (lines==1) {

	if (order_array!=NULL) {
	    if(selected_row>=0) {
		if(order_array[selected_row]!= NULL) {
		    order = order_array[selected_row];
		}
	    }
	}

	c_order_open_window(app1, vbox, supplier_loaded);
	c_order_load(order);
	clean_screen();

	i_shortcut_change_dialog (GTK_WIDGET(button), 2, 3);
    } else {
	if (lines>1) {
	    c_util_open_error_window (_("Select just one order"));
	} else {
	    if (lines<1) {
		c_util_open_error_window (_("Select at least one order"));
	    }
	}
    }

}

static void on_filter_supplier_button_clicked(GtkButton *button,
								       gpointer data) {
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget * vbox;

    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry-filtros-proveedor-busqueda-pedidos")), "");
    gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget(xml, "entry-filtros-proveedor-busqueda-pedidos")), 0);

    vbox = lookup_widget(GTK_WIDGET(button), "main_container");
    c_clients_control_open_supplier (app1, vbox, c_find_order_open_window,2,4);

    i_shortcut_change_dialog (GTK_WIDGET(button),0,0);

}

static void on_filter_enterprise_combo_entry_changed() {
    fill_warehouse_combo();
}


static void on_order_list_select_row(GtkCList *list,
							       gint      row,
							       gint      column,
							       GdkEvent *event,
							       gpointer  user_data) {

    if (event!=NULL) {
	if (event->type != GDK_2BUTTON_PRESS) {

	    GtkWidget * lines_list = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");
	    if (order_array!=NULL) {
		if(order_array[row]!= NULL) {
		    Supplier * supplier;
		    selected_row = row;

		    fill_lines_list(GTK_CLIST(lines_list), order_array[row]);
		    if (supplier_loaded!=NULL)
			d_participant_clear(supplier_loaded);
		    supplier = m_supplier_find_by_code (d_order_get_supplier_code (order_array[row]));

		    supplier_loaded =
			d_participant_duplicate(d_supplier_get_participant (supplier));
		    d_supplier_clear(supplier);
		    //                g_free(supplier);

		}
	    }

	}
    }
}

static void on_order_list_unselect_row(GtkCList *list,
							       gint      row,
							       gint      column,
							       GdkEvent *event,
							       gpointer  user_data) {

    GtkWidget * lines_list = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");
    GtkWidget * orders_list = glade_xml_get_widget(xml,"clist-lista-pedidos-busqueda-pedidos");
    GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(orders_list));

    gint lines = selected->len;

    d_participant_clear (supplier_loaded);
    supplier_loaded = NULL;

    if (lines>0) {
	Supplier * supplier;

	if (supplier_loaded!=NULL)
	    d_participant_clear(supplier_loaded);

	selected_row = g_array_index(selected,gint,0);
	supplier = m_supplier_find_by_code(d_order_get_supplier_code(order_array[row]));
	supplier_loaded =
	    d_participant_duplicate(d_supplier_get_participant (supplier));
	d_supplier_clear(supplier);

    } else {
	selected_row = -1;
    }

    gtk_clist_clear ((GtkCList *) lines_list);
}

static void fill_lines_list(GtkCList * list, Order *order) {

    gchar * temp;
    gint   index=0;
    OrderLine ** lines = d_order_get_order_lines (order);

    gchar *empty_row[]={"","","","","","",""};

    gtk_clist_clear ((GtkCList *) list);

    if (lines!=NULL) {
	while (lines[index]!=NULL) {
	    gtk_clist_append ((GtkCList *) list,empty_row);

	    temp = d_piece_get_euroCode (d_order_line_get_piece (lines[index]));
	    gtk_clist_set_text((GtkCList *) list, index, C_EUROCODE_COLUMN, temp);
	    g_free(temp);

	    temp = d_product_get_description ((Product *)d_order_line_get_piece (lines[index]));
	    gtk_clist_set_text((GtkCList *) list, index, C_DESCRIPTION_COLUMN, temp);
	    g_free(temp);

	    temp = d_product_get_family_code ((Product *)d_order_line_get_piece (lines[index]));
	    gtk_clist_set_text((GtkCList *) list, index, C_FAMILY_COLUMN, temp);
	    g_free(temp);

	    if (d_order_line_get_quantity (lines[index])!=G_MINDOUBLE)
		temp = g_strdup_printf("%.3f", d_order_line_get_quantity (lines[index]));
	    else
                temp = g_strdup("");
	    gtk_clist_set_text((GtkCList *) list, index, C_QUANTITY_COLUMN, temp);
	    g_free(temp);


	    temp = d_order_line_get_delivery_payment (lines[index]);
	    if (temp!=NULL) {
		gtk_clist_set_text((GtkCList *) list, index, C_DELIVERY_PAYMENT_COLUMN, temp);
		g_free(temp);
	    }

	    if (d_order_line_get_charge_invoice_number (lines[index])!=ULONG_MAX) {
		temp = g_strdup_printf("%lu",d_order_line_get_charge_invoice_number (lines[index]));
		if (temp!=NULL) {
		    gtk_clist_set_text((GtkCList *) list, index, C_INVOICE_NUMBER_COLUMN, temp);
		    g_free(temp);
		}
	    }

	    if (d_order_line_get_charge_line_number (lines[index])!=ULONG_MAX) {
		temp = g_strdup_printf("%lu", d_order_line_get_charge_line_number (lines[index]));
		gtk_clist_set_text((GtkCList *) list, index, C_LINE_NUMBER_COLUMN, temp);
		g_free(temp);
	    }


	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) list);

}


static void fill_warehouse_combo() {
    GtkWidget * warehouseCombo;
    GtkWidget * enterpriseCombo;
    Warehouse ** warehouses;
    gint code_combo;
    gchar * temp;
    enterpriseCombo = glade_xml_get_widget (xml, "combo-filtro-empresa-busqueda-pedidos");
    warehouseCombo = glade_xml_get_widget (xml, "combo-filtro-almacen-busqueda-pedidos");
    code_combo = c_util_get_code_from_combo (enterpriseCombo);


    warehouses = m_warehouse_find_by_enterprise_code (code_combo);

    clean_warehouse_combo();
    temp = g_strdup(_("All the warehouses"));
    c_util_set_code_and_name_on_combo (C_ALL_WAREHOUSES_OF_ENTERPRISE,temp,GTK_COMBO(warehouseCombo));
    g_free(temp);
    c_util_fill_combo_with_warehouses(warehouseCombo,warehouses);

    m_warehouse_clear_array (warehouses);
    warehouses = NULL;

}

static void fill_enterprise_combo() {
    GtkWidget * combo;
    Enterprise ** enterprises;

    combo = glade_xml_get_widget (xml, "combo-filtro-empresa-busqueda-pedidos");

    c_util_set_code_and_name_on_combo (C_ALL_ENTERPRISES,g_strdup(_("All the enterprises")),GTK_COMBO(combo));
    enterprises = m_enterprise_find_all ();

    c_util_fill_combo_with_enterprises (combo,enterprises);
}

static void clean_warehouse_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-filtro-almacen-busqueda-pedidos");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-filtro-almacen-busqueda-pedidos");
    c_util_clean_code_and_name_combo (combo);
}

static void clean_enterprise_combo() {
    GtkWidget * combo;
    GtkWidget * entry;

    entry = glade_xml_get_widget (xml, "combo-entry-filtro-empresa-busqueda-pedidos");
    gtk_entry_set_text(GTK_ENTRY(entry),"");

    combo = glade_xml_get_widget (xml, "combo-filtro-empresa-busqueda-pedidos");
    c_util_clean_code_and_name_combo (combo);

}

static void clean_combos() {
    gchar *e_name = NULL;
    gchar *w_name = NULL;
    clean_enterprise_combo();
    clean_warehouse_combo();
    fill_enterprise_combo();

    e_name = u_parameter_manager_get_parameter ("empresa");
    if (e_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-filtro-empresa-busqueda-pedidos")), e_name);
    }

    w_name = u_parameter_manager_get_parameter ("almacen");
    if (w_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-filtro-almacen-busqueda-pedidos")), w_name);
    }
}

static void clean_screen() {
    GtkWidget *vbox;
    GtkWidget * orders_list;
    GtkWidget * lines_list;

    orders_list = glade_xml_get_widget(xml,"clist-lista-pedidos-busqueda-pedidos");
    lines_list = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    if (order_array!=NULL) {
	gtk_clist_clear ((GtkCList *) orders_list);
	gtk_clist_clear ((GtkCList *) lines_list);
	m_order_clear_array (order_array);
	order_array=NULL;
    }

    if (supplier_loaded != NULL){
	d_participant_clear (supplier_loaded);
	supplier_loaded = NULL;
    }


    clean_combos();
}

static gint on_order_list_search_button_press_event(GtkWidget *widget,
							 GdkEventButton *event, gpointer user_data) {


    GtkWidget * menu_item3 = gtk_menu_item_new_with_label(_("Select all the lines"));
    GtkWidget * menu_item4 = gtk_menu_item_new_with_label(_("Unselect all the lines"));
    GtkWidget * menu_item1 = gtk_menu_item_new_with_label(_("Link delivery payment"));
//    GtkWidget * menu_item2 = gtk_menu_item_new_with_label("Asociar factura cargo");
    GtkWidget * menu = gtk_menu_new();
    GdkEventButton *event_button;

    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item1));
//    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item2));
    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item3));
    gtk_menu_item_activate(GTK_MENU_ITEM(menu_item4));
    gtk_menu_set_title(GTK_MENU(menu),_("Select the command"));
    gtk_menu_prepend(GTK_MENU(menu),menu_item4);
    gtk_menu_prepend(GTK_MENU(menu),menu_item3);
    gtk_menu_prepend(GTK_MENU(menu),menu_item1);

    gtk_widget_show(menu_item1);
    gtk_widget_show(menu_item3);
    gtk_widget_show(menu_item4);

    gtk_signal_connect_object(GTK_OBJECT(menu_item1), "activate",
       		       GTK_SIGNAL_FUNC (on_payment_note_activate), NULL);
    gtk_signal_connect_object(GTK_OBJECT(menu_item3), "activate",
       		       GTK_SIGNAL_FUNC (on_select_activate), NULL);
    gtk_signal_connect_object(GTK_OBJECT(menu_item4), "activate",
       		       GTK_SIGNAL_FUNC (on_unselect_activate), NULL);
    if (event->type == GDK_BUTTON_PRESS)
    {
	event_button = (GdkEventButton *) event;
	if (event_button->button == 3)
	{

	    gtk_menu_popup (GTK_MENU(menu), NULL, NULL, NULL, NULL,
			    event_button->button, event_button->time);
	    return TRUE;
	}
    }

  return FALSE;
}


static void on_payment_note_activate(GtkMenuItem *menuitem,
				gpointer user_data) {

    GtkWidget * clist = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(clist), "app1");
    GArray * selected = c_util_get_selected_rows_from_clist (GTK_CLIST(clist));

    if (selected->len>0){
	gchar * temp = g_strdup(_("Delivery payment number"));
	c_update_entry_open_window(app1,temp,C_GCHAR_TYPE,set_delivery_note_number);
        c_update_entry_set_max_length(40);

	g_free(temp);
    } else {
        c_util_open_info_window (_("You must select at least one order line"));
    }

}
static void on_select_activate(GtkMenuItem *menuitem,
				   gpointer user_data) {
    GtkWidget * clist = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");

    gtk_clist_select_all(GTK_CLIST(clist));
}
static void on_unselect_activate(GtkMenuItem *menuitem,
				   gpointer user_data) {
    GtkWidget * clist = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");

    gtk_clist_unselect_all(GTK_CLIST(clist));
}

void set_delivery_note_number(gpointer payment_note_number_value) {
    gchar * payment_note_number = (gchar *) payment_note_number_value;
    GtkWidget * lines_list = glade_xml_get_widget(xml,"clist-lista-lineas-pedido-busqueda-pedidos");
    GtkWidget * orders_list = glade_xml_get_widget(xml,"clist-lista-pedidos-busqueda-pedidos");
    GArray * selected_order = c_util_get_selected_rows_from_clist (GTK_CLIST(orders_list));
    GArray * selected = c_util_get_selected_rows_from_clist (GTK_CLIST(lines_list));
    GList * order_line_list = NULL;
    gint line = 0;
    gint index_order = g_array_index(selected_order,gint,0);
    gulong supplier_code = d_order_get_supplier_code (order_array[index_order]);
    OrderLine ** order_lines = d_order_get_order_lines (order_array[index_order]);

    if (selected->len>0){
	while (line != (selected->len)) {
	    gint nindex = g_array_index(selected,gint,line);
            d_order_line_set_delivery_payment (order_lines[nindex],payment_note_number);
	    order_line_list = g_list_append(order_line_list,(gpointer) order_lines[nindex]);
            line++;
	}
	m_order_line_update_lines (order_line_list,supplier_code);

    }

    g_free(payment_note_number);
    g_list_free(order_line_list);
    fill_lines_list((GtkCList *) lines_list,order_array[index_order]);

}


