/* find_product_cost.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "parameter_manager.h"
#include "support.h"
#include "control_util.h"
#include "supplier_price_model.h"
#include "product.h"
#include "piece.h"
#include "piece_model.h"
#include "find_stocked_pieces_control.h"
#include "find_product_cost.h"
#include "supplier_price_control.h"
#include <../interface/printing/listing-print-wrapper.h>

#define C_SUPPLIER_LEGAL_ID_COLUMN 0
#define C_PROVIDER_NAME_COLUMN 1
#define C_COST_PRICE_COLUMN 2

static GtkWidget *open_product_price=NULL;
static GladeXML *xml;
static Piece * general_piece;
static SupplierPrice ** supplierPrices;

static gchar widget_name[]="vbox_precios";
static gchar url[]="/src/interface/glade/product_view.glade";


static void on_search_prices_button_clicked(GtkButton *button, gpointer user_data);
static void on_new_price_button_clicked(GtkButton *button, gpointer user_data);
static void on_set_prices_button_clicked(GtkButton *button, gpointer user_data);
static void on_clean_prices_button_clicked(GtkButton *button, gpointer user_data);
static void on_modify_price_button_clicked(GtkButton *button, gpointer user_data);
static void on_search_product_button_clicked(GtkButton *button,gpointer user_data);
static void set_piece_data_on_screen(Piece *piece);
static void clean_screen(void);
static void do_prices_search(void);
static void fill_prices_list(GtkCList * clist,SupplierPrice ** suplier_prices);
static void insert_prices(gboolean modify);
static void repaint_prices(void);
static void assign_prices(void);

GtkWidget *c_find_product_cost_open_window(GtkWidget *app1, GtkWidget *vbox) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (open_product_price==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);

	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_button_buscar_precios_clicked",
				  GTK_SIGNAL_FUNC (on_search_prices_button_clicked));
	glade_xml_signal_connect (xml, "on_button_asignar_precios_clicked",
				  GTK_SIGNAL_FUNC (on_set_prices_button_clicked));
	glade_xml_signal_connect (xml, "on_button_limpiar_precios_clicked",
				  GTK_SIGNAL_FUNC (on_clean_prices_button_clicked));
	glade_xml_signal_connect (xml, "on_button_modificar_precios_clicked",
				  GTK_SIGNAL_FUNC (on_modify_price_button_clicked));
	glade_xml_signal_connect (xml, "on_button_buscar_producto_clicked",
				  GTK_SIGNAL_FUNC (on_search_product_button_clicked));
	glade_xml_signal_connect (xml, "on_button_nuevo_precios_clicked",
			          GTK_SIGNAL_FUNC (on_new_price_button_clicked));

	open_product_price=dialog;
    }
    g_free(directory);
    return open_product_price;
}



static void on_search_prices_button_clicked(GtkButton *button,gpointer user_data) {

    if (general_piece!=NULL) {
	do_prices_search();
    } else {
        c_util_open_info_window (_("You must select a piece"));
    }
}

static void do_prices_search() {
    Product product = d_piece_get_product (general_piece);
    GtkWidget * list;
    GtkWidget * window;
    GdkCursor * cursor;

    list = glade_xml_get_widget(xml,"clist_precios");

    window = lookup_widget(GTK_WIDGET(list),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    if (supplierPrices!=NULL) {
	m_supplier_price_clear_array (supplierPrices);
        supplierPrices=NULL;
    }
    supplierPrices = model_find_prices_by_product(d_product_get_product_code (&product));

    if (supplierPrices!=NULL) {
	fill_prices_list(GTK_CLIST(list),supplierPrices);
    } else {
        c_util_open_info_window (_("There is no prices"));
    }
    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

}

static void on_set_prices_button_clicked(GtkButton *button,gpointer user_data) {

    assign_prices();

}

static void assign_prices() {
    GArray * selected;
    gint value;
    gboolean result=FALSE;
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "clist_precios"));
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(list),"app1");


    selected = c_util_get_selected_rows_from_clist (list);
    if (selected->len==1) {
	    value = g_array_index(selected,gint,0);
            result = m_supplier_price_assign (supplierPrices[value]);
	    g_array_free(selected,TRUE);
	    if (result == FALSE) {
		c_util_open_error_window (_("Data has not been updated correctly"));
	    } else {
		c_util_open_info_window (_("Data has been updated correctly"));
	    }
    } else if (selected->len==0) {
	c_util_open_error_window (_("Select a cost price"));
    } else {
	c_util_open_error_window (_("Select just one cost price"));
    }


}


static void on_clean_prices_button_clicked(GtkButton *button,gpointer user_data) {
    clean_screen();

}
static void on_modify_price_button_clicked(GtkButton *button,gpointer user_data) {

    insert_prices(TRUE);

}

static void on_new_price_button_clicked(GtkButton *button, gpointer user_data) {

    insert_prices(FALSE);
}


static void repaint_prices() {
    do_prices_search();
}
static void insert_prices(gboolean modify) {
    GArray * selected;
    gint value;
    GtkCList * list = GTK_CLIST(glade_xml_get_widget (xml, "clist_precios"));
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(list),"app1");


    if (modify !=FALSE) {
	selected = c_util_get_selected_rows_from_clist (list);
	if (selected->len==1) {
	    GtkWidget * insert_price =  c_supplier_price_open_window(app1,repaint_prices);
	    gtk_window_set_position(GTK_WINDOW(insert_price),GTK_WIN_POS_CENTER);
	    value = g_array_index(selected,gint,0);
	    c_supplier_price_load_data(supplierPrices[value],modify);

	    gtk_widget_show(insert_price);
	    g_array_free(selected,TRUE);
	} else if (selected->len==0) {

	} else {
	    c_util_open_error_window (_("Select just one cost price"));
	}
    } else {
	SupplierPrice * supplier_price = d_supplier_price_create_empty ();
        Piece * piece;
	GtkWidget * insert_price =  c_supplier_price_open_window(app1,repaint_prices);

	gtk_window_set_position(GTK_WINDOW(insert_price),GTK_WIN_POS_CENTER);

	piece = d_piece_duplicate (general_piece);

	d_supplier_price_set_piece (supplier_price,piece);
	c_supplier_price_load_data(supplier_price,modify);

	gtk_widget_show(insert_price);
    }



}
static void on_search_product_button_clicked(GtkButton *button,gpointer user_data) {
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");

    gchar * w_name = u_parameter_manager_get_parameter ("almacen");
    gchar * enterprise = u_parameter_manager_get_parameter ("empresa");


#ifndef IS_SHOP
    i_shortcut_change_dialog (GTK_WIDGET(vbox),2,1);
#else
    i_shortcut_change_dialog (GTK_WIDGET(vbox),2,0);
#endif

    c_find_stocked_pieces_open_window(app1, vbox);
    c_find_stocked_pieces_set_combos(enterprise, w_name);

    g_free(w_name);
    g_free(enterprise);

}
/*
 static void on_button_imprimir_precios_clicked(GtkButton *button,gpointer user_data) {
    if (listing!=NULL) {
	p_listing_wrap_print (listing);
    } else {
	c_util_open_info_window ("Es necesario realizar la bsqueda previamente");
    }
    }
    */


void c_load_piece(Piece * piece) {

    clean_screen();
/*    if (supplierPrices!=NULL) {
	m_supplier_price_clear_array (supplierPrices);
        supplierPrices=NULL;
    }

    if (general_piece!=NULL) {
	d_piece_clear (general_piece);
        general_piece = NULL;
    }
*/
    set_piece_data_on_screen(piece);

    general_piece = d_piece_duplicate (piece);
}

static void set_piece_data_on_screen(Piece *piece) {
    GtkWidget * entry = glade_xml_get_widget(xml,"entry_eurocode_precios");
    GtkWidget * text = glade_xml_get_widget(xml,"text_descripcion_precios");
    Product product = d_piece_get_product (piece);
    gchar * eurocode_temp=d_piece_get_euroCode (piece);
    gchar * desc_temp=d_product_get_description (&product);
    gint * position = (gint *) g_malloc0(sizeof(gint));
    gtk_entry_set_text(GTK_ENTRY(entry),eurocode_temp);

    position[0] = gtk_text_get_length(GTK_TEXT(text));
    gtk_editable_delete_text(GTK_EDITABLE(text),0,position[0]);
    position[0] = 0;
    if (desc_temp!=NULL) {
        g_strstrip(desc_temp);
	gtk_editable_insert_text(GTK_EDITABLE(text),desc_temp,strlen(desc_temp),position);
    }
    g_free(eurocode_temp);
    g_free(desc_temp);
}

static void clean_screen() {

    GtkWidget *vbox;
    GtkWidget *list;

    list = glade_xml_get_widget(xml,"clist_precios");
    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);
    gtk_clist_clear ((GtkCList *) list);

    if (general_piece!=NULL) {
	d_piece_clear (general_piece);
        general_piece = NULL;
    }
    if (supplierPrices!=NULL) {
	m_supplier_price_clear_array (supplierPrices);
        supplierPrices=NULL;
    }

}


/*static void on_search_prices_button_clicked(GtkButton *button, gpointer user_data) {
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    ConditionFindHistory * condition = create_condition_from_screen();
    PieceHistory ** piece_history = m_piece_history_find_by_condition (condition,tuples);
    GtkWidget * clist = glade_xml_get_widget(xml,"clist_precios");
    if (piece_history!=NULL) {
	Listing list;
	gulong    rows = 0;
	gchar * warehouse = NULL;
	gchar * enterprise = NULL;

	if (listing!=NULL) {
	    p_listing_clear (listing);
            listing = NULL;
	}
	listing = (Listing *) g_malloc0(sizeof(Listing));
	warehouse = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_almacen_precios"))));
	enterprise = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_pieza_precios"))));

	list.content_title = d_piece_history_get_content_title ();
	list.listing_title = d_piece_history_get_listing_title (enterprise,warehouse);
	list.content       = d_piece_history_array_to_content (piece_history,&rows);
	list.width_of_cols = d_piece_history_get_listing_columns_width ();
	list.alignment_of_cols = d_piece_history_get_listing_columns_alignment ();
	list.n_rows        = rows;
	list.n_columns     = d_piece_history_get_content_size ();
	list.orientation   = 4;
	list.type_of_paper = 2;


	*listing = list;

	fill_clist_with_listing(GTK_CLIST(clist),listing->content);
        m_piece_history_clear_array (piece_history);
	if (warehouse !=NULL) g_free(warehouse);
	if (enterprise !=NULL) g_free(enterprise);
    } else {
        c_util_open_info_window ("No se han encontrado resultados");
    }

    g_free(tuples);
}
*/
static void fill_prices_list(GtkCList * clist,SupplierPrice ** supplier_prices) {
    gint index=0;
    gchar *empty_row[]={"","",""};
    gint assigned_index=-1;
    gulong assigned_provider =  ULONG_MAX;


    index=0;
    if (supplier_prices!=NULL) {
	Product product = d_piece_get_product (general_piece);
        assigned_provider = m_piece_find_supplier_assigned (d_product_get_product_code (&product));
	while (supplier_prices[index]!=NULL) {
	    gchar * temp_char;
            gdouble price;
	    if (index==30) gtk_clist_freeze ((GtkCList *) clist);
	    gtk_clist_append ((GtkCList *) clist,empty_row);

	    temp_char = d_participant_get_legal_id (d_supplier_price_get_participant (supplier_prices[index]));
	    if (temp_char!=NULL) {
		gtk_clist_set_text((GtkCList *) clist, index, C_SUPPLIER_LEGAL_ID_COLUMN, temp_char);
                g_free(temp_char);
	    }
	    temp_char = d_participant_get_nick (d_supplier_price_get_participant (supplier_prices[index]));
	    if (temp_char!=NULL) {
		gtk_clist_set_text((GtkCList *) clist, index, C_PROVIDER_NAME_COLUMN, temp_char);
                g_free(temp_char);
	    }
            price = d_supplier_price_get_cost_price (supplier_prices[index]);
	    if (price!=G_MINDOUBLE) {
		temp_char = g_strdup_printf("%.2f",price);
		gtk_clist_set_text((GtkCList *) clist, index, C_COST_PRICE_COLUMN, temp_char);
                g_free(temp_char);
	    }
	    if (assigned_provider == d_participant_get_code (d_supplier_price_get_participant (supplier_prices[index]))) {
                assigned_index=index;
	    }

	    index++;

	}
	if (supplier_prices[0]==NULL) {
	    c_util_open_info_window (_("There is not any supplier for this piece"));
	}
    }
    if (assigned_index>=0)
	gtk_clist_select_row(clist,assigned_index,0);


}

