
/* find_repairs_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <time.h>

#include "repair_object_model.h"
#include "repair_line_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "find_repairs_control.h"
#include "repairs_control.h"
#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "repair_object.h"
#include "parameter_manager.h"

#define C_LICENSE_NUMBER_COLUMN 0
#define C_LEGAL_ID_COLUMN       1
#define C_NAME_COLUMN    2
#define C_COMPANY_COLUMN  3
#define C_IN_COLUMN   4
#define C_GARAGE_COLUMN    5


static void on_search_button_clicked(GtkButton *button, gpointer data);
static void on_delete_button_clicked(GtkButton *button, gpointer data);
static void on_clean_button_clicked(GtkButton *button, gpointer data);
static void on_detail_button_clicked(GtkButton *button, gpointer data);
static void on_press_key(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void on_shop_entry_changed(GtkEntry *entry, gpointer user_data);
static void fill_enterprise_from_combo(void);
static void fill_shop_combo(void);
static void fill_repairs_list(void);
static void find_repairs_list(void);
static void find_repairs_list_selected(GArray *selected);
static void clean_screen(void);
static void free_objects(void);
static Repair_filter * obtain_data_to_filter(void);
static void on_ok_delete_repairs(gint reply,gpointer user_data);


static gchar dialog_file[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="find_repairs_vbox";
static GtkWidget *find_repairs=NULL;
static GladeXML *xml = NULL;	
static Repair **repairs = NULL;
static gulong enterprise_code = ULONG_MAX;
static gulong shop_code = ULONG_MAX;

GtkWidget *c_find_repair_open_window(GtkWidget *app1, GtkWidget *vbox)
{
    gchar * directory = g_strconcat(AA_DATA_DIR,dialog_file,NULL);
    gchar *s_name = NULL;
   
  if (find_repairs==NULL)
    {
      GtkWidget *dialog;
      time_t ini = time(NULL) - 86400*7;
      time_t fin = time(NULL) + 86400*7;
      GtkWidget * init_dateedit = NULL;
      GtkWidget * end_dateedit = NULL;
#ifdef IS_SHOP
      GtkWidget * shop_combo= NULL;
/*       GtkWidget * button_guardar = glade_xml_get_widget(xml,"button_guardar_facturas");  */
#endif

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);

#ifdef IS_SHOP
      shop_combo = glade_xml_get_widget (xml, "tienda_combo_listado_reparaciones");
      
      gtk_widget_set_sensitive(shop_combo,FALSE);
#endif

      init_dateedit = glade_xml_get_widget(xml,"dateedit_ini_listado");
      end_dateedit = glade_xml_get_widget(xml,"dateedit2_listado");

      gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);  

      glade_xml_signal_connect (xml, "on_press_key",
             			GTK_SIGNAL_FUNC (on_press_key));     
      glade_xml_signal_connect (xml, "on_limpiar_button_clicked",
				GTK_SIGNAL_FUNC (on_clean_button_clicked));
      glade_xml_signal_connect (xml, "on_detalle_button_clicked",
				GTK_SIGNAL_FUNC (on_detail_button_clicked));
      glade_xml_signal_connect (xml, "on_buscar_button_clicked",
				GTK_SIGNAL_FUNC (on_search_button_clicked));
      glade_xml_signal_connect (xml, "on_borrar_button_clicked",
				GTK_SIGNAL_FUNC (on_delete_button_clicked));
      glade_xml_signal_connect (xml, "on_tienda_entry_changed",
             			GTK_SIGNAL_FUNC (on_shop_entry_changed));     


      gnome_date_edit_set_time(GNOME_DATE_EDIT(init_dateedit), ini),
      gnome_date_edit_set_time(GNOME_DATE_EDIT(end_dateedit), fin),

      fill_shop_combo();

      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	  gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "tienda_entry_listado")), s_name);
      }
      g_free(s_name);

      find_repairs=dialog;
      
    }

  g_free(directory);
  
  return find_repairs;
  
}

static void on_shop_entry_changed(GtkEntry *entry, gpointer user_data){
    fill_enterprise_from_combo();
}


static void fill_enterprise_from_combo() {
    GtkWidget * entry;
    GtkWidget * combo;
    gchar * aux=NULL;
    gulong codeCombo;
    Enterprise * enterprise = NULL;

    entry = glade_xml_get_widget (xml, "empresa_entry_listado");

    combo = glade_xml_get_widget (xml, "tienda_combo_listado_reparaciones");

    codeCombo = c_util_get_code_from_combo (combo);

    shop_code = codeCombo;

    enterprise = m_enterprise_find_by_shop (codeCombo);

    if (enterprise!=NULL) {
      aux = d_participant_get_name (d_enterprise_get_participant (enterprise));
      gtk_entry_set_text(GTK_ENTRY(entry),aux);
      
      enterprise_code = d_participant_get_code (d_enterprise_get_participant (enterprise));

      d_enterprise_clear (enterprise);
      enterprise=NULL;
      g_free(aux);
    }
}

static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;
                                        
    combo = glade_xml_get_widget (xml, "tienda_combo_listado_reparaciones");

    shops = m_shop_find_all ();

    c_util_fill_combo_with_shops (combo,shops);

}
static Repair_filter * obtain_data_to_filter(){
    GtkWidget * entry_license_number = glade_xml_get_widget(xml,"matricula_listado");
    GtkWidget * entry_legal_id = glade_xml_get_widget(xml,"dni_listado");
    GtkWidget * entry_name = glade_xml_get_widget(xml,"nombre_cliente_listado");
    GtkWidget * entry_agente = glade_xml_get_widget(xml,"nombre_agente_listado");
    GtkWidget * entry_company = glade_xml_get_widget(xml,"razon_social_compania_listado");
    GtkWidget * entry_garage = glade_xml_get_widget(xml,"razon_social_taller_listado");
    GtkWidget * dateedit_inicio = glade_xml_get_widget(xml,"dateedit_ini_listado");
    GtkWidget * dateedit_fin = glade_xml_get_widget(xml,"dateedit2_listado");
    GtkWidget * combo_tienda = glade_xml_get_widget(xml,"tienda_combo_listado_reparaciones");
    gchar * license_number = NULL;
    gchar * legal_id = NULL;
    gchar * name = NULL;
    gchar * agente = NULL;
    gchar * company = NULL;
    gchar * garage = NULL;
    gchar * inicio = NULL;
    gchar * fin = NULL;
    gchar * tienda = NULL;
    gchar * empresa = NULL;
    gulong shop_code = ULONG_MAX;
    gulong enterprise_code = ULONG_MAX;
    time_t time_inicio;
    time_t time_fin;
    Repair_filter * filter = (Repair_filter *) g_malloc0(sizeof(Repair_filter));

    license_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_license_number)));
    if ((license_number!=NULL) && (strcmp(license_number,""))) {
        filter-> license_number= g_strdup(license_number);
    }
    legal_id = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_legal_id)));
    if ((legal_id!=NULL)  && (strcmp(legal_id,""))) {
        filter-> legal_id= g_strdup(legal_id);
    }

    name = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_name)));
    if ((name!=NULL) && (strcmp(name,""))) {
        filter-> name= g_strdup(name);
    }

    agente = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_agente)));
    if ((agente!=NULL) && (strcmp(agente,""))) {
        filter->agent_name = g_strdup(agente);
    }

    company = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_company)));
    if ((company!=NULL) && (strcmp(company,""))) {
        filter->company = g_strdup(company);
    }

    garage = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_garage)));
    if ((garage!=NULL) && (strcmp(garage,""))) {
        filter->garage= g_strdup(garage);
    }

    shop_code = c_util_get_code_from_combo (combo_tienda);
    if ((shop_code>0) && (shop_code<ULONG_MAX)) {
	Enterprise * enterprise = m_enterprise_find_by_shop (shop_code);
	tienda = g_strdup_printf("%lu",shop_code);
	if (enterprise !=NULL) {
	    enterprise_code = d_participant_get_code (d_enterprise_get_participant (enterprise));
	    d_enterprise_clear (enterprise);
	}
    }
    if (enterprise_code!=ULONG_MAX) {
        empresa = g_strdup_printf("%lu",enterprise_code);
    }

    if (tienda != NULL) {
        filter->shop_code = g_strdup(tienda);
    }
    if (empresa !=NULL) {
        filter->enterprise_code = g_strdup(empresa);
    }

    time_inicio = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_inicio));
    if (time_inicio>-1) {
	filter -> initial_date = g_strdup(ctime(&time_inicio));
    } else {
        filter->initial_date=NULL;
    }

    time_fin = gnome_date_edit_get_date(GNOME_DATE_EDIT(dateedit_fin));
    if (time_fin>-1) {
	filter -> ending_date = g_strdup(ctime(&time_fin));
    } else {
        filter->ending_date=NULL;
    }

    if (license_number != NULL) g_free(license_number);
    if (legal_id != NULL) g_free(legal_id);
    if (name != NULL) g_free(name);
    if (agente != NULL) g_free(agente);
    if (company != NULL) g_free(company);
    if (garage != NULL) g_free(garage);
    if (inicio != NULL) g_free(inicio);
    if (fin != NULL) g_free(fin);
    if (tienda != NULL) g_free(tienda);
    if (empresa != NULL) g_free(empresa);

    return filter;
}

static void on_search_button_clicked(GtkButton *button, gpointer data) {

    GtkWidget * window;
    GdkCursor * cursor;
    GtkWidget * clist = glade_xml_get_widget(xml,"reparaciones_clist");

    window = lookup_widget(GTK_WIDGET(clist),"app1");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    find_repairs_list();
    fill_repairs_list();

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

}


static void on_delete_button_clicked(GtkButton *button, gpointer data) {

   	GtkWidget * clist = glade_xml_get_widget(xml,"reparaciones_clist");
        GArray * selected;
	c_util_open_question_dialog(_("Are you sure you want to delete the selected repairs?"),
		*on_ok_delete_repairs);

	selected = c_util_get_selected_rows_from_clist (GTK_CLIST(clist));
	if (selected->len > 0) {
		
		c_util_open_info_window (_("Marked lines can't de removed because they represent repairs already managed. !!! \n Just the unpaid movements can be removed"));
		
		/*tk_clist_unselect_all(GTK_CLIST(clist));*/
		g_array_free(selected, TRUE);
	}
	
		 
}


static void on_ok_delete_repairs(gint reply,gpointer user_data) {
     
   if (!reply) {
    	
   	GtkWidget * clist = glade_xml_get_widget(xml,"reparaciones_clist");
   	GArray * selected = c_util_get_selected_rows_from_clist (GTK_CLIST(clist));

   	m_repair_delete (&selected, repairs);
  
   	if (selected->len == 0) {
			
		g_array_free(selected, TRUE);
		selected = NULL;
		c_repair_clean_screen();
		
	}
	   
   	find_repairs_list_selected(selected);
    }
}

static void find_repairs_list() {

	find_repairs_list_selected(NULL);
}

static void find_repairs_list_selected(GArray * selected) {

    GtkWidget * list = glade_xml_get_widget(xml,"reparaciones_clist");
    Repair_filter * repair_filter = obtain_data_to_filter();

    free_objects();
    c_util_clean_widget (list, NULL);
    
    repairs = m_repair_find_by_filter (repair_filter);
    fill_repairs_list();
    
    if (selected != NULL) c_util_list_select_rows (GTK_CLIST(list), selected);
                 	
    m_repair_clear_filter (repair_filter);    

}

static void fill_repairs_list() {
 
  gint index=0;
  gchar *empty_row[]={"","","","","",""};
  GtkWidget * list = glade_xml_get_widget(xml,"reparaciones_clist");

  
  gtk_clist_clear(GTK_CLIST(list));
  
  if (repairs!=NULL) {
    while (repairs[index]!=NULL) {
      Vehicle * vehicle = d_repair_get_vehicle (repairs[index]);
      Participant *participant = d_client_get_participant (d_repair_get_client (repairs[index]));
      Participant *participant_company = d_company_get_participant (d_repair_get_company (repairs[index]));
      Participant *participant_garage = d_garage_get_participant (d_repair_get_garage (repairs[index]));
      gchar *license_number = d_vehicle_get_license_number (vehicle);
      gchar *legal_id = d_participant_get_legal_id (participant);
      gchar *name = d_participant_get_nick (participant);
      gchar *company_name = d_participant_get_nick (participant_company);
      gchar *name_garage = d_participant_get_nick (participant_garage);
      time_t fecha= d_repair_get_in_date (repairs[index]);
      gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);
      size_t stored;
 
      stored=strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&fecha));

      if (index==30) gtk_clist_freeze ((GtkCList *) list);
      
      gtk_clist_append ((GtkCList *) list,empty_row);

      if (license_number!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_LICENSE_NUMBER_COLUMN, license_number);
      }
      
      if (legal_id!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_LEGAL_ID_COLUMN, legal_id);
      }

      if (name!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_NAME_COLUMN, name);
      }

      if (company_name!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_COMPANY_COLUMN, company_name);
      }

      gtk_clist_set_text((GtkCList *) list, index, C_IN_COLUMN, timer_char);

      if (name_garage!=NULL) {
	gtk_clist_set_text((GtkCList *) list, index, C_GARAGE_COLUMN, name_garage);
      }
      
      index++;
      g_free(timer_char);
      g_free(license_number);
      g_free(legal_id);
      g_free(name);
      g_free(company_name);
      g_free(name_garage);	    
      
    }
  }

  gtk_clist_thaw ((GtkCList *) list);
}

static void clean_screen() {


    GtkWidget *vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gchar *s_name = NULL;
    time_t ini = time(NULL) - 86400*7;
    time_t fin = time(NULL) + 86400*7;
    GtkWidget * init_dateedit= NULL;
    GtkWidget * end_dateedit= NULL;

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    init_dateedit = glade_xml_get_widget(xml,"dateedit_ini_listado");
    end_dateedit = glade_xml_get_widget(xml,"dateedit2_listado");
    gnome_date_edit_set_time(GNOME_DATE_EDIT(init_dateedit), ini);
    gnome_date_edit_set_time(GNOME_DATE_EDIT(end_dateedit), fin);

    fill_enterprise_from_combo();
    s_name = u_parameter_manager_get_parameter ("tienda");
    if (s_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "tienda_entry_listado")), s_name);
    }
    g_free(s_name);

    free_objects();
}


static void free_objects() {

  if (repairs != NULL)
    m_repair_clear_array (repairs);

  repairs = NULL;
}

static void on_clean_button_clicked(GtkButton *button, gpointer data) {
 
  clean_screen();
}


static void on_detail_button_clicked(GtkButton *button, gpointer data) {

  GtkWidget * list = glade_xml_get_widget (xml, "reparaciones_clist");
  GArray *selected = c_util_get_selected_rows_from_clist (GTK_CLIST(list));
  
  if (selected->len != 0) {
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");
    GtkWidget * vbox  = lookup_widget(GTK_WIDGET(button), "main_container");
    gint line = g_array_index(selected,gint,0);
    RepairId *repairid = (RepairId *) g_malloc (sizeof(RepairId));
    gulong repair_code_gulong = d_repair_get_repair_code (repairs[line]);
    gchar *repair_code = NULL;
    gchar *enterprise_code_char = NULL;
    gchar *shop_code_char = NULL;
    Repair_line ** repair_lines = NULL;
    Repair *repair = NULL;
    
    if (repair_code_gulong!=ULONG_MAX)
      repair_code = g_strdup_printf("%lu",repair_code_gulong);

    if (enterprise_code!=ULONG_MAX)
      enterprise_code_char = g_strdup_printf("%lu",enterprise_code);

    if (shop_code!=ULONG_MAX)
      shop_code_char = g_strdup_printf("%lu",shop_code);

    repairid->repair_code = repair_code;
    repairid->enterprise_code = enterprise_code_char;
    repairid->shop_code = shop_code_char;
    
    repair_lines = m_repair_line_find_by_id (repairid);
    repair = m_repair_find_by_id (repairid, repair_lines);

    c_repair_open_window(app1, vbox); 

    c_repair_load_repair(repair);		

    i_shortcut_change_dialog (GTK_WIDGET(button),0,1);

    g_free(repairid->repair_code);
    g_free(repairid->enterprise_code);
    g_free(repairid->shop_code);
    g_free(repairid);
    
    repair = NULL;
    repair_lines = NULL;
  }
  else c_util_open_error_window (_("You must select a repair"));

  g_array_free(selected, FALSE);

}

static void on_press_key(GtkWidget *widget, GdkEventKey *event, gpointer user_data) {

    GtkWidget * wid;
    GtkContainer * container;
    GList * list=NULL;
    GList * last_list;
    GList * first_focusable_of_list;


    if (event->keyval==GDK_Return) {
	on_search_button_clicked(NULL,user_data);
    } else if (event->keyval==GDK_Tab) {
     
	container = (GtkContainer *) widget->parent;
	list = gtk_container_children(container);
      	wid=GTK_WIDGET((list)->data);
        while (((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
	      ((GTK_WIDGET_VISIBLE(wid))==0) ||
	       ((GTK_WIDGET_SENSITIVE(wid))==0)) {
	        list=g_list_next(list);
		wid=GTK_WIDGET((list)->data);
	 	}

	first_focusable_of_list = list;

        last_list = (GList *) g_list_last(list);
        while (last_list!=list) {
	    wid=GTK_WIDGET((list)->data);
	    if (GTK_WIDGET_HAS_FOCUS(wid)==1) {
	       break;
	    }
	    list=g_list_next(list);

	}

	list=g_list_next(list);
	wid=GTK_WIDGET((list)->data);
	while ((((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
		((GTK_WIDGET_VISIBLE(wid))==0) ||
		((GTK_WIDGET_SENSITIVE(wid))==0)) && (last_list!=list)){
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}
	if (last_list==list) {
	    list=first_focusable_of_list;
            wid=GTK_WIDGET((list)->data);
	}

	gtk_signal_emit_stop_by_name(GTK_OBJECT (widget),"key_press_event");

/* 	gtk_widget_draw_focus(wid);  */
/* 	gtk_container_set_focus_child(container,wid); */
 	gtk_widget_grab_focus(wid);

/*  	g_print("Nombre activable: %s\n",wid->name);   */

    } 
}


