
/* generic_repair_line.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "generic_repair_line.h"
#include "control_util.h"
#include "repair_line_object.h"
#include "family_model.h"
#include "product_model.h"
#include "repairs_control.h"

static GtkWidget *generic_repair_line=NULL;
static GladeXML *xml;

static gchar url[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="insercion_lineas_reparacion";

static void on_generic_repair_line_accept_button_clicked(GtkButton *button,gpointer user_data);
static void on_generic_repair_line_cancel_button_clicked(GtkButton *button,gpointer user_data);
static void on_generic_repair_line_destroy_window(GtkWidget *widget,gpointer user_data);
static void fill_family_combo_repair_line(void);

GtkWidget *c_generic_repair_line_open_window(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (generic_repair_line==NULL)
     {
	GtkWidget *window;

	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	glade_xml_signal_connect (xml, "on_aceptar_button_clicked",
				  GTK_SIGNAL_FUNC (on_generic_repair_line_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_cancelar_button_clicked",
				  GTK_SIGNAL_FUNC (on_generic_repair_line_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_insercion_lineas_reparacion_destroy",
				  GTK_SIGNAL_FUNC (on_generic_repair_line_destroy_window));

	fill_family_combo_repair_line();
	generic_repair_line = window;
	
     }

   g_free(directory);
#ifdef IS_SHOP
   gtk_widget_set_sensitive(glade_xml_get_widget(xml,"tipoiva_spinbutton"),FALSE);
#endif

   return generic_repair_line;
   
}

static void fill_family_combo_repair_line() {
    GtkWidget * combo;
    Family ** families;


    combo = glade_xml_get_widget (xml, "familia_combo");
    families = m_family_find_all ();

    c_util_fill_combo_with_families (combo, families);

}

static void on_generic_repair_line_accept_button_clicked(GtkButton *button,gpointer user_data){

  Repair_line *repair_line = d_repair_line_create_empty ();
  GtkWidget *combo = glade_xml_get_widget (xml, "familia_combo");    
  GtkSpinButton *spinbutton =  NULL;
  gchar *family = g_strdup(c_util_get_code_from_combo_as_gchar (combo));
  gchar *description = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "descripcion_entry")),0,-1);
  gchar *price;
  gchar *quantity;
  gchar *discount;
  gchar *total;
  gchar *vat_type;
  Product *product = (Product *) g_malloc(sizeof(Product));
  gchar ** stop;
  gdouble quantity_gulong;
  
  d_product_init (product);

  stop = (gchar **) g_malloc0(sizeof(gchar *));

  spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "precio_spinbutton"));
  gtk_spin_button_update(spinbutton);
  price = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "precio_spinbutton")),0,-1);

  spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "descuento_spinbutton"));
  gtk_spin_button_update(spinbutton);
  discount = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "descuento_spinbutton")),0,-1);

  spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "importe_spinbutton"));
  gtk_spin_button_update(spinbutton);
  total = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "importe_spinbutton")),0,-1);

  spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "tipoiva_spinbutton"));
  gtk_spin_button_update(spinbutton);
  vat_type = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "tipoiva_spinbutton")),0,-1);

  spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "unidades_spinbutton"));
  gtk_spin_button_update(spinbutton);
  quantity = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "unidades_spinbutton")),0,-1);

  //quantity_gulong = strtoul(quantity,stop,10);
  quantity_gulong = atof(quantity);
  d_repair_line_set_quantity (repair_line, quantity_gulong);

  u_points_to_commas (discount);
  d_repair_line_set_discount (repair_line, atof(discount));

  u_points_to_commas (total);
  if (atof(total) != G_MINDOUBLE) {
      d_repair_line_set_price (repair_line, atof(total));
  } else {
      d_repair_line_set_price (repair_line, G_MINDOUBLE);
  }

  u_points_to_commas (vat_type);
  if (atof(vat_type) >= 0.0) {
      d_repair_line_set_vat_type (repair_line, atof(vat_type));
      d_repair_line_set_historical_vat_type (repair_line, atof(vat_type));
      d_product_set_vat_type (product,atof(vat_type));
  } else {
      d_repair_line_set_vat_type (repair_line, -1.0);
      d_repair_line_set_historical_vat_type (repair_line, -1.0);
      d_product_set_vat_type (product,-1);
  }

  u_points_to_commas (price);
  if (atof(price) != G_MINDOUBLE) {
      d_product_set_price (product, atof(price));
  } else {
      d_product_set_price (product, G_MINDOUBLE);
  }
    
  if (strcasecmp(family,""))
    d_product_set_family_code (product, family);
    
  if (g_strcasecmp(description,""))
    d_product_set_description (product, description);
  
  d_repair_line_set_product (repair_line, product);

  c_repair_add_line(repair_line);

  m_product_insert (product);

  repair_line = NULL;  
  
  g_free(family);
  g_free(description);
  g_free(price);
  g_free(quantity);
  g_free(discount);
  g_free(total);
  g_free(vat_type);
  g_free(stop);   

  gtk_widget_destroy(generic_repair_line);

}

static void on_generic_repair_line_cancel_button_clicked(GtkButton *button,gpointer user_data){
  
  gtk_widget_destroy(generic_repair_line);
}

static void on_generic_repair_line_destroy_window(GtkWidget *widget,gpointer user_data){
 
  generic_repair_line = NULL;
  xml = NULL;
  
}

