/* insert_cash_operation_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "util.h"
#include "control_util.h"
#include "insert_cash_operation_control.h"
#include "insert_account_control.h"
#include "account_model.h"
#include "cash_operation_model.h"
#include "cash_model.h"

static GtkWidget *insert_cash_operations=NULL;
static GladeXML *xml;
static Payment * payment_to_operate;
gboolean positive;
gboolean from_acept=FALSE;
static void (*function_to_call)(gpointer value);

static gchar widget_name[]="introduccion_de_movimientos_caja";
static gchar url[]="/src/interface/glade/cash_view.glade";
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_cancel_button_clicked(GtkButton *button, gpointer data);
static void on_window_destroy(GtkWidget * widget,gpointer user_data);
static gboolean insert_cash_operation(void);
static void fill_total_data(gdouble total_data);
//static void fill_codigo_pago(void);
static void clean_screen(void);

GtkWidget *c_cash_operation_open_window(GtkWidget *app1,gpointer f_to_call,gboolean pos)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
  if (insert_cash_operations==NULL)
    {
      GtkWidget *dialog;

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);

      glade_xml_signal_connect (xml, "on_button_aceptar_movimientos_caja_clicked",
				GTK_SIGNAL_FUNC (on_accept_button_clicked));
      glade_xml_signal_connect (xml, "on_button_cancelar_movimientos_caja_clicked",
				GTK_SIGNAL_FUNC (on_cancel_button_clicked));
      glade_xml_signal_connect (xml, "on_introduccion_de_movimientos_caja_destroy",
				GTK_SIGNAL_FUNC (on_window_destroy));

      insert_cash_operations=dialog;
      
    }
  else {
      clean_screen();
  }
  /*That payment mustn't be freed*/
  positive = pos;
  payment_to_operate=NULL;
  function_to_call=f_to_call;
  g_free(directory);
  return insert_cash_operations;
  
}
void load_payment_cash_operation(Payment * payment,gdouble total) {
    GtkWidget * shop_entry = glade_xml_get_widget(xml,"entry_tienda_movimientos_caja");
    GtkWidget * enterprise_entry = glade_xml_get_widget(xml,"entry_empresa_movimientos_caja");
    GtkWidget * payment_entry = glade_xml_get_widget(xml,"entry_codigo_pago_movimientos_caja");
    gulong payment_code = d_payment_get_payment_code (payment);
    gchar * shop= d_participant_get_name (d_shop_get_participant (d_payment_get_shop (payment)));
    gchar * enterprise =d_participant_get_name (d_enterprise_get_participant (d_payment_get_enterprise (payment)));
    gchar * payment_gchar = g_strdup_printf("%lu",payment_code);
    gtk_entry_set_text(GTK_ENTRY(shop_entry),shop);
    gtk_entry_set_text(GTK_ENTRY(enterprise_entry),enterprise);
    gtk_entry_set_text(GTK_ENTRY(payment_entry),payment_gchar);
    payment_to_operate = payment;
    fill_total_data(total);
}


static void fill_total_data(gdouble total_data) {
    GtkWidget * entry_total = glade_xml_get_widget(xml,"entry_cantidad_movimiento_caja");
    gchar * total_char;

    if (positive==FALSE) {
	total_char = g_strdup_printf("%.2f",(-1)*total_data);
    } else {
	total_char = g_strdup_printf("%.2f",total_data);
    }
    gtk_entry_set_text(GTK_ENTRY(entry_total),total_char);
}

static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

}


static void on_accept_button_clicked(GtkButton *button, gpointer data) {
    gboolean error = FALSE;
    gboolean * pointer_error = (gboolean *)g_malloc0(sizeof(gboolean));
    gpointer data1;


    error = insert_cash_operation();
    *pointer_error = error;
    data1 = (gpointer) pointer_error;

    if (function_to_call!=NULL) function_to_call(data1);
    from_acept=TRUE;
    if (error == FALSE) {
	gtk_widget_destroy(insert_cash_operations);
	insert_cash_operations = NULL;
    } else {
        c_util_open_error_window (_("Error inserting the movement"));
    }
}
static void on_cancel_button_clicked(GtkButton *button, gpointer data){

    gtk_widget_destroy(insert_cash_operations);
    insert_cash_operations = NULL;
}


static void on_window_destroy(GtkWidget * widget,gpointer user_data) {

    GtkWidget * button_cancelar = glade_xml_get_widget(xml,"button_cancelar_movimientos_caja");
    if ((!GTK_WIDGET_VISIBLE(button_cancelar)) && (from_acept==FALSE)) {
	gboolean error = FALSE;
	gboolean * pointer_error = (gboolean *)g_malloc0(sizeof(gboolean));
	gpointer data1;

	error = insert_cash_operation();
	*pointer_error = error;
	data1 = (gpointer) pointer_error;

	if (function_to_call!=NULL) function_to_call(data1);

	if (error == FALSE) {
	    gtk_widget_destroy(insert_cash_operations);
	    insert_cash_operations = NULL;
	} else {
	    c_util_open_error_window (_("Error inserting the movement"));
	}

    }
    from_acept = FALSE;
}

static gboolean insert_cash_operation() {
    GtkWidget * text_concept = glade_xml_get_widget(xml,"text_concepto_movimiento_caja");
    GtkWidget * movement_date_dateedit = glade_xml_get_widget(xml,"dateedit_fecha_movimiento_caja");
    GtkWidget * quantity_movement_entry = glade_xml_get_widget(xml,"entry_cantidad_movimiento_caja");
    Cash * cash=NULL; 
    gchar * concept;
    time_t date;
    gchar * quantity_char=NULL;
    gdouble quantity;
    gulong shop_code;
    gulong enterprise_code;
    gboolean error=FALSE;
    gint length;
    CashOperation * cash_operation = d_cash_operation_create_empty ();

    shop_code = d_participant_get_code (d_shop_get_participant (d_payment_get_shop (payment_to_operate)));
    enterprise_code = d_participant_get_code (d_enterprise_get_participant (d_payment_get_enterprise (payment_to_operate)));
    cash = m_cash_find_by_shop (enterprise_code,shop_code);
    d_cash_operation_set_cash (cash_operation,cash);
    if (cash!=NULL) {
	d_cash_operation_set_payment (cash_operation,d_payment_duplicate (payment_to_operate));
    }
    date = gnome_date_edit_get_date(GNOME_DATE_EDIT(movement_date_dateedit));

    length = gtk_text_get_length(GTK_TEXT(text_concept));
    if (length<200) {
	concept = gtk_editable_get_chars(GTK_EDITABLE(text_concept),0,length);
    } else {
	concept = gtk_editable_get_chars(GTK_EDITABLE(text_concept),0,199);
    }
    quantity_char = g_strdup(gtk_entry_get_text(GTK_ENTRY(quantity_movement_entry)));
    u_points_to_commas (quantity_char);
    quantity = atof(quantity_char);
    g_free(quantity_char);
    d_cash_operation_set_amount (cash_operation,quantity);
    d_cash_operation_set_date (cash_operation,date);
    d_cash_operation_set_section (cash_operation,concept);
    if (concept!=NULL) {
	g_free(concept);
    }

    m_cash_operation_insert (cash_operation);
    d_cash_operation_clear (cash_operation);
    cash_operation=NULL;
    return error;
}

/*
static void fill_codigo_pago(void) {
        GtkWidget * entry_codigo_pago=glade_xml_get_widget(xml,"entry_codigo_pago_movimientos_caja");
	gchar * value_gchar = g_strdup_printf("%lu",d_payment_get_payment_code (payment_to_operate));
	gtk_entry_set_text(GTK_ENTRY(entry_codigo_pago),value_gchar);
	g_free(value_gchar);
}

*/
void hide_cancelar_button_on_cash_operation() {
    GtkWidget * cancel_button = glade_xml_get_widget(xml,"button_cancelar_movimientos_caja");
    if GTK_WIDGET_VISIBLE(cancel_button) gtk_widget_hide(cancel_button);
}
