/* insert_cash_operation_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "util.h"
#include "control_util.h"
#include "insert_cash_operation_control_2.h"
#include "insert_account_control.h"
#include "account_model.h"
#include "cash_operation_model.h"
#include "cash_model.h"
#include "shop_model.h"
#include "enterprise_model.h"
#include "participant.h"
#include "enterprise.h"
#include "parameter_manager.h"


static GtkWidget *insert_cash_operations=NULL;
static GladeXML *xml;
static gulong shop_code=0;
static gulong enterprise_code=0;

static gchar widget_name[]="vbox_introduccion_de_movimientos_caja_2";
static gchar url[]="/src/interface/glade/cash_view.glade";
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_destroy_window(GtkWidget * widget,gpointer data);
static void on_shop_combo_entry_changed(GtkWidget * widget,gpointer data);
static void fill_shop_combo(void);
static void fill_enterprise_entry(void);
static void clean_screen(void);

GtkWidget *open_cash_operation_2(GtkWidget *app1,GtkWidget *vbox)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   gchar * s_name;
   GtkAdjustment * adj;
   
  if (insert_cash_operations==NULL)
    {
      GtkWidget *dialog;

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);

      gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

      glade_xml_signal_connect (xml, "on_button_aceptar_movimientos_caja_clicked",
				GTK_SIGNAL_FUNC (on_accept_button_clicked));
      glade_xml_signal_connect (xml, "on_introduccion_de_movimientos_caja_destroy",
				GTK_SIGNAL_FUNC (on_destroy_window));
      glade_xml_signal_connect (xml, "on_combo_entry_tienda_introduccion_movimientos_caja_changed",
				GTK_SIGNAL_FUNC (on_shop_combo_entry_changed));

      fill_shop_combo();
      s_name = u_parameter_manager_get_parameter ("tienda");
      if (s_name!=NULL) {
	gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-tienda-introduccion-movimientos-caja")), s_name);

#ifndef IS_SHOP
      gtk_widget_set_sensitive(glade_xml_get_widget(xml, "combo-tienda-introduccion-movimientos-caja"),FALSE);
#endif
      }
      g_free(s_name);
#ifndef IS_SHOP
      adj = gtk_spin_button_get_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(xml,"spinbutton-cantidad-introduccion-movimientos-caja")));
      adj -> upper = 1000000;
      gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(glade_xml_get_widget(xml,"spinbutton-cantidad-introduccion-movimientos-caja")),adj);
#endif

      insert_cash_operations=dialog;
      
    }

  g_free(directory);
  return insert_cash_operations; 
}

static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;

    combo = glade_xml_get_widget (xml, "combo-tienda-introduccion-movimientos-caja");
    shops = m_shop_find_all ();

    c_util_fill_combo_with_shops (combo, shops);

#ifdef IS_SHOP

    gtk_widget_set_sensitive(combo,FALSE);

#endif    
}

static void fill_enterprise_entry() {
  GtkWidget * entry;
  Enterprise * enterprise;
  gchar* enterprise_name;
  shop_code=c_util_get_code_from_combo (glade_xml_get_widget (xml, "combo-tienda-introduccion-movimientos-caja"));
  enterprise = m_enterprise_find_by_shop (shop_code);
  entry = glade_xml_get_widget (xml,"entry-empresa-introduccion-movimientos-caja");
  enterprise_name=d_participant_get_name (d_enterprise_get_participant (enterprise));
  enterprise_code=d_participant_get_code (d_enterprise_get_participant (enterprise));
  gtk_entry_set_text(GTK_ENTRY(entry),enterprise_name);
  d_enterprise_clear (enterprise);
}

 static void on_accept_button_clicked(GtkButton *button, gpointer data) {
   CashOperation* cash_operation=NULL;
   Cash* cash=NULL;
   time_t date;
   gdouble quantity=0;

   gchar     *description    = g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "text-descripcion-introduccion-movimientos-caja")),0,-1));
   GtkWidget * movement_date_dateedit = glade_xml_get_widget(xml,"dateedit-introduccion-movimientos-caja");
   GtkSpinButton * spin_quantity = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spinbutton-cantidad-introduccion-movimientos-caja"));
   quantity     = (gdouble)gtk_spin_button_get_value_as_float(spin_quantity);
   if (quantity!=0) {
     date = gnome_date_edit_get_date(GNOME_DATE_EDIT(movement_date_dateedit));
     cash = m_cash_find_by_shop (enterprise_code,shop_code);
     cash_operation=d_cash_operation_create_empty ();
     d_cash_operation_set_operation_code (cash_operation,0);
     d_cash_operation_set_amount (cash_operation, quantity);
     d_cash_operation_set_date (cash_operation,date);
     d_cash_operation_set_section (cash_operation,description);
     d_cash_operation_set_cash (cash_operation, cash);
     d_cash_operation_set_payment (cash_operation, NULL);
     m_cash_operation_insert (cash_operation);
     d_cash_operation_clear (cash_operation);
     clean_screen();
   } else {
     c_util_open_error_window (_("You can't insert a movement with cero amount in the cash"));
   }
 }

static void on_destroy_window(GtkWidget * widget,gpointer user_data) {

    gtk_widget_destroy(insert_cash_operations);
    insert_cash_operations = NULL;

}

static void on_shop_combo_entry_changed(GtkWidget * widget,gpointer data) {
fill_enterprise_entry();
}

static void clean_screen() {
    GtkSpinButton * spin_quantity = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spinbutton-cantidad-introduccion-movimientos-caja"));
    GtkWidget * description=glade_xml_get_widget(xml, "text-descripcion-introduccion-movimientos-caja");
    gtk_spin_button_set_value(spin_quantity,0.00);
    c_util_clean_widget (description,NULL);
}
