
/* insert_charge_invoice_line.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "insert_charge_invoice_line.h"
#include "charge_invoice_line.h"
#include "control_util.h"


static GtkWidget *insert_charge_invoice_line=NULL;
static GladeXML *xml;
static ChargeInvoiceLine * charge_line = NULL;
static gboolean insert = TRUE;

static gchar url[]="/src/interface/glade/invoice_view.glade";
static gchar widget_name[]="insercion_lineas_factura_cargo";


static void on_cancel_button_clicked(GtkButton *button,gpointer user_data);
static void on_accept_button_clicked(GtkButton *button,gpointer user_data);
static void on_charge_invoice_window_destroy(GtkWidget *widget,gpointer user_data);

GtkWidget *c_charge_invoice_line_open_window(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   gboolean * aux = (gboolean *) data;   

   if (insert_charge_invoice_line==NULL)
     {
	GtkWidget *window;

	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	glade_xml_signal_connect (xml, "on_cancelar_button_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_insert_charge_invoice_line_destroy",
				  GTK_SIGNAL_FUNC (on_charge_invoice_window_destroy));
	glade_xml_signal_connect (xml, "on_aceptar_button_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));


	insert_charge_invoice_line = window;
	
     }

   insert = *aux;

   g_free(directory);

   return insert_charge_invoice_line;
   
}


void c_charge_invoice_line_load(ChargeInvoiceLine *charge_invoice_line){

  if (charge_invoice_line != NULL) {
    gchar *description = d_charge_invoice_line_get_description (charge_invoice_line);
    gdouble vat_type_gdouble = d_charge_invoice_line_get_vat_type (charge_invoice_line);
    gdouble importe_gdouble = d_charge_invoice_line_get_total (charge_invoice_line);
    GtkWidget *spinbutton = NULL;

    if (vat_type_gdouble != -1.0) {
      spinbutton =  glade_xml_get_widget(xml, "tipoiva_spinbutton");
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) vat_type_gdouble);
    }

    if (importe_gdouble != G_MINDOUBLE) {
      spinbutton =  glade_xml_get_widget(xml, "precio_spinbutton");
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) importe_gdouble);
    }

    if (description != NULL) {
      GtkWidget *description_entry =glade_xml_get_widget (xml, "descripcion_entry");
      gtk_entry_set_text (GTK_ENTRY(description_entry),description);
      g_free(description);
    }

    charge_line = charge_invoice_line;        
    
    }
}

static void on_cancel_button_clicked(GtkButton *button,gpointer user_data) {
  
  gtk_widget_destroy(insert_charge_invoice_line);
}


static void on_accept_button_clicked(GtkButton *button,gpointer user_data) {

  GtkSpinButton *spinbutton =  NULL;

  gchar *description = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "descripcion_entry")),0,-1);
  gchar *price = NULL;
  gchar *vat_type = NULL;

  if (insert)
    charge_line = d_charge_invoice_line_create_empty ();
  
  spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "precio_spinbutton"));
  gtk_spin_button_update(spinbutton);
  price = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "precio_spinbutton")),0,-1);

  spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "tipoiva_spinbutton"));
  gtk_spin_button_update(spinbutton);
  vat_type = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "tipoiva_spinbutton")),0,-1);
  
  u_points_to_commas (vat_type);

  if (atof(vat_type) > 0.0)
    d_charge_invoice_line_set_vat_type (charge_line, atof(vat_type));
  else d_charge_invoice_line_set_vat_type (charge_line, -1.0);
  
  u_points_to_commas (price);
  if (!strcmp(price,""))
      d_charge_invoice_line_set_total (charge_line, G_MINDOUBLE);
  else
      d_charge_invoice_line_set_total (charge_line, atof(price));

  if (strcasecmp(description,""))
    d_charge_invoice_line_set_description (charge_line, description);
  
  if (insert)
    c_charge_invoice_add_charge_line (charge_line);
  else
    c_charge_invoice_load_charge_lines_from_list ();

  g_free(description);
  g_free(price);
  g_free(vat_type);

  charge_line = NULL;

  gtk_widget_destroy(insert_charge_invoice_line);  
  
} 


static void on_charge_invoice_window_destroy(GtkWidget *widget,gpointer user_data){
 
  insert_charge_invoice_line = NULL;  
  xml = NULL;
  
}
