/* modify_invoice_number.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "modify_invoice_number.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "invoice_model.h"
#include "parameter_manager.h"
#include "update_entry_control.h"

static GtkWidget *modify_invoice=NULL;
static GladeXML *xml;
static gchar widget_name[]="modifica_numeros";
static gchar url[]="/src/interface/glade/accounting_view.glade";

static void fill_shop_combo(void);
static void on_button_press_event(GtkWidget *widget,
					   GdkEventKey *event,
					   gpointer user_data);

static void on_cancel_button_clicked(GtkButton *button, gpointer data);
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_shop_combo_entry_changed(GtkEntry *entry,gpointer user_data);
static void set_invoice_number(gpointer number);

GtkWidget *c_modify_invoice_open_window(GtkWidget *app1, GtkWidget *vbox)

{
   
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
    

    if (modify_invoice==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);
      
//	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_button_cancelar_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_button_aceptar_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_combo_entry_tienda_changed",
				  GTK_SIGNAL_FUNC (on_shop_combo_entry_changed));
	glade_xml_signal_connect (xml, "on_button_press_factura_event",
				  GTK_SIGNAL_FUNC (on_button_press_event));

                                  

	fill_shop_combo();


	modify_invoice=dialog;

    } else {
	gtk_widget_show(modify_invoice);

    }

  
    return modify_invoice;

}



static void fill_shop_combo() {
    GtkWidget * combo;
    Shop ** shops;

    combo = glade_xml_get_widget (xml, "combo_tienda");

    shops = m_shop_find_all ();

    c_util_fill_combo_with_shops (combo,shops);

}

static void on_cancel_button_clicked(GtkButton *button, gpointer data) {
    gtk_widget_hide(modify_invoice);
}

static void on_accept_button_clicked(GtkButton *button, gpointer data) {

    gulong enterprise_code = 0;
    gulong shop_code = ULONG_MAX;
    GtkWidget * wid=glade_xml_get_widget(xml,"combo_tienda");
    gchar * invoice_number = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_numero_factura"))));
    Enterprise * enter = NULL;
    shop_code = c_util_get_code_from_combo (wid);

    if (strcmp(invoice_number,"")) {
        gboolean result = FALSE;
	if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(glade_xml_get_widget(xml,"checkbutton_tipo")))==FALSE) {
	    enter = m_enterprise_find_by_shop (shop_code);
	    if (enter!=NULL)
		enterprise_code = d_participant_get_code (d_enterprise_get_participant (enter));
	    d_enterprise_clear (enter);
	}
	result = m_invoice_update_number (enterprise_code,shop_code,invoice_number);
	if (result != FALSE) {
	    c_util_open_info_window (_("Data has been modified correctly"));
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_numero_factura")),"");
	} else
	    c_util_open_error_window (_("Error during the modification of the data"));

	gtk_widget_hide(modify_invoice);

    } else {
        c_util_open_info_window (_("Insert the number of the invoice"));
    }
}


static void set_invoice_number(gpointer number) {
    gulong * invoice_number = (gulong *) number;
    gchar * temp = g_strdup_printf("%lu",*invoice_number);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_numero_factura")),temp);
    g_free(temp);

}
static void on_button_press_event(GtkWidget *widget,
					  GdkEventKey *event,
					  gpointer user_data) {
    GtkWidget * app1 = lookup_widget(widget, "app1");
    gchar * temp = g_strdup(_("Invoice number"));
    c_update_entry_open_window(app1,temp,C_GULONG_TYPE,set_invoice_number);
    g_free(temp);

}


static void on_shop_combo_entry_changed(GtkEntry *entry,gpointer user_data) {
    GtkWidget * entry_empresa;
    GtkWidget * combo;
  gchar * aux=NULL;
  gulong code_combo;
  Enterprise * enterprise = NULL;
  
  entry_empresa = glade_xml_get_widget (xml, "entry_empresa");
  
  combo = glade_xml_get_widget (xml, "combo_tienda");
  
  code_combo = c_util_get_code_from_combo (combo);
  
  enterprise = m_enterprise_find_by_shop (code_combo);
  if (enterprise!=NULL) {
    aux = d_participant_get_name (d_enterprise_get_participant (enterprise));
    gtk_entry_set_text(GTK_ENTRY(entry_empresa),aux);
    d_enterprise_clear (enterprise);
    enterprise=NULL;
    g_free(aux);
  }

}
                                                               
