#ifndef _ORDER_CONTROL_
#define _ORDER_CONTROL_

/* order_control.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "control_util.h"
#include "participant.h"
#include "order.h"

/**
 * c_order_open_window:
 * @app1: #Widget que contiene la aplicacin.
 * @vbox: Contenedor que est dentro de la ventana de contabilidad.
 * @supplier: Proveedor al que se le realiza el pedido.
 *
 * Inicializa la ventana de pedido. Esta pantalla permite insertar y modificar pedidos,
 * dando posibilidad de insertar las piezas desde la pantalla de bsqueda de piezas.
 * Adems, permite imprimir el pedido con la platilla normal.
 *
 * Returns:
 * La ventana inicializada.
 */
GtkWidget *c_order_open_window(GtkWidget *app1, GtkWidget *vbox, Participant *supplier);
/**
 * c_order_load:
 * @order: Pedido que se desea cargar en la pantalla.
 *
 * Carga un pedido en la pantalla para posteriormente dar la posibilidad de modificarlo.
 *
 */
void c_order_load(Order *order);
/**
 * c_order_load_pieces:
 * @pieces_ids: Array con los identificadores de las piezas que se desean aadir a un pedido.
 * @number: Nmero de piezas que se aaden al pedido.
 * @enterprise_code: Cdigo de empresa al que pertenece el almacn para el que se solicita el pedido.
 * @warehouse_code: Cdigo de almacn para el que se solicita el pedido.
 * 
 * Carga las piezas de un pedido en la pantalla de pedido.
 */
void c_order_load_pieces(gulong *pieces_ids, gulong *number,
		       gulong enterprise_code, gulong warehouse_code);

#endif

