
/* piece_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <time.h>

#include "a-shortcut.h"
#include "support.h"
#include "piece_model.h"
#include "client_model.h"
#include "control_util.h"
#include "find_pieces_model.h"
#include "shop_model.h"
#include "measure_model.h"
#include "insert_measure_control.h"
#include "enterprise_model.h"
#include "piece_control.h"

#define C_PIECE_DEFAULT_WIDTH             500
#define C_PIECE_DEFAULT_LONG             850
#define C_PIECE_DEFAULT_VAT           16
#define C_PIECE_DEFAULT_QUANTITY          1
#define C_PIECE_DEFAULT_COST_PRICE       100.00
#define C_PIECE_DEFAULT_PRICE            100.00
#define C_PIECE_DEFAULT_DISCOUNT         0.00
#define C_MEASURE_NONE        ULONG_MAX

#ifndef IS_SHOP
static GtkWidget *widget_piece=NULL;
static gchar url[]="/src/interface/glade/store_view.glade";
static gchar widget_name[]="vbox_introduccionpiezas";
static GladeXML *xml = NULL;
static Piece *piece_loaded = NULL;

static void clean_screen(void);
static void fill_family_combo(void);
static void clean_screen(void);
static void on_accept_button_clicked(GtkButton *button, gpointer data);
static void on_cancel_button_clicked(GtkButton *button, gpointer data);
static void on_new_measure_button_clicked(GtkButton *button, gpointer data);

#endif

GtkWidget *c_piece_open_window(GtkWidget *app1, GtkWidget *vbox) {

#ifndef IS_SHOP

    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (widget_piece==NULL) {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	gtk_box_pack_start (GTK_BOX (vbox), dialog, TRUE, TRUE, 0);

	glade_xml_signal_connect (xml, "on_boton-cancelar-introduccionpiezas_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_boton-aceptar-introduccionpiezas_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_button_nueva_medida_clicked",
				  GTK_SIGNAL_FUNC (on_new_measure_button_clicked));

        c_piece_repaint_measures();
	fill_family_combo();
	widget_piece =dialog;
    }

    g_free(directory);
    return widget_piece;
#else
    return NULL;
#endif

}


void c_piece_load(Piece *piece) {

#ifndef IS_SHOP
    gchar  *eurocode = g_strstrip(d_piece_get_euroCode (piece));
    gshort  width    = d_piece_get_width (piece);
    gshort  length    = d_piece_get_length (piece);
    gdouble  price_cost = d_piece_get_cost_price (piece);
    gdouble  packet_quantity = d_piece_get_units_per_packet (piece);
    gulong measure_code  = d_piece_get_meassure_code (piece);

    gchar *family_codetype = g_strstrip(d_product_get_family_code ((Product *)piece));
    gchar *description = g_strstrip(d_product_get_description ((Product *)piece));
    gdouble vat_type = d_product_get_vat_type ((Product *)piece);
    gdouble price  = d_product_get_price ((Product *)piece);
    gdouble max_discount = d_product_get_max_discount ((Product *)piece);

    gchar * measure_name = NULL;
    GtkWidget *combo = glade_xml_get_widget (xml, "combo-familia-introduccionpiezas");
    GtkWidget *spinbutton =  NULL;
    GtkWidget *provider_entry =  glade_xml_get_widget(xml,"entry_proveedor_asignado");
    gchar     *family_name =  g_strdup(c_util_get_name_of_codechar_from_combo (combo, family_codetype));
    Participant * participant = m_client_get_participant_by_code (d_piece_get_supplier_code (piece));
    gchar * provider_name = NULL;
    d_piece_clear (piece_loaded);
    piece_loaded = NULL;
    piece_loaded = d_piece_create_empty ();

    d_piece_set_euroCode (piece_loaded, g_strdup(eurocode));
    d_piece_set_width (piece_loaded, d_piece_get_width (piece));
    d_piece_set_length (piece_loaded, d_piece_get_length (piece));
    d_piece_set_cost_price (piece_loaded, d_piece_get_cost_price (piece));
    if (packet_quantity!=G_MINDOUBLE)
	d_piece_set_units_per_packet (piece_loaded, d_piece_get_units_per_packet (piece));

    d_product_set_product_code ((Product *)piece_loaded, d_product_get_product_code ((Product *) piece));
    d_product_set_family_code ((Product *)piece_loaded, family_codetype);
    d_product_set_description ((Product *)piece_loaded, description);
    d_product_set_vat_type ((Product *)piece_loaded, d_product_get_vat_type ((Product *) piece));
    d_product_set_is_piece ((Product *)piece_loaded, d_product_get_is_piece ((Product *) piece));
    d_product_set_is_man_power ((Product *)piece_loaded, d_product_get_is_man_power ((Product *) piece));
    d_product_set_price ((Product *)piece_loaded, d_product_get_price ((Product *) piece));
    d_product_set_max_discount ((Product *)piece_loaded, d_product_get_max_discount ((Product *) piece));

    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry-eurocode-introduccionpiezas")), eurocode);
    gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget(xml, "entry-eurocode-introduccionpiezas")), 0);
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "entry-descripcion-introduccionpiezas")),description);
    gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget (xml, "entry-descripcion-introduccionpiezas")),0);

    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget (xml, "combo-entry-familia-introduccionpiezas")), family_name);

    spinbutton =  glade_xml_get_widget(xml, "spinbutton-largo-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) length);

    spinbutton = glade_xml_get_widget (xml, "spinbutton-ancho-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) width);

    spinbutton = glade_xml_get_widget (xml, "spinbutton-IVA-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) vat_type);

    spinbutton = glade_xml_get_widget (xml, "spinbutton-precio-introduccionpiezas");
    if (price!=G_MINDOUBLE)
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) price);

    spinbutton  = glade_xml_get_widget (xml, "spinbutton-preciocoste-introduccionpiezas");
    if (price_cost!=G_MINDOUBLE)
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) price_cost);

    spinbutton  = glade_xml_get_widget (xml, "spinbutton-dtomax-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton),  max_discount);

    if (packet_quantity!=G_MINDOUBLE) {
	spinbutton = glade_xml_get_widget (xml, "spin_cantidad_paquete");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) packet_quantity);
    }

    if (measure_code != ULONG_MAX) {
	measure_name = m_meassure_get_name_from_code (measure_code);
        if (measure_name!=NULL)
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_combo_medida")),measure_name);
    }

    if (participant!=NULL) {
	if (d_participant_get_code (participant)!=ULONG_MAX) {
	    provider_name = d_participant_get_nick (participant);
	    gtk_entry_set_text(GTK_ENTRY(provider_entry),provider_name);
	    g_free(provider_name);
	}
	d_participant_clear (participant);
        participant=NULL;
    }
    g_free(eurocode);
    g_free(family_codetype);
    g_free(description);
#endif

}

#ifndef IS_SHOP
static void on_accept_button_clicked(GtkButton *button, gpointer data) {

    Piece     *piece = NULL;

    GtkWidget *combo = glade_xml_get_widget (xml, "combo-familia-introduccionpiezas");
    GtkSpinButton *spinbutton =  NULL;
    gchar     *eurocode    = g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-eurocode-introduccionpiezas")),0,-1));
    gchar     *description = g_strstrip(gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "entry-descripcion-introduccionpiezas")),0,-1));
    gchar     *family_codetype = g_strdup(c_util_get_code_from_combo_as_gchar (combo));
    gshort     length, width;
    gdouble    vat;
    gulong measure_code;
    gdouble    price, price_cost,packet_quantity;
    gdouble       max_discount;
    gboolean   correct = FALSE;

    if ((strlen(eurocode)>1) && (strlen(description)>1) ) {
	if (piece_loaded!=NULL) {
	    piece = piece_loaded;
	} else {
	    piece   = d_piece_create_empty ();
	}

	gtk_entry_set_position(GTK_ENTRY(glade_xml_get_widget(xml, "entry-eurocode-introduccionpiezas")), 0);
	spinbutton =  GTK_SPIN_BUTTON(glade_xml_get_widget(xml, "spinbutton-largo-introduccionpiezas"));
	gtk_spin_button_update(spinbutton);
	length      = (gshort)gtk_spin_button_get_value_as_int(spinbutton);

	spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spinbutton-ancho-introduccionpiezas"));
	gtk_spin_button_update(spinbutton);
	width      = (gshort)gtk_spin_button_get_value_as_int(spinbutton);

	spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spinbutton-IVA-introduccionpiezas"));
	gtk_spin_button_update(spinbutton);
	vat        = (gdouble)gtk_spin_button_get_value_as_float(spinbutton);

	spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spinbutton-precio-introduccionpiezas"));
	gtk_spin_button_update(spinbutton);
	price     = (gdouble)gtk_spin_button_get_value_as_float(spinbutton);

	spinbutton  = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spinbutton-preciocoste-introduccionpiezas"));
	gtk_spin_button_update(spinbutton);
	price_cost = (gdouble)gtk_spin_button_get_value_as_float(spinbutton);

	spinbutton  = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spinbutton-dtomax-introduccionpiezas"));
	gtk_spin_button_update(spinbutton);
	max_discount = gtk_spin_button_get_value_as_float(spinbutton);

	spinbutton  = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "spin_cantidad_paquete"));
	gtk_spin_button_update(spinbutton);
	packet_quantity = gtk_spin_button_get_value_as_float(spinbutton);

	measure_code = c_util_get_code_from_combo (combo);

	d_piece_set_euroCode (piece, eurocode);
	d_piece_set_width (piece, width);
	d_piece_set_length (piece, length);
	d_piece_set_cost_price (piece, price_cost);
	d_piece_set_units_per_packet (piece, packet_quantity);
	d_piece_set_meassure_code (piece, measure_code);

	d_product_set_family_code ((Product *)piece, family_codetype);
	d_product_set_description ((Product *)piece, description);
	d_product_set_vat_type ((Product *)piece, vat);
	d_product_set_price ((Product *)piece, price);
	d_product_set_max_discount ((Product *)piece, max_discount);
	d_product_set_is_piece ((Product *)piece, TRUE);

	if (piece_loaded!=NULL) {
	    correct = m_piece_update (piece);
	    piece = NULL;
	    d_piece_clear (piece_loaded);
	    piece_loaded = NULL;
	} else {
	    correct = m_piece_insert (piece);
	    d_piece_clear (piece);
            piece = NULL;
	}

	if(!correct) {
	  c_util_open_error_window (_("Piece update hasn't been done successfully. There has been a problem in the database.\n Try it again"));
	}


    } else {
	c_util_open_error_window (_("Fill the eurocode and description fields to insert a piece"));
    }
    clean_screen();

    g_free(eurocode);
    g_free(description);
    g_free(family_codetype);
}


static void clean_screen() {

    GtkWidget *vbox;
    GtkWidget *spinbutton;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));
    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

    d_piece_clear (piece_loaded);
    piece_loaded = NULL;

    spinbutton =  glade_xml_get_widget(xml, "spinbutton-largo-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) C_PIECE_DEFAULT_LONG);

    spinbutton = glade_xml_get_widget (xml, "spinbutton-ancho-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) C_PIECE_DEFAULT_WIDTH);

    spinbutton = glade_xml_get_widget (xml, "spinbutton-IVA-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) C_PIECE_DEFAULT_VAT);

    spinbutton = glade_xml_get_widget (xml, "spinbutton-precio-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), C_PIECE_DEFAULT_PRICE);

    spinbutton  = glade_xml_get_widget (xml, "spinbutton-preciocoste-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), C_PIECE_DEFAULT_COST_PRICE);

    spinbutton  = glade_xml_get_widget (xml, "spinbutton-dtomax-introduccionpiezas");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), C_PIECE_DEFAULT_DISCOUNT);

    spinbutton  = glade_xml_get_widget (xml, "spin_cantidad_paquete");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), C_PIECE_DEFAULT_QUANTITY);

}

static void on_cancel_button_clicked(GtkButton *button,
						       gpointer data) {
    clean_screen();
}

static void fill_family_combo() {
    GtkWidget * combo;
    Family ** families;


    combo = glade_xml_get_widget (xml, "combo-familia-introduccionpiezas");
    families = m_family_find_all ();

    c_util_fill_combo_with_families (combo, families);
}

static void on_new_measure_button_clicked(GtkButton *button, gpointer data) {
    GtkWidget * new_measure;
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(button), "app1");

    new_measure = c_meassure_open_window(app1);
    if (new_measure!=NULL) {
	gtk_window_set_modal(GTK_WINDOW(new_measure),1);
	gtk_window_set_position(GTK_WINDOW(new_measure),GTK_WIN_POS_CENTER);
	gtk_window_set_transient_for(GTK_WINDOW(new_measure),
				     GTK_WINDOW(app1));
	gtk_widget_show(GTK_WIDGET(new_measure));
    }

}

void c_piece_repaint_measures(void) {
    GtkWidget * combo = glade_xml_get_widget(xml,"combo_medidas");
    Measure ** measures = m_meassure_reload ();
    c_util_clean_code_and_name_combo (combo);
    c_util_set_code_and_name_on_combo (C_MEASURE_NONE,g_strdup(_("<None>")),GTK_COMBO(combo));
    c_util_fill_combo_with_medidas (combo,measures);


}


#endif
