/*
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "post_address_model.h"
#include "post_address_control.h"
#include "invoice_control.h"

static GtkWidget *post_address_window=NULL;
static GladeXML *xml;
static void (*function_to_call)(gpointer value);
static gboolean new_post = FALSE;
static gulong participant_code;
static PostAddress * general_post_address=NULL;
static gchar widget_name[]="ventana_direccion_envio";
static gchar url[]="/src/interface/glade/clients_view.glade";


static void on_delete_address_clicked(GtkButton *button, gpointer user_data);
static void on_save_address_button_clicked(GtkButton *button,  gpointer user_data);
static void on_cancel_address_button_clicked(GtkButton *button,  gpointer user_data);
static void on_modify_address_button_clicked(GtkButton *button,  gpointer user_data);
static PostAddress * create_post_address_from_screen(void);
static void on_press_key(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void on_window_destroy(GtkWidget *widget,gpointer user_data);
static void inicialize_buttons(void);
static gboolean save_address(void);
static gboolean delete_address(void);
static gboolean modify_address(void);

GtkWidget *c_post_address_open_window(gboolean new_post_address,gpointer f_to_call) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (post_address_window==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_button_borrar_direccion_clicked",
				  GTK_SIGNAL_FUNC (on_delete_address_clicked));
	glade_xml_signal_connect (xml, "on_ventana_direccion_envio_destroy",
				  GTK_SIGNAL_FUNC (on_window_destroy));
	glade_xml_signal_connect (xml, "on_button_guardar_direccion_clicked",
			          GTK_SIGNAL_FUNC (on_save_address_button_clicked));
	glade_xml_signal_connect (xml, "on_button_modificar_direccion_clicked",
				  GTK_SIGNAL_FUNC (on_modify_address_button_clicked));
	glade_xml_signal_connect (xml, "on_button_cancelar_direccion_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_address_button_clicked));
	glade_xml_signal_connect (xml, "on_press_key",
			          GTK_SIGNAL_FUNC (on_press_key));


	post_address_window=dialog;
    }
    function_to_call = f_to_call;
    if (general_post_address !=NULL) {
	d_post_address_clear (general_post_address);
        general_post_address=NULL;
    }
    new_post = new_post_address;
    inicialize_buttons();
    g_free(directory);
    return post_address_window;
}


static void on_delete_address_clicked(GtkButton *button,gpointer user_data) {
    gboolean error;
    error = delete_address();
    if (error==FALSE) {
	if (function_to_call!=NULL) function_to_call(NULL);
	if (general_post_address !=NULL) {
	    d_post_address_clear (general_post_address);
	    general_post_address=NULL;
	}
	gtk_widget_destroy(post_address_window);
	post_address_window = NULL;
    } else {
        c_util_open_error_window (_("There has been an error during the remove command. Check the data"));
    }
}

static void on_press_key(GtkWidget *widget, GdkEventKey *event,
			 gpointer user_data) {
    GtkWidget * wid;
    GtkContainer * container;
    GList * list=NULL;
    GList * last_list;
    GList * first_focusable_of_list;


    if (event->keyval==GDK_Return) {
    } else if (event->keyval==GDK_Tab) {

	container = (GtkContainer *) widget->parent;
	list = gtk_container_children(container);
	wid=GTK_WIDGET((list)->data);

	while (((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
	       ((GTK_WIDGET_VISIBLE(wid))==0) ||
	       ((GTK_WIDGET_SENSITIVE(wid))==0)) {
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}

	first_focusable_of_list = list;

	last_list = (GList *) g_list_last(list);
	while (last_list!=list) {
	    wid=GTK_WIDGET((list)->data);
	    if (GTK_WIDGET_HAS_FOCUS(wid)==1) {
		break;
	    }
	    list=g_list_next(list);

	}
	list=g_list_next(list);
	wid=GTK_WIDGET((list)->data);
	while ((((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
		((GTK_WIDGET_VISIBLE(wid))==0) ||
		((GTK_WIDGET_SENSITIVE(wid))==0)) && (last_list!=list)){
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}
	if (last_list==list) {
	    list=first_focusable_of_list;
	    wid=GTK_WIDGET((list)->data);
	}

	gtk_signal_emit_stop_by_name(GTK_OBJECT (widget),"key_press_event");

	gtk_widget_grab_focus(wid);

    }

}

static void on_save_address_button_clicked(GtkButton *button,gpointer user_data) {
    gboolean error;

    error = save_address();
    if (error==FALSE) {
        participant_code = ULONG_MAX;
	if (function_to_call!=NULL) function_to_call(NULL);
	if (general_post_address !=NULL) {
	    d_post_address_clear (general_post_address);
	    general_post_address=NULL;
	}
	gtk_widget_destroy(post_address_window);
	post_address_window = NULL;
    } else {
        c_util_open_error_window (_("There has been an error during the save command. Check the data"));
    }

}

static void on_modify_address_button_clicked(GtkButton *button,gpointer user_data) {
    gboolean error;

    error = modify_address();
    if (error==FALSE) {
	if (function_to_call!=NULL) function_to_call(NULL);
	if (general_post_address !=NULL) {
	    d_post_address_clear (general_post_address);
	    general_post_address=NULL;
	}
	gtk_widget_destroy(post_address_window);
	post_address_window = NULL;
    } else {
        c_util_open_error_window (_("There has been an error during the modify command. Check the data"));
    }

}

static void on_window_destroy(GtkWidget *widget,gpointer user_data) {
    gtk_widget_destroy(post_address_window);
    post_address_window = NULL;
}

void c_post_address_set_participant_code(gulong party_code) {
    participant_code = party_code;
    if (general_post_address!=NULL) {
	d_post_address_clear (general_post_address);
        general_post_address=NULL;
    }
}

void c_post_address_load(PostAddress * post_address) {
    GtkWidget * entry_street = glade_xml_get_widget(xml,"entry_calle_direccion");
    GtkWidget * entry_floor = glade_xml_get_widget(xml,"entry_piso_direccion");
    GtkWidget * entry_number = glade_xml_get_widget(xml,"entry_numero_direccion");
    GtkWidget * entry_postal_code = glade_xml_get_widget(xml,"entry_cp_direccion");
    GtkWidget * entry_town = glade_xml_get_widget(xml,"entry_municipio_direccion");
    GtkWidget * entry_province = glade_xml_get_widget(xml,"entry_provincia_direccion");
    gchar * street =d_post_address_get_street (post_address);
    gchar * floor =d_post_address_get_floor (post_address);
    gchar * number =d_post_address_get_number (post_address);
    gchar * postal_code_char=NULL;
    gchar * town =d_post_address_get_town (post_address);
    gchar * province =d_post_address_get_province (post_address);

    if (d_post_address_get_postal_code (post_address) != ULONG_MAX) {
        postal_code_char = g_strdup_printf("%lu",d_post_address_get_postal_code (post_address));
    }

    if (street!=NULL) {
        gtk_entry_set_text(GTK_ENTRY(entry_street),street);
    }
    if (floor!=NULL) {
        gtk_entry_set_text(GTK_ENTRY(entry_floor),floor);
    }
    if (postal_code_char!=NULL) {
        gtk_entry_set_text(GTK_ENTRY(entry_number),postal_code_char);
    }
    if (number!=NULL) {
        gtk_entry_set_text(GTK_ENTRY(entry_postal_code),number);
    }
    if (town!=NULL) {
        gtk_entry_set_text(GTK_ENTRY(entry_town),town);
    }
    if (province!=NULL) {
        gtk_entry_set_text(GTK_ENTRY(entry_province),province);
    }

    general_post_address=post_address;
    participant_code = ULONG_MAX;

}

static PostAddress * create_post_address_from_screen(void) {
    GtkWidget * entry_street = glade_xml_get_widget(xml,"entry_calle_direccion");
    GtkWidget * entry_floor = glade_xml_get_widget(xml,"entry_piso_direccion");
    GtkWidget * entry_number = glade_xml_get_widget(xml,"entry_numero_direccion");
    GtkWidget * entry_postal_code = glade_xml_get_widget(xml,"entry_cp_direccion");
    GtkWidget * entry_town = glade_xml_get_widget(xml,"entry_municipio_direccion");
    GtkWidget * entry_province = glade_xml_get_widget(xml,"entry_provincia_direccion");
    gchar * street = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_street)));
    gchar * floor = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_floor)));
    gchar * number = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_number)));
    gchar * postal_code_char = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_postal_code)));
    gchar * town = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_town)));
    gchar * province = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry_province)));
    gulong postal_code=ULONG_MAX;
    PostAddress * post_address = d_post_address_create_empty ();

    d_post_address_set_participant_code (post_address,participant_code);

    if (strcmp(street,"")) {
        d_post_address_set_street (post_address,street);
    } 
    if (strcmp(floor,"")) {
        d_post_address_set_floor (post_address,floor);
    }
    if (strcmp(number,"")) {
        d_post_address_set_number (post_address,number);
    }

    if (strcmp(postal_code_char,"")) {
        gchar ** stop = (gchar**) g_malloc0(sizeof(gchar *));
        postal_code = strtoul(postal_code_char,stop,10);
	d_post_address_set_postal_code (post_address,postal_code);
        g_free(stop);
    }

    if (strcmp(town,"")) {
        d_post_address_set_town (post_address,town);
    }

    if (strcmp(province,"")) {
        d_post_address_set_province (post_address,province);
    }

    g_free(street);
    g_free(floor);
    g_free(number);
    g_free(postal_code_char);
    g_free(town);
    g_free(province);
    
    return post_address;
}

static void inicialize_buttons() {
    GtkWidget * save_button = glade_xml_get_widget(xml,"button_guardar_direccion");
    GtkWidget * modify_button = glade_xml_get_widget(xml,"button_modificar_direccion");
    GtkWidget * delete_button = glade_xml_get_widget(xml,"button_borrar_direccion");

    if (new_post==TRUE) {
	if (!GTK_WIDGET_VISIBLE(save_button)) gtk_widget_show(save_button);
	if (GTK_WIDGET_VISIBLE(modify_button)) gtk_widget_hide(modify_button);
	if (GTK_WIDGET_VISIBLE(delete_button)) gtk_widget_hide(delete_button);
    } else {
	if (GTK_WIDGET_VISIBLE(save_button)) gtk_widget_hide(save_button);
	if (!GTK_WIDGET_VISIBLE(modify_button)) gtk_widget_show(modify_button);
	if (!GTK_WIDGET_VISIBLE(delete_button)) gtk_widget_show(delete_button);
    }
}
static void on_cancel_address_button_clicked(GtkButton *button,  gpointer user_data) {
    on_window_destroy(GTK_WIDGET(button),user_data);
}

static gboolean save_address(void) {
    gboolean error=FALSE;
    if (new_post==FALSE) {
        error = TRUE;
    } else {
	if (general_post_address==NULL) {
	    PostAddress * post_address = create_post_address_from_screen();
	    error = (!m_post_address_insert (post_address));
	} else {
            c_util_open_error_window (_("The address is already in the database.\n The address window will be closed"));
	}
    }
    return error;
}
static gboolean delete_address(void) {
    gboolean error=FALSE;
    if (new_post!=FALSE) {
        error = TRUE;
    } else {
	if ((d_post_address_get_participant_code (general_post_address)!=ULONG_MAX) &&
	    (d_post_address_get_code (general_post_address)!=ULONG_MAX)) {

	} else {
	    c_util_open_error_window (_("Error during the removal of the client.\n The post address window will be closed"));
	}
    }
    return error;
}
static gboolean modify_address(void) {
    gboolean error=FALSE;
    if (new_post!=FALSE) {
        error = TRUE;
    } else {
	if ((d_post_address_get_participant_code (general_post_address)!=ULONG_MAX) &&
	    (d_post_address_get_code (general_post_address)!=ULONG_MAX)) {

	} else {
	    c_util_open_error_window (_("Error during the modification of the address.\n The post address window will be closed"));
	}
    }
    return error;
}




