
/* repair_line_data.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include <gnome.h>
#include <glade/glade.h>
#include <glib.h>

#include "repair_line_data.h"
#include "control_util.h"
#include "repair_line_object.h"
#include "family_model.h"
#include "product_model.h"
#include "repairs_control.h"
#include "manodeobra_model.h"

static GtkWidget *repair_line_data=NULL;
static GladeXML *xml;
static Repair_line *repair_line = NULL;
static Company *company = NULL;

static gchar url[]="/src/interface/glade/repairs_view.glade";
static gchar widget_name[]="modifica_lineas_reparacion";

static void on_accept_button_clicked(GtkButton *button,gpointer user_data);
static void on_cancel_button_clicked(GtkButton *button,gpointer user_data);
static void on_repair_line_window_destroy(GtkWidget *widget,gpointer user_data);
static void load_repair_line(Repair_line *repair_line);
static void fill_family_combo_repair_line(void);

GtkWidget *c_repair_line_open_window(GtkWidget *app1, gpointer data)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
   if (repair_line_data==NULL)
     {
	GtkWidget *window;

	xml = glade_xml_new (directory, widget_name);
	window = glade_xml_get_widget (xml, widget_name);
	
	glade_xml_signal_connect (xml, "on_aceptar_button_clicked",
				  GTK_SIGNAL_FUNC (on_accept_button_clicked));
	glade_xml_signal_connect (xml, "on_cancelar_button_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_button_clicked));
	glade_xml_signal_connect (xml, "on_modifica_lineas_reparacion_destroy",
				  GTK_SIGNAL_FUNC (on_repair_line_window_destroy));

	if (data != NULL) {
	  repair_line = (Repair_line *) data;
	  load_repair_line((Repair_line *) data);
	}	

	fill_family_combo_repair_line();

	repair_line_data = window;
	
     }
#ifdef IS_SHOP
   gtk_widget_set_sensitive(glade_xml_get_widget(xml,"tipoiva_spinbutton"),FALSE);
   gtk_widget_set_sensitive(glade_xml_get_widget(xml,"precio_spinbutton"),FALSE);

#endif

   g_free(directory);

   return repair_line_data;
   
}

void c_repair_line_set_inssurance_company(Company *local_company) {
  
  company = local_company;
}

static void fill_family_combo_repair_line() {
    GtkWidget * combo;
    Family ** families;


    combo = glade_xml_get_widget (xml, "familia_combo");
    families = m_family_find_all ();

    c_util_fill_combo_with_families (combo, families);
    
    if (repair_line != NULL){
      Product *product = d_repair_line_get_product (repair_line);
      gchar *family_codetype = d_product_get_family_code (product);

      GtkWidget *combo_entry =glade_xml_get_widget (xml, "familia_combo_entry");
      gchar *family =  g_strdup(c_util_get_name_of_codechar_from_combo (combo, family_codetype));

      if (family != NULL){
	gtk_entry_set_text(GTK_ENTRY(combo_entry),family);
	g_free(family);
      }
      
      if (family_codetype != NULL) g_free(family_codetype);
    }
}


static void load_repair_line(Repair_line *repair_line){
  gulong line_number_gulong = d_repair_line_get_line_position_number (repair_line);
  gchar *line_number = NULL;
  gdouble quantity_gulong = u_round_to_decimal (d_repair_line_get_quantity (repair_line),3);
  gdouble total_gdouble = d_repair_line_get_price (repair_line);
  gdouble dto_gint = d_repair_line_get_discount (repair_line);
  Product *product = d_repair_line_get_product (repair_line);
  GtkWidget *spinbutton = NULL;
  gdouble vat_gdouble = d_repair_line_get_historical_vat_type (repair_line);

  if (line_number_gulong!=ULONG_MAX)
    line_number = g_strdup_printf("%lu",line_number_gulong);

  if (line_number != NULL) {
    GtkWidget *line_number_entry =glade_xml_get_widget (xml, "numero_spinbutton");
    gtk_entry_set_text (GTK_ENTRY(line_number_entry),line_number);
    g_free(line_number);
  }

  if (quantity_gulong > 0) {
    spinbutton =  glade_xml_get_widget(xml, "unidades_spinbutton");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) quantity_gulong);
  }

  spinbutton =  glade_xml_get_widget(xml, "tipoiva_spinbutton");
  if (vat_gdouble != G_MINDOUBLE) {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) vat_gdouble);
  } else {
      gdouble product_vat = d_product_get_vat_type(product);
      if (product_vat!=G_MINDOUBLE) {
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) product_vat);
      } else {
	  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) 16);
      }
  }


  if (total_gdouble != G_MINDOUBLE) {
      gchar * temp_total= g_strdup_printf("%.2f",total_gdouble);
    spinbutton =  glade_xml_get_widget(xml, "importe_spinbutton");
    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml, "entry_importe_reparacion")),temp_total);
    g_free(temp_total);
  }

  if (dto_gint != -1.0) {
    spinbutton =  glade_xml_get_widget(xml, "descuento_spinbutton");
    gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) dto_gint);
  }

  if (product != NULL){
    gchar *descripcion = d_product_get_description (product);  
    gdouble price_gdouble = d_product_get_price (product);
    gdouble dtomax_gint = d_product_get_max_discount (product);

    if (descripcion != NULL) {
      GtkWidget *descripcion_entry =glade_xml_get_widget (xml, "descripcion_entry");
      gtk_entry_set_text (GTK_ENTRY(descripcion_entry),descripcion);
      g_free(descripcion);
    }

    if (dtomax_gint != -1.0) {
	spinbutton =  glade_xml_get_widget(xml, "dtomax_spinbutton");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) dtomax_gint);
    }


    if (price_gdouble != G_MINDOUBLE) {
	spinbutton =  glade_xml_get_widget(xml, "precio_spinbutton");
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinbutton), (gfloat) price_gdouble);
    }

    if (d_product_get_is_piece (product)){
      gchar *eurocode = d_piece_get_euroCode ((Piece *) product);
      
      if (eurocode != NULL) {
	GtkWidget *eurocode_entry =glade_xml_get_widget (xml, "eurocode_entry");
	gtk_entry_set_text (GTK_ENTRY(eurocode_entry),eurocode);
	g_free(eurocode);
      }
    }
  }

}

static void on_accept_button_clicked(GtkButton *button,gpointer user_data){

  if (repair_line != NULL){
    GtkWidget *combo = glade_xml_get_widget (xml, "familia_combo");    
    GtkSpinButton *spinbutton =  NULL;
    gboolean mo_from_company=FALSE;
    GtkWidget * entry = glade_xml_get_widget(xml, "entry_importe_reparacion");
    Product *product = d_repair_line_get_product (repair_line);
    gchar *number = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "numero_spinbutton")),0,-1);
    gchar *family = g_strdup(c_util_get_code_from_combo_as_gchar (combo));
    gchar *price;
    gchar *quantity;
    gchar *discount;
    gchar *discount_max;
    gchar *total;
    gchar *vat;
    gchar ** stop;
    gdouble aux;

    
    stop = (gchar **) g_malloc0(sizeof(gchar *));

    spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "precio_spinbutton"));
    gtk_spin_button_update(spinbutton);
    price = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "precio_spinbutton")),0,-1);

    spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "descuento_spinbutton"));
    gtk_spin_button_update(spinbutton);
    discount = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "descuento_spinbutton")),0,-1);

    total = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

    spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "dtomax_spinbutton"));
    gtk_spin_button_update(spinbutton);
    discount_max = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "dtomax_spinbutton")),0,-1);

    spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "unidades_spinbutton"));
    gtk_spin_button_update(spinbutton);
    quantity = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "unidades_spinbutton")),0,-1);

    spinbutton = GTK_SPIN_BUTTON(glade_xml_get_widget (xml, "tipoiva_spinbutton"));
    gtk_spin_button_update(spinbutton);
    vat = gtk_editable_get_chars(GTK_EDITABLE(glade_xml_get_widget (xml, "tipoiva_spinbutton")),0,-1);

    aux = atof(quantity);
    d_repair_line_set_quantity (repair_line, aux);

    u_points_to_commas (discount);
    d_repair_line_set_discount (repair_line, atof(discount));

    u_points_to_commas (total);

    if (strcmp(total,"")) {
	d_repair_line_set_price (repair_line, atof(total));
    } else {
	d_repair_line_set_price (repair_line, G_MINDOUBLE);
    }

    if (product != NULL){
      if (strcasecmp(family,""))
	  d_product_set_family_code (product, family);

      u_points_to_commas (price);


      if ((atof(price) != G_MINDOUBLE) && (strcmp(price,""))){
	  d_product_set_price (product, atof(price));
	  if (d_product_get_is_man_power (product)) {
	      m_man_power_update_price ((ManPower *) product,company);
	      if (company!=NULL)
		  mo_from_company=TRUE;
	  }
      } else {
	  d_product_set_price (product, G_MINDOUBLE);
      }

      u_points_to_commas (discount_max);
      d_product_set_max_discount (product, atof(discount_max));

      u_points_to_commas (vat);
//      d_product_set_vat_type (product, atof(vat));
      d_repair_line_set_historical_vat_type (repair_line, atof(vat));


      m_product_update (product,mo_from_company);

    }
      
    g_free(number);
    g_free(family);
    g_free(price);
    g_free(quantity);
    g_free(discount);
    g_free(discount_max);
    g_free(total);
    g_free(stop);
  }

  c_repair_on_line_changed();
  gtk_widget_destroy(repair_line_data);

}

static void on_cancel_button_clicked(GtkButton *button,gpointer user_data){
  
  gtk_widget_destroy(repair_line_data);
}

static void on_repair_line_window_destroy(GtkWidget *widget,gpointer user_data){
 
  d_company_clear (company);
  company = NULL;
  repair_line=NULL;
  repair_line_data = NULL;  
  xml = NULL;
  
}
