
/* repairs_control.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#ifndef _REPAIRS_CONTROL_
#define _REPAIRS_CONTROL_

/**
 * C_COMPANY_NONE:
 *
 * Cdigo para indicar que no se insertar una compaa de seguros.
 *
 */
#define C_COMPANY_NONE ULONG_MAX

#include "piece.h"
#include "client.h"
#include "garage.h"
#include "agent.h"
#include "receipt.h"
#include "policy.h"
#include "repair_object.h"

/**
 * c_repair_open_window:
 * @app1: #Widget que contiene la aplicacin.
 * @vbox: Contenedor que est dentro de la ventana de contabilidad.
 *
 * Inicializa la ventana de introduccin de reparacin. La pantalla de introduccin de reparaciones incluye la
 * inroduccin y modificacin de todo lo referente a una reparacin, permite introducir un cliente, un vehculo,
 * el seguro del vehculo, el agente de la reparacin, el taller por el cul lleg el vehculo a ser reparado en
 * un taller del grupo y las lneas de reparacin. Las lneas de reparacin pueden ser piezas existentes, manos de obra o
 * productos genricos que se pueden crear en el momento de insertarlos en la reparacin.
 *
 * Returns:
 * La ventana inicializada.
 */
GtkWidget *c_repair_open_window(GtkWidget *app1, GtkWidget *vbox);
/**
 * c_repair_load_client:
 * @client: Cliente a cargar en la pantalla de reparacin.
 * @clean: Indica si se debe limpiar la reparacin o si se trata de un cambio de cliente en la misma
 * reparacin.
 *
 * Carga el clietne seleccionado en la reparacin actual, si se procede de cambio de cliente o
 * crea una nueva reparacin en otro caso.
 * 
 */
void c_repair_load_client(Client *client, gboolean clean);
/**
 * c_repair_load_pieces:
 * @stocks: Lista de stocks de piezas a cargar en la reparacin
 *
 * Carga los stocks de las piezas de las lneas de reparacin.
 *
 */
void c_repair_load_pieces(Stock **stocks);
/**
 * c_repair_toggle_garage:
 * @active: Estado en el que se desea que quede el botn de taller.
 * 
 * Establece el estado en el que se desea que est el botn de taller, pulsado o no. Estar pulsado
 * cuando la reparacin tenga un taller asociado.
 *
 */
void c_repair_toggle_garage(gboolean active);
/**
 * c_repair_toggle_contrary_garage:
 * 
 * Invierte el estado actual del botn de taller asociado.
 *
 */
void c_repair_toggle_contrary_garage(void);
/**
 * c_repair_insert_garage:
 * @garage: Garage a insertar.
 *
 * Inserta un taller en la reparacin que est actualmente cargada en pantalla.
 *
 */
void c_repair_insert_garage(Garage *garage);
/**
 * c_repair_toggle_agent:
 * @active: Estado en el que se desea que quede el botn de agente.
 * 
 * Establece el estado en el que se desea que est el botn de agente, pulsado o no. Estar pulsado
 * cuando la reparacin tenga un agente asociado.
 *
 */
void c_repair_toggle_agent(gboolean active);
/**
 * c_repair_toggle_contrary_agent:
 * 
 * Invierte el estado actual del botn de agente asociado.
 * 
 */
void c_repair_toggle_contrary_agent(void);
/**
 * c_repair_insert:
 * @agent: Agente que se desea insertar.
 * 
 * Inserta un agente en la reparacin que est actualmente cargada en pantalla.
 * 
 */
void c_repair_insert(Agent *agent);
/**
 * c_repair_toggle_insurance:
 * @active: Estado en el que se desea que quede el botn de taller.
 * @local_new_policy: Indica si la poliza es nueva o no.
 * @local_new_receipt: Indica si el recibo es nuevo o no.
 * 
 * Establece el estado en el que se desea que est el botn del seguro, pulsado o no. Estar pulsado
 * cuando la reparacin tenga datos de siniestro y seguro asociados.
 *
 */
void c_repair_toggle_insurance(gboolean active, gboolean local_new_policy, gboolean local_new_receipt);
/**
 * c_repair_toggle_contrary_insurance:
 * 
 * Invierte el estado del botn de datos de seguro.
 */
void c_repair_toggle_contrary_insurance(void);
/**
 * c_repair_free_insurance:
 * 
 * Libera la memoria de los datos relacionados con el seguro de la reparacin: el siniestro, el recibo y
 * la pliza.
 * 
 */
void c_repair_free_insurance(void) ;
/**
 * c_repair_insert_insurance:
 * @insurance: Indica si existe seguro asociado o no.
 * 
 * Introduce el nmero de pliza en la entrada de texto que est al lado del botn de seguro en caso
 * de que la reparacin tenga asociado un seguro.
 * 
 */
void c_repair_insert_insurance(gboolean insurance);
/**
 * c_repair_on_line_changed:
 * 
 * Recarga las lneas de reparacin de una reparacin cuando se produce una modificacin de las lneas.
 * 
 */
void c_repair_on_line_changed(void);
/**
 * c_repair_load_repair:
 * @repair: Reparacin a cargar.
 * 
 * Carga en la pantalla una reparacin ya existente para poder modificarla.
 * 
 */
void c_repair_load_repair(Repair *repair);
/**
 * c_repair_add_line:
 * @repair_line: Lnea de reparacin a aadir
 * 
 * Aade una lnea de reparacin a la reparacin actual.
 * 
 */
void c_repair_add_line(Repair_line * repair_line);
/**
 * c_repair_document_destroyed:
 * 
 * Desrefencia la ventana de documentos.
 * 
 */
void c_repair_document_destroyed(void);


/**
 * c_repair_clean_screen:
 * 
 * Limpia la pantalla de introduccin de reparaciones.
 * 
 */
void c_repair_clean_screen(void);


#endif

