/* select_fields_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>

#define C_FILL_FIELD 1
#define C_FILL_COMPLETE 2
#define C_FILL_INVOICE 3
#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "select_fields_control.h"
#include "../interface/printing/select_fields.h"
#include "invoice.h"

static GtkWidget *select_fields=NULL;
static GladeXML *xml;
static gint invoice_template;
static void (*function_to_call)(gpointer value,gpointer value2);
static gchar widget_name[]="relleno_de_facturas";
static gchar url[]="/src/interface/glade/invoice_view.glade";


static void on_cancel_fill_clicked(GtkButton *button, gpointer user_data);
static void on_acept_fill_clicked(GtkButton *button,  gpointer user_data);
static void on_fill_invoice_radiobutton_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void on_complete_fields_radiobutton_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void on_fill_fields_radiobutton_toggled(GtkToggleButton *togglebutton,
						 gpointer user_data);
static void reset_buttons(gint invoice_temp,gint what_to_do);
static void set_payment_from_template(gint temp);
static GeneralCondition * get_state_buttons(void);
void button_set_active (GtkWidget * wid,gpointer data);

GtkWidget *c_select_fields_open_window(gpointer f_to_call,gint inv_temp) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (select_fields==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_cancelar_rellenar_clicked",
				  GTK_SIGNAL_FUNC (on_cancel_fill_clicked));
	glade_xml_signal_connect (xml, "on_aceptar_rellenar_clicked",
			          GTK_SIGNAL_FUNC (on_acept_fill_clicked));
	glade_xml_signal_connect (xml, "on_radiobutton_factura_rellenar_toggled",
			          GTK_SIGNAL_FUNC (on_fill_invoice_radiobutton_toggled));
	glade_xml_signal_connect (xml, "on_radiobutton_campos_rellenar_toggled",
			          GTK_SIGNAL_FUNC (on_fill_fields_radiobutton_toggled));
	glade_xml_signal_connect (xml, "on_radiobutton_completar_rellenar_toggled",
			          GTK_SIGNAL_FUNC (on_complete_fields_radiobutton_toggled));

        set_payment_from_template(inv_temp);
	reset_buttons(inv_temp,C_FILL_INVOICE);

	select_fields=dialog;
    }
    invoice_template = inv_temp;
    function_to_call = f_to_call;
    g_free(directory);
    return select_fields;
}


static void on_cancel_fill_clicked(GtkButton *button,gpointer user_data) {

    gtk_widget_destroy(select_fields);
    function_to_call = NULL;
    select_fields = NULL;
}

static void set_payment_from_template(gint temp) {
    GtkWidget * frame_normal = glade_xml_get_widget(xml,"frame_normal");

    if (temp==D_INVOICE_NORMAL_CODE) {
        if (!GTK_WIDGET_VISIBLE(frame_normal)) gtk_widget_show(frame_normal);
    }
}

static void on_acept_fill_clicked(GtkButton *button,gpointer user_data) {
    GeneralCondition * general_condition = get_state_buttons();
    gint * value1 = (gint *) g_malloc0(sizeof(gint));

    *value1 = invoice_template;
    gtk_widget_hide(select_fields);
    function_to_call(value1,general_condition);
    gtk_widget_destroy(select_fields);
    function_to_call = NULL;
    select_fields = NULL;
}

static void on_fill_invoice_radiobutton_toggled(GtkToggleButton *togglebutton,
						    gpointer user_data) {
    if (gtk_toggle_button_get_active(togglebutton)!=FALSE) {
        reset_buttons(invoice_template,C_FILL_INVOICE);
    }
}
static void on_complete_fields_radiobutton_toggled(GtkToggleButton *togglebutton,
						      gpointer user_data) {
    if (gtk_toggle_button_get_active(togglebutton)!=FALSE) {
        reset_buttons(invoice_template,C_FILL_COMPLETE);
    }

}
static void on_fill_fields_radiobutton_toggled(GtkToggleButton *togglebutton,
						   gpointer user_data) {
    if (gtk_toggle_button_get_active(togglebutton)!=FALSE) {
        reset_buttons(invoice_template,C_FILL_FIELD);
    }

}

void button_set_active (GtkWidget * wid,gpointer data) {
    gboolean * value = (gboolean *) data;
    if (GTK_IS_TOGGLE_BUTTON(wid))
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(wid),*value);

}

static void reset_buttons(gint invoice_temp,gint what_to_do) {
    switch (invoice_temp) {
    case D_INVOICE_NORMAL_CODE:{
	if (what_to_do==C_FILL_INVOICE) {
	    GtkWidget * wid = glade_xml_get_widget(xml,"fixed_normal");
	    gboolean * value = (gboolean *) g_malloc0(sizeof(gboolean));
	    *value = TRUE;
	    gtk_container_foreach(GTK_CONTAINER(wid),&button_set_active,value);
            g_free(value);

	} else if  (what_to_do==C_FILL_COMPLETE) {
	    GtkWidget * wid = glade_xml_get_widget(xml,"fixed_normal");
	    GtkWidget * check_invoice = glade_xml_get_widget(xml,"factura_normal_rellenar");
	    GtkWidget * check_quantity = glade_xml_get_widget(xml,"unidades_normal_rellenar");
	    GtkWidget * check_price = glade_xml_get_widget(xml,"precio_normal_rellenar");
	    GtkWidget * check_total2 = glade_xml_get_widget(xml,"importe_normal_rellenar");
	    GtkWidget * check_discount = glade_xml_get_widget(xml,"dto_normal_rellenar");
	    GtkWidget * check_total = glade_xml_get_widget(xml,"total_normal_rellenar");
	    GtkWidget * check_discounts = glade_xml_get_widget(xml,"descuentos_normal_rellenar");

	    GtkWidget * check_subtotal = glade_xml_get_widget(xml,"base_imponible_normal_rellenar");
	    GtkWidget * check_vat = glade_xml_get_widget(xml,"iva_normal_rellenar");
	    GtkWidget * check_total_invoice = glade_xml_get_widget(xml,"total_factura_normal_rellenar");
	    GtkWidget * check_payment_mode = glade_xml_get_widget(xml,"forma_pago_normal_rellenar");
	    GtkWidget * check_account = glade_xml_get_widget(xml,"cuenta_normal_rellenar");

	    GtkWidget * check_instalments = glade_xml_get_widget(xml,"vencimientos_normal_rellenar");
	    GtkWidget * check_entidad = glade_xml_get_widget(xml,"entidad_pago_normal_rellenar");
	    GtkWidget * check_amount = glade_xml_get_widget(xml,"cantidad_normal_rellenar");
	    GtkWidget * check_lines = glade_xml_get_widget(xml,"lineas_factura_normal_rellenar");

	    gboolean * value = (gboolean *) g_malloc0(sizeof(gboolean));
	    *value = FALSE;
	    gtk_container_foreach(GTK_CONTAINER(wid),&button_set_active,value);
            g_free(value);

            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_invoice),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_quantity),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_price),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_total2),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_discount),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_discounts),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_total),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_subtotal),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_vat),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_total_invoice),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_payment_mode),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_account),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_instalments),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_entidad),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_amount),TRUE);
            gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_lines),FALSE);
	} else {
	}
	break;
    }

    }
}

static GeneralCondition * get_state_buttons() {
    GeneralCondition * general_condition = (GeneralCondition *) g_malloc0(sizeof(GeneralCondition));
    switch (invoice_template) {
    case D_INVOICE_NORMAL_CODE:{
        GtkWidget * check_date = glade_xml_get_widget(xml,"fecha_normal_rellenar");
        GtkWidget * check_sheet = glade_xml_get_widget(xml,"hoja_normal_rellenar");
        GtkWidget * check_enterprise_legal_id_template_1 = glade_xml_get_widget(xml,"cif_normal_rellenar");
        GtkWidget * check_cliente = glade_xml_get_widget(xml,"cliente_normal_rellenar");
	GtkWidget * check_invoice = glade_xml_get_widget(xml,"factura_normal_rellenar");

	GtkWidget * check_agent = glade_xml_get_widget(xml,"agente_normal_rellenar");
        GtkWidget * check_nick = glade_xml_get_widget(xml,"nick_normal_rellenar");
        GtkWidget * check_street = glade_xml_get_widget(xml,"domicilio_normal_rellenar");
        GtkWidget * check_town = glade_xml_get_widget(xml,"municipio_normal_rellenar");
	GtkWidget * check_province = glade_xml_get_widget(xml,"provincia_normal_rellenar");

	GtkWidget * check_company = glade_xml_get_widget(xml,"compania_normal_rellenar");
        GtkWidget * check_policy = glade_xml_get_widget(xml,"poliza_normal_rellenar");
        GtkWidget * check_payment_note = glade_xml_get_widget(xml,"albaran_normal_rellenar");
        GtkWidget * check_reference = glade_xml_get_widget(xml,"referencia_normal_rellenar");
        GtkWidget * check_description = glade_xml_get_widget(xml,"descripcion_normal_rellenar");

	GtkWidget * check_quantity = glade_xml_get_widget(xml,"unidades_normal_rellenar");
        GtkWidget * check_price = glade_xml_get_widget(xml,"precio_normal_rellenar");
        GtkWidget * check_total2 = glade_xml_get_widget(xml,"importe_normal_rellenar");
        GtkWidget * check_discount = glade_xml_get_widget(xml,"dto_normal_rellenar");
        GtkWidget * check_discounts = glade_xml_get_widget(xml,"descuentos_normal_rellenar");
        GtkWidget * check_total = glade_xml_get_widget(xml,"total_normal_rellenar");

	GtkWidget * check_subtotal = glade_xml_get_widget(xml,"base_imponible_normal_rellenar");
	GtkWidget * check_vat = glade_xml_get_widget(xml,"iva_normal_rellenar");
        GtkWidget * check_total_invoice = glade_xml_get_widget(xml,"total_factura_normal_rellenar");
        GtkWidget * check_payment_mode = glade_xml_get_widget(xml,"forma_pago_normal_rellenar");
        GtkWidget * check_account = glade_xml_get_widget(xml,"cuenta_normal_rellenar");

	GtkWidget * check_instalments = glade_xml_get_widget(xml,"vencimientos_normal_rellenar");
	GtkWidget * check_entidad = glade_xml_get_widget(xml,"entidad_pago_normal_rellenar");
	GtkWidget * check_amount = glade_xml_get_widget(xml,"cantidad_normal_rellenar");
	GtkWidget * check_enterprise = glade_xml_get_widget(xml,"empresa_normal_rellenar");
	GtkWidget * check_enterprise_legal_id = glade_xml_get_widget(xml,"cif_empresa_normal_rellenar");
	GtkWidget * check_lines = glade_xml_get_widget(xml,"lineas_factura_normal_rellenar");
	GtkWidget * check_dossier = glade_xml_get_widget(xml,"expediente_normal_rellenar");
	GtkWidget * check_license_number = glade_xml_get_widget(xml,"matricula_normal_rellenar");

	general_condition->date=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_date));
	general_condition->page_number=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_sheet));
	general_condition->enterprise_legal_id_template_1=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_enterprise_legal_id_template_1));
	general_condition->client_number=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_cliente));
	general_condition->invoice_number=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_invoice));

	general_condition->agent=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_agent));
	general_condition->nick=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_nick));
	general_condition->street=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_street));
	general_condition->town=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_town));
	general_condition->province=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_province));

	general_condition->company=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_company));
	general_condition->policy_number=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_policy));
	general_condition->work_number=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_payment_note));
	general_condition->reference=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_reference));
	general_condition->description=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_description));

	general_condition->quantity=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_quantity));
	general_condition->prices=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_price));
	general_condition->total=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_total2));
	general_condition->discount=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_discount));
	general_condition->discounts=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_discounts));
	general_condition->line_total=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_total));
	general_condition->invoice_total=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_total_invoice));

	general_condition->payment_way=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_payment_mode));
	general_condition->account=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_account));
	general_condition->payment_entity=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_entidad));
	general_condition->instalments=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_instalments));
	general_condition->amount=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_amount));

	general_condition->subtotal=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_subtotal));
	general_condition->vat=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_vat));
	general_condition->enterprise_name=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_enterprise));
	general_condition->enterprise_legal_id=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_enterprise_legal_id));
	general_condition->invoice_lines=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_lines));
	general_condition->license_number=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_license_number));
	general_condition->dossier=gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_dossier));

	general_condition->garage_data=FALSE;
	general_condition->name=FALSE;
	general_condition->surnames=FALSE;
	general_condition->postal_code=FALSE;
	general_condition->phone=FALSE;
	general_condition->accident_number=FALSE;
	general_condition->accident_date=FALSE;
	general_condition->field=FALSE;
	general_condition->province_code=FALSE;
	general_condition->reason_code=FALSE;
	general_condition->made=FALSE;
	general_condition->model=FALSE;
	general_condition->chassis=FALSE;
	general_condition->kilometers=FALSE;
	general_condition->colour=FALSE;
	general_condition->in_date=FALSE;
	general_condition->out_date=FALSE;
	general_condition->accident_description=FALSE;
	general_condition->repairer_centre_code=FALSE;
	general_condition->repairer_centre_data=FALSE;
	general_condition->material_quantity=FALSE;
	general_condition->material_description=FALSE;
	general_condition->totals=FALSE;
	general_condition->man_power_description=FALSE;
	general_condition->man_power_total=FALSE;
	general_condition->receiver_legal_id=FALSE;
	general_condition->observations=FALSE;
	general_condition->totals=FALSE;
	general_condition->validity_period=FALSE;
	general_condition->holder=FALSE;
	general_condition->place=FALSE;
	general_condition->way=FALSE;
	general_condition->damages=FALSE;
	general_condition->receiver_nick=FALSE;
	general_condition->receipt_number=FALSE;
	general_condition->branch=FALSE;

	break;
    }
    }

    return general_condition;
}



