/* update_entry_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "update_entry_control.h"
#include "invoice_control.h"

static GtkWidget *update_entry=NULL;
static GladeXML *xml;
gint tipo=-1;
static void (*function_to_call)(gpointer value);
static gchar widget_name[]="ventana_modifica_entry";
static gchar url[]="/src/interface/glade/invoice_view.glade";


static void on_modify_cancel_clicked(GtkButton *button, gpointer user_data);
static void on_modify_accept_clicked(GtkButton *button,  gpointer user_data);
static void on_press_key(GtkWidget *widget, GdkEventKey *event, gpointer user_data);
static void on_entry_to_modify_key_press_event(GtkWidget *widget,
						 GdkEventKey *event,
						 gpointer user_data);

GtkWidget *c_update_entry_open_window(GtkWidget *app1, gchar * label,gint tipo_entry,gpointer f_to_call) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (update_entry==NULL)
    {
	GtkWidget *dialog;
	GtkWidget * label1;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_cancelar_modificacion_clicked",
				  GTK_SIGNAL_FUNC (on_modify_cancel_clicked));
	glade_xml_signal_connect (xml, "on_aceptar_modificacion_clicked",
			          GTK_SIGNAL_FUNC (on_modify_accept_clicked));
	glade_xml_signal_connect (xml, "on_entry_a_modificar_key_press_event",
			          GTK_SIGNAL_FUNC (on_entry_to_modify_key_press_event));

	label1 = glade_xml_get_widget(xml,"label_entry_a_modificar");
	gtk_label_set_text(GTK_LABEL(label1),label);

	update_entry=dialog;
    }
    tipo = tipo_entry;
    function_to_call = f_to_call;
    g_free(directory);
    return update_entry;
}

void c_update_entry_set_max_length(guint16 max) {
    GtkWidget * entry = glade_xml_get_widget(xml,"entry_a_modificar");
    gtk_entry_set_max_length(GTK_ENTRY(entry),max);

}

static void on_modify_cancel_clicked(GtkButton *button,gpointer user_data) {
    gtk_widget_destroy(update_entry);
    function_to_call = NULL;
    update_entry = NULL;
}

static void on_entry_to_modify_key_press_event(GtkWidget *widget,
						 GdkEventKey *event,
						 gpointer user_data) {
    on_press_key(widget, event, user_data);
}

static void on_press_key(GtkWidget *widget, GdkEventKey *event,
			 gpointer user_data) {
    GtkWidget * wid;
    GtkContainer * container;
    GList * list=NULL;
    GList * last_list;
    GList * first_focusable_of_list;


    if (event->keyval==GDK_Return) {
	on_modify_accept_clicked(NULL, user_data);
    } else if (event->keyval==GDK_Tab) {

	container = (GtkContainer *) widget->parent;
	list = gtk_container_children(container);
	wid=GTK_WIDGET((list)->data);

	while (((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
	       ((GTK_WIDGET_VISIBLE(wid))==0) ||
	       ((GTK_WIDGET_SENSITIVE(wid))==0)) {
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}

	first_focusable_of_list = list;

	last_list = (GList *) g_list_last(list);
	while (last_list!=list) {
	    wid=GTK_WIDGET((list)->data);
	    if (GTK_WIDGET_HAS_FOCUS(wid)==1) {
		break;
	    }
	    list=g_list_next(list);

	}
	list=g_list_next(list);
	wid=GTK_WIDGET((list)->data);
	while ((((GTK_WIDGET_CAN_FOCUS(wid))==0) ||
		((GTK_WIDGET_VISIBLE(wid))==0) ||
		((GTK_WIDGET_SENSITIVE(wid))==0)) && (last_list!=list)){
	    list=g_list_next(list);
	    wid=GTK_WIDGET((list)->data);
	}
	if (last_list==list) {
	    list=first_focusable_of_list;
	    wid=GTK_WIDGET((list)->data);
	}

	gtk_signal_emit_stop_by_name(GTK_OBJECT (widget),"key_press_event");

	gtk_widget_grab_focus(wid);

	/*  	g_print("Nombre activable: %s\n",wid->name);  */
    }

}

static void on_modify_accept_clicked(GtkButton *button,gpointer user_data) {
    GtkWidget * entry = glade_xml_get_widget(xml,"entry_a_modificar");
    gint * error = (gint * ) g_malloc0(sizeof(gint));
    gpointer data = NULL;
    *error = 0;
    if (tipo == C_GCHAR_TYPE) {
	gchar * value = (gchar * ) g_malloc0(sizeof(gchar));
	value = c_update_entry_get_char_value(GTK_ENTRY(entry));
        data = (gpointer) value;
    } else if (tipo == C_GULONG_TYPE) {
	gulong * value = (gulong * ) g_malloc0(sizeof(gulong));
	*value = c_update_entry_get_gulong_value(GTK_ENTRY(entry),error);
        data = (gpointer) value;
    } else if (tipo == C_GLONG_TYPE) {
	glong * value = (glong * ) g_malloc0(sizeof(glong));
	*value = c_update_entry_get_glong_value(GTK_ENTRY(entry),error);
        data = (gpointer) value;
    } else if (tipo == C_GINT_TYPE) {
	gint * value = (gint * ) g_malloc0(sizeof(gint));
	*value = c_update_entry_get_gint_value(GTK_ENTRY(entry),error);
        data = (gpointer) value;

    } else if (tipo == C_GDOUBLE_TYPE) {
	gdouble * value = (gdouble * ) g_malloc0(sizeof(gdouble));
	*value = c_update_entry_get_double_value(GTK_ENTRY(entry),error);
        data = (gpointer) value;
    } else {
	g_print(_("Not yet implemented"));
        *error = 1;
    }

    if (*error==0) {
	function_to_call(data);
	gtk_widget_destroy(update_entry);
	function_to_call = NULL;
	update_entry = NULL;                     
    } else if (*error==2) {
	gtk_widget_destroy(update_entry);
	function_to_call = NULL;
	update_entry = NULL;

    }
    g_free(error);

}

gchar * c_update_entry_get_char_value(GtkEntry * entry){
    gchar * temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
    return temp;
}

gulong c_update_entry_get_gulong_value(GtkEntry *  entry,gint * error){
    gchar * temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
	gulong aux_value=ULONG_MAX;

    if (strcmp(temp,"")) {

	aux_value = strtoul(temp,stop,10);
	if (strcmp(*stop,"")) {
	    *error = 1;
	    c_util_open_error_window (_("Wrong value in the field"));
	}
    } else {
        *error=2;
    }
    g_free(temp);
    g_free(stop);
    return aux_value;
}

glong c_update_entry_get_glong_value(GtkEntry *entry,gint * error){
    gchar * temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
    glong aux_value=LONG_MAX;

    aux_value = strtol(temp,stop,10);
    if (strcmp(*stop,"")) {
	*error = 1;
	c_util_open_error_window (_("Wrong value in the field"));
    }
    g_free(temp);
    g_free(stop);
    return aux_value;
}
gint c_update_entry_get_gint_value(GtkEntry *entry,gint * error){
    gchar * temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));

    gint aux_value = atoi(temp);

    if ((aux_value == 0) && (strcmp(temp,"0"))) {
        *error = 1;
	c_util_open_error_window (_("Wrong value in the field"));
    }

    g_free(temp);
    return aux_value;

}
gdouble c_update_entry_get_double_value(GtkEntry *entry,gint * error){
    gchar * temp = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
    gdouble aux_value = 0.0;


    if (strcmp(temp,"")) {
        u_points_to_commas (temp);
	aux_value = strtod(temp,stop);
	if (strcmp(*stop,"")) {
	    *error = 1;
	    c_util_open_error_window (_("Wrong value in the field"));
	}
    } else {
        *error=2;
    }
    g_free(temp);
    g_free(stop);
    return aux_value;

}

