/* update_entry_control.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#ifndef _UPDATE_ENTRY_CONTROL_
#define _UPDATE_ENTRY_CONTROL_

/**
 * C_GCHAR_TYPE:
 *
 * Cdigo que indica que el tipo del valor que se editar en el entry es un puntero a carcter.
 *
 */
#define C_GCHAR_TYPE 0
/**
 * C_GULONG_TYPE:
 *
 * Cdigo que indica que el tipo del valor que se editar en el entry es un #gulong.
 *
 */
#define C_GULONG_TYPE 1
/**
 * C_GLONG_TYPE:
 *
 * Cdigo que indica que el tipo del valor que se editar en el entry es un #glong.
 *
 */
#define C_GLONG_TYPE 2
/**
 * C_GINT_TYPE:
 *
 * Cdigo que indica que el tipo del valor que se editar en el entry es un #gint.
 *
 */
#define C_GINT_TYPE 3
/**
 * C_GDOUBLE_TYPE:
 *
 * Cdigo que indica que el tipo del valor que se editar en el entry es #gdouble.
 *
 */
#define C_GDOUBLE_TYPE 4


/**
 * open_update_entry:
 * @app1: #Widget que contiene la aplicacin.
 * @label: Texto que se desea que aparezca delante de la entrada de texto.
 * @tipo_entry: Tipo que se espera recibir en la entrada de texto.
 * @f_to_call: Funcin que se llamar al cerrar la ventana pulsando "Aceptar".
 * 
 * Inicializa la ventana de modificacin de una entrada de texto. Esta pantalla se utiliza
 * para confirmar la introduccin de un valor de un tipo esperado para que no se produzcan errores de
 * parseo en la introduccin de un valor de un atributo.
 *
 * Returns:
 * La ventana inicializada.
 */
GtkWidget *c_update_entry_open_window(GtkWidget *app1, gchar * label,gint tipo_entry,gpointer f_to_call);

/**
 * get_char_value:
 * @entry: Entrada de texto de la que se desea obtaner el valor.
 *
 * Obtiene el valor de la entrada de texto en cadena de caracteres y la devuelve.
 *
 * Returns:
 * La cadena de caracteres que se escribi en la entrada de texto.
 */
gchar * c_update_entry_get_char_value(GtkEntry *entry);
/**
 * c_update_entry_get_gulong_value:
 * @entry: Entrada de texto de la que se desea obtaner el valor.
 * @error: Puntero a entero que se utiliza para indicar si se produjo un error de parseo.
 *
 * Obtiene el valor de una entrada de texto, la parsea al tipo gulong y devuelve el valor en
 * en gulong.
 *
 * Returns:
 * El valor de la entrada de texto parseado a tipo gulong.
 */
gulong c_update_entry_get_gulong_value(GtkEntry *entry,gint * error);
/**
 * c_update_entry_get_glong_value:
 * @entry: Entrada de texto de la que se desea obtaner el valor.
 * @error: Puntero a entero que se utiliza para indicar si se produjo un error de parseo.
 *
 * Obtiene el valor de una entrada de texto, la parsea al tipo glong y devuelve el valor en
 * en glong.
 *
 * Returns:
 * El valor de la entrada de texto parseado a tipo glong.
 */
glong c_update_entry_get_glong_value(GtkEntry *entry,gint * error);
/**
 * c_update_entry_get_gint_value:
 * @entry: Entrada de texto de la que se desea obtaner el valor.
 * @error: Puntero a entero que se utiliza para indicar si se produjo un error de parseo.
 *
 * Obtiene el valor de una entrada de texto, la parsea al tipo gint y devuelve el valor en
 * en gint.
 *
 * Returns:
 * El valor de la entrada de texto parseado a tipo gint.
 */
gint c_update_entry_get_gint_value(GtkEntry *entry,gint * error);
/**
 * c_update_entry_get_double_value:
 * @entry: Entrada de texto de la que se desea obtaner el valor.
 * @error: Puntero a entero que se utiliza para indicar si se produjo un error de parseo.
 *
 * Obtiene el valor de una entrada de texto, la parsea al tipo double y devuelve el valor en
 * en double.
 *
 * Returns:
 * El valor de la entrada de texto parseado a tipo gdouble.
 */
gdouble c_update_entry_get_double_value(GtkEntry *entry,gint * error);

/**
 * c_update_entry_set_max_length:
 * @max: Mximo nmero de caracteres que se permiten en el entry.
 *
 * Establece el mximo nmero de caracteres que se permiten el el entry.
 *
 */
void c_update_entry_set_max_length(guint16 max);


#endif

