/* update_delivery_payment_control.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "update_repair_line_control.h"

static GtkWidget *update_repair_lines=NULL;
static GladeXML *xml;
static Repair_line * repair_line_to_modify = NULL;
gint from_screen=-1;

static gchar widget_name[]="modifica_lineas_reparacion_factura";
static gchar url[]="/src/interface/glade/invoice_view.glade";


static void on_cancel_button_clicked(GtkButton *button,gpointer user_data);
static void on_window_destroy(GtkWidget * widget,gpointer user_data);
static void on_accept_button_clicked(GtkButton *button,gpointer user_data);
static void clean_screen(void);

GtkWidget *c_update_repair_line_open_window(GtkWidget *app1, gint from)
{
   gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);
   
  if (update_repair_lines==NULL)
    {
      GtkWidget *dialog;

      xml = glade_xml_new (directory, widget_name);
      dialog = glade_xml_get_widget (xml, widget_name);
      

      glade_xml_signal_connect (xml, "on_cancelar_linea_reparacion_clicked",
        			GTK_SIGNAL_FUNC (on_cancel_button_clicked));
      glade_xml_signal_connect (xml, "on_aceptar_linea_reparacion_clicked",
        			GTK_SIGNAL_FUNC (on_accept_button_clicked));
      glade_xml_signal_connect (xml, "on_modifica_lineas_reparacion_factura_destroy",
        			GTK_SIGNAL_FUNC (on_window_destroy));

      if (from==C_FROM_INVOICE) {
	  GtkWidget * entry;

	  entry = glade_xml_get_widget(xml,"entry_numero_albaran_modificacion");
	  gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_numero_reparacion_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_codigo_ck_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_eurocode_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_descripcion_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_familia_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_precio_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_iva_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_unidades_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);
	  entry = glade_xml_get_widget(xml,"entry_importe_modificacion");
          gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
          gtk_widget_set_sensitive(entry,FALSE);

      }
      update_repair_lines=dialog;
      
    }
  else {
      if (repair_line_to_modify!=NULL) {
	    repair_line_to_modify=NULL;
	    clean_screen();
	}
  }
  from_screen=from;
  g_free(directory);

  return update_repair_lines;
  
}

void c_update_repair_line_load_data(Repair_line * repair_line,gulong delivery_payment_number,gulong repair_number){
    GtkWidget *  entry;
    gchar * temp;
    gdouble price=G_MINDOUBLE;
    gdouble linea_price=G_MINDOUBLE;
    /* repair_line_to_modify will never be liberated */
    repair_line_to_modify = repair_line;
    if (delivery_payment_number!=ULONG_MAX) {
	entry = glade_xml_get_widget(xml,"entry_numero_albaran_modificacion");
	temp = g_strdup_printf("%lu",delivery_payment_number);
	gtk_entry_set_text(GTK_ENTRY(entry),temp);
	g_free(temp);
    }

    if (repair_number!=ULONG_MAX) {
	entry = glade_xml_get_widget(xml,"entry_numero_reparacion_modificacion");
	temp = g_strdup_printf("%lu",repair_number);
	gtk_entry_set_text(GTK_ENTRY(entry),temp);
	g_free(temp);
    }

    if (d_product_get_is_piece (d_repair_line_get_product(repair_line))==TRUE) {
	Piece * piece = (Piece *) d_repair_line_get_product(repair_line);

	entry = glade_xml_get_widget(xml,"entry_eurocode_modificacion");
	temp= d_piece_get_euroCode (piece);
	gtk_entry_set_text(GTK_ENTRY(entry),temp);
	g_free(temp);
    }
    /*
    entry = glade_xml_get_widget(xml,"entry_codigo_ck_modificacion");
    temp= get_codigo_ck_product(d_repair_line_get_product (repair_line));
    gtk_entry_set_text(GTK_ENTRY(entry),temp);
    g_free(temp);
    */

    entry = glade_xml_get_widget(xml,"entry_descripcion_modificacion");
    temp=d_product_get_description (d_repair_line_get_product(repair_line));

    gtk_entry_set_text(GTK_ENTRY(entry),temp);
    g_free(temp);

    entry = glade_xml_get_widget(xml,"entry_familia_modificacion");
    temp=d_product_get_family_code (d_repair_line_get_product(repair_line));

    gtk_entry_set_text(GTK_ENTRY(entry),temp);
    g_free(temp);

    linea_price = d_repair_line_get_price(repair_line);
    price = d_product_get_price (d_repair_line_get_product(repair_line));


    if (price!=G_MINDOUBLE) {
	entry = glade_xml_get_widget(xml,"entry_precio_modificacion");
	temp=g_strdup_printf("%.2f",price);
	gtk_entry_set_text(GTK_ENTRY(entry),temp);
	g_free(temp);
    }

    if (d_repair_line_get_historical_vat_type (repair_line)!= G_MINDOUBLE) {

	entry = glade_xml_get_widget(xml,"entry_iva_modificacion");
	temp=g_strdup_printf("%.2f",d_repair_line_get_historical_vat_type (repair_line));

	gtk_entry_set_text(GTK_ENTRY(entry),temp);
	g_free(temp);
    }


    entry = glade_xml_get_widget(xml,"entry_unidades_modificacion");
    if (d_repair_line_get_quantity (repair_line)!=G_MINDOUBLE)
	temp=g_strdup_printf("%.3f",d_repair_line_get_quantity (repair_line));
    else
        temp= g_strdup("");
    gtk_entry_set_text(GTK_ENTRY(entry),temp);
    g_free(temp);

    if (linea_price!=G_MINDOUBLE) {
	entry = glade_xml_get_widget(xml,"entry_importe_modificacion");
	temp=g_strdup_printf("%.2f",linea_price*d_repair_line_get_quantity (repair_line));
	gtk_entry_set_text(GTK_ENTRY(entry),temp);
	g_free(temp);
    } else if (price!=G_MINDOUBLE) {
	gdouble line_discount =d_repair_line_get_discount (repair_line);
        gdouble line_total = d_repair_line_calculate_total(repair_line);
	if (line_discount<0) {
            line_discount = 0.0;
	}
	entry = glade_xml_get_widget(xml,"entry_importe_modificacion");
        
	if (line_total!=G_MINDOUBLE) {
	    temp=g_strdup_printf("%.2f",line_total);
	    gtk_entry_set_text(GTK_ENTRY(entry),temp);
	    g_free(temp);
	}
    }

    if (d_repair_line_get_discount (repair_line)>0) {
	entry = glade_xml_get_widget(xml,"entry_dto_modificacion");
	temp=g_strdup_printf("%.2f",d_repair_line_get_discount (repair_line));
	gtk_entry_set_text(GTK_ENTRY(entry),temp);
	g_free(temp);
    }
}

static void on_window_destroy(GtkWidget * widget,gpointer user_data) {
    gtk_widget_destroy(update_repair_lines);
    update_repair_lines = NULL;
}

static void on_cancel_button_clicked(GtkButton *button,gpointer user_data) {
    gtk_widget_destroy(update_repair_lines);
    update_repair_lines = NULL;
}

static void on_accept_button_clicked(GtkButton *button,gpointer user_data) {

    if (from_screen==C_FROM_INVOICE) {
        GtkWidget * entry;
	gchar * temp;
	gchar ** stop;
	gdouble aux_discount;
        gint error=0;

        stop = (gchar **) g_malloc0(sizeof(gchar *));

	entry = glade_xml_get_widget(xml,"entry_dto_modificacion");
	temp = g_strdup((gtk_entry_get_text(GTK_ENTRY(entry))));
	u_points_to_commas (temp);

        aux_discount = atof(temp);
//	aux_discount = strtoul(temp,stop,10);
//	if (!strcmp(*stop,"")) {
	    d_repair_line_set_discount (repair_line_to_modify,aux_discount);
//	} else {
//            error = 1;
//	    c_util_open_error_window ("Valor incorrecto en el campo descuento");
//	}
	g_free(temp);
	g_free(stop);
	if (error == 0) {
	    gtk_widget_destroy(update_repair_lines);
	    update_repair_lines = NULL;
            c_invoice_repaint_repair_lines();
	}
    }

}

static void clean_screen() {

    GtkWidget *vbox;

    vbox = GTK_WIDGET(glade_xml_get_widget(xml, widget_name));

    gtk_container_foreach(GTK_CONTAINER(vbox),&c_util_clean_widget,NULL);

}


