/* warehouse_piece_history.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include <gnome.h>
#include <glade/glade.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <glib.h>
#include <stdlib.h>


#include "a-shortcut.h"
#include "support.h"
#include "control_util.h"
#include "warehouse_piece_history.h"
#include "piece_history_model.h"
#include "find_pieces_model.h"
#include "warehouse_model.h"
#include "enterprise_model.h"
#include <../interface/printing/listing-print-wrapper.h>

#define C_TYPE_COLUMN 0
#define C_DATE_COLUMN 1
#define C_DATA_COLUMN 3
#define C_QUANTITY_COLUMN 2

static GtkWidget *open_warehouse=NULL;
static GladeXML *xml;
//static void (*function_to_call)(gpointer value);

static gchar widget_name[]="ventana_historia";
static gchar url[]="/src/interface/glade/store_view.glade";


static void on_print_button_clicked(GtkButton *button, gpointer user_data);
static void on_close_button_clicked(GtkButton *button, gpointer user_data);
static void on_search_button_clicked(GtkButton *button, gpointer user_data);
static ConditionFindHistory * create_condition_from_screen(void);
static void fill_list_with_listing(GtkCList * clist,gchar *** content);

Listing * listing=NULL;

GtkWidget *c_warehouse_history_open_window(void) {
    gchar * directory = g_strconcat(AA_DATA_DIR,url,NULL);

    if (open_warehouse==NULL)
    {
	GtkWidget *dialog;

	xml = glade_xml_new (directory, widget_name);
	dialog = glade_xml_get_widget (xml, widget_name);


	glade_xml_signal_connect (xml, "on_button_imprimir_historia_clicked",
				  GTK_SIGNAL_FUNC (on_print_button_clicked));
	glade_xml_signal_connect (xml, "on_button_cerrar_historia_clicked",
				  GTK_SIGNAL_FUNC (on_close_button_clicked));
	glade_xml_signal_connect (xml, "on_button_buscar_historia_clicked",
				  GTK_SIGNAL_FUNC (on_search_button_clicked));


	open_warehouse=dialog;
    }
    g_free(directory);
    return open_warehouse;
}


static void on_close_button_clicked(GtkButton *button,gpointer user_data) {
    gtk_widget_destroy(open_warehouse);
    open_warehouse = NULL;
}

static void on_print_button_clicked(GtkButton *button,gpointer user_data) {
    if (listing!=NULL) {
	p_listing_wrap_print (listing);
    } else {
        c_util_open_info_window (_("Previously you have to search"));
    }
}

static ConditionFindHistory * create_condition_from_screen() {
    ConditionFindHistory * condition = (ConditionFindHistory * ) g_malloc0(sizeof(ConditionFindHistory));
    gchar * temp_name;

    temp_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_empresa_historia"))));
    condition->enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);

    temp_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_almacen_historia"))));
    condition->warehouse_code = m_warehouse_get_code_from_name (temp_name);
    g_free(temp_name);

    temp_name = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_pieza_historia"))));
    condition->product_code = m_piece_get_product_code_by_eurocode (temp_name);
    g_free(temp_name);

    condition->initial_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget(xml,"fecha_inicio_historia")));
    condition->ending_date = gnome_date_edit_get_date(GNOME_DATE_EDIT(glade_xml_get_widget(xml,"fecha_fin_historia")));

    return condition;
}

void c_warehouse_history_load_data(gulong warehouse_code,gulong enterprise_code,gchar * eurocode) {
        gchar * enterprise_name = NULL;
	gchar * warehouse_name = NULL;
        gchar * eurocode_piece = g_strdup(eurocode);

	enterprise_name = m_enterprise_get_name_from_code (enterprise_code);
        if (enterprise_name!=NULL)
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_empresa_historia")),enterprise_name);

	warehouse_name = m_warehouse_get_name_from_code (warehouse_code);
        if (warehouse_name!=NULL)
	    gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_almacen_historia")),warehouse_name);

        gtk_entry_set_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_pieza_historia")),eurocode_piece);


        if (enterprise_name !=NULL) g_free(enterprise_name);
        if (warehouse_name !=NULL) g_free(warehouse_name);
        if (eurocode_piece !=NULL) g_free(eurocode_piece);
}

static void on_search_button_clicked(GtkButton *button, gpointer user_data) {
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    ConditionFindHistory * condition = create_condition_from_screen();
    PieceHistory ** piece_history = m_piece_history_find_by_condition (condition,tuples);
    GtkWidget * clist = glade_xml_get_widget(xml,"clist_historia");
    GtkWidget * window;
    GdkCursor * cursor;


    window = glade_xml_get_widget (xml, widget_name);
    
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);

    while (gtk_events_pending())
	gtk_main_iteration();

    if (piece_history!=NULL) {
	Listing list;
	gulong    rows = 0;
	gchar * warehouse = NULL;
	gchar * enterprise = NULL;

	if (listing!=NULL) {
	    p_listing_clear (listing);
            listing = NULL;
	}
	listing = (Listing *) g_malloc0(sizeof(Listing));
	warehouse = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_almacen_historia"))));
	enterprise = g_strdup(gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(xml,"entry_pieza_historia"))));

	list.content_title = d_piece_history_get_content_title ();
	list.listing_title = d_piece_history_get_listing_title (enterprise,warehouse);
	list.content       = d_piece_history_array_to_content (piece_history,&rows);
	list.width_of_cols = d_piece_history_get_listing_columns_width ();
	list.alignment_of_cols = d_piece_history_get_listing_columns_alignment ();
	list.n_rows        = rows;
	list.n_columns     = d_piece_history_get_content_size ();
	list.orientation   = 4;
	list.type_of_paper = 2;


	*listing = list;

	fill_list_with_listing(GTK_CLIST(clist),listing->content);
	while (gtk_events_pending())
	    gtk_main_iteration();
        m_piece_history_clear_array (piece_history);
	if (warehouse !=NULL) g_free(warehouse);
	if (enterprise !=NULL) g_free(enterprise);
    } else {
        c_util_open_info_window (_("No result have been found"));
    }

    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
    g_free(tuples);
}

static void fill_list_with_listing(GtkCList * clist,gchar *** content) {
    gint index=0;
    gchar *empty_row[]={"","","",""};
    GtkWidget * window;
    GdkCursor * cursor;

    window = glade_xml_get_widget(xml,"ventana_historia");
    cursor = gdk_cursor_new(GDK_WATCH);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);
    gtk_clist_clear(GTK_CLIST(clist));

    if (content!=NULL) {
	while (content[index]!=NULL) {
	    if (index==30) gtk_clist_freeze ((GtkCList *) clist);
	    gtk_clist_append ((GtkCList *) clist,empty_row);

	    gtk_clist_set_text((GtkCList *) clist, index, C_TYPE_COLUMN, content[index][0]);
	    gtk_clist_set_text((GtkCList *) clist, index, C_DATE_COLUMN, content[index][1]);
	    gtk_clist_set_text((GtkCList *) clist, index, C_DATA_COLUMN, content[index][2]);
	    gtk_clist_set_text((GtkCList *) clist, index, C_QUANTITY_COLUMN, content[index][3]);

	    index++;
	}
    }
    gtk_clist_thaw ((GtkCList *) clist);
    cursor = gdk_cursor_new(GDK_LEFT_PTR);
    gdk_window_set_cursor(GTK_WIDGET(window)->window, cursor);
    gdk_cursor_destroy(cursor);


}


