/* a-shortcut.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "a-shortcut.h"
#include "support.h"

/* Fixed in gnome-print-0.37 */
/* #include <locale.h> */

#include "find_repairs_control.h"
#include "repairs_control.h"
#include "find_stocked_pieces_control.h"
#include "piece_control.h"
#include "exchange_pieces_control.h"
#include "find_delivery_payment_control.h"
#include "invoice_control.h"
#include "find_invoice_control.h"
#include "find_product_cost.h"
#include "listing_operation_control.h"
#include "find_charge_invoice_control.h"
#include "charge_invoice_control.h"
#include "update_repair_line_control.h"
#include "clients_control.h"
#include "order_control.h"
#include "find_order_control.h"
#include "find_charge_delivery_payment_control.h"
#include "insert_cash_operation_control_2.h"
#include "find_cash_movement_control.h"
#include "accounting_control.h"
#include "insert_document_control.h"
#include "find_documents_control.h"
#include "modify_invoice_number.h"

/**
 * I_SHORTCUT_TYPES:
 *
 * Nmero de tipos de accesos directos.
 *
 */
#define I_SHORTCUT_TYPES 11
/**
 * I_NUM_OPTIONS:
 *
 * Nmero de grupos de accesos directos de la barra de tareas.
 *
 */
#define I_NUM_OPTIONS 6
/**
 * I_NUM_ITEMS1:
 *
 * Nmero de accesos directos del primer grupo de accesos directos.
 *
 */
#define I_NUM_ITEMS1 3
/**
 * I_NUM_ITEMS2:
 *
 * Nmero de accesos directos del segundo grupo de accesos directos.
 *
 */
#define I_NUM_ITEMS2 5
#ifndef IS_SHOP
/**
 * I_NUM_ITEMS3:
 *
 * Nmero de accesos directos del tercer grupo de accesos directos.
 *
 */
#define I_NUM_ITEMS3 7
#else
/**
 * I_NUM_ITEMS3:
 *
 * Nmero de accesos directos del tercer grupo de accesos directos.
 *
 */
#define I_NUM_ITEMS3 2
#endif
/**
 * I_NUM_ITEMS4:
 *
 * Nmero de accesos directos del cuarto grupo de accesos directos.
 *
 */
#define I_NUM_ITEMS4 2
/**
 * I_NUM_ITEMS5:
 *
 * Nmero de accesos directos del quinto grupo de accesos directos.
 *
 */
#define I_NUM_ITEMS5 3
/**
 * I_NUM_ITEMS6:
 *
 * Nmero de accesos directos del sexto grupo de accesos directos.
 *
 */
#define I_NUM_ITEMS6 2


/* FIXME: Horrible initialitation. Create a method to take the information about the
 interface from file (change function -> i_shortcut_add_groups): XML */

static GtkWidget *showing=NULL;

gchar *shortcut_types[I_SHORTCUT_TYPES] = {
    N_("introduction:"), N_("search:"), N_("reports:"), N_("listing:"), N_("documents:"),
    N_("interchange:"),N_("clients:"),N_("orders:"),N_("store:"),N_("refresh:"),N_("prices:")
};

gchar *icon_filenames[I_SHORTCUT_TYPES] = {
    "/images/gnome-ccdesktop.png",
    "/images/cdeappmenu.png",
    "/images/gconf-editor.png",
    "/images/listados4.png",
    "/images/gdict.png",
    "/images/inter.png",
    "/images/malehead.png",
    "/images/evolution-tasks.png",
    "/images/almacen.png",
    "/images/manos.png",
    "/images/resources.png",
};
gchar *options[I_NUM_OPTIONS] = {
    N_("Repairs"),
    N_("Invoices"),
    N_("Store"),
    N_("Cash box"),
#ifndef IS_SHOP
    N_("Accounting"),
    N_("Management")
#endif
};
/////////////////////////
gchar *items1[I_NUM_ITEMS1] = {
    N_("Insert clients"),
    N_("Insert repairs"),
    N_("Search repairs"),
};

gint items1_types[I_NUM_ITEMS1] = {6,2,1};

/////////////////////////
gchar *items2[I_NUM_ITEMS2] = {
    N_("Insert invoices"),
    N_("Search for delivery payments"),
    N_("Search for invoices"),
    N_("Insert invoices to pay"),
    N_("Search for invoices to pay"),
};

gint items2_types[I_NUM_ITEMS2] = {0,1,1,0,1};

/////////////////////////
gchar *items3[I_NUM_ITEMS3] = {
#ifndef IS_SHOP
    N_("Insert pieces"),
#endif
    N_("Search for stock pieces"),
    N_("Pieces interchange"),
#ifndef IS_SHOP
    N_("Insert orders"),
    N_("Search for orders"),
    N_("Search for charge delivery payment"),
    N_("Compare prices cost"),
#endif
};

#ifndef IS_SHOP
gint items3_types[I_NUM_ITEMS3] = {0,8,5,7,1,1,10};
#else
gint items3_types[I_NUM_ITEMS3] = {8,5};
#endif

/////////////////////////
gchar *items4[I_NUM_ITEMS4] = {
    N_("Insert cash movement"),
    N_("List cash movement")
};

gint items4_types[I_NUM_ITEMS4] = {0,3};

/////////////////////////
gchar *items5[I_NUM_ITEMS5] = {
    N_("Listing or receipts"),
    N_("Book of VAT"),
    N_("Invoice numbers modification")
};

gint items5_types[I_NUM_ITEMS5] = {3,4,0};

/////////////////////////
gchar *items6[I_NUM_ITEMS6] = {
    N_("Insert documents"),
    N_("Search for documents")
};

gint items6_types[I_NUM_ITEMS6] = {0,3};

/////////////////////////
GdkPixbuf *icon_pixbufs[I_SHORTCUT_TYPES];

/**
 * i_shortcut_init:
 * @shortcut_bar: Barra de accesos directos.
 *
 * Inicializa la barra de accesos directos de la izquierda.
 *
 */
void i_shortcut_init (EShortcutBar *shortcut_bar)
{
    gchar *pathname;
    gint i;
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(shortcut_bar), "app1");
    GtkWidget * vbox  = lookup_widget(GTK_WIDGET(shortcut_bar), "main_container");

    for (i = 0; i < I_SHORTCUT_TYPES; i++) {
      pathname = g_strconcat(AA_DATA_DIR, icon_filenames[i], NULL);
      icon_pixbufs[i] = gdk_pixbuf_new_from_file (pathname);

      g_free(pathname);
    }
    
    i_shortcut_add_groups (E_SHORTCUT_BAR (shortcut_bar));
    
    gtk_signal_connect (GTK_OBJECT (shortcut_bar), "item_selected",
			GTK_SIGNAL_FUNC (i_shortcut_bar_item_selected),
			NULL);
    
    showing = c_repair_open_window(app1, vbox);
}

/**
 * i_shortcut_quit:
 * @window: Ventana a destruir.
 * @event: Evento recibido.
 * @data: Puntero genrico utilizado para guardar datos de usuario.
 *
 * Sale ordenadamente del programa.
 *
 */
void
i_shortcut_quit (GtkWidget *window, GdkEvent *event, gpointer data)
{
    gtk_widget_destroy (window);
    gtk_exit (0);
}

/**
 * i_shortcut_add_groups:
 * @shortcut_bar: Barra de accesos directos.
 *
 * Aade los grupos en los que se divide a aplicacin as como sus elementos a la barra de
 * accesos directos.
 *
 */

void
i_shortcut_add_groups (EShortcutBar *shortcut_bar)
{
    
    //  add_test_group (shortcut_bar, 1, option1);
    i_shortcut_bar_load_items(shortcut_bar,I_NUM_ITEMS1, 
			      items1, items1_types, 0);

    /*  add_test_group (shortcut_bar, 2, option2); */
    i_shortcut_bar_load_items(shortcut_bar,I_NUM_ITEMS2,
			      items2, items2_types, 1);

    /*   add_test_group (shortcut_bar, 3, option3); */
    i_shortcut_bar_load_items(shortcut_bar,I_NUM_ITEMS3,
			      items3, items3_types, 2);

    /*   add_test_group (shortcut_bar, 4, option4); */
    i_shortcut_bar_load_items(shortcut_bar,I_NUM_ITEMS4, 
			      items4, items4_types, 3);

#ifndef IS_SHOP
    /*   add_test_group (shortcut_bar, 5, option5); */
    i_shortcut_bar_load_items(shortcut_bar,I_NUM_ITEMS5, 
			      items5, items5_types, 4);

    /*   add_test_group (shortcut_bar, 6, option6); */
    i_shortcut_bar_load_items(shortcut_bar,I_NUM_ITEMS6, 
			      items6, items6_types, 5);

#endif
    
}

/**
 * i_shortcut_bar_load_items:
 * @shortcut_bar: Barra de accesos directos.
 * @num_items: Nmero de items del grupo que vamos a aadir.
 * @items: Descripcion de cada item que se aadir.
 * @item_types: Tipo de los items a aadir, indica el icono que se usa para representarlos.
 *
 * Carga un grupo de iconos en la barra de acceso del programa.
 *
 */
void
i_shortcut_bar_load_items(EShortcutBar *shortcut_bar, gint num_items, 
			  gchar **items, gint *item_types, gint option)
{
  gchar * option_name = e_utf8_from_charset_string("iso-8859-1",_(options[option]));
  gchar *string_utf8;
  gchar buffer[128];
  gint j;
  gint item_num;
  gint group_num;
  
  group_num = e_shortcut_model_add_group (shortcut_bar->model, -1,
					  option_name);
  
  e_shortcut_bar_set_view_type (shortcut_bar, group_num,
				E_ICON_BAR_LARGE_ICONS);
  
  for (j = 0; j < num_items; j++) {
    string_utf8 = e_utf8_from_charset_string("iso-8859-1", _(items[j]));
    
    sprintf (buffer,string_utf8, 1, j+1);
    
    item_num = e_shortcut_model_add_item (shortcut_bar->model,
					  group_num, -1,
					  shortcut_types[item_types[j]],
					  buffer,
					  icon_pixbufs [item_types[j]]);
    g_free(string_utf8);
  }

  g_free(option_name);
}

/**
 * i_shortcut_bar_item_selected:
 * @shortcut_bar: Barra de accesos directos.
 * @event: Evento que se ha producido.
 * @group_num: Nmero del grupo actual.
 * @item_num: Nmero de item actual.
 *
 * Cambia al dilogo adecuado si se selecciona un elemento de la barra de accesos directos.
 *
 */
void
i_shortcut_bar_item_selected (EShortcutBar *shortcut_bar,
			       GdkEvent *event, gint group_num, gint item_num)
{
    gchar buffer[256];
    GtkWidget * status_bar  = lookup_widget(GTK_WIDGET(shortcut_bar), "appbar1");

    if (event->button.button == 1) {
	sprintf (buffer, _("---- Igalia S.L. ----   Item Selected - %i:%i"),
		 group_num + 1, item_num + 1);
	gnome_appbar_set_status (GNOME_APPBAR (status_bar), buffer);

	i_shortcut_change_dialog (GTK_WIDGET(shortcut_bar),group_num,item_num);

    }
    else i_shortcut_standard_popup (shortcut_bar, event, group_num);
}

/**
 * i_shortcut_standard_popup:
 * @shortcut_bar: Barra de accesos directos.
 * @event: Evento que se ha producido.
 * @group_num: Nmero de grupo en el que se est actualmente.
 *
 * Muestra un men popup, que permite elegir entre iconos grandes e iconos pequeos.
 *
 */
void
i_shortcut_standard_popup (EShortcutBar *shortcut_bar,
		     GdkEvent *event,
		     gint group_num)
{
    GtkWidget *menu, *menuitem;

    /* We don't have any commands if there aren't any groups yet. */
    if (group_num == -1)
	return;

    menu = gtk_menu_new ();

    menuitem = gtk_menu_item_new_with_label (_("Large Icons"));
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (menu), menuitem);
    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
			GTK_SIGNAL_FUNC (i_shortcut_set_large_icons), shortcut_bar);

    menuitem = gtk_menu_item_new_with_label (_("Small Icons"));
    gtk_widget_show (menuitem);
    gtk_menu_append (GTK_MENU (menu), menuitem);
    gtk_signal_connect (GTK_OBJECT (menuitem), "activate",
			GTK_SIGNAL_FUNC (i_shortcut_set_small_icons), shortcut_bar);

    /* Save the group num so we can get it in the callbacks. */
    gtk_object_set_data (GTK_OBJECT (menu), "group_num",
			 GINT_TO_POINTER (group_num));

    /* FIXME: Destroy menu when finished with it somehow? */
    gtk_menu_popup (GTK_MENU (menu), NULL, NULL, NULL, NULL,
		    event->button.button, event->button.time);
}


/**
 * i_shortcut_set_large_icons:
 * @menuitem: Men al que establecern los iconos grandes.
 * @shortcut_bar: Barra de accesos directos.
 *
 * Establece los iconos grandes.
 *
 */
void
i_shortcut_set_large_icons (GtkWidget *menuitem,
		 EShortcutBar *shortcut_bar)
{
    GtkWidget *menu;
    gint group_num;

    menu = menuitem->parent;
    g_return_if_fail (GTK_IS_MENU (menu));

    group_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (menu),
						      "group_num"));

    e_shortcut_bar_set_view_type (shortcut_bar, group_num,
				  E_ICON_BAR_LARGE_ICONS);
}


/**
 * i_shortcut_set_small_icons:
 * @menuitem: Men al que establecern los iconos pequeos.
 * @shortcut_bar: Barra de accesos directos.
 *
 * Establece los iconos pequeos.
 *
 */
void
i_shortcut_set_small_icons (GtkWidget *menuitem,
		 EShortcutBar *shortcut_bar)
{
    GtkWidget *menu;
    gint group_num;

    menu = menuitem->parent;
    g_return_if_fail (GTK_IS_MENU (menu));

    group_num = GPOINTER_TO_INT (gtk_object_get_data (GTK_OBJECT (menu),
						      "group_num"));

    e_shortcut_bar_set_view_type (shortcut_bar, group_num,
				  E_ICON_BAR_SMALL_ICONS);
}

/**
 * i_shortcut_change_dialog:
 * @opened: El widget que permanece actualmente abierto.
 * @group_num: Nmero de grupo al que se desea cambiar.
 * @item_num: Item del grupo que se desea mostrar.
 *
 * Cambia el item seleccionado de la barra de accesos directos al item solicitado del grupo
 * #group_num mostrando tambin la ventana asociada a dicho acceso directo.
 *
 */
void i_shortcut_change_dialog (GtkWidget *opened, gint group_num, gint item_num)
{
    GtkWidget * app1 = lookup_widget(GTK_WIDGET(opened), "app1");
    GtkWidget * vbox  = lookup_widget(GTK_WIDGET(opened), "main_container");
    GtkWidget * shortcut_bar  = lookup_widget(GTK_WIDGET(opened), "shortcut_bar");
    GtkWidget * dialog=NULL;

      /* Fixed in gnome-print-0.37, this line is needed if you want to use 
       *    the program and don't loose your locale when printing or print previewing:
       *        setlocale(LC_ALL,"es_ES@euro");
       */

    switch (group_num) {
    case 0 :
	e_group_bar_set_current_group_num (E_GROUP_BAR (shortcut_bar),0, TRUE);
	switch (item_num) {
	case 1:
	    dialog = c_repair_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	case 0:
	    dialog = c_clients_control_open (app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);

		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	case 2:
	    dialog = c_find_repair_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);

		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	default : if (showing!=NULL) gtk_widget_hide(showing);
	showing=NULL;
	break;
	}
	break;
    case 1 :
	e_group_bar_set_current_group_num (E_GROUP_BAR (shortcut_bar),1, TRUE);
	switch (item_num) {

	case 0:
	    dialog = c_invoice_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;

	case 1:
	    dialog = c_find_delivery_payment_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;

	case 2:
	    dialog = c_find_invoice_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	case 3:
	    dialog = c_charge_invoice_open (app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	case 4:
	    dialog = c_find_charge_invoice_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;

	default : if (showing!=NULL) gtk_widget_hide(showing);
	showing=NULL;
	break;
	}
	break;
    case 2 :
	e_group_bar_set_current_group_num (E_GROUP_BAR (shortcut_bar),2, TRUE);
	switch (item_num) {
#ifndef IS_SHOP
	case 0:
	    dialog = c_piece_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
#endif
#ifndef IS_SHOP
	case 1:
#else
	case 0:
#endif
	    dialog = c_find_stocked_pieces_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
#ifndef IS_SHOP
	case 2:
#else
	case 1:
#endif
	    dialog = c_exchange_piece_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
#ifndef IS_SHOP
	case 3:
	    dialog = c_order_open_window(app1, vbox, NULL);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	case 4:
	    dialog = c_find_order_open_window(app1, vbox, NULL);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	case 5:
	    dialog = c_find_charge_delivery_payment_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
	case 6:
	    dialog = c_find_product_cost_open_window(app1, vbox);

	    if (!(showing==dialog))
	    {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    };
	    break;
#endif
	default : if (showing!=NULL) gtk_widget_hide(showing);
	showing=NULL;
	break;
	}
	break;

    case 3 :
	e_group_bar_set_current_group_num (E_GROUP_BAR (shortcut_bar),3, TRUE);
	switch (item_num)
	{
	case 0:
	    dialog = open_cash_operation_2(app1, vbox);
	    if (!(showing==dialog)) {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    }
	    break;
	case 1:
	    dialog = c_find_cash_movement_open_window(app1, vbox);
	    if (!(showing==dialog)) {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    }
	    break;

	default : if (showing!=NULL) gtk_widget_hide(showing);
	showing=NULL;
	}

	break;

    case 4 :
	e_group_bar_set_current_group_num (E_GROUP_BAR (shortcut_bar),4, TRUE);
	switch (item_num)
	{
	case 0:
	    dialog = c_listing_operation_open_window(app1, vbox);
	    if (!(showing==dialog)) {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    }
	    break;
	case 1:
	    dialog = c_accounting_open_accounting (app1, vbox);
	    if (!(showing==dialog)) {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    }
	    break;
	case 2:
	    dialog = c_modify_invoice_open_window(app1, vbox);
	    if (!(showing==dialog)) {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    }
	    break;

	default : if (showing!=NULL) gtk_widget_hide(showing);
	showing=NULL;
	}

	break;

    case 5 :
	e_group_bar_set_current_group_num (E_GROUP_BAR (shortcut_bar),5, TRUE);
	switch (item_num)
	{
	case 0:
	    dialog = c_document_open_insertion_window(app1, vbox);
	    if (!(showing==dialog)) {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    }
	    break;
	case 1:
	    dialog = c_find_documents_open_window(app1, vbox);
	    if (!(showing==dialog)) {
		if (showing!=NULL) gtk_widget_hide(showing);
		gtk_widget_show(dialog);
		showing=dialog;
	    }
	    break;

	default : if (showing!=NULL) gtk_widget_hide(showing);
	showing=NULL;
	}

	break;

    default : if (showing!=NULL) gtk_widget_hide(showing);
    showing=NULL;
    };
}
