
/* interface.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

#include <gnome.h>
#include <gal/shortcut-bar/e-shortcut-bar.h>
#include <gal/e-paned/e-hpaned.h>
#include <glade/glade.h>

#include "interface.h"
#include "support.h"
#include "a-shortcut.h"
#include "find_repairs_control.h"
#include "model.h"

static void destroy_cb (GtkObject *object, gpointer data);

/**
 * i_create_app:
 *
 * Crea la aplicacin de gestin.
 *
 * Returns:
 * El widget con la aplicacin inicializada.
 */
GtkWidget*
i_create_app (void)
{
  GtkWidget *app1;
  GtkWidget *appbar1;	  
  GtkWidget *hpaned, *shortcut_bar, *vbox;
  EShortcutModel *shortcut_model;

  app1 = gnome_app_new ("ventana_general", "Fisterra: Open Source ERP");
  gtk_window_set_default_size (GTK_WINDOW (app1), 900, 700);
  gtk_window_set_policy (GTK_WINDOW (app1), FALSE, TRUE, FALSE);

  gtk_object_set_data (GTK_OBJECT (app1), "app1", app1);

  /********* Toolbar and menu bars ****************/

  hpaned = e_hpaned_new ();
  gnome_app_set_contents (GNOME_APP (app1), hpaned);
  gtk_widget_show (hpaned);

  /******************************************************************************  
                                      SHORTCUT BAR
   *****************************************************************************/

  shortcut_model = e_shortcut_model_new ();
  
  shortcut_bar = e_shortcut_bar_new ();
  e_shortcut_bar_set_model (E_SHORTCUT_BAR (shortcut_bar),
				  shortcut_model);

  e_paned_pack1 (E_PANED (hpaned), shortcut_bar, FALSE, TRUE);
  gtk_widget_show (shortcut_bar);
  gtk_object_set_data (GTK_OBJECT (app1),"shortcut_bar", shortcut_bar);
  
  e_paned_set_position (E_PANED (hpaned), 100);
  
  vbox = gtk_vbox_new (FALSE, 0);
  e_paned_pack2 (E_PANED (hpaned), vbox, TRUE, TRUE);

  gtk_widget_show (vbox);
  gtk_object_set_data (GTK_OBJECT (app1), "main_container", vbox);
  
  gtk_widget_pop_visual ();
  gtk_widget_pop_colormap ();
  
  /* Load our default icons. */

  i_shortcut_init (E_SHORTCUT_BAR(shortcut_bar));

  appbar1 = gnome_appbar_new (TRUE, TRUE, GNOME_PREFERENCES_NEVER);
  gtk_widget_ref (appbar1);
  gtk_object_set_data_full (GTK_OBJECT (app1), "appbar1", appbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (appbar1);
  gnome_app_set_statusbar (GNOME_APP (app1), appbar1);

  gtk_signal_connect (GTK_OBJECT (app1), "destroy",
                      GTK_SIGNAL_FUNC (destroy_cb),
                      NULL);

  return app1;
}


static void
destroy_cb (GtkObject *object, gpointer data)
{
  m_model_destroy_connection ();
  gtk_main_quit ();
}

