/* main.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "config.h"

#include <glade/glade.h>
#include <gnome.h>

#include "interface.h"
#include "support.h"
#include "model.h"
#include "util.h"
#include "parameter_manager.h"
#include "time.h"
#include "config.h"
#include <signal.h>
#include <stdlib.h>

#define DELETE_LOCK "/borrarlock.pl"

struct sigaction old_sigaction_fpe;
struct sigaction old_sigaction_ill;
struct sigaction old_sigaction_segv;
struct sigaction old_sigaction_int;
struct sigaction old_sigaction_abrt;

static void accion_senhales(int senhal,siginfo_t * info, void * user_data) {
    gchar * dir = g_strconcat(DIR_LOCAL,DELETE_LOCK,NULL);

    g_print(_("\nLock deletion in folder: %s\n"), dir);
    system(dir);
    g_free(dir);

    exit(-1);

}

int
    main (int argc, char *argv[])
{
    GtkWidget *window1;
    struct sigaction my_sigaction;
    sigset_t * my_set = (sigset_t *) g_malloc0(sizeof(sigset_t));

#ifdef ENABLE_NLS
    bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
    textdomain (GETTEXT_PACKAGE);
#endif

    u_parameter_manager_init ();
    gnome_init ("Fisterra: Open Source ERP", VERSION, argc, argv);
    m_model_init (argc,argv);

    free(malloc(10));
    glade_gnome_init ();

    window1 = i_create_app ();
    gtk_widget_show (window1);

    sigemptyset(my_set);
    sigaddset(my_set,SIGSEGV);
    sigaddset(my_set,SIGINT);
    sigaddset(my_set,SIGILL);
    sigaddset(my_set,SIGFPE);
    sigaddset(my_set,SIGABRT);

    my_sigaction.sa_sigaction=accion_senhales;
    my_sigaction.sa_mask=(*my_set);
    my_sigaction.sa_flags = SA_RESTART;
    my_sigaction.sa_restorer = NULL;

    sigaction(SIGFPE,&my_sigaction,&old_sigaction_fpe);
    sigaction(SIGILL,&my_sigaction,&old_sigaction_ill);
    sigaction(SIGSEGV,&my_sigaction,&old_sigaction_segv);
    sigaction(SIGINT,&my_sigaction,&old_sigaction_int);
    sigaction(SIGABRT,&my_sigaction,&old_sigaction_abrt);

    //Init the semaphore which control the access to the log file
    u_mutex_init ();

    gtk_main ();

    g_free(my_set);

    return 0;
}

