/* delivery_payment-print.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "delivery_payment-print.h"
#include "select_fields.h"
#include "invoice.h"

/**
 * p_delivery_payment_print_multipage:
 * @pc: Contexto en el que se aaden los datos a imprimir.
 * @delivery_payment: Albarn de abono a imprimir.
 * @valued: Permite elegir entre imprimir los precios de las lneas o no.
 *
 * Imprime los valores de un albarn generado por una empresa con la plantilla normal en varias pginas. Calcula
 * la lnea en la que se empezar en cada pgina y la pgina que toca imprimir.
 *
 */
void p_delivery_payment_print_multipage (GnomePrintContext *pc,DeliveryPayment * delivery_payment,gboolean valued) {
    gint index_page = 1;
    gchar * temp=NULL;
    gint next_row=0;
    gint number_of_lines = p_delivery_payment_get_number_of_lines (delivery_payment);
    gint number_of_pages=0;

    g_print(_("\nNumber of lines%d\n"),number_of_lines);
    if (number_of_lines%P_NUMBER_OF_LINES!=0) {
	number_of_pages = number_of_lines/P_NUMBER_OF_LINES + 1;
    } else {
	number_of_pages = number_of_lines/P_NUMBER_OF_LINES;
    }


    g_print(_("\nNumber of pages: %d\n"),number_of_pages);
    while (index_page<=(number_of_pages)) {
	temp = g_strdup_printf(_("Invoice, page %d"),index_page);
	gnome_print_beginpage (pc, temp);
	g_free(temp);
	if (index_page == 0) {
	    next_row = 0;
	} else {
	    next_row = P_NUMBER_OF_LINES*(index_page-1);
	}
	p_delivery_payment_print_one_page (pc,delivery_payment,valued,next_row,index_page);
	gnome_print_showpage (pc);
        index_page++;
    }
    gnome_print_context_close(pc);


}

/**
 * p_delivery_payment_print_one_page:
 * @pc: Contexto en el que se aaden los datos a imprimir.
 * @delivery_payment: Albarn de abono a imprimir.
 * @valued: Permite elegir entre imprimir los precios de las lneas o no.
 * @from_line: Lnea del albarn desde la cual se empezar a imprimir en esta pgina.
 * @n_page: Nmero de pgina.
 *
 * Imprime en una pgina los valores de un albarn generado por una empresa con la plantilla normal. La
 * impresin del albarn es igual para todas las pginas excepto las lneas del albarn que se imprimirn
 * desde la lnea #from_line, y el nmero de pgina que ser #n_page.
 *
 */
void p_delivery_payment_print_one_page (GnomePrintContext *pc,DeliveryPayment * delivery_payment,gboolean valued,gint from_line,gint n_page) {
   GnomeFont *font;
   gchar ** total = (gchar **) g_malloc0(sizeof(gchar *));


   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }

   gnome_print_setfont (pc, font);

   {
       double col_width[3] = {75, 110, 80};
       gchar * client_number = p_delivery_payment_get_stringify_client_number (delivery_payment);
       gchar * legal_id = p_delivery_payment_get_stringify_legal_id (delivery_payment);
       //       gchar * page_number = p_delivery_payment_get_stringify_page_number (delivery_payment);
       gchar * page_number = g_strdup_printf("%d",n_page);
     char *contents[6] = {
       _("CLIENT No."), _("Tax code/Legal ID"), _("Page No."),
       client_number, legal_id, page_number
     };

     p_print_complex_table (pc, font, 25, 700, 2, 3, col_width, 18, contents, 2, 0);

   }

   {
       double col_width[3] = {75, 110, 80};
       gchar * date = p_delivery_payment_get_stringify_date (delivery_payment);
       gchar * delivery_payment_number = p_delivery_payment_get_stringify_number (delivery_payment);
       gchar * agent_number = p_delivery_payment_get_stringify_agent_number (delivery_payment);

     char *contents[6] = {
       _("DATE"), _("DEL. PAY. No."), _("AGENT No."),
       date, delivery_payment_number, agent_number
     };

     p_print_complex_table (pc, font, 25, 650, 2, 3, col_width, 18, contents, 2, 0);
   }

   {
       gchar ** post_address = p_delivery_payment_get_stringify_post_address (delivery_payment);
       gchar * post_address_first_line=g_strdup(post_address[0]);
       gchar * post_address_second_line=g_strdup(post_address[1]);
       gchar * post_address_third_line=g_strdup(post_address[2]);
       gchar * post_address_fourth_line =g_strdup(post_address[3]);

       g_strfreev(post_address);
       p_print_box (pc, 325, 700, 250, 86);
       gnome_print_moveto(pc, 335, 680);
       p_print_show (pc, post_address_first_line);
       gnome_print_moveto(pc, 335, 662);
       p_print_show (pc, post_address_second_line);
       gnome_print_moveto(pc, 335, 644);
       p_print_show (pc, post_address_third_line);
       gnome_print_moveto(pc, 335, 626);
       p_print_show (pc, post_address_fourth_line);
   }

   {
       gchar * company_aux = p_delivery_payment_get_stringify_company (delivery_payment);
       gchar * policy_number_aux=p_delivery_payment_get_stringify_policy (delivery_payment);
       gchar * company =g_strconcat(_("Co.: "),company_aux,NULL);
       gchar * policy_number  =g_strconcat(_("PN: "),policy_number_aux,NULL);
       p_print_box (pc, 25, 604, 550, 18);
       gnome_print_moveto(pc, 35, 590); /* FIXME: centrado en altura */
       p_print_show (pc, company);
       gnome_print_moveto(pc, 310, 590); /* FIXME: centrado en altura */
       p_print_show (pc, policy_number);
       g_free(company_aux);
       g_free(policy_number_aux);
   }

   {
       gdouble col_width[7]={93, 240, 43, 43, 43, 38, 50};

       GList * list = p_delivery_payment_get_stringify_lines (delivery_payment);
       GList * list_it;
       gchar ** contents;
       gint index=1;
       gint current_line = 0;
//       guint length = g_list_length(list);


       contents = (gchar **) g_malloc0(((P_NORMAL_TEMPLATE_LINES+1)*7+1)*sizeof(gchar *));

       contents[0]=g_strdup(_("Reference"));
       contents[1]=g_strdup(_("Description"));
       contents[2]=g_strdup(_("Amount"));
       contents[3]=g_strdup(_("Price"));
       contents[4]=g_strdup(_("Coste"));
       contents[5]=g_strdup(_("% Disc"));
       contents[6]=g_strdup(_("Total"));

       list_it=g_list_first(list);
       while ((list_it!=NULL) && (current_line<from_line)) {
	   list_it = g_list_next(list_it);
           current_line++;
       }

       while ((list_it!=NULL) && (index<=P_NORMAL_TEMPLATE_LINES)) {
	   gint i;
	   gchar ** line = (gchar **) list_it->data;
	   for (i=0;i<7;i++) {
	       if ((i==0) || (i==1) || (i==2))
	       {
		   contents[index*7+i] = g_strdup(line[i]);
	       } else {
		   if (valued == FALSE) {
		       contents[index*7+i] = g_strdup("");
		   } else {
		       contents[index*7+i] = g_strdup(line[i]);
		   }
	       }
	   }
	   list_it = g_list_next(list_it);
	   index++;
       }
                                         
     p_print_normal_complex_table (pc, font, 25, 580, P_NORMAL_TEMPLATE_LINES+2, 7, col_width, 23,
			  contents, index, 0);
   }

   if (valued!=FALSE)
       p_delivery_payment_calculate_stringify_total (delivery_payment,total);
   else
       *total = g_strdup("");

   p_print_box (pc, 25, 212, 75, 36); /* FIXME: print_text_centered + de 1 linea */
   p_print_centered_text (pc, font, 25, 198, 75, _("Totals"));
   p_print_centered_text (pc, font, 25, 186, 75, _("VAT"));


   p_print_box (pc, 25, 172, 550, 54);
   gnome_print_moveto(pc, 100, 172);
   gnome_print_lineto(pc, 100, 118);

   {
       gchar * license_number = d_repair_get_stringify_license_number (p_delivery_payment_get_repair (delivery_payment),TRUE);
       gchar * dossier = d_repair_get_stringify_accident_reference (p_delivery_payment_get_repair (delivery_payment),TRUE);
       p_print_box (pc, 310, 212, 265, 36);
       gnome_print_moveto(pc, 320, 198);
       p_print_show (pc, _("License number:"));
       gnome_print_moveto(pc, 420, 198);
       p_print_show (pc, license_number);
       gnome_print_moveto(pc, 320, 182);
       p_print_show (pc, _("File number"));
       gnome_print_moveto(pc, 420, 182);
       p_print_show (pc, dossier);

       g_free(license_number);
       g_free(dossier);
   }

   gnome_print_moveto(pc, 30, 156);
   p_print_show (pc, _("Payment way"));
   gnome_print_moveto(pc, 105, 156);
   gnome_print_moveto(pc, 30, 140);
   p_print_show (pc, _("Payment entity"));
   gnome_print_moveto(pc, 105, 140);
   p_print_show (pc, " ");
   gnome_print_moveto(pc, 30, 122);
   p_print_show (pc, _("Account number"));
   gnome_print_moveto(pc, 105, 122);
   p_print_show (pc, " ");
   
   p_print_box (pc, 25, 114, 550, 18);
   gnome_print_moveto(pc, 100, 114);
   gnome_print_lineto(pc, 100, 96);

   gnome_print_moveto(pc, 30, 98);
   p_print_show (pc, _("Expiries"));
   gnome_print_gsave(pc);                        
   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }
   gnome_print_setfont(pc,font);

   {
       gchar * discount = g_strdup("");
       gchar * total_discount = g_strdup("");
       gdouble col_width[6] = {100, 90, 90, 75, 80, 115};
       gchar * contents[12] = {_("Costs addition"), _("% EP. Disc."), _("% EP. Disc."), _("Subtotal"), _("Total VAT"), _("Invoice Total"),
       *total, discount, total_discount, "", "", ""};

     p_print_complex_table (pc, font, 25, 252, 2, 6, col_width, 18, contents, 2, 0);
   }

   {
     double col_width[2] ={100, 80};
     char *contents[4] ={_("Taxable income"), _("VAT amount"),
			  "",  ""};

     p_print_complex_table (pc, font, 100, 212, 2, 2, col_width, 18, contents, 2, 0);
   }
   
      gnome_print_grestore(pc);


}






