
/* extract_invoice.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "extract_invoice.h"
#include "extract_repair.h"
#include "account_model.h"
#include "invoice_model.h"
#include "glib.h"
#include "delivery_payment_model.h"


/***********************/
/* FIXME: esta funcin debera estar dentro de dataobjects */

static gchar *  g_strconcat_ref (gchar * string1, gchar * string2);
static GList * repair_lines_get_string_lines(Repair_line ** repair_lines,GeneralCondition * general_condition,gchar * n_albaran,gchar * license_number);
static GList * garage_delivery_payment_get_string_lines(DeliveryPayment * delivery_payment,GeneralCondition * general_condition);

/**
 * p_invoice_get_first_repair:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 *
 * Devuelve la primera reparacin de una factura. Si la factura proviene de una reparacin
 * la devuelve, en otro caso obtiene la reparacin del primer albarn.
 *
 * Returns:
 * Devuelve el atributo solicitado.
 *
 */
Repair * p_invoice_get_first_repair (Invoice * invoice)
{
  Repair * repair;

   if (invoice->delivery_payments!=NULL)
     {	
	if (invoice->delivery_payments[0]!=NULL)
	  {
	    repair = d_delivery_payment_get_repair (invoice->delivery_payments[0]);
	  }	
	else 
	  {
	    repair = NULL;
	  }
     }
   
   else if (invoice->repair!=NULL)
     {
	repair = d_invoice_get_repair (invoice);
     }
   
   else
     {
        repair = NULL;
     }   
   return repair;
}


static gchar *  g_strconcat_ref (gchar * string1, gchar * string2)
{   
   gchar * aux;
   
   aux  = g_strconcat(string1, string2, NULL);
   g_free(string1);
   return aux;
}


/*******************************************/

/**
 * p_invoice_get_stringify_client_number:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Delvuelve el nmero de cliente del receptor de la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_client_number (Invoice * invoice,gboolean show)
{
   gulong client_number_gulong;
   gchar * client_number = NULL;


//   repair = p_invoice_get_first_repair (invoice);
//   client_number_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));

   client_number_gulong = d_participant_get_code (d_invoice_get_participant (invoice));
   if (client_number_gulong!=ULONG_MAX)
       client_number = g_strdup_printf("%lu",client_number_gulong);

   if ((client_number!=NULL) && (show!=FALSE)) {
       return client_number;
   } else {
       if (client_number!=NULL)
	   g_free(client_number);
       return g_strdup("");
   }
}


/**
 * p_invoice_get_stringify_legal_id:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el cif/nif del receptor de la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_legal_id (Invoice * invoice,gboolean show)
{
   gchar * legal_id = NULL;
   
   if (d_invoice_get_participant (invoice)!=NULL) {
       legal_id = d_participant_get_legal_id (d_invoice_get_participant (invoice));
   }
   if ((legal_id!=NULL) && (show!=FALSE)) {
       return legal_id;
   } else {
       if (legal_id!=NULL)
	   g_free(legal_id);
       return g_strdup("");
   }
}

/**
 * p_invoice_get_stringify_date:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve la fecha de emisin de la factura en formato aaaa-mm-dd.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_date (Invoice * invoice,gboolean show)
{
   gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   
   date = d_invoice_get_issue_date (invoice);  
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

   if ((timer_char!=NULL) && (show!=FALSE)) {
       return timer_char;
   } else {
       if (timer_char!=NULL)
	   g_free(timer_char);
       return g_strdup("");
   }

}

/**
 * p_invoice_get_stringify_number:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero de la factura con el cdigo del taller antepuesto.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_number (Invoice * invoice,gboolean show)
{
   gulong invoice_number_gulong;
   gchar * invoice_number = NULL;
   gchar * invoice_number_aux = NULL;
   gchar * content_aux;

   content_aux = d_shop_get_group_code (
					 d_invoice_get_shop (invoice)
					);
   invoice_number_gulong =  d_invoice_get_invoice_number (invoice);

   if (invoice_number_gulong!=ULONG_MAX) {
       invoice_number_aux = g_strdup_printf("%lu",invoice_number_gulong);
       invoice_number = g_strconcat(content_aux," ",invoice_number_aux,NULL);
       g_free(invoice_number_aux);
   }
   g_free(content_aux);

   if ((invoice_number!=NULL) && (show!=FALSE)) {
       return invoice_number;
   } else {
       if (invoice_number!=NULL)
	   g_free(invoice_number);
       return g_strdup("");
   }
}

/**
 * p_invoice_get_stringify_work_number:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero de albarn al que se refiere la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_work_number (Invoice * invoice,gboolean show) {
   gulong delivery_payment_number_gulong=ULONG_MAX;
   gchar * delivery_payment_number = NULL;

   if (invoice->delivery_payments!=NULL) {
       if ((invoice->delivery_payments[0]!=NULL) && (invoice->delivery_payments[0]==NULL)) {
           delivery_payment_number_gulong = d_delivery_payment_get_number (invoice->delivery_payments[0]);
       }
   }
   if ((delivery_payment_number_gulong!=ULONG_MAX) && (show!=FALSE)) {
       delivery_payment_number = g_strdup_printf("%lu",delivery_payment_number_gulong);
   } else {
       delivery_payment_number = g_strdup("");
   }
   return delivery_payment_number;

}


/**
 * p_invoice_get_stringify_agent_number:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero de agente del agente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_agent_number (Invoice * invoice,gboolean show)
{
    if ((d_repair_get_agent ((p_invoice_get_first_repair (invoice)))!=NULL) && (show!=FALSE)) {
	if (d_participant_get_number (d_agent_get_participant (d_repair_get_agent (p_invoice_get_first_repair (invoice))))!=NULL) {
	    return d_participant_get_number (d_agent_get_participant (d_repair_get_agent ((p_invoice_get_first_repair (invoice)))));
	} else {
	    return g_strdup(_(" Without agent "));
	}
    } else {
	return g_strdup("");
    }
}

/**
 * p_invoice_get_stringify_post_address:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @general_condition: Permite elegir si se muestran los atributos o no.
 *
 * Devuelve la direccin que se imprimir como direccin de envo.
 *
 * Returns:
 * Devuelve el atributo solicitado en un array de cadena de caracteres, para devolver la direccin separadas
 * en cuatro lneas.
 *
 */
gchar ** p_invoice_get_stringify_post_address (Invoice * invoice,GeneralCondition * general_condition) /* FIXME: 4 LINES */
{
   Repair * repair;
   Participant * participant;   
   gchar **dest_address = NULL;
   gchar * name= NULL;
   gchar * street= NULL;
   gchar * floor= NULL;
   gchar * number= NULL;
   gchar * postal_code= NULL;
   gchar * town= NULL;
   gchar * province= NULL;
   gulong aux_code = ULONG_MAX;
   dest_address = (gchar **) g_malloc0 (sizeof(gchar *)*4);

   repair = p_invoice_get_first_repair (invoice);


   participant = d_invoice_get_participant (invoice);
   if (participant!=NULL) {
//       gchar * temp = d_participant_get_nick (participant);
       //name = u_get_short_name (temp);
       aux_code = d_participant_get_code (participant);
//       name = d_company_get_short_name (NULL,aux_code,temp);
       name = d_participant_get_short_name (participant);
       //g_free(temp);
   }
   if (name == NULL) {
       
       name = g_strdup(" ");
   }
   if (participant!=NULL)
       street = d_participant_get_street (participant);
   if (street == NULL) {
       street = g_strdup("");
   }
   if (participant!=NULL)
       floor = d_participant_get_floor (participant);
   if (floor == NULL) {
       floor = g_strdup("");
   }
   if (participant!=NULL)
       number = d_participant_get_number (participant);
   if (number == NULL) {
       
       number = g_strdup("");
   }
   if (participant!=NULL)
       postal_code = g_strdup_printf("%lu, ", d_participant_get_postal_code (participant)); /* FIXME: liberar el number?? */
   if ((participant==NULL) || (d_participant_get_postal_code (participant) == ULONG_MAX)) {
       if (postal_code!=NULL)
	   g_free(postal_code);
       postal_code = g_strdup("");
   }
   if (participant!=NULL)
       town = d_participant_get_town (participant);
   if (town == NULL) {
       town = g_strdup("");
   }
   if (participant!=NULL)
       province = d_participant_get_province (participant);
   if (province == NULL) {
       province = g_strdup(" ");
   }
   if (general_condition->nick!=FALSE)
       dest_address[0] = name;
   else {
       if (name!=NULL)
	   g_free(name);
       dest_address[0] = g_strdup("");
   }

   if (general_condition->street!=FALSE)
       dest_address[1] = g_strconcat(street, "   ", number, "   ", floor, NULL);
   else
       dest_address[1] = g_strdup("");

   if (general_condition->town!=FALSE)
       dest_address[2] = g_strconcat(postal_code, " " ,town, NULL);
   else
       dest_address[2] = g_strdup("");

   if (general_condition->province!=FALSE)
       dest_address[3] = province;
   else {
       if (province!=NULL)
	   g_free(province);
       dest_address[3] = g_strdup("");
   }
   /* Liberamos las cadenas que han sido duplicadas */
   g_free(street);
   g_free(number);
   g_free(floor);
   g_free(postal_code);
   g_free(town);
   
   return dest_address;
}


/**
 * p_invoice_get_stringify_company:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve la empresa que emiti la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_company (Invoice * invoice,gboolean show)
{
    gchar * result=NULL;
    
    if (d_repair_get_company (p_invoice_get_first_repair (invoice))!=NULL) {
        gchar * temp =d_participant_get_name (d_company_get_participant (d_repair_get_company (p_invoice_get_first_repair (invoice))));
//	result = d_company_get_short_name (d_repair_get_company (p_invoice_get_first_repair (invoice)),ULONG_MAX,temp);
	result = d_participant_get_short_name (d_company_get_participant(d_repair_get_company (p_invoice_get_first_repair (invoice))));
	g_free(temp);
    }

    if ((result !=NULL) && (show!=FALSE)) return result;
    else {
        if (result!=NULL)
	    g_free(result);
	return g_strdup("");
    }
}

/**
 * p_invoice_get_stringify_place:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el municipio en el que se asienta la tienda que emiti la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_place (Invoice * invoice,gboolean show)
{
    gchar * result=NULL;
    
    if (d_invoice_get_shop (invoice)!=NULL) {
	result = d_participant_get_town (d_shop_get_participant (d_invoice_get_shop (invoice)));
    }

    if ((result !=NULL) && (show!=FALSE)) return result;
    else {
        if (result!=NULL)
	    g_free(result);
	return g_strdup("");
    }
}


/**
 * p_invoice_get_stringify_policy:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero de pliza del cliente.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_policy (Invoice * invoice,gboolean show)
{
    gchar * result=NULL;
    if (d_repair_get_policy (p_invoice_get_first_repair (invoice))!=NULL) {
	result = d_policy_get_policy_number(d_repair_get_policy (p_invoice_get_first_repair (invoice)));
    }
    if ((result !=NULL)  && (show!=FALSE)) return result;
    else {
        if (result!=NULL)
	    g_free(result);
	return g_strdup("");
    }
}

static GList * repair_lines_get_string_lines(Repair_line ** repair_lines,GeneralCondition * general_condition,gchar * n_albaran,gchar * license_number)
{  
   gint j;
   GList * list = NULL;
   gchar ** line;
   gdouble quantity;
   gint index_line=0;
   gdouble price;
   gdouble line_price;
   gdouble discount;

   for (j=0; (repair_lines[j])!=NULL; j++)
   {
       gchar * description;
	gchar * description_aux=g_strdup("");
        gchar * aux;
        gchar * par = NULL;
        line = (gchar **) g_malloc0 (sizeof(gchar *)*7); /* 7 columnas de strings */

	if (n_albaran==NULL) {
	    par=")";
	} else {
	    if (!strcmp(n_albaran,""))
		par=")";
            else
		par="";
	}
	if (license_number!=NULL) {
	    if (strcmp(license_number,"")) {
		aux=g_strconcat(description_aux,_(" (LN: "),license_number,par,NULL);
		g_free(description_aux);
		description_aux = aux;
                aux=NULL;
		par=" ";
	    } else {
		par=" (";
	    }

	} else {
	    par=" (";
	}
	if (n_albaran!=NULL)
	    if (strcmp(n_albaran,"")) {
		aux=g_strconcat(description_aux,par,_("DP No.:"),n_albaran,")",NULL);
		g_free(description_aux);
		description_aux = aux;
                aux=NULL;
	    }
	    
	quantity = d_repair_line_get_quantity (repair_lines[j]);
	line_price = d_repair_line_get_price (repair_lines[j]);
	price = d_product_get_price (d_repair_line_get_product(repair_lines[j]));
	discount = d_repair_line_get_discount (repair_lines[j]);
	
	/* 7 is the number of cols in the invoice table for the repair_lines */

        if (discount<0) discount = 0;

	if ((d_product_get_is_piece(d_repair_line_get_product (repair_lines[j]))) && (general_condition->reference!=FALSE)){
	    Piece * piece = (Piece *) d_repair_line_get_product (repair_lines[j]);
	    line[0] = d_piece_get_euroCode (piece);

	} else if (general_condition->reference!=FALSE){
	    line[0] =  g_strdup_printf("%lu",
	    d_product_get_product_code(d_repair_line_get_product (repair_lines[j])));
	} else {
            line[0] = g_strdup("");
	}
	/* Referencia */
	
	description = d_product_get_description(d_repair_line_get_product (repair_lines[j]));
	if (general_condition->description!=FALSE) {
	    if (description!=NULL) {
		g_strup(description);
                g_strstrip(description);
	    }

            if (description!=NULL)
		line[1] = g_strconcat(description,description_aux,NULL);
	    else
		line[1] = g_strconcat(description_aux,NULL);

	    if ((n_albaran!=NULL) || (license_number!=NULL)) {
                g_free(description_aux);
	    }
	} else {
	    line[1] = g_strdup("");
	}
	if (description!=NULL) {
	    g_free(description);
	}
        if (general_condition->quantity!=FALSE)
	    line[2] = g_strdup_printf("%.3f", quantity); /* Cantidad */
	else
	    line[2] = g_strdup("");

	if ((price!=G_MINDOUBLE) && (general_condition->prices!=FALSE)){
            if (line_price==G_MINDOUBLE)
		line[3] = g_strdup_printf("%.2f", price); /* Precio */
            else
		line[3] = g_strdup(""); /* Precio */
	} else {
	    line[3] = g_strdup(""); /* Precio */
	}
	if ((line_price!=G_MINDOUBLE) && (general_condition->total!=FALSE)) {
	    line[4] = g_strdup_printf("%.2f", line_price*quantity); /* Importe */
	} else if ((price!=G_MINDOUBLE) && (general_condition->prices!=FALSE)) {
	    line[4] = g_strdup_printf("%.2f", price*quantity); /* Importe */
	} else {
            line[4] = g_strdup("");
	}
	if ((discount>0) && (general_condition->discounts!=FALSE)) {
            if (line_price==G_MINDOUBLE)
		line[5] = g_strdup_printf("%.2f", discount); /* Dcto. */
	    else
		line[5] = g_strdup("");
	} else {
	    line[5] = g_strdup("");
	}

	if ((line_price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE)) {
            line[6] = g_strdup_printf("%.2f", line_price*quantity);
	} else if ((price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE))  {
	    line[6] = g_strdup_printf("%.2f", price*quantity*(1-(0.01*discount))); /* Total */
	} else {
            line[6] = g_strdup("");
	}

	for (index_line=0;index_line<7;index_line++)
	    if (line[index_line]==NULL) line[index_line] = g_strdup("");
	list = g_list_append(list,line);
        /* creo que non se debera liberar*/
	//g_free(line); /* FIXME: SHOULD FREE ALSO EACH ELEMENT OF THE LINE */
     }
  
  return list;
}
static GList * garage_delivery_payment_get_string_lines(DeliveryPayment * delivery_payment,GeneralCondition * general_condition)
{  
   GList * list = NULL;
   gchar ** line;
   gint index_line=0;
   gdouble line_price;
   gchar * delivery_payment_number=NULL;
   gchar * description;
   gchar ** description_aux = (gchar **) g_malloc0(sizeof(gchar *));
   gchar ** vat_aux = (gchar **) g_malloc0(sizeof(gchar *));
   gchar ** quantity_aux = (gchar **) g_malloc0(sizeof(gchar *));

   line = (gchar **) g_malloc0 (sizeof(gchar *)*7); /* 7 columnas de strings */

   m_invoice_get_data_to_delivery_payment_line(description_aux,vat_aux,quantity_aux);
   line_price = d_delivery_payment_get_garage_total (delivery_payment);
   delivery_payment_number=g_strdup_printf("%lu",d_delivery_payment_get_number (delivery_payment));
   description = g_strconcat(_(*description_aux),delivery_payment_number,NULL);
   g_free(delivery_payment_number);
	/* 7 is the number of cols in the invoice table for the repair_lines */

   line[0] = g_strdup("");
   
   if (general_condition->quantity!=FALSE)
       line[1] = g_strdup(description);
   else
       line[1] = g_strdup("");

   if (description!=NULL) {
	    g_free(description);
   }

   if (general_condition->quantity!=FALSE)
       line[2] = g_strdup(*quantity_aux); /* Cantidad */
   else
       line[2] = g_strdup("");

   if ((line_price!=G_MINDOUBLE) && (general_condition->prices!=FALSE)){
       line[3] = g_strdup_printf("-%.2f", line_price); /* Precio */
   } else {
       line[3] = g_strdup(""); /* Precio */
	}
   if ((line_price!=G_MINDOUBLE) && (general_condition->total!=FALSE)) {
       line[4] = g_strdup_printf("-%.2f", line_price); /* Precio */
   } else {
       line[4] = g_strdup(""); /* Precio */
   }

   line[5] = g_strdup("");

   if ((line_price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE))  {
       line[6] = g_strdup_printf("-%.2f", line_price); /* Precio */
   } else {
       line[6] = g_strdup(""); /* Precio */
   }

   for (index_line=0;index_line<7;index_line++)
       if (line[index_line]==NULL) line[index_line] = g_strdup("");
   list = g_list_append(list,line);

   g_free(*description_aux);
   g_free(*vat_aux);
   g_free(*quantity_aux);
   g_free(description_aux);
   g_free(vat_aux);
   g_free(quantity_aux);
  
   return list;
}


/**
 * p_invoice_get_stringify_lines:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @general_condition: Permite elegir si se muestran los atributos o no..
 * @from_garage: Indica si la factura es para un taller.
 * @normal_template: Plantilla normal.
 *
 * Devuelve las lineas de reparacin de la factura uniendo las de los albaranes y
 * tratando de forma especial las de los albaranes de talleres como talleres de una reparacin
 * los cuales son devueltos como lneas.
 *
 * Returns:
 * Devuelve la lista de lneas de factura.
 *
 */
GList * p_invoice_get_stringify_lines (Invoice * invoice,GeneralCondition * general_condition,gboolean from_garage,gboolean normal_template)
{
   gint i;
   Repair_line ** repair_lines;
   Repair * repair;
   GList * list = NULL;
   GList * list_aux;
   gchar * number_of_delivery_payments=NULL;
   gchar * license_number=NULL;

   if (from_garage==FALSE) {
       if (invoice->delivery_payments!=NULL)
       {

	   for (i=0; invoice->delivery_payments[i]!=NULL; i++)
	   {
	       if (normal_template!=FALSE) {
		   number_of_delivery_payments = p_delivery_payment_get_stringify_number (invoice->delivery_payments[i]);
                   license_number = d_repair_get_stringify_license_number (d_delivery_payment_get_repair (invoice->delivery_payments[i]),general_condition->description);
	       }

	       repair_lines = d_repair_get_repair_lines (d_delivery_payment_get_repair (invoice->delivery_payments[i]));
	       list_aux = repair_lines_get_string_lines (repair_lines,general_condition,number_of_delivery_payments,license_number);
	       list = g_list_concat (list, list_aux);
	   }
       }
       else if (invoice->repair!=NULL)
       {
	   if (normal_template!=FALSE) {
	       license_number = d_repair_get_stringify_license_number (invoice->repair,general_condition->description);
	   }
	   repair = d_invoice_get_repair (invoice);
	   repair_lines = d_repair_get_repair_lines (repair);
	   list = repair_lines_get_string_lines (repair_lines,general_condition,number_of_delivery_payments,license_number);
       }
   } else {
       if (invoice->delivery_payments!=NULL)
       {
	   for (i=0; invoice->delivery_payments[i]!=NULL; i++)
	   {
	       gulong client_code=ULONG_MAX;
	       gulong garage_code=ULONG_MAX;
               gboolean do_add_albaran_line=FALSE;
               Repair * repair = d_delivery_payment_get_repair (invoice->delivery_payments[i]);

	       if (d_repair_get_garage (repair)!=NULL) {
		   if (d_invoice_get_participant (invoice)!=NULL) {
		       client_code = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
		       garage_code = d_participant_get_code (d_invoice_get_participant (invoice));

		       if (client_code == garage_code) {
			   client_code = ULONG_MAX;
			   garage_code =ULONG_MAX;
		       }
		   }
		   else {
		       client_code = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
		       garage_code = d_participant_get_code (d_garage_get_participant (d_repair_get_garage (repair)));
		   }
	       }
	       if (m_delivery_payment_client_is_not_from_garage (garage_code,
						  client_code,
						  d_delivery_payment_get_number (invoice->delivery_payments[i]),
						  d_participant_get_code (d_shop_get_participant (d_repair_get_shop (repair))),
						  d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (repair)))
						 )!=FALSE) {
		   do_add_albaran_line = TRUE;
	       } else
		   do_add_albaran_line = FALSE;

	       if (do_add_albaran_line==FALSE) {
		   if (normal_template!=FALSE) {
		       number_of_delivery_payments = p_delivery_payment_get_stringify_number (invoice->delivery_payments[i]);
		       license_number = d_repair_get_stringify_license_number (d_delivery_payment_get_repair (invoice->delivery_payments[i]),general_condition->description);
		   }
		   repair_lines = d_repair_get_repair_lines (d_delivery_payment_get_repair (invoice->delivery_payments[i]));
		   list_aux = repair_lines_get_string_lines (repair_lines,general_condition,number_of_delivery_payments,license_number);
	       } else
		   list_aux = garage_delivery_payment_get_string_lines(invoice->delivery_payments[i],general_condition);
	       list = g_list_concat (list, list_aux);
	   }
       } else if (invoice->repair!=NULL) {
	   if (normal_template!=FALSE) {
	       license_number = d_repair_get_stringify_license_number (invoice->repair,general_condition->description);
	   }
	   repair = d_invoice_get_repair (invoice);
	   repair_lines = d_repair_get_repair_lines (repair);
	   list = repair_lines_get_string_lines (repair_lines,general_condition,number_of_delivery_payments,license_number);
       }

   }
   return list;
}

/**
 * p_invoice_get_stringify_payment_way:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Delvuelve el modo de pago de una factura de un cliente. El modo de pago puede ser:
 * A plazos o no..
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_payment_way (Invoice * invoice,gboolean show)
{   
   Payment * payment;
   gchar * payment_way = g_strdup(" ");
   
   payment = d_invoice_get_payment (invoice);


   if (d_payment_get_is_hire_purchase (payment))
     payment_way = g_strconcat_ref(payment_way, _("Hire purchase. "));
   else
       payment_way = g_strconcat_ref(payment_way, " ");
   if (show)
       return payment_way;
   else {
       if (payment_way!=NULL)
	   g_free(payment_way);
       return g_strdup("");
   }
}



/**
 * p_invoice_get_stringify_payment_mode:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve la forma de pago de la factura de un cliente. Puede tomar set:
 * por transferencia, por reposicin taln, por domiciliacin bancaria, con tarjeta o
 * en efectivo.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_payment_mode (Invoice * invoice,gboolean show)
{   
   Payment * payment;
   gchar * payment_way = g_strdup(" ");
   
   payment = d_invoice_get_payment (invoice);
    
     
   if (d_payment_get_is_transfer (payment))       
       payment_way = g_strconcat_ref(payment_way, _("By transference. "));

   else if (d_payment_get_is_cheque (payment))
       payment_way = g_strconcat_ref(payment_way, _("Replacement cheque. "));

   else if (d_payment_get_is_direct_debit (payment))
       payment_way = g_strconcat_ref(payment_way,  _("Direct debit. "));

   else if (d_payment_get_is_credit_card (payment))
       payment_way = g_strconcat_ref(payment_way,  _("Credit card. "));

   else
       payment_way = g_strconcat_ref(payment_way,  _("Effective. "));

   if (show)
       return payment_way;
   else {

       if (payment_way!=NULL)
	   g_free(payment_way);
       return g_strdup("");
   }
}

/**
 * p_invoice_get_stringify_payment_entity:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve la/s entidad/es de pago de una factura. Permite devolver hasta 2 entidades.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_payment_entity (Invoice * invoice,gboolean show)
{
    Payment * payment = d_invoice_get_payment (invoice);
    gboolean is_direct_debit = FALSE;
    gboolean is_transfer = FALSE;
    gboolean is_cheque = FALSE;
    gboolean is_credit_card = FALSE;
    GList * list_of_accounts=NULL;
    gchar * result = NULL;
    gchar * result_aux = NULL;
    
    is_direct_debit = d_payment_get_is_direct_debit (payment);
    is_transfer = d_payment_get_is_transfer (payment);
    is_credit_card = d_payment_get_is_credit_card (payment);
    is_cheque = d_payment_get_is_cheque (payment);

    if (is_transfer!=FALSE) {
	GList * list;
        gint index = 0;
	list_of_accounts = m_account_find_by_participant_code (d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice))));
	if (list_of_accounts != NULL) {
	    list = g_list_first(list_of_accounts);
	    while (list != NULL) {
		Account * account = (Account *) list->data;
		if (index<2) {
		    gint bank_code = d_office_get_bank_code (d_account_get_office (account));
                    gchar * bank_code_char=g_strdup_printf("%d",bank_code);
                    gchar * name_aux = m_account_find_bank_by_code (bank_code_char);
		    if (index == 0)
			result_aux = g_strconcat(name_aux,NULL);
		    else
			result_aux = g_strconcat(result,"  /  ",name_aux,NULL);

		    if (result!=NULL) {
			g_free(result);
		    }

		    result = result_aux;
		    result_aux = NULL;
		    g_free(bank_code_char);
                    if (name_aux!=NULL)
			g_free(name_aux);
		}
		list = g_list_next(list);
		index++;
	    }
	} else {
	    result = g_strdup("");
	}
    } else if (is_direct_debit!=FALSE) {
        result = g_strdup("");
    } else if (is_credit_card!=FALSE) {
        result = g_strdup("");
    } else if (is_cheque!=FALSE){
        result = g_strdup("");
    } else {
        result = g_strdup("");
    }

    return result;
}

/**
 * p_invoice_get_stringify_account_number:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero de cuenta en las que puede pagar la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_account_number (Invoice * invoice,gboolean show)
{
    Payment * payment = d_invoice_get_payment (invoice);
    gboolean is_direct_debit = FALSE;
    gboolean is_transfer = FALSE;
    gboolean is_cheque = FALSE;
    gboolean is_credit_card = FALSE;
    gchar * result = NULL;
    gchar * result_aux = NULL;
    GList * list_of_accounts=NULL;
    is_direct_debit = d_payment_get_is_direct_debit (payment);
    is_transfer = d_payment_get_is_transfer (payment);
    is_credit_card = d_payment_get_is_credit_card (payment);
    is_cheque = d_payment_get_is_cheque (payment);

    if ((is_transfer!=FALSE) && (show!=FALSE)) {
	GList * list;
        gint index = 0;
	list_of_accounts = m_account_find_by_participant_code (d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice))));
	if (list_of_accounts != NULL) {
	    list = g_list_first(list_of_accounts);
	    while (list != NULL) {
		Account * account = (Account *) list->data;
		if (index<2) {
		    gchar * cuenta_aux = d_account_get_complete_number (account);
		    if (index == 0)
			result_aux = g_strconcat(cuenta_aux,NULL);
		    else
			result_aux = g_strconcat(result,"  /  ",cuenta_aux,NULL);

		    if (result!=NULL) {
			g_free(result);
		    }
		    if (cuenta_aux!=NULL) {
                        g_free(cuenta_aux);
		    }
		    result = result_aux;
		    result_aux = NULL;
		}
		list = g_list_next(list);
		index++;
	    }
	} else {
	    result = g_strdup("");
	}
    } else if ((is_direct_debit!=FALSE) && (show!=FALSE)) {
        result = g_strdup("");
    } else if ((is_credit_card!=FALSE) && (show!=FALSE)) {
        result = g_strdup("");
    } else if ((is_cheque!=FALSE) && (show!=FALSE)) {
        result = g_strdup("");
    } else {
        result = g_strdup("");
    }

    return result;
}

/**
 * p_invoice_get_stringify_expiries_number:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nmero de vencimientos de las facturas a plazos.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_expiries_number (Invoice * invoice,gboolean show)
{
   gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   
   date = d_payment_get_init_date (d_invoice_get_payment (invoice));
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));
   if ((timer_char!=NULL) && (show!=FALSE))
       return timer_char;
   else
       if (timer_char!=NULL)
	   g_free(timer_char);
       return g_strdup("");
}

/**
 * p_invoice_get_stringify_company_name:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nombre de una empresa del grupo en la que se emiti la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_company_name (Invoice * invoice,gboolean show)
{
    gchar * result=NULL;
    if (d_invoice_get_enterprise (invoice)!=NULL) {
	if (d_invoice_get_vat_type (invoice)<0) {
	    gchar * temp=NULL;
            gulong aux_code=ULONG_MAX;
	    temp = d_participant_get_name (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
            aux_code = d_participant_get_code (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
	    //	    result = d_company_get_short_name (NULL,aux_code,temp);
	    result = d_participant_get_short_name (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
	    g_free(temp);
	} else {
            result = g_strdup("");
	}
    }
    if ((result !=NULL) && (show!=FALSE))
	return result;
    else {
        if (result!=NULL)
	    g_free(result);
	return g_strdup("");
    }
}

/**
 * p_invoice_get_stringify_company_legal_id:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el nif/cif de una empresa del grupo que emiti la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_company_legal_id (Invoice * invoice,gboolean show)
{
    gchar * result=NULL;
    if (d_invoice_get_enterprise (invoice)!=NULL) {
	if (d_invoice_get_vat_type (invoice)<0) {
	    result = d_participant_get_legal_id (d_enterprise_get_participant (d_invoice_get_enterprise (invoice)));
	} else {
            result = g_strdup("");
	}
    }
    if ((result !=NULL) && (show!=FALSE))
	return result;
    else {
        if (result!=NULL)
	    g_free(result);
	return g_strdup("");
    }
}


/**
 * p_invoice_get_stringify_discount:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el porcentaje descuento por pronto pago de la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_discount (Invoice * invoice,gboolean show)
{
    gdouble invoice_discount = d_invoice_get_invoice_discount (invoice);

    if ((invoice_discount<100)  &&  (invoice_discount>0) && (show!=FALSE)){
	return g_strdup_printf("%.2f",invoice_discount);
    } else if (show!=FALSE){
	return g_strdup("0");
    } else {
	return g_strdup("");
    }
}

/**
 * p_invoice_get_stringify_total_discount:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @show: Permite elegir si se muestra el atributo o no.
 *
 * Devuelve el importe del descuento por pronto pago de la factura.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_invoice_get_stringify_total_discount (Invoice * invoice,gboolean show)
{
    gdouble invoice_discount = d_invoice_get_invoice_discount (invoice);
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * subtotal = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
    gchar * discount_char;


    if (((invoice_discount>100) || (invoice_discount<0)) && (show!=FALSE))  {
        discount_char=g_strdup("0");
    } else if (show!=FALSE) {
	gboolean previous_state = d_invoice_get_modified_total (invoice);
	gdouble imp_discount=0;

	if (previous_state !=FALSE) {
	    d_invoice_set_modified_total (invoice,FALSE);
	    d_invoice_calculate_total (subtotal,vat,total,invoice,FALSE);
	    d_invoice_set_modified_total (invoice,previous_state);
	    imp_discount = *subtotal/((100.0-invoice_discount)/100.0)-(*subtotal);
            g_print(_("\nThe 1\n"));
            g_print(_("\nTaxable base %.2f"),*subtotal);
            g_print(_("\nThe VAT %.2f"),*vat);
            g_print(_("\nThe total %.2f"),*total);
            g_print(_("\nThe imp_discount %.2f"),imp_discount);
	} else {
	    d_invoice_calculate_total (subtotal,vat,total,invoice,FALSE);
	    imp_discount = *subtotal/((100.0-invoice_discount)/100.0)-(*subtotal);
            g_print(_("\nThe 2\n"));
            g_print(_("\nThe taxable base %.2f"),*subtotal);
            g_print(_("\nThe VAT %.2f"),*vat);
            g_print(_("\nThe total %.2f"),*total);
            g_print(_("\nThe imp_discount %.2f"),imp_discount);

	}

        discount_char=g_strdup_printf("%.2f",imp_discount);
    } else {
	discount_char=g_strdup("");
    }

   return discount_char;
}

/**
 * p_invoice_calculate_total_data:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @subtotal: Doble puntero a char en el que se rellenar la base imponible de la factura.
 * @vat_char: Doble puntero a char en el que se rellenar el total de iva de la factura.
 * @total_char: Doble puntero a char en el que se rellenar el total de la factura.
 * @general_condition: Permite elegir si se muestran los atributos o no.
 * @from_garage: Indica si la factura es para un taller.
 * 
 * Devuelve los valores totales de la factura: la base imponible, el total del iva de la factura y
 * el total de la factura.
 *
 */
void p_invoice_calculate_total_data (Invoice * invoice,gchar ** subtotal,gchar ** vat_char,gchar ** total_char,GeneralCondition * general_condition,gboolean from_garage)
{
    gdouble * subtotal_gdouble = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));

    if (from_garage==FALSE)
	d_invoice_calculate_total (subtotal_gdouble,vat,total,invoice,FALSE);
    else
	d_invoice_calculate_total_to_garage (subtotal_gdouble,vat,total,invoice,FALSE);

    if (general_condition->subtotal!=FALSE) {
	*subtotal = g_strdup_printf("%.2f",*subtotal_gdouble);
    } else {
        *subtotal = g_strdup("");
    }

    if (general_condition->vat!=FALSE) {
	*vat_char = g_strdup_printf("%.2f",*vat);
    } else {
	*vat_char = g_strdup("");
    }

    if (general_condition->invoice_total!=FALSE) {
	*total_char = g_strdup_printf("%.2f",*total);
    } else {
        *total_char = g_strdup("");
    }
    g_free(vat);
    g_free(subtotal_gdouble);
    g_free(total);

}

/**
 * p_invoice_get_stringify_total_not_modified:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @subtotal: Doble puntero a char en el que se rellenar la base imponible de la factura.
 * @vat_char: Doble puntero a char en el que se rellenar el total de iva de la factura.
 * @total_char: Doble puntero a char en el que se rellenar el total de la factura.
 * @general_condition: Permite elegir si se muestran los atributos o no.
 * @from_garage: Indica si la factura es para un taller.
 *
 * Calcula los valores totales de la factura cuando la factura es por mdulo pactado, pero
 * calculandolo como si no lo fuese. Se utiliza  para la plantilla de Royal nueva que necesita
 * los datos antiguos a pesar de ser por mdigo pactado.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gdouble p_invoice_get_stringify_total_not_modified (Invoice * invoice,gchar ** subtotal,gchar ** vat_char,gchar ** total_char,GeneralCondition * general_condition,gboolean from_garage)
{
    gdouble * subtotal_gdouble = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * vat = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble * total = (gdouble *) g_malloc0(sizeof(gdouble));
    gdouble result=0;
    gboolean modified_subtotal = d_invoice_get_modified_total (invoice);

    d_invoice_set_modified_total (invoice,FALSE);
    if (from_garage==FALSE)
	d_invoice_calculate_total (subtotal_gdouble,vat,total,invoice,FALSE);
    else
	d_invoice_calculate_total_to_garage (subtotal_gdouble,vat,total,invoice,FALSE);
    d_invoice_set_modified_total (invoice,modified_subtotal);

    if (general_condition->subtotal!=FALSE) {
	*subtotal = g_strdup_printf("%.2f",*subtotal_gdouble);
    } else {
        *subtotal = g_strdup("");
    }

    if (general_condition->vat!=FALSE) {
	*vat_char = g_strdup_printf("%.2f",*vat);
    } else {
	*vat_char = g_strdup("");
    }

    if (general_condition->invoice_total!=FALSE) {
	*total_char = g_strdup_printf("%.2f",*total);
    } else {
        *total_char = g_strdup("");
    }
    g_free(vat);
    g_free(total);
    result = *subtotal_gdouble;
    g_free(subtotal_gdouble);
    return result;
}

/**
 * p_invoice_get_number_of_lines:
 * @invoice: Factura generada por una empresa del grupo de la que se extraer el atributo.
 * @from_garage: Indica si la factura es para un taller.
 *
 * Calcula el nmero de lneas de factura de una factura, para poder realizar la
 * impresin de facturas en multipgina.
 *
 * Returns:
 * Nmero total de lneas de factura.
 *
 */
gint p_invoice_get_number_of_lines (Invoice * invoice,gboolean from_garage)
{
   gint i,j;
   Repair_line ** repair_lines;
   Repair * repair;
   gint number_of_lines=0;

   if (from_garage==FALSE) {
       if (invoice->delivery_payments!=NULL)
       {
	   for (i=0; invoice->delivery_payments[i]!=NULL; i++)
	   {
	       repair_lines = d_repair_get_repair_lines (d_delivery_payment_get_repair (invoice->delivery_payments[i]));
	       for (j=0; repair_lines[j]!=NULL; j++) {
                   number_of_lines ++;
	       }
	   }
       }
       else if (invoice->repair!=NULL)
       {
	   repair = d_invoice_get_repair (invoice);
	   repair_lines = d_repair_get_repair_lines (repair);
	   for (j=0; repair_lines[j]!=NULL; j++) {
	       number_of_lines ++;
	   }
       }
   } else {
       if (invoice->delivery_payments!=NULL)
       {
	   for (i=0; invoice->delivery_payments[i]!=NULL; i++)
	   {
	       gulong client_code=ULONG_MAX;
	       gulong garage_code=ULONG_MAX;
               gboolean do_add_albaran_line=FALSE;
               Repair * repair = d_delivery_payment_get_repair (invoice->delivery_payments[i]);

	       if (d_repair_get_garage (repair)!=NULL) {
		   if (d_invoice_get_participant (invoice)!=NULL) {
		       client_code = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
		       garage_code = d_participant_get_code (d_invoice_get_participant (invoice));

		       if (client_code == garage_code) {
			   client_code = ULONG_MAX;
			   garage_code =ULONG_MAX;
		       }
		   }
		   else {
		       client_code = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
		       garage_code = d_participant_get_code (d_garage_get_participant (d_repair_get_garage (repair)));
		   }
	       }
	       if (m_delivery_payment_client_is_not_from_garage (garage_code,
						  client_code,
						  d_delivery_payment_get_number (invoice->delivery_payments[i]),
						  d_participant_get_code (d_shop_get_participant (d_repair_get_shop (repair))),
						  d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (repair)))
						 )!=FALSE) {
		   do_add_albaran_line = TRUE;
	       } else
		   do_add_albaran_line = FALSE;

	       if (do_add_albaran_line==FALSE) {
		   repair_lines = d_repair_get_repair_lines (d_delivery_payment_get_repair (invoice->delivery_payments[i]));
		   for (j=0; repair_lines[j]!=NULL; j++) {
		       number_of_lines ++;
		   }
	       } else
                   number_of_lines++;
	   }
       } else if (invoice->repair!=NULL) {
	   repair = d_invoice_get_repair (invoice);
	   repair_lines = d_repair_get_repair_lines (repair);
	   for (j=0; repair_lines[j]!=NULL; j++) {
	       number_of_lines ++;
	   }
       }

   }
   return number_of_lines;
}



