 /* extract_payment_note.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "extract_payment_note.h"
#include "glib.h"


/***********************/
/* FIXME: esta funcin debera estar dentro de dataobjects */

static GList * repair_lines_get_string_lines(Repair_line ** repair_lines);


/*******************************************/

/**
 * p_payment_note_get_stringify_client_number:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve el nmero de cliente al que se le emitir la nota.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_client_number (PaymentNote * payment_note)
{
   gulong client_number_gulong;
   gchar * client_number = NULL;


//   repair = d_payment_note_get_repair_detail (payment_note);
//   client_number_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));

   client_number_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (d_payment_note_get_repair_detail (payment_note))));
   if (client_number_gulong!=ULONG_MAX)
       client_number = g_strdup_printf("%lu",client_number_gulong);

   if (client_number!=NULL) {
       return client_number;
   } else {
       return g_strdup("");
   }
}

/**
 * p_payment_note_get_stringify_license_number:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve la matrcula del vehculo de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_license_number (PaymentNote * payment_note) {
    gchar * license_number=NULL;
    if (d_repair_get_vehicle (d_payment_note_get_repair_detail (payment_note))!=NULL) {
        license_number = d_vehicle_get_license_number (d_repair_get_vehicle (d_payment_note_get_repair_detail (payment_note)));
    }
    if (license_number!=NULL) {
        return license_number;
    } else {
        return g_strdup("");
    }
}

/**
 * p_payment_note_get_stringify_legal_id:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve el cif/nif del cliente de la reparacin
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_legal_id (PaymentNote * payment_note)
{
   Repair * repair;
   gchar * legal_id = NULL;
   
   repair = d_payment_note_get_repair_detail (payment_note);
   legal_id = d_participant_get_legal_id (d_client_get_participant (d_repair_get_client (repair)));

   if (legal_id!=NULL) {
       return legal_id;
   } else {
       return g_strdup("");
   }
}

/* FIXME: estudiar cmo gestionar facturas multipgina */
/**
 * p_payment_note_get_stringify_page_number:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * NO SE UTILIZA.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_page_number (PaymentNote * payment_note)
{
   return "1";
}

/**
 * p_payment_note_get_stringify_date:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve la fecha de emisin de la nota en formato aaaa-mm-dd.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_date (PaymentNote * payment_note)
{
   gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   
   date = d_payment_note_get_emission_date (payment_note);
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

   if (timer_char!=NULL) {
       return timer_char;
   } else {
       return g_strdup("");
   }

}

/**
 * p_payment_note_get_stringify_get_number:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve el nmero de nota.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_get_number (PaymentNote * payment_note)
{
   gulong payment_note_number_gulong;
   gchar * payment_note_number = NULL;

   payment_note_number_gulong =  d_payment_note_get_note_code (payment_note);

   payment_note_number = g_strdup_printf("%lu",payment_note_number_gulong);


   if (payment_note_number!=NULL) {
       return payment_note_number;
   } else {
       return g_strdup("");
   }
}


/**
 * p_payment_note_get_stringify_post_address:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve la direccin de envo de la nota.
 *
 * Returns:
 * Devuelve el atributo solicitado en un array de cadenas de caracteres, en la cual se
 * devuelven las cuatro lneas de direccin.
 *
 */
gchar ** p_payment_note_get_stringify_post_address (PaymentNote * payment_note) /* FIXME: 4 LINES */
{
   Repair * repair;
   Participant * participant;   
   gchar **dest_address = NULL;
   gchar * name;
     
   dest_address = (gchar **) g_malloc0 (sizeof(gchar *)*1);

   repair = d_payment_note_get_repair_detail (payment_note);


   participant = d_client_get_participant (d_repair_get_client (d_payment_note_get_repair_detail (payment_note)));
   //   name = d_participant_get_nick (participant);
   name = d_participant_get_short_name (participant);
   if (name == NULL) {
       g_free(name);
       name = g_strdup(" ");
   }
   
   dest_address[0] = name;
   
   /* Liberamos las cadenas que han sido duplicadas */
   
   return dest_address;
}


static GList * repair_lines_get_string_lines(Repair_line ** repair_lines)
{  
   gint j;
   GList * list = NULL;
   gchar ** line;
   gdouble quantity;
   gint index_line=0;
   gdouble price;
   gdouble line_price;
   gdouble discount;

   for (j=0; (repair_lines[j])!=NULL; j++)
   {
        gchar * description;
        line = (gchar **) g_malloc (sizeof(gchar *)*7); /* 7 columnas de strings */
	
	quantity = d_repair_line_get_quantity (repair_lines[j]);
	line_price = d_repair_line_get_price (repair_lines[j]);
	price = d_product_get_price (d_repair_line_get_product(repair_lines[j]));
	discount = d_repair_line_get_discount (repair_lines[j]);
	
	/* 7 is the number of cols in the payment_note table for the repair_lines */
	if (discount<0) discount=0;


	if (d_product_get_is_piece(d_repair_line_get_product (repair_lines[j]))) {
	    Piece * piece = (Piece *) d_repair_line_get_product (repair_lines[j]);
	    line[0] = d_piece_get_euroCode (piece);

	} else {
	    line[0] =  g_strdup_printf("%lu",
	    d_product_get_product_code (d_repair_line_get_product(repair_lines[j])));
	}
	/* Referencia */

	description = d_product_get_description (d_repair_line_get_product(repair_lines[j]));

	if (description!=NULL) {
            g_strup(description);
	} else {
            description=g_strdup("");
	}
	line[1] = g_strdup(description);
	g_free(description);
	line[2] = g_strdup_printf("%.3f", quantity); /* Cantidad */
	if (price!=G_MINDOUBLE) {
            if (line_price==G_MINDOUBLE)
		line[3] = g_strdup_printf("%.2f", price); /* Precio */
            else
		line[3] = g_strdup(" "); /* Precio */
	} else {
	    line[3] = g_strdup(" "); /* Precio */
	}
	if (line_price!=G_MINDOUBLE) {
	    line[4] = g_strdup_printf("%.2f", line_price*quantity); /* Importe */
	} else if (price!=G_MINDOUBLE) {
	    line[4] = g_strdup_printf("%.2f", price*quantity); /* Importe */
	} else {
	    line[4] = g_strdup_printf(" "); /* Importe */
	}
	if (discount>0) {
            if (line_price==G_MINDOUBLE)
		line[5] = g_strdup_printf("%.2f", discount); /* Dcto. */
            else
		line[5] = g_strdup(" ");
	} else {
	    line[5] = g_strdup(" ");
	}
	if (line_price==G_MINDOUBLE) {
            if (price!=G_MINDOUBLE)
		line[6] = g_strdup_printf("%.2f", price*quantity*(1-(0.01*discount))); /* Total */
            else
		line[6] = g_strdup_printf(" "); /* Total */
	} else {
            line[6] = g_strdup_printf("%.2f", line_price*quantity);
	}

	for (index_line=0;index_line<7;index_line++)
	    if (line[index_line]==NULL) line[index_line] = g_strdup("");
	list = g_list_append(list,line);
        /* creo que non se debera liberar*/
	//g_free(line); /* FIXME: SHOULD FREE ALSO EACH ELEMENT OF THE LINE */
     }
  
  return list;
}


/**
 * p_payment_note_get_lines:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve las lneas de reparacin de una nota.
 *
 * Returns:
 * La lista de lneas de reparacin.
 *
 */
GList * p_payment_note_get_lines (PaymentNote * payment_note) /* FIXME: totales... */
{
   Repair_line ** repair_lines;
   Repair * repair;
   GList * list = NULL;
   
   repair = d_payment_note_get_repair_detail (payment_note);
   repair_lines = d_repair_get_repair_lines (repair);
   list = repair_lines_get_string_lines (repair_lines);

   return list;
}

/**
 * p_payment_note_get_number_of_lines:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Devuelve el nmero de lneas de reparacin de una nota. Utilizada para calcular las pginas que
 * ocupar la nota.
 *
 * Returns:
 * El total de lneas de reparacin de la nota.
 *
 */
gint p_payment_note_get_number_of_lines (PaymentNote * payment_note) /* FIXME: totales... */
{
   Repair_line ** repair_lines;
   Repair * repair;
   gint number_of_lines=0;
   
   repair = d_payment_note_get_repair_detail (payment_note);
   repair_lines = d_repair_get_repair_lines (repair);
   while (repair_lines[number_of_lines]!=NULL) {
       number_of_lines++;
   }

   return number_of_lines;
}





/**
 * p_payment_note_get_stringify_company_name:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Nombre de la empresa que emiti la nota.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_company_name (PaymentNote * payment_note)
{
    gchar * result=NULL;
    if (d_repair_get_enterprise (d_payment_note_get_repair_detail (payment_note))!=NULL) {
//	result = d_participant_get_name (d_enterprise_get_participant (d_repair_get_enterprise (d_payment_note_get_repair_detail (payment_note))));
	result = d_participant_get_short_name (d_enterprise_get_participant (d_repair_get_enterprise (d_payment_note_get_repair_detail (payment_note))));
    }
    if (result !=NULL)
	return result;
    else
	return g_strdup("");
}

/**
 * p_payment_note_get_stringify_total:
 * @payment_note: Nota de la que se extraer el atributo.
 *
 * Obtiene el total de la nota.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * p_payment_note_get_stringify_total (PaymentNote * payment_note)
{
    gchar * result=NULL;
    gdouble total;
    total = d_payment_note_get_total (payment_note);

    result = g_strdup_printf("%.2f",total);
    return result;
}
