/* extract_repair.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "extract_repair.h"
#include "glib.h"
#include "util.h"
#include "client_model.h"
#include "particular_object.h"

/***********************/
/* FIXME: esta funcin debera estar dentro de dataobjects */

static GList * repair_lines_get_string_lines(Repair_line ** repair_lines,GeneralCondition * general_condition);

/*******************************************/

/**
 * d_repair_get_stringify_client_number:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el nmero de cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_client_number (Repair * repair,gboolean show)
{
   gulong client_number_gulong;
   gchar * client_number = NULL;


//   repair = repair_get_first_repair (repair);
//   client_number_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));

   client_number_gulong = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
   if (client_number_gulong!=ULONG_MAX)
       client_number = g_strdup_printf("%lu",client_number_gulong);

   if ((client_number!=NULL) && (show==TRUE)) {
       return client_number;
   } else {
       if (client_number!=NULL) g_free(client_number);
       return g_strdup("");
   }
}


/**
 * d_repair_get_stringify_legal_id:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Devuelve el cif/nif del cliente de la reparacin
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_legal_id (Repair * repair,gboolean show)
{
   
   gchar * legal_id = NULL;
   
   legal_id = d_participant_get_legal_id (d_client_get_participant (d_repair_get_client (repair)));

   if ((legal_id!=NULL) && (show==TRUE)) {
       return legal_id;
   } else {
       if (legal_id!=NULL) g_free(legal_id);
       return g_strdup("");
   }
}

/* Fecha entrada,salida,siniestro */
/*gchar * d_repair_get_stringify_date (Repair * repair)
{
   gchar *timer_char = (gchar *) g_malloc(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   
   date = get_date_emision_repair(repair);  
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

   if (timer_char!=NULL) {
       return timer_char;
   } else {
       return g_strdup("");
   }

}
*/

/**
 * d_repair_get_stringify_agent_number:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el cdigo del agente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_agent_number (Repair * repair,gboolean show)
{
    if ((d_repair_get_agent (repair)!=NULL) && (show==TRUE)) {
	if (d_participant_get_number (d_agent_get_participant (d_repair_get_agent (repair)))!=NULL) {
	    return d_participant_get_number (d_agent_get_participant (d_repair_get_agent (repair)));
	} else {
	    return g_strdup(_(" Without agent "));
	}
    } else {
	return g_strdup("");
    }
}


/**
 * d_repair_get_stringify_post_address:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @general_condition: Permite decidir si mostrar los atributos o no.
 * 
 * Devuelve la direccin del cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar ** d_repair_get_stringify_post_address (Repair * repair,GeneralCondition * general_condition) /* FIXME: 4 LINES */
{
   Participant * participant;   
   gchar **dest_address = NULL;
   gchar * name;
   gchar * street;
   gchar * floor;
   gchar * numero;
   gchar * postal_code;
   gchar * town;
   gchar * province;
     
      
   dest_address = (gchar **) g_malloc0 (sizeof(gchar *)*4);



   participant = d_client_get_participant (d_repair_get_client (repair));
   //   name = d_participant_get_nick (participant);
   name = d_participant_get_short_name (participant);
   if (name == NULL) {
       name = g_strdup(" ");
   }
   street = d_participant_get_street (participant);
   if (street == NULL) {
       street = g_strdup("");
   }
   floor = d_participant_get_floor (participant);
   if (floor == NULL) {
       floor = g_strdup("");
   }
   numero = d_participant_get_number (participant);   
   if (numero == NULL) {
       numero = g_strdup("");
   }
   postal_code = g_strdup_printf("%lu, ", d_participant_get_postal_code (participant)); /* FIXME: liberar el numero?? */
   if (d_participant_get_postal_code (participant) == ULONG_MAX) {
       if (postal_code!=NULL) g_free(postal_code);
       postal_code = g_strdup("");
   }
   town = d_participant_get_town (participant);
   if (town == NULL) {
       town = g_strdup("");
   }
   province = d_participant_get_province (participant);
   if (province == NULL) {
       province = g_strdup(" ");
   }

   if (general_condition->nick!=FALSE)
       dest_address[0] = name;
   else
       if (name!=NULL) g_free(name);
       dest_address[0] = g_strdup("");

   if (general_condition->street!=FALSE)
       dest_address[1] = g_strconcat(street, "   ", numero, "   ", floor, NULL);
   else
       dest_address[1] = g_strdup("");

   if ((general_condition->postal_code!=FALSE) && (general_condition->town!=FALSE))
       dest_address[2] = g_strconcat(postal_code, town, NULL);
   else
       dest_address[2] = g_strdup("");

   if (general_condition->province!=FALSE)
       dest_address[3] = province;
   else
       if (province!=NULL)
	   g_free(province);
       dest_address[3] = g_strdup("");
   
   /* Liberamos las cadenas que han sido duplicadas */
   g_free(street);
   g_free(numero);
   g_free(floor);
   g_free(postal_code);
   g_free(town);
   
   return dest_address;
}


/**
 * d_repair_get_stringify_company:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el name de la compaa de seguros asociada a la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_company (Repair * repair,gboolean show)
{
    gchar * result=NULL;
    
    if (d_repair_get_company (repair)!=NULL) {
//	result = d_participant_get_name (d_company_get_participant (d_repair_get_company (repair)));
	result = d_participant_get_short_name( d_company_get_participant (d_repair_get_company (repair)));
    }

    if ((result !=NULL) && (show==TRUE)) return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_policy:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el nmero de pliza de seguro del cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_policy (Repair * repair,gboolean show)
{
    gchar * result=NULL;
    if (d_repair_get_policy (repair)!=NULL) {
	result = d_policy_get_policy_number(d_repair_get_policy (repair));
    }
    if ((result !=NULL) && (show==TRUE)) return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}


static GList * repair_lines_get_string_lines(Repair_line ** repair_lines,GeneralCondition * general_condition)
{  
   gint j;
   GList * list = NULL;
   gchar ** line;
   gdouble quantity;
   gint index_line=0;
   gdouble price;
   gdouble line_price;
   gdouble discount;
   gboolean show;

   for (j=0; (repair_lines[j])!=NULL; j++)
   {
        gchar * description;
        line = (gchar **) g_malloc0 (sizeof(gchar *)*7); /* 7 columnas de strings */
	

	quantity = d_repair_line_get_quantity (repair_lines[j]);
	line_price = d_repair_line_get_price (repair_lines[j]);
	price = d_product_get_price (d_repair_line_get_product(repair_lines[j]));
	discount = d_repair_line_get_discount (repair_lines[j]);
	
	/* 7 is the number of cols in the repair table for the repair_lines */
        if (discount<0) discount=0;
        show = general_condition->reference;

	if ((d_product_get_is_piece(d_repair_line_get_product (repair_lines[j]))) && (show==TRUE)){
	    Piece * piece = (Piece *) d_repair_line_get_product (repair_lines[j]);
	    line[0] = d_piece_get_euroCode (piece);

	} else if (show==TRUE){
	    line[0] =  g_strdup_printf("%lu",
	    d_product_get_product_code (d_repair_line_get_product(repair_lines[j])));

	} else {
            line[0] =  g_strdup("");
	}
	/* Referencia */
	description = d_product_get_description (d_repair_line_get_product(repair_lines[j]));

        g_strup(description);
	if (general_condition->description!=FALSE) {
	    line[1] = g_strndup(description,120);
	} else
	    line[1] = g_strdup("");

        if (description!=NULL)
	    g_free(description);
        if (general_condition->quantity!=FALSE)
	    line[2] = g_strdup_printf("%.3f", quantity); /* Cantidad */
	else
	    line[2] = g_strdup("");


	if ((price!=G_MINDOUBLE) && (general_condition->prices!=FALSE)) {
	    line[3] = g_strdup_printf("%.2f", price); /* Precio */
	} else {
	    line[3] = g_strdup(" "); /* Precio */
	}

        show = general_condition->total;
	if ((line_price!=G_MINDOUBLE) && (show==TRUE)) {
	    line[4] = g_strdup_printf("%.2f", line_price*quantity); /* Importe */
	} else if ((price!=G_MINDOUBLE) && (show==TRUE)) {
	    line[4] = g_strdup_printf("%.2f", price*quantity); /* Importe */
	} else {
	    line[4] = g_strdup(" ");
	}

	if ((discount>0) && (general_condition->discounts!=FALSE)) {
	    line[5] = g_strdup_printf("%.2f", discount); /* Dcto. */
	} else {
	    line[5] = g_strdup(" ");
	}
        show = general_condition->line_total;
	if ((line_price!=G_MINDOUBLE) && (show==TRUE)) {
	    line[6] = g_strdup_printf("%.2f", line_price*quantity);
	} else if ((price!=G_MINDOUBLE) && (show==TRUE)) {
	    line[6] = g_strdup_printf("%.2f", price*quantity*(1-(0.01*discount))); /* Total */
	} else {
            line[6] = g_strdup("");
	}

	for (index_line=0;index_line<7;index_line++)
	    if (line[index_line]==NULL) line[index_line] = g_strdup("");
	list = g_list_append(list,line);
        /* creo que non se debera liberar*/
	//g_free(line); /* FIXME: SHOULD FREE ALSO EACH ELEMENT OF THE LINE */
     }
  
  return list;
}


/**
 * d_repair_get_lines:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve la lista de lneas de reparacin.
 *
 * Returns:
 * Devuelve la lista enlazada con las lneas de reparacin.
 *
 */
GList * d_repair_get_lines (Repair * repair,GeneralCondition *  show) /* FIXME: totales... */
{
   Repair_line ** repair_lines;
   GList * list = NULL;

   repair_lines = d_repair_get_repair_lines (repair);
   list = repair_lines_get_string_lines (repair_lines,show);

  return list;
}




/**
 * d_repair_get_stringify_company_name:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el nombre de la empresa en la que se gener la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_company_name (Repair * repair,gboolean show)
{
    gchar * result=NULL;
    if (d_repair_get_enterprise (repair)!=NULL) {
	gchar * temp;
	temp = d_participant_get_name (d_enterprise_get_participant (d_repair_get_enterprise (repair)));
	//	result = d_company_get_short_name (NULL,aux_code,temp);
	result = d_participant_get_short_name (d_enterprise_get_participant (d_repair_get_enterprise (repair)));
	g_free(temp);

    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_company_legal_id:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el legal_id/nif de la empresa en la que se gener la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_company_legal_id (Repair * repair,gboolean show)
{
    gchar * result=NULL;
    if (d_repair_get_enterprise (repair)!=NULL) {
	result = d_participant_get_legal_id (d_enterprise_get_participant (d_repair_get_enterprise (repair)));
    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_entry_date:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la fecha de entrada de la reparacin en formato aaaa-mm-dd.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_entry_date (Repair * repair,gboolean show) {
   gchar *timer_char = (gchar *) g_malloc0(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   
   date = d_repair_get_in_date (repair);
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

   if ((timer_char!=NULL) && (show==TRUE)) {
       return timer_char;
   } else {
       if (timer_char!=NULL) g_free(timer_char);
       return g_strdup("");
   }

}

/**
 * d_repair_get_stringify_entry_hour:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la hora de entrada de la reparacin en formato hh:mm.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_entry_hour (Repair * repair,gboolean show) {
   time_t date;
   gint hour;
   gint minute;
   gchar * hour_char = NULL;
   gchar * minute_char = NULL;

   gchar * result;

   date = d_repair_get_in_date (repair);

   hour = u_get_hour (date);
   minute = u_get_minute (date);
   hour_char = g_strdup_printf("%d",hour);
   if (minute<10) {
       minute_char = g_strdup_printf("0%d",minute);
   } else {
       minute_char = g_strdup_printf("%d",minute);
   }
   result = g_strconcat(hour_char,":",minute_char,NULL);

   if ((result!=NULL) && (show==TRUE)) {
       return result;
   } else {
       if (result!=NULL) g_free(result);
       return g_strdup("");
   }
}

/**
 * d_repair_get_stringify_exit_date:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la fecha de salida de la reparacin en formato aaaa-mm-dd.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_exit_date (Repair * repair,gboolean show) {
   gchar *timer_char = (gchar *) g_malloc0(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   
   date = d_repair_get_out_date (repair);
   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

   if ((timer_char!=NULL) && (show==TRUE)) {
       return timer_char;
   } else {
       if (timer_char!=NULL) g_free(timer_char);
       return g_strdup("");
   }

}

/**
 * d_repair_get_stringify_exit_hour:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la hora de salida de la reparacin en formato hh:mm.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_exit_hour (Repair * repair,gboolean show) {
   time_t date;
   gint hour;
   gint minute;
   gchar * hour_char = NULL;
   gchar * minute_char = NULL;

   gchar * result;

   date = d_repair_get_out_date (repair);

   hour = u_get_hour (date);
   minute = u_get_minute (date);
   hour_char = g_strdup_printf("%d",hour);
   if (minute<10) {
       minute_char = g_strdup_printf("0%d",minute);
   } else {
       minute_char = g_strdup_printf("%d",minute);
   }
   result = g_strconcat(hour_char,":",minute_char,NULL);

   if ((result!=NULL) && (show==TRUE)){
       return result;
   } else {
       if (result!=NULL) g_free(result);
       return g_strdup("");
   }
}



/**
 * d_repair_get_stringify_vehicle_made:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la marca del vehculo de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_vehicle_made (Repair * repair,gboolean show) {
    gchar * result=NULL;
    if (d_repair_get_vehicle (repair)!=NULL) {
	result = d_vehicle_get_make (d_repair_get_vehicle (repair));
    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_vehicle_model:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el modelo del vehculo de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_vehicle_model (Repair * repair,gboolean show) {
    gchar * result=NULL;
    if (d_repair_get_vehicle (repair)!=NULL) {
	result = d_vehicle_get_model (d_repair_get_vehicle (repair));
    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_vehicle_colour:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el color del vehculo de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_vehicle_colour (Repair * repair,gboolean show) {
    gchar * result=NULL;
    if (d_repair_get_vehicle (repair)!=NULL) {
	result = d_vehicle_get_colour (d_repair_get_vehicle (repair));
    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_license_number:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la matrcula de vehculo de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_license_number (Repair * repair,gboolean show) {
    gchar * result=NULL;
    if (d_repair_get_vehicle (repair)!=NULL) {
	result = d_vehicle_get_license_number (d_repair_get_vehicle (repair));
    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_chassis_number:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el nmero de bastidor del vehculo de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_chassis_number (Repair * repair,gboolean show) {
    gchar * result=NULL;
    if (d_repair_get_vehicle (repair)!=NULL) {
	result = d_vehicle_get_chassis_number (d_repair_get_vehicle (repair));
    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_kilemoters:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene los kilmetros del vehculo de la reparacin en el momento de
 * realizar la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_kilemoters (Repair * repair,gboolean show) {
    gchar * result=NULL;
    gulong kms_gulong = ULONG_MAX;
    if (d_repair_get_vehicle (repair)!=NULL) {
	kms_gulong = d_vehicle_get_kilometres (d_repair_get_vehicle (repair));
    }
    if ((kms_gulong != ULONG_MAX) && (show==TRUE))
	return result = g_strdup_printf("%lu",kms_gulong);
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }

}

/**
 * d_repair_get_stringify_holder:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el nombre del titular de la pliza de seguro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_holder (Repair * repair,gboolean show) {
    gchar * holder = NULL;

//    holder = d_participant_get_nick (d_client_get_participant (d_repair_get_client (repair)));
    holder = d_participant_get_short_name (d_client_get_participant (d_repair_get_client (repair)));

    if ((holder!=NULL) && (show==TRUE)) {
	return holder;
    } else {
        if (holder!=NULL) g_free(holder);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_date:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la fecha del siniestro en formato aaaa-mm-dd.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_date (Repair * repair,gboolean show) {
   gchar *timer_char = NULL;
   size_t stored;
   time_t date;

   if (d_repair_get_accident (repair)!=NULL) {
       timer_char = (gchar *) g_malloc0(sizeof(gchar)*100);
       date = d_accident_get_date (d_repair_get_accident (repair));
       stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));
   }

   if ((timer_char!=NULL) && (show==TRUE)) {
       return timer_char;
   } else {
       return g_strdup("");
   }
}

/**
 * d_repair_get_stringify_accident_hour:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la hora en la que se produjo el siniestro, en formato hh:mm.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_hour (Repair * repair,gboolean show) {
   time_t date;
   gint hour;
   gint minute;
   gchar * hour_char = NULL;
   gchar * minute_char = NULL;

   gchar * result=NULL;

   if (d_repair_get_accident (repair)!=NULL) {
       date = d_accident_get_date (d_repair_get_accident (repair));
       hour = u_get_hour (date);
       minute = u_get_minute (date);
       hour_char = g_strdup_printf("%d",hour);
       if (minute<10) {
	   minute_char = g_strdup_printf("0%d",minute);
       } else {
	   minute_char = g_strdup_printf("%d",minute);
       }
       result = g_strconcat(hour_char,":",minute_char,NULL);
   }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }

}

/**
 * d_repair_get_stringify_accident_place:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el lugar en el que se produjo el accidente.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_place (Repair * repair,gboolean show) {
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL) {
	result = d_accident_get_place (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }

}

/**
 * d_repair_get_stringify_accident_mode:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Devuelve el modo en que se produjo el accidente.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_mode (Repair * repair,gboolean show) {
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL) {
	result = d_accident_get_way (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)){
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_damages:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene los daos sufridos por el vehculo en el siniestro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_damages (Repair * repair,gboolean show) {
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL)
	result = d_accident_get_damages (d_repair_get_accident (repair));

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_reference:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve la referencia del sinietro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_reference (Repair * repair,gboolean show) {
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL) {
	result = d_accident_get_accident_reference (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_field:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el ramo del siniestro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_field (Repair * repair,gboolean show){
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL) {
	result = d_accident_get_field (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_province:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el cdigo de provincia en la que se produjo el siniestro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_province (Repair * repair,gboolean show){
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL) {
	result = d_accident_get_province_code (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_reason:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el cdigo de la causa del accidente para la compaa de seguros.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_reason (Repair * repair,gboolean show){
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL) {
	result = d_accident_get_reason_code (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_photo1:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene la primera foto del siniestro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_photo1 (Repair * repair,gboolean show){
    gchar * result = NULL;

    if ((d_repair_get_accident (repair)!=NULL) && (show==TRUE)) {
	result = d_accident_get_photo1 (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_photo2:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene la segunda foto del siniestro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_photo2 (Repair * repair,gboolean show){
    gchar * result = NULL;

    if ((d_repair_get_accident (repair)!=NULL) && (show==TRUE)) {
	result = d_accident_get_photo2 (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_description:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la descripcin del siniestro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_description (Repair * repair,gboolean show){
    gchar * result = NULL;

    if (d_repair_get_accident (repair)!=NULL) {
	result = d_accident_get_description (d_repair_get_accident (repair));
    }

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_receipt:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el nmero del ltimo recibo de la pliza de seguro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_receipt (Repair * repair,gboolean show) {
    gchar * result = NULL;

    if (d_repair_get_receipt (repair)!=NULL)
	result = d_receipt_get_receipt_number (d_repair_get_receipt (repair));

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }

}

/**
 * d_repair_get_stringify_policy_number:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Devuelve el nmero de pliza del seguro. REPETIDO. Ya existe una que se utilizaba en
 * otro sitio. Se debera homogeneizar, eliminando las distintas apariciones de las dos y
 * eliminar una de las dos.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_policy_number (Repair * repair,gboolean show) {
    gchar * result = NULL;

    if (d_repair_get_policy (repair)!=NULL)
	result = d_policy_get_policy_number (d_repair_get_policy (repair));

    if ((result!=NULL) && (show==TRUE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }

}

/**
 * d_repair_get_stringify_validity:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el perodo de validez de la pliza de seguros.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_validity (Repair * repair,gboolean show) {
   gchar *timer_char = (gchar *) g_malloc0(sizeof(gchar)*100);
   size_t stored;
   time_t date;
   gchar *final_timer_char = (gchar *) g_malloc0(sizeof(gchar)*100);
   size_t final_stored;
   time_t final_date;
   gchar * period = NULL;

   if (d_repair_get_receipt (repair)!=NULL) {
       date = d_receipt_get_validity_start (d_repair_get_receipt (repair));
       stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));

       final_date = d_receipt_get_validity_end (d_repair_get_receipt (repair));
       final_stored = strftime(final_timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&final_date));

       period = g_strconcat(timer_char,_(" until ") ,final_timer_char,NULL);
       if (timer_char!=NULL)
	   g_free(timer_char);
       if (final_timer_char!=NULL)
	   g_free(final_timer_char);
   }
   if ((period!=NULL) && (show==TRUE)) {
       return period;
   } else {
       if (period!=NULL) g_free(period);
       return g_strdup("");
   }

}

/**
 * d_repair_get_stringify_nick:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el nombre completo del cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_nick (Repair * repair,gboolean show) {
    gchar * nick = NULL;
   
//   nick = d_participant_get_nick (d_client_get_participant (d_repair_get_client (repair)));
    nick = d_participant_get_short_name (d_client_get_participant (d_repair_get_client (repair)));
   if ((nick!=NULL) && (show==TRUE)) {
       return nick;
   } else {
       if (nick!=NULL) g_free(nick);
       return g_strdup("");
   }

}

/**
 * d_repair_get_stringify_town:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el municipio en el que reside el cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_town (Repair * repair,gboolean show) {
    gchar * town = NULL;
   
   town = d_participant_get_town (d_client_get_participant (d_repair_get_client (repair)));

   if ((town!=NULL) && (show==TRUE)) {
       return town;
   } else {
       if (town!=NULL) g_free(town);
       return g_strdup("");
   }

}

/**
 * d_repair_get_stringify_province:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la provincia en la que reside el cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_province (Repair * repair,gboolean show) {
    gchar * province = NULL;
   
   province = d_participant_get_province (d_client_get_participant (d_repair_get_client (repair)));

   if ((province!=NULL) && (show==TRUE)) {
       return province;
   } else {
       if (province!=NULL) g_free(province);
       return g_strdup("");
   }

}

/**
 * d_repair_get_stringify_telephone:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el telfono del cliente de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_telephone (Repair * repair,gboolean show) {
    gchar * phone = NULL;

    phone = d_participant_get_phone (d_client_get_participant (d_repair_get_client (repair)));

    if ((phone!=NULL) && (show==TRUE)) {
	return phone;
    } else {
	if (phone!=NULL) g_free(phone);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_postal_code:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el cdigo postal de la direccin del cliente.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_postal_code (Repair * repair,gboolean show) {

    gulong postal_code_gulong = ULONG_MAX;

    postal_code_gulong = d_participant_get_postal_code (d_client_get_participant (d_repair_get_client (repair)));

    if ((postal_code_gulong != ULONG_MAX) && (show==TRUE))
	return g_strdup_printf("%lu",postal_code_gulong);
    else
	return g_strdup("");

    
}

/**
 * d_repair_get_stringify_name:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @general_condition: Permite decidir si mostrar los atributos o no.
 * 
 * Obtiene el nombre del cliente separado en dos lneas en caso de nombre y apellidos.
 *
 * Returns:
 * Devuelve el atributo solicitado en un array de cadenas de caracteres.
 *
 */
gchar ** d_repair_get_stringify_name (Repair * repair,GeneralCondition * general_condition) {
    gchar ** result;
    gulong code = d_participant_get_code (d_client_get_participant (d_repair_get_client (repair)));
    gint type = m_client_get_type (code);

    result = (gchar**) g_malloc0(sizeof(gchar *)*3);
    if (type==M_CLIENT_PERSON_CODE) {
	ParticularObject * auxiliar_data;
	gchar * name=d_participant_get_short_name (d_client_get_participant (d_repair_get_client (repair)));
	gchar * surname1=NULL;
	gchar * surname2=NULL;
	auxiliar_data= (ParticularObject *) m_client_get_data (M_CLIENT_PERSON_CODE,code);
	surname1=d_particular_get_first_surname (auxiliar_data);
	surname2=d_particular_get_second_surname (auxiliar_data);
	if (general_condition->name!=FALSE) {
	    result[0]=g_strdup(name);
	} else {
            result[0]=g_strdup("");
	}
	if (general_condition->surnames!=FALSE) {
	    result[1]=g_strconcat(surname1," ",surname2,NULL);
	} else {
            result[1]=g_strdup("");
	}
    } else {
	if (general_condition->name!=FALSE) {
	    result[0] = d_participant_get_short_name (d_client_get_participant (d_repair_get_client (repair)));
	} else {
            result[0]=g_strdup("");
	}
    }
    return result;
}

/**
 * d_repair_get_stringify_address:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la direccin del cliente en una sola lnea.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_address (Repair * repair,gboolean show){
    gchar * aux1;
    gchar * aux2;
    gchar * aux3;
    gchar * result = NULL;

    if (d_repair_get_client (repair)!=NULL) {
	aux1= d_participant_get_street (d_client_get_participant (d_repair_get_client (repair)));
	if (aux1==NULL) {
	    aux1 = g_strdup("");
	}
	aux2= d_participant_get_number (d_client_get_participant (d_repair_get_client (repair)));
	if (aux2==NULL) {
	    aux2 = g_strdup("");
	}
	aux3= d_participant_get_floor (d_client_get_participant (d_repair_get_client (repair)));
	if (aux3==NULL) {
	    aux3 = g_strdup("");
	}

	result = g_strconcat(aux1," ",aux2," ",aux3,NULL);
	g_free(aux1);
	g_free(aux2);
	g_free(aux3);
    }
    if ((result!=NULL) && (show!=FALSE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}


/**
 * d_repair_get_stringify_garage_name:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el nombre del taller de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_name (Repair * repair,gboolean show) {
    gchar * result = NULL;


    if (d_repair_get_enterprise (repair)!=NULL)
	result = d_participant_get_name (d_enterprise_get_participant (d_repair_get_enterprise (repair)));
//	result = d_participant_get_short_name (d_enterprise_get_participant (d_repair_get_enterprise (repair)));

    if ((result!=NULL)  && (show!=FALSE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_garage_address:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene la direccin del taller de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_address (Repair * repair,gboolean show){
    gchar * aux1;
    gchar * aux2;
    gchar * aux3;
    gchar * result = NULL;

    if (d_repair_get_shop (repair)!=NULL) {
	aux1= d_participant_get_street (d_shop_get_participant (d_repair_get_shop (repair)));
	if (aux1==NULL) {
	    aux1 = g_strdup("");
	}
	aux2= d_participant_get_number (d_shop_get_participant (d_repair_get_shop (repair)));
	if (aux2==NULL) {
	    aux2 = g_strdup("");
	}
	aux3= d_participant_get_floor (d_shop_get_participant (d_repair_get_shop (repair)));
	if (aux3==NULL) {
	    aux3 = g_strdup("");
	}

	result = g_strconcat(aux1," ",aux2," ",aux3,NULL);
	g_free(aux1);
	g_free(aux2);
	g_free(aux3);
    }
    if ((result!=NULL) && (show!=FALSE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_garage_place:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el lugar en el que reside el taller.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_place (Repair * repair,gboolean show){
    gchar * result = NULL;
/*
    if (d_repair_get_garage (repair)!=NULL)
	result = d_participant_get_town (d_garage_get_participant (d_repair_get_garage (repair)));
	*/
    if (d_repair_get_shop (repair)!=NULL)
	result = d_participant_get_town (d_shop_get_participant (d_repair_get_shop (repair)));

    if ((result!=NULL) && (show!=FALSE))  {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_garage_postal_code:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el cdigo postal del taller de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_postal_code (Repair * repair,gboolean show){
    gchar * result = NULL;
    gulong postal_code=ULONG_MAX;

    /*
    if (d_repair_get_garage (repair)!=NULL) {
	postal_code = d_participant_get_postal_code (d_garage_get_participant (d_repair_get_garage (repair)));
	}
        */
    if (d_repair_get_shop (repair)!=NULL) {
	postal_code = d_participant_get_postal_code (d_shop_get_participant (d_repair_get_shop (repair)));
	}

    if ((postal_code!=ULONG_MAX)  && (show!=FALSE)) {
        result = g_strdup_printf("%lu",postal_code);
    } else {
	if (result!=NULL) g_free(result);
	result = g_strdup("");
    }
    return result;
}

/**
 * d_repair_get_stringify_garage_telephone:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el telfono del taller de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_telephone (Repair * repair,gboolean show){
    gchar * result = NULL;

/*    if (d_repair_get_garage (repair)!=NULL)
	result = d_participant_get_phone (d_garage_get_participant (d_repair_get_garage (repair)));
	*/
    if (d_repair_get_shop (repair)!=NULL)
	result = d_participant_get_phone (d_shop_get_participant (d_repair_get_shop (repair)));

    if ((result!=NULL) && (show!=FALSE))  {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_garage_email:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene la direccin de correo del taller.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_email (Repair * repair,gboolean show){
    gchar * result = NULL;

    if (d_repair_get_shop (repair)!=NULL)
	result = d_participant_get_email (d_shop_get_participant (d_repair_get_shop (repair)));

    if ((result!=NULL) && (show!=FALSE))  {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_garage_legal_id:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el nif/legal_id del taller de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_legal_id (Repair * repair,gboolean show){
    gchar * result = NULL;

    if (d_repair_get_enterprise (repair)!=NULL)
	result = d_participant_get_legal_id (d_enterprise_get_participant (d_repair_get_enterprise (repair)));

    if ((result!=NULL) && (show!=FALSE))  {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_garage_fax:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el nmero de fax del taller de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_garage_fax (Repair * repair,gboolean show){
    gchar * result = NULL;

    if (d_repair_get_shop (repair)!=NULL)
	result = d_participant_get_fax (d_shop_get_participant (d_repair_get_shop (repair)));

    if ((result!=NULL)  && (show!=FALSE)) {
	return result;
    } else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_description_on_many_lines:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene la descripcin del accidente en varias lneas, como mximo 8 lneas.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar ** d_repair_get_stringify_accident_description_on_many_lines (Repair * repair,gboolean show){
    gchar ** result = NULL;
    gchar * description;
    gchar * description_aux;
    gchar * beginning;
    gchar * final;
    gchar * line;
    gint length;
    gint how_many=30;
    gint number=0;

    if (d_repair_get_accident (repair)!=NULL) {
	description_aux= d_accident_get_description (d_repair_get_accident (repair));
    } else {
        description_aux=NULL;
    }
    if (description_aux!=NULL) {
	length = strlen(description_aux);
    } else {
	length=0;
    }
    if (length<240) {
	description = u_fill_n_spaces (description_aux,(240-length));
    } else
	description = g_strdup(description_aux);
    g_free(description_aux);

    result = (gchar **) g_malloc0(sizeof(gchar*)*8);
    beginning=description;
    final=description+sizeof(gchar)*30;

    while (number<8) {
	how_many =30;
	while ((*final!=' ') && (how_many>0)) {
	    final = final - sizeof(gchar);
	    how_many = how_many -1;
	}
	line = g_strndup(beginning,how_many);
	if (show!=FALSE) {
	    result[number]=g_strdup(line);
            g_free(line);
	} else {
 	    result[number]=g_strdup("");
	}
	beginning = final;
	final = beginning + sizeof(gchar)*30;
	number++;
    }

    return result;
}

/**
 * d_repair_get_stringify_repairer_code:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el cdigo de centro reparador de la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_repairer_code (Repair * repair,gboolean show) {
    gchar * result=NULL;
    if (d_repair_get_shop (repair)!=NULL) {
	if (d_repair_get_company (repair)!=NULL) {
	    result = d_shop_get_repairer_code (d_repair_get_shop (repair));
	} else {
	    result = g_strdup(" ");
	}

    }
    if ((result == NULL) || (show==FALSE)) {
        if (result != NULL) g_free(result);
        result = g_strdup("");
    }
    return result;
}

/**
 * d_repair_get_stringify_repairer_data:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el nombre de la tienda en la que se genera la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_repairer_data (Repair * repair,gboolean show) {
    gchar * result=NULL;

    if ((d_repair_get_shop (repair)!=NULL) && (show!=FALSE)) {
	if (d_shop_get_group_code (d_repair_get_shop (repair))!=NULL) {
	    result = d_participant_get_short_name (d_enterprise_get_participant (d_repair_get_enterprise (repair)));
	} else {
	    result = d_participant_get_short_name (d_shop_get_participant (d_repair_get_shop (repair)));
	}
    } else {
	result = g_strdup("");
    }
    return result;
}

/**
 * d_repair_get_stringify_mediator:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el la sucursal mediadora de la pliza de seguro.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_mediator (Repair * repair,gboolean show) {
    gchar * result = NULL;

    if (d_repair_get_receipt (repair)!=NULL) 
	result = d_receipt_get_subsidiary_mediator (d_repair_get_receipt (repair));
    if ((result!=NULL) && (show!=FALSE)) {
	return result;
    } else {
        if (result != NULL) g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_product_lines:
 * @repair_lines: Lineas de reparacin de las que se obtendrn los valores a imprimir.
 * @general_condition: Permite decidir si mostrar los atributos o no.
 *
 * Obtiene las lneas de reparacin para las plantillas Royal con solo los productos.
 * Las lneas se devuelven un array de como cadenas de caracteres con los datos necesarios.
 *
 * Returns:
 * Devuelve una lista de lneas de reparacin (en cadenas de caracteres) con solo productos.
 *
 */
GList * d_repair_get_stringify_product_lines (Repair_line ** repair_lines,GeneralCondition * general_condition)
{
   gint j;
   GList * list = NULL;
   gchar ** line;
   gchar ** line2;
   gdouble quantity;
   gint index_line=0;
   gdouble price;
   gdouble line_price;
   gdouble discount;
   gchar * temp_desc=NULL;
   for (j=0; (repair_lines[j])!=NULL; j++)
   {
        gchar * description;
        line = (gchar **) g_malloc (sizeof(gchar *)*7); /* 7 columnas de strings */

	if (!d_product_get_is_man_power(d_repair_line_get_product (repair_lines[j]))) {
	    quantity = d_repair_line_get_quantity (repair_lines[j]);
	    line_price = d_repair_line_get_price (repair_lines[j]);
	    price = d_product_get_price (d_repair_line_get_product(repair_lines[j]));
	    discount = d_repair_line_get_discount (repair_lines[j]);

	    /* 7 is the number of cols in the invoice table for the repair_lines */

            /* Referencia */
	    if (d_product_get_is_piece(d_repair_line_get_product (repair_lines[j]))) {
		Piece * piece = (Piece *) d_repair_line_get_product (repair_lines[j]);
		line[0] = d_piece_get_euroCode (piece);

	    } else {
		line[0] =  g_strdup_printf("%lu",
		d_product_get_product_code (d_repair_line_get_product(repair_lines[j])));
	    }
	    
	    if (general_condition->material_description!=FALSE) {
		description = d_product_get_description (d_repair_line_get_product(repair_lines[j]));

		g_strup(description);
		line[1] = g_strndup(description,37);
                if (description!=NULL)
		    g_free(description);
	    } else {
                line[1] = g_strdup("");
	    }
	    if (general_condition->material_quantity)
		line[2] = g_strdup_printf("%.3f", quantity); /* Cantidad */
	    else
                line[2] = g_strdup("");

	    if ((price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE)) {
		line[3] = g_strdup_printf("%.2f", price); /* Precio */
	    } else {
		line[3] = g_strdup(" "); /* Precio */
	    }
	    if ( (line_price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE)) {
		line[4] = g_strdup_printf("%.2f", line_price*quantity); /* Importe */
	    } else if ((price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE)) {
		line[4] = g_strdup_printf("%.2f", price*quantity); /* Importe */
	    } else {
		line[4] = g_strdup("");
	    }

	    if (discount>0) {
		line[5] = g_strdup_printf("%.2f", discount); /* Dcto. */
	    } else {
		line[5] = g_strdup(" ");
	    }
	    if ( (line_price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE)) {
		line[6] = g_strdup_printf("%.2f", line_price*quantity);
	    } else if ((price!=G_MINDOUBLE) && (general_condition->line_total!=FALSE)) {
		line[6] = g_strdup_printf("%.2f", price*quantity*(1-(0.01*discount))); /* Total */
	    } else {
                line[6] = g_strdup("");
	    }
	    for (index_line=0;index_line<7;index_line++)
		if (line[index_line]==NULL) line[index_line] = g_strdup("");
	    list = g_list_append(list,line);

	    temp_desc = d_product_get_description (d_repair_line_get_product(repair_lines[j]));

	    g_strstrip(temp_desc);
	    if (strlen(temp_desc)>37)  {
		line2 = (gchar **) g_malloc (sizeof(gchar *)*7); /* 7 columnas de strings */
		line2[0]=g_strdup("");
		if (general_condition->material_description!=FALSE) {
		    g_strup(temp_desc);
		    line2[1]=u_get_substring (temp_desc,37,74);
		} else {
		    line2[1]=g_strdup("");
		}
		line2[2]=g_strdup("");
		line2[3]=g_strdup("");
		line2[4]=g_strdup("");
		line2[5]=g_strdup("");
		line2[6]=g_strdup("");
		list = g_list_append(list,line2);
	    }
	    /* creo que non se debera liberar*/
	    //g_free(line); /* FIXME: SHOULD FREE ALSO EACH ELEMENT OF THE LINE */
	}
   }

  return list;
   }

/**
 * d_repair_get_stringify_man_power_lines:
 * @repair_lines: Lineas de reparacin de las que se obtendrn los valores a imprimir.
 * @general_condition: Permite decidir si mostrar los atributos o no.
 * @royal_nueva: Indica si la plantilla es nueva o no.
 *
 * Obtiene las lneas de la reparacin para las plantillas Royal con solo las manos de obra.
 * Las lneas se devuelven un array de como cadenas de caracteres con los datos necesarios.
 *
 * Returns:
 * Devuelve una lista de lneas de reparacin (en cadenas de caracteres) con solo las manos de obra.
 *
 */
GList * d_repair_get_stringify_man_power_lines (Repair_line ** repair_lines,GeneralCondition * general_condition,gboolean new_royal)
{
   gint j;
   GList * list = NULL;
   gchar ** line;
   gchar ** line2;
   gdouble quantity;
   gint index_line=0;
   gint number_of_char;
   gdouble price;
   gdouble line_price;
   gdouble discount;
   gchar * temp_desc=NULL;
   gint * final_descrip1 = (gint *) g_malloc0(sizeof(gint));

   if (new_royal==FALSE)
       number_of_char = 45;
   else
       number_of_char = 75;

   for (j=0; (repair_lines[j])!=NULL; j++)
   {
        gchar * description;
        line = (gchar **) g_malloc (sizeof(gchar *)*7); /* 7 columnas de strings */

	if (d_product_get_is_man_power(d_repair_line_get_product (repair_lines[j]))) {
	    quantity = d_repair_line_get_quantity (repair_lines[j]);
	    line_price = d_repair_line_get_price (repair_lines[j]);
	    price = d_product_get_price (d_repair_line_get_product(repair_lines[j]));
	    discount = d_repair_line_get_discount (repair_lines[j]);

	    /* 7 is the number of cols in the invoice table for the repair_lines */
	    if (d_product_get_is_piece(d_repair_line_get_product (repair_lines[j]))) {
		Piece * piece = (Piece *) d_repair_line_get_product (repair_lines[j]);
		line[0] = d_piece_get_euroCode (piece);

	    } else {
		line[0] =  g_strdup_printf("%lu",
	        d_product_get_product_code (d_repair_line_get_product(repair_lines[j])));
	    } 
	    /* Referencia */
	    description = d_product_get_description (d_repair_line_get_product(repair_lines[j]));

            if (general_condition->man_power_description!=FALSE) {
		g_strup(description);
                *final_descrip1=number_of_char;
        	line[1] = u_get_substring_complete_words (description,0,final_descrip1);
	    } else {
                line[1] = g_strdup("");
	    }
            if (description!=NULL)
		g_free(description);
	    line[2] = g_strdup_printf("%.3f", quantity); /* Cantidad */

	    if ((price!=G_MINDOUBLE) && (general_condition->man_power_total!=FALSE)) {
		line[3] = g_strdup_printf("%.2f", price); /* Precio */
	    } else {
		line[3] = g_strdup(" "); /* Precio */
	    }
	    if ((line_price!=G_MINDOUBLE) && (general_condition->man_power_total!=FALSE)) {
		line[4] = g_strdup_printf("%.2f", line_price*quantity); /* Importe */
	    } else if ((price!=G_MINDOUBLE) && (general_condition->man_power_total!=FALSE)) {
		line[4] = g_strdup_printf("%.2f", price*quantity); /* Importe */
	    } else {
		line[4] = g_strdup("");
	    }
	    if (discount>0) {
		line[5] = g_strdup_printf("%.2f", discount); /* Dcto. */
	    } else {
		line[5] = g_strdup(" ");
	    }
	    if ((line_price!=G_MINDOUBLE) && (general_condition->man_power_total!=FALSE)) {
		line[6] = g_strdup_printf("%.2f", line_price*quantity);
	    } else if ((price!=G_MINDOUBLE) && (general_condition->man_power_total!=FALSE)) {
		line[6] = g_strdup_printf("%.2f", price*quantity*(1-(0.01*discount))); /* Total */
	    } else {
                line[6] = g_strdup("");
	    }

	    for (index_line=0;index_line<7;index_line++)
		if (line[index_line]==NULL) line[index_line] = g_strdup("");
	    list = g_list_append(list,line);
	    /* creo que non se debera liberar*/
	    //g_free(line); /* FIXME: SHOULD FREE ALSO EACH ELEMENT OF THE LINE */
	    temp_desc = d_product_get_description (d_repair_line_get_product(repair_lines[j]));

	    g_strstrip(temp_desc);



	    if ((strlen(temp_desc)>number_of_char) && (new_royal!=TRUE))  {
		line2 = (gchar **) g_malloc (sizeof(gchar *)*7); /* 7 columnas de strings */
		line2[0]=g_strdup("");
		if (general_condition->man_power_description!=FALSE) {
		    g_strup(temp_desc);
		    line2[1]=u_get_substring (temp_desc,*final_descrip1,105);
		} else {
		    line2[1]=g_strdup("");
		}
		line2[2]=g_strdup("");
		line2[3]=g_strdup("");
		line2[4]=g_strdup("");
		line2[5]=g_strdup("");
		line2[6]=g_strdup("");
		list = g_list_append(list,line2);
	    }

	}
   }

  return list;
   }

/**
 * d_repair_get_stringify_shop_place:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene el municipio en el que reside la tienda de una empresa del grupo que gener la reparacin.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_shop_place (Repair * repair,gboolean show)
{
    gchar * result=NULL;
    
    if (d_repair_get_shop (repair)!=NULL) {
	result = d_participant_get_town (d_enterprise_get_participant (d_repair_get_enterprise (repair)));
    }

    if ((result !=NULL) && (show!=FALSE)) return result;
    else {
        if (result!=NULL)
	    g_free(result);
	return g_strdup("");
    }
}

/**
 * d_repair_get_stringify_accident_open_date:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 *
 * Obtiene la fecha de apertura de expediente del siniestro. Se solicita posteriormente a la compaa
 * de seguros.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_accident_open_date (Repair * repair,gboolean show) {
   gchar *timer_char = NULL;
   size_t stored;
   time_t date;
   gboolean needed = FALSE;
   if (d_repair_get_accident (repair)!=NULL) {
       if (d_repair_get_company (repair)!=NULL) {
	   if (d_participant_get_code (d_company_get_participant (d_repair_get_company (repair)))!=ULONG_MAX) {
//	       gulong company_code = d_participant_get_code (d_company_get_participant (d_repair_get_company (repair)));
               if (needed) {
		   timer_char = (gchar *) g_malloc0(sizeof(gchar)*100);
		   date = d_accident_get_open_date (d_repair_get_accident (repair));
		   stored = strftime(timer_char,sizeof(gchar)*100,"%Y-%m-%d", localtime(&date));
	       }
	   }
       }
   }

   if ((timer_char!=NULL) && (show==TRUE)) {
       return timer_char;
   } else {
       return g_strdup("");
   }
}

/**
 * d_repair_get_stringify_company_discount:
 * @repair: Reparacin de la que se obtendr el atributo.
 * @show: Permite decidir si mostrar el atributo.
 * 
 * Obtiene el descuento aplicado a la compaa de seguros.
 *
 * Returns:
 * Devuelve el atributo solicitado en una cadena de caracteres.
 *
 */
gchar * d_repair_get_stringify_company_discount (Repair * repair,gboolean show) {
    gchar * result=NULL;
    if (d_repair_get_company (repair)!=NULL) {
	gchar * temp;
        if ((d_company_get_discount (d_repair_get_company (repair))>=0) && (d_company_get_discount (d_repair_get_company (repair))<100))
	    temp = g_strdup_printf("%.2f%%",d_company_get_discount (d_repair_get_company (repair)));
	else
            temp = g_strdup("");
        result = g_strdup(temp);
	g_free(temp);

    }
    if ((result !=NULL) && (show==TRUE))
	return result;
    else {
	if (result!=NULL) g_free(result);
	return g_strdup("");
    }
}


