/* invoice-print.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "invoice-print.h"

/**
 * p_invoice_print_envelope:
 * @pc: Contexto en el que se rellenarn los datos a imprimir.
 * @post_address: Direccin de envo de la que se extraern los datos a imprimir.
 * @client_name: Nombre completo del cliente que aparecer en la direccin de envo.
 *
 * Indica al contexto los valores y posiciones de las lneas de la direccin de envo, para
 * poder imprimir un sobre.
 *
 */
void p_invoice_print_envelope (GnomePrintContext *pc,PostAddress * post_address,gchar *client_name)
{
   GnomeFont *font;



   gnome_print_beginpage (pc, _("Invoice - Fisterra"));

   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }
       gnome_print_rotate(pc,90);

   gnome_print_setfont (pc, font);

   {
       gchar * post_address_first_line=client_name;
       gchar * post_address_second_line=p_post_address_get_stringify_second_line (post_address);
       gchar * post_address_third_line=p_post_address_get_stringify_third_line (post_address);
       gchar * post_address_fourth_line=p_post_address_get_stringify_fourth_line (post_address);

       gnome_print_moveto(pc, 580, -395);
       p_print_show (pc, post_address_fourth_line);
       gnome_print_moveto(pc, 580, -375);
       p_print_show (pc, post_address_third_line);
       gnome_print_moveto(pc, 580, -355);
       p_print_show (pc, post_address_second_line);
       gnome_print_moveto(pc, 580, -335);
       p_print_show (pc, post_address_first_line);

       g_free(post_address_first_line);
       g_free(post_address_second_line);
       g_free(post_address_third_line);
       g_free(post_address_fourth_line);
       d_post_address_clear (post_address);
   }

   gnome_print_showpage (pc);

}

/**
 * p_invoice_print_one_page:
 * @pc: Contexto en el que se rellenarn los datos a imprimir.
 * @invoice: Factura de la que se extraern los datos a imprimir.
 * @prev: NO UTILIZADO / NOT USED
 * @general_condition: Permite elegir los atributos que se mostrarn a la hora de imprimir la factura.
 * @from_line: Lnea desde la que se va a imprimir en esta hoja de la factura.
 * @n_page: Nmero actual de pgina.
 *
 * Nueva versin de impresin de una factura con plantilla normal para poder imprimir la
 * factura en varias pginas. Esta funcin imprime una pgina de la factura con las lneas solicitadas.
 *
 */
void p_invoice_print_one_page (GnomePrintContext *pc,Invoice * invoice,gboolean prev,GeneralCondition * general_condition,gint from_line,gint n_page) {
   GnomeFont *font;
   gchar ** total = (gchar **) g_malloc0(sizeof(gchar *));
   gchar ** vat = (gchar **) g_malloc0(sizeof(gchar *));
   gchar ** subtotal = (gchar **) g_malloc0(sizeof(gchar *));


   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }

   gnome_print_setfont (pc, font);

   {
       double col_width[3] = {75, 110, 80};
       gchar * client_number = p_invoice_get_stringify_client_number (invoice,general_condition->client_number);
       gchar * legal_id = p_invoice_get_stringify_legal_id (invoice,general_condition->enterprise_legal_id_template_1);
       gchar * page_number = NULL;
       char *contents[6];
       if (general_condition->page_number) page_number = g_strdup_printf("%d",n_page);
       else page_number = g_strdup(" ");
       contents[0]=_("Client No.");
       contents[1]=_("Tax code/Legal ID");
       contents[2]=_("Page No.");
       contents[3]=client_number;
       contents[4]=legal_id;
       contents[5]=page_number;

     p_print_complex_table (pc, font, 25, 700, 2, 3, col_width, 18, contents, 2, 0);
     g_free(client_number);
     g_free(legal_id);
     g_free(page_number);

   }

   {
       double col_width[3] = {75, 110, 80};
       gchar * date = p_invoice_get_stringify_date (invoice,general_condition->date);
       gchar * invoice_number = p_invoice_get_stringify_number (invoice,general_condition->invoice_number);
       gchar * agent_number = p_invoice_get_stringify_agent_number (invoice,general_condition->agent);

     char *contents[6] = {
       _("DATE"), _("Invoice No."), _("AGENT No."),
       date, invoice_number, agent_number

     };

     p_print_complex_table (pc, font, 25, 650, 2, 3, col_width, 18, contents, 2, 0);
     g_free(date);
     g_free(invoice_number);
     g_free(agent_number);

   }


   {
       gchar ** post_address = p_invoice_get_stringify_post_address (invoice,general_condition);
       gchar * post_address_first_line=g_strdup(post_address[0]);
       gchar * post_address_second_line=g_strdup(post_address[1]);
       gchar * post_address_third_line=g_strdup(post_address[2]);
       gchar * post_address_fourth_line =g_strdup(post_address[3]);

       g_strfreev(post_address);
       p_print_box (pc, 325, 700, 250, 86);
       gnome_print_moveto(pc, 335, 680);
       p_print_show (pc, post_address_first_line);
       gnome_print_moveto(pc, 335, 662);
       p_print_show (pc, post_address_second_line);
       gnome_print_moveto(pc, 335, 644);
       p_print_show (pc, post_address_third_line);
       gnome_print_moveto(pc, 335, 626);
       p_print_show (pc, post_address_fourth_line);
       g_free(post_address_first_line);
       g_free(post_address_second_line);
       g_free(post_address_third_line);
       g_free(post_address_fourth_line);

   }

   {
       gchar * company_aux = p_invoice_get_stringify_company (invoice,general_condition->company);
       gchar * policy_number_aux=p_invoice_get_stringify_policy (invoice,general_condition->policy_number);
       gchar * delivery_payment_number_aux=p_invoice_get_stringify_work_number (invoice,general_condition->work_number);

       gchar * company =g_strconcat(_("Co.: "),company_aux,NULL);
       gchar * policy_number  =g_strconcat(_("PN: "),policy_number_aux,NULL);
       gchar * delivery_payment_number  =g_strconcat(_("Del. Pay. No.: "),delivery_payment_number_aux,NULL);
       p_print_box (pc, 25, 604, 550, 18);
       gnome_print_moveto(pc, 35, 590); /* FIXME: centrado en altura */
       p_print_show (pc, company);
       gnome_print_moveto(pc, 310, 590); /* FIXME: centrado en altura */
       p_print_show (pc, policy_number);
       gnome_print_moveto(pc, 430, 590); /* FIXME: centrado en altura */
       p_print_show (pc, delivery_payment_number);
       g_free(company_aux);
       g_free(policy_number_aux);
       g_free(company);
       g_free(policy_number);
       g_free(delivery_payment_number_aux);
       g_free(delivery_payment_number);
   }

   {
       gdouble col_width[7]={88, 240, 48, 43, 43, 38, 50};
       GList * list = p_invoice_get_stringify_lines (invoice,general_condition,FALSE,TRUE);
       GList * list_it;
       gchar ** contents;
       gint index=1;
       gint current_line=0;

       contents = (gchar **) g_malloc0(((P_NORMAL_TEMPLATE_LINES+1)*7+1)*sizeof(gchar *));

       contents[0]=g_strdup(_("Reference"));
       contents[1]=g_strdup(_("Description"));
       contents[2]=g_strdup(_("Amount"));
       contents[3]=g_strdup(_("Price"));
       contents[4]=g_strdup(_("Cost"));
       contents[5]=g_strdup(_("% Disc."));
       contents[6]=g_strdup(_("Total"));

       list_it=g_list_first(list);
       while ((list_it!=NULL) && (current_line<from_line)) {
	   list_it = g_list_next(list_it);
           current_line++;
       }

       while ((list_it!=NULL) && (index<=P_NORMAL_TEMPLATE_LINES)) {
	   gint i;
	   gchar ** line = (gchar **) list_it->data;
	   for (i=0;i<7;i++) {
	       contents[index*7+i] = g_strdup(line[i]);
	   }
	   list_it = g_list_next(list_it);
	   index++;
       }
       gnome_print_setfont (pc, font);

       p_print_normal_complex_table (pc, font, 25, 580, P_NORMAL_TEMPLATE_LINES+2, 7, col_width, 23,
			    contents, index, 0);
       g_list_free(list);
       g_strfreev(contents);
   }

   
   p_invoice_calculate_total_data (invoice,subtotal,vat,total,general_condition,FALSE);
   p_print_box (pc, 25, 212, 75, 36); /* FIXME: print_text_centered + de 1 linea */
   p_print_centered_text (pc, font, 25, 198, 75, _("Totals"));
   p_print_centered_text (pc, font, 25, 186, 75, _("VAT"));

   p_print_box (pc, 25, 172, 550, 54);
   gnome_print_moveto(pc, 100, 172);
   gnome_print_lineto(pc, 100, 118);

   gnome_print_moveto(pc, 30, 156);
   p_print_show (pc, _("Payment way"));
   gnome_print_moveto(pc, 105, 156);
   {
       gchar * payment_way=p_invoice_get_stringify_payment_mode (invoice,general_condition->payment_way);
       if (payment_way==NULL) {
           payment_way = g_strdup(" ");
       }
       p_print_show (pc,payment_way);
       g_free(payment_way);
   }
   gnome_print_moveto(pc, 30, 140);
   p_print_show (pc, _("Payment entity"));
   gnome_print_moveto(pc, 105, 140);
   {
       gchar * payment_entity = p_invoice_get_stringify_payment_entity (invoice,general_condition->payment_entity);
       font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 8);
       if (font == NULL)
       {
	   g_warning(_("Font cannot be found"));
       }

       gnome_print_setfont (pc, font);

       
       p_print_show (pc, payment_entity);
       g_free(payment_entity);

       font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
       if (font == NULL)
       {
	   g_warning(_("Font cannot be found"));
       }

       gnome_print_setfont (pc, font);
   }

   gnome_print_moveto(pc, 30, 122);
   p_print_show (pc, _("Account No."));
   gnome_print_moveto(pc, 105, 122);
   {
       gchar * account_number = p_invoice_get_stringify_account_number (invoice,general_condition->account);
       p_print_show (pc, account_number);
       g_free(account_number);
   }
   
   p_print_box (pc, 25, 114, 550, 18);
   gnome_print_moveto(pc, 100, 114);
   gnome_print_lineto(pc, 100, 96);

   gnome_print_moveto(pc, 30, 98);
   p_print_show (pc, _("Expiries"));
   {
       gchar * date=p_invoice_get_stringify_expiries_number (invoice,general_condition->instalments);
       gchar * char_instalments=g_strconcat(date,_("   Amount:"),*total,NULL);
       gnome_print_moveto(pc, 105, 98);
       p_print_show (pc, char_instalments);
       g_free(date);
       g_free(char_instalments);
   }

   {
       gchar *enterprise = p_invoice_get_stringify_company_name (invoice,general_condition->company);
       gchar *enterprise_legal_id = p_invoice_get_stringify_company_legal_id (invoice,general_condition->company);
       gchar *all = g_strconcat(enterprise,"    TAX CODE:",enterprise_legal_id,NULL);
       if (d_invoice_get_vat_type (invoice)>=0) {
	   g_free(all);
           all = g_strdup("");
       }

       p_print_centered_text (pc, font, 25, 80, 560, all);
       g_free(enterprise);
       g_free(enterprise_legal_id);
       g_free(all);
   }

   gnome_print_gsave(pc);
   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }
   gnome_print_setfont(pc,font);

   {
       gchar * discount = p_invoice_get_stringify_discount (invoice,general_condition->discount);
       gchar * total_discount = p_invoice_get_stringify_total_discount (invoice,general_condition->discount);
       gchar * total_concat = g_strconcat(*total," Euros",NULL);
       gchar * total_sum = g_strdup_printf("%.2f",atof(*subtotal)+atof(total_discount));
       gdouble col_width[6] = {100, 90, 90, 75, 80, 115};
       gchar * contents[12] = {_("Costs addition"), _("% EP. Disc."), _("% EP. Disc."), _("Subtotal"), _("Total VAT"), _("Invoice Total"),
       total_sum, discount, total_discount, *subtotal, *vat, total_concat

     };

     p_print_complex_table (pc, font, 25, 252, 2, 6, col_width, 18, contents, 2, 0);
     g_free(discount);
     g_free(total_discount);
     g_free(total_concat);
     g_free(total_sum);

   }
   

   {
     double col_width[2] ={100, 80};
     char *contents[4] ={_("Taxable income"), _("VAT amount"),
			  *subtotal,  *vat};

     p_print_complex_table (pc, font, 100, 212, 2, 2, col_width, 18, contents, 2, 0);
   }

   {
       gchar * license_number = d_repair_get_stringify_license_number (p_invoice_get_first_repair (invoice),general_condition->license_number);
       gchar * dossier = d_repair_get_stringify_accident_reference (p_invoice_get_first_repair (invoice),general_condition->dossier);
       p_print_box (pc, 310, 212, 265, 36);
       gnome_print_moveto(pc, 320, 198);
       p_print_show (pc, _("License number:"));
       gnome_print_moveto(pc, 420, 198);
       p_print_show (pc, license_number);
       gnome_print_moveto(pc, 320, 182);
       p_print_show (pc, _("File number"));
       gnome_print_moveto(pc, 420, 182);
       p_print_show (pc, dossier);

       g_free(license_number);
       g_free(dossier);
   }

      gnome_print_grestore(pc);

   g_free(*vat);
   g_free(vat);
   g_free(*total);
   g_free(total);
   g_free(*subtotal);
   g_free(subtotal);

}

/**
 * p_invoice_print_garage_invoice_page:
 * @pc: Contexto en el que se rellenarn los datos a imprimir.
 * @invoice: Factura de la que se extraern los datos a imprimir.
 * @prev: NO UTILIZADO / NOT USED
 * @general_condition: Permite elegir los atributos que se mostrarn a la hora de imprimir la factura.
 * @from_line: Lnea desde la que se va a imprimir en esta hoja de la factura.
 * @n_page: Nmero actual de pgina.
 *
 * Permite imprimir una factura para un taller indicado los datos necesarios para cada pgina de la factura.
 * Nueva versin de impresin de una factura con plantilla normal para un taller para poder imprimir la
 * factura en varias pginas. Esta funcin imprime una pgina de la factura con las lneas solicitadas.
 *
 */
void p_invoice_print_garage_invoice_page (GnomePrintContext *pc,Invoice * invoice,gboolean prev,GeneralCondition * general_condition,gint from_line,gint n_page) {
   GnomeFont *font;
   gchar ** total = (gchar **) g_malloc0(sizeof(gchar *));
   gchar ** vat = (gchar **) g_malloc0(sizeof(gchar *));
   gchar ** subtotal = (gchar **) g_malloc0(sizeof(gchar *));

   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }

   gnome_print_setfont (pc, font);

   {
       double col_width[3] = {75, 110, 80};
       gchar * client_number = p_invoice_get_stringify_client_number (invoice,general_condition->client_number);
       gchar * legal_id = p_invoice_get_stringify_legal_id (invoice,general_condition->enterprise_legal_id_template_1);
       gchar * page_number;
       char *contents[6];
       if (general_condition->page_number) page_number = g_strdup_printf("%d",n_page);
       else page_number = g_strdup(" ");
       contents[0]=_("CLIENT No.");
       contents[1]=_("Tax code/Legal ID");
       contents[2]=_("Page No.");
       contents[3]=client_number;
       contents[4]=legal_id;
       contents[5]=page_number;


     p_print_complex_table (pc, font, 25, 700, 2, 3, col_width, 18, contents, 2, 0);
     g_free(client_number);
     g_free(legal_id);
     g_free(page_number);

   }

   {
       double col_width[3] = {75, 110, 80};
       gchar * date = p_invoice_get_stringify_date (invoice,general_condition->date);
       gchar * invoice_number = p_invoice_get_stringify_number (invoice,general_condition->invoice_number);
       gchar * agent_number = p_invoice_get_stringify_agent_number (invoice,general_condition->agent);

     char *contents[6] = {
        _("DATE"), _("DEL. PAY. No."), _("AGENT No."),
       date, invoice_number, agent_number

     };

     p_print_complex_table (pc, font, 25, 650, 2, 3, col_width, 18, contents, 2, 0);
     g_free(date);
     g_free(invoice_number);
     g_free(agent_number);

   }


   {
       gchar ** post_address = p_invoice_get_stringify_post_address (invoice,general_condition);
       gchar * post_address_first_line=g_strdup(post_address[0]);
       gchar * post_address_second_line=g_strdup(post_address[1]);
       gchar * post_address_third_line=g_strdup(post_address[2]);
       gchar * post_address_fourth_line =g_strdup(post_address[3]);

       g_strfreev(post_address);
       p_print_box (pc, 325, 700, 250, 86);
       gnome_print_moveto(pc, 335, 680);
       p_print_show (pc, post_address_first_line);
       gnome_print_moveto(pc, 335, 662);
       p_print_show (pc, post_address_second_line);
       gnome_print_moveto(pc, 335, 644);
       p_print_show (pc, post_address_third_line);
       gnome_print_moveto(pc, 335, 626);
       p_print_show (pc, post_address_fourth_line);
       g_free(post_address_first_line);
       g_free(post_address_second_line);
       g_free(post_address_third_line);
       g_free(post_address_fourth_line);

   }

   {
       gchar * company_aux = p_invoice_get_stringify_company (invoice,general_condition->company);
       gchar * policy_number_aux=p_invoice_get_stringify_policy (invoice,general_condition->policy_number);
       gchar * delivery_payment_number_aux=p_invoice_get_stringify_work_number (invoice,general_condition->work_number);

       gchar * company =g_strconcat(" ",NULL);
       gchar * policy_number  =g_strconcat(" ",NULL);
       gchar * delivery_payment_number  =g_strconcat(" ",NULL);
       p_print_box (pc, 25, 604, 550, 18);
       gnome_print_moveto(pc, 35, 590); /* FIXME: centrado en altura */
       p_print_show (pc, company);
       gnome_print_moveto(pc, 310, 590); /* FIXME: centrado en altura */
       p_print_show (pc, policy_number);
       gnome_print_moveto(pc, 430, 590); /* FIXME: centrado en altura */
       p_print_show (pc, delivery_payment_number);
       g_free(company_aux);
       g_free(policy_number_aux);
       g_free(company);
       g_free(policy_number);
       g_free(delivery_payment_number_aux);
       g_free(delivery_payment_number);
   }

   {
       gdouble col_width[7]={88, 240, 48, 43, 43, 38, 50};
       GList * list = p_invoice_get_stringify_lines (invoice,general_condition,TRUE,TRUE);
       GList * list_it;
       gchar ** contents;
       gint index=1;
       gint current_line = 0;


       contents = (gchar **) g_malloc0(((P_NORMAL_TEMPLATE_LINES+1)*7+1)*sizeof(gchar *));

       contents[0]=g_strdup(_("Reference"));
       contents[1]=g_strdup(_("Description"));
       contents[2]=g_strdup(_("Amount"));
       contents[3]=g_strdup(_("Price"));
       contents[4]=g_strdup(_("Cost"));
       contents[5]=g_strdup(_("% Disc"));
       contents[6]=g_strdup(_("Total"));

       list_it=g_list_first(list);
       while ((list_it!=NULL) && (current_line<from_line)) {
	   list_it = g_list_next(list_it);
           current_line++;
       }

       list_it=g_list_first(list);
       while ((list_it!=NULL) && (index<=P_NORMAL_TEMPLATE_LINES)) {
	   gint i;
	   gchar ** line = (gchar **) list_it->data;
	   for (i=0;i<7;i++) {
	       contents[index*7+i] = g_strdup(line[i]);
	   }
	   list_it = g_list_next(list_it);
	   index++;
       }
       gnome_print_setfont (pc, font);

       p_print_normal_complex_table (pc, font, 25, 580, P_NORMAL_TEMPLATE_LINES+2, 7, col_width, 23,
			    contents, index, 0);
       g_list_free(list);
       g_strfreev(contents);
   }

   
   p_invoice_calculate_total_data (invoice,subtotal,vat,total,general_condition,TRUE);
   p_print_box (pc, 25, 212, 75, 36); /* FIXME: print_text_centered + de 1 linea */
   p_print_centered_text (pc, font, 25, 198, 75, _("Totals"));
   p_print_centered_text (pc, font, 25, 186, 75, _("VAT"));


   p_print_box (pc, 25, 172, 550, 54);
   gnome_print_moveto(pc, 100, 172);
   gnome_print_lineto(pc, 100, 118);

   gnome_print_moveto(pc, 30, 156);
   p_print_show (pc, _("Payment way"));
   gnome_print_moveto(pc, 105, 156);
   {
       gchar * payment_way=p_invoice_get_stringify_payment_mode (invoice,general_condition->payment_way);
       if (payment_way==NULL) {
           payment_way = g_strdup(_(" CASH DOWN"));
       }
       p_print_show (pc,payment_way);
       g_free(payment_way);
   }
   gnome_print_moveto(pc, 30, 140);
   p_print_show (pc, _("Payment entity"));
   gnome_print_moveto(pc, 105, 140);
   p_print_show (pc, " ");
   gnome_print_moveto(pc, 30, 122);
   p_print_show (pc, _("Account No."));
   gnome_print_moveto(pc, 105, 122);
   p_print_show (pc, " ");
   
   p_print_box (pc, 25, 114, 550, 18);
   gnome_print_moveto(pc, 100, 114);
   gnome_print_lineto(pc, 100, 96);

   gnome_print_moveto(pc, 30, 98);
   p_print_show (pc, _("Expiries"));
   {
       gchar * date=p_invoice_get_stringify_expiries_number (invoice,general_condition->instalments);
       gchar * char_instalments=g_strconcat(date,_("   Amount:"),*total,NULL);
       gnome_print_moveto(pc, 105, 98);
       p_print_show (pc, char_instalments);
       g_free(date);
       g_free(char_instalments);
   }

   {
       gchar *enterprise = p_invoice_get_stringify_company_name (invoice,general_condition->company);
       gchar *enterprise_legal_id = p_invoice_get_stringify_company_legal_id (invoice,general_condition->company);
       gchar *all = g_strconcat(enterprise,_("    TAX CODE:"),enterprise_legal_id,NULL);
       if (d_invoice_get_vat_type (invoice)>=0) {
	   g_free(all);
           all = g_strdup("");
       }

       p_print_centered_text (pc, font, 25, 80, 560, all);
       g_free(enterprise);
       g_free(enterprise_legal_id);
       g_free(all);
   }

   gnome_print_gsave(pc);                        /*BOOK*/  /*8*/
   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }
   gnome_print_setfont(pc,font);

   {
       gchar * discount = p_invoice_get_stringify_discount (invoice,general_condition->discount);
       gchar * total_discount = p_invoice_get_stringify_total_discount (invoice,general_condition->discount);
       gchar * total_concat = g_strconcat(*total," Euros",NULL);
       gdouble col_width[6] = {100, 90, 90, 75, 80, 115};
       gchar * contents[12] = {_("Costs addition"), _("% EP. Disc."), _("% EP. Disc."), _("Subtotal"), _("Total VAT"), _("Invoice Total"),
       *subtotal, discount, total_discount, *subtotal, *vat, total_concat

     };

     p_print_complex_table (pc, font, 25, 252, 2, 6, col_width, 18, contents, 2, 0);
     g_free(discount);
     g_free(total_discount);
     g_free(total_concat);

   }
   

   {
     double col_width[2] ={100, 80};
     char *contents[4] ={_("Taxable income"), _("VAT amount"),
			  *subtotal,  *vat};

     p_print_complex_table (pc, font, 100, 212, 2, 2, col_width, 18, contents, 2, 0);
   }

   {
       gchar * license_number = d_repair_get_stringify_license_number (p_invoice_get_first_repair (invoice),general_condition->license_number);
       gchar * dossier = d_repair_get_stringify_accident_reference (p_invoice_get_first_repair (invoice),general_condition->dossier);
       p_print_box (pc, 310, 212, 265, 36);
       gnome_print_moveto(pc, 320, 198);
       p_print_show (pc, _("License number:"));
       gnome_print_moveto(pc, 420, 198);
       p_print_show (pc, license_number);
       gnome_print_moveto(pc, 320, 182);
       p_print_show (pc, _("File number"));
       gnome_print_moveto(pc, 420, 182);
       p_print_show (pc, dossier);

       g_free(license_number);
       g_free(dossier);
   }

   gnome_print_grestore(pc);

   g_free(*vat);
   g_free(vat);
   g_free(*total);
   g_free(total);
   g_free(*subtotal);
   g_free(subtotal);



}

/**
 * p_invoice_print_multipage:
 * @pc: Contexto en el que se rellenarn los datos a imprimir.
 * @invoice: Factura de la que se extraern los datos a imprimir.
 * @prev: NO UTILIZADO.
 * @general_condition: Permite elegir los atributos que se mostrarn a la hora de imprimir la factura.
 * @from_garage: Permite elegir si la factura es para un taller o no.
 *
 * Permite indicarle al contexto la impresin de una factura en varias pginas, para poder imprimir todas las
 * lneas de factura. Calcula las lneas que le tocan a cada pgina de la factura y el nmero de pgina actual.
 *
 */
void p_invoice_print_multipage (GnomePrintContext *pc,Invoice * invoice,gboolean prev,GeneralCondition * general_condition,gboolean from_garage) {
    gint index_page = 1;
    gchar * temp=NULL;
    gint next_row=0;
    gint number_of_lines = p_invoice_get_number_of_lines (invoice,from_garage);
    gint number_of_pages=0;

    g_print(_("\nNumber of lines: %d\n"),number_of_lines);
    if (number_of_lines%P_NUMBER_OF_LINES!=0) {
	number_of_pages = number_of_lines/P_NUMBER_OF_LINES + 1;
    } else {
	number_of_pages = number_of_lines/P_NUMBER_OF_LINES;
    }


    g_print(_("\nNumber de pages: %d\n"),number_of_pages);
    while (index_page<=(number_of_pages)) {
	temp = g_strdup_printf("Invoice, page %d",index_page);
	gnome_print_beginpage (pc, temp);
	g_free(temp);
	if (index_page == 0) {
	    next_row = 0;
	} else {
	    next_row = P_NUMBER_OF_LINES*(index_page-1);
	}
        if (from_garage==FALSE)
	    p_invoice_print_one_page (pc,invoice,prev,general_condition,next_row,index_page);
	else
	    p_invoice_print_garage_invoice_page (pc,invoice,prev,general_condition,next_row,index_page);

	gnome_print_showpage (pc);
        index_page++;
    }
    gnome_print_context_close(pc);


}
