/* listing-print-wrapper.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "listing-print-wrapper.h"

#include "listing-print.h"

static int do_preview;
static int copies=1, collate;
static void (*function_to_call)(void) = NULL;

/*static struct poptOption options [] =
{
    {"copies", 0, POPT_ARG_INT, &copies },
    { "collate", 0, POPT_ARG_NONE, &collate },
    { NULL }
};
*/
/**
 * p_listing_close_window:
 * @o: Objeto a liberar.
 * @data: Datos a utilizar por el deserrollador.
 * 
 * Cierra correctamente la ventana de vista preliminar de los listados.
 *
 */
void p_listing_close_window (GtkObject *o, void *data)
{
    if (function_to_call !=NULL) {
	function_to_call();
    }
    gtk_widget_destroy(GTK_WIDGET(o));
    //    exit(0);
}


/**
 * p_listing_wrap_print:
 * @listing: Listado a imprimir.
 * 
 * Permite imprimir un listado, seleccionando el tipo de papel segn la anchura del listado y la
 * cantidad de datos a imprimir. Segn el listado calcula los parmetros del mismo: espacio horizontal,
 * espacio vertical, altura de la fila, nmero de pginas, nmero defilas en primera pgina y nmero de
 * pginas en el resto de pginas.
 *
 */
void p_listing_wrap_print (Listing * listing)
{

    GnomePrintContext *pc;
    GnomePrintMaster *gpm = NULL;
    GnomePrintDialog *gpd;
    const gchar * papername_A4 = "A4";
    const gchar * papername_A3 = "A3";
    const GnomePaper * paper=NULL;
    ListingParameters * listing_parameters;

    if (listing->type_of_paper == P_PAPER_A4) {
	paper= gnome_paper_with_name(papername_A4);
    } else if (listing->type_of_paper == P_PAPER_A3) {
	paper= gnome_paper_with_name(papername_A3);
    } else {
	g_print(_("\nNot yet implemented\n"));
    }

    gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(_("Print options"), GNOME_PRINT_DIALOG_COPIES));
    gnome_print_dialog_set_copies(gpd, copies, collate);

    gtk_signal_connect(GTK_OBJECT(gpd), "destroy", p_listing_close_window , NULL);

    switch (gnome_dialog_run(GNOME_DIALOG(gpd)))
    {
    case GNOME_PRINT_PRINT:
	do_preview = 0;
	break;
    case GNOME_PRINT_PREVIEW:
	do_preview = 1;
	break;
    case GNOME_PRINT_CANCEL:
	gtk_widget_destroy(GTK_WIDGET(gpd));
	do_preview=2;
	break;
    default:
	do_preview=2;
	break;
    }

    if (do_preview<2) {
	GnomePrinter * printer = gnome_print_dialog_get_printer(gpd);
	gpm = gnome_print_master_new();
	gnome_print_dialog_get_copies(gpd, &copies, &collate);
	gnome_print_master_set_copies(gpm, copies, collate);
	gnome_print_master_set_printer(gpm, printer);

	gnome_dialog_close (GNOME_DIALOG(gpd));

	pc = gnome_print_master_get_context(gpm);

	/* Pages to print: */
	listing_parameters = p_listing_calculate_parameters (listing);
	p_listing_multipage (pc,listing,listing_parameters);

	gnome_print_master_close(gpm);
	if (paper!=NULL)
	    gnome_print_master_set_paper(gpm,paper);
	if (do_preview==1)
	{
	    gboolean landscape=FALSE;
	    GnomePrintMasterPreview *pmp;
	    if (listing->orientation==P_LANSCAPE) {
		landscape = TRUE;
	    }
	    pmp = gnome_print_master_preview_new_with_orientation (
								   gpm, _("Print Preview"), landscape);

	    gtk_signal_connect(GTK_OBJECT(pmp), "destroy", p_listing_close_window , NULL);
	    gtk_widget_show(GTK_WIDGET(pmp));
	}
	else
	{
	    gnome_print_master_print(gpm);
	}
    
    }
   
        
      /* Fixed in gnome-print-0.37, this line is needed if you want to use 
       *    the program and don't loose your locale when printing or print previewing:
       *        setlocale(LC_ALL,"es_ES@euro");
       */
   
 }



/**
 * p_listing_calculate_parameters:
 * @listing: Listado a partir del cual se calcularn los parmetros de impresin.
 * 
 * Calcula los parmetros necesarios de un listado. Calcula el alto, el ancho del papel, las
 * filas por pgina (la primera o dems) segn sea horizontal o vertical.
 *
 * Returns:
 * Un puntero a una estructura %ListingParameters con los parmetros del listado calculados.
 */
ListingParameters * p_listing_calculate_parameters (Listing * listing) {
    ListingParameters * listing_parameters = (ListingParameters *) g_malloc0(sizeof(ListingParameters));
    gint aux1,aux2;
    if (listing->type_of_paper == P_PAPER_A3) {
	g_print(_("\nNot yet implemented\n"));
    } else {
	if (listing->orientation==P_PORTRAIT) {
	    listing_parameters->horizontal_space = P_PORTRAIT_WIDTH;
	    listing_parameters->vertical_space = P_PORTRAIT_LONG;
	    listing_parameters->n_rows_on_first_page = P_ROW_BY_SHEET_PORTRAIT-3;
	    listing_parameters->n_rows_on_page = P_ROW_BY_SHEET_PORTRAIT;
	    g_print(_("\nANCHURA: %d\n"),listing_parameters->horizontal_space);
	    g_print(_("\nAlTURA: %d\n"),listing_parameters->vertical_space);
	} else {
	    listing_parameters->horizontal_space = P_WIDTH_LANDSCAPE;
	    listing_parameters->vertical_space = P_LANDSCAPE_WIDTH;
	    listing_parameters->n_rows_on_first_page = P_ROW_BY_SHEET_LANDSCAPE-3;
	    listing_parameters->n_rows_on_page = P_ROW_BY_SHEET_LANDSCAPE;

	}
	listing_parameters->row_heigth= P_ROW_HEIGHT;

	if (listing->n_rows<listing_parameters->n_rows_on_first_page) {
	    listing_parameters->n_pages = 1;
	} else {
	    gdouble n_rows_aux = listing->n_rows-listing_parameters->n_rows_on_first_page;
	    gint remainder=0;
	    gint quotient=0;

	    aux1 = (gint) (listing_parameters->n_rows_on_page);
	    remainder = ((int) n_rows_aux)%((int) aux1);
	    aux2 = (gint) (listing_parameters->n_rows_on_page);
	    quotient = n_rows_aux/aux2;
	    if (remainder>0)
		listing_parameters->n_pages = 2 + quotient;
	    else
		listing_parameters->n_pages = 1 + quotient;
	}
    }
    return listing_parameters;
}

/**
 * p_listing_clear:
 * @list: Listado a liberar.
 * 
 * Libera la memoria utilizada por el listado.
 *
 */
void p_listing_clear (Listing *list){
    if (list!=NULL) {
	if (list->listing_title!=NULL) {
	    g_free(list->listing_title);
	}
	g_strfreev(list->content_title);
	if (list->content!=NULL) {
	    gint index=0;
	    while (list->content[index]!=NULL) {
		g_strfreev(list->content[index]);
		index++;
	    }
	}
	g_free(list->width_of_cols);
    g_free(list->alignment_of_cols);
    g_free(list);
    list = NULL;
    }
}

/**
 * p_listing_init:
 * 
 * Inicializa un listado, reservando memoria para el mismo.
 *
 * Returns:
 * El listado inicializado y con la memoria reservada.
 */
Listing * p_listing_init (){
    Listing * list = (Listing * ) g_malloc(sizeof(Listing));
    list->listing_title = (gchar *) g_malloc0(sizeof(gchar));
    list->content_title = (gchar **) g_malloc0(sizeof(gchar*));
    list->content = (gchar ***) g_malloc0(sizeof(gchar**));
    list->width_of_cols =  (gint *) g_malloc0(sizeof(gint));
    list->alignment_of_cols = (gint *) g_malloc0(sizeof(gint));
    list->n_rows=0;
    list->n_columns=0;
    list->orientation=P_PORTRAIT;
    list->type_of_paper=P_PAPER_A4;

    return list;

}
