/* invoice-print-wrapper.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _LISTING_PRINT_WRAPPER_
#define _LISTING_PRINT_WRAPPER_

#include "config.h"
#include "glib.h"
#include "gnome.h"
#include "gtk/gtk.h"
#include "print_utils.h"

/**
 * P_ROW_HEIGHT:
 *
 * Especifica el alto de cada fila de un listado.
 *
 */
#define P_ROW_HEIGHT 14
/**
 * P_ROW_BY_SHEET_PORTRAIT:
 *
 * Nmero de filas de una listado que caben en una hoja vertical.
 *
 */
#define P_ROW_BY_SHEET_PORTRAIT 53
/**
 * P_ROW_BY_SHEET_LANDSCAPE:
 *
 * Nmero de filas de una listado que caben en una hoja horizontal.
 *
 */
#define P_ROW_BY_SHEET_LANDSCAPE 36
/**
 * P_PORTRAIT_LONG:
 *
 * Indica la altura del papel vertical.
 *
 */
#define P_PORTRAIT_LONG 800
/**
 * P_PORTRAIT_WIDTH:
 *
 * Indica la anchura del papel vertical.
 *
 */
#define P_PORTRAIT_WIDTH 560
/**
 * P_WIDTH_LANDSCAPE:
 *
 * Indica la anchura del papel horizontal.
 *
 */
#define P_WIDTH_LANDSCAPE 800
/**
 * P_LANDSCAPE_WIDTH:
 *
 * Indica la altura del papel horizontal.
 *
 */
#define P_LANDSCAPE_WIDTH 560
/**
 * P_LEFT_LIMIT:
 *
 * Puntos que delimitan el lmite a la izquierda para imprimir.
 *
 */
#define P_LEFT_LIMIT 20
/**
 * P_PAPER_A3:
 *
 * Cdigo para identificar el papel de tamao A3.
 *
 */
#define P_PAPER_A3 1
/**
 * P_PAPER_A4:
 *
 * Cdigo para identificar el papel de tamao A4.
 *
 */
#define P_PAPER_A4 2
/**
 * P_LANSCAPE:
 *
 * Cdigo para identificar la orientacion horizontal del papel.
 *
 */
#define P_LANSCAPE 3
/**
 * P_PORTRAIT:
 *
 * Cdigo para identificar la orientacion vertical del papel.
 *
 */
#define P_PORTRAIT 4

typedef struct _listing {
  gchar * listing_title;    /*Ttulo que solo va en la primera hoja*/
  gchar ** content_title;   /*Ttulo de cada hoja del listado */
  gchar *** content; /*Contenido del listado: por filas y columnas*/
  gint * width_of_cols; /* ancho de las columnas */
  gint * alignment_of_cols; /* alineamiento de las columnas */
  gulong n_rows; /* nmero de filas */
  gint n_columns; /* nmero de columnas */
  gint orientation; /* orientacin */
  gint type_of_paper; /* tipo de papel */
} Listing;

typedef struct _listing_parameters {
  gint horizontal_space;
  gint vertical_space;
  gint row_heigth;
  gint n_pages;
  gint n_rows_on_first_page;
  gint n_rows_on_page;
} ListingParameters;

void p_listing_close_window (GtkObject *o, void *data);
void p_listing_wrap_print (Listing * listing);
ListingParameters * p_listing_calculate_parameters (Listing * listing);
void p_listing_clear (Listing *list);
Listing * p_listing_init (void);
#endif

