/* order-print-wrapper.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "order-print-wrapper.h"
#include "locale.h"

static int do_preview;
static int copies=1, collate;
static void (*function_to_call)(void) = NULL;

/*
static struct poptOption options [] = 
{   
     {"copies", 0, POPT_ARG_INT, &copies },
     { "collate", 0, POPT_ARG_NONE, &collate },
     { NULL }
};
*/

/**
 * p_order_close_window:
 * @o: Objeto a destruir.
 * @data: Datos a utilizar por el desarrollador.
 * 
 * Destruye  la ventana de impresin de un pedido y libera la memoria utilizada por ella.
 *
 */
void p_order_close_window (GtkObject *o, void *data)
{
    if (function_to_call !=NULL) {
        function_to_call();
    }
        
    gtk_widget_destroy(GTK_WIDGET(o));

}

/**
 * p_order_wrap_print:
 * @order: Pedido que se desea imprimir.
 * @number_of_lines: Nmero de lneas de pedido que tiene el mismo.
 * 
 * Imprime un pedido, llamando a la funcin de impresin de mltiples pginas de un pedido.
 * Permite previsualizar el pedido antes de imprimirlo.
 *
 */
void p_order_wrap_print (Order * order,gulong number_of_lines)

{
   
   GnomePrintContext *pc;
   GnomePrintMaster *gpm = NULL;
   GnomePrintDialog *gpd;
   const gchar * papername = "A4";
   const GnomePaper * paper;
   gint number_of_pages;

   //Linea que estaba na inicializacin que non sei se  neceSARIA
//   gnome_init_with_popt_table ("TestPrint", VERSION, 0, NULL, options, 0, NULL);


   gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(_("Print options"), GNOME_PRINT_DIALOG_COPIES));
   gnome_print_dialog_set_copies(gpd, copies, collate);
   paper= gnome_paper_with_name(papername);

   gtk_signal_connect(GTK_OBJECT(gpd), "destroy", p_order_close_window , NULL);
   
   switch (gnome_dialog_run(GNOME_DIALOG(gpd))) 
     {	
     case GNOME_PRINT_PRINT:
	do_preview = 0;
	break;
      case GNOME_PRINT_PREVIEW:
	do_preview = 1;
	break;
      case GNOME_PRINT_CANCEL:
	  gtk_widget_destroy(GTK_WIDGET(gpd));
	  do_preview=2;
          break;
     default:
	 do_preview=2;
	 break;
     }

   if (do_preview<2) {
       /*    printf("printer = %p\n", gnome_print_dialog_get_printer(gpd));*/

       gpm = gnome_print_master_new();
       gnome_print_dialog_get_copies(gpd, &copies, &collate);
       gnome_print_master_set_copies(gpm, copies, collate);
       gnome_print_master_set_printer(gpm, gnome_print_dialog_get_printer(gpd));

       gnome_dialog_close (GNOME_DIALOG(gpd));

       pc = gnome_print_master_get_context(gpm);

       /* Pages to print: */

       if (number_of_lines%P_ORDER_NUMBER_OF_ORDERS!=0) {
           number_of_pages = number_of_lines/P_ORDER_NUMBER_OF_ORDERS + 1;
       } else {
	   number_of_pages = number_of_lines/P_ORDER_NUMBER_OF_ORDERS;

       }
       p_order_multipage (pc,order,number_of_pages);
       gnome_print_master_close(gpm);
       gnome_print_master_set_paper(gpm,paper);
       if (do_preview==1)
       {
	   GnomePrintMasterPreview *pmp;
	   pmp = gnome_print_master_preview_new(gpm, "Test preview");
	   gtk_signal_connect(GTK_OBJECT(pmp), "destroy", p_order_close_window , NULL);
	   gtk_widget_show(GTK_WIDGET(pmp));
       }
       else
       {
	 gnome_print_master_print(gpm); 
       }
   }

   
     
      /* Fixed in gnome-print-0.37, this line is needed if you want to use 
       *    the program and don't loose your locale when printing or print previewing:
       *        setlocale(LC_ALL,"es_ES@euro");
       */
   
  
}

