/* order-print.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "order-print.h"
#include "extract_order.h"
#include "client_model.h"
#include "warehouse_model.h"

static void print_order_page(GnomePrintContext *pc,Order * order,Participant * p_enterprise,Participant * p_supplier,Warehouse * p_warehouse,gulong from_line,gulong page_number_actual);

/**
 * p_order_multipage:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @order: Pedido a imprimir.
 * @number_of_pages: Nmero de hojas que contendr el pedido.
 * 
 * Rellena el contexto con las hojas del pedido que se deseen imprimir.
 *
 */
void p_order_multipage (GnomePrintContext *pc,Order * order,gulong number_of_pages) {
    gint index_page = 1;
    gchar * temp=NULL;
    gint next_row=0;
    Participant *  p_enterprise;
    Participant *  p_supplier;
    Warehouse *  p_warehouse;


    p_enterprise = m_client_get_participant_by_code (d_order_get_enterprise_code (order));
    p_supplier   = m_client_get_participant_by_code (d_order_get_supplier_code (order));
    p_warehouse = m_warehouse_find_by_codes (d_order_get_warehouse_code (order),d_order_get_enterprise_code (order));


    g_print(_("\nNumber of pages: %lu\n"),number_of_pages);
    while (index_page<=(number_of_pages)) {
	temp = g_strdup_printf(_("Listing, page %d"),index_page);
	gnome_print_beginpage (pc, temp);
	g_free(temp);
	if (index_page == 0) {
	    next_row = 0;
	} else {
	    next_row = P_ORDER_NUMBER_OF_ORDERS*(index_page-1);
	}
	print_order_page (pc,order,p_enterprise,p_supplier,p_warehouse,next_row,index_page);
	gnome_print_showpage (pc);
        index_page++;
    }
    gnome_print_context_close(pc);
    d_participant_clear (p_enterprise);
    d_participant_clear (p_supplier);
    d_warehouse_clear (p_warehouse);

}

/**
 * print_order_page:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @order: Pedido a imprimir.
 * @p_enterprise: Datos del participante asociado a la empresa que solicita el pedido.
 * @p_supplier: Datos del participante asociado al proveedor al que se solicita el pedido.
 * @p_warehouse: Datos del participante asociado al almacn para el que se solicita el pedido.
 * @from_line: Indica la lnea de pedido desde la que se iniciar la impresin de la hoja actual.
 * @current_page_number: Nmero de hoja actual.
 *
 * Indica al contexto los datos del pedido y las posiciones de los mismos para poder ralizar la impresin.
 * Incluye tambin la impresin de la base del pedido.
 *
 */
static void print_order_page(GnomePrintContext *pc,Order * order,Participant * p_enterprise,Participant * p_supplier,Warehouse * p_warehouse,gulong from_line,gulong current_page_number)
{
   GnomeFont *font;

   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }

   gnome_print_setfont (pc, font);

   {
       double colWidth[3] = {75, 110, 80};
       gchar * supplier_number = p_order_get_stringify_supplier_number (order,TRUE);
       gchar * legal_id = p_order_get_stringify_legal_id (p_supplier,TRUE);
       gchar * page_number = g_strdup_printf("%lu",current_page_number);
              

     char *contents[6] = {
       _("Provider No."), _("Tax code/Legal ID"), _("Page No."),
       supplier_number, legal_id, page_number
     };

     p_print_complex_table (pc, font, 25, 700, 2, 3, colWidth, 18, contents, 2, 0);
     g_free(supplier_number);
     g_free(legal_id);
     g_free(page_number);

   }

   {
       double colWidth[3] = {75, 110, 80};
       gchar * date = p_order_get_stringify_date (order,TRUE);
       gchar * order_number = p_order_get_stringify_order_number (order,TRUE);
     char *contents[6] = {
       _("DATE"), _("ORDER CODE"), "",
       date, order_number, ""

     };

     p_print_complex_table (pc, font, 25, 650, 2, 3, colWidth, 18, contents, 2, 0);
     g_free(date);
     g_free(order_number);
     

   }


   {
       gchar ** post_address = p_order_get_stringify_post_address (p_supplier,TRUE);
       gchar * post_address_first_line=g_strdup(post_address[0]);
       gchar * post_address_second_line=g_strdup(post_address[1]);
       gchar * post_address_third_line=g_strdup(post_address[2]);
       gchar * post_address_fourth_line =g_strdup(post_address[3]);

       g_strfreev(post_address);
       p_print_box (pc, 325, 700, 250, 86);
       gnome_print_moveto(pc, 335, 680);
       p_print_show (pc, post_address_first_line);
       gnome_print_moveto(pc, 335, 662);
       p_print_show (pc, post_address_second_line);
       gnome_print_moveto(pc, 335, 644);
       p_print_show (pc, post_address_third_line);
       gnome_print_moveto(pc, 335, 626);
       p_print_show (pc, post_address_fourth_line);
       g_free(post_address_first_line);
       g_free(post_address_second_line);
       g_free(post_address_third_line);
       g_free(post_address_fourth_line);

   }

   {
       gchar * warehouse_nick_aux = p_order_get_stringify_name (p_enterprise,TRUE);
       gchar * warehouse_nick =g_strconcat("SEND TO: ",warehouse_nick_aux,NULL);
       p_print_box (pc, 25, 604, 550, 18);
       gnome_print_moveto(pc, 35, 590);
       p_print_show (pc, warehouse_nick);
       g_free(warehouse_nick_aux);
       g_free(warehouse_nick);
   }

   {
       gdouble colWidth[7]={88, 240, 48, 43, 43, 38, 50};
       GList * list = p_order_get_lines (order,TRUE,from_line,P_ORDER_NUMBER_OF_ORDERS);
       GList * list_it;
       gchar ** contents;
       gint index=1;


       contents = (gchar **) g_malloc0(((P_ORDER_NUMBER_OF_ORDERS+1)*7+1)*sizeof(gchar *));

       contents[0]=g_strdup(_("Reference"));
       contents[1]=g_strdup(_("Description"));
       contents[2]=g_strdup(_("Amount"));
       contents[3]=g_strdup(_("Price"));
       contents[4]=g_strdup(_("Cost"));
       contents[5]=g_strdup(_("% Disc"));
       contents[6]=g_strdup(_("Total"));

       if (list!=NULL) {
	   list_it=g_list_first(list);
	   while ((list_it!=NULL) && (index<=P_ORDER_NUMBER_OF_ORDERS)) {
	       gint i;
	       gchar ** line = (gchar **) list_it->data;
	       for (i=0;i<7;i++) {
		   contents[index*7+i] = g_strdup(line[i]);
	       }
	       list_it = g_list_next(list_it);
	       index++;
	   }
	   gnome_print_setfont (pc, font);

	   p_print_normal_complex_table (pc, font, 25, 580, P_NORMAL_TEMPLATE_LINES+2, 7, colWidth, 23,
				       contents, index, 0);
	   g_list_free(list);
	   g_strfreev(contents);
       }
   }



   p_print_box (pc, 25, 172, 550, 20);
   gnome_print_moveto(pc, 100, 172);
   gnome_print_lineto(pc, 100, 152);

   gnome_print_moveto(pc, 30, 156);
   p_print_show (pc, _("Payment way"));
   gnome_print_moveto(pc, 105, 156);
   gnome_print_moveto(pc, 30, 140);



   {
       gchar *enterprise = p_order_get_stringify_name (p_enterprise,TRUE);
       gchar *enterprise_legal_id = p_order_get_stringify_legal_id (p_enterprise,TRUE);
       gchar *all = g_strconcat(enterprise,_("    TAX CODE:"),enterprise_legal_id,NULL);

       p_print_centered_text (pc, font, 25, 80, 560, all);
       g_free(enterprise);
       g_free(enterprise_legal_id);
       g_free(all);
   }

   gnome_print_gsave(pc);
   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
   {
	g_warning(_("Font cannot be found"));
     }
   gnome_print_setfont(pc,font);

   {
       gdouble colWidth[6] = {100, 90, 90, 75, 80, 115};
       gchar * contents[12] = {_("Costs addition"), _("% EP. Disc."), _("% EP. Disc."), _("Subtotal"), _("Total VAT"), _("Invoice Total"),       "", "", "", "", "", ""

     };

     p_print_complex_table (pc, font, 25, 252, 2, 6, colWidth, 18, contents, 2, 0);

   }
   

   {

     p_print_box (pc,  400, 205,175,20);
   }


      gnome_print_grestore(pc);



}

