/* payment_note-print.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "payment_note-print.h"

/**
 * p_payment_note_one_page:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @payment_note: Nota a imprimir.
 * @from_line: Indica la lnea de nota desde la que se iniciar la impresin de la hoja actual.
 * @n_page: Nmero de hoja actual.
 *
 * Indica al contexto los datos de la nota y las posiciones de los mismos para poder ralizar la impresin de
 * una pgina de la nota. A cada contexto se le indican las lneas que se imprimen a partir de #from_line.
 * Incluye tambin la impresin de la base de la nota.
 *
 */
void p_payment_note_one_page (GnomePrintContext *pc,PaymentNote * payment_note,gint from_line,gint n_page) {
   GnomeFont *font;
   gchar * total = p_payment_note_get_stringify_total (payment_note);




   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }

   gnome_print_setfont (pc, font);

   {
       double colWidth[3] = {75, 110, 80};
       gchar * client_number = p_payment_note_get_stringify_client_number (payment_note);
       gchar * legal_id = p_payment_note_get_stringify_legal_id (payment_note);
       gchar * page_number = g_strdup_printf("%d",n_page);
     char *contents[6] = {
       _("CLIENT No."), _("Tax code/Legal ID"), _("Page No."),
       client_number, legal_id, page_number
     };

     p_print_complex_table (pc, font, 25, 700, 2, 3, colWidth, 18, contents, 2, 0);

   }

   {
       double colWidth[3] = {75, 110, 80};
       gchar * date = p_payment_note_get_stringify_date (payment_note);
       gchar * payment_note_number = p_payment_note_get_stringify_get_number (payment_note);

     char *contents[4] = {
	 _("DATE"), _("No. Note"),
	 date, payment_note_number
     };

     p_print_complex_table (pc, font, 25, 650, 2, 2, colWidth, 18, contents, 2, 0);
   }

   {
       gchar ** post_address = p_payment_note_get_stringify_post_address (payment_note);
       gchar * post_address_first_line=g_strdup(_("Client: "));
       gchar * post_address_second_line=g_strdup(post_address[0]);
       gchar * post_address_third_line=g_strdup(_("Vehicle: "));
       gchar * post_address_fourth_line =p_payment_note_get_stringify_license_number (payment_note);

       g_strfreev(post_address);
       p_print_box (pc, 325, 700, 250, 86);
       gnome_print_moveto(pc, 335, 680);
       p_print_show (pc, post_address_first_line);
       gnome_print_moveto(pc, 335, 662);
       p_print_show (pc, post_address_second_line);
       gnome_print_moveto(pc, 335, 644);
       p_print_show (pc, post_address_third_line);
       gnome_print_moveto(pc, 335, 626);
       p_print_show (pc, post_address_fourth_line);
   }

   {
       gdouble colWidth[7]={93, 240, 43, 43, 43, 38, 50};
       GList * list = p_payment_note_get_lines (payment_note);
       GList * list_it;
       gchar ** contents;
       gint index=1;
       gint current_line = 0;
       //       guint length = g_list_length(list);


       contents = (gchar **) g_malloc0(((P_NORMAL_TEMPLATE_LINES+1)*7+1)*sizeof(gchar *));

       contents[0]=g_strdup(_("Reference"));
       contents[1]=g_strdup(_("Description"));
       contents[2]=g_strdup(_("Amount"));
       contents[3]=g_strdup(_("Price"));
       contents[4]=g_strdup(_("Cost"));
       contents[5]=g_strdup(_("% Disc."));
       contents[6]=g_strdup(_("Total"));

       list_it=g_list_first(list);
       while ((list_it!=NULL) && (current_line<from_line)) {
	   list_it = g_list_next(list_it);
           current_line++;
       }

       while ((list_it!=NULL) && (index<=(P_NORMAL_TEMPLATE_LINES))) {
	   gint i;
	   gchar ** line = (gchar **) list_it->data;
	   for (i=0;i<7;i++) {
	       contents[index*7+i] = g_strdup(line[i]);
	   }
	   list_it = g_list_next(list_it);
	   index++;
       }

     p_print_normal_complex_table (pc, font, 25, 580, P_NORMAL_TEMPLATE_LINES+2, 7, colWidth, 23,
			  contents, index, 0);
   }

   {
       gchar * license_number = d_repair_get_stringify_license_number (d_payment_note_get_repair_detail (payment_note),TRUE);
       p_print_box (pc, 25, 212, 265, 23);
       gnome_print_moveto(pc, 35, 198);
       p_print_show (pc, _("License number:"));
       gnome_print_moveto(pc, 135, 198);
       p_print_show (pc, license_number);

       g_free(license_number);
   }
   
      gnome_print_gsave(pc);                        /*BOOK*/  /*8*/
   font = gnome_font_new_closest ("Helvetica", GNOME_FONT_BOOK, 1, 9);
   if (font == NULL)
     {	
	g_warning(_("Font cannot be found"));
     }
   gnome_print_setfont(pc,font);

   {
       gdouble colWidth[6] = {115};
       gchar * contents[12] = {_("Total Note"),
       g_strconcat(total,_(" Euros"),NULL) };

     p_print_complex_table (pc, font, 460, 182, 2, 1, colWidth, 18, contents, 2, 0);
   }

/*   {
     double colWidth[2] ={100, 80};
     char *contents[4] ={_("Taxable base"), _("VAT amount"),
			  *base_imponible,  *iva};

     p_print_complex_table (pc, font, 90, 212, 2, 2, colWidth, 18, contents, 2, 0);
   }
*/
      gnome_print_grestore(pc);

}

/**
 * p_payment_note_multipage:
 * @pc: Contexto en el que se indicar lo que se quiere imprimir.
 * @payment_note: Nota a imprimir.
 *
 * Permite realizar la impresin de una nota de varias pginas, para poder imprimir todas
 * las lneas de la nota.
 *
 */
void p_payment_note_multipage (GnomePrintContext *pc,PaymentNote * payment_note) {
    gint index_page = 1;
    gchar * temp=NULL;
    gint next_row=0;
    gint number_of_lines = p_payment_note_get_number_of_lines (payment_note);
    gint number_of_pages=0;

    g_print(_("\nNumber of lines%d\n"),number_of_lines);
    if (number_of_lines%P_NUMBER_OF_LINES!=0) {
	number_of_pages = number_of_lines/P_NUMBER_OF_LINES + 1;
    } else {
	number_of_pages = number_of_lines/P_NUMBER_OF_LINES;
    }


    g_print(_("\nNumber of pages: %d\n"),number_of_pages);
    while (index_page<=(number_of_pages)) {
	temp = g_strdup_printf(_("Invoice, page %d"),index_page);
	gnome_print_beginpage (pc, temp);
	g_free(temp);
	if (index_page == 0) {
	    next_row = 0;
	} else {
	    next_row = P_NUMBER_OF_LINES*(index_page-1);
	}
	p_payment_note_one_page (pc,payment_note,next_row,index_page);
	gnome_print_showpage (pc);
        index_page++;
    }
    gnome_print_context_close(pc);



}


