
/* print_utils.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */
#ifndef _PRINT_UTILS_
#define _PRINT_UTILS_

#include "config.h"

#include <stdlib.h> /* para mbstowcs */
#include <gal/unicode/gunicode.h> /* para unicode */

#include <gnome.h>
#include <sys/stat.h>
#include <util.h>

#include <libgnomeprint/gnome-printer.h>
#include <libgnomeprint/gnome-print.h>
#include <libgnomeprint/gnome-print-meta.h>
#include <libgnomeprint/gnome-print-preview.h>
#include <libgnomeprint/gnome-print-pixbuf.h>
#include <libgnomeprint/gnome-font.h>
#include <libgnomeprint/gnome-printer-dialog.h>
#include <libgnomeprint/gnome-print-master.h>
#include <libgnomeprint/gnome-print-master-preview.h>
#include <libgnomeprint/gnome-print-dialog.h>

#include <gdk-pixbuf/gdk-pixbuf.h>

/* The font to use */

/**
 * P_NORMAL_TEMPLATE_LINES:
 *
 * Nmero de lneas de factura que se permiten por cada hoja de factura con plantilla normal.
 *
 */
#define P_NORMAL_TEMPLATE_LINES 12

/**
 * P_NUMBER_OF_LINES:
 *
 * Nmero de lneas que se permiten para resto de plantillas de impresin por hoja.
 *
 */
#define P_NUMBER_OF_LINES 12


/**
 * P_DEFAULT_FONT:
 *
 * Fuente por defecto.
 *
 */
#define P_DEFAULT_FONT "Times"

/* The font size to use for normal text. */

/**
 * P_NORMAL_FONT_SIZE:
 *
 * Tamao de letra de la letra normal.
 *
 */
#define P_NORMAL_FONT_SIZE 12



/**
 * _PROPER_I18N:
 *
 * Variable definida para realizar transformaciones de texto a ISO5589, si no estuviese definida se realizaran
 * transformaciones a UTF8.
 *
 */
#define _PROPER_I18N


/**
 * P_READ_CHUNK_SIZE:
 *
 * Tamao de memoria a leer.
 *
 */
#define P_READ_CHUNK_SIZE 8192


/**
 * P_LEFT_MARGIN:
 *
 * Margen izquierdo de la hoja.
 *
 */
#define P_LEFT_MARGIN 50

/**
 * P_RIGHT_MARGIN:
 *
 * MArgen derecho de la hoja.
 *
 */
#define P_RIGHT_MARGIN 540

/**
 * P_CELL_LEFT_MARGIN:
 *
 * Margen izquierdo en la impresin de celdas.
 *
 */
#define P_CELL_LEFT_MARGIN 2

/**
 * P_CELL_BOTTOM_MARGIN:
 *
 * Margen inferior en la impresin de celdas.
 *
 */
#define P_CELL_BOTTOM_MARGIN 2


enum align_box
{  
   ALIGN_LEFT=1,
     ALIGN_RIGHT,
     ALIGN_CENTER,
     ALIGN_BORDER= 1<<8
};

/* Utils.h API */
void p_print_show_xy_and_move (GnomePrintContext *pc, GnomeFont *font, 
			     double x, double y, char * text);
void p_print_n_text (GnomePrintContext *pc, GnomeFont *font, 
		   double n, double x, double y, char **texts);
void p_print_two_text (GnomePrintContext *pc, GnomeFont *font, 
		     double x, double y, char *text1, char *text2);    
void p_print_centered_text (GnomePrintContext *pc, GnomeFont *font,
			  double x, double y, double width, char* text);
void p_print_box (GnomePrintContext *pc, double x, double y,
		double width, double height);
void p_black_box (GnomePrintContext *pc, double x, double y,
		      double width, double height);
void p_text_centered_in_box (GnomePrintContext *pc, GnomeFont *font,
				 double x, double y, double width, double height,
				 char* text);
void p_print_table (GnomePrintContext *pc, GnomeFont *font, double x, double y,
		  double numRows, double width, double rowHeight, char ** contents);
void p_print_table_with_condition (GnomePrintContext *pc, GnomeFont *font, double x, double y,
		  double numRows, double width, double rowHeight, char ** contents,gboolean show);

void p_print_complex_table (GnomePrintContext *pc, GnomeFont *font, 
			  double x, double y,
			 double numRows, double numCols, 
			  double *colWidth, double rowHeight, 
			  char **contents, double numRowsWithContent,
			  double printLines);
void p_print_normal_complex_table (GnomePrintContext *pc, GnomeFont *font,
			  double x, double y,
			 double numRows, double numCols, 
			  double *colWidth, double rowHeight, 
			  char **contents, double numRowsWithContent,
			  double printLines);

int p_print_show (GnomePrintContext *pc, char const *text);
void p_print_image (GnomePrintContext *ctx, double x, double y, 
		  char* filename, float scale);
void p_print_text (GnomePrintContext *pc, GnomeFont *font, const char *text,
		enum align_box align, double l, double r, double t, double b);    
void p_print_image_filling (GnomePrintContext *ctx, double x, double y, char* filename);
void p_print_text_righted (GnomePrintContext *pc, GnomeFont *font,
			 double x ,double y,double width, char* text) ;
void p_print_images_horizontal (GnomePrintContext *ctx, double x, double y, char* filename,char * filename2);
#endif


