/* accident_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#define TABLE         "Siniestro"
#define ATTRIBUTES    "referenciaSiniestro as referenciasiniestroaccident , \
    fecha as fechaaccident , \
    lugar as lugaraccident , \
    ramo as ramoaccident, \
    danos as danosaccident , \
    modo as modoaccident, \
    codprov as codprovaccident, \
    codcausa as codcausaaccident, \
    foto1 as foto1accident, \
    descripcion as descripcionaccident, \
    fechaApertura as fechaAperturaaccident, \
    foto2 as foto2accident"



#include "accident_model.h"
#include "parameter_manager.h"

/**
 * m_accident_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto y con un alias cada uno.
 */
gchar *m_accident_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_accident_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_accident_get_table () {
    return TABLE;
}


/**
 * m_accident_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Accident con los campos rellenados con los datos de la base de datos.
 */
Accident *m_accident_create (GdaRecordset *rs) {
    GdaField * field;
    gchar *temp;
//    Accident *accident = (Accident *) g_malloc (sizeof(Accident));

//    accident_init(accident);

    Accident * accident = d_accident_create_empty();

    field = gda_recordset_field_name(rs,"referenciasiniestroaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->accident_reference = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    accident->date = (*my_time);
	    g_free(my_time);
	    g_free(temp);
	    my_time = NULL;
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaAperturaaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    accident->open_date = (*my_time);
	    g_free(my_time);
	    g_free(temp);
	    my_time = NULL;
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"lugaraccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->place = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"ramoaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->field = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"danosaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->damages = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"modoaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->way = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codprovaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->province_code = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codcausaaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->reason_code = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"foto1accident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->photo1 = g_strdup(temp);
            g_strstrip(accident->photo1);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"foto2accident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->photo2 = g_strdup(temp);
            g_strstrip(accident->photo2);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"descripcionaccident");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    temp=gda_stringify_value(NULL, 0, field);
	    accident->description = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }


    return accident;
}

