
/* charge_invoice_line_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "config.h"
#include "util.h"
#include "order_line_model.h"
#include "charge_invoice_line_model.h"

#define TABLE         "LineaFacturaCargo"
#define ATTRIBUTES    "codigoProveedor as codigoProveedorLineaFacturaCargo, \
    numeroFactura as numeroFacturalineafacturacargo, \
    numeroLineaFactura as numeroLineaFacturalineafacturacargo, \
    descripcion as descripcionfacturacargo, \
    total as totalfacturacargo, \
    tipoIva as tipoIVAlineafacturacargo"



/**
 * m_charge_invoice_line_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_charge_invoice_line_get_table () {
    return TABLE;
}

/**
 * m_charge_invoice_line_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_charge_invoice_line_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_charge_invoice_line_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns: Un objeto #ChargeInvoiceLine con los campos rellenados con los datos de la base de datos.
 */
ChargeInvoiceLine  *m_charge_invoice_line_create (GdaRecordset *rs){
    GdaField    *field;
    ChargeInvoiceLine *charge_invoice_line = d_charge_invoice_line_create_empty ();

    field = gda_recordset_field_name(rs,"numeroLinealineaFacturaC");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    charge_invoice_line->line_number = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"codigoproveedorlineaFacturaC");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    charge_invoice_line->supplier_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "descripcionlineaFacturaC");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    charge_invoice_line->description = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "numeroFacturalineaFacturaC");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
                    charge_invoice_line->invoice_number = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }
	    field = gda_recordset_field_name(rs, "tipoIVAFacturaC");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    charge_invoice_line->vat_type = atof(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "totalFacturaC");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    charge_invoice_line->total = atof(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }


	    charge_invoice_line -> order_line = m_order_line_create (rs);
	    
	} else {
	    gda_field_free(field);
	}
    }
    return charge_invoice_line;
}

/**
 * m_charge_invoice_line_is_in_rs:
 * @rs: Record set en el que busca la lnea de factura de cargo.
 *
 * Comprueba si el record set contiene una linea de factura de cargo
 *
 * Returns: %TRUE si se encuentra en el record_set un nmero de lnea, %FALSE en otro caso.
 */
gboolean m_charge_invoice_line_is_in_rs (GdaRecordset *rs) {
  
  GdaField *field;
  gboolean result = FALSE;

  field = gda_recordset_field_name(rs,"numeroLineaFacturalineaFacturaCargo");

  if (field != NULL) {
    if (!gda_field_is_null(field))
      result = TRUE;
    
    gda_field_free(field);
  }

  return result;
}

/**
 * m_charge_invoice_line_find_by_invoice: 
 * @supplier_code: Cdigo del proveedor, que forma parte de la clave de factura de cargo.
 * @invoice_number: Nmero de la factura de cargo de la que queremos el nmero de factura.
 *
 * Funcin que busca las lneas de una factura de cargo (de pago) por cdigo de proveedor y nmero de
 * factura. OJO: El nmero de la factura se refiere aqu al cdigo de la factura de la factura en cuestin.
 * Se mantiene como nombre en la lnea de factura nombreFactura para mentener compatibilidad con las BD antiguas,
 * ya que este bug se corrigi en fechas posteriores a la implantacin de la aplicacin en las tiendas.
 *
 * Returns: Lista de lneas de factura de cargo pertenecientes a esa factura.
 */
ChargeInvoiceLine **  m_charge_invoice_line_find_by_invoice (gchar *supplier_code, gchar *invoice_number) {
  GdaCommand         *my_command;
  gchar              *gchar_query;
  GdaConnection      *my_connection = m_model_get_connection ();
  GdaRecordset       *record_set = NULL;
  ChargeInvoiceLine ** charge_invoice_lines;
  ChargeInvoiceLine * charge_invoice_line;
  gulong * tuples;
  gint index;
  
  my_command = gda_command_new();
  tuples = (gulong *) g_malloc0(sizeof(gulong));
  
  gda_command_set_connection(my_command,my_connection);
  
  gchar_query = g_strconcat("select ",
			    "codigoPedido as codigopedidoorderline,",
			    "codigoEmpresa as codigoempresaorderline,",
			    "codigoAlmacen as codigoalmacenorderline,",
			    "numeroLinea as numeroLineaorderline,",
			    "numeroAlbaran as numeroAlbaranorderline,",
			    "numeroFacturaCargo as numeroFacturaorderline,",
			    "numeroLineaCargo as numeroLineaCargoaorderline,",
			    "codigoProveedorFacturaCargo as codigoProveedorFCorderline,",
			    "to_char(unidades,'999999999D999') as unidadesorderline,",
			    "numerofactura as numeroFacturalineaFacturaC,",
			    "lineafacturacargo.codigoproveedor as codigoproveedorlineaFacturaC,",
			    "numerolineafactura as numeroLinealineaFacturaC,",
			    "producto.codigoProducto as codigoproductoproduct,",
			    "producto.descripcion as descripcionproduct,",
			    "pieza.eurocode as eurocodepiece,",
			    "lineafacturacargo.descripcion as descripcionlineaFacturaC,",
			    "to_char(lineafacturacargo.tipoiva,'999D99') as tipoIVAFacturaC,",
			    "to_char(total,'9999999999D99') as totalFacturaC",
			    " from lineafacturacargo left outer join lineapedido on (lineafacturacargo.numerofactura = lineapedido.numerofacturacargo and lineafacturacargo.numerolineafactura = lineapedido.numerolineacargo and lineafacturacargo.codigoproveedor = lineapedido.codigoproveedorfacturacargo)",
			    " left outer join producto on (lineapedido.codigoproducto = producto.codigoproducto) ",
			    " left outer join pieza on (lineapedido.codigoproducto = pieza.codigoproducto) ",
			    " where lineafacturacargo.codigoproveedor = ",supplier_code," and numerofactura = '",invoice_number,"'",
			    NULL);
  
  //g_print("Query: %s\n",gchar_query);
  
  m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);
  
  
  if (record_set!=NULL) {
    if (*tuples!=0) {
      
      charge_invoice_lines = (ChargeInvoiceLine **) g_malloc0(sizeof(ChargeInvoiceLine*)*(*tuples+1));
      index = gda_recordset_move_first(record_set);
      while (!gda_recordset_eof(record_set)) {
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	charge_invoice_line = m_charge_invoice_line_create (record_set);
	charge_invoice_lines[index-1] = charge_invoice_line;
	index = gda_recordset_move_next(record_set);
      }
      charge_invoice_lines[*tuples]=NULL;
    } else {
      charge_invoice_lines=NULL;
    }
    
    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
    
  } else {
    charge_invoice_lines=NULL;
  }
  
  igalia_command_free(my_command);
  
  g_free(tuples);
  g_free(gchar_query);
  
  return charge_invoice_lines;
  
}


/**
 * m_charge_invoice_line_convert_to_glist: 
 * @list: Representa un puntero a una lista de lneas de factura de cargo. Esta %GList ser la que
 * se rellene con las lneas de factura de cargo.
 * @charge_invoice_lines:Array de lneas de factura de cargo que deseamos transformar.
 * 
 * Funcin utilizada para transformar un array de lneas de una factura de cargo a una %GList de lneas. Por eso,
 * para poder modificar la %GList, es necesario pasar como parmetro un puntero a la misma.
 *
 */
void m_charge_invoice_line_convert_to_glist (GList **list, ChargeInvoiceLine ** charge_invoice_lines) {
  gint index = 0;

  if (charge_invoice_lines!=NULL) {   
    ChargeInvoiceLine **charge_lines_aux = d_charge_invoice_line_duplicate (charge_invoice_lines);
    GList *last = g_list_last(*list);
    gint last_pos = 0;

    if (last != NULL ) last_pos = (g_list_position(*list, last))+1;

    while (charge_lines_aux[index]!=NULL) {
      *list = g_list_insert(*list, charge_lines_aux[index], last_pos++);

      index++;
    }

    g_free(charge_lines_aux);	
  }

}


/**
 * m_charge_invoice_line_convert_glist_on_array: 
 * @list: Representa una lista de lneas de factura de cargo. 
 *
 * Funcin utilizada para transformar una %GList de lneas de una factura de cargo a un array de lneas . Esta funcin
 * realiza la operacin inversa de %convert_charge_lines_glist.
 *
 * Returns: Array de lneas de factura de cargo.
 */
ChargeInvoiceLine **m_charge_invoice_line_convert_glist_on_array (GList *list) {

  ChargeInvoiceLine **charge_invoice_lines = NULL;
  ChargeInvoiceLine **lines = NULL;

  if (list != NULL) {
    GList *element = g_list_last(list);
    gint number_elements = g_list_position(list, element);

    guint line;
    
    charge_invoice_lines = (ChargeInvoiceLine **) g_malloc0(sizeof(ChargeInvoiceLine*)*(number_elements+2));
    
    charge_invoice_lines[number_elements+1] = NULL;
    
    for (line = 0; line <= number_elements; line ++) {

      charge_invoice_lines[line] = g_list_nth_data(list,line);

    }

    lines = d_charge_invoice_line_duplicate (charge_invoice_lines);
   
    g_free(charge_invoice_lines);
  }
  
  return lines;
}


/** 
 * m_charge_invoice_line_associate_charge_invoice_to_glist:
 * @list: Lista enlazada que contiene las lneas de la factura de cargo.
 * @invoice_number: Cdigo de la factura de cargo que deseamos asociar a las lneas
 * @supplier_code: Cdigo del proveedor que emite la factura de cargo.
 *
 * Asocia una lista enlazada de lneas de una factura de cargo a la factura correspondiente.
 *
 */
void m_charge_invoice_line_associate_charge_invoice_to_glist (GList *list, gulong invoice_number, gulong supplier_code) {

  GList *element = g_list_first (list);
  gint line = 1;
 
  while (element != NULL) {
    d_charge_invoice_line_set_invoice_number (element->data, invoice_number);
    d_charge_invoice_line_set_supplier_code (element->data, supplier_code);
    element = g_list_next(element);
    line ++;
  }

}


/** 
 * m_charge_invoice_line_associate_charge_invoice:
 * @lines: %Array que contiene las lneas de la factura de cargo.
 * @invoice_number: Cdigo de la factura de cargo que deseamos asociar a las lneas
 * @supplier_code: Cdigo del proveedor que emite la factura de cargo.
 * 
 * Asocia un array de lneas de una factura de cargo a la factura correspondiente.
 *
 */
void m_charge_invoice_line_associate_charge_invoice (ChargeInvoiceLine **lines, gulong invoice_number, gulong supplier_code) {

  gint line = 0;
  if (lines!=NULL) {
      while (lines[line] != NULL) {
	  d_charge_invoice_line_set_invoice_number (lines[line], invoice_number);
	  d_charge_invoice_line_set_supplier_code (lines[line], supplier_code);
	  line ++;
      }
  }

}


/** 
 * m_charge_invoice_line_insert: 
 * @charge_line: Lnea de factura de cargo.
 * @command: Comando de la base de datos que ejecutar la queey en la misma.
 * @connection: Conexin a la base de datos.
 * @update: Indica si se debe insertar o modificar.
 * 
 * Inserta o modifica una lnea de factura de cargo en la BD dentro de una transaccin.
 *
 * Returns: %TRUE si se ha podido insertar la lnea en la base de datos, %FALSE si no.
 */
gboolean m_charge_invoice_line_insert (ChargeInvoiceLine *charge_line, GdaCommand * command, GdaConnection *connection, gboolean update) {

  gboolean result = FALSE;
  if(charge_line != NULL) {
      gchar * query = NULL;
      gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
      GdaRecordset *record_set = NULL;
      gulong invoice_number_gulong= d_charge_invoice_line_get_invoice_number (charge_line);
      gchar *invoice_number = NULL;

      gulong invoice_line_number_gulong = d_charge_invoice_line_get_line_number (charge_line);
      gchar *invoice_line_number = NULL;
      gulong supplier_code_gulong = d_charge_invoice_line_get_supplier_code (charge_line);
      gchar *supplier_code = NULL;
      gchar *description = d_charge_invoice_line_get_description (charge_line);
      gdouble vat_type_gdouble = d_charge_invoice_line_get_vat_type (charge_line);
      gchar *vat_type = NULL;
      gdouble total_gdouble = d_charge_invoice_line_get_total (charge_line);
      gchar *total = NULL;
      OrderLine *order_line = d_charge_invoice_line_get_order_line (charge_line);


      if (invoice_number_gulong!=ULONG_MAX)
	  invoice_number=g_strdup_printf("%lu",invoice_number_gulong);

      if (invoice_line_number_gulong != ULONG_MAX)
	  invoice_line_number = g_strdup_printf("%lu",invoice_line_number_gulong);
      else invoice_line_number = g_strdup("NULL");

      if (supplier_code_gulong != ULONG_MAX)
	  supplier_code = g_strdup_printf("%lu",supplier_code_gulong);
      else supplier_code = g_strdup("NULL");

      if (invoice_number == NULL) invoice_number = g_strdup("NULL");
      else {
	  gchar *invoice_number_aux = NULL;

	  invoice_number_aux = g_strconcat(invoice_number,NULL);
	  g_free(invoice_number);
	  invoice_number = invoice_number_aux;
	  invoice_number_aux = NULL;
      }

      if (description == NULL) description = g_strdup("NULL");
      else{
	  gchar *description_aux = NULL;

	  description_aux = g_strconcat("'",description,"'",NULL);
	  g_free(description);
	  description = description_aux;
	  description_aux = NULL;
      }

      if (vat_type_gdouble != -1)
	  vat_type = g_strdup_printf("%.2f",vat_type_gdouble);
      else vat_type = g_strdup("NULL");

      u_commas_to_points (vat_type);

      if (total_gdouble != G_MINDOUBLE)
	  total = g_strdup_printf("%.2f",total_gdouble);
      else total = g_strdup("NULL");

      u_commas_to_points (total);

      if (update) {

	  query = g_strconcat("UPDATE ",m_charge_invoice_line_get_table ()," SET ",
			      " descripcion = ",description,",",
			      " tipoiva = ",vat_type,",",
			      " total = ",total,
			      " WHERE numerofactura = ",invoice_number,
			      " AND codigoproveedor = ",supplier_code,
			      " AND numerolineafactura = ",invoice_line_number,
			      NULL);

    } else {

	  invoice_line_number = m_model_nextval_sequence ("seqlineafacturacargo",connection);

	  query = g_strconcat("INSERT INTO ",m_charge_invoice_line_get_table ()," VALUES (",
			      invoice_number,",",
			      supplier_code,",",
			      invoice_line_number,",",
			      description,",",
			      vat_type,",",
			      total,")",
			      NULL);

	  g_print("\n charge_invoice_line (insert_charge_invoice_line): %s \n", query);
      if (invoice_line_number != NULL) {
	  gchar ** stop=(gchar **)g_malloc0(sizeof(gchar *));

	  invoice_line_number_gulong = strtoul(invoice_line_number,stop,10);
	  g_free(stop);
	  d_charge_invoice_line_set_line_number (charge_line, invoice_line_number_gulong);
      }

    }

      result = m_model_command_execute (&record_set, query, connection,
					command,tuples,0,FALSE);

      if ((order_line != NULL)&&(d_order_line_get_code (order_line)!=ULONG_MAX)) {
	  d_order_line_set_charge_invoice_number (order_line,invoice_number_gulong);
	  d_order_line_set_charge_invoice_supplier_code (order_line,supplier_code_gulong);
	  d_order_line_set_charge_line_number (order_line,invoice_line_number_gulong);

	  result = result && m_order_line_update (order_line, ULONG_MAX, connection, command);

      }


      if (record_set != NULL) gda_recordset_free(record_set);

      g_free(invoice_number);
      g_free(supplier_code);
      g_free(invoice_line_number);
      g_free(description);
      g_free(vat_type);
      g_free(total);
      g_free(query);
      g_free(tuples);

  }

  return result;
}


/** 
 * m_charge_invoice_line_is_new: 
 * @invoice_number: Cdigo de la factura a la que referencia la lnea
 * @supplier_code: Proveedor que emiti la factura de cargo
 * @invoice_line_number: Nmero de la lnea de cargo en cuestin.
 * 
 * Comprueba la existencia de una lnea de una factura de cargo a partir de la clave de
 * la lnea de la factura.
 *
 * Returns: %TRUE si ya existe en la base de datos, %FALSE si no existe.
 */
gboolean m_charge_invoice_line_is_new (gulong invoice_number, gchar *supplier_code, gchar *invoice_line_number){

  gboolean result = TRUE;
  GdaCommand *command = gda_command_new();
  gchar *gchar_query = NULL;
  GdaConnection *my_connection = m_model_get_connection ();
  GdaRecordset *record_set = NULL;
  gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
  gchar * invoice_number_char =g_strdup_printf("%lu",invoice_number);

  gda_command_set_connection(command,my_connection);

  gchar_query = g_strconcat("SELECT numerolineafactura FROM lineafacturacargo",
			    " WHERE numerofactura = '", invoice_number_char,"'",
			    " AND numerolineafactura = ", invoice_line_number,
			    " AND codigoproveedor = ", supplier_code,
			    NULL);

  //g_print("\n charge_invoice_line (is_new_charge_line): %s\n",gchar_query);

  m_model_command_execute (&record_set, gchar_query, my_connection,
			command,tuples,0,TRUE);

  igalia_command_free(command);

  result = !(*tuples);

  if (record_set != NULL) 
    gda_recordset_free(record_set);

  g_free(gchar_query);
  g_free(tuples);
  g_free(invoice_number_char);
  return result;

}


/** 
 * m_charge_invoice_line_set_new: 
 * @charge_lines: Lista enlazada con lneas de factura que se van a borrar.
 * @charge_invoice: Factura de cargo de la que vamos a obtener las nuevas lneas de factura de cargo.
 * 
 * A partir de las lneas de factura de cargo borra las lineas de la %GList
 * y las reemplaza por las de las de la factura de cargo.
 *
 */
void m_charge_invoice_line_set_new (GList **charge_lines, ChargeInvoice *charge_invoice) {
  ChargeInvoiceLine **charge_invoice_lines = d_charge_invoice_get_charge_invoice_lines (charge_invoice);
  
  m_charge_invoice_line_delete_glist (charge_lines);
  
  m_charge_invoice_line_convert_to_glist (charge_lines, charge_invoice_lines);
  
}


/** 
 * m_charge_invoice_line_delete_glist: 
 * @charge_lines: Puntelor a la lista enlazada de lneas de facturas de cargo.
 *
 * Elimina las lneas de factura de la lista enlazada apuntada por el puntero
 * charge_lines.
 *
 */
void m_charge_invoice_line_delete_glist (GList **charge_lines) {

  GList *element = g_list_first (*charge_lines);
  gint line = 1;
 
  while (element != NULL) {
    d_charge_invoice_line_clear ((ChargeInvoiceLine *)element->data);
    element->data = NULL;
    element = g_list_next(element);
    line ++;
  }

  g_list_free (*charge_lines);
  *charge_lines = NULL;  
  
}


/** 
 * m_charge_invoice_line_delete:
 * @charge_invoice_line: Lnea de factura de cargo en cuestin.
 * @enter_code_gulong: Cdigo de la empresa en la que se ejecuta la operacin.
 * @shop_code_gulong:Cdigo de la tienda en la que se ejecuta  la operacin.
 * 
 * Realiza el borrado de las lneas de factura de cargo. El cdigo de empresa y tienda se utiliza para
 * el log.
 *
 * Returns: %TRUE si se ha borrado correctamente, %FALSE en otro caso.
 */
gboolean m_charge_invoice_line_delete (ChargeInvoiceLine *charge_invoice_line, gulong enter_code_gulong, gulong shop_code_gulong) {
  gboolean result = FALSE;

  if (charge_invoice_line != NULL) {
    gchar * query;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong invoice_number_gulong = d_charge_invoice_line_get_invoice_number (charge_invoice_line);
    gchar *invoice_number = NULL;
    gulong invoice_line_number_gulong = d_charge_invoice_line_get_line_number (charge_invoice_line);
    gchar *invoice_line_number = NULL;    
    gulong supplier_code_gulong = d_charge_invoice_line_get_supplier_code (charge_invoice_line);
    gchar *supplier_code = NULL;

    if (invoice_number_gulong!=ULONG_MAX)
	invoice_number = g_strdup_printf("%lu",invoice_number_gulong);
    else invoice_number = g_strdup("NULL");

    if (invoice_line_number_gulong != ULONG_MAX)
      invoice_line_number = g_strdup_printf("%lu",invoice_line_number_gulong);
    else invoice_line_number = g_strdup("NULL");

    if (supplier_code_gulong != ULONG_MAX)
      supplier_code = g_strdup_printf("%lu",supplier_code_gulong);
    else supplier_code = g_strdup("NULL");

    if (invoice_number == NULL) invoice_number = g_strdup("NULL");
    else {
      gchar *invoice_number_aux = NULL;

      invoice_number_aux = g_strconcat("'",invoice_number,"'",NULL);
      g_free(invoice_number);
      invoice_number = invoice_number_aux;
      invoice_number_aux = NULL;
    }

    gda_command_set_connection(command,connection);

    m_model_connection_begin_mutex_transaction_logged (connection,enter_code_gulong,shop_code_gulong, ULONG_MAX);

    query = g_strconcat("UPDATE LINEAPEDIDO SET numeroFacturaCargo=NULL,numeroLineaCargo=NULL,codigoProveedorFacturaCargo=NULL",
			    " WHERE numerofacturaCargo = ", invoice_number,
			    " AND numerolineaCargo = ", invoice_line_number,
			    " AND codigoproveedorFacturaCargo = ", supplier_code,
			    NULL);

    //g_print("Query modificado pedido: %s\n",query);


    result = m_model_command_execute (&record_set, query, connection,
                                   command,tuples,0,FALSE);

    g_free(query);

    if (record_set!=NULL)
	if (GDA_IS_RECORDSET(record_set))
	    gda_recordset_free(record_set);

    query = g_strconcat("DELETE FROM ",m_charge_invoice_line_get_table (),
			    " WHERE numerofactura = ", invoice_number,			      
			    " AND numerolineafactura = ", invoice_line_number,
			    " AND codigoproveedor = ", supplier_code,
			    NULL);

    //g_print(_("\nLine delete query: %s\n"),query);

    result = result &&m_model_command_execute (&record_set, query, connection,
                                   command,tuples,0,FALSE);



    if (record_set!=NULL)
	if (GDA_IS_RECORDSET(record_set))
	    gda_recordset_free(record_set);

    result = m_model_execute_mutex_modification_logged (connection) && result;
    
    g_free(query);
    g_free(tuples);
    g_free(invoice_number);
    g_free(supplier_code);
    g_free(invoice_line_number);
    
    igalia_command_free(command);

  }

  return result;
}
