/* client_model.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _CLIENT_MODEL_
#define _CLIENT_MODEL_

#include <gda-client.h>
#include <gda-field.h>
#include "gda-recordset.h"
#include "model.h"
#include "client.h"
#include "participant.h"
#include "enterprise.h"
#include "supplier.h"
#include "consortium.h"
#include "company.h"
#include "particular_object.h"
#include "garage.h"
#include "gda_igalia_util.h"

/**
 * M_CLIENT_PERSON_SEARCH:
 *
 * Codigo que indica el tipo de bsqueda por personas.
 *
 */
#define M_CLIENT_PERSON_SEARCH 1
/**
 * M_CLIENT_ENTERPRISE_SEARCH:
 *
 * Codigo que indica el tipo de bsqueda por empresas.
 *
 */
#define M_CLIENT_ENTERPRISE_SEARCH 2
/**
 * C_NO_SPECIFIED_SEARCH:
 *
 * Codigo que indica el tipo de bsqueda por ambos (empresas y personas).
 *
 */
#define C_NO_SPECIFIED_SEARCH 0

/**
 * M_CLIENT_PERSON_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente es un particular.
 *
 */
#define M_CLIENT_PERSON_CODE 0
/**
 * M_CLIENT_ENTERPRISE_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente es una empresa.
 *
 */
#define M_CLIENT_ENTERPRISE_CODE 1
/**
 * M_CLIENT_SUPPLIER_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente es un proveedor.
 *
 */
#define M_CLIENT_SUPPLIER_CODE 2
/**
 * M_CLIENT_COMPANY_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente es una compaa de seguros.
 *
 */
#define M_CLIENT_COMPANY_CODE 3
/**
 * M_CLIENT_CONSORTIUM_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente es un consorcio.
 *
 */
#define M_CLIENT_CONSORTIUM_CODE 4
/**
 * M_CLIENT_GARAGE_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente es un taller.
 *
 */
#define M_CLIENT_GARAGE_CODE 5
/**
 * M_CLIENT_SHOP_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente es una tienda.
 *
 */
#define M_CLIENT_SHOP_CODE 6
/**
 * M_CLIENT_ALL_PARTICIPANT_CODE:
 *
 * Cdigo que indica el tipo de cliente por el que se realizar una bsqueda. En este caso el cliente engloba todos los tipos.
 *
 */
#define M_CLIENT_ALL_PARTICIPANT_CODE 7

/* tipo Busqueda es : 0 si no se especifica nada, 1 si es para personas y 2 si es para empresas */
/**
 * CoditionFindCLients - Estructura que acta como filtro para la bsqueda de clientes.
 */
typedef struct conditionFindClientsStruct {
    gchar *enterprise_code;
    gchar *shop_code;
    gchar *code;/* Cdigo del cliente*/
    gchar *complete_name;/* Nombre del cliente*/
    gint search_type;/*Tipo de bsqueda que se desea relizar*/
    gboolean insurance;/*Bsqueda con seguro o sin seguro*/
    gchar *initial_date;
    gchar *ending_date; 
} ConditionFindClients;


gchar *m_client_get_table (void);
gchar *m_client_get_attributes (void);
Client * m_client_create (GdaRecordset *rs);
Client **  m_client_find_by_delivery_payment_condition (ConditionFindClients *condition);
void m_client_clear_array (Client ** clients);
gboolean m_client_insert (gpointer *client_data, Client *client, gulong type,GList * accounts);
gboolean m_client_delete (Participant *participant, gulong type);
Participant **  m_client_find_participants (Participant *participant, gulong type, gpointer data);
gint m_client_get_type (gulong code);
Client *m_client_get_by_code (gulong code);
Participant *m_client_get_participant_by_name (gchar * name,gint type,gboolean belong_to_principal);
gpointer *m_client_get_data (gint type, gulong code);
void m_client_clear_participant (Participant ** participants);
gboolean m_client_update (gpointer *client_data, Client *client, gulong type,GList * accounts);
gint m_client_get_type_in_transaction (gulong code,GdaConnection * connection);
Participant *m_client_get_participant_by_code (gulong code);
Participant **m_client_get_participants_by_legal_id (gchar ** enteprise_legal_id_tokens);

#endif

