/* accident.c - Siniestro de una reparacin.
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "accident.h"

gint accident_init(Accident *accident);


/**
 * d_accident_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Accident vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Accident *d_accident_create_empty () {
    Accident *accident = (Accident *) g_malloc (sizeof(Accident));

    accident_init(accident);

    return accident;
}


/**
 * d_accident_clear:
 * @accident: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_accident_clear (Accident *accident) {

    if (accident!=NULL){
	if (accident->accident_reference!=NULL)
	    g_free(accident->accident_reference);
	if (accident->place!=NULL)
	    g_free(accident->place);
	if (accident->field!=NULL)
	    g_free(accident->field);
	if (accident->damages!=NULL)
	    g_free(accident->damages);
	if (accident->way!=NULL)
	    g_free(accident->way);
	if (accident->province_code!=NULL)
	    g_free(accident->province_code);
	if (accident->reason_code!=NULL)
	    g_free(accident->reason_code);
	if (accident->photo1!=NULL)
	    g_free(accident->photo1);
	if (accident->photo2!=NULL)
	    g_free(accident->photo2);
	if (accident->description!=NULL)
	    g_free(accident->description);


	g_free(accident);
    }

    return 0;
}

/**
 * d_accident_set_accident_reference:
 * @accident: Siniestro en el que se establece el atributo.
 * @accident_reference: Atributo a insertar.
 *
 * Establece el atributo @accident_reference en el objeto. El atributo @accident_reference
 * indica la referencia del siniestro que se solicita a una compaa de seguros.
 *
 */
void d_accident_set_accident_reference (Accident *accident, gchar *accident_reference){
    if (accident->accident_reference!=NULL) g_free(accident->accident_reference);
    accident->accident_reference = g_strdup(accident_reference);
}

/**
 * d_accident_set_date:
 * @accident: Siniestro en el que se establece el atributo.
 * @date: Atributo a insertar.
 *
 * Establece el atributo @date en el objeto. El atributo @
 * indica la fecha en que se produjo un siniestro.
 *
 */
void d_accident_set_date (Accident *accident, time_t date){
    accident->date = date;
}

/**
 * d_accident_set_open_date:
 * @accident: Siniestro en el que se establece el atributo.
 * @date: Atributo a insertar.
 *
 * Establece el atributo @fecha_apertura en el objeto. El atributo @fecha_apertura
 * indica la fecha en el que se abri el expediente del siniestro.
 *
 */
void d_accident_set_open_date (Accident *accident, time_t date){
    accident->open_date = date;
}

/**
 * d_accident_set_place:
 * @accident: Siniestro en el que se establece el atributo.
 * @place: Atributo a insertar.
 *
 * Establece el atributo @ en el objeto. El atributo @place
 * indica el lugar del siniestro.
 *
 */
void d_accident_set_place (Accident *accident, gchar *place){
    if (accident->place != NULL) g_free(accident->place);
    accident->place = g_strdup(place);
}

/**
 * d_accident_set_field:
 * @accident: Siniestro en el que se establece el atributo.
 * @field: Atributo a insertar.
 *
 * Establece el atributo @ en el objeto. El atributo @field
 * indica un atributo del dominio de los seguros que es el ramo.
 *
 */
void d_accident_set_field (Accident *accident, gchar *field){
    if (accident->field != NULL) g_free(accident->field);
    accident->field = g_strdup(field);
}

/**
 * d_accident_set_damages:
 * @accident: Siniestro en el que se establece el atributo.
 * @damages: Atributo a insertar.
 *
 * Establece el atributo @damages en el objeto. El atributo @damages
 * indica los daos producidos en el vehculo en el momento del accidente.
 *
 */
void d_accident_set_damages (Accident *accident, gchar *damages){
    if (accident->damages != NULL) g_free(accident->damages);
    accident->damages = g_strdup(damages);
}

/**
 * d_accident_set_way:
 * @accident: Siniestro en el que se establece el atributo.
 * @way: Atributo a insertar.
 *
 * Establece el atributo @ en el objeto. El atributo @way
 * indica el modo en el que se produjo el accidente.
 *
 */
void d_accident_set_way (Accident *accident, gchar *way){
    if (accident->way != NULL) g_free(accident->way);
    accident->way = g_strdup(way);
}

/**
 * d_accident_set_province_code:
 * @accident: Siniestro en el que se establece el atributo.
 * @province_code: Atributo a insertar.
 *
 * Establece el atributo @province_code en el objeto. El atributo @province_code
 * indica el cdigo de provincia en la que se produjo el siniestro.
 *
 */
void d_accident_set_province_code (Accident *accident, gchar *province_code){
    if (accident->province_code != NULL) g_free(accident->province_code);
    accident->province_code = g_strdup(province_code);
}

/**
 * d_accident_set_reason_code:
 * @accident: Siniestro en el que se establece el atributo.
 * @reason_code: Atributo a insertar.
 *
 * Establece el atributo @reason_code en el objeto. El atributo @reason_code
 * indica el cdigo para identificar la causa de un siniestro.
 *
 */
void d_accident_set_reason_code (Accident *accident, gchar *reason_code){
    if (accident->reason_code != NULL) g_free(accident->reason_code);
    accident->reason_code = g_strdup(reason_code);
}

/**
 * d_accident_set_photo1:
 * @accident: Siniestro en el que se establece el atributo.
 * @photo1: Atributo a insertar.
 *
 * Establece el atributo @photo1 en el objeto. El atributo @photo1
 * indica el path de la primera foto sacada al vehculo a reparar.
 *
 */
void d_accident_set_photo1 (Accident *accident, gchar *photo1){
    if (accident->photo1 != NULL) g_free(accident->photo1);
    accident->photo1 = g_strdup(photo1);
}

/**
 * d_accident_set_photo2:
 * @accident: Siniestro en el que se establece el atributo.
 * @photo2: Atributo a insertar.
 *
 * Establece el atributo @photo2 en el objeto. El atributo @photo2
 * indica el path de la segunda foto sacada al vehculo a reparar.
 *
 */
void d_accident_set_photo2 (Accident *accident, gchar *photo2){
    if (accident->photo2 != NULL) g_free(accident->photo2);
    accident->photo2 = g_strdup(photo2);
}

/**
 * d_accident_set_description:
 * @accident: Siniestro en el que se establece el atributo.
 * @description: Atributo a insertar.
 *
 * Establece el atributo @description en el objeto. El atributo @description
 * indica la descripcin breve del siniestro.
 *
 */
void d_accident_set_description (Accident *accident, gchar *description){
    if (accident->description != NULL) g_free(accident->description);
    accident->description = g_strdup(description);
}

/**
 * d_accident_get_accident_reference:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * evuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 *
 */
gchar *d_accident_get_accident_reference (Accident *accident){
    return g_strdup(accident->accident_reference);
}
/**
 * d_accident_get_date:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
time_t d_accident_get_date (Accident *accident){
    return accident->date;
}


/**
 * d_accident_get_open_date:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
time_t d_accident_get_open_date (Accident *accident){
    return accident->open_date;
}

/**
 * d_accident_get_place:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_place (Accident *accident){
    return g_strdup(accident->place);
}


/**
 * d_accident_get_field:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_field (Accident *accident){
    return g_strdup(accident->field);
}


/**
 * d_accident_get_damages:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_damages (Accident *accident){
    return g_strdup(accident->damages);
}


/**
 * d_accident_get_way:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_way (Accident *accident){
    return g_strdup(accident->way);
}


/**
 * d_accident_get_province_code:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_province_code (Accident *accident){
    return g_strdup(accident->province_code);
}


/**
 * d_accident_get_reason_code:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_reason_code (Accident *accident){
    return g_strdup(accident->reason_code);
}


/**
 * d_accident_get_photo1:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_photo1 (Accident *accident){
    return g_strdup(accident->photo1);
}


/**
 * d_accident_get_photo2:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_photo2 (Accident *accident){
    return g_strdup(accident->photo2);
}


/**
 * d_accident_get_description:
 * @accident: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado.
 */
gchar *d_accident_get_description (Accident *accident){
    return g_strdup(accident->description);
}

/**
 * d_accident_duplicate :
 * @accident: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto #Accident
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Accident *d_accident_duplicate (Accident *accident){

    Accident *result = d_accident_create_empty ();
    gchar *accident_ref = d_accident_get_accident_reference (accident);
    time_t accident_date = d_accident_get_date (accident);
    time_t accident_open_date = d_accident_get_open_date (accident);
    gchar *place = d_accident_get_place (accident);
    gchar *field = d_accident_get_field (accident);
    gchar *damages = d_accident_get_damages (accident);
    gchar *way = d_accident_get_way (accident);
    gchar *cprov = d_accident_get_province_code (accident);
    gchar *ccausa = d_accident_get_reason_code (accident);
    gchar *photo1 = d_accident_get_photo1 (accident);
    gchar *photo2 = d_accident_get_photo2 (accident);
    gchar *description = d_accident_get_description (accident);

    d_accident_set_accident_reference (result, accident_ref);
    d_accident_set_date (result, accident_date);
    d_accident_set_open_date (result, accident_open_date);
    d_accident_set_place (result, place);
    d_accident_set_field (result, field);
    d_accident_set_damages (result, damages);
    d_accident_set_way (result, way);
    d_accident_set_province_code (result, cprov);
    d_accident_set_reason_code (result, ccausa);
    d_accident_set_photo1 (result, photo1);
    d_accident_set_photo2 (result, photo2);
    d_accident_set_description (result, description);

    g_free(accident_ref);
    g_free(place);
    g_free(field);
    g_free(damages);
    g_free(way);
    g_free(cprov);
    g_free(ccausa);
    g_free(photo1);
    g_free(photo2);
    g_free(description);

    return result;

}


/*Private functions*/

gint accident_init(Accident *accident) {

    accident->accident_reference=NULL;
    accident->date= time(NULL);
    accident->open_date= time(NULL);
    accident->place=NULL;
    accident->damages=NULL;
    accident->field=NULL;
    accident->way=NULL;
    accident->province_code=NULL;
    accident->reason_code=NULL;
    accident->photo1=NULL;
    accident->photo2=NULL;
    accident->description=NULL;

    return 0;
}
