  /* accounting_period.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */



#include "accounting_period.h"
#include "participant.h"
#include "util.h"




static gint accounting_period_init(AccountingPeriod *accounting_period);


/**
 * d_accounting_period_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Accounting_period vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
AccountingPeriod * d_accounting_period_create_empty () {

    AccountingPeriod * accounting_period;

    accounting_period = (AccountingPeriod *) g_malloc0(sizeof(AccountingPeriod));

    accounting_period_init(accounting_period);

    return accounting_period;
}



/**
 * d_accounting_set_initial_date:
 * @accounting_period: Objeto en el que se establece el atributo.
 * @initial_date: Atributo a insertar.
 *
 * Establece el atributo @initial_date en el objeto. El atributo @initial_date
 * indica la fecha de inicio de un perodo de contabilidad.
 *
 */
void d_accounting_set_initial_date (AccountingPeriod * accounting_period, time_t initial_date) {
    accounting_period->initial_date = initial_date;
}
/**
 * d_accounting_set_ending_date:
 * @accounting_period: Objeto en el que se establece el atributo.
 * @ending_date: Atributo a insertar.
 *
 * Establece el atributo @ending_date en el objeto. El atributo @ending_date
 * indica el fin del perodo de contabilidad.
 *
 */
void d_accounting_set_ending_date (AccountingPeriod * accounting_period, time_t ending_date) {
    accounting_period->ending_date = ending_date;
}
/**
 * d_accounting_set_accounting_code:
 * @accounting_period: Objeto en el que se establece el atributo.
 * @accounting_code: Atributo a insertar.
 *
 * Establece el atributo @accounting_code en el objeto. El atributo @accounting_code
 * indica el cdigo interno que identifica la el perodo de contabilidad.
 *
 */
void d_accounting_set_accounting_code (AccountingPeriod * accounting_period, gulong accounting_code) {
    accounting_period->accounting_code = accounting_code;
}
/**
 * d_accounting_set_enterprise_code:
 * @accounting_period: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa a la que se refiere el perodo de contabilidad.
 *
 */
void d_accounting_set_enterprise_code (AccountingPeriod * accounting_period, gulong enterprise_code) {
    accounting_period->enterprise_code = enterprise_code;
}

/**
 * d_accounting_set_closed:
 * @accounting_period: Objeto en el que se establece el atributo.
 * @closed: Atributo a insertar.
 *
 * Establece el atributo @closed en el objeto. El atributo @closed
 * indica si el perodo de contabilidad est cerrado o no.
 *
 */
void d_accounting_set_closed (AccountingPeriod * accounting_period, gboolean closed) {
    accounting_period->closed=closed;
}

/**
 * d_accounting_get_accounting_code:
 * @accounting_period: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado
 */
gulong d_accounting_get_accounting_code (AccountingPeriod * accounting_period) {
    return (accounting_period->accounting_code);
}
/**
 * d_accounting_get_enterprise_code:
 * @accounting_period: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado
 */
gulong d_accounting_get_enterprise_code (AccountingPeriod * accounting_period) {
    return (accounting_period->enterprise_code);
}

/**
 * d_accounting_get_initial_date:
 * @accounting_period: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado
 */
time_t d_accounting_get_initial_date (AccountingPeriod * accounting_period) {
    return (accounting_period->initial_date);

}
/**
 * d_accounting_get_ending_date:
 * @accounting_period: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado
 */
time_t d_accounting_get_ending_date (AccountingPeriod * accounting_period) {
    return (accounting_period->ending_date);

}
/**
 * d_accounting_get_closed:
 * @accounting_period: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El atributo solicitado
 */
gboolean d_accounting_get_closed (AccountingPeriod * accounting_period) {
    return accounting_period->closed;
}

/**
 d_accounting_period_clear:
 * @accounting_period: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_accounting_period_clear (AccountingPeriod * accounting_period) {

    if (accounting_period!=NULL) {
	g_free(accounting_period);
	accounting_period = NULL;
    }

    return 0;
}



/* Private functions */

static gint accounting_period_init(AccountingPeriod *accounting_period) {
    accounting_period->accounting_code=ULONG_MAX;
    accounting_period->enterprise_code=ULONG_MAX;
    accounting_period->initial_date = time(NULL);
    accounting_period->ending_date = time(NULL);
    accounting_period->closed=FALSE;

    return 0;
}

