/* charge_invoice.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "charge_invoice.h"
#include "util.h"

#define D_CHARGE_INVOICE_CONTENT_SIZE 5

static gint charge_invoice_init(ChargeInvoice *charge_invoice);


/**
 * d_charge_invoice_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #ChargeInvoice vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
ChargeInvoice * d_charge_invoice_create_empty () {

    ChargeInvoice * charge_invoice;

    charge_invoice = (ChargeInvoice *) g_malloc0(sizeof(ChargeInvoice));

    charge_invoice_init(charge_invoice);

    return charge_invoice;
}

/**
 * d_charge_invoice_set_code:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @code: Atributo a insertar.
 *
 * Establece el atributo @code en el objeto. El atributo @code
 * indica el cdigo de factura o nota interno que identifica la misma.
 *
 */
void d_charge_invoice_set_code (ChargeInvoice * charge_invoice,gulong code) {
    charge_invoice->code = code;
}


/**
 * d_charge_invoice_set_invoice_number:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @invoice_number: Atributo a insertar.
 *
 * Establece el atributo @invoice_number en el objeto. El atributo @invoice_number
 * indica el nmero de la factura.
 *
 */
void d_charge_invoice_set_invoice_number (ChargeInvoice * charge_invoice,gchar * invoice_number) {
    charge_invoice->invoice_number=g_strdup(invoice_number);
}
/**
 * d_charge_invoice_set_date:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @date: Atributo a insertar.
 *
 * Establece el atributo @date en el objeto. El atributo @date
 * indica la fecha de emisin de la factura.
 *
 */
void d_charge_invoice_set_date (ChargeInvoice * charge_invoice,time_t date) {
    charge_invoice->date = date;
}

/**
 * d_charge_invoice_set_section:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @section: Atributo a insertar.
 *
 * Establece el atributo @section en el objeto. El atributo @section
 * indica el concepto de la factura, por si se desea introducir algn comentario.
 *
 */
void d_charge_invoice_set_section (ChargeInvoice * charge_invoice,gchar * section) {
    charge_invoice->section = g_strdup(section);
}
/**
 * d_charge_invoice_set_enterprise:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @enterprise: Atributo a insertar.
 *
 * Establece el atributo @enterprise en el objeto. El atributo @enterprise
 * indica la empresa del grupo a la que se emiti la factura.
 *
 */
void d_charge_invoice_set_enterprise (ChargeInvoice * charge_invoice,Enterprise * enterprise) {
    charge_invoice->enterprise = enterprise;
}
/**
 * d_charge_invoice_set_supplier:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @supplier: Atributo a insertar.
 *
 * Establece el atributo @supplier en el objeto. El atributo @supplier
 * indica el proveedor que emiti la factura.
 *
 */
void d_charge_invoice_set_supplier (ChargeInvoice * charge_invoice,Supplier * supplier) {
    charge_invoice->supplier = supplier;
}

/**
 * d_charge_invoice_set_shop:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @shop: Atributo a insertar.
 *
 * Establece el atributo @shop en el objeto. El atributo @shop
 * indica la tienda que recibe la factura.
 *
 */
void d_charge_invoice_set_shop (ChargeInvoice * charge_invoice,Shop * shop) {
    charge_invoice->shop = shop;
}

/**
 * d_charge_invoice_set_state:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @state: Atributo a insertar.
 *
 * Establece el atributo @state en el objeto. El atributo @state
 * indica el estado en el que se encuentra actualmente la factura: %D_CHARGE_INVOICE_PAID o
 * %D_CHARGE_INVOICE_NOT_PAID
 *
 */
void d_charge_invoice_set_state (ChargeInvoice * charge_invoice,gint state) {
    charge_invoice->state = state;
}

/**
 * d_charge_invoice_set_payment:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @payment: Atributo a insertar.
 *
 * Establece el atributo @payment en el objeto. El atributo @payment
 * indica el pago que tiene asociado esta factura. Toda factura o nota debe
 * tener un pago asociado.
 *
 */
void d_charge_invoice_set_payment (ChargeInvoice * charge_invoice,Payment * payment) {
    charge_invoice->payment = payment;
}

/**
 * d_charge_invoice_set_charge_invoice_lines:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @cils: Atributo a insertar.
 *
 * Establece el atributo @charge_invoice_lines en el objeto. El atributo @charge_invoice_lines
 * indica las lneas de la factura.
 *
 */
void d_charge_invoice_set_charge_invoice_lines (ChargeInvoice * charge_invoice,ChargeInvoiceLine ** cils){
    charge_invoice->charge_invoice_lines = cils;

}
/**
 * d_charge_invoice_set_total:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el total de la factura.
 *
 */
void d_charge_invoice_set_total (ChargeInvoice * charge_invoice,gdouble total) {
    charge_invoice->total = total;
}


/**
 * d_charge_invoice_set_vat:
 * @charge_invoice: Objeto en el que se establece el atributo.
 * @vat: Atributo a insertar.
 *
 * Establece el atributo @vat en el objeto. El atributo @vat
 * indica si el objeto es una factura (%TRUE)  o una nota (%FALSE).
 *
 */
void d_charge_invoice_set_vat (ChargeInvoice * charge_invoice,gboolean vat){
    charge_invoice->vat = vat;

}

/**
 * d_charge_invoice_get_vat:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_charge_invoice_get_vat (ChargeInvoice * charge_invoice) {
    return charge_invoice->vat;
}

/**
 * d_charge_invoice_get_state:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_charge_invoice_get_state (ChargeInvoice * charge_invoice){
    return charge_invoice->state;
}

/**
 * d_charge_invoice_get_date:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_charge_invoice_get_date (ChargeInvoice * charge_invoice) {
    return charge_invoice->date;
}

/**
 * d_charge_invoice_get_section:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_charge_invoice_get_section (ChargeInvoice * charge_invoice){
    return g_strdup(charge_invoice->section);
}

/**
 * d_charge_invoice_get_charge_invoice_lines:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
ChargeInvoiceLine ** d_charge_invoice_get_charge_invoice_lines (ChargeInvoice * charge_invoice) {
    return charge_invoice->charge_invoice_lines;
}

/**
 * d_charge_invoice_get_total:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_charge_invoice_get_total (ChargeInvoice * charge_invoice) {
    return charge_invoice->total;
}

/**
 * d_charge_invoice_get_invoice_number:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_charge_invoice_get_invoice_number (ChargeInvoice * charge_invoice){
    return g_strdup(charge_invoice->invoice_number);
}

/**
 * d_charge_invoice_get_enterprise:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Enterprise * d_charge_invoice_get_enterprise (ChargeInvoice * charge_invoice) {
    return charge_invoice->enterprise;
}

/**
 * d_charge_invoice_get_shop:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Shop * d_charge_invoice_get_shop (ChargeInvoice * charge_invoice) {
    return charge_invoice->shop;
}

/**
 * d_charge_invoice_get_payment:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Payment * d_charge_invoice_get_payment (ChargeInvoice * charge_invoice) {
    return charge_invoice->payment;
}

/**
 * d_charge_invoice_get_supplier:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Supplier * d_charge_invoice_get_supplier (ChargeInvoice * charge_invoice) {
    return charge_invoice->supplier;
}

/**
 * d_charge_invoice_get_code:
 * @charge_invoice: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_charge_invoice_get_code (ChargeInvoice * charge_invoice) {
    return charge_invoice->code;
}


/**
 * d_charge_invoice_clear:
 * @charge_invoice: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns: 0 si se ha acabado la funcin correctamente.
 */
gint d_charge_invoice_clear (ChargeInvoice * charge_invoice) {

    gint index=0;

    if (charge_invoice!=NULL) {
	d_shop_clear (charge_invoice->shop);
	d_enterprise_clear (charge_invoice->enterprise);
	d_supplier_clear (charge_invoice->supplier);
	d_payment_clear (charge_invoice->payment);
	if (charge_invoice->charge_invoice_lines!=NULL) {
	    while (charge_invoice->charge_invoice_lines[index]!=NULL)
		d_charge_invoice_line_clear (charge_invoice->charge_invoice_lines[index++]);
	}
	if (charge_invoice->section!=NULL) g_free(charge_invoice->section);

	g_free(charge_invoice);
    }
    return 0;
}


/**
 * d_charge_d_invoice_array_to_content:
 * @array: Array de facturas de cargo que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar el total de objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_charge_d_invoice_array_to_content (ChargeInvoice **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;
    gdouble * saldoAct = (gdouble *) g_malloc0(sizeof(gdouble));

    *saldoAct = 0;
    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) g_malloc0(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_charge_invoice_get_content (array[*reccount],saldoAct);
        (*reccount)++;
    }

    content[*reccount] = NULL;
    g_free(saldoAct);
    return content;

}

/**
 * d_charge_invoice_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto factura de cargo.
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_charge_invoice_get_content_title () {
    gchar **content = (gchar **) g_malloc0 (sizeof(gchar *)*d_charge_invoice_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup_printf(_("ISSUE DATE"));
    content[index++] = g_strdup_printf(_("INVOICE No."));
    content[index++] = g_strdup_printf(_("DEBIT"));
    content[index++] = g_strdup_printf(_("CREDIT"));
    content[index++] = g_strdup_printf(_("BALANCE"));
    content[index++] = NULL;
    return content;
}

/**
 * d_charge_invoice_get_listing_title:
 * @supplier_name: Nombre del proveedor para el que se saca el listado de facturas.
 *
 * Obtiene el ttulo del listado segn el proveedor para el que se obtiene el listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_charge_invoice_get_listing_title (gchar * supplier_name) {
    gchar *content = g_strdup_printf(_("INVOICE PAYMENT LISTING OF %s"),supplier_name);
    return content;
}

/**
 * d_charge_invoice_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados de facturas.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_charge_invoice_get_listing_columns_width () {
    gint *widths = (gint *) g_malloc0 (sizeof(gint)*d_charge_invoice_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 20;
    widths[index++] = 20;
    widths[index++] = 20;
    widths[index++] = 20;
    widths[index++] = 20;
    widths[index++] = -1;

    return widths;
}

/**
 * d_charge_invoice_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_charge_invoice_get_listing_columns_alignment () {
    gint *orientation = (gint *) g_malloc0 (sizeof(gint)*d_charge_invoice_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_charge_invoice_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas
 */
gint d_charge_invoice_get_content_size () {
    return D_CHARGE_INVOICE_CONTENT_SIZE;
}


/**
 * d_charge_invoice_get_content:
 * @invoice: Factura de cargo a partir de la que se obtendrn los valores para el listado.
 * @saldoAct: Puntero a gdouble utilizado para sacar el total dels saldo de la tienda de una empresa del grupo
 * con respecto al proveedor para el que se saca el listado.
 *
 * Transforma una factura de cargo en un array de cadenas de caracteres para aadir al listado
 * de facturas de cargo.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_charge_invoice_get_content (ChargeInvoice *invoice,gdouble * actual_balance) {
    time_t  date = d_charge_invoice_get_date (invoice);
    gchar  *temp = u_get_parsed_date (&date);
    gchar **content = (gchar **) g_malloc0(sizeof(gchar *)*d_charge_invoice_get_content_size ()+1);
    gint    index   = 0;
    gchar *temp_invoice=NULL;
    gdouble debit=-1;
    gdouble credit=-1;
    gchar * debit_string=NULL;
    gchar * credit_string=NULL;

    u_delete_intros (temp);
    temp_invoice = d_charge_invoice_get_invoice_number (invoice);

    if (d_charge_invoice_get_state (invoice)==D_CHARGE_INVOICE_PAID) {
        debit = d_charge_invoice_get_total (invoice);
    } else {
        credit = d_charge_invoice_get_total (invoice);
    }

    if (debit>0) {
	debit_string = g_strdup_printf("%.2f",debit);
	credit_string =  g_strdup("");
        *actual_balance = *actual_balance - debit;
    } else if (credit>0) {
	credit_string = g_strdup_printf("%.2f",credit);
	debit_string =  g_strdup("");
        *actual_balance = *actual_balance + credit;
    }
    if (debit_string == NULL)
        debit_string = g_strdup("");
    if (credit_string == NULL)
        credit_string = g_strdup("");

    content[index++] = temp;
    content[index++] = g_strdup(temp_invoice);
    content[index++] = debit_string;
    content[index++] = credit_string;
    content[index++] = g_strdup_printf("%.2f",*actual_balance);
    content[index++] = NULL;
    g_free(temp_invoice);
    return content;
}

/* Private functions */

static gint charge_invoice_init(ChargeInvoice *charge_invoice) {

    charge_invoice->invoice_number=NULL;
    charge_invoice->code=ULONG_MAX;
    charge_invoice->date = time(NULL);
    charge_invoice->charge_invoice_lines = NULL;
    charge_invoice->shop = NULL;
    charge_invoice->supplier = NULL;
    charge_invoice->payment = NULL;
    charge_invoice->enterprise = NULL;
    charge_invoice->total= G_MINDOUBLE;
    charge_invoice->state=-1;
    charge_invoice->vat=FALSE;
    charge_invoice->section=NULL;
    return 0;
}

