/* client.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "client.h"

static gint client_init(Client *client);

/**
 * d_client_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Client vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Client * d_client_create_empty (void)
{
  Client *client = (Client *) g_malloc(sizeof(Client));
  
  client_init(client);
  
  return client;
}


/**
 * d_client_clear:
 * @client: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_client_clear (Client *client) {
  
  if (client!=NULL){
    d_participant_clear (client->participant);
  
    g_free(client);
  }

  return 0;
}

/**
 * d_client_set_participant:
 * @client: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el rol de participante del cliente.
 *
 */
void d_client_set_participant (Client *client, Participant *participant) {
    client->participant = participant;
}

/**
 * d_client_set_discount:
 * @client: Objeto en el que se establece el atributo.
 * @client_discount: Atributo a insertar.
 *
 * Establece el atributo @client_discount en el objeto. El atributo @client_discount
 * indica el descuento que se le aplica a un cliente. Este descuento est supeditado
 * a otros descuentos como el de compaa de seguros.
 *
 */
void d_client_set_discount (Client *client, gdouble client_discount){
    if (client_discount>100)
	client->client_discount=100;
    else if (client_discount<0)
	client->client_discount=0;
    else
	client->client_discount=client_discount;
}

/**
 * d_client_set_agent:
 * @client: Objeto en el que se establece el atributo.
 * @agent: Atributo a insertar.
 *
 * Establece el atributo @agent en el objeto. El atributo @agent
 * indica si el cliente realizar el rol de agenta.
 *
 */
void d_client_set_agent (Client *client, gboolean agent){
    client->agent = agent;
}

/**
 * d_client_get_participant:
 * @client: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant *d_client_get_participant (Client *client) {
    return client->participant;
}

/**
 * d_client_get_discount:
 * @client: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_client_get_discount (Client *client){
    return client->client_discount;
}

/**
 * d_client_get_agent:
 * @client: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_client_get_agent (Client *client){
    return client->agent;
}

/**
 * d_client_duplicate:
 * @client: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el un objeto #client.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Client *d_client_duplicate (Client *client) {

  Client *result = d_client_create_empty ();
  Participant *participant = d_participant_duplicate (d_client_get_participant (client));
  gdouble client_discount = d_client_get_discount (client);
  gboolean agent = d_client_get_agent (client);
  
  d_client_set_participant (result, participant);
  d_client_set_discount (result, client_discount);
  d_client_set_agent (result, agent);

  return result;
}

/*Private function*/

static gint client_init(Client *client) 
{
  client->participant = NULL;
  client->client_discount=-1;
  client->agent=FALSE;

  return 0;
}
