/* delivery_payment.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "delivery_payment.h"
#include "company.h"
#include "shop.h"
#include "util.h"
#include "repair_object.h"
#include "enterprise.h"
#include "object_utils.h"


#define D_DELIVERY_PAYMENT_CONTENT_SIZE 6
/*    codigoReceptor as codigoReceptorAlbaranAbono,\
*/

static gint delivery_payment_init(DeliveryPayment *delivery_payment);
static gboolean is_valid_garage_repair(Garage *garage);
static gboolean is_valid_company_repair(Company *company);


/**
 * d_delivery_payment_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns: Un objeto #DeliveryPayment vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
DeliveryPayment * d_delivery_payment_create_empty () {

    DeliveryPayment * delivery_payment;

    delivery_payment = (DeliveryPayment *) g_malloc0(sizeof(DeliveryPayment));
    delivery_payment_init(delivery_payment);

    return delivery_payment;
}




/**
 * d_delivery_payment_clear:
 * @delivery_payment: Albarn de abono a liberar
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_delivery_payment_clear (DeliveryPayment *delivery_payment) {
    if (delivery_payment!=NULL) {
	//d_repair_clear (get_repair_delivery_payment(delivery_payment));
	d_repair_clear (d_delivery_payment_get_repair (delivery_payment));
	g_free(delivery_payment);
    }
    return 0;
}


/**
 * d_delivery_payment_set_number:
 * @delivery_payment: Objeto en el que se establece el atributo.
 * @delivery_payment_number: Atributo a insertar.
 *
 * Establece el atributo @delivery_payment_number en el objeto. El atributo @delivery_payment_number
 * indica 
 *
 */
void d_delivery_payment_set_number (DeliveryPayment * delivery_payment,gulong delivery_payment_number){
    delivery_payment->delivery_payment_number = delivery_payment_number;
}
/*
void set_codigo_receptor_delivery_payment(DeliveryPayment * delivery_payment,gulong codigo_receptor){
    delivery_payment->codigoReceptor = codigo_receptor;
    }
*/

/**
 * d_delivery_payment_set_invoice_code:
 * @delivery_payment: Objeto en el que se establece el atributo.
 * @invoice_code: Atributo a insertar.
 *
 * Establece el atributo @invoice_code en el objeto. El atributo @invoice_code
 * indica el nmero de factura a la que referencia el albarn.
 *
 */
void d_delivery_payment_set_invoice_code (DeliveryPayment * delivery_payment,gulong invoice_code){
    delivery_payment->invoice_code = invoice_code;
}
/**
 * d_delivery_payment_set_repair:
 * @delivery_payment: Objeto en el que se establece el atributo.
 * @repair: Atributo a insertar.
 *
 * Establece el atributo @repair en el objeto. El atributo @repair
 * indica la reparacin a la que referencia el albarn.
 *
 */
void d_delivery_payment_set_repair (DeliveryPayment * delivery_payment,Repair * repair){
    delivery_payment->repair = repair;
}

/*
gulong get_codigo_receptor_delivery_payment(DeliveryPayment * delivery_payment){
    return delivery_payment->codigoReceptor;
}
*/

/**
 * d_delivery_payment_set_issue_date:
 * @delivery_payment: Objeto en el que se establece el atributo.
 * @issue_date: Atributo a insertar.
 *
 * Establece el atributo @issue_date en el objeto. El atributo @issue_date
 * indica la fecha en la que se emiti el albarn.
 *
 */
void d_delivery_payment_set_issue_date (DeliveryPayment * delivery_payment,time_t  issue_date) {
    delivery_payment->issue_date = issue_date;
}

/**
 * d_delivery_payment_set_garage_total:
 * @delivery_payment: Objeto en el que se establece el atributo.
 * @garage_total: Atributo a insertar.
 *
 * Establece el atributo @garage_total en el objeto. El atributo @garage_total
 * indica el total de comisin que se le facturar a un taller del que procede
 * la reparacin del albarn.
 */
void d_delivery_payment_set_garage_total (DeliveryPayment * delivery_payment,gdouble garage_total) {
    delivery_payment->garage_total = garage_total;

}
/**
 * d_delivery_payment_get_invoice_code:
 * @delivery_payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_get_invoice_code (DeliveryPayment * delivery_payment){
    return delivery_payment->invoice_code;
}

/**
 * d_delivery_payment_get_number:
 * @delivery_payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_get_number (DeliveryPayment * delivery_payment){
    return delivery_payment->delivery_payment_number;
}

/**
 * d_delivery_payment_get_repair:
 * @delivery_payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Repair *  d_delivery_payment_get_repair (DeliveryPayment * delivery_payment){
    return delivery_payment->repair;
}

/**
 * d_delivery_payment_get_issue_date:
 * @delivery_payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_delivery_payment_get_issue_date (DeliveryPayment * delivery_payment) {
    return delivery_payment->issue_date;
}

/**
 * d_delivery_payment_get_garage_total:
 * @delivery_payment: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_delivery_payment_get_garage_total (DeliveryPayment * delivery_payment) {
    return delivery_payment->garage_total;
}


/* Private functions */

static gint delivery_payment_init(DeliveryPayment *delivery_payment) {



    delivery_payment->delivery_payment_number  = ULONG_MAX;

    delivery_payment->issue_date = time(NULL);

    delivery_payment->invoice_code  = ULONG_MAX;

    delivery_payment->garage_total  = -1;

    delivery_payment->repair = NULL;
    return 0;
}

/* DOMAIN FUNCTIONS*/
/**
 * d_delivery_payment_calculate_total_data:
 * @total: Puntero a un #gdouble. Apunta al valor calculado dentro de la funcin
 * @delivery_payment:
 *
 * Calcula el total del albarn de abono. Para el clculo del total se recorren
 * todas las lneas de la reparacin comprobando si existe un precio para la lnea
 * de la reparacin, en caso de que exista se utiliza ese precio (multiplicado por el nmero
 * de unidades) sin tener en cuenta posibles descuentos para el total de la lnea, en otro
 * caso se comprueba si existe el precio del producto, y si existe se utiliza ste, multiplicado
 * por las unidades y restndole el descuento de la lnea. Si no existiese ninguno de los
 * precios el total de la lnea sera 0.
 *
 */
void d_delivery_payment_calculate_total_data (gdouble * total,DeliveryPayment * delivery_payment) {

    gdouble discount;
    gdouble price;
    gdouble sub_total=0;
    gdouble lines_total;


    Repair * repair = d_delivery_payment_get_repair (delivery_payment);
    gint aux_count=0;
    while (repair->repair_line[aux_count]!=NULL) {
	Repair_line * repair_line = repair->repair_line[aux_count];
	gdouble line_price = d_repair_line_get_price (repair_line);
	gdouble aux_price=0;
	if (line_price==G_MINDOUBLE) {
	    if (d_product_get_price (d_repair_line_get_product(repair_line))!=G_MINDOUBLE) {
		aux_price = d_product_get_price (d_repair_line_get_product(repair_line));

	    } else {
                aux_price = 0;
	    }
	} else {
	    line_price = line_price * d_repair_line_get_quantity (repair_line);

	}

	discount = d_repair_line_get_discount (repair_line);
	if ((discount>100) || (discount<0)) {
            discount=0;
	}
	price = 1.0 * aux_price * d_repair_line_get_quantity (repair_line);

	if (line_price!=G_MINDOUBLE) {
	    lines_total = line_price;
	} else if (price!=G_MINDOUBLE) {
	    lines_total = price - (((1.0*(discount))/100)*price);
	} else {
            lines_total = 0;
	}

	sub_total = sub_total + lines_total;
	aux_count++;
    }


    *total =sub_total;
}

/**
 * d_delivery_payment_obtain_garage:
 * @delivery_payments: Array con los albaranes a inspeccionar.
 *
 * Busca el participante del primer albarn de abono que sea un taller. Primero comprueba que
 * la reparacin tenga un seguro asociado, si tiene seguro devuelve el taller, sino devuelve el cliente.
 * En caso de que no tenga seguro devuelve el cliente. Se parte del la precondicin de que se sabe que el cliente
 * o el taller e un taller.
 *
 * Returns:
 * El objeto #Participante del taller.
 */
Participant * d_delivery_payment_obtain_garage (DeliveryPayment ** delivery_payments) {

/* 30-06-2003 -- Si no se detectan fallos, eliminar el codigo comentado */

    if (delivery_payments!=NULL) {

/*
	if (d_repair_get_company (d_delivery_payment_get_repair (delivery_payments[0]))!=NULL) {
	    if (d_participant_get_code (d_company_get_participant (d_repair_get_company (d_delivery_payment_get_repair (delivery_payments[0]))))!=ULONG_MAX) {
		return d_garage_get_participant (d_repair_get_garage (d_delivery_payment_get_repair (delivery_payments[0])));
	    } else {
		return d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (delivery_payments[0])));
	    }
	} else {
	    return d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (delivery_payments[0])));
	}
    }
    return NULL;
*/
	Company *company = d_repair_get_company (d_delivery_payment_get_repair (delivery_payments[0]));
/*	
	Garage *garage = d_garage_get_participant (d_repair_get_garage (d_delivery_payment_get_repair (delivery_payments[0])));
*/
	Garage *garage = d_repair_get_garage (d_delivery_payment_get_repair (delivery_payments[0]));
	
	if ((is_valid_company_repair(company)) || (is_valid_garage_repair(garage))) {	
		
		/* Si la comnaia es valida, se puede asegurar que el cliente se trata de un taller. */
		return d_garage_get_participant(garage);
				
	} else {
	
	    /* El cliente es un partitcular */	
	    return d_client_get_participant (d_repair_get_client (d_delivery_payment_get_repair (delivery_payments[0])));
	}
    }
            
    return NULL;
}

/**
 * is_valid_company :
 *
 * Determina si el objeto @Company es distinto de NULL y si el objeto @Participante es valido.
 * 
 *
 * Returns:
 * TRUE si es un objeto valido, FALSE en otro caso.
 */
static gboolean is_valid_company_repair(Company * company) {

    return ((company !=NULL) && (d_object_utils_is_valid_participant(d_company_get_participant(company))));	
}

/**
 * is_valid_garage :
 *
 * Determina si el objeto @Garage es distinto de NULL y si el objeto @Participante es valido.
 * 
 *
 * Returns:
 * TRUE si es un objeto valido, FALSE en otro caso.
 */
static gboolean is_valid_garage_repair(Garage * garage) {

    return ((garage !=NULL) && (d_object_utils_is_valid_participant(d_garage_get_participant(garage))));	
}



/**
 * d_delivery_payment_array_to_content:
 * @array: Array de albaranes  que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar el total de objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_delivery_payment_array_to_content (DeliveryPayment **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) g_malloc0(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_delivery_payment_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}

/**
 * d_delivery_payment_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto albarn.
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_delivery_payment_get_content_title () {
    gchar **content = (gchar **) malloc (sizeof(gchar *)*d_delivery_payment_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup_printf(_("DEL. PAY. No."));
    content[index++] = g_strdup_printf(_("DATE"));
    content[index++] = g_strdup_printf(_("REG No."));
    content[index++] = g_strdup_printf(_("CLIENT"));
    content[index++] = g_strdup_printf(_("INSURANCE"));
    content[index++] = g_strdup_printf(_("OBV"));
    content[index++] = NULL;
    return content;
}

/**
 * d_delivery_payment_get_listing_title:
 * @shop_name: Nombre de la para la que se saca el listado de albaranes.
 *
 * Obtiene el ttulo del listado segn el proveedor para el que se obtiene el listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_delivery_payment_get_listing_title (gchar * shop_name) {
    gchar *content = g_strdup_printf(_("Delivery payment without invoice listing %s"),shop_name);
    return content;
}

/**
 * d_delivery_payment_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados de albaranes.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_delivery_payment_get_listing_columns_width () {
    gint *widths = (gint *) malloc (sizeof(gint)*d_delivery_payment_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 10;
    widths[index++] = 10;
    widths[index++] = 15;
    widths[index++] = 38;
    widths[index++] = 22;
    widths[index++] = 5;
    widths[index++] = -1;

    return widths;
}

/**
 * d_delivery_payment_get_listing_columns_alignment:
 *
 * Obtiene el alineamiento de las columnas del listado. Los alineamientos pueden ser U_LEFT, RIGHT o
 * CENTER.
 *
 * Returns:
 * Un array con los alineamientos de cada columna. EL array acaba en -1.
 */
gint *d_delivery_payment_get_listing_columns_alignment () {
    gint *orientation = (gint *) malloc (sizeof(gint)*d_delivery_payment_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_delivery_payment_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas
 */
gint d_delivery_payment_get_content_size () {
    return D_DELIVERY_PAYMENT_CONTENT_SIZE;
}


/**
 * d_delivery_payment_get_content:
 * @delivery_payment: ALbarn a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma un albarn en un array de cadenas de caracteres para aadir al listado
 * de albaranes.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_delivery_payment_get_content (DeliveryPayment *delivery_payment) {
    gchar * license_number=NULL;
    gchar * client = NULL;
    gchar * observation=NULL;
    gchar * company=NULL;
    gchar * temp_company=NULL;
    gulong temp_code_company=ULONG_MAX;
    gchar * temp_client=NULL;
    gchar **content = (gchar **) g_malloc(sizeof(gchar *)*d_delivery_payment_get_content_size ()+1);
    gint    index   = 0;
    time_t  date =  d_delivery_payment_get_issue_date (delivery_payment);
    gchar  *temp_date = u_get_parsed_date (&date);


    if (d_delivery_payment_get_repair (delivery_payment)!=NULL) {
	Repair * repair = d_delivery_payment_get_repair (delivery_payment);
	if (d_repair_get_company (repair)!=NULL) {
	    if (d_company_get_participant (d_repair_get_company (repair))!=NULL)
		//temp_company = d_participant_get_nick (d_company_get_participant (d_repair_get_company (repair)));
		company = d_participant_get_short_name (d_company_get_participant (d_repair_get_company (repair)));
		temp_code_company = d_participant_get_code (d_company_get_participant (d_repair_get_company (repair)));
	}
	if (d_repair_get_vehicle (repair)!=NULL) {
	    license_number = d_vehicle_get_license_number (d_repair_get_vehicle (repair));
	}
	if (d_repair_get_client (repair)!=NULL) {
            if (d_client_get_participant (d_repair_get_client (repair))!=NULL)
		//            temp_client = d_participant_get_nick (d_client_get_participant (d_repair_get_client (repair)));
		temp_client = d_participant_get_short_name (d_client_get_participant (d_repair_get_client (repair)));
	}
	if (d_repair_get_observations (repair)!=NULL) {
            observation=g_strdup("*");
	} else {
            observation=g_strdup("");
	}
    }

    if (license_number==NULL)
	license_number = g_strdup("");
    else
	g_strup(license_number);

    if (temp_client==NULL)
	client = g_strdup("");
    else {
        client = g_strndup(temp_client,30);
	g_strup(client);
    }
//    if (temp_company==NULL)
    if (company==NULL)
	company = g_strdup("");
    else {

	//	company = d_company_get_short_name (NULL,temp_code_company,temp_company);
	//g_free(temp_company);
	//g_strup(company);
    }

    temp_company = g_strndup(company,25);
    g_free(company);

    content[index++] = g_strdup_printf("%lu",d_delivery_payment_get_number (delivery_payment));
    content[index++] = temp_date;
    content[index++] = license_number;
    content[index++] = client;
    content[index++] = temp_company;
    content[index++] = observation;
    content[index++] = NULL;
    return content;
}


