/* delivery_payment_movement.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "delivery_payment_movement.h"
#include "util.h"

static gint delivery_payment_movement_init(DeliveryPaymentMovement *delivery_payment_movement);


/**
 * d_delivery_payment_movement_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #DeliveryPaymentMovement vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
DeliveryPaymentMovement * d_delivery_payment_movement_create_empty () {
    DeliveryPaymentMovement *delivery_payment_movement = (DeliveryPaymentMovement *) g_malloc(sizeof(DeliveryPaymentMovement));

    delivery_payment_movement_init(delivery_payment_movement);

    return delivery_payment_movement;                           
}

/**
 * d_delivery_payment_movement_clear:
 * @delivery_payment_movement: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_delivery_payment_movement_clear (DeliveryPaymentMovement *delivery_payment_movement) {
    if (delivery_payment_movement!=NULL) {
        if (delivery_payment_movement->delivery_payment_number!=NULL) g_free(delivery_payment_movement->delivery_payment_number);
	g_free(delivery_payment_movement);
    }
    return 0;
}

/**
 * d_delivery_payment_movement_set_enterprise_code:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa en la que se realiza la regularizacin del stock.
 *
 */
void d_delivery_payment_movement_set_enterprise_code (DeliveryPaymentMovement *delivery_payment_movement, gulong enterprise_code) {
    delivery_payment_movement->enterprise_code = enterprise_code;
}

/**
 * d_delivery_payment_movement_set_warehouse_code:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @warehouse_code: Atributo a insertar.
 *
 * Establece el atributo @warehouse_code en el objeto. El atributo @warehouse_code
 * indica el cdigo del almacn en el que se realizar la regularizacin del stock.
 *
 */
void d_delivery_payment_movement_set_warehouse_code (DeliveryPaymentMovement *delivery_payment_movement, gulong warehouse_code) {
    delivery_payment_movement->warehouse_code = warehouse_code;
}

/**
 * d_delivery_payment_movement_set_quantity:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @quantity: Atributo a insertar.
 *
 * Establece el atributo @quantity en el objeto. El atributo @quantity
 * indica la unidades que se regularizarn.
 *
 */
void d_delivery_payment_movement_set_quantity (DeliveryPaymentMovement *delivery_payment_movement, gdouble quantity) {
    delivery_payment_movement->quantity = quantity;
}

/**
 * d_delivery_payment_movement_set_movement_code:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @movement_code: Atributo a insertar.
 *
 * Establece el atributo @movement_code en el objeto. El atributo @movement_code
 * indica el cdigo del movimiento provocado por el albarn.
 *
 */
void d_delivery_payment_movement_set_movement_code (DeliveryPaymentMovement *delivery_payment_movement, gulong movement_code) {
    delivery_payment_movement->movement_code=movement_code;
}
/**
 * d_delivery_payment_movement_set_supplier_code:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @supplier_code: Atributo a insertar.
 *
 * Establece el atributo @supplier_code en el objeto. El atributo @supplier_code
 * indica el cdigo del proveedor que emiti el albarn.
 *
 */
void d_delivery_payment_movement_set_supplier_code (DeliveryPaymentMovement *delivery_payment_movement, gulong supplier_code) {
    delivery_payment_movement->supplier_code=supplier_code;
}
/**
 * d_delivery_payment_movement_set_delivery_payment_number:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @delivery_payment_number: Atributo a insertar.
 *
 * Establece el atributo @delivery_payment_number en el objeto. El atributo @delivery_payment_number
 * indica el nmero de albarn que provoc el movimiento.
 *
 */
void d_delivery_payment_movement_set_delivery_payment_number (DeliveryPaymentMovement *delivery_payment_movement, gchar * delivery_payment_number) {
    delivery_payment_movement->delivery_payment_number=g_strdup(delivery_payment_number);
}
/**
 * d_delivery_payment_movement_set_order_line_number:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @order_line_number: Atributo a insertar.
 *
 * Establece el atributo @order_line_number en el objeto. El atributo @order_line_number
 * indica el nmero de la lnea del pedido que ocupa la pieza que provoc el movimiento.
 *
 */
void d_delivery_payment_movement_set_order_line_number (DeliveryPaymentMovement *delivery_payment_movement, gulong order_line_number) {
    delivery_payment_movement->order_line_number=order_line_number;

}
/**
 * d_delivery_payment_movement_set_order_code:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @order_code: Atributo a insertar.
 *
 * Establece el atributo @order_code en el objeto. El atributo @order_code
 * indica el cdigo del pedido al que se refiere el albarn.
 *
 */
void d_delivery_payment_movement_set_order_code (DeliveryPaymentMovement *delivery_payment_movement, gulong order_code) {
    delivery_payment_movement->order_code=order_code;
}

/**
 * d_delivery_payment_movement_set_product_code:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @product_code: Atributo a insertar.
 *
 * Establece el atributo @product_code en el objeto. El atributo @product_code
 * indica el cdigo del producto para el que produce el movimiento.
 *
 */
void d_delivery_payment_movement_set_product_code (DeliveryPaymentMovement *delivery_payment_movement, gulong product_code) {
    delivery_payment_movement->product_code=product_code;
}

/**
 * d_delivery_payment_movement_set_pending:
 * @delivery_payment_movement: Objeto en el que se establece el atributo.
 * @pending: Atributo a insertar.
 *
 * Establece el atributo @pending en el objeto. El atributo @pending
 * indica si el movimiento es un movimiento ficticio por estar pendiente de llegada.
 *
 */
void d_delivery_payment_movement_set_pending (DeliveryPaymentMovement *delivery_payment_movement, gboolean pending) {
    delivery_payment_movement->pending=pending;
}

/**
 * d_delivery_payment_movement_get_enterprise_code:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_movement_get_enterprise_code (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->enterprise_code;
}

/**
 * d_delivery_payment_movement_get_warehouse_code:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_movement_get_warehouse_code (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->warehouse_code;
}

/**
 * d_delivery_payment_movement_get_quantity:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_delivery_payment_movement_get_quantity (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->quantity;
}

/**
 * d_delivery_payment_movement_get_date:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_delivery_payment_movement_get_date (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->date;
}


/**
 * d_delivery_payment_movement_get_movement_code:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_movement_get_movement_code (DeliveryPaymentMovement *delivery_payment_movement) {
    return(delivery_payment_movement->movement_code);
}

/**
 * d_delivery_payment_movement_get_supplier_code:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_movement_get_supplier_code (DeliveryPaymentMovement *delivery_payment_movement) {
    return(delivery_payment_movement->supplier_code);
}

/**
 * d_delivery_payment_movement_get_delivery_payment_number:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_delivery_payment_movement_get_delivery_payment_number (DeliveryPaymentMovement *delivery_payment_movement) {
    return g_strdup(delivery_payment_movement->delivery_payment_number);
}

/**
 * d_delivery_payment_movement_get_order_line_number:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_movement_get_order_line_number (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->order_line_number;
}

/**
 * d_delivery_payment_movement_get_order_code:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_movement_get_order_code (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->order_code;
}

/**
 * d_delivery_payment_movement_get_product_code:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_delivery_payment_movement_get_product_code (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->product_code;
}

/**
 * d_delivery_payment_movement_get_pending:
 * @delivery_payment_movement: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_delivery_payment_movement_get_pending (DeliveryPaymentMovement *delivery_payment_movement) {
    return delivery_payment_movement->pending;
}

/* Private functions*/

static gint delivery_payment_movement_init(DeliveryPaymentMovement *delivery_payment_movement) {
    delivery_payment_movement->enterprise_code = ULONG_MAX;
    delivery_payment_movement->warehouse_code = ULONG_MAX;
    delivery_payment_movement->movement_code = ULONG_MAX;
    delivery_payment_movement->order_code = ULONG_MAX;
    delivery_payment_movement->supplier_code = ULONG_MAX;
    delivery_payment_movement->order_line_number = ULONG_MAX;
    delivery_payment_movement->delivery_payment_number = NULL;
    delivery_payment_movement->product_code = ULONG_MAX;
    delivery_payment_movement->quantity = G_MINDOUBLE;
    delivery_payment_movement->date = time(NULL);
    delivery_payment_movement->pending = FALSE;

    return 0;
}


