/* direct_debit.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "direct_debit.h"
#include "model.h"
#include "util.h"
#include "stdlib.h"
static gint direct_debit_init(DirectDebit *direct_debit);

/**
 * d_direct_debit_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #DirectDebit vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
DirectDebit *d_direct_debit_create_empty () {
  DirectDebit *direct_debit = (DirectDebit *) g_malloc (sizeof(DirectDebit));

  direct_debit_init(direct_debit);
  
  return direct_debit;
}




/**
 * d_direct_debit_clear:
 * @direct_debit: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_direct_debit_clear (DirectDebit *direct_debit) {

    if (direct_debit!=NULL) {
	g_free(direct_debit);
    }

  return 0;
}

/**
 * d_direct_debit_set_state:
 * @direct_debit: Objeto en el que se establece el atributo.
 * @state: Atributo a insertar.
 *
 * Establece el atributo @state en el objeto. El atributo @state
 * indica el estado de la domiciliacin.
 *
 */
void d_direct_debit_set_state (DirectDebit *direct_debit,gint state) {
    direct_debit-> state= state;
    
}
/**
 * d_direct_debit_set_code:
 * @direct_debit: Objeto en el que se establece el atributo.
 * @direct_debit_code: Atributo a insertar.
 *
 * Establece el atributo @direct_debit_code en el objeto. El atributo @direct_debit_code
 * indica el cdigo de la domiciliacin.
 *
 */
void d_direct_debit_set_code (DirectDebit *direct_debit,gulong direct_debit_code){
    direct_debit-> direct_debit_code= direct_debit_code;
}
/**
 * d_direct_debit_set_enterprise_code:
 * @direct_debit: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica cdigo de la empresa que acta como ordenante de la domiciliacin.
 *
 */
void d_direct_debit_set_enterprise_code (DirectDebit *direct_debit,gulong enterprise_code){
    direct_debit-> enterprise_code= enterprise_code;
}
/**
 * d_direct_debit_set_shop_code:
 * @direct_debit: Objeto en el que se establece el atributo.
 * @shop_code: Atributo a insertar.
 *
 * Establece el atributo @shop_code en el objeto. El atributo @shop_code
 * indica el cdigo de tienda en el que se emiti la factura.
 *
 */
void d_direct_debit_set_shop_code (DirectDebit *direct_debit,gulong shop_code){
    direct_debit-> shop_code= shop_code;
}
/**
 * d_direct_debit_set_invoice_code:
 * @direct_debit: Objeto en el que se establece el atributo.
 * @invoice_code: Atributo a insertar.
 *
 * Establece el atributo @invoice_code en el objeto. El atributo @invoice_code
 * indica el cdigo de empresa en el que se emiti la factura.
 *
 */
void d_direct_debit_set_invoice_code (DirectDebit *direct_debit,gulong invoice_code){
    direct_debit-> invoice_code= invoice_code;
}

/**
 * d_direct_debit_set_is_hire_purchase:
 * @direct_debit: Objeto en el que se establece el atributo.
 * @is_hire_purchase: Atributo a insertar.
 *
 * Establece el atributo @is_hire_purchase en el objeto. El atributo @is_hire_purchase
 * indica si la factura se paga a plazos.
 *
 */
void d_direct_debit_set_is_hire_purchase (DirectDebit *direct_debit, gboolean is_hire_purchase) {
    direct_debit-> is_hire_purchase= is_hire_purchase;
}
/**
 * d_direct_debit_set_current_instalment:
 * @direct_debit: Objeto en el que se establece el atributo.
 * @current_instalment: Atributo a insertar.
 *
 * Establece el atributo @current_instalment en el objeto. El atributo @current_instalment
 * indica el plazo actual de gestin.
 *
 */
void d_direct_debit_set_current_instalment (DirectDebit *direct_debit, gint current_instalment) {
    direct_debit-> current_instalment= current_instalment;

}

/**
 * d_direct_debit_get_state:
 * @direct_debit: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_direct_debit_get_state (DirectDebit *direct_debit){
  return direct_debit->state;
}

/**
 * d_direct_debit_get_code:
 * @direct_debit: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_direct_debit_get_code (DirectDebit *direct_debit){
  return direct_debit->direct_debit_code;
}

/**
 * d_direct_debit_get_enterprise_code:
 * @direct_debit: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_direct_debit_get_enterprise_code (DirectDebit *direct_debit){
  return direct_debit->enterprise_code;
}

/**
 * d_direct_debit_get_shop_code:
 * @direct_debit: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_direct_debit_get_shop_code (DirectDebit *direct_debit){
  return direct_debit->shop_code;
}

/**
 * d_direct_debit_get_invoice_code:
 * @direct_debit: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_direct_debit_get_invoice_code (DirectDebit *direct_debit){
  return direct_debit->invoice_code;
}

/**
 * d_direct_debit_get_is_hire_purchase:
 * @direct_debit: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_direct_debit_get_is_hire_purchase (DirectDebit *direct_debit) {
     return direct_debit->is_hire_purchase;
}
/**
 * d_direct_debit_get_current_instalment:
 * @direct_debit: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_direct_debit_get_current_instalment (DirectDebit *direct_debit) {
     return direct_debit->current_instalment;
}



/* Frees the memory used by all the elements of the list and, then, the GList itself */

/**
 * d_direct_debit_free_glist:
 * @list: Lista de domiciliaciones a liberar.
 *
 * Libera la memoria utilizada por la lista.
 *
 */
void d_direct_debit_free_glist (GList * list) {
  GList * element;

  if (list != NULL)
    {
      element = g_list_first(list);
      
      do {
	d_direct_debit_clear (element->data);
	element = g_list_next(element);
      }
      while (element != NULL);
      
      g_list_free(list);
    }
}


/* Private functions*/

static gint direct_debit_init(DirectDebit *direct_debit) {

    direct_debit->direct_debit_code = ULONG_MAX;
    direct_debit->enterprise_code = ULONG_MAX;
    direct_debit->shop_code = ULONG_MAX;
    direct_debit->invoice_code = ULONG_MAX;
    direct_debit->state = -1;
    direct_debit->is_hire_purchase = FALSE;
    direct_debit->current_instalment = -1;
    return 0;
}

/**
 * d_direct_debit_get_state_message:
 * @state: Estado de una domiciliacin.
 *
 * Devuelve una cadena de caracteres con un mensaje
 * segn el estado de una domiciliacin.
 *
 * Returns:
 * La cadena con el mensaje.
 */
gchar * d_direct_debit_get_state_message (gint state) {
    gchar * result;
    switch (state) {
    case D_DIRECT_DEBIT_GENERATED  :{
	result=g_strdup(D_DIRECT_DEBIT_MESSAGE_GENERATED);
        break;
    }
    case D_DIRECT_DEBIT_PAID  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_PAID);
	break;
    }
    case D_DIRECT_DEBIT_TOTAL_ZERO  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_TOTAL_ZERO);
	break;
    }
    case D_DIRECT_DEBIT_NOT_UP_TO_DATE  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_NOT_UP_TO_DATE);
	break;
    }
    case D_DIRECT_DEBIT_NO_DIRECT_DEBIT  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_NO_DIRECT_DEBIT);
	break;
    }
    case D_DIRECT_DEBIT_NO_OFFICE  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_NO_OFFICE);
	break;
    }
    case D_DIRECT_DEBIT_LEGAL_ID  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_LEGAL_ID);
	break;
    }
    case D_DIRECT_DEBIT_ERROR_OR_CANCEL  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_ERROR_OR_CANCEL);
	break;
    }
    case D_DIRECT_DEBIT_DISAGREEMENT  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_DISAGREEMENT);
	break;
    }
    case D_DIRECT_DEBIT_DUPLICATED  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_DUPLICATED);
	break;
    }
    case D_DIRECT_DEBIT_NO_USED  :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_NO_USED);
	break;
    }
    default :{
        result=g_strdup(D_DIRECT_DEBIT_MESSAGE_NO_USED);
	break;
    }

    }
    return result;
}



