
/* exception.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "exception.h"

static void exception_init(Exception *exception);



/**
 * d_exception_create:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Exception vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Exception * d_exception_create () {

    Exception * exception;

    exception = (Exception *) g_malloc0(sizeof(Exception));
    exception_init(exception);

    return exception;
}

/**
 * d_exception_create_full:
 * @message: Mensaje con el que inicializa la excepcin.
 *
 * Crea un objeto a partir del valor del mensaje.
 *
 * Returns:
 * Un objeto #Exception con inicializado con un mensaje.
 */
Exception * d_exception_create_full (gchar *message) {

    Exception * exception;

    exception = (Exception *) g_malloc0(sizeof(Exception));
    exception_init(exception);

    d_exception_set_message (exception, message);

    return exception;
}

/**
 * d_exception_set_code:
 * @exception: Objeto en el que se establece el atributo.
 * @code: Atributo a insertar.
 *
 * Establece el atributo @code en el objeto. El atributo @code
 * indica 
 *
 */
void d_exception_set_code (Exception * exception,gulong code){
  exception -> code = code;
}

/**
 * d_exception_set_message:
 * @exception: Objeto en el que se establece el atributo.
 * @message: Atributo a insertar.
 *
 * Establece el atributo @message en el objeto. El atributo @message
 * indica 
 *
 */
void d_exception_set_message (Exception * exception, gchar *message){
  gchar *aux = NULL;

  if (message != NULL) {
    if (exception -> message != NULL) { 
      aux = g_strconcat(exception->message, "\n", message, NULL);    
      g_free(exception->message);    
    }
    else aux = g_strdup(message);
    
    exception -> message = aux;
  }
  else {
    if ((exception != NULL)&&(exception->message != NULL)) {
      g_free(exception->message);    
      
      exception -> message = NULL;
    }
  }
}

/**
 * d_exception_get_code:
 * @exception: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_exception_get_code (Exception * exception){
  return exception->code;
}

/**
 * d_exception_get_message:
 * @exception: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_exception_get_message (Exception * exception){
  return g_strdup(exception->message);
}

/**
 * d_exception_destroy:
 * @exception: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 */
void d_exception_destroy (Exception *exception){
  
  if (exception != NULL) {
    if (exception -> message != NULL) g_free(exception->message);
    
    g_free(exception);
  }
}


/* Private functions */

static void exception_init(Exception *exception) {

    exception-> code = ULONG_MAX;
    exception-> message = NULL;
    
}
