/* family.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */



#include "family.h"

static gint family_init(Family *family);

/**
 * d_family_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Family vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Family *d_family_create_empty () {
    Family * family = (Family *) g_malloc (sizeof(Family));

    family_init(family);

    return family;
}


/**
 * d_family_clear:
 * @family: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_family_clear (Family *family) {

    if (family!=NULL){
	if (family->family_code!=NULL) {
	    g_free(family->family_code);
            family->family_code = NULL;
	}
	g_free(family);
        family = NULL;
    }
    return 0;
}

/**
 * d_family_set_code:
 * @family: Objeto en el que se establece el atributo.
 * @family_code: Atributo a insertar.
 *
 * Establece el atributo @family_code en el objeto. El atributo @family_code
 * indica el cdigo que identifica la familia.
 *
 */
void d_family_set_code (Family *family, gchar *family_code){
    if(family->family_code!=NULL) {
	g_free(family->family_code);
    }
    family->family_code = g_strdup (family_code);
}

/**
 * d_family_set_description:
 * @family: Objeto en el que se establece el atributo.
 * @description: Atributo a insertar.
 *
 * Establece el atributo @description en el objeto. El atributo @description
 * indica la descripcin de la familia a la que pertenece una pieza.
 *
 */
void d_family_set_description (Family *family, gchar *description){
    if(family->description!=NULL) {
	g_free(family->description);
    }
    family->description = g_strdup (description);
}

/**
 * d_family_get_code:
 * @family: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_family_get_code (Family *family) {
    return strdup(family->family_code);
}

/**
 * d_family_get_description:
 * @family: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_family_get_description (Family *family){
    return strdup(family->description);
}


/* Private functions*/

static gint family_init(Family *family) {
    family->family_code = NULL;
    family->description  = NULL;

    return 0;
}

