

/* garage.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "garage.h"


static gint garage_init(Garage *garage);
/**
 * d_garage_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Garage vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Garage *d_garage_create_empty () {
    Garage * garage = (Garage *) g_malloc(sizeof(Garage));

    garage_init(garage);

    return garage;
}

/**
 * d_garage_clear:
 * @garage: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_garage_clear (Garage *garage) {

    if (garage!=NULL) {
	d_participant_clear (garage->participant);

	g_free(garage);
    }
    return 0;
}

/**
 * d_garage_set_participant:
 * @garage: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica los campos relacionados con el objeto participante al que referencia el taller.
 *
 */
void d_garage_set_participant (Garage *garage, Participant * participant){
  garage->participant = participant;
}

/**
 * d_garage_set_commission:
 * @garage: Objeto en el que se establece el atributo.
 * @commission: Atributo a insertar.
 *
 * Establece el atributo @commission en el objeto. El atributo @commission
 * indica la comisin que se le pasa a un taller por reparacin enviada desde al
 * mismo a una empresa del grupo.
 *
 */
void d_garage_set_commission (Garage *garage, gdouble commission){
  garage->commission = commission;
}

/**
 * d_garage_get_participant:
 * @garage: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_garage_get_participant (Garage *garage){
  return garage->participant;
}

/**
 * d_garage_get_commission:
 * @garage: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_garage_get_commission (Garage *garage){
  return garage->commission;
}

/**
 * d_garage_duplicate:
 * @garage: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto y devuelve el nuevo objeto.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Garage *d_garage_duplicate (Garage *garage){

  Garage *result = d_garage_create_empty ();
  gdouble commission = d_garage_get_commission (garage);

  d_garage_set_participant (result,d_participant_duplicate (d_garage_get_participant (garage)));
  d_garage_set_commission (result, commission);
  
  return result;

}

/* Private functions*/

static gint garage_init(Garage *garage) 
{
  garage->participant = NULL;
  garage->commission = -1;
  return 0;
}
