/* invoice.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#ifndef _INVOICE_
#define _INVOICE_

#include "config.h"
#include "participant.h"
#include "enterprise.h"
#include "company.h"
#include "shop.h"
#include "payment.h"
#include "post_address.h"
#include "delivery_payment.h"
#include "payment_note.h"
#include "time.h"

#ifndef _GLIB_LIBRARY_
#define _GLIB_LIBRARY_
#include <glib.h>
#endif

/**
 * D_INVOICE_NOT_ISSUED:
 *
 * Cdigo que indica que una factura est pendiente de emisin. Es decir, se ha guardado en la
 * base de datos pero no se ha emitido.
 *
 */
#define D_INVOICE_NOT_ISSUED 0
/**
 * D_INVOICE_NOT_PAID:
 *
 * Cdigo que indica que una factura est pendiente de pago. Es decir, se ha guardado en la
 * base de datos y se ha imprimido pero no ha sido pagada.
 *
 */
#define D_INVOICE_NOT_PAID 1
/**
 * D_INVOICE_PAID:
 *
 * Cdigo que indica que una factura est pagada. Es decir, se ha guardado en la
 * base de datos, se ha imprimido y se ha pagado.
 *
 */
#define D_INVOICE_PAID 2
/**
 * D_INVOICE_MANAGING:
 *
 * Cdigo que indica que una factura est en gestin. Este estado es un estado especial
 * creado a propsito para las facturas de ck y las facturas que se cobran domiciliadas.
 * Es este estado estn las facturas que se han enviado a CK o las que se han pasado al banco
 * a travs del cuaderno 58.
 *
 */
#define D_INVOICE_MANAGING 3

/**
 * D_INVOICE_MESSAGE_NOT_ISSUED:
 *
 * Texto utilizado para mostrar que una factura est pendiente de emisin.
 *
 */
#define D_INVOICE_MESSAGE_NOT_ISSUED _("Not issued")
/**
 * D_INVOICE_MESSAGE_NOT_PAID:
 *
 * Texto utilizado para mostrar que una factura est pendiente de pago.
 *
 */
#define D_INVOICE_MESSAGE_NOT_PAID _("Not paid")
/**
 * D_INVOICE_MESSAGE_PAID:
 *
 * Texto utilizado para mostrar que una factura est pagada.
 *
 */
#define D_INVOICE_MESSAGE_PAID _("Paid")
/**
 * D_INVOICE_MESSAGE_MANAGING:
 *
 * Texto utilizado para mostrar que una factura est en gestin.
 *
 */
#define D_INVOICE_MESSAGE_MANAGING _("In management")

/**
 * D_INVOICE_NORMAL_CODE:
 *
 * Cdigo utilizado para distinguir que una factura se imprime con la plantilla normal.
 *
 */
#define D_INVOICE_NORMAL_CODE 10

/**
 * D_INVOICE_CLIENT_SELECTION:
 *
 * Cdigo que indica que el receptor de una factura ser el cliente de la reparacin.
 *
 */
#define D_INVOICE_CLIENT_SELECTION 0
/**
 * D_INVOICE_COMPANY_SELECTION:
 *
 * Cdigo que indica que el receptor de una factura ser la compaa de seguros.
 *
 */
#define D_INVOICE_COMPANY_SELECTION 1
/**
 * D_INVOICE_CONSORTIUM_SELECTION:
 *
 * Cdigo que indica que el receptor de una factura ser el consorcio.
 *
 */
#define D_INVOICE_CONSORTIUM_SELECTION 2

/**
 * D_INVOICE_CODE:
 *
 * Cdigo que indica el tipo de la factura. En este caso se trata de una factura emitida.
 *
 */
#define D_INVOICE_CODE 4
/**
 * D_INVOICE_SPLIT_INVOICE_CODE:
 *
 * Cdigo que indica el tipo de la factura. En este caso se trata de una refactura.
 *
 */
#define D_INVOICE_SPLIT_INVOICE_CODE 20
/**
 * D_INVOICE_GARAGE_INVOICE_CODE:
 *
 * Cdigo que indica el tipo de la factura. En este caso se trata de una factura de una taller.
 *
 */
#define D_INVOICE_GARAGE_INVOICE_CODE 21
/**
 * D_INVOICE_MESSAGE:
 *
 * Texto para mostrar que la factura es de tipo factura emitida por la empresa.
 *
 */
#define D_INVOICE_MESSAGE _("Invoice")
/**
 * D_INVOICE_SPLIT_INVOICE:
 *
 * Texto para mostrar que la factura es de tipo refactura.
 *
 */
#define D_INVOICE_SPLIT_INVOICE _("Reinvoice")
/**
 * D_INVOICE_MESSAGE_GARAGE_INVOICE:
 *
 * Texto para mostrar que la factura es una factura de un taller.
 *
 */
#define D_INVOICE_MESSAGE_GARAGE_INVOICE _("Invoices for garages")

/**
 * D_INVOICE_MESSAGE_NORMAL:
 *
 * Texto para la plantilla normal.
 *
 */
#define D_INVOICE_MESSAGE_NORMAL _("Normal")

typedef struct invoiceStruct {
    gulong invoice_number;
    gulong invoice_code; /* SE CONVERTIR EN LA CLAVE DE LA FACTURA*/
    Enterprise * enterprise;
    Shop * shop;
    Participant * participant;
    Payment * payment;
    time_t issue_date;
    gint state;
    gdouble invoice_discount;
    gdouble total;
    gboolean modified_total;
    Repair * repair;
    gboolean split_invoice;
    gdouble vat_type;
    gboolean garage_is_receiver;
    gulong enterprise_code_split_invoice;
    gulong shop_code_split_invoice;
    gulong repair_code_split_invoice;
    DeliveryPayment ** delivery_payments;
} Invoice;

/* TipoIva es utilizado solo para saber si la factura es sin Iva.*/
Invoice * d_invoice_create_empty (void);
void d_invoice_set_enterprise (Invoice * invoice,Enterprise * enterprise);
void d_invoice_set_shop (Invoice * invoice,Shop * shop);
void d_invoice_set_invoice_number (Invoice * invoice,gulong invoice_number);
void d_invoice_set_state (Invoice * invoice,gint state);
void d_invoice_set_invoice_discount (Invoice * invoice,gdouble invoice_discount);
void d_invoice_set_participant (Invoice * invoice,Participant * participant);
void d_invoice_set_payment (Invoice * invoice,Payment * payment);
void d_invoice_set_total (Invoice * invoice,gdouble total);
void d_invoice_set_modified_total (Invoice * invoice,gboolean modified_total);
void d_invoice_set_delivery_payments (Invoice * invoice,DeliveryPayment ** delivery_payments);
void d_invoice_set_repair (Invoice * invoice,Repair * repair,gboolean free_repair);
void d_invoice_set_shop_code_split_invoice (Invoice * invoice,gulong shop_code_split_invoice);
void d_invoice_set_enterprise_code_split_invoice (Invoice * invoice,gulong enterprise_code_split_invoice);
void d_invoice_set_repair_code_split_invoice (Invoice * invoice,gulong repair_code_split_invoice);
void d_invoice_set_split_invoice (Invoice * invoice,gboolean split_invoice);
void d_invoice_set_garage_is_receiver (Invoice * invoice,gboolean garage_is_receiver);
void d_invoice_set_vat_type (Invoice * invoice,gdouble vat_type);
void d_invoice_set_invoice_code (Invoice * invoice,gulong invoice_code);

Enterprise * d_invoice_get_enterprise (Invoice * invoice);
Shop * d_invoice_get_shop (Invoice * invoice);
gulong d_invoice_get_invoice_number (Invoice * invoice);
gint d_invoice_get_state (Invoice * invoice);
gdouble d_invoice_get_invoice_discount (Invoice * invoice);
Participant * d_invoice_get_participant (Invoice * invoice);
Payment * d_invoice_get_payment (Invoice * invoice);
DeliveryPayment ** d_invoice_get_delivery_payments (Invoice * invoice);
Repair * d_invoice_get_repair (Invoice * invoice);
void d_invoice_set_issue_date (Invoice * invoice,time_t issue_date);
time_t d_invoice_get_issue_date (Invoice * invoice);
gdouble d_invoice_get_total (Invoice * invoice);
gboolean d_invoice_get_modified_total (Invoice * invoice);
gulong d_invoice_get_shop_code_split_invoice (Invoice * invoice);
gulong d_invoice_get_enterprise_code_split_invoice (Invoice * invoice);
gulong d_invoice_get_repair_code_split_invoice (Invoice * invoice);
gboolean d_invoice_get_split_invoice (Invoice * invoice);
gboolean d_invoice_get_garage_is_receiver (Invoice * invoice);
gdouble d_invoice_get_vat_type (Invoice * invoice);
gulong d_invoice_get_invoice_code (Invoice * invoice);
gboolean d_invoice_get_with_insurance (Invoice * invoice);
gboolean d_invoice_get_is_from_garage (Invoice * invoice);
gchar * d_invoice_get_template_name (gint template_id);
Repair * d_invoice_get_first_repair (Invoice * invoice);
gchar * d_invoice_get_invoice_number_with_group_code (Invoice * invoice);

PaymentNote * d_invoice_to_payment_note (Invoice * invoice);
gint d_invoice_clear (Invoice *invoice);

void d_invoice_calculate_total (gdouble * base,gdouble * vat,
					      gdouble * total,Invoice * invoice,gboolean set_on_invoice);
void d_invoice_calculate_total_to_garage (gdouble * base,gdouble * vat,
					      gdouble * total,Invoice * invoice,gboolean set_on_invoice);
gdouble d_invoice_calculate_total_discount (Invoice * invoice);

gchar ***d_invoice_array_to_content (Invoice **array,Invoice * invoice_total, gulong *reccount);
gchar  **d_invoice_get_content (Invoice *invoice);
gchar  **d_invoice_get_content_title (void);
gchar   *d_invoice_get_listing_title (void);
gint    *d_invoice_get_listing_columns_width (void);
gint    *d_invoice_get_listing_columns_alignment (void);
gint     d_invoice_get_content_size (void);
gchar ***d_invoice_array_to_content_by_company (Invoice **array,Invoice * invoice_total, gulong *reccount);
gchar **d_invoice_to_content_by_company (Invoice * invoice,gdouble * total_by_company,gboolean resumen);
gint d_invoice_get_number_of_delivery_payments (Invoice * invoice);
#endif

