/* listing_operation.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "listing_operation.h"
#include "stdlib.h"
#include "time.h"
#include "util.h"

#define D_LISTING_OPERATION_CONTENT_SIZE 5

static gint listing_operation_init(ListingOperation *listing_operation);


/**
 * d_listing_operation_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #ListingOperation vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
ListingOperation *d_listing_operation_create_empty () {
  ListingOperation *listing_operation = (ListingOperation *) g_malloc (sizeof(ListingOperation));

  listing_operation_init(listing_operation);
  
  return listing_operation;
}



/**
 * d_listing_operation_set_company_name:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @company_name: Atributo a insertar.
 *
 * Establece el atributo @company_name en el objeto. El atributo @company_name
 * indica la compaa para la que se desea listar los movimientos.
 *
 */
void d_listing_operation_set_company_name (ListingOperation *listing_operation, gchar * company_name) {
    if(listing_operation->company_name!=NULL) {
	g_free(listing_operation->company_name);
    }
    listing_operation->company_name = g_strdup(company_name);
}

/**
 * d_listing_operation_set_company_code:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @company_code: Atributo a insertar.
 *
 * Establece el atributo @company_code en el objeto. El atributo @company_code
 * indica el cdigo de la compaa de seguros del objeto
 *
 */
void d_listing_operation_set_company_code (ListingOperation *listing_operation, gulong company_code) {
    listing_operation->company_code = company_code;
}

/**
 * d_listing_operation_set_operation_code:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @operation_code: Atributo a insertar.
 *
 * Establece el atributo @operation_code en el objeto. El atributo @operation_code
 * indica el cdigo del movimiento del objeto.
 *
 */
void d_listing_operation_set_operation_code (ListingOperation *listing_operation, gulong operation_code) {
    listing_operation->operation_code = operation_code;
}


/**
 * d_listing_operation_set_bank_code:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @bank_code: Atributo a insertar.
 *
 * Establece el atributo @bank_code en el objeto. El atributo @bank_code
 * indica el cdigo del banco del nmero de cuenta para el que se quiere listar el movimiento.
 *
 */
void d_listing_operation_set_bank_code (ListingOperation *listing_operation, gint bank_code) {
    listing_operation->bank_code = bank_code;
}
/**
 * d_listing_operation_set_office_code:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @office_code: Atributo a insertar.
 *
 * Establece el atributo @office_code en el objeto. El atributo @office_code
 * indica el cdigo de la oficina del nmero de cuenta para el que se quiere listar el movimiento.
 *
 */
void d_listing_operation_set_office_code (ListingOperation *listing_operation, gint office_code) {
    listing_operation->office_code = office_code;
}

/**
 * d_listing_operation_set_account_number:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @account_number: Atributo a insertar.
 *
 * Establece el atributo @account_number en el objeto. El atributo @account_number
 * indica el nmero de cuenta para el movimiento a listar
 *
 */
void d_listing_operation_set_account_number (ListingOperation *listing_operation, gchar * account_number) {
    if(listing_operation->account_number!=NULL) {
        g_free(listing_operation->account_number);
    }
    listing_operation->account_number = g_strdup(account_number);
}

/**
 * d_listing_operation_set_number:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @number: Atributo a insertar.
 *
 * Establece el atributo @number en el objeto. El atributo @number
 * actualmente no se utiliza.
 *
 */
void d_listing_operation_set_number (ListingOperation *listing_operation, gchar * number) {
    if(listing_operation->number!=NULL) {
        g_free(listing_operation->number);
    }
    listing_operation->number = g_strdup(number);
}

/**
 * d_listing_operation_set_amount:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @amount: Atributo a insertar.
 *
 * Establece el atributo @amount en el objeto. El atributo @amount
 * indica el total del movimiento.
 *
 */
void d_listing_operation_set_amount (ListingOperation *listing_operation, gdouble amount){
    listing_operation->amount = amount;
}
/**
 * d_listing_operation_set_receipt_reference:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @receipt_reference: Atributo a insertar.
 *
 * Establece el atributo @receipt_reference en el objeto. El atributo @receipt_reference
 * indica la referencia del recibo bancario.
 *
 */
void d_listing_operation_set_receipt_reference (ListingOperation *listing_operation, gchar * receipt_reference){
    if(listing_operation->receipt_reference!=NULL) {
        g_free(listing_operation->receipt_reference);
    }
    listing_operation->receipt_reference = g_strdup(receipt_reference);
}

/**
 * d_listing_operation_set_date:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @date: Atributo a insertar.
 *
 * Establece el atributo @date en el objeto. El atributo @date
 * indica la fecha en la que se realiz el movimiento.
 *
 */
void d_listing_operation_set_date (ListingOperation *listing_operation, time_t date){
    listing_operation->date = date;
}
/**
 * d_listing_operation_set_notification_date:
 * @listing_operation: Objeto en el que se establece el atributo.
 * @notification_date: Atributo a insertar.
 *
 * Establece el atributo @notification_date en el objeto. El atributo @notification_date
 * indica la fecha en la que se realiz la notificacin del movimiento.
 *
 */
void d_listing_operation_set_notification_date (ListingOperation *listing_operation, time_t notification_date){
    listing_operation->notification_date = notification_date;
}

/**
 * d_listing_operation_get_company_name:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_listing_operation_get_company_name (ListingOperation *listing_operation) {
    return g_strdup(listing_operation->company_name);
}

/**
 * d_listing_operation_get_company_code:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_listing_operation_get_company_code (ListingOperation *listing_operation) {
    return listing_operation->company_code;
}

/**
 * d_listing_operation_get_operation_code:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_listing_operation_get_operation_code (ListingOperation *listing_operation) {
    return listing_operation->operation_code;
}

/**
 * d_listing_operation_get_bank_code:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_listing_operation_get_bank_code (ListingOperation *listing_operation) {
    return listing_operation->bank_code;
}

/**
 * d_listing_operation_get_office_code:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gint d_listing_operation_get_office_code (ListingOperation *listing_operation) {
    return listing_operation->office_code;
}

/**
 * d_listing_operation_get_account_number:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_listing_operation_get_account_number (ListingOperation *listing_operation) {
    return g_strdup(listing_operation->account_number);
}

/**
 * d_listing_operation_get_number:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_listing_operation_get_number (ListingOperation *listing_operation) {
    return g_strdup(listing_operation->number);
}


/**
 * d_listing_operation_get_amount:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_listing_operation_get_amount (ListingOperation *listing_operation){
    return listing_operation->amount;
}

/**
 * d_listing_operation_get_receipt_reference:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_listing_operation_get_receipt_reference (ListingOperation *listing_operation){
    return g_strdup(listing_operation->receipt_reference);
}

/**
 * d_listing_operation_get_date:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_listing_operation_get_date (ListingOperation *listing_operation){
    return listing_operation->date;
}

/**
 * d_listing_operation_get_notification_date:
 * @listing_operation: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_listing_operation_get_notification_date (ListingOperation *listing_operation){
    return listing_operation->notification_date;
}


/**
 * d_listing_operation_clear:
 * @listing_operation: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_listing_operation_clear (ListingOperation *listing_operation) {

    if (listing_operation!=NULL) {
	if (listing_operation->company_name!=NULL)
	    g_free(listing_operation->company_name);
	if (listing_operation->account_number!=NULL)
	    g_free(listing_operation->account_number);
	if (listing_operation->number!=NULL)
	    g_free(listing_operation->number);
	if (listing_operation->receipt_reference!=NULL)
	    g_free(listing_operation->receipt_reference);
	g_free(listing_operation);
    }

  return 0;
}



/**
 * d_listing_d_operation_array_to_content:
 * @array: Array de movimientos que se transformarn en listables.
 * @reccount: Puntero a un %gulong que indicar la suma total de los objetos transformados.
 *
 * Funcin utilizada para transformar un array de objetos en un array de arrays de cadenas
 * de caracteres listables. Es utilizada para realizar los listados impresos en papel.
 *
 * Returns:
 * Un triple puntero a %gchar, es decir, un puntero a un array de arrays de cadenas de caracteres. Se
 * podra ver como un puntero a una tabla con filas y colummnas de cadenas de caracteres.
 */
gchar ***d_listing_d_operation_array_to_content (ListingOperation **array, gulong *reccount) {
    gint     array_size = 0;
    gchar ***content = NULL;

    *reccount = 0;
    while(array[array_size++]!=NULL);

    content = (gchar ***) malloc(sizeof(gchar **)*array_size);

    while(array[*reccount] != NULL) {
	content[*reccount] = d_listing_operation_get_content (array[*reccount]);
        (*reccount)++;
    }

    content[*reccount] = NULL;

    return content;

}

/**
 * d_listing_operation_get_content_title:
 *
 * Devuelve las cabaceras de cada pgina de los listados para el objeto #ListingOperation. 
 *
 * Returns:
 * Un array de cadena de caracteres con las cabeceras de pgina. EL array acaba en NULL.
 */
gchar **d_listing_operation_get_content_title () {
    gchar **content = (gchar **) malloc (sizeof(gchar *)*d_listing_operation_get_content_size ()+1);
    gint index = 0;

    content[index++] = g_strdup_printf(_("NOTIF."));
    content[index++] = g_strdup_printf(_("NUMBER"));
    content[index++] = g_strdup_printf(_("TRADE NAME"));
    content[index++] = g_strdup_printf(_("DATE"));
    content[index++] = g_strdup_printf(_("AMOUNT"));
    content[index++] = NULL;
    return content;
}

/**
 * d_listing_operation_get_listing_title:
 * @tipo: Tipo de pago por el que se realiz el movimiento.
 *
 * Obtiene el ttulo del listado.
 *
 * Returns:
 * El ttulo del listado.
 */
gchar *d_listing_operation_get_listing_title (gint tipo) {

    gchar *content = NULL;
    if (tipo == D_PAYMENT_TRANSFER_CODE)
	content = g_strdup(_("PAYMENTS BY TRANSFER LISTING"));
    else if (tipo == D_PAYMENT_CREDIT_CARD)
	content = g_strdup(_("PAYMENTS BY CREDIT CARD LISTING"));
    else if (tipo == D_PAYMENT_CHEQUE)
	content = g_strdup(_("PAYMENTS BY CHEQUE LISTING"));
    else if (tipo == D_PAYMENT_DIRECT_DEBIT_CODE)
	content = g_strdup(_("PAYMENTS BY DIRECT DEBIT LISTING"));
    else
	content = g_strdup(_("PAYMENTS LISTING"));

    return content;
}

/**
 * d_listing_operation_get_listing_columns_width:
 *
 * Indica los anchos de columna de los listados de movimientos.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_listing_operation_get_listing_columns_width () {
    gint *widths = (gint *) malloc (sizeof(gint)*d_listing_operation_get_content_size ()+1);
    gint index = 0;

    widths[index++] = 10;
    widths[index++] = 15;
    widths[index++] = 45;
    widths[index++] = 15;
    widths[index++] = 15;
    widths[index++] = -1;

    return widths;
}

/**
 * d_listing_operation_get_listing_columns_alignment:
 *
 * Indica los anchos de columna de los listados de movimientos.
 *
 * Returns:
 * Un array de enteros que indican el ancho de cada columna en el listado. EL array acaba en -1.
 */
gint *d_listing_operation_get_listing_columns_alignment () {
    gint *orientation = (gint *) malloc (sizeof(gint)*d_listing_operation_get_content_size ()+1);
    gint index = 0;

    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_LEFT;
    orientation[index++] = U_RIGHT;
    orientation[index++] = -1;

    return orientation;
}

/**
 * d_listing_operation_get_content_size:
 *
 * Obtiene el nmero de columnas que tendr el listado.
 *
 * Returns:
 * El nmero de columnas.
 */
gint d_listing_operation_get_content_size () {
    return D_LISTING_OPERATION_CONTENT_SIZE;
}

/**
 * d_listing_operation_get_content:
 * @listing_operation: Objeto #ListingOperation a partir de la que se obtendrn los valores para el listado.
 *
 * Transforma un objeto factura en un array de cadenas de caracteres para aadir al listado
 * de movimientos.
 *
 * Returns:
 * Array con los valores a sacar por listado en modo de cadenas de caracteres.
 */
gchar **d_listing_operation_get_content (ListingOperation *listing_operation) {
    time_t  date = d_listing_operation_get_date (listing_operation);
    time_t  notification_date = d_listing_operation_get_notification_date (listing_operation);
    gchar  *temp = u_get_parsed_date (&date);
    gchar  *notification_temp = u_get_parsed_date (&notification_date);
    gchar **content = (gchar **) g_malloc0(sizeof(gchar *)*d_listing_operation_get_content_size ()+1);
    gint    index   = 0;
    gchar * number= NULL;
    gulong aux_code = ULONG_MAX;
//    gchar *temp_company=NULL;
    gchar *company_name;
    u_delete_intros (temp);
    u_delete_intros (notification_temp);

    number = d_listing_operation_get_number (listing_operation);
    if (number == NULL) {
//	number = d_listing_operation_get_complete_account_number (listing_operation);
        
//        if (number == NULL)
	    number = g_strdup("");
    }
//    temp_company = d_listing_operation_get_company_name (listing_operation);
    aux_code = d_listing_operation_get_company_code (listing_operation);
//    company_name = d_company_get_short_name (NULL,aux_code,temp_company);
    company_name = d_listing_operation_get_company_name (listing_operation);

    if (company_name==NULL)
        company_name = g_strdup("");
    content[index++] = temp;
    content[index++] = number;
    content[index++] = company_name;
    content[index++] = notification_temp;
    content[index++] = g_strdup_printf("%.2f",d_listing_operation_get_amount (listing_operation));
    content[index++] = NULL;
//    g_free(temp_company);
    return content;
}

/**
 * d_listing_operation_get_complete_account_number:
 * @listing_operation: Objeto del que se obtendr el nmero de cuenta completo.
 *
 * Calcula el nmero de cuenta completo, es decir, el cdigo cuenta cliente para un
 * movimiento.
 *
 * Returns:
 * La cadena de caracteres con el CCC.
 */
gchar * d_listing_operation_get_complete_account_number (ListingOperation * listing_operation) {
    gchar * bank_code;
    gchar * office_code ;
    gchar * account_code ;
    gchar * complete_account_number= NULL ;
    gchar * cd=NULL ;
    bank_code = g_strdup_printf("%d",d_listing_operation_get_bank_code (listing_operation));
    office_code = g_strdup_printf("%d",d_listing_operation_get_office_code (listing_operation));
    account_code = d_listing_operation_get_account_number (listing_operation);
    if ((bank_code!=NULL) && (office_code!=NULL) && (account_code!=NULL)) {
	gchar * aux;
	aux = u_fill_with_zeros_until (bank_code,4);
	g_free(bank_code);
	bank_code = aux;
	aux=NULL;

	aux = u_fill_with_zeros_until (office_code,4);
	g_free(office_code);
	office_code = aux;
	aux=NULL;

	aux = u_fill_with_zeros_until (account_code,10);
	g_free(account_code);
	account_code = aux;
	aux=NULL;

	cd = u_calculate_control_digit (bank_code,office_code,account_code);

	complete_account_number=g_strconcat(bank_code," ",office_code," ",cd," ",account_code,NULL);
    }
    return complete_account_number;
}




/* Private functions*/

static gint listing_operation_init(ListingOperation *listing_operation) {

    listing_operation->account_number=NULL;
    listing_operation->number=NULL;
    listing_operation->office_code=-1;
    listing_operation->amount = -1;
    listing_operation->operation_code = ULONG_MAX;
    listing_operation->company_code = ULONG_MAX;
    listing_operation->bank_code=-1;
    listing_operation->company_name = NULL;
    listing_operation->receipt_reference = NULL;
    listing_operation->date = time(NULL);
    listing_operation->notification_date = time(NULL);
    return 0;
}
