/* manodeobra.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "manodeobra.h"


static gint man_power_init(ManPower *man_power);

/**
 * d_man_power_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #ManPower vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
ManPower *d_man_power_create_empty () {
    ManPower * man_power = (ManPower *) g_malloc (sizeof(ManPower));

    man_power_init(man_power);

    return man_power;
}


/**
 * d_man_power_clear:
 * @man_power: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_man_power_clear (ManPower *man_power) {
    if (man_power!=NULL) {
	d_product_clear ((Product *)man_power);
	g_free(man_power);
        man_power=NULL;
    }

    return 0;
}

/**
 * d_man_power_set_product:
 * @man_power: Objeto en el que se establece el atributo.
 * @product: Atributo a insertar.
 *
 * Establece el atributo @product en el objeto. El atributo @product
 * indica la referencia de la mano de obra al producto que la compone.
 *
 */
void d_man_power_set_product (ManPower *man_power, Product product){
    d_product_clear (&man_power->product);
    man_power->product = product;
}

/**
 * d_man_power_set_code:
 * @man_power: Objeto en el que se establece el atributo.
 * @man_power_code: Atributo a insertar.
 *
 * Establece el atributo @man_power_code en el objeto. El atributo @man_power_code
 * indica el identificador interno de la mano de obra.
 *
 */
void d_man_power_set_code (ManPower *man_power, gulong man_power_code){
    man_power->man_power_code = man_power_code;
}

/**
 * d_man_power_get_product:
 * @man_power: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Product d_man_power_get_product (ManPower *man_power){
  return man_power->product;
}

/**
 * d_man_power_get_code:
 * @man_power: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_man_power_get_code (ManPower *man_power){
    return man_power->man_power_code;
}

/**
 * d_man_power_duplicate:
 * @man_power: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
ManPower *d_man_power_duplicate (ManPower *man_power){

  ManPower *result = d_man_power_create_empty ();
  Product product = d_man_power_get_product (man_power);
  Product *dup_product = d_product_duplicate (&product);
  gulong man_power_code = d_man_power_get_code (man_power);
  
  d_man_power_set_code (result, man_power_code);
  d_man_power_set_product (result, *dup_product);

  g_free(dup_product);

  return result;
  
}

/* Private functions*/

static gint man_power_init(ManPower *man_power)  {
  man_power->product = d_product_create_empty ();
  man_power->man_power_code = ULONG_MAX;
  
  return 0;
}

