/* payment.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#ifndef _PAYMENT_
#define _PAYMENT_

#include "config.h"
#include <time.h>
#include "enterprise.h"
#include "shop.h"

#ifndef _GLIB_LIBRARY_
#define _GLIB_LIBRARY_
#include <glib.h>
#endif

/**
 * D_PAYMENT_TRANSFER_CODE:
 *
 * Cdigo que indica que el medio de pago es por transferencia.
 *
 */
#define D_PAYMENT_TRANSFER_CODE 5
/**
 * D_PAYMENT_IN_CASH:
 *
 * Cdigo que indica que el medio de pago es en efectivo.
 *
 */
#define D_PAYMENT_IN_CASH 6
/**
 * D_PAYMENT_CREDIT_CARD:
 *
 * Cdigo que indica que el medio de pago es por tarjeta.
 *
 */
#define D_PAYMENT_CREDIT_CARD 7
/**
 * D_PAYMENT_CHEQUE:
 *
 * Cdigo que indica que el medio de pago es por taln.
 *
 */
#define D_PAYMENT_CHEQUE 8
/**
 * D_PAYMENT_DIRECT_DEBIT_CODE:
 *
 * Cdigo que indica que el medio de pago es por domiciliacin.
 *
 */
#define D_PAYMENT_DIRECT_DEBIT_CODE 9
/**
 * D_PAYMENT_AT_MOMENT:
 *
 * Cdigo que indica que el modo de pago es al contado.
 *
 */
#define D_PAYMENT_AT_MOMENT 0
/**
 * D_PAYMENT_MONEY_ORDER:
 *
 * Cdigo que indica que el modo de pago es mediante un giro a x dias.
 *
 */
#define D_PAYMENT_MONEY_ORDER  1
/**
 * D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH:
 *
 * Cdigo que indica que el modo de pago es mediante un giro a la vista
 *
 */
#define D_PAYMENT_MONEY_ORDER_AT_END_OF_MONTH  2
/**
 * D_PAYMENT_HIRE_PURCHASE:
 *
 * Cdigo que indica que el modo de pago es a plazos.
 *
 */
#define D_PAYMENT_HIRE_PURCHASE 15

/**
 * D_PAYMENT_MESSAGE_TRANSFER:
 *
 * Texto utilizado para mostrar que el pago es por transferencia.
 *
 */
#define D_PAYMENT_MESSAGE_TRANSFER _("Transfer")
/**
 * D_PAYMENT_MESSAGE_IN_CASH:
 *
 * Texto utilizado para mostrar que el pago es en efectivo.
 *
 */
#define D_PAYMENT_MESSAGE_IN_CASH _("Cash")
/**
 * D_PAYMENT_MESSAGE_CREDIT_CARD:
 *
 * Texto utilizado para mostrar que el pago es con tarjeta.
 *
 */
#define D_PAYMENT_MESSAGE_CREDIT_CARD _("Credit Card")
/**
 * D_PAYMENT_MESSAGE_CHEQUE:
 *
 * Texto utilizado para mostrar que el pago es con taln.
 *
 */
#define D_PAYMENT_MESSAGE_CHEQUE _("Cheque")
/**
 * D_PAYMENT_MESSAGE_DIRECT_DEBIT:
 *
 * Texto utilizado para mostrar que el pago es por domiciliacin.
 *
 */
#define D_PAYMENT_MESSAGE_DIRECT_DEBIT _("Direct debit")
/**
 * D_PAYMENT_MESSAGE_AT_MOMENT:
 *
 * Texto utilizado para mostrar que el pago es al contado .
 * 
 */
#define D_PAYMENT_MESSAGE_AT_MOMENT _("At moment")
/**
 * D_PAYMENT_MESSAGE_MONEY_ORDER:
 *
 * Texto utilizado para mostrar que el pago es mediante giro.
 *
 */
#define D_PAYMENT_MESSAGE_MONEY_ORDER  _("Money order")
/**
 * D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH:
 *
 * Texto utilizado para mostrar que el pago es mediante giro a la vista .
 *
 */
#define D_PAYMENT_MESSAGE_MONEY_ORDER_AT_END_OF_MONTH  _("Money order at end of month")
/**
 * D_PAYMENT_MESSAGE_HIRE_PURCHASE:
 *
 * Texto utilizado para mostrar que el pago es a plazos.
 *
 */
#define D_PAYMENT_MESSAGE_HIRE_PURCHASE _("Hire purchase")


typedef struct paymentStruct {
    Enterprise * enterprise;
    Shop * shop;
    gulong payment_code;
    gulong achieved_payments;
    gdouble entry_percentage;
    gshort number_expiries;
    time_t achieved_date;
    time_t init_date;
    gshort interval_days;
    gboolean is_hire_purchase;
    gboolean is_transfer;
    gboolean is_cheque;
    gchar * cheque_number;
    gboolean is_credit_card;
    gboolean is_direct_debit;
    gint bank_code;
    gint office_code;
    gchar *  account_number;
    gchar * credit_card_number;
    time_t expiry_date;
    gboolean is_paid_cash;
} Payment;

Payment * d_payment_create_empty (void);
void d_payment_set_enterprise (Payment * payment,Enterprise * enterprise);
void d_payment_set_shop (Payment * payment,Shop * shop);
void d_payment_set_payment_code (Payment * payment,gulong payment_code);
void d_payment_set_achieved_payments (Payment * payment,gulong achieved_payments);
void d_payment_set_achieved_date (Payment * payment,time_t achieved_date);
void d_payment_set_entry_percentage (Payment * payment,gdouble entry_percentage);
void d_payment_set_number_expiries (Payment * payment,gshort number_expiries);
void d_payment_set_init_date (Payment * payment,time_t init_date);
void d_payment_set_interval_days (Payment * payment,gshort interval_days);
void d_payment_set_is_hire_purchase (Payment * payment,gboolean is_hire_purchase);
void d_payment_set_is_transfer (Payment * payment,gboolean is_transfer);
void d_payment_set_is_cheque (Payment * payment,gboolean is_cheque);
void d_payment_set_cheque_number (Payment * payment,gchar * cheque_number);
void d_payment_set_is_credit_card (Payment * payment,gboolean is_credit_card);
void d_payment_set_is_direct_debit (Payment * payment,gboolean is_direct_debit);
void d_payment_set_credit_card_number (Payment * payment,gchar * credit_card_number);
void d_payment_set_expiry_date (Payment * payment,time_t expiry_date);
void d_payment_set_is_paid_cash (Payment * payment,gboolean is_paid_cash);
void d_payment_set_bank_code (Payment * payment,gint bank_code);
void d_payment_set_office_code (Payment * payment,gint office_code);
void d_payment_set_account_number (Payment * payment,gchar * account_number);

Enterprise * d_payment_get_enterprise (Payment * payment);
Shop * d_payment_get_shop (Payment * payment);
gulong d_payment_get_payment_code (Payment * payment);
gulong d_payment_get_achieved_payments (Payment * payment);
time_t d_payment_get_achieved_date (Payment * payment);
gdouble d_payment_get_entry_percentage (Payment * payment);
gshort d_payment_get_number_expiries (Payment * payment);
time_t d_payment_get_init_date (Payment * payment);
gshort d_payment_get_interval_days (Payment * payment);
gboolean d_payment_get_is_hire_purchase (Payment * payment);
gboolean d_payment_get_is_transfer (Payment * payment);
gboolean d_payment_get_is_cheque (Payment * payment);
gchar * d_payment_c_util_get_cheque_number (Payment * payment);
gboolean d_payment_get_is_credit_card (Payment * payment);
gboolean d_payment_get_is_direct_debit (Payment * payment);
gboolean d_payment_get_is_paid_in_cash (Payment * payment);
gchar * d_payment_get_credit_card_number (Payment * payment);
time_t d_payment_get_expiry_date (Payment * payment);
gboolean d_payment_get_is_paid_cash (Payment * payment);
gint d_payment_get_bank_code (Payment * payment);
gint d_payment_get_office_code (Payment * payment);
gchar *  d_payment_get_account_number (Payment * payment);

gint d_payment_clear (Payment *payment);

Payment * d_payment_duplicate (Payment * payment_to_dup);





#endif

