/* payment_note.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "payment_note.h"


static gint payment_note_init(PaymentNote *payment_note);



/**
 * d_payment_note_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #PaymentNote vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
PaymentNote * d_payment_note_create_empty () {

    PaymentNote * payment_note;

    payment_note = (PaymentNote *) g_malloc0(sizeof(PaymentNote));

    payment_note_init(payment_note);

    return payment_note;
}


/**
 * d_payment_note_set_note_code:
 * @payment_note: Objeto en el que se establece el atributo.
 * @note_code: Atributo a insertar.
 *
 * Establece el atributo @emission_date en el objeto. El atributo @emission_date
 * indica el cdigo interno que identifica la nota de abono.
 *
 */
void d_payment_note_set_note_code (PaymentNote * payment_note,gulong note_code){
    payment_note->note_code = note_code;
}

/**
 * d_payment_note_set_repair_detail:
 * @payment_note: Objeto en el que se establece el atributo.
 * @repair_detail: Atributo a insertar.
 *
 * Establece el atributo @repair_detail en el objeto. El atributo @repair_detail
 * indica la reparacin a la que se refiere la nota.
 *
 */
void d_payment_note_set_repair_detail (PaymentNote * payment_note,Repair * repair_detail){
    payment_note->repair_detail=repair_detail;
}

/**
 * d_payment_note_set_payment:
 * @payment_note: Objeto en el que se establece el atributo.
 * @payment: Atributo a insertar.
 *
 * Establece el atributo @payment en el objeto. El atributo @payment
 * indica el pago asociado a la nota.
 *
 */
void d_payment_note_set_payment (PaymentNote * payment_note,Payment * payment){
    payment_note->payment=payment;
}

/**
 * d_payment_note_set_total:
 * @payment_note: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el total de la nota.
 *
 */
void d_payment_note_set_total (PaymentNote * payment_note,gdouble total){
    payment_note->total=total;
}

/**
 * d_payment_note_set_emission_date:
 * @payment_note: Objeto en el que se establece el atributo.
 * @emission_date: Atributo a insertar.
 *
 * Establece el atributo @emission_date en el objeto. El atributo @emission_date
 * indica la fecha de emisin de la nota.
 *
 */
void d_payment_note_set_emission_date (PaymentNote * payment_note,time_t emission_date) {
    payment_note->emission_date=emission_date;
}

/**
 * d_payment_note_get_note_code:
 * @payment_note: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_payment_note_get_note_code (PaymentNote * payment_note){
    return payment_note->note_code;
}

/**
 * d_payment_note_get_repair_detail:
 * @payment_note: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Repair * d_payment_note_get_repair_detail (PaymentNote * payment_note){
    return payment_note->repair_detail;
}

/**
 * d_payment_note_get_payment:
 * @payment_note: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Payment * d_payment_note_get_payment (PaymentNote * payment_note){
    return payment_note->payment;
}

/**
 * d_payment_note_get_total:
 * @payment_note: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_payment_note_get_total (PaymentNote * payment_note){
    return payment_note->total;
}

/**
 * d_payment_note_get_emission_date:
 * @payment_note: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_payment_note_get_emission_date (PaymentNote * payment_note){
    return payment_note->emission_date;
}


/**
 * d_payment_note_clear:
 * @payment_note: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_payment_note_clear (PaymentNote * payment_note) {
    if (payment_note!=NULL) {
	d_repair_clear (payment_note->repair_detail);
	//    clear_repair_detail(payment_note->repair_detail);
	d_payment_clear (payment_note->payment);
	g_free(payment_note);
    }
    return 0;
}

/* Private functions */

static gint payment_note_init(PaymentNote *payment_note) {

    payment_note->emission_date=ULONG_MAX;
    payment_note->repair_detail=NULL;
    payment_note->payment=NULL;
    payment_note->emission_date=time(NULL);
    payment_note->total=G_MINDOUBLE;
    return 0;
}

/**
 * d_payment_note_calculate_total:
 * @base: Puntero a gdouble que almacenar el total de la nota.
 * @iva: Puntero a gdouble que almacenar 0 ya que la nota no posee iva
 * @total: Puntero a gdouble que almacenar el total  de la nota.
 * @payment_note: Nota a partir de la cual se realizar el clculo.
 *
 * Realiza el clculo de los totales de una nota.
 * Para realizar los clculos se obtiene la reparacin, se recorren
 * las lneas de la reparacin comprobando si existe un total de lnea o un precio del producto para calcular
 * el total de la lnea final y se calcula la suma de todas las lneas. El cdigo es muy similar a
 * d_delivery_payment_calculate_total_data ().
 *
 *
 */
void d_payment_note_calculate_total (gdouble * base,gdouble * iva,
						   gdouble * total,PaymentNote * payment_note) {
    gdouble descuento;
    gdouble precio=0;
    gdouble importeTotal=0;
    gdouble totalLinea;
    Repair * repair_detail = d_payment_note_get_repair_detail (payment_note);
    gint countAux2=0;


    if (d_payment_note_get_total (payment_note)!=G_MINDOUBLE) {
	while (repair_detail->repair_line[countAux2]!=NULL) {
	    Repair_line * repair_line = repair_detail->repair_line[countAux2];
	    gdouble precioLinea = d_repair_line_get_price (repair_line);
	    gdouble precioAux=G_MINDOUBLE;
	    if (precioLinea==G_MINDOUBLE) {
		precioAux = d_product_get_price (d_repair_line_get_product(repair_line));

	    } else {
		precioLinea = precioLinea * u_round_to_decimal (d_repair_line_get_quantity (repair_line),3);

	    }

	    descuento = d_repair_line_get_discount (repair_line);
	    if ((descuento<0) || (descuento>100))
		descuento=0;

            if (precioAux!=G_MINDOUBLE)
		precio = 1.0 * precioAux * u_round_to_decimal (d_repair_line_get_quantity (repair_line),3);

	    if (precioLinea!=G_MINDOUBLE) {
		totalLinea = precioLinea;
	    } else if (precio!=G_MINDOUBLE) {
		totalLinea = precio - (((1.0*descuento)/100)*precio);
	    } else {
                totalLinea = 0;
	    }

	    importeTotal = importeTotal + totalLinea;
	    countAux2++;
	}
    } else {
        importeTotal = d_payment_note_get_total (payment_note);
    }

    *base = importeTotal;
    *iva = 0;
    *total = importeTotal;

}


