/* post_address.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "post_address.h"



static gint post_address_init(PostAddress *post_address);

/**
 * d_post_address_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #PostAddress vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
PostAddress * d_post_address_create_empty ()
{
    PostAddress * post_address = (PostAddress *) g_malloc (sizeof(PostAddress));

    post_address_init(post_address);

    return post_address;
}



/**
 * d_post_address_clear:
 * @post_address: Direccin de envo.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_post_address_clear (PostAddress *post_address)
{
    if (post_address!=NULL) {
	if (post_address->street!=NULL) {
	    g_free(post_address->street);
	    post_address->street = NULL;
	}
	if (post_address->number!=NULL) {
	    g_free(post_address->number);
	    post_address->number = NULL;
	}
	if (post_address->floor!=NULL) {
	    g_free(post_address->floor);
	    post_address->floor = NULL;
	}
	if (post_address->town!=NULL) {
	    g_free(post_address->town);
	    post_address->town = NULL;
	}
	if (post_address->province!=NULL) {
	    g_free(post_address->province);
	    post_address->province = NULL;
	}
	g_free(post_address);
	post_address = NULL;
    }

    return 0;
}

/**
 * d_post_address_set_code:
 * @post_address: Objeto en el que se establece el atributo.
 * @code: Atributo a insertar.
 *
 * Establece el atributo @code en el objeto. El atributo @code
 * indica cdigo interno que identifica la direccin de envo.
 *
 */
void d_post_address_set_code (PostAddress * post_address,gulong code){
    post_address->code = code;
}


/**
 * d_post_address_set_participant_code:
 * @post_address: Objeto en el que se establece el atributo.
 * @participant_code: Atributo a insertar.
 *
 * Establece el atributo @participant_code en el objeto. El atributo @participant_code
 * indica cdigo del participante al que pertenece la direccin de envo.
 *
 */
void d_post_address_set_participant_code (PostAddress * post_address,gulong participant_code){
    post_address->participant_code = participant_code;
}



/**
 * d_post_address_set_street:
 * @post_address: Objeto en el que se establece el atributo.
 * @street: Atributo a insertar.
 *
 * Establece el atributo @street en el objeto. El atributo @street
 * indica la calle de la direccin de envo.
 *
 */
void d_post_address_set_street (PostAddress * post_address,gchar * street){
    if (post_address->street!=NULL) {
	g_free(post_address->street);
    }
    post_address->street = g_strdup (street);
}

/**
 * d_post_address_set_number:
 * @post_address: Objeto en el que se establece el atributo.
 * @number: Atributo a insertar.
 *
 * Establece el atributo @number en el objeto. El atributo @number
 * indica el nmero de la direccin de envo.
 *
 */
void d_post_address_set_number (PostAddress * post_address,gchar * number){
    if (post_address->number!=NULL) {
	g_free(post_address->number);
    }
    post_address->number = g_strdup (number);
}

/**
 * d_post_address_set_floor:
 * @post_address: Objeto en el que se establece el atributo.
 * @floor: Atributo a insertar.
 *
 * Establece el atributo @floor en el objeto. El atributo @floor
 * indica el piso de la direccin de envo.
 *
 */
void d_post_address_set_floor (PostAddress * post_address,gchar * floor){
    if (post_address->floor!=NULL) {
	g_free(post_address->floor);
    }
    post_address->floor = g_strdup (floor);
}


/**
 * d_post_address_set_postal_code:
 * @post_address: Objeto en el que se establece el atributo.
 * @postal_code: Atributo a insertar.
 *
 * Establece el atributo @postal_code en el objeto. El atributo @postal_code
 * indica el cdigo postal.
 *
 */
void d_post_address_set_postal_code (PostAddress * post_address,gulong postal_code){
    post_address->postal_code = postal_code;
}

/**
 * d_post_address_set_town:
 * @post_address: Objeto en el que se establece el atributo.
 * @town: Atributo a insertar.
 *
 * Establece el atributo @town en el objeto. El atributo @town
 * indica el municipio al que pertenece la direccin de envo.
 *
 */
void d_post_address_set_town (PostAddress * post_address,gchar * town){
    if (post_address->town!=NULL) {
	g_free(post_address->town);
    }
    post_address->town = g_strdup (town);
}

/**
 * d_post_address_set_province:
 * @post_address: Objeto en el que se establece el atributo.
 * @province: Atributo a insertar.
 *
 * Establece el atributo @province en el objeto. El atributo @province
 * indica la provincia a la que pertenece el municipio.
 *
 */
void d_post_address_set_province (PostAddress * post_address,gchar * province){
    if (post_address->province!=NULL) {
	g_free(post_address->province);
    }
    post_address->province = g_strdup (province);
}

/**
 * d_post_address_get_code:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_post_address_get_code (PostAddress * post_address){
    return post_address->code;
}

/**
 * d_post_address_get_participant_code:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_post_address_get_participant_code (PostAddress * post_address){
    return post_address->participant_code;
}

/**
 * d_post_address_get_street:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_post_address_get_street (PostAddress * post_address){
    return g_strdup(post_address->street);
}

/**
 * d_post_address_get_number:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_post_address_get_number (PostAddress * post_address){
    return g_strdup(post_address->number);
}

/**
 * d_post_address_get_floor:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_post_address_get_floor (PostAddress * post_address){
    return g_strdup(post_address->floor);
}

/**
 * d_post_address_get_postal_code:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_post_address_get_postal_code (PostAddress * post_address){
    return post_address->postal_code;
}


/**
 * d_post_address_get_town:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_post_address_get_town (PostAddress * post_address){
    return g_strdup(post_address->town);
}

/**
 * d_post_address_get_province:
 * @post_address: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar * d_post_address_get_province (PostAddress * post_address){
    return g_strdup(post_address->province);
}



/**
 * d_post_address_duplicate:
 * @post_address: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
PostAddress *d_post_address_duplicate (PostAddress *post_address){

    PostAddress *result = d_post_address_create_empty ();
    gulong code = d_post_address_get_code (post_address);
    gulong participant_code = d_post_address_get_participant_code (post_address);
    gchar *street = d_post_address_get_street (post_address);
    gchar *number = d_post_address_get_number (post_address);
    gchar *floor = d_post_address_get_floor (post_address);
    gulong postal_code = d_post_address_get_postal_code (post_address);
    gchar *town = d_post_address_get_town (post_address);
    gchar *province = d_post_address_get_province (post_address);

    d_post_address_set_code (result, code);
    d_post_address_set_participant_code (result, participant_code);
    d_post_address_set_street (result, street);
    d_post_address_set_number (result, number);
    d_post_address_set_floor (result, floor);
    d_post_address_set_postal_code (result, postal_code);
    d_post_address_set_town (result, town);
    d_post_address_set_province (result, province);

    g_free(street);
    g_free(number);
    g_free(floor);
    g_free(town);
    g_free(province);

    return result;
}


/* Private functions*/

static gint post_address_init(PostAddress *post_address)
{
    post_address->code = ULONG_MAX;
    post_address->participant_code = ULONG_MAX;
    post_address->street = NULL;
    post_address->number = NULL;
    post_address->floor = NULL;
    post_address->postal_code = ULONG_MAX;
    post_address->town = NULL;
    post_address->province = NULL;

    return 0;
}



