
/* receipt.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "receipt.h"

/**
 * d_receipt_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Receipt vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Receipt *d_receipt_create_empty () {
    Receipt *receipt = (Receipt *) g_malloc (sizeof(Receipt));

    d_receipt_init (receipt);

    return receipt;
}


/**
 * d_receipt_clear:
 * @receipt: Recibo a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_receipt_clear (Receipt *receipt) {
    if (receipt!=NULL) {
	if (receipt->policy_number!=NULL)
	    g_free(receipt->policy_number);
	if (receipt->receipt_number!=NULL)
	    g_free(receipt->receipt_number);
	if (receipt->subsidiary_mediator!=NULL)
	    g_free(receipt->subsidiary_mediator);

	g_free(receipt);
    }
  return 0;
}

/**
 * d_receipt_set_policy_number:
 * @receipt: Objeto en el que se establece el atributo.
 * @policy_number: Atributo a insertar.
 *
 * Establece el atributo @policy_number en el objeto. El atributo @policy_number
 * indica el nmero de pliza contratado con una compaa de seguros.
 *
 */
void d_receipt_set_policy_number (Receipt * receipt, gchar *policy_number){
  if (receipt->policy_number != NULL) g_free(receipt -> policy_number);
  receipt->policy_number = g_strdup(policy_number);
}

/**
 * d_receipt_set_receipt_number:
 * @receipt: Objeto en el que se establece el atributo.
 * @receipt_number: Atributo a insertar.
 *
 * Establece el atributo @receipt_number en el objeto. El atributo @receipt_number
 * indica el nmero del recibo pagado a la compaa.
 *
 */
void d_receipt_set_receipt_number (Receipt * receipt, gchar *receipt_number){
  if (receipt->receipt_number != NULL) g_free(receipt -> receipt_number);
    receipt->receipt_number = g_strdup(receipt_number);
}

/**
 * d_receipt_set_company_code:
 * @receipt: Objeto en el que se establece el atributo.
 * @company_code: Atributo a insertar.
 *
 * Establece el atributo @company_code en el objeto. El atributo @company_code
 * indica cdigo que identifica a una compania de seguros.
 *
 */
void d_receipt_set_company_code (Receipt * receipt, gulong company_code){
    receipt->company_code = company_code;
}

/**
 * d_receipt_set_holder_code:
 * @receipt: Objeto en el que se establece el atributo.
 * @holder_code: Atributo a insertar.
 *
 * Establece el atributo @holder_code en el objeto. El atributo @holder_code
 * indica el cdigo del titular de la pliza de seguros.
 *
 */
void d_receipt_set_holder_code (Receipt * receipt, gulong holder_code){
    receipt->holder_code = holder_code;
}

/**
 * d_receipt_set_subsidiary_mediator:
 * @receipt: Objeto en el que se establece el atributo.
 * @subsidiary_mediator: Atributo a insertar.
 *
 * Establece el atributo @subsidiary_mediator en el objeto. El atributo @subsidiary_mediator
 * indica cual es la sucursal mediadora entre el titular y el seguro.
 *
 */
void d_receipt_set_subsidiary_mediator (Receipt * receipt, gchar *subsidiary_mediator){
  if (receipt->subsidiary_mediator != NULL) g_free(receipt -> subsidiary_mediator);
  receipt->subsidiary_mediator = g_strdup(subsidiary_mediator);
}

/**
 * d_receipt_set_validity_start:
 * @receipt: Objeto en el que se establece el atributo.
 * @validity_start: Atributo a insertar.
 *
 * Establece el atributo @validity_start en el objeto. El atributo @validity_start
 * indica el inicio de la validez de la pliza del seguro.
 *
 */
void d_receipt_set_validity_start (Receipt * receipt, time_t validity_start){
    receipt->validity_start = validity_start;
}

/**
 * d_receipt_set_validity_end:
 * @receipt: Objeto en el que se establece el atributo.
 * @validity_end: Atributo a insertar.
 *
 * Establece el atributo @validity_end en el objeto. El atributo @validity_end
 * indica el fin de la validez de la pliza del seguro.
 *
 */
void d_receipt_set_validity_end (Receipt * receipt, time_t validity_end){
    receipt->validity_end = validity_end;
}

/**
 * d_receipt_get_policy_number:
 * @receipt: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_receipt_get_policy_number (Receipt * receipt){
  return g_strdup(receipt->policy_number);
}

/**
 * d_receipt_get_receipt_number:
 * @receipt: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_receipt_get_receipt_number (Receipt * receipt){
  return    g_strdup(receipt->receipt_number);
}

/**
 * d_receipt_get_company_code:
 * @receipt: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_receipt_get_company_code (Receipt * receipt){
    return receipt->company_code;
}

/**
 * d_receipt_get_holder_code:
 * @receipt: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_receipt_get_holder_code (Receipt * receipt){
    return receipt->holder_code;
}

/**
 * d_receipt_get_subsidiary_mediator:
 * @receipt: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_receipt_get_subsidiary_mediator (Receipt * receipt){
  return g_strdup(receipt->subsidiary_mediator);
}

/**
 * d_receipt_get_validity_start:
 * @receipt: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_receipt_get_validity_start (Receipt * receipt){
    return receipt->validity_start;
}

/**
 * d_receipt_get_validity_end:
 * @receipt: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_receipt_get_validity_end (Receipt * receipt){
    return receipt->validity_end;
}


/**
 * d_receipt_init:
 * @receipt: Recibo al que se le inicializarn todos los atributos.
 *
 * Inicializa los atributos de un recibo a un valor predeterminado segn el tipo.
 *
 * Returns:
 * 0 si la funcin ha acabado correctamente.
 */
gint d_receipt_init (Receipt *receipt) {

    receipt->policy_number = NULL;
    receipt->receipt_number = NULL;
    receipt->company_code = ULONG_MAX;
    receipt->holder_code = ULONG_MAX;
    receipt->subsidiary_mediator = NULL;
    receipt->validity_end = time(NULL);
    receipt->validity_start = time(NULL);
    
    return 0;
}

/**
 * d_receipt_duplicate:
 * @receipt: Objeto del que se desea obtener el duplicado.
 *
 * Duplica un recibo.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Receipt *d_receipt_duplicate (Receipt *receipt){

  Receipt *result = d_receipt_create_empty ();
  gchar *numero_poliza = d_receipt_get_policy_number (receipt);    
  gchar *numero_recibo = d_receipt_get_receipt_number (receipt);
  gulong codigo_compania = d_receipt_get_company_code (receipt);
  gulong codigo_titular = d_receipt_get_holder_code (receipt);
  gchar *sucursal = d_receipt_get_subsidiary_mediator (receipt);
  time_t validity = d_receipt_get_validity_end (receipt);
  time_t ini_validity = d_receipt_get_validity_start (receipt);
  
  d_receipt_set_policy_number (result, numero_poliza);
  d_receipt_set_receipt_number (result, numero_recibo);
  d_receipt_set_company_code (result, codigo_compania);
  d_receipt_set_holder_code (result, codigo_titular);
  d_receipt_set_subsidiary_mediator (result, sucursal);
  d_receipt_set_validity_end (result, validity);
  d_receipt_set_validity_start (result, ini_validity);

  g_free(numero_poliza);
  g_free(numero_recibo);
  g_free(sucursal);
  
  return result;
  
}
