/* repair_object.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "repair_object.h"



static gint repair_init(Repair *repair);




/**
 * d_repair_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Repair vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Repair *d_repair_create_empty () {

    Repair *repair = (Repair *) g_malloc (sizeof(Repair));

    repair_init(repair);

    return repair;
}

/**
 * d_repair_clear:
 * @repair: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_repair_clear (Repair * repair) {

  gint index=0;
  if (repair!=NULL) {
      if (repair->observations!=NULL)
	  g_free(repair->observations);
      if (repair->mobile_unit_place!=NULL)
	  g_free(repair->mobile_unit_place);

      d_enterprise_clear (repair->enterprise);
      d_shop_clear (repair->shop);
      d_garage_clear (repair->garage);
      d_company_clear (repair->company);
      d_vehicle_clear (repair->vehicle);
      d_receipt_clear (repair->receipt);
      d_policy_clear (repair->policy);
      d_accident_clear (repair->accident);
      d_agent_clear (repair->agent);
      d_client_clear (repair->client);

      if (repair->repair_line!=NULL) {
      while (repair->repair_line[index]!=NULL)
	  d_repair_line_clear (repair->repair_line[index++]);
      }
      g_free(repair);
  }
  return 0;
}

/**
 * d_repair_set_repair_code:
 * @repair: Objeto en el que se establece el atributo.
 * @repair_code: Atributo a insertar.
 *
 * Establece el atributo @repair_code en el objeto. El atributo @repair_code
 * indica el cdigo que identifica de la reparacin.
 *
 */
void d_repair_set_repair_code (Repair *repair,gulong repair_code){
    repair->repair_code = repair_code;
}

/**
 * d_repair_set_in_date:
 * @repair: Objeto en el que se establece el atributo.
 * @in_date: Atributo a insertar.
 *
 * Establece el atributo @in_date en el objeto. El atributo @in_date
 * indica la fecha de entrada del vehculo a reparar.
 *
 */
void d_repair_set_in_date (Repair *repair,time_t in_date){
    repair->in_date = in_date;
}

/**
 * d_repair_set_out_date:
 * @repair: Objeto en el que se establece el atributo.
 * @out_date: Atributo a insertar.
 *
 * Establece el atributo @out_date en el objeto. El atributo @out_date
 * indica la fecha de salida del vehculo a reparar.
 *
 */
void d_repair_set_out_date (Repair *repair,time_t out_date){
    repair->out_date = out_date;
}

/**
 * d_repair_set_observations:
 * @repair: Objeto en el que se establece el atributo.
 * @observations: Atributo a insertar.
 *
 * Establece el atributo @observations en el objeto. El atributo @observations
 * indica las observaciones que se deseen aadir a la reparacin.
 *
 */
void d_repair_set_observations (Repair *repair,gchar *observations){
  if (repair->observations != NULL) g_free(repair -> observations);
  repair->observations = g_strdup(observations);
}

/**
 * d_repair_set_mobile_unit_place:
 * @repair: Objeto en el que se establece el atributo.
 * @mobile_unit_place: Atributo a insertar.
 *
 * Establece el atributo @mobile_unit_place en el objeto. El atributo @mobile_unit_place
 * indica el lugar de la unidad mvil que tuvo que acceder al siniestro.
 *
 */
void d_repair_set_mobile_unit_place (Repair *repair,gchar *mobile_unit_place){
  if (repair->mobile_unit_place != NULL) g_free(repair -> mobile_unit_place);
  repair->mobile_unit_place = g_strdup(mobile_unit_place);
}

/**
 * d_repair_set_mobile_unit_distance:
 * @repair: Objeto en el que se establece el atributo.
 * @mobile_unit_distance: Atributo a insertar.
 *
 * Establece el atributo @mobile_unit_distance en el objeto. El atributo @mobile_unit_distance
 * indica la distancia que tuvo que recorrer la unidad mvil.
 *
 */
void d_repair_set_mobile_unit_distance (Repair *repair,gulong mobile_unit_distance){
    repair->mobile_unit_distance = mobile_unit_distance;
}

/**
 * d_repair_set_enterprise:
 * @repair: Objeto en el que se establece el atributo.
 * @enterprise: Atributo a insertar.
 *
 * Establece el atributo @enterprise en el objeto. El atributo @enterprise
 * indica la empresa a la que pertenece la tienda en la que se realiza la reparacin.
 *
 */
void d_repair_set_enterprise (Repair *repair,Enterprise *enterprise){
  if (repair->enterprise != NULL) d_enterprise_clear (repair -> enterprise);
  repair->enterprise = enterprise;
}

/**
 * d_repair_set_shop:
 * @repair: Objeto en el que se establece el atributo.
 * @shop: Atributo a insertar.
 *
 * Establece el atributo @shop en el objeto. El atributo @shop
 * indica la tienda en la que se realiza la reparacin.
 *
 */
void d_repair_set_shop (Repair *repair,Shop *shop){
  if (repair->shop != NULL) d_shop_clear (repair -> shop);
  repair->shop = shop;
}

/**
 * d_repair_set_garage:
 * @repair: Objeto en el que se establece el atributo.
 * @garage: Atributo a insertar.
 *
 * Establece el atributo @garage en el objeto. El atributo @garage
 * indica el taller que envi la reparacin a una tienda del grupo.
 *
 */
void d_repair_set_garage (Repair *repair,Garage *garage){
  if (repair->garage != NULL) d_garage_clear (repair -> garage);
  repair->garage = garage;
}

/**
 * d_repair_set_company:
 * @repair: Objeto en el que se establece el atributo.
 * @company: Atributo a insertar.
 *
 * Establece el atributo @company en el objeto. El atributo @company
 * indica la compaa de seguros a la que dar parte de la reparacin.
 *
 */
void d_repair_set_company (Repair *repair,Company *company){
  if (repair->company != NULL) d_company_clear (repair -> company);
  repair->company = company;
}

/**
 * d_repair_set_receipt:
 * @repair: Objeto en el que se establece el atributo.
 * @receipt: Atributo a insertar.
 *
 * Establece el atributo @receipt en el objeto. El atributo @receipt
 * indica el ltimo recibo de la pliza del cliente.
 *
 */
void d_repair_set_receipt (Repair *repair,Receipt *receipt){
  if (repair->receipt != NULL) d_receipt_clear (repair -> receipt);
  repair->receipt = receipt;
}

/**
 * d_repair_set_policy:
 * @repair: Objeto en el que se establece el atributo.
 * @policy: Atributo a insertar.
 *
 * Establece el atributo @policy en el objeto. El atributo @policy
 * indica la pliza de seguros del cliente.
 *
 */
void d_repair_set_policy (Repair *repair,Policy *policy){
  if (repair->policy != NULL) d_policy_clear(repair -> policy);
  repair->policy = policy;
}

/**
 * d_repair_set_accident:
 * @repair: Objeto en el que se establece el atributo.
 * @accident: Atributo a insertar.
 *
 * Establece el atributo @accident en el objeto. El atributo @accident
 * indica el siniestro por el cul se llev el vehculo a reparar.
 *
 */
void d_repair_set_accident (Repair *repair,Accident *accident){
  if (repair->accident != NULL) d_accident_clear (repair -> accident);
  repair->accident = accident;
}
/**
 * d_repair_set_agent:
 * @repair: Objeto en el que se establece el atributo.
 * @agent: Atributo a insertar.
 *
 * Establece el atributo @agent en el objeto. El atributo @agent
 * indica el agente que acto de agente en la reparacin.
 *
 */
void d_repair_set_agent (Repair *repair,Agent *agent){
  if (repair->agent != NULL) d_agent_clear (repair -> agent);
  repair->agent = agent;
}

/**
 * d_repair_set_client:
 * @repair: Objeto en el que se establece el atributo.
 * @client: Atributo a insertar.
 *
 * Establece el atributo @client en el objeto. El atributo @client
 * indica el cliente de la reparacin.
 *
 */
void d_repair_set_client (Repair *repair,Client *client){
  if (repair->client != NULL) d_client_clear (repair -> client);
  repair->client = client;
}

/**
 * d_repair_set_vehicle:
 * @repair: Objeto en el que se establece el atributo.
 * @vehicle: Atributo a insertar.
 *
 * Establece el atributo @vehicle en el objeto. El atributo @vehicle
 * indica el vehculo a reparar.
 *
 */
void d_repair_set_vehicle (Repair *repair,Vehicle *vehicle) {
  if (repair->vehicle != NULL) d_vehicle_clear (repair -> vehicle);
  repair->vehicle = vehicle;
}

/**
 * d_repair_set_repair_lines:
 * @repair: Objeto en el que se establece el atributo.
 * @repair_line: Array de lneas de reparacin a insertar.
 *
 * Establece el atributo @repair_line en el objeto. El atributo @repair_line
 * indica el array de lneas de reparacin de una reparacin.
 *
 */
void d_repair_set_repair_lines (Repair *repair,Repair_line **repair_line){
  if (repair->repair_line != NULL){
    gint index = 0;
    while (repair->repair_line[index]!=NULL) {
	  d_repair_line_clear (repair->repair_line[index]);
	  index++;
    }        
    g_free(repair -> repair_line);
  }
  
  repair->repair_line = repair_line;
}

/**
 * d_repair_set_total:
 * @repair: Objeto en el que se establece el atributo.
 * @total: Atributo a insertar.
 *
 * Establece el atributo @total en el objeto. El atributo @total
 * indica el importe total de una reparacin.
 *
 */
void d_repair_set_total (Repair *repair,gdouble total){
    repair->total = total;
}

/**
 * d_repair_get_repair_code:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_get_repair_code (Repair *repair){
    return repair->repair_code;
}

/**
 * d_repair_get_in_date:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_repair_get_in_date (Repair *repair){
    return repair->in_date;
}

/**
 * d_repair_get_out_date:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
time_t d_repair_get_out_date (Repair *repair){
    return repair->out_date;
}

/**
 * d_repair_get_observations:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_repair_get_observations (Repair *repair){
    return g_strdup(repair->observations);
}

/**
 * d_repair_get_mobile_unit_place:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_repair_get_mobile_unit_place (Repair *repair){
    return g_strdup(repair->mobile_unit_place);
}

/**
 * d_repair_get_mobile_unit_distance:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_repair_get_mobile_unit_distance (Repair *repair){
    return repair->mobile_unit_distance;
}

/**
 * d_repair_get_enterprise:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Enterprise *d_repair_get_enterprise (Repair *repair){
    return repair->enterprise;
}

/**
 * d_repair_get_shop:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Shop *d_repair_get_shop (Repair *repair){
    return repair->shop;
}

/**
 * d_repair_get_garage:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Garage *d_repair_get_garage (Repair *repair){
    return repair->garage;
}

/**
 * d_repair_get_company:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Company *d_repair_get_company (Repair *repair){
    return repair->company;
}

/**
 * d_repair_get_receipt:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Receipt *d_repair_get_receipt (Repair *repair){
    return repair->receipt;
}

/**
 * d_repair_get_policy:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Policy *d_repair_get_policy (Repair *repair){
    return repair->policy;
}

/**
 * d_repair_get_accident:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Accident *d_repair_get_accident (Repair *repair){
    return repair->accident;
}

/**
 * d_repair_get_agent:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Agent *d_repair_get_agent (Repair *repair){
    return repair->agent;
}

/**
 * d_repair_get_client:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Client *d_repair_get_client (Repair *repair){
    return repair->client;
}

/**
 * d_repair_get_repair_lines:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Repair_line **d_repair_get_repair_lines (Repair *repair){
    return repair->repair_line;
}

/**
 * d_repair_get_vehicle:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Vehicle *d_repair_get_vehicle (Repair *repair) {
    return repair->vehicle;
}

/**
 * d_repair_get_total:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gdouble d_repair_get_total (Repair *repair){
    return repair->total;

}

/**
 * d_repair_is_with_insurance:
 * @repair: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gboolean d_repair_is_with_insurance (Repair * repair) {
    gboolean result = FALSE;
    Company * company = d_repair_get_company (repair);

    if (company!=NULL) {
	Participant * participant = d_company_get_participant(company);
	if (participant!=NULL) {
	    if (d_participant_get_code(participant)!=ULONG_MAX) {
		result=TRUE;
	    }
	}
    }
    return result;
}




/**
 * d_repair_unlink:
 * @repair: Objeto del que se quieren desreferenciar los puntaros sin liberarlos.
 *
 * Desreferencia los atributos de la reparacin.
 *
 */
void d_repair_unlink (Repair *repair) {

    repair -> enterprise = NULL;
    repair -> shop = NULL;
    repair -> garage = NULL;
    repair -> company = NULL;
    repair -> vehicle = NULL;
    repair -> receipt = NULL;
    repair -> policy = NULL;
    repair -> accident = NULL;
    repair -> agent = NULL;
    repair -> client = NULL;
    repair -> repair_line = NULL;

}

/* Private functions  */

static gint repair_init(Repair *repair) {

    repair -> repair_code = ULONG_MAX;
    repair -> in_date = time(NULL);
    repair -> out_date = time(NULL);
    repair -> observations = NULL;
    repair -> mobile_unit_distance = ULONG_MAX;
    repair -> total = G_MINDOUBLE;
    repair -> mobile_unit_place = NULL;
    repair -> enterprise = NULL;
    repair -> shop = NULL;
    repair -> garage = NULL;
    repair -> company = NULL;
    repair -> vehicle = NULL;
    repair -> receipt = NULL;
    repair -> policy = NULL;
    repair -> accident = NULL;
    repair -> agent = NULL;
    repair -> client = NULL;
    repair -> repair_line = NULL;
    repair -> completed = FALSE;

    return 0;
}
