/* supplier.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "supplier.h"

#include "participant.h"

static gint supplier_init(Supplier *supplier);
  

/**
 * d_supplier_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Supplier vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Supplier * d_supplier_create_empty () {

    Supplier * supplier;

    supplier = (Supplier *) g_malloc0(sizeof(Supplier));
    supplier_init(supplier);

    return supplier;
}


/**
 * d_supplier_clear:
 * @supplier: Objeto a liberar.
 *
 * Libera la memoria utilizada por el objeto.
 *
 * Returns:
 * 0 si se ha acabado la funcin correctamente.
 */
gint d_supplier_clear (Supplier *supplier) {
    if (supplier!=NULL) {
	d_participant_clear (d_supplier_get_participant (supplier));
	g_free(supplier);
        supplier = NULL;
    }
    return 0;
}

/**
 * d_supplier_set_participant:
 * @supplier: Objeto en el que se establece el atributo.
 * @participant: Atributo a insertar.
 *
 * Establece el atributo @participant en el objeto. El atributo @participant
 * indica el rol de participante del proveedor.
 *
 */
void d_supplier_set_participant (Supplier * supplier,Participant * participant) {
    supplier->participant = participant;
}


/**
 * d_supplier_get_participant:
 * @supplier: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
Participant * d_supplier_get_participant (Supplier * supplier){
    return supplier->participant;
}


/**
 * d_supplier_duplicate:
 * @supplier: Objeto del que se desea obtener el duplicado.
 *
 * Duplica el objeto proveedor.
 *
 * Returns: Un objeto con todos los campos duplicados y con nueva memoria reservada.
 */
Supplier * d_supplier_duplicate (Supplier * supplier) {
    Supplier * supplier_clone = d_supplier_create_empty ();
    if (supplier!=NULL) {
	d_supplier_set_participant (supplier_clone,d_participant_duplicate (d_supplier_get_participant (supplier)));
    }
    return supplier_clone;
}


/* Private functions */

static gint supplier_init(Supplier *supplier) {



    supplier->participant = NULL;

    return 0;
}

