 /* warehouse.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "warehouse.h"

static gint warehouse_init(Warehouse *warehouse);

/**
 * d_warehouse_create_empty:
 *
 * Crea un objeto vaco, con memoria reservada e inicializado.
 *
 * Returns:
 * Un objeto #Warehouse vaco con los campos inicializados a un valor de inicializacin segn el tipo.
 */
Warehouse *d_warehouse_create_empty () {
    Warehouse *warehouse = (Warehouse *) g_malloc(sizeof(Warehouse));

    warehouse_init(warehouse);

    return warehouse;
}


/**
 * d_warehouse_clear:
 * @warehouse: Objeto a liberar
 *
 * Libera la memoria utilizada por el objeto.
 *
 */
void d_warehouse_clear (Warehouse *warehouse) {
    if (warehouse!=NULL) {
	if (warehouse->nick!=NULL) {
	    g_free(warehouse->nick);
            warehouse->nick = NULL;
	}
	if (warehouse->phone!=NULL) {
	    g_free(warehouse->phone);
            warehouse->phone = NULL;
	}
	if (warehouse->fax!=NULL) {
	    g_free(warehouse->fax);
	    warehouse->fax = NULL;
	}
	if (warehouse->email!=NULL) {
	    g_free(warehouse->email);
	    warehouse->email = NULL;
	}
	if (warehouse->street!=NULL) {
	    g_free(warehouse->street);
	    warehouse->street = NULL;
	}
	if (warehouse->number!=NULL) {
	    g_free(warehouse->number);
	    warehouse->number = NULL;
	}
	if (warehouse->floor!=NULL) {
	    g_free(warehouse->floor);
	    warehouse->floor = NULL;
	}
	if (warehouse->town!=NULL) {
	    g_free(warehouse->town);
	    warehouse->town = NULL;
	}
	if (warehouse->province!=NULL) {
	    g_free(warehouse->province);
	    warehouse->province = NULL;
	}
	g_free(warehouse);
        warehouse = NULL;
    }
}

/**
 * d_warehouse_set_code:
 * @warehouse: Objeto en el que se establece el atributo.
 * @code: Atributo a insertar.
 *
 * Establece el atributo @code en el objeto. El atributo @code
 * indica el cdigo interno que identifica el almacn.
 *
 */
void d_warehouse_set_code (Warehouse *warehouse, gulong code) {
    warehouse->code = code;
}

/**
 * d_warehouse_set_enterprise_code:
 * @warehouse: Objeto en el que se establece el atributo.
 * @enterprise_code: Atributo a insertar.
 *
 * Establece el atributo @enterprise_code en el objeto. El atributo @enterprise_code
 * indica el cdigo de la empresa a la que pertenece el almacn.
 *
 */
void d_warehouse_set_enterprise_code (Warehouse *warehouse, gulong enterprise_code) {
    warehouse->enterprise_code = enterprise_code;
}

/**
 * d_warehouse_set_nick:
 * @warehouse: Objeto en el que se establece el atributo.
 * @nick: Atributo a insertar.
 *
 * Establece el atributo @nick en el objeto. El atributo @nick
 * indica el nombre completo del almacn.
 *
 */
void d_warehouse_set_nick (Warehouse *warehouse,gchar *nick) {
    if (warehouse->nick != NULL) {
	g_free(warehouse->nick);
    }
    warehouse->nick = g_strdup (nick);
}

/**
 * d_warehouse_set_phone:
 * @warehouse: Objeto en el que se establece el atributo.
 * @phone: Atributo a insertar.
 *
 * Establece el atributo @phone en el objeto. El atributo @phone
 * indica el nmero de telfono del almacn.
 *
 */
void d_warehouse_set_phone (Warehouse *warehouse, gchar *phone) {
    if (warehouse->phone != NULL) {
	g_free(warehouse->phone);
    }
    warehouse->phone = g_strdup (phone);
}

/**
 * d_warehouse_set_fax:
 * @warehouse: Objeto en el que se establece el atributo.
 * @fax: Atributo a insertar.
 *
 * Establece el atributo @fax en el objeto. El atributo @fax
 * indica el nmero de fax del almacn.
 *
 */
void d_warehouse_set_fax (Warehouse *warehouse,gchar *fax) {
    if (warehouse->fax != NULL) {
	g_free(warehouse->fax) ;
    }
    warehouse->fax = g_strdup (fax);
}

/**
 * d_warehouse_set_email:
 * @warehouse: Objeto en el que se establece el atributo.
 * @email: Atributo a insertar.
 *
 * Establece el atributo @email en el objeto. El atributo @email
 * indica la direccin de correo electrnico del almacn.
 *
 */
void d_warehouse_set_email (Warehouse *warehouse, gchar *email) {
    if (warehouse->email != NULL) {
	g_free(warehouse->email);
    }
    warehouse->email = g_strdup (email);
}

/**
 * d_warehouse_set_street:
 * @warehouse: Objeto en el que se establece el atributo.
 * @street: Atributo a insertar.
 *
 * Establece el atributo @street en el objeto. El atributo @street
 * indica la calle de la direccin del almacn.
 *
 */
void d_warehouse_set_street (Warehouse *warehouse,gchar *street) {
    if (warehouse->street != NULL) {
	g_free(warehouse->street);
    }
    warehouse->street = g_strdup (street);
}

/**
 * d_warehouse_set_number:
 * @warehouse: Objeto en el que se establece el atributo.
 * @number: Atributo a insertar.
 *
 * Establece el atributo @number en el objeto. El atributo @number
 * indica el nmero del edificio del almacn.
 *
 */
void d_warehouse_set_number (Warehouse *warehouse,gchar *number) {
    if (warehouse->number != NULL) {
	g_free(warehouse->number);
    }
    warehouse->number = g_strdup (number);
}

/**
 * d_warehouse_set_floor:
 * @warehouse: Objeto en el que se establece el atributo.
 * @floor: Atributo a insertar.
 *
 * Establece el atributo @floor en el objeto. El atributo @floor
 * indica el piso del edificio del almacn.
 *
 */
void d_warehouse_set_floor (Warehouse *warehouse,gchar *floor) {
    if (warehouse->floor !=NULL) {
	g_free(warehouse->floor);
    }
    warehouse->floor = g_strdup (floor);
}

/**
 * d_warehouse_set_postal_code:
 * @warehouse: Objeto en el que se establece el atributo.
 * @postal_code: Atributo a insertar.
 *
 * Establece el atributo @postal_code en el objeto. El atributo @postal_code
 * indica cdigo postal del almacn.
 *
 */
void d_warehouse_set_postal_code (Warehouse *warehouse,gulong postal_code) {
    warehouse->postal_code = postal_code;
}

/**
 * d_warehouse_set_town:
 * @warehouse: Objeto en el que se establece el atributo.
 * @town: Atributo a insertar.
 *
 * Establece el atributo @town en el objeto. El atributo @town
 * indica el municipio del almacn.
 *
 */
void d_warehouse_set_town (Warehouse *warehouse,gchar *town) {
    if (warehouse->town != NULL) {
	g_free(warehouse->town);
    }
    warehouse->town = g_strdup (town);
}

/**
 * d_warehouse_set_province:
 * @warehouse: Objeto en el que se establece el atributo.
 * @province: Atributo a insertar.
 *
 * Establece el atributo @province en el objeto. El atributo @province
 * indica la provincia a la que pertenece el almacn.
 *
 */
void d_warehouse_set_province (Warehouse *warehouse,gchar *province) {
    if (warehouse->province != NULL) {
	g_free(warehouse->province);
    }
    warehouse->province = g_strdup (province);
}

/**
 * d_warehouse_get_code:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_warehouse_get_code (Warehouse *warehouse) {
    return warehouse->code;
}

/**
 * d_warehouse_get_enterprise_code:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_warehouse_get_enterprise_code (Warehouse *warehouse) {
    return warehouse->enterprise_code;
}

/**
 * d_warehouse_get_nick:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_nick (Warehouse *warehouse) {
    return g_strdup(warehouse->nick);
}

/**
 * d_warehouse_get_phone:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_phone (Warehouse *warehouse) {
    return g_strdup(warehouse->phone);
}

/**
 * d_warehouse_get_fax:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_fax (Warehouse *warehouse) {
    return g_strdup(warehouse->fax);
}

/**
 * d_warehouse_get_email:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_email (Warehouse *warehouse) {
    return g_strdup(warehouse->email);
}

/**
 * d_warehouse_get_street:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_street (Warehouse *warehouse) {
    return g_strdup(warehouse->street);
}

/**
 * d_warehouse_get_number:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_number (Warehouse *warehouse) {
    return g_strdup(warehouse->number);
}

/**
 * d_warehouse_get_floor:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_floor (Warehouse *warehouse) {
    return g_strdup(warehouse->floor);
}

/**
 * d_warehouse_get_postal_code:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gulong d_warehouse_get_postal_code (Warehouse *warehouse) {
    return warehouse->postal_code;
}

/**
 * d_warehouse_get_town:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_town (Warehouse *warehouse) {
    return g_strdup(warehouse->town);
}

/**
 * d_warehouse_get_province:
 * @warehouse: Objeto del que se desea obtener el atributo.
 *
 * Devuelve el atributo solicitado del objeto pasado como parmetro.
 *
 * Returns: El valor del atributo solicitado.
 */
gchar *d_warehouse_get_province (Warehouse *warehouse) {
    return g_strdup(warehouse->province);
}

/* Private functions */

static gint warehouse_init(Warehouse *warehouse) {
    warehouse->code = ULONG_MAX;
    warehouse->enterprise_code = ULONG_MAX;
    warehouse->nick = NULL;
    warehouse->phone = NULL;
    warehouse->fax = NULL;
    warehouse->email = NULL;
    warehouse->street = NULL;
    warehouse->number = NULL;
    warehouse->floor = NULL;
    warehouse->postal_code = ULONG_MAX;
    warehouse->town = NULL;
    warehouse->province = NULL;

    return 0;
}

