/* delivery_payment_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "delivery_payment_model.h"

#define TABLE         "AlbaranAbono"
#define ATTRIBUTES    "codigoEmpresa as codigoEmpresaAlbaranAbono,\
    codigoTienda as codigoTiendaAlbaranAbono,\
    numeroAlbaran as numeroAlbaranAlbaranAbono,\
    numeroFactura as numeroFacturaAlbaranAbono,\
    fechaEmision as fechaEmisionAlbaranAbono,\
    codigoReparacion as codigoReparacionAlbaranAbono";

/**
 * m_delivery_payment_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_delivery_payment_get_table () {
    return TABLE;
}

/**
 * m_delivery_payment_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_delivery_payment_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_delivery_payment_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #DeliveryPayment con los campos rellenados con los datos de la base de datos.
 */
DeliveryPayment * m_delivery_payment_create (GdaRecordset *rs) {

    GdaField * field;
    DeliveryPayment * delivery_payment = d_delivery_payment_create_empty();

//    delivery_payment = (DeliveryPayment *) g_malloc0(sizeof(DeliveryPayment));

    field = gda_recordset_field_name(rs,"numeroAlbaranAlbaranAbono");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {

	    delivery_payment->delivery_payment_number = igalia_field_get_bigint_value (field);

	    /*        field = gda_recordset_field_name(rs,"codigoReceptorAlbaranAbono");
	     if ((field!=NULL) && (!gda_field_is_null(field))) {
	     delivery_payment->codigoReceptor = igalia_field_get_bigint_value (field);
	     } else {
	     delivery_payment->codigoReceptor = -1;
	     }
	     */
	    gda_field_free(field);


	    field = gda_recordset_field_name(rs,"numeroFacturaAlbaranAbono");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    delivery_payment->invoice_code = igalia_field_get_bigint_value (field);
		} else {
		    delivery_payment->invoice_code = -1;
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "totalTallerAlbaranAbono");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp=gda_stringify_value(NULL, 0, field);
		    delivery_payment->garage_total = atof(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"fechaEmisionAlbaranAbono");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    delivery_payment->issue_date = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	    delivery_payment->repair=m_repair_create (rs,NULL);
	} else {
	    gda_field_free(field);
	}
    } else {
//	delivery_payment_init(delivery_payment);
    }

    return delivery_payment;
}


/**
 * m_delivery_payment_find_by_condition :
 * @condition: Filtro de bsqueda. Permite buscar por: cdigo de la empresa emisora, cdigo de la tienda
 * emisora, cif/nif del cliente, nombre del cliente, perodo de fechas de emisin y por albaranes con o sin seguro.
 *
 * Busca albaranes de abono segn los filtros de la estructura
 * de tipo ConditionDeliveryPaymentClients. Esta estructura define unos filtros de bsqueda, y se
 * incluye la posibilidad de buscar los albaranes de abono que tienen como taller de la reparacin un
 * taller en particular para despus poder facturar y devolver la comisin del taller. La busqueda de
 * albaranes de abono es muy distinta en las tiendas que en la central. La bsqueda de albaranes de abono
 * con taller solo se puede realizar en la central.
 *
 * Returns:
 * Lista de albaranes de abono que cumplen la condicin de bsqueda.
 */
DeliveryPayment ** m_delivery_payment_find_by_condition (ConditionDeliveryPaymentClients * condition) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
#ifndef IS_SHOP
    gchar              *gchar_select_garage=NULL;
#endif
    gchar              *gchar_where=NULL;
#ifndef IS_SHOP
    gchar              *gchar_where_garage=NULL;
#endif
    gchar              *gchar_where_aux=NULL;
#ifndef IS_SHOP
    gchar              *gchar_where_aux_garage=NULL;
#endif
    gchar              *gchar_from_aux=NULL;
    gchar              *gchar_select_aux=NULL;
    gchar              *gchar_from=NULL;
#ifndef IS_SHOP
    gchar              *gchar_from_garage=NULL;
    gchar              *gchar_union=NULL;
#endif
    gchar              *gchar_order=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    DeliveryPayment ** delivery_payments;
    DeliveryPayment * delivery_payment;
    gint type_of_client=-1;
    gint index;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    if (condition->participant_code!=NULL) {
        type_of_client = condition->client_type;
    }

#ifndef IS_SHOP
    if (type_of_client==M_CLIENT_GARAGE_CODE) {
        gchar * from_garage_aux;
        gchar * where_garage_aux;
	gchar_from_garage = g_strconcat(" from albaranabono left outer join (select facturaTaller.numeroAlbaran from facturataller,albaranabono a ",
					" where facturaTaller.numeroAlbaran=a.numeroalbaran ",NULL);

	if (condition->enterprise_code!=NULL) {
	    from_garage_aux = g_strconcat(gchar_from_garage," and facturaTaller.codigoEmpresaalbaran=",condition->enterprise_code,NULL);
	    g_free(gchar_from_garage);
            gchar_from_garage = from_garage_aux;
	}
	if (condition->shop_code!=NULL) {
	    from_garage_aux = g_strconcat(gchar_from_garage," and facturaTaller.codigoTiendaalbaran=",condition->shop_code,NULL);
	    g_free(gchar_from_garage);
            gchar_from_garage = from_garage_aux;
	}

	from_garage_aux = g_strconcat(gchar_from_garage,
				      ") as facturasTaller on (albaranabono.numeroAlbaran=facturasTaller.numeroAlbaran), reparacion left outer ",
				      " join vehiculo on (vehiculo.codigoVehiculo=reparacion.codigoVehiculo) left",
				      " outer join participante participante2 on (participante2.codigoParty=reparacion.codigoCompania),participante participanteCliente, ",
				      " participante participante3 ",
					NULL);
	g_free(gchar_from_garage);
	gchar_from_garage = from_garage_aux;


	gchar_where_garage= g_strconcat(
					" where albaranabono.codigoReparacion=reparacion.codigoReparacion ",
					" and albaranabono.codigoTienda=participante3.codigoParty  and ",
					" reparacion.codigotaller=participanteCliente.codigoParty  and ",
					" albaranabono.codigoEmpresa=reparacion.codigoEmpresa  and ",
					" albaranabono.codigoTienda=reparacion.codigoTienda and albaranabono.totalTaller>0",NULL);

	if (condition->enterprise_code!=NULL) {
            where_garage_aux = g_strconcat(gchar_where_garage,"and (albaranabono.codigoEmpresa=",condition->enterprise_code,")",NULL);
	    g_free(gchar_where_garage);
            gchar_where_garage = where_garage_aux;
	}
	if (condition->shop_code!=NULL) {
	    where_garage_aux = g_strconcat(gchar_where_garage," and (albaranabono.codigoTienda=",condition->shop_code,")",NULL);
	    g_free(gchar_where_garage);
            gchar_where_garage = where_garage_aux;
	}

	where_garage_aux = g_strconcat(gchar_where_garage," and (participanteCliente.codigoParty=",condition->participant_code,") and ",
				       " facturasTaller.numeroAlbaran is NULL ",NULL);
	g_free(gchar_where_garage);
	gchar_where_garage = where_garage_aux;

        gchar_union = g_strconcat(" union ",NULL);
    } 
#endif


    gchar_select = g_strconcat("(SELECT ",
                               "albaranabono.codigoEmpresa as codigoPartyParticipantEnter,",
			       "albaranabono.codigoTienda as codigoTiendaAlbaranAbono,",
			       "albaranabono.numeroAlbaran as numeroAlbaranAlbaranAbono,",
			       "albaranabono.codigoReparacion as codigoReparacionAlbaranAbono,",
			       "albaranabono.fechaEmision as fechaEmisionAlbaranAbono,",
			       "to_char(albaranabono.totalTaller,'99999999D99') as totalTallerAlbaranAbono,",
			       "participanteCliente.codigoParty as codigoPartyparticipantclient,",
			       "participanteCliente.nick as nickparticipantclient,",
			       "participanteCliente.nombreCorto as shortNameparticipantclient,",
                               "Participante3.codigoParty as codigoPartyparticipantshop,",
			       "Participante3.nick as nickparticipantshop,",
			       "Vehiculo.codigoVehiculo as codigoVehiculovehicle,",
			       "reparacion.matricula as matriculavehicle,",
			       "reparacion.codigoTaller as codigoPartyParticipantGarage,",
			       "reparacion.observaciones as observacionesrepair,"
			       "to_char(reparacion.importeTotal,'99999999D99') as importetotalRepair, ",
			       "reparacion.codigoReparacion as codigoReparacionRepair",
			       NULL );
#ifndef IS_SHOP
    gchar_select_garage = g_strconcat("(SELECT ",
                               "albaranabono.codigoEmpresa as codigoPartyParticipantEnter,",
			       "albaranabono.codigoTienda as codigoTiendaAlbaranAbono,",
			       "albaranabono.numeroAlbaran as numeroAlbaranAlbaranAbono,",
			       "albaranabono.codigoReparacion as codigoReparacionAlbaranAbono,",
			       "albaranabono.fechaEmision as fechaEmisionAlbaranAbono,",
			       "to_char(albaranabono.totalTaller,'99999999D99') as totalTallerAlbaranAbono,",
       			       "participanteCliente.codigoParty as codigoPartyparticipantclient,",
			       "participanteCliente.nick as nickparticipantclient,",
			       "participanteCliente.nombreCorto as shortNameparticipantclient,",
                               "Participante3.codigoParty as codigoPartyparticipantshop,",
			       "Participante3.nick as nickparticipantshop,",
			       "Vehiculo.codigoVehiculo as codigoVehiculovehicle,",
			       "reparacion.matricula as matriculavehicle,",
			       "reparacion.codigoTaller as codigoPartyParticipantGarage,",
			       "reparacion.observaciones as observacionesrepair,"
			       "to_char(reparacion.importeTotal,'99999999D99') as importetotalRepair, ",
			       "reparacion.codigoReparacion as codigoReparacionRepair,",
			       "Participante2.codigoParty as codigoPartyparticipantcompany,",
			       "Participante2.nick as nickparticipantcompany, ",
			       "Participante2.nombrecorto as shortnameparticipantcompany ",
			       NULL );
#endif

    gchar_from = g_strconcat(" from albaranabono,reparacion left outer join vehiculo on (vehiculo.codigoVehiculo=reparacion.codigoVehiculo) left outer ",
			     " join compania on compania.codigoPArty=reparacion.codigoCOmpania left outer join nota on (reparacion.codigoReparacion=nota.codigoReparacion ",
                             " and reparacion.codigoTienda=nota.codigoTienda and reparacion.codigoEmpresa=nota.codigoEmpresa) ",
			     ",cliente,",
			     "participante participanteCliente,",
                             "participante participante3",NULL);


    gchar_where=g_strconcat(" where ","(albaranabono.numeroFactura is null)",
			    " and albaranabono.codigoReparacion=reparacion.codigoReparacion ",
			    " and albaranabono.codigoTienda=participante3.codigoParty ",
			    " and cliente.codigoParty = participanteCliente.codigoParty",
			    " and reparacion.codigoCliente=participanteCliente.codigoParty ",
			    " and albaranabono.codigoEmpresa=reparacion.codigoEmpresa ",
			    " and albaranabono.codigoTienda=reparacion.codigoTienda and nota.codigoReparacion is null ",NULL);

    if (condition->insurance==TRUE) {
        gchar_select_aux=g_strconcat(gchar_select,",Participante2.codigoParty as codigoPartyparticipantcompany,",
			       "Participante2.nick as nickparticipantcompany",NULL);
	g_free(gchar_select);
	gchar_select = gchar_select_aux;
	gchar_select_aux=NULL;

	gchar_from_aux=g_strconcat(gchar_from,",participante participante2 ",NULL);
	g_free(gchar_from);
	gchar_from = gchar_from_aux;
	gchar_from_aux=NULL;

	gchar_where_aux=g_strconcat(gchar_where," and Participante2.codigoParty=compania.codigoParty"
				," and reparacion.codigoCompania=participante2.codigoParty ",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;

    } else {
        gchar_select_aux=g_strconcat(gchar_select,",NULL as codigoPartyParticipantCompany,NULL as nickparticipantcompany,",
			      "Participante3.nombrecorto as shortnameparticipantcompany ",NULL);
	g_free(gchar_select);
	gchar_select = gchar_select_aux;
	gchar_select_aux=NULL;

        gchar_where_aux=g_strconcat(gchar_where," and (reparacion.codigoCompania is null)",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    if (((condition->enterprise_code) != NULL) && (strlen(condition->enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoEmpresa=",
		     condition->enterprise_code,")",NULL);
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (albaranabono.codigoEmpresa=",
						 condition->enterprise_code,")",NULL);
	}
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    g_free(gchar_where_garage);
	    gchar_where_garage = gchar_where_aux_garage;
	    gchar_where_aux_garage=NULL;
	}
#endif
    }

    if (((condition->shop_code) != NULL) && (strlen(condition->shop_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoTienda=",
				      condition->shop_code,")","and (reparacion.codigoTienda=",condition->shop_code,
				      ") and (participante3.codigoParty=",condition->shop_code,")",NULL);
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (albaranabono.codigoTienda=",
						 condition->shop_code,")",NULL);
	}
#endif
    
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
	
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    g_free(gchar_where_garage);
	    gchar_where_garage = gchar_where_aux_garage;
	    gchar_where_aux_garage=NULL;
	}
#endif
    }


    if (((condition->initial_date) != NULL) && (strlen(condition->initial_date)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.fechaEmision>='",
		     condition->initial_date,"')",NULL);
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (albaranabono.fechaEmision>='",
						 condition->initial_date,"')",NULL);
	}
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    g_free(gchar_where_garage);
	    gchar_where_garage = gchar_where_aux_garage;
	    gchar_where_aux_garage=NULL;
	}
#endif
    }

    if (((condition->ending_date) != NULL) && (strlen(condition->ending_date)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.fechaEmision<='",
		     condition->ending_date,"')",NULL);
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (albaranabono.fechaEmision<='",
						 condition->ending_date,"')",NULL);
	}
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    g_free(gchar_where_garage);
	    gchar_where_garage = gchar_where_aux_garage;
	    gchar_where_aux_garage=NULL;
	}
#endif
    }


    if (((condition->participant_code) != NULL) && (strlen(condition->participant_code)!=0)) {
    	gchar_where_aux = g_strconcat(gchar_where, " and (participanteCliente.codigoParty=",
				  condition->participant_code,") and (reparacion.codigoCliente=",condition->participant_code,") ",NULL);
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    gchar_where_aux_garage = g_strconcat(gchar_where_garage, " and (participanteCliente.codigoParty=",
						 condition->participant_code,")"," and (reparacion.codigoTaller=",condition->participant_code,") ",NULL);
	}
#endif
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
#ifndef IS_SHOP
	if (type_of_client==M_CLIENT_GARAGE_CODE) {
	    g_free(gchar_where_garage);
	    gchar_where_garage = gchar_where_aux_garage;
	    gchar_where_aux_garage=NULL;
	}
#endif

    }

    gchar_order = g_strconcat(" ORDER BY ALBARANABONO.NUMEROALBARAN) ",NULL);

#ifndef IS_SHOP
    if (type_of_client==M_CLIENT_GARAGE_CODE) {
	gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order,gchar_union,
				  gchar_select_garage,gchar_from_garage,gchar_where_garage,gchar_order,NULL);

    } else {
	gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order, NULL);
    }
#else
    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order, NULL);
#endif
    g_print(_("\nFind delivery payment: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {
	    delivery_payments = (DeliveryPayment **) g_malloc0(sizeof(DeliveryPayment*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		delivery_payment = m_delivery_payment_create (record_set);
		delivery_payments[index-1] = delivery_payment;
		index = gda_recordset_move_next(record_set);
	    }
	    delivery_payments[*tuples]=NULL;
	} else {
	    delivery_payments=NULL;
	}
	gda_recordset_close(record_set);
    } else {
	delivery_payments=NULL;
    }

    igalia_command_free(my_command);
    gda_recordset_free(record_set);


#ifndef IS_SHOP
    if (gchar_where_garage!=NULL)
	g_free(gchar_where_garage);
    if (gchar_from_garage!=NULL)
	g_free(gchar_from_garage);
    if (gchar_select_garage!=NULL)
	g_free(gchar_select_garage);
#endif
    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_order);
    g_free(gchar_from);
    g_free(gchar_query);
    g_free(tuples);

    return delivery_payments;

}

/**
 * m_delivery_payment_find_by_repair:
 * @condition: Condicin que identifica la reparacin.
 * 
 * Busca los datos del albarn de abono que contiene referencia a la reparacin que nos interesa.
 * Esta funcin solo devuelve los datos concretos del albarn de abono.
 *
 * Returns:
 * Albarn que contiene como clave fornea la reparacin en cuestin.
 */
DeliveryPayment * m_delivery_payment_find_by_repair (RepairId *condition){
    GdaCommand         *my_command;
    gchar              *gchar_query;
    gchar              *gchar_select;
    gchar              *gchar_where;
    gchar              *gchar_where_aux;
    gchar              *gchar_from;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    DeliveryPayment * delivery_payment=NULL;
    gint index;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_select = g_strconcat("SELECT ",
                               "albaranabono.codigoEmpresa as codigoPartyParticipantEnter,",
			       "albaranabono.codigoTienda as codigoTiendaAlbaranAbono,",
			       "albaranabono.numeroAlbaran as numeroAlbaranAlbaranAbono,",
			       "albaranabono.numeroFactura as numeroFacturaAlbaranAbono,",
			       "albaranabono.codigoReparacion as codigoReparacionAlbaranAbono,",
			       "albaranabono.fechaEmision as fechaEmisionAlbaranAbono,",
			       "to_char(albaranabono.totalTaller,'99999999D99') as totalTallerAlbaranAbono",
			       NULL );
    gchar_from = g_strconcat(" from albaranabono",
                             NULL);


    gchar_where=g_strconcat(" where ",NULL);
    if (((condition->repair_code) != NULL) && (strlen(condition->repair_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " (albaranabono.codigoReparacion=",
		     condition->repair_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    if (((condition->enterprise_code) != NULL) && (strlen(condition->enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoEmpresa=",
		     condition->enterprise_code,") ",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    if (((condition->shop_code) != NULL) && (strlen(condition->shop_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoTienda=",
		     condition->shop_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

                               
    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);


    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0,TRUE);


    if (record_set!=NULL) {
	if (*tuples==1) {
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		delivery_payment = m_delivery_payment_create (record_set);
		index = gda_recordset_move_next(record_set);
	    } 
	    gda_recordset_close(record_set);
	} else {
	    delivery_payment=NULL;
	}
    } else {
	    delivery_payment=NULL;
    }


    igalia_command_free(my_command);
    gda_recordset_free(record_set);


    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_from);
    g_free(tuples);

    return delivery_payment;

}

/**
 * m_delivery_payment_find_by_id:
 * @condition: Identificador del albarn de abono.
 * 
 * Funcin que permite buscar los datos de albarn de abono con identificado @condition.
 * Adems devuelve la reparacin a la que referencia y sus lneas de reparacin.
 *
 * Returns:
 * Albarn de abono que es identificado por @condition.
 */
DeliveryPayment * m_delivery_payment_find_by_id (DeliveryPaymentId * condition){
    GdaCommand         *my_command;
    gchar              *gchar_query;
    gchar              *gchar_select;
    gchar              *gchar_where;
    gchar              *gchar_where_aux;
    gchar              *gchar_from;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    DeliveryPayment * delivery_payment=NULL;
    gint index;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_select = g_strconcat("SELECT ",
                               "albaranabono.codigoEmpresa as codigoPartyParticipantEnter,",
			       "albaranabono.codigoTienda as codigoTiendaAlbaranAbono,",
			       "albaranabono.numeroAlbaran as numeroAlbaranAlbaranAbono,",
			       "albaranabono.numeroFactura as numeroFacturaAlbaranAbono,",
			       "albaranabono.codigoReparacion as codigoReparacionAlbaranAbono,",
			       "albaranabono.fechaEmision as fechaEmisionAlbaranAbono,",
			       "to_char(albaranabono.totalTaller,'99999999D99') as totalTallerAlbaranAbono",
			       NULL );
    gchar_from = g_strconcat(" from albaranabono",
                             NULL);


    gchar_where=g_strconcat(" where ",NULL);
    if (((condition->delivery_payment_number) != NULL) && (strlen(condition->delivery_payment_number)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " (albaranabono.numeroAlbaran=",
		     condition->delivery_payment_number,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    if (((condition->enterprise_code) != NULL) && (strlen(condition->enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoEmpresa=",
		     condition->enterprise_code,") ",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

    if (((condition->shop_code) != NULL) && (strlen(condition->shop_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (albaranabono.codigoTienda=",
		     condition->shop_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }

                               
    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);


    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,my_command,tuples,0,TRUE);


    if (record_set!=NULL) {
	if (*tuples==1) {
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		Repair * repair=NULL;
		Repair_line ** repair_lines=NULL;
		RepairId * repair_id=(RepairId *)g_malloc0(sizeof(RepairId));
                GdaField * field;
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		field = gda_recordset_field_name(record_set,"codigoReparacionAlbaranAbono");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			repair_id->repair_code=g_strdup_printf("%lu",(gulong) igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		repair_id -> enterprise_code = g_strdup(condition->enterprise_code);
		repair_id -> shop_code = g_strdup(condition->shop_code);

		repair_lines =  m_repair_line_find_by_id (repair_id);

		repair = m_repair_find_by_id (repair_id,repair_lines);

		delivery_payment = m_delivery_payment_create (record_set);
		if (repair!=NULL) {
		    d_repair_clear (d_delivery_payment_get_repair (delivery_payment));
		    d_delivery_payment_set_repair (delivery_payment,repair);
		}

		index = gda_recordset_move_next(record_set);
	    } 
	    gda_recordset_close(record_set);
	} else {
	    delivery_payment=NULL;
	}
    } else {
	    delivery_payment=NULL;
    }


    igalia_command_free(my_command);
    gda_recordset_free(record_set);


    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_from);
    g_free(tuples);

    return delivery_payment;

}


/**
 * m_delivery_payment_insert:
 * @delivery_payment: Albarn de abono a insertar.
 * 
 * Inserta un albarn de abono en la base de datos. Previamente a la insercin obtiene el
 * nmero de albarn de abono de una tabla de la base de datos y lo aumenta en 1. Despus este
 * nmero lo devuelve como resultado de la funcin.
 *
 * Returns:
 * Nmero del albarn de abono, %ULONG_MAX si no se ha podido insertar.
 */
gulong m_delivery_payment_insert (DeliveryPayment * delivery_payment) {
    gulong result=0;
    gchar * query1=NULL;
    gchar * query2;
    gchar * query3;
    gchar * query4;
    gchar * query;
    gchar * query_aux;
    gchar * values=NULL;
    gchar * values_aux;
    gchar * delivery_payment_number=NULL;
    gchar * temp_delivery_payment=NULL;
    gulong shop_code=ULONG_MAX;
    gulong repair_code=ULONG_MAX;
    gulong enterprise_code=ULONG_MAX;
    gulong invoice_number=ULONG_MAX;
    gboolean exists=FALSE;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;
    GdaRecordset *record_set3=NULL;
    GdaRecordset *record_set4=NULL;
    GdaConnection * connection = m_model_get_connection ();
    Repair * repair;
    GdaCommand * command = gda_command_new();
    time_t issue_date=time(NULL);



    repair = d_delivery_payment_get_repair (delivery_payment);
    shop_code=d_participant_get_code (d_shop_get_participant (d_repair_get_shop (
									     repair
									    )));
    enterprise_code=d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (
											  repair
											 )));

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code,shop_code,ULONG_MAX);


    repair_code=d_repair_get_repair_code (repair);
    invoice_number=d_delivery_payment_get_invoice_code (delivery_payment);


    if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX) && (repair_code!=ULONG_MAX)) {
	gchar * aux1=NULL;
	gchar * aux2=NULL;
	gchar * aux3=NULL;
        aux1= g_strdup_printf("%lu",repair_code);
        aux2= g_strdup_printf("%lu",shop_code);
        aux3= g_strdup_printf("%lu",enterprise_code);
	query4 = g_strconcat("select numeroAlbaran from albaranabono where ",
                             " codigoReparacion=",aux1,
                             " and codigoTienda=",aux2,
			     " and codigoEmpresa=",aux3,
			     NULL);
	if (m_model_command_execute (&record_set4, query4, connection,command,tuples,0,TRUE)==FALSE) {
	    exists=TRUE;
            result=ULONG_MAX;
	}
	if (record_set4!=NULL) {
	    if (*tuples==1) {
		exists=TRUE;
	    } else if (*tuples>1) {
		exists=TRUE;
                result=ULONG_MAX;
	    } else {
                exists=FALSE;
	    }
	} else {
	    exists=TRUE;
	    result=ULONG_MAX;
	}
	if (query4!=NULL) g_free(query4);
        if (aux1!=NULL) g_free(aux1);
        if (aux2!=NULL) g_free(aux2);
	if (aux3!=NULL) g_free(aux3);
	if (record_set4!=NULL)
	    gda_recordset_free(record_set4);

    }
    if (exists==FALSE) {
	gchar * enterprise = g_strdup_printf("%lu",enterprise_code);
        gchar * shop = g_strdup_printf("%lu",shop_code);
	query1 = g_strconcat(" select numeroAlbaran from numeroscorrelativosAlbaran where codigoEmpresa=",enterprise,
			     " and codigoTienda=",shop,NULL);
	if (m_model_command_execute (&record_set1, query1, connection,command,tuples,0,TRUE)==FALSE) {
	    result = ULONG_MAX;
	}

	if (record_set1!=NULL) {
	    if (*tuples!=0) {
		GdaField * field;
		gda_recordset_move_first(record_set1);
		field  = gda_recordset_field_idx(record_set1,0);
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
                        gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
			delivery_payment_number = gda_stringify_value(NULL, 0, field);
			result = strtoul(delivery_payment_number,stop,10);
                        g_free(stop);
		    }
		    gda_field_free(field);
		}

	    } else {
		result = ULONG_MAX;
	    }
	} else {
	    result = ULONG_MAX;
	}


	g_free(delivery_payment_number);

	if (result != ULONG_MAX) {
	    result = result +1;
	}
        delivery_payment_number = g_strdup_printf("%lu",result);
	query2 = g_strconcat(" update numerosCorrelativosAlbaran set numeroAlbaran=",delivery_payment_number,"where codigoEmpresa="
			     ,enterprise," and codigoTienda=",shop,NULL);
        g_free(enterprise);
	g_free(shop);
	enterprise=NULL;
        shop=NULL;
	if (m_model_command_execute (&record_set2, query2, connection,command,tuples,0,FALSE)==FALSE) {
	    result = ULONG_MAX;
	}



	values = g_strconcat(" ",NULL);
	query3 = g_strconcat("INSERT INTO ALBARANABONO VALUES (",NULL);

	temp_delivery_payment = g_strdup_printf("%lu",enterprise_code);
	values_aux = g_strconcat(values,temp_delivery_payment,",",NULL);
	g_free(values);
	g_free(temp_delivery_payment);
	values = values_aux;

	temp_delivery_payment = g_strdup_printf("%lu",shop_code);
	values_aux = g_strconcat(values,temp_delivery_payment,",",NULL);
	g_free(values);
	g_free(temp_delivery_payment);
	values = values_aux;

	values_aux = g_strconcat(values,delivery_payment_number,",",NULL);
	g_free(values);
	values = values_aux;

	if (invoice_number!=ULONG_MAX) {
	    temp_delivery_payment = g_strdup_printf("%lu",invoice_number);
	    values_aux = g_strconcat(values,temp_delivery_payment,",",NULL);
	    g_free(values);
	    g_free(temp_delivery_payment);
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;

	}


	if (repair_code!=ULONG_MAX) {
	    temp_delivery_payment = g_strdup_printf("%lu",repair_code);
	    values_aux = g_strconcat(values,temp_delivery_payment,",",NULL);
	    g_free(values);
	    g_free(temp_delivery_payment);
	    values = values_aux;
	} else {
	    values_aux = g_strconcat(values,"NULL,",NULL);
	    g_free(values);
	    values = values_aux;

	}

	values_aux = g_strconcat(values,"'",ctime(&issue_date),"'",NULL);
	g_free(values);
	values = values_aux;

	query_aux=g_strconcat(query3,values,")",NULL);
	g_free(query3);
	query3=query_aux;
	g_free(values);
	query_aux=NULL;


	if (m_model_command_execute (&record_set3, query3, connection,command,tuples,0,FALSE)==FALSE) {
	    result = ULONG_MAX;
	}
	query = g_strconcat (query1,";\n",query2,";\n",query3,";\n",NULL);

        g_free(query1);
        g_free(query2);
	g_free(query3);
	g_free(query);
	g_free(tuples);
	g_free(delivery_payment_number);



	if (record_set1!=NULL)
	    gda_recordset_free(record_set1);
	if (record_set2!=NULL)
	    gda_recordset_free(record_set2);
	if (record_set3!=NULL)
	    gda_recordset_free(record_set3);
    }
    if (m_model_execute_mutex_modification_logged (connection)!=TRUE) {
	result = ULONG_MAX;
    }

    igalia_command_free(command);

    return result;
}



/**
 * m_delivery_payment_clear_array:
 * @delivery_payments: Array de albaranes de abono a liberar.
 * 
 * Libera la memoria reservada por un array de albaranes de abono.
 *
 */
void m_delivery_payment_clear_array (DeliveryPayment ** delivery_payments) {
    if (delivery_payments!=NULL) {
	gint index = 0;
	while (delivery_payments[index]!=NULL) {
	    d_delivery_payment_clear (delivery_payments[index]);
	    index++;
	}
    }
    g_free(delivery_payments);

}

/**
 * m_delivery_payment_is_from_garage:
 * @garage_code: Cdigo del taller que se desea comprobar.
 * @client_code: Cdigo del cliente de la reparacin
 * @delivery_payment: Albarn de abono que se desea comprobar.
 * @connection: Conexin a la base de datos.
 * @command: Comando en la que se ejecutar la sentencia sql.
 *
 * Comprueba si un albarn de abono es de una reparacin con un
 * taller asociado en particular dentro de una transaccin.
 *
 * Returns:
 * %TRUE si la reparacin asociada al albarn tiene asociada el taller identificado por @garage_code.
 */
gboolean  m_delivery_payment_is_from_garage (gulong garage_code,gulong client_code,DeliveryPayment * delivery_payment,GdaConnection *connection, GdaCommand *command) {
    gchar * query;
    gchar * garage_code_aux=g_strdup_printf("%lu",garage_code);
    gchar * enterprise_code=g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment)))));
    gchar * shop_code=g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (d_repair_get_shop (d_delivery_payment_get_repair (delivery_payment)))));
    gchar * delivery_payment_number=g_strdup_printf("%lu",d_delivery_payment_get_number (delivery_payment));
    gchar * client_code_char=g_strdup_printf("%lu",client_code);
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gboolean result=FALSE;

    if (garage_code!=client_code) {
	query = g_strconcat("select numeroalbaran from albaranabono, reparacion, participante ",
			    "where albaranabono.codigoreparacion = reparacion.codigoreparacion ",
			    "and reparacion.codigoCliente = participante.codigoParty and albaranabono.totalTaller is not null and ",
			    "participante.codigoParty=",client_code_char, "and reparacion.codigoTaller=",garage_code_aux," and albaranabono.numeroalbaran=",delivery_payment_number,
			    "and albaranabono.codigoTienda=",shop_code, "and albaranabono.codigoEmpresa=",enterprise_code, " union ",
			    " select numeroalbaran from albaranabono, reparacion ",
			    "where albaranabono.codigoreparacion = reparacion.codigoreparacion ",
			    " and reparacion.codigoCliente = ",garage_code_aux," and albaranabono.totalTaller is null ",
			    " and albaranabono.numeroalbaran=",delivery_payment_number,
			    " and albaranabono.codigoTienda=",shop_code, " and albaranabono.codigoEmpresa=",enterprise_code,
			    NULL);
    } else {
	query = g_strconcat(" select codigoPArty from participante where "
			    " participante.codigoParty=",client_code_char, " and taller=TRUE ",
			    NULL);

    }
    if (m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE)==FALSE) {
	result = FALSE;
    }
    g_print(_("\nQuery: %s\n"),query);
    if (record_set!=NULL) {
	if (*tuples==1) {
            result=TRUE;
	}
    }

    if (record_set!=NULL)
	gda_recordset_free(record_set);
    g_free(tuples);
    g_free(garage_code_aux);
    g_free(enterprise_code);
    g_free(shop_code);
    if (client_code_char!=NULL)
	g_free(client_code_char);
    g_free(delivery_payment_number);
    return result;
}

/**
 * m_delivery_payment_client_is_not_from_garage: 
 * @garage_code: Cdigo del taller que se desea comprobar.
 * @client_code: Cdigo del cliente del albarn de abono que se desea comprobar.
 * @delivery_payment_number: Nmero del albarn que se desea comprobar
 * @shop_code: Cdigo de la tienda en la que se emiti el albarn
 * @enterprise_code: Cdigo de la empresa en la que se emiti el albarn.
 *
 * Funcin que realiza dos tipos de comprobacines segn los parmetros: Si @client_code es igual a @garage_code
 * entonces solo comprueba si dicho cdigo se corresponde con un taller en la tabla de talleres, es decir,
 * si el cliente es un taller devuelve %TRUE. En caso contrario,
 * comprueba si el @client_code es cliente del albarn solicitado y @garage_code es taller asociado a la reparacin de
 * ese albarn, o bien, si @garage_code aparece como cliente en la reparacin asociada a ese albarn. Si una de
 * las dos ltimas condiciones son ciertas, devuelve %TRUE.
 *
 * Returns:
 * %TRUE en caso de que sea cierto lo solicitado , %FALSE en caso contrario.
 */
gboolean m_delivery_payment_client_is_not_from_garage (gulong garage_code,gulong client_code,gulong delivery_payment_number,gulong shop_code,gulong enterprise_code) {

    gchar * query;
    gchar * garage_code_aux=g_strdup_printf("%lu",garage_code);
    gchar * client_code_aux=g_strdup_printf("%lu",client_code);
    gchar * enterprise_code_aux=g_strdup_printf("%lu",enterprise_code);
    gchar * shop_code_aux=g_strdup_printf("%lu",shop_code);
    gchar * delivery_payment_number_aux=g_strdup_printf("%lu",delivery_payment_number);
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gboolean result=FALSE;
    GdaCommand * command = gda_command_new();
    GdaConnection * connection = m_model_get_connection ();


    gda_command_set_connection(command,connection);

    if (garage_code!=client_code) {
	query = g_strconcat("select numeroalbaran from albaranabono, reparacion, participante ",
			    "where albaranabono.codigoreparacion = reparacion.codigoreparacion ",
			    "and reparacion.codigoCliente = participante.codigoParty and albaranabono.totalTaller is not null and ",
			    "participante.codigoParty=",client_code_aux, "and reparacion.codigoTaller=",garage_code_aux," and albaranabono.numeroalbaran=",delivery_payment_number_aux,
			    "and albaranabono.codigoTienda=",shop_code_aux, "and albaranabono.codigoEmpresa=",enterprise_code_aux, " union ",
			    " select numeroalbaran from albaranabono, reparacion ",
			    "where albaranabono.codigoreparacion = reparacion.codigoreparacion ",
			    " and reparacion.codigoCliente = ",garage_code_aux," and albaranabono.totalTaller is null ",
			    " and albaranabono.numeroalbaran=",delivery_payment_number_aux,
			    " and albaranabono.codigoTienda=",shop_code_aux, " and albaranabono.codigoEmpresa=",enterprise_code_aux,
			    NULL);
    } else {
	query = g_strconcat(" select codigoPArty from participante where "
			    " participante.codigoParty=",client_code_aux, " and taller=TRUE ",
			    NULL);

    }

    g_free(client_code_aux);
    g_print(_("\nQuery: %s\n"),query);

    if (m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE)==FALSE) {
	result = FALSE;
    }

    if (query!=NULL)
        g_free(query);
    if (record_set!=NULL) {
	if (*tuples==1) {
            result=TRUE;
	}
    }

    if (record_set!=NULL)
	gda_recordset_free(record_set);
    g_free(tuples);
    g_free(garage_code_aux);
    g_free(enterprise_code_aux);
    g_free(shop_code_aux);
    g_free(delivery_payment_number_aux);
    return result;

}

/**
 * m_delivery_payment_update_garage_total:
 * @delivery_payment: Albarn de abono que ser modificado.
 *
 * Modifica el total de comisin que se le pagar al taller asociado.
 * Una empresa paga comisiones a los talleres que envan clientes con algn problema a la misma, estos talleres
 * eviarn a estos clientes cuando estos quieren declarar el problema al seguro. Por ello, despues se
 * aadir un total al albarn de abono para pagar al taller, que aparecer en el momento que
 * se desee facturar el albarn para el taller.
 *
 * Returns:
 * %TRUE en caso de correcta modificacin, %FALSE en caso contrario.
 */
gboolean m_delivery_payment_update_garage_total (DeliveryPayment * delivery_payment) {
    gboolean result =FALSE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar * delivery_payment_number=NULL;
    gchar * shop_code=NULL;
    gchar * enterprise_code=NULL;
    gchar * garage_total = NULL;
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();

    if (delivery_payment!=NULL) {
	delivery_payment_number = g_strdup_printf("%lu",d_delivery_payment_get_number (delivery_payment));
	enterprise_code = g_strdup_printf("%lu",d_participant_get_code (d_enterprise_get_participant (
												d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))
											       )));
	shop_code =  g_strdup_printf("%lu",d_participant_get_code (d_shop_get_participant (
											  d_repair_get_shop (d_delivery_payment_get_repair (delivery_payment))
											 )));

        if (d_delivery_payment_get_garage_total (delivery_payment)>0)
	    garage_total = g_strdup_printf("%.2f",d_delivery_payment_get_garage_total (delivery_payment));
	else
            garage_total = g_strdup("NULL");
	if (delivery_payment_number!=NULL) {
	    gulong code_participant_shop=ULONG_MAX;
	    gulong code_participant_enterprise=ULONG_MAX;

	    if (d_delivery_payment_get_repair (delivery_payment)!=NULL) {
		if (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
		    code_participant_enterprise = d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (d_delivery_payment_get_repair (delivery_payment))));
		}
		if (d_repair_get_shop (d_delivery_payment_get_repair (delivery_payment))!=NULL) {
		    code_participant_shop =d_participant_get_code (d_shop_get_participant (d_repair_get_shop (d_delivery_payment_get_repair (delivery_payment))));
		}
	    }
	    gda_command_set_connection(command,connection);

	    
	    query = g_strconcat("update albaranabono set totalTaller=to_number('",garage_total,
				"','99999999D99') where (numeroAlbaran=",delivery_payment_number,
				") and (codigoEmpresa=",enterprise_code,
				") and (codigoTienda=",shop_code,
				")",NULL);
	    g_print(_("\nUpdate: %s\n"),query);
	    result = m_model_mutex_transactional_command_execute_logged (&record_set,
                                                               query,
							       connection,
							       command, tuples,
							       0, FALSE,
							       code_participant_enterprise,
							       code_participant_shop,
                                                               ULONG_MAX
							      );

	    g_free(delivery_payment_number);

	}
	if (record_set!=NULL)
	    gda_recordset_free(record_set);

	g_free(query);
	g_free(tuples);
	g_free(shop_code);
	g_free(enterprise_code);
	g_free(garage_total);

    }
    igalia_command_free(command);

    return result;
}

/**
 * m_delivery_payment_find_by_garage:
 * @invoice_number: Cdigo de la factura de abono que tiene asociados unos albaranes.
 * @shop_code: Cdigo de la tienda de emisin de la factura.
 * @enterprise_code: Cdigo de la empresa de emisin de la factura.
 * 
 * Esta funcin realiza la busqueda de los albaranes asociados a una factura de un taller. La necesidad de
 * esta funcin viene dada porque existen albaranes que tienen como cliente a otra persona/empresa y
 * se le facturan negativamente (para pagarle una comisin) al taller, con lo que han de ser tratados
 * de modo diferente a los albaranes convencionales.
 *
 * Returns:
 * Lista de albaranes asociados a la factura del taller
 */
DeliveryPayment ** m_delivery_payment_find_by_garage (gchar * invoice_number,gchar * shop_code,gchar * enterprise_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_order=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    GdaField * field;
    DeliveryPayment ** new_delivery_payments;
    DeliveryPayment * delivery_payment;
    gint index;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);


    gchar_select = g_strconcat("SELECT codigoempresafactura,codigotiendafactura,numerofacturataller,codigoempresaalbaran, ",
			       " codigotiendaalbaran, numeroalbaran from facturataller where codigoEmpresafactura=",enterprise_code,
                               " and codigoTiendaFactura=",shop_code," and numeroFacturataller=",invoice_number,
			       NULL );


    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,gchar_order, NULL);
    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {
	    new_delivery_payments = (DeliveryPayment **) g_malloc0(sizeof(DeliveryPayment*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		DeliveryPaymentId * delivery_payment_id = (DeliveryPaymentId *) g_malloc0(sizeof(DeliveryPayment));
		field = gda_recordset_field_name(record_set,"numeroalbaran");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			delivery_payment_id->delivery_payment_number=g_strdup_printf("%lu",(gulong) igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field = gda_recordset_field_name(record_set,"codigoempresaalbaran");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			delivery_payment_id->enterprise_code=g_strdup_printf("%lu",(gulong) igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}
		field = gda_recordset_field_name(record_set,"codigotiendaalbaran");
		if (field!=NULL) {
		    if (!gda_field_is_null(field)) {
			delivery_payment_id->shop_code=g_strdup_printf("%lu",(gulong) igalia_field_get_bigint_value (field));
		    }
		    gda_field_free(field);
		}

		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;

		delivery_payment = m_delivery_payment_find_by_id (delivery_payment_id);
		new_delivery_payments[index-1] = delivery_payment;
		index = gda_recordset_move_next(record_set);
	    }
	    new_delivery_payments[*tuples]=NULL;
	} else {
	    new_delivery_payments=NULL;
	}
	gda_recordset_close(record_set);
    } else {
	new_delivery_payments=NULL;
    }

    igalia_command_free(my_command);
    gda_recordset_free(record_set);


    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_order);
    g_free(gchar_from);
    g_free(gchar_query);
    g_free(tuples);

    return new_delivery_payments;

}


/**
 * m_delivery_payment_join_array:
 * @delivery_payments1: Array con los albaranes de abono que irn en primer lugar.
 * @delivery_payments2: Array con los albaranes de abono que irn en segundo lugar.
 *
 *  Une dos arrays de albaranes de abono en uno solo que contiene primero
 * los del primer parmetro de entrada.
 *
 * Returns:
 * Array de albaranes que une los dos arrays de entrada. El ltimo albarnn de abono es %NULL.
 */
DeliveryPayment ** m_delivery_payment_join_array (DeliveryPayment ** delivery_payments1,DeliveryPayment ** delivery_payments2) {
    gulong index =0;
    gulong total_index =0;
    gulong length = 0;
    DeliveryPayment ** result=NULL;
    if (delivery_payments1!=NULL) {
	while (delivery_payments1[index]!=NULL) {
            index++;
	}
    }
    length = index;

    index=0;
    if (delivery_payments2!=NULL) {
	while (delivery_payments2[index]!=NULL) {
            index++;
	}
    }
    length = length + index;
    result = (DeliveryPayment **) g_malloc0(sizeof(DeliveryPayment*)*(length+1));
    index=0;
    if (delivery_payments1!=NULL) {
	while (delivery_payments1[index]!=NULL) {
	    result[total_index]=delivery_payments1[index];
            delivery_payments1[index]=NULL;
	    index++;
            total_index++;
	}
    }
    length = index;

    index=0;
    if (delivery_payments2!=NULL) {
	while (delivery_payments2[index]!=NULL) {
	    result[total_index]=delivery_payments2[index];
            delivery_payments2[index]=NULL;
            index++;
            total_index++;
	}
    }

    result[total_index]=NULL;
    if (delivery_payments1!=NULL) {

	//	g_free(delivery_payments1);
    }
    if (delivery_payments2!=NULL) {

	//	g_free(delivery_payments2);
    }
    delivery_payments1=NULL;
    delivery_payments2=NULL;
    return result;

}

gboolean m_delivery_payment_dissasociate_invoice (gchar * delivery_payment_number,gchar * shop_code,gchar *enterprise_code, gchar * invoice_code) {

	return m_delivery_payment_set_invoice_to_null (delivery_payment_number, shop_code, enterprise_code, invoice_code, NULL, NULL);
}

gboolean m_delivery_payment_set_invoice_to_null (gchar * delivery_payment_number,gchar * shop_code,gchar *enterprise_code,gchar * invoice_code,GdaConnection * connection,GdaCommand * command) {
    gboolean result = TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    gboolean new_transaction=FALSE;
    gulong shop_code_gulong;
    gulong enterprise_code_gulong;
    gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));

    if (connection==NULL) {
	connection = m_model_get_connection ();
	new_transaction = TRUE;
	command = gda_command_new();
	gda_command_set_connection(command,connection);
    }

    shop_code_gulong = strtoul(shop_code,stop,10);

    enterprise_code_gulong = strtoul(enterprise_code,stop,10);

    g_free(stop);

    if (new_transaction!=FALSE)
	m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code_gulong,shop_code_gulong,ULONG_MAX);

    if ((delivery_payment_number!=NULL) && (shop_code!=NULL) && (enterprise_code!=NULL)) {

	query = g_strconcat("update albaranabono set numeroFactura=null ",
			    " where (numeroAlbaran=",delivery_payment_number,
			    ") and (numeroFactura=",invoice_code,
			    ") and (codigoEmpresa=",enterprise_code,
			    ") and (codigoTienda=",shop_code,
			    ")",NULL);
	g_print(_("\nSet NULL: %s\n"),query);
	result =m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE) && result;
	
	query = g_strconcat("delete from facturataller ",
			    " where (numeroAlbaran=",delivery_payment_number,
			    ") and (numeroFacturaTaller=",invoice_code,
			    ") and (codigoEmpresaAlbaran=",enterprise_code,
			    ") and (codigoTiendaAlbaran=",shop_code,
			    ") ",NULL);
			    
	g_print(_("\nDelete repair shop invoice: %s\n"),query);
	result =m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE) && result;		
    }

    if (record_set!=NULL)
	gda_recordset_free(record_set);

    g_free(query);
    g_free(tuples);

    if (new_transaction!=FALSE) {
	igalia_command_free(command);
	if (m_model_execute_mutex_modification_logged (connection)!=TRUE) {
	    result = FALSE;
	}
    }

    return result;

}

