/* find_pieces_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "find_pieces_model.h"
#include "piece_model.h"
#include "product_model.h"


/**
 * m_piece_find_by_condition:
 * @condition: Condicin de bsqueda. Premite realizar la bsqueda por eurocode y descripcin.
 * @tuples:Nmero de tuplas que cumplen la condicin de bsqueda.
 *
 * Busca las piezas que cumplan todos o algunos de los
 * filtros de condicin
 *
 * Returns:
 * Array de piezas que cumplieron la condicin de bsqueda.
 */
Piece ** m_piece_find_by_condition (ConditionFindPieces *condition, gulong * tuples) {


    gint           in;
    GdaCommand    *my_command;
    GString       *g_string_query;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Piece        **pieces_array;
    gchar * table_product;
    gchar * table_piece;
    gboolean       correct = FALSE;

    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);

    g_string_query = g_string_new("SELECT ");
    table_product = u_table_put_name (m_product_get_attributes (), m_product_get_table ());
    table_piece = u_table_put_name (m_piece_get_attributes (), m_piece_get_table ());
    g_string_append(g_string_query, table_product);
    g_string_append(g_string_query, ", ");
    g_string_append(g_string_query, table_piece);
    g_string_append(g_string_query, " FROM ");
    g_string_append(g_string_query, m_piece_get_table ());
    g_string_append(g_string_query, ", ");
    g_string_append(g_string_query, m_product_get_table ());
    g_string_append(g_string_query,
		    " WHERE (Producto.codigoProducto = Pieza.codigoProducto) ");

    g_free(table_product);
    g_free(table_piece);
    if (((condition->description) != NULL) && (strlen(condition->description)!=0)) {
	g_string_append(g_string_query, " AND (producto.descripcion ILIKE '%");
	g_string_append(g_string_query, condition->description);
	g_string_append(g_string_query, "%')");
    }


    if (((condition->eurocode) != NULL) && (strlen(condition->eurocode)!=0)) {
	g_string_append(g_string_query, "AND (pieza.eurocode ILIKE '");
	g_string_append(g_string_query, condition->eurocode);
	g_string_append(g_string_query, "%')");
    }


    correct = m_model_command_execute (&record_set, g_string_query->str, my_connection,my_command,tuples,0,TRUE);


    pieces_array=NULL;
    if (correct) {
	if (*tuples!=0) {
	    pieces_array = (Piece **) g_malloc ((sizeof(Piece *))*(*tuples +1));
	    in = m_piece_obtain_pieces_array_from_record_set (record_set,pieces_array);
	}
    }
    gda_recordset_free(record_set);
    igalia_command_free(my_command);

    g_string_free(g_string_query, TRUE);
    return pieces_array;

}

/**
 * m_piece_find_by_product_code:
 * @codes: Lista de cdigos por los que se desea buscar.
 * @number: Nmero de piezas que desean obtener, que coincide con el nmero de elementos de la
 * lista de cdigos.
 *
 *  Busca las piezas que tengan como cdigo de producto
 * uno de los de la lista de cdigos.
 *
 * Returns:
 * Lista de piezas que cumplen la condicin.
 */
Piece ** m_piece_find_by_product_code (glong *codes, gulong *number) {

    gint           index = 0;
    gulong        tuples = 0;
    gchar         *temp = NULL;
    GdaCommand    *my_command = NULL;
    GString       *g_string_query = NULL;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    Piece        **pieces_array = NULL;
    gboolean       correct = FALSE;

    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);

    pieces_array = (Piece **) g_malloc ((sizeof(Piece *))*(*number));
    while (codes[index]!= -1) {
	gchar * table_product = u_table_put_name (m_product_get_attributes (), m_product_get_table ());
	gchar * table_piece = u_table_put_name (m_piece_get_attributes (), m_piece_get_table ());

	g_string_query = g_string_new("SELECT ");

	g_string_append(g_string_query, table_product);
	g_string_append(g_string_query, ", ");
	g_string_append(g_string_query, table_piece);
	g_string_append(g_string_query, " FROM ");
	g_string_append(g_string_query, m_piece_get_table ());
	g_string_append(g_string_query, ", ");
	g_string_append(g_string_query, m_product_get_table ());
	g_string_append(g_string_query, " WHERE (Producto.codigoProducto = ");
	temp = g_strdup_printf("%lu", codes[index]);
	g_string_append(g_string_query, temp);
	g_string_append(g_string_query, " ) ");

	g_string_append(g_string_query, " AND (Pieza.codigoProducto = ");
	g_string_append(g_string_query, temp);
	g_string_append(g_string_query, " ) ");
        g_free(table_product);
        g_free(table_piece);
	g_free(temp);

	correct = m_model_command_execute (&record_set, g_string_query->str, my_connection,my_command, &tuples, 0,TRUE);
	gda_recordset_move_first(record_set);


	pieces_array[index] = NULL;
	if (correct) {
	    if ((tuples==1) && m_model_record_set_contains_piece (record_set)) {
		pieces_array[index] = m_piece_create (record_set);
	    }
	}
	index++;
    }
    pieces_array[index] = NULL;
    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_string_free(g_string_query, TRUE);
    return pieces_array;
}

/**
 * m_piece_obtain_pieces_array_from_record_set:
 * @rs: Record set del que se desea obtener la lista de piezas.
 * @pieces_array: Lista de piezas obtenida. Es necesario reservar memoria previamente.
 *
 * Obtiene una lista de piezas a partir de un record set de la base de datos.
 * @rs: Record set del que se desea obtener la lista de piezas.
 *
 * Returns:
 * Tamao de la lista de piezas.
 */
gint m_piece_obtain_pieces_array_from_record_set (GdaRecordset *rs,
			       Piece **pieces_array) {
    gint index;
    gint column = 0;
    index = gda_recordset_move_first(rs);

    while (!gda_recordset_eof(rs)) {
	column = 0;

	if (index==GDA_RECORDSET_INVALID_POSITION) return -1;

	pieces_array[index-1] = m_piece_create (rs);
	index = gda_recordset_move_next(rs);
    }
    pieces_array[index] = NULL;

    return index;
}

/**
 * m_piece_clear_array: 
 * @pieces_array: Array de piezas que se desea liberar.
 *
 * Libera la memoria utilizada por un array de piezas.
 *
 * Returns: 0 si se ha salido correctamente.
 */
gint m_piece_clear_array (Piece ** pieces_array) {
    gulong index = 0;

    if (pieces_array!=NULL) {
	while(pieces_array[index]!=NULL) {
	    d_piece_clear (pieces_array[index]);
            pieces_array[index] = NULL;
	    index++;
	}
    }
    g_print(_(" %lu pieces liberated\n"), index);
    g_free(pieces_array);
    pieces_array=NULL;
    return 0;
}

/**
 * m_piece_get_product_code_by_eurocode:
 * @eurocode: Eurocode por el que se desea buscar.
 *
 * Busca el cdigo de producto de la pieza que tiene un eurocode en particular
 *
 * Returns:
 * El cdigo de producto.
 */
gulong m_piece_get_product_code_by_eurocode (gchar * eurocode) {
    gulong        tuples = 0;
    GdaCommand    *my_command = NULL;
    gchar * query;
    gulong result=ULONG_MAX;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gboolean       correct = FALSE;

    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);


    query = g_strconcat("SELECT CODIGOPRODUCTO FROM PIEZA WHERE EUROCODE='",eurocode,"'",NULL);

    correct = m_model_command_execute (&record_set, query, my_connection,my_command, &tuples, 0,TRUE);

    g_print(_("\nQuery: %s\n"),query);
    if (record_set!=NULL) {
	if (GDA_IS_RECORDSET(record_set)) {
	    GdaField * field = NULL;
	    gda_recordset_move_first(record_set);

	    field = gda_recordset_field_name(record_set,"codigoProducto");

	    result = igalia_field_get_bigint_value (field);
	}
    }
    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_free(query);
    g_print(_("\nCode: %lu\n"),result);
    return result;

}

/**
 * m_piece_get_eurocode_by_product_code:
 * @product_code: Cdigo del producto a buscar.
 *
 * Busca el eurocode de la pieza que tiene un cdigo de producto en particular
 *
 * Returns:
 * El cdigo de producto.
 */
gchar * m_piece_get_eurocode_by_product_code (gulong product_code) {
    gulong        tuples = 0;
    GdaCommand    *my_command = NULL;
    gchar * query;
    gchar * result = NULL;
    GdaConnection *my_connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gboolean       correct = FALSE;
    gchar * product_code_gchar = g_strdup_printf("%lu",product_code);
    my_command = gda_command_new();

    gda_command_set_connection(my_command,my_connection);


    query = g_strconcat("SELECT EUROCODE FROM PIEZA WHERE CODIGOPRODUCTO=",product_code_gchar,"",NULL);

    if (product_code_gchar!=NULL)
	g_free(product_code_gchar);

    correct = m_model_command_execute (&record_set, query, my_connection,my_command, &tuples, 0,TRUE);

    g_print(_("\nQuery: %s\n"),query);

    if (record_set!=NULL) {
	if (GDA_IS_RECORDSET(record_set)) {
	    GdaField * field = NULL;
            gchar * temp;
	    gda_recordset_move_first(record_set);

	    field = gda_recordset_field_name(record_set,"eurocode");
	    temp=gda_stringify_value(NULL, 0, field);
	    result = g_strdup(temp);
	    g_free(temp);
	}
    }
    gda_recordset_free(record_set);
    igalia_command_free(my_command);
    g_free(query);
    g_print(_("\nEurocode %s\n"),result);
    return result;
}
