/* gda_igalia_util.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */


#include "gda_igalia_util.h"
#include "util.h"

/**
 * igalia_field_get_bigint_value:
 * @field: Campo del que se quiere obtener el valor.
 *
 * Obtiene el valor con tipo gulong que hai en un campo de la base de datos.
 *
 * Returns:
 * El valor que exista en el campo de la base de datos con ese tipo.
 */
gulong igalia_field_get_bigint_value (GdaField * field) {
    gulong result = ULONG_MAX;
    gchar * temp=NULL;

    temp=gda_stringify_value(NULL, 0, field);
    if (temp!=NULL) {
        gchar ** stop = (gchar **) g_malloc0(sizeof(gchar *));
        result = strtoul(temp,stop,10);
	if (strcmp(*stop,"")) {
            result = ULONG_MAX;
	}
	g_free(stop);
	g_free(temp);
    }
    return result;
}

/**
 * igalia_field_get_double_value:
 * @field: Campo del que se quiere obtener el valor.
 *
 * Obtiene el valor con tipo gulong que hai en un campo de la base de datos.
 *
 * Returns:
 * El valor que exista en el campo de la base de datos con ese tipo.
 */
gdouble igalia_field_get_double_value (GdaField * field) {
    gdouble result = ULONG_MAX;
    gchar * temp=NULL;

    temp=gda_stringify_value(NULL, 0, field);
    g_strstrip(temp);
    u_points_to_commas (temp);
    if (temp!=NULL) {
        result = atof(temp);
	g_free(temp);
    } else {
        result = 0;
    }
    return result;
}

void igalia_command_free (GdaCommand * command) {

    GtkObjectClass *parent_class;

    g_return_if_fail(command!=NULL);
    g_return_if_fail(GDA_IS_COMMAND(command));



    if (command->text!=NULL)
	g_free(command->text);
    command->text=NULL;

    parent_class = gtk_type_class (gtk_object_get_type ());
    if (parent_class && parent_class->destroy)
	parent_class->destroy ((GtkObject *) command);

//    gda_command_set_connection(command,NULL);
    gda_command_free(command);

    g_free(command);




}

