 /* incoming_book_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "incoming_book_model.h"
#include "participant_model.h"
#include "invoice.h"

/**
 * m_incoming_book_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #IncomingBook con los campos rellenados con los datos de la base de datos.
 */
IncomingBook * m_incoming_book_create (GdaRecordset *rs) {

    GdaField * field;
    IncomingBook * incoming_book;

    incoming_book = d_incoming_book_create_empty();

    incoming_book->participant = m_participant_create (rs,"Ingresos");


    field = gda_recordset_field_name(rs, "totalIngresos");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    incoming_book->total = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    return incoming_book;
}



/**
 * m_incoming_book_find_by_condition:
 * @condition: Condicin de bsqueda. Permite buscar por empresa y perodo de tiempo.
 * @tuples: Nmero de elementos que de se devolvern.
 *
 * Busca los objetos que representan una entrada en el libro de
 * ingresos por condicin.
 *
 * Returns:
 * Lista de ingresos que cumplen la condicin de bsqueda.
 */
IncomingBook **  m_incoming_book_find_by_condition (ConditionAccountingPeriod *condition,gulong * tuples) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where_empresa=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_groupby=NULL;
    gchar              *gchar_having=NULL;
    gchar              *gchar_order=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar * state = g_strdup_printf("%d",D_INVOICE_NOT_ISSUED);
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * enterprise_code = FALSE;
    IncomingBook ** incoming_books;
    IncomingBook * incoming_book;
    gint index;

    my_command = gda_command_new();


    gda_command_set_connection(my_command,my_connection);

    if ((condition->enterprise_code!=ULONG_MAX) && (condition->enterprise_code!=0))
        enterprise_code = g_strdup_printf("%lu",condition->enterprise_code);

    if (((enterprise_code) != NULL) && ((enterprise_code)!=0)) {
	gchar_where_empresa = g_strconcat(" and (facturaAbono.codigoEmpresa=",
					  enterprise_code,")",NULL);
    } else {
	gchar_where_empresa = g_strdup(" ");
    }

    gchar_select=g_strconcat(" select nick as nickPArticipantIngresos , ",
			     "cif as cifParticipantIngresos,",
			     "codigoParty as codigoPArtyParticipantIngresos,",
			     "calle as calleParticipantIngresos,",
			     "numero as numeroParticipantIngresos,",
			     "piso as pisoParticipantIngresos,",
			     "cp as cpParticipantIngresos,",
			     "municipio as municipioParticipantIngresos,",
			     "provincia as provinciaParticipantIngresos,",
			     "to_char(sum(baseImponibleNueva),'99999999D99') as totalIngresos ",NULL);


    gchar_from= g_strconcat(" FROM facturaabono,participante ",NULL);


    gchar_where=g_strconcat(" where facturaabono.estado>",state," and ",
                            " facturaabono.tipoIva is NULL and ",
			    " facturaabono.codigoPagador=participante.codigoParty ",
			    NULL);
    g_free(state);
    if (((enterprise_code) != NULL) && (strlen(enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.codigoEmpresa=",
				      enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->initial_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.fechaEmision>='",
				      condition->initial_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->ending_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturaabono.fechaEmision<='",
				      condition->ending_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if (condition->greater_than!=FALSE) {
	gchar_having = " having SUM(FacturaAbono.baseImponibleNueva)>=CAST(3005.6 as DECIMAL(12,2))";
    } else {
	gchar_having = " ";
    }


    gchar_order = g_strconcat(" ORDER BY  nick,cif ",NULL);
    
    gchar_groupby= g_strconcat(" group by codigoPArty,nick,cif,"
			       "calle,numero,piso,cp,municipio,"
			       "provincia ", NULL);
    gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where, gchar_groupby,gchar_having,gchar_order,NULL);

    g_print(_("\nQuery %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {

	    incoming_books = (IncomingBook **) g_malloc0(sizeof(IncomingBook*)*(*tuples+1));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		incoming_book = m_incoming_book_create (record_set);
		incoming_books[index-1] = incoming_book;
		    index = gda_recordset_move_next(record_set);
	    }
	    incoming_books[*tuples]=NULL;
	} else {
		incoming_books=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	incoming_books=NULL;
    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(gchar_where);
    g_free(gchar_where_empresa);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_order);
    g_free(gchar_groupby);

    return incoming_books;
    
}

/**
 * model_incoming_book_clear_array:
 * @incoming_books: Array de objetos IncomingBook que se desea liberar.
 * 
 * Libera la memoria utilizada por un array de objetos IncomingBook.
 *
 */
void m_incoming_book_clear_array  (IncomingBook ** incoming_books) {
    if (incoming_books!=NULL) {
	gint index = 0;
	while (incoming_books[index]!=NULL) {
	    d_incoming_book_clear (incoming_books[index]);
	    index++;
	}
    }
    g_free(incoming_books);

}

