
/* invoice_model.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _INVOICE_MODEL_
#define _INVOICE_MODEL_

#include <gda-client.h>
#include <gda-field.h>

#ifndef _MODEL_
#include "model.h"
#endif

#include "gda_igalia_util.h"
#include "invoice.h"
#include "payment.h"
#include "repair_line_model.h"
#include "delivery_payment_model.h"
#include "delivery_payment.h"
#include "payment_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "direct_debit.h"
#include "gda_igalia_util.h"

typedef struct conditionFindInvoiceStruct {
    gint type; /*Tipo de factura: FACTURA, REFACTURA o NOTA*/
    gulong enterprise_code;/*Cdigo de la empresa*/
    gulong shop_code;/*Cdigo de la tienda*/
    gchar * client_name;/*Nombre de cliente para el que se busca la factura*/
    gchar * client_legal_id;/*Cif/Nif del cliente*/
    gulong invoice_number;/*Nmero de la factura*/
    gint invoice_state;/*Estado de la factura*/
    time_t * initial_issue_date;/*Fecha de emisin inicio: el nombre es para mantener compatibilidad con versiones anterior*/
    time_t * ending_issue_date;/*Fecha de emisin fin: el nombre es para mantener compatibilidad con versiones anterior*/
    gulong payment_type;/*Tipo de pago de la factura*/
    gdouble amount;/*Total de la factura*/
    gboolean vat;/*Si es con iva o sin iva*/
    gchar * license_number;/*Matrcula del coche*/
    gchar * accident_reference;/*Referencia del siniestro*/
    gulong company_code;/*Cdigo de la compaa de seguros*/
} ConditionFindInvoice;

typedef struct invoiceId {
    gchar * invoice_number;
    gchar * shop_code ;
    gchar * enterprise_code;
} InvoiceId;


Invoice * m_invoice_create (GdaRecordset *rs,DeliveryPayment ** delivery_payments);
gchar *m_invoice_get_table (void);
gchar *m_invoice_get_attributes (void);
Invoice * m_invoice_obtain_data (DeliveryPaymentId ** delivery_payments_client);
Invoice * m_invoice_find_by_id (InvoiceId * invoice_id);
Invoice ** m_invoice_find_by_condition (ConditionFindInvoice * condition);
void m_invoice_clear_array (Invoice ** invoices);
gulong m_invoice_insert (Invoice * invoice);
gboolean m_invoice_update_state (Invoice * invoice,gint state);
gboolean m_invoice_update (Invoice * invoice);
gulong  m_invoice_get_number (gchar * enterprise,gchar * shop, GdaConnection *connection, GdaCommand *command);
void m_invoice_set_state_selected  (Invoice ** invoices, GArray * selected, gint old_state, gint new_state);
gboolean m_invoice_update_number (gulong enterprise_code,gulong shop_code,gchar * invoice_number_char);
PaymentId * m_invoice_find_last_payment_from_client (Invoice * invoice);
gboolean m_invoice_get_state_from_direct_debit (DirectDebit ** direct_debits,GdaConnection * connection);
gboolean m_invoice_set_state_of_invoices (InvoiceId ** invoice_codes,gint new_state);
InvoiceId * m_invoice_get_split_invoice (gulong invoice_code,gulong shop_code,gulong enterprise_code);
gchar * m_invoice_delete (Invoice * invoice);
gdouble m_invoice_modify_subtotal (Invoice * invoice,gdouble new_subtotal,gboolean with_discount);

gulong m_invoice_get_shop_code_as_gulnog (Invoice *invoice);
gchar *m_invoice_get_shop_code_as_gchar (Invoice *invoice);
gulong m_invoice_get_enterprise_code_as_gulong (Invoice *invoice);
gchar *m_invoice_get_enterprise_code_as_gchar (Invoice *invoice);
gchar *m_invoice_get_number_as_gchar (Invoice *invoice);
InvoiceId * m_invoice_get_id (Invoice *invoice);
void m_invoice_get_data_to_delivery_payment_line(gchar ** description,gchar ** vat,gchar ** quantity);
#endif

