 /* issued_book_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "issued_book_model.h"
#include "participant_model.h"
#include "invoice.h"


/**
 * m_issued_book_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #IssuedBook con los campos rellenados con los datos de la base de datos.
 */
IssuedBook * m_issued_book_create (GdaRecordset *rs) {

    GdaField * field;
    IssuedBook * issued_book;

    issued_book = d_issued_book_create_empty();


    field = gda_recordset_field_name(rs,"numeroFacturaLibroEmitidas" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp_number=gda_stringify_value(NULL, 0, field);
	    issued_book->invoice_number = g_strdup(temp_number);
	    g_free(temp_number);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"codigoPertenenciaLibroEmitidas" );
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp_number=gda_stringify_value(NULL, 0, field);
	    issued_book->group_code = g_strdup(temp_number);
	    g_free(temp_number);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs,"fechaEmisionLibroEmitidas");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp;
	    time_t * my_time;
	    temp=gda_stringify_value(NULL, 0, field);
	    my_time = u_parse_date (temp);
	    issued_book->invoice_issue_date = (*my_time);
	    my_time = NULL;
	    g_free(temp);
	}
	gda_field_free(field);
    }


    field = gda_recordset_field_name(rs, "razonSocialLibroEmitidas");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    issued_book->trade_name = g_strdup(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }


    field = gda_recordset_field_name(rs, "modificadaLibroEmitidas");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    issued_book->agreed_module = gda_field_get_boolean_value(field);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs, "tipoIvaLibroEmitidas");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    issued_book->vat_type = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    field = gda_recordset_field_name(rs, "descuentoFacturaLibroEmitidas");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    issued_book->invoice_discount = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }
    
    field = gda_recordset_field_name(rs, "totalLibroEmitidas");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    issued_book->total = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }
    field = gda_recordset_field_name(rs, "subtotalLineas");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    gchar * temp=gda_stringify_value(NULL, 0, field);
	    issued_book->line_subtotal = atof(temp);
	    g_free(temp);
	}
	gda_field_free(field);
    }

    return issued_book;
}


/**
 * m_issued_books_find_by_condition:
 * @condition: Condicin de bsqueda. Permite buscar por cdigo de empresa y perodo de contabilidad.
 * @tuples: Nmero de tuplas que sern devueltas. Es necesario reservar memoria antes de llamar a la funcin.
 * 
 * Permite buscar las factura emitidas para el libro de iva por
 * condicin de bsqueda.
 *
 * Returns:
 * Lista de entradas del libro de facturas emitidas perteneciente al libro de Iva.
 */
IssuedBook **  m_issued_books_find_by_condition (ConditionAccountingPeriod *condition,gulong * tuples) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where_enterprise=NULL;
    gchar              *gchar_where_initial_date=NULL;
    gchar              *gchar_where_ending_date=NULL;
    gchar              *gchar_where_enterprise2=NULL;
    gchar              *gchar_where_enterprise_ref=NULL;
    gchar              *gchar_where_enterprise_ref2=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_groupby=NULL;
    gchar              *gchar_order=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar * state = g_strdup_printf("%d",D_INVOICE_NOT_ISSUED);
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * enterprise_code = FALSE;
    IssuedBook ** issued_books;
    IssuedBook * issued_book;
    gint index;

    my_command = gda_command_new();


    gda_command_set_connection(my_command,my_connection);

    if ((condition->enterprise_code!=ULONG_MAX) && (condition->enterprise_code!=0))
        enterprise_code = g_strdup_printf("%lu",condition->enterprise_code);

    if (((enterprise_code) != NULL) && ((enterprise_code)!=0)) {
	gchar_where_enterprise = g_strconcat(" and (facturaAbono.codigoEmpresa=",
					  enterprise_code,") and (reparacion.codigoEmpresa=",enterprise_code
					  ,") and (tienda.codigoEmpresa=",enterprise_code,")",NULL);
	gchar_where_enterprise2 = g_strconcat(" and (albaranes.codigoEmpresa=",
					  enterprise_code,")",NULL);
	gchar_where_enterprise_ref = g_strconcat(" and (refacturacion.codigoEmpresaReFactura=",
					  enterprise_code,")",NULL);
	gchar_where_enterprise_ref2= g_strconcat(" and (facturaAbono.codigoEmpresa=",
					  enterprise_code,") and (tienda.codigoEmpresa=",enterprise_code,")",NULL);
    } else {
	gchar_where_enterprise = g_strdup(" ");
	gchar_where_enterprise2 = g_strdup(" ");
	gchar_where_enterprise_ref = g_strdup(" ");
    }
    if ((condition->initial_date) != NULL) {
	gchar_where_initial_date=  g_strconcat(" and (facturaAbono.fechaEmision>='",
				      condition->initial_date,"')",NULL);

    }
    if ((condition->ending_date) != NULL) {
	gchar_where_ending_date=  g_strconcat(" and (facturaAbono.fechaEmision<='",
				      condition->ending_date,"')",NULL);
    }

    gchar_select=g_strconcat("SELECT FACTURASEMITIDAS.NUMEROFACTURAFACTURA as numeroFacturaLibroEmitidas, ",
		 " FACTURASEMITIDAS.FECHAEMISIONFACTURA as fechaEMisionLibroEmitidas, ",
		 " FACTURASEMITIDAS.NICKPARTICIPANTPAGADOR as razonSocialLibroEmitidas, ",
		 " FACTURASEMITIDAS.baseImponibleModificadaFactura as modificadaLibroEmitidas, ",
		 " FACTURASEMITIDAS.baseImponibleNuevaFactura as totalLibroEmitidas, ",
		 " FACTURASEMITIDAS.codigoPertenenciaGrupoShop as codigoPertenenciaLibroEmitidas, ",
		 " FACTURASEMITIDAS.descuentoFactura as descuentoFacturaLibroEmitidas, ",
		 " to_char(lineaReparacion.tipoIVAHistorico,'999D99') as tipoIvaLibroEmitidas, ",
		 " to_char(SUM(CAST((CASE WHEN (LineaReparacion.precio IS NOT NULL) THEN ",
		 " (LineaReparacion.precio) WHEN ((LineaReparacion.precioHistorico IS NOT ",
		 " NULL) AND (LineaReparacion.dto is NOT NULL)) THEN ",
		 " ((LineaReparacion.unidades*Lineareparacion.precioHistorico)- ",
		 " (LineaReparacion.unidades*(LineaReparacion.dto/100)* ",
		 " LineaReparacion.precioHistorico)) WHEN ",
		 " ((LineaReparacion.precioHistorico IS NOT NULL) AND (LineaReparacion.dto ",
		 " IS NULL)) THEN ",
		 " (LineaReparacion.unidades*LineaReparacion.precioHistorico) ELSE 0.0 ",
 		 " END) as DECIMAL(10,2))),'99999999D999999') as subtotalLineas ",NULL);


    gchar_from= g_strconcat("FROM producto,",
			    "((SELECT facturaabono.numeroFactura as ",
 			    " numeroFacturaFactura,facturaabono.dtoFactura as ",
 			    " descuentoFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
			    " estadoFactura,facturaAbono.baseImponibleModificada as ",
			    " baseImponibleModificadaFactura, ",
 			    " to_char(facturaAbono.baseImponibleNueva,'99999999D999999') as ",
			    " baseImponibleNuevaFactura, ",
			    " facturaAbono.fechaEmision as fechaEmisionFactura, ",
                            " participanteCliente.nick as nickParticipantPagador,"
			    " participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador,reparacion.codigoReparacion as ",
			    " codigoReparacionRepair,reparacion.codigoTienda as ",
			    " codigoTiendaRepair, reparacion.codigoEmpresa as codigoEmpresaRepair,",
			    " tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as ",
			    " codigoPartyParticipantEnter ",
			    " from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono where ",
			    " (facturaabono.numeroFactura is not null)  and ",
                            " facturaabono.tipoIva is NULL and ",
			    " facturaabono.codigoTienda=participanteShop.codigoParty  and ",
			    " facturaabono.codigoTienda=reparacion.codigoTienda  and ",
			    " facturaabono.codigoEmpresa=reparacion.codigoEmpresa  and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty  and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
			    " (facturaabono.codigoReparacion = reparacion.codigoReparacion) ",
                            " and facturaabono.estado>",state,
			    gchar_where_enterprise,
                            gchar_where_initial_date,gchar_where_ending_date,
			    " order by ",
			    "  facturaabono.numeroFactura) ",
			    " union (SELECT facturaabono.numeroFactura ",
 			    " as numeroFacturaFactura,facturaabono.dtoFactura as ",
 			    " descuentoFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
			    " estadoFactura,facturaAbono.baseImponibleModificada as ",
			    " baseImponibleModificadaFactura, ",
 			    " to_char(facturaAbono.baseImponibleNueva,'99999999D999999') as ",
			    " baseImponibleNuevaFactura, ",
			    "   facturaAbono.fechaEmision as fechaEmisionFactura, ",
                            " participanteCliente.nick as nickParticipantPagador,"
			    "   participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador,(case WHEN reparacion.codigoCompania is not NULL ",
			    "THEN reparacion.codigoReparacion ",
                            " WHEN reparacion.codigoReparacion is not NULL THEN reparacion.codigoReparacion",
			    " ELSE NULL END) as ",
			    " codigoReparacionRepair,reparacion.codigoTienda as ",
			    " codigoTiendaRepair, reparacion.codigoEmpresa as codigoEmpresaRepair,",
			    " tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as ",
			    " codigoPartyParticipantEnter ",
			    " from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono, (select ",
			    " a.codigofactura,albaranabono.codigoReparacion,a.codigoTienda, ",
			    " a.codigoEmpresa from albaranabono,facturaabono a  where ",
			    " albaranabono.numeroFactura=a.codigoFactura and ",
			    " a.codigoEmpresa=albaranAbono.codigoEmpresa and ",
			    " a.codigoTienda=albaranabono.codigoTienda) as albaranes  where ",
			    " (facturaabono.numeroFactura is not null)  and ",
                            " facturaabono.tipoIva is NULL and ",
			    " facturaabono.codigoTienda=participanteShop.codigoParty and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
			    " (albaranes.codigoFactura = facturaAbono.codigoFactura)  and ",
			    " (albaranes.codigoTienda = facturaAbono.codigoTienda)  and ",
			    " (albaranes.codigoEmpresa = facturaAbono.codigoEmpresa)  and ",
			    " (albaranes.codigoReparacion=reparacion.codigoReparacion)  and ",
			    " (albaranes.codigoTienda=reparacion.codigoTienda)  and ",
			    " (albaranes.codigoEmpresa=reparacion.codigoEmpresa) ",
                            " and facturaabono.estado>",state,
			    gchar_where_enterprise,
                            gchar_where_enterprise2,
                            gchar_where_initial_date,gchar_where_ending_date,
			    "  order by ",
			    " facturaabono.numeroFactura)  union (SELECT facturaabono.numeroFactura ",
 			    " as numeroFacturaFactura,facturaabono.dtoFactura as ",
 			    " descuentoFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
/*			    " estadoFactura,TRUE as ",					*/
			    " estadoFactura,facturaAbono.baseImponibleModificada as ",					
			    " baseImponibleModificadaFactura, ",
			    " to_char(facturaAbono.baseImponibleNueva,'99999999D99') as ",
			    " baseImponibleNuevaFactura, ",
			    "   facturaAbono.fechaEmision as fechaEmisionFactura, ",
                            " participanteCliente.nick as nickParticipantPagador,"
			    " participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador, "
/*			    " NULL as codigoReparacionRepair, ",			*/
			    " reparacion.codigoReparacion as codigoReparacionRepair, ",	
			    " reparacion.codigoTienda as ",
			    " codigoTiendaRepair, reparacion.codigoEmpresa as codigoEmpresaRepair, ",
			    " tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as codigoPartyParticipantEnter ",
			    "  from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono ",
			    " ,  (select distinct ",
			    " facturaabono.numerofactura as ",
			    " numeroFacturaFactura, ",
			    " facturaabono.codigoFactura as numeroFactura, albaranabono.numeroFactura as numeroFacturaAlbaran, ",
			    " codigoreparacionTaller,facturatalleres.codigoEmpresaAlbaran as ",
			    " codigoEMpresaAlbaranTaller,facturatalleres.codigoTiendaAlbaran as ",
			    " codigoTiendaAlbaranTaller from facturaabono  left outer join ",
			    " albaranabono on ((facturaabono.codigoFactura=albaranabono.numeroFactura) ",
                            " and (albaranabono.codigoTienda=facturaabono.codigoTienda) and (albaranabono.codigoEmpresa=facturaabono.codigoEmpresa)),",
			    " (select codigoReparacion as codigoReparacionTaller,numeroFacturaTaller ",
			    " as numeroFacturaTaller  ,codigoEmpresaFactura,codigoTiendaFactura, ",
			    " codigoEmpresaAlbaran,codigoTiendaAlbaran  from ",
			    " facturataller,albaranabono where ",
			    " albaranabono.numeroAlbaran=facturaTaller.numeroalbaran and albaranabono.codigoEmpresa=facturaTaller.codigoEmpresaAlbaran ",
			    " and albaranabono.codigoTienda=facturaTaller.codigoTiendaAlbaran)  as ",
			    " facturaTalleres  where (",
			    " facturaabono.codigoFactura=numeroFacturaTaller and ",
                            " facturaabono.tipoIva is NULL and ",
			    " facturaabono.codigoTienda=facturaTalleres.codigoTiendaFactura  and ",
			    " facturaabono.codigoEmpresa=facturaTalleres.codigoEmpresaFactura )) as ",
/*			    " facturasTalleres  where (facturaabono.numeroFactura is not null)  and ",	*/
			    " facturasTalleres  where ",						
			    " facturaabono.codigoTienda=participanteShop.codigoParty  and ",
			    " facturaabono.codigoTienda=reparacion.codigoTienda  and ",
			    " facturaabono.codigoEmpresa=reparacion.codigoEmpresa  and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty  and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
			    " facturaabono.codigoFactura=facturasTalleres.numeroFactura and ",		
			    " reparacion.codigoReparacion=facturasTalleres.codigoReparacionTaller and ",
			    " reparacion.codigoTienda = facturasTalleres.codigoTiendaAlbaranTaller ",
			    " and  reparacion.codigoEmpresa = ",
			    " facturasTalleres.codigoEmpresaAlbaranTaller",
                            " and facturaabono.estado>",state,
                            gchar_where_enterprise,
                            gchar_where_initial_date,gchar_where_ending_date,
			    " group by ",
			    " facturaabono.numeroFactura, facturaabono.codigoempresa, facturaabono.dtoFactura, ",
			    " facturaabono.codigotienda, facturaabono.estado, ",
			    " facturaabono.baseimponiblemodificada, ",
/*			    " to_char(facturaAbono.baseImponibleNueva,'99999999D999999'), ",	*/
			    " facturaAbono.baseImponibleNueva, ",
			    " facturaabono.fechaEmision, ",
                            " participanteCliente.nick,"
			    " participantecliente.nombreCorto, ",
			    " reparacion.codigotienda,reparacion.codigoEmpresa,tienda.codigoPertenenciaGrupo ",
			    "  order by facturaabono.numeroFactura) ",
			    " union ( (SELECT facturaabono.numeroFactura as ",
 			    " numeroFacturaFactura,facturaabono.dtoFactura as ",
 			    " descuentoFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
			    " estadoFactura,facturaAbono.baseImponibleModificada as ",
			    " baseImponibleModificadaFactura, ",
 			    " to_char(facturaAbono.baseImponibleNueva,'99999999D999999') as ",
			    " 	 baseImponibleNuevaFactura, ",
			    "   facturaAbono.fechaEmision as fechaEmisionFactura, ",
                            " participanteCliente.nick as nickParticipantPagador,"
			    " 	participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador,reparacion.codigoReparacion as ",
			    " codigoReparacionRepair,reparacion.codigoTienda as ",
			    " codigoTiendaRepair, reparacion.codigoEmpresa as codigoEmpresaRepair, " ,
			    " tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as ",
			    " codigoPartyParticipantEnter ",
			    "  from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono ",
			    " , (select numeroRefactura as ",
			    " numeroFactura,refacturacion.codigoReparacion as codigoReparacionRefactura, ",
			    " codigoEmpresaRefactura,codigoTiendaRefactura,codigoEmpresaAbono, ",
			    " codigoTiendaAbono from refacturacion,facturaabono  where ",
			    " numeroRefactura=facturaabono.codigofactura and ",
			    " codigoEmpresaRefactura=facturaabono.codigoEmpresa and ",
			    " codigoTiendaRefactura=facturaabono.codigoTienda ",
			    gchar_where_enterprise_ref,
			    " ) as refacturas  where ",
			    " facturaabono.codigoTienda=participanteShop.codigoParty and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
                            " facturaabono.tipoIva is NULL and ",
			    " (refacturas.numeroFactura = facturaAbono.codigoFactura)  and ",
			    " (refacturas.codigoReparacionRefactura=reparacion.codigoReparacion)  and ",
			    " (refacturas.codigoEmpresaRefactura=facturaabono.codigoEmpresa)  and ",
			    " (refacturas.codigoTiendaRefactura=facturaabono.codigoTienda)  and ",
			    " (refacturas.codigoEmpresaAbono=reparacion.codigoEmpresa)  and ",
			    " (refacturas.codigoTiendaAbono=reparacion.codigoTienda) ",
                            " and facturaabono.estado>",state,
                            gchar_where_enterprise_ref2,
                            gchar_where_initial_date,gchar_where_ending_date,
			    " order by facturaabono.numeroFactura))) as FACTURASEMITIDAS ",
/*
			    " left outer join ",
			    " lineaReparacion on ( ",
                            " LineaReparacion.codigoReparacion = FACTURASEMITIDAS.codigoReparacionRepair and ",
			    " LineaReparacion.codigoEmpresa = FACTURASEMITIDAS.codigoEmpresaRepair and ",
			    " LineaReparacion.codigoTienda = FACTURASEMITIDAS.codigoTiendaRepair )",
			    NULL);
			    			 
    gchar_where=g_strconcat(" where lineaReparacion.codigoProducto = Producto.codigoProducto ",
			    NULL);
*/
			    NULL);
    gchar_where=g_strconcat(" where ", 
    			    " lineaReparacion.codigoProducto = Producto.codigoProducto and ",
//                            " LineaReparacion.codigoReparacion = FACTURASEMITIDAS.codigoReparacionRepair and ",
			    " LineaReparacion.codigoEmpresa = FACTURASEMITIDAS.codigoEmpresaRepair and ",
			    " LineaReparacion.codigoTienda = FACTURASEMITIDAS.codigoTiendaRepair ",
			    NULL);			   

    if (((enterprise_code) != NULL) && (strlen(enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (FACTURASEMITIDAS.codigoEmpresaFactura=",
				      enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->initial_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturasEmitidas.fechaEmisionFactura>='",
				      condition->initial_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->ending_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturasEmitidas.fechaEmisionFactura<='",
				      condition->ending_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    gchar_order = g_strconcat(" ORDER BY FECHAEMISIONLIBROEMITIDAS,NUMEROFACTURALIBROEMITIDAS ",NULL);
    
    gchar_groupby= g_strconcat(" group by  NUMEROFACTURALIBROEMITIDAS, ",
			       " FECHAEMISIONLIBROEMITIDAS, RAZONSOCIALLIBROEMITIDAS, ",
			       " MODIFICADALIBROEMITIDAS, descuentoFacturaLibroEmitidas, ",
			       " TOTALLIBROEMITIDAS,codigoPertenenciaLibroEmitidas, ",
			       " tipoIvaLibroEmitidas,totalLibroEmitidas ",NULL);
    gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where, gchar_groupby,gchar_order,NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {

	    issued_books = (IssuedBook **) g_malloc0(sizeof(IssuedBook*)*(*tuples+2));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		issued_book = m_issued_book_create (record_set);
		issued_books[index-1] = issued_book;
		    index = gda_recordset_move_next(record_set);
	    }
	    issued_books[*tuples]=NULL;
	} else {
		issued_books=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	issued_books=NULL;
    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(gchar_where);
    g_free(gchar_where_enterprise);
    g_free(gchar_where_enterprise2);
    g_free(gchar_where_enterprise_ref);
    g_free(gchar_where_enterprise_ref2);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_order);
    g_free(gchar_groupby);
    g_free(state);
    return issued_books;
    
}


/**
 * m_issued_books_group_by_vat:
 * @condition: Condicin de bsqueda. Permite buscar por empresa y perodo de contabilidad.
 * @tuples: Nmero de tuplas que se devolvern de la base de datos. Es necesario reservar memoria antes
 * de llamar a la funcin.
 * 
 * Agrupa por iva las facturas emitidas dentro de un perodo de contabilidad.
 *
 * Returns:
 * Array con las agrupaciones por iva de las facturas emitidas.
 */
IssuedBook **  m_issued_books_group_by_vat (ConditionAccountingPeriod *condition,gulong * tuples) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where_enterprise=NULL;
    gchar              *gchar_where_enterprise_ref2=NULL;
    gchar              *gchar_where_initial_date=NULL;
    gchar              *gchar_where_ending_date=NULL;
    gchar              *gchar_where_enterprise2=NULL;
    gchar              *gchar_where_enterprise_ref=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_groupby=NULL;
    gchar              *gchar_order=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar * state = g_strdup_printf("%d",D_INVOICE_NOT_ISSUED);
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * enterprise_code = FALSE;
    IssuedBook ** issued_books;
    IssuedBook * issued_book;
    gint index;

    my_command = gda_command_new();


    gda_command_set_connection(my_command,my_connection);

    if ((condition->enterprise_code!=ULONG_MAX) && (condition->enterprise_code!=0))
        enterprise_code = g_strdup_printf("%lu",condition->enterprise_code);

    if (((enterprise_code) != NULL) && ((enterprise_code)!=0)) {
	gchar_where_enterprise = g_strconcat(" and (facturaAbono.codigoEmpresa=",
					  enterprise_code,") and (reparacion.codigoEmpresa=",enterprise_code
					  ,") and (tienda.codigoEmpresa=",enterprise_code,")",NULL);
	gchar_where_enterprise2 = g_strconcat(" and (albaranes.codigoEmpresa=",
					  enterprise_code,")",NULL);
	gchar_where_enterprise_ref = g_strconcat(" and (refacturacion.codigoEmpresaReFactura=",
					      enterprise_code,")",NULL);
        gchar_where_enterprise_ref2= g_strconcat(" and (facturaAbono.codigoEmpresa=",
					  enterprise_code,")  and (tienda.codigoEmpresa=",enterprise_code,")",NULL);
    } else {
	gchar_where_enterprise = g_strdup(" ");
	gchar_where_enterprise2 = g_strdup(" ");
	gchar_where_enterprise_ref = g_strdup(" ");
    }

    if ((condition->initial_date) != NULL) {
	gchar_where_initial_date=  g_strconcat(" and (facturaAbono.fechaEmision>='",
				      condition->initial_date,"')",NULL);

    }
    if ((condition->ending_date) != NULL) {
	gchar_where_ending_date=  g_strconcat(" and (facturaAbono.fechaEmision<='",
				      condition->ending_date,"')",NULL);
    }

/*
    gchar_select=g_strconcat("SELECT tipoIvaLibroE as tipoIvaLibroEmitidas,to_char(SUM(CASE WHEN (modificadaLibroE=FALSE) THEN ",
			     " to_number(subtotalLibroE,'99999999D99') ELSE ((totalLibroE)/(1+",
			     "((to_number(tipoIvaLibroE,'99999999D99'))/100))) END),'99999999D99')",
			     " as subtotalLineas, ",
			     " to_char(SUM(CASE WHEN (modificadaLibroE=TRUE) ",
			     " THEN totalLibroE ",
			     " ELSE CAST(to_number(subtotalLibroE,'99999999D99')*(1+(to_number(tipoIvaLibroE,'99999999D99')/100)) as DECIMAL(8,2) ) ",
			     "   	 END),'99999999D99') as totalLibroEmitidas ",
			     " FROM (SELECT FACTURASEMITIDAS.NUMEROFACTURAFACTURA as numeroFacturaLibroE, ",
			     " FACTURASEMITIDAS.FECHAEMISIONFACTURA as fechaEMisionLibroE, ",
			     " FACTURASEMITIDAS.NICKPARTICIPANTPAGADOR as razonSocialLibroE, ",
			     " FACTURASEMITIDAS.baseImponibleModificadaFactura as modificadaLibroE, ",
			     " to_number(FACTURASEMITIDAS.baseImponibleNuevaFactura,'99999999D99') as totalLibroE, ",
			     " FACTURASEMITIDAS.codigoPertenenciaGrupoShop as codigoPertenenciaGrupoLibroE, ",
			     " to_char(lineaReparacion.tipoIVAHistorico,'999D99') as tipoIvaLibroE, ",
			     " to_char(SUM(CASE WHEN (LineaReparacion.precio IS NOT NULL) THEN ",
			     " (LineaReparacion.precio) WHEN ((LineaReparacion.precioHistorico IS NOT ",
			     " NULL) AND (LineaReparacion.dto is NOT NULL)) THEN ",
			     " ((LineaReparacion.unidades*Lineareparacion.precioHistorico)- ",
			     " (LineaReparacion.unidades*(LineaReparacion.dto/100)* ",
			     " LineaReparacion.precioHistorico)) WHEN ",
			     " ((LineaReparacion.precioHistorico IS NOT NULL) AND (LineaReparacion.dto ",
			     " IS NULL)) THEN ",
			     " (LineaReparacion.unidades*LineaReparacion.precioHistorico) ELSE 0.0 ",
			     " END),'99999999D99') as subtotalLibroE ",NULL);
*/
    gchar_select=g_strconcat("SELECT tipoIvaLibroE as tipoIvaLibroEmitidas,to_char(SUM(CAST((CASE WHEN (modificadaLibroE=FALSE AND ",
			     " descuentoFacturaLibroE is NULL) THEN to_number(subtotalLibroE,'99999999D999999') ",
			     " WHEN (modificadaLibroE=FALSE AND descuentoFacturaLibroE is not NULL) ",
			     " THEN to_number(subtotalLibroE,'99999999D999999') - (to_number(subtotalLibroE,'99999999D999999')",
			     " *descuentoFacturaLibroE/100) ELSE ((totalLibroE)/(1+",
			     "((to_number(tipoIvaLibroE,'99999999D999999'))/100))) END) as DECIMAL(10,2))),'99999999D999999')",
			     " as subtotalLineas,		 ",
			     " to_char(SUM(CASE WHEN (modificadaLibroE=TRUE) ",
			     " THEN totalLibroE ",
			     " WHEN (descuentoFacturaLibroE is not NULL) ",
			     " THEN CAST((to_number(subtotalLibroE,'99999999D999999')-(to_number(subtotalLibroE,'99999999D999999')*descuentoFacturaLibroE/100))"
			     " *(1+(to_number(tipoIvaLibroE,'99999999D999999')/100)) as DECIMAL(8,2))  ",			     
			     " ELSE CAST(to_number(subtotalLibroE,'99999999D999999')*(1+(to_number(tipoIvaLibroE,'99999999D999999')/100)) as DECIMAL(8,2) ) ",
			     "   	 END),'99999999D999999') as totalLibroEmitidas ",
			     " FROM (SELECT FACTURASEMITIDAS.NUMEROFACTURAFACTURA as numeroFacturaLibroE, ",
			     " FACTURASEMITIDAS.FECHAEMISIONFACTURA as fechaEMisionLibroE, ",
			     " FACTURASEMITIDAS.descuentoFactura as descuentoFacturaLibroE, ",			     
			     " FACTURASEMITIDAS.NICKPARTICIPANTPAGADOR as razonSocialLibroE, ",
			     " FACTURASEMITIDAS.baseImponibleModificadaFactura as modificadaLibroE, ",
			     " to_number(FACTURASEMITIDAS.baseImponibleNuevaFactura,'99999999D999999') as totalLibroE, ",
/*			     " FACTURASEMITIDAS.codigoAutoArteShop as codigoAutoArteLibroE, ",		*/
			     " FACTURASEMITIDAS.codigoPertenenciaGrupoShop as codigoPertenenciaGrupoLibroE, ",
			     " to_char(Producto.tipoIVA,'999D999999') as tipoIvaLibroE, ",
			     " to_char(SUM(CAST((CASE WHEN (LineaReparacion.precio IS NOT NULL) THEN ",
			     " (LineaReparacion.precio) WHEN ((LineaReparacion.precioHistorico IS NOT ",
			     " NULL) AND (LineaReparacion.dto is NOT NULL)) THEN ",
			     " ((LineaReparacion.unidades*Lineareparacion.precioHistorico)- ",
			     " (LineaReparacion.unidades*(LineaReparacion.dto/100)* ",
			     " LineaReparacion.precioHistorico)) WHEN ",
			     " ((LineaReparacion.precioHistorico IS NOT NULL) AND (LineaReparacion.dto ",
			     " IS NULL)) THEN ",
			     " (LineaReparacion.unidades*LineaReparacion.precioHistorico) ELSE 0.0 ",
			     " END) as DECIMAL(10,2))),'99999999D999999') as subtotalLibroE ",NULL);

/*			     " to_char(SUM(CASE WHEN (modificadaLibroE=TRUE) THEN (totalLibroE ",
			     "- CAST((totalLibroE/(1+(to_number(tipoIvaLibroE,'99999999D99')/100))) as DECIMAL(10,2))) ELSE ",
			     " (to_number(subtotalLibroE,'99999999D99')*(to_number(tipoIvaLibroE,'99999999D99')/100)) ",
			     " END),'99999999D99') as totalLibroEmitidas ",
*/

    gchar_from= g_strconcat("FROM LineaReparacion,producto,",
			    "((SELECT facturaabono.numeroFactura as ",
			    " numeroFacturaFactura,facturaabono.dtoFactura as ",
			    " descuentoFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
			    " estadoFactura,facturaAbono.baseImponibleModificada as ",
			    " baseImponibleModificadaFactura, ",
			    " to_char(facturaAbono.baseImponibleNueva,'99999999D99') as ",
			    " baseImponibleNuevaFactura, ",
			    " facturaAbono.fechaEmision as fechaEmisionFactura, ",
			    " participanteCliente.nick as nickParticipantPagador,"
			    " participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador,reparacion.codigoReparacion as ",
			    " codigoReparacionRepair,reparacion.codigoTienda as ",
			    " codigoTiendaRepair,reparacion.codigoEmpresa as codigoEmpresaRepair,tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as ",
			    " codigoPartyParticipantEnter ",
			    " from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono where ",
			    " (facturaabono.numeroFactura is not null)  and ",
                            " facturaabono.tipoIva is NULL and ",
			    " facturaabono.codigoTienda=participanteShop.codigoParty  and ",
			    " facturaabono.codigoTienda=reparacion.codigoTienda  and ",
			    " facturaabono.codigoEmpresa=reparacion.codigoEmpresa  and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty  and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
			    " (facturaabono.codigoReparacion = reparacion.codigoReparacion) ",
                            " and facturaabono.estado>",state,
                            gchar_where_enterprise,
                            gchar_where_initial_date,gchar_where_ending_date,
			    " order by ",
			    "  facturaabono.numeroFactura) ",
			    " union (SELECT facturaabono.numeroFactura ",
			    " as numeroFacturaFactura,facturaabono.dtoFactura as ",
			    " descuentoFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
			    " estadoFactura,facturaAbono.baseImponibleModificada as ",
			    " baseImponibleModificadaFactura, ",
			    " to_char(facturaAbono.baseImponibleNueva,'99999999D99') as ",
			    " baseImponibleNuevaFactura, ",
			    "   facturaAbono.fechaEmision as fechaEmisionFactura, ",
			    " participanteCliente.nick as nickParticipantPagador,"
			    "   participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador,(case WHEN reparacion.codigoCompania is not NULL ",
			    "THEN reparacion.codigoReparacion ",
                            " WHEN reparacion.codigoReparacion is not NULL THEN reparacion.codigoReparacion",
			    " ELSE NULL END) as ",
			    " codigoReparacionRepair,reparacion.codigoTienda as ",
			    " codigoTiendaRepair,reparacion.codigoEmpresa as codigoEmpresaRepair,tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as ",
			    " codigoPartyParticipantEnter ",
			    " from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono, (select ",
			    " a.codigofactura,albaranabono.codigoReparacion,a.codigoTienda, ",
			    " a.codigoEmpresa from albaranabono,facturaabono a  where ",
			    " albaranabono.numeroFactura=a.codigoFactura and ",
			    " a.codigoEmpresa=albaranAbono.codigoEmpresa and ",
			    " a.codigoTienda=albaranabono.codigoTienda) as albaranes  where ",
			    " (facturaabono.numeroFactura is not null)  and ",
                            " facturaabono.tipoIva is NULL and ",
			    " facturaabono.codigoTienda=participanteShop.codigoParty and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
			    " (albaranes.codigoFactura = facturaAbono.codigoFactura)  and ",
			    " (albaranes.codigoTienda = facturaAbono.codigoTienda)  and ",
			    " (albaranes.codigoEmpresa = facturaAbono.codigoEmpresa)  and ",
			    " (albaranes.codigoReparacion=reparacion.codigoReparacion)  and ",
			    " (albaranes.codigoTienda=reparacion.codigoTienda)  and ",
			    " (albaranes.codigoEmpresa=reparacion.codigoEmpresa) ",
                            " and facturaabono.estado>",state,
			    gchar_where_enterprise,
                            gchar_where_enterprise2,
                            gchar_where_initial_date,gchar_where_ending_date,
			    "  order by ",
			    " facturaabono.numeroFactura)  union (SELECT facturaabono.numeroFactura ",
			    " as numeroFacturaFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.dtoFactura as ",
			    " descuentoFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
			    " estadoFactura,TRUE as ",
			    " baseImponibleModificadaFactura, ",
			    " to_char(facturaAbono.baseImponibleNueva,'99999999D99') as ",
			    " baseImponibleNuevaFactura, ",
			    "   facturaAbono.fechaEmision as fechaEmisionFactura, ",
			    " participanteCliente.nick as nickParticipantPagador,"
			    " participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador, "
 			    " NULL as codigoReparacionRepair, ", 
/*			    " reparacion.codigoReparacion as codigoReparacionRepair, ",	*/
			    " reparacion.codigoTienda as ",
			    " codigoTiendaRepair,reparacion.codigoEmpresa as codigoEmpresaRepair,tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as codigoPartyParticipantEnter ",
			    "  from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono ",
			    " ,  (select distinct ",
			    " facturaabono.codigofactura as ",
			    " numeroFacturaFactura, ",
			    " facturaabono.codigoFactura as numeroFactura, albaranabono.numeroFactura as numeroFacturaAlbaran, ",
			    " codigoreparacionTaller,facturatalleres.codigoEmpresaAlbaran as ",
			    " codigoEMpresaAlbaranTaller,facturatalleres.codigoTiendaAlbaran as ",
			    " codigoTiendaAlbaranTaller from facturaabono  left outer join ",
			    " albaranabono on ((facturaabono.codigoFactura=albaranabono.numeroFactura) ",
                            " and (albaranabono.codigoTienda=facturaabono.codigoTienda) and (albaranabono.codigoEmpresa=facturaabono.codigoEmpresa)),",
			    " (select codigoReparacion as codigoReparacionTaller,numeroFacturaTaller ",
			    " as numeroFacturaTaller  ,codigoEmpresaFactura,codigoTiendaFactura, ",
			    " codigoEmpresaAlbaran,codigoTiendaAlbaran  from ",
			    " facturataller,albaranabono where ",
			    " albaranabono.numeroAlbaran=facturaTaller.numeroalbaran and albaranabono.codigoEmpresa=facturaTaller.codigoEmpresaAlbaran ",
			    " and albaranabono.codigoTienda=facturaTaller.codigoTiendaAlbaran)  as ",
			    " facturaTalleres  where (facturaabono.numeroFactura is not null)  and ",	
/*			    " facturaTalleres  where ( ",						*/
			    " (facturaabono.codigoFactura=numeroFacturaTaller and ",
                            " facturaabono.tipoIva is NULL and ",
			    " facturaabono.codigoTienda=facturaTalleres.codigoTiendaFactura  and ",
			    " facturaabono.codigoEmpresa=facturaTalleres.codigoEmpresaFactura )) as ",
			    " facturasTalleres  where (facturaabono.numeroFactura is not null)  and ",
			    " facturaabono.codigoTienda=participanteShop.codigoParty  and ",
			    " facturaabono.codigoTienda=reparacion.codigoTienda  and ",
			    " facturaabono.codigoEmpresa=reparacion.codigoEmpresa  and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty  and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
			    " facturaabono.codigoFactura=facturasTalleres.numeroFactura and ",
			    " reparacion.codigoReparacion=facturasTalleres.codigoReparacionTaller and ",
			    " reparacion.codigoTienda = facturasTalleres.codigoTiendaAlbaranTaller ",
			    " and  reparacion.codigoEmpresa = ",
			    " facturasTalleres.codigoEmpresaAlbaranTaller",
                            " and facturaabono.estado>",state,
                            gchar_where_enterprise,
                            gchar_where_initial_date,gchar_where_ending_date,
			    " group by ",
			    " facturaabono.numeroFactura,facturaabono.codigoempresa, ",
			    " facturaabono.codigotienda, facturaabono.estado,  facturaabono.dtoFactura,",
			    " facturaabono.baseimponiblemodificada,",
			    " facturaabono.baseimponiblenueva, ",
			    " facturaabono.fechaEmision, ",
			    " participanteCliente.nick,"
			    " participantecliente.nombreCorto, ",
			    " reparacion.codigotienda,reparacion.codigoEmpresa,tienda.codigoPertenenciaGrupo ",
			    "  order by facturaabono.numeroFactura) ",
			    " union ( (SELECT facturaabono.numeroFactura as ",
			    " numeroFacturaFactura,facturaabono.dtoFactura as ",
			    " descuentoFactura,facturaabono.codigoEmpresa as ",
			    " codigoEmpresaFactura,facturaabono.codigoTienda as ",
			    " codigoTiendaFactura,facturaabono.estado as ",
			    " estadoFactura,facturaAbono.baseImponibleModificada as ",
			    " baseImponibleModificadaFactura, ",
			    " to_char(facturaAbono.baseImponibleNueva,'99999999D99') as ",
			    " 	 baseImponibleNuevaFactura, ",
			    "   facturaAbono.fechaEmision as fechaEmisionFactura,",
			    " participanteCliente.nick as nickParticipantPagador,"
			    " 	 participanteCliente.nombreCorto as ",
			    " shortNameParticipantPagador,reparacion.codigoReparacion as ",
			    " codigoReparacionRepair,reparacion.codigoTienda as ",
			    " codigoTiendaRepair,reparacion.codigoEmpresa as codigoEmpresaRepair,tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop, ",
			    " facturaAbono.codigoEmpresa as ",
			    " codigoPartyParticipantEnter ",
			    "  from reparacion, ",
			    " participante participanteCliente, participante participanteShop, ",
			    " tienda, facturaabono ",
			    " , (select numeroRefactura as ",
			    " numeroFactura,refacturacion.codigoReparacion as codigoReparacionRefactura, ",
			    " codigoEmpresaRefactura,codigoTiendaRefactura,codigoEmpresaAbono, ",
			    " codigoTiendaAbono from refacturacion,facturaabono  where ",
			    " numeroRefactura=facturaabono.codigofactura and ",
			    " codigoEmpresaRefactura=facturaabono.codigoEmpresa and ",
			    " codigoTiendaRefactura=facturaabono.codigoTienda ",
			    gchar_where_enterprise_ref,
			    " ) as refacturas  where ",
			    " facturaabono.codigoTienda=participanteShop.codigoParty and ",
			    " facturaabono.codigoPagador = participanteCliente.codigoParty and ",
			    " tienda.codigoTienda = facturaabono.codigoTienda  and ",
                            " facturaabono.tipoIva is NULL and ",
			    " (refacturas.numeroFactura = facturaAbono.codigoFactura)  and ",
			    " (refacturas.codigoReparacionRefactura=reparacion.codigoReparacion)  and ",
			    " (refacturas.codigoEmpresaRefactura=facturaabono.codigoEmpresa)  and ",
			    " (refacturas.codigoTiendaRefactura=facturaabono.codigoTienda)  and ",
			    " (refacturas.codigoEmpresaAbono=reparacion.codigoEmpresa)  and ",
			    " (reparacion.codigoEmpresa=refacturas.CodigoEmpresaAbono) and ",
                            " (reparacion.codigoTienda=refacturas.codigoTiendaAbono) and ",
			    " (refacturas.codigoTiendaAbono=reparacion.codigoTienda) ",
                            " and facturaabono.estado>",state,
                            gchar_where_enterprise_ref2,
                            gchar_where_initial_date,gchar_where_ending_date,
			    " order by facturaabono.numeroFactura))) as FACTURASEMITIDAS ",NULL);

    gchar_where=g_strconcat(" where lineaReparacion.codigoProducto = Producto.codigoProducto and ",
                            " LineaReparacion.codigoReparacion = FACTURASEMITIDAS.codigoReparacionRepair and ",
			    " LineaReparacion.codigoEmpresa = FACTURASEMITIDAS.codigoEmpresaRepair and ",
			    " LineaReparacion.codigoTienda = FACTURASEMITIDAS.codigoTiendaRepair ",
			    NULL);

    if (((enterprise_code) != NULL) && (strlen(enterprise_code)!=0)) {
	gchar_where_aux = g_strconcat(gchar_where, " and (FACTURASEMITIDAS.codigoEmpresaFactura=",
				      enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->initial_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturasEmitidas.fechaEmisionFactura>='",
				      condition->initial_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }
    if ((condition->ending_date) != NULL) {
	gchar_where_aux = g_strconcat(gchar_where, " and (facturasEmitidas.fechaEmisionFactura<='",
				      condition->ending_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
    }


    gchar_order = g_strconcat(" ORDER BY FECHAEMISIONLIBROE,NUMEROFACTURALIBROE) as facturasE group by tipoIvaLibroE",NULL);
//    gchar_order = g_strconcat(" ORDER BY FECHAEMISIONLIBROE,NUMEROFACTURALIBROE) as facturasE ",NULL);
    gchar_groupby= g_strconcat(" group by  NUMEROFACTURALIBROE, ",
			       " FECHAEMISIONLIBROE, RAZONSOCIALLIBROE, descuentoFacturaLibroE,",
			       " MODIFICADALIBROE, ",
			       " TOTALLIBROE,codigoPertenenciaGrupoLibroE, ",
			       " tipoIvaLibroE,totalLibroE ",NULL);
    gchar_query = g_strconcat(gchar_select,gchar_from, gchar_where, gchar_groupby,gchar_order,NULL);

    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection,
			  my_command,tuples,0, TRUE);


    if (record_set!=NULL) {
	if (*tuples!=0) {

	    issued_books = (IssuedBook **) g_malloc0(sizeof(IssuedBook*)*(*tuples+2));
	    index = gda_recordset_move_first(record_set);
	    while (!gda_recordset_eof(record_set)) {
		if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		issued_book = m_issued_book_create (record_set);
                d_issued_book_set_vat_summary (issued_book,TRUE);
		issued_books[index-1] = issued_book;
		    index = gda_recordset_move_next(record_set);
	    }
	    issued_books[*tuples]=NULL;
	} else {
		issued_books=NULL;
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
	issued_books=NULL;
    }

    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    g_free(gchar_where);
    g_free(gchar_where_enterprise);
    g_free(gchar_where_enterprise2);
    g_free(gchar_where_enterprise_ref);
    g_free(gchar_where_enterprise_ref2);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_order);
    g_free(gchar_groupby);
    g_free(state);
    return issued_books;
    
}




/**
 * m_issued_books_clear_array:
 * @issued_books: Array a liberar.
 * 
 * Libera la memoria asignada a la lista de objetos IssuedBook.
 *
 */
void m_issued_books_clear_array  (IssuedBook ** issued_books) {
    if (issued_books!=NULL) {
	gint index = 0;
	while (issued_books[index]!=NULL) {
	    d_issued_book_clear (issued_books[index]);
	    index++;
	}
    }
    g_free(issued_books);

}
