/* model.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"

#ifndef _MODEL_
#define _MODEL_

#ifndef _GDA_LIBRARY_
#define _GDA_LIBRARY_
#include <gda-client.h>
#endif

#ifndef _GNOME_LIBRARY_
#define _GNOME_LIBRARY_
#include <gnome.h>
#endif

#ifndef _GTK_LIBRARY_
#define _GTK_LIBRARY_
#include <gtk/gtk.h>
#endif

#define IS_FIELD_NULL(field) ((field!=NULL) && (!gda_field_is_null(field)))


void m_model_list_tables (GdaRecordset * rs);

gchar *m_model_list_providers (void);

gint m_model_init (int argc, char *argv[]);

GdaConnection * m_model_get_connection (void);

void  m_model_destroy_connection (void);

// Begin a transaction blocking the log file
void m_model_connection_begin_mutex_transaction_logged (GdaConnection *cnc,
						     gulong enterprise_code,
						     gulong shop_code,
						     gulong warehouse_code);

// End a transaction and free the log file
// Returns TRUE if all OK
gboolean m_model_execute_mutex_modification_logged (GdaConnection * connection);
// Do a NO-transactional command without block the log file.
// Use in a transaction or with a readonly command.
// Returns TRUE if all OK
gboolean m_model_command_execute (GdaRecordset **record_set,
                               gchar *query,
			       GdaConnection *cnc,
			       GdaCommand *cmd,
			       gulong *reccount, gulong flags,
			       gboolean readonly);

// Do a transactional command blocking the log file.
// Returns TRUE if all OK
gboolean m_model_transactional_command_execute (GdaRecordset **record_set,
						   gchar *query,
						   GdaConnection *cnc,
						   GdaCommand *cmd,
						   gulong *reccount,
						   gulong flags, gboolean readonly);

// Do a transactional command blocking the log file.
// Do not use in a transaction!!! =>
//      They share the same semaphore over the log file.
// Returns TRUE if all OK
gboolean m_model_mutex_transactional_command_execute_logged (GdaRecordset **record_set,
						   gchar *query,
						   GdaConnection *cnc,
						   GdaCommand *cmd,
						   gulong *reccount,
						   gulong flags, gboolean readonly,
						   gulong enterprise_code,gulong shop_code,gulong warehouse_code);

gboolean m_model_record_set_contains_piece (GdaRecordset *rs);
gboolean m_model_record_set_contains_man_power (GdaRecordset *rs);

gchar *m_model_nextval_sequence (const gchar *sequence, GdaConnection *connection);
gchar *m_model_currval_sequence (const gchar *sequence, GdaConnection *connection);

#endif


