/* particular_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "particular_model.h"
#include "participant_model.h"

#define TABLE         "Particular"
#define IDENTIFICATOR "codigoParty"
#define ATTRIBUTES    "primerApellido as primerapellidopartic, \
                       segundoApellido as segundoapellidopartic"

/**
 * d_particular_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *d_particular_get_table () {
    return TABLE;
}

/**
 * d_particular_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *d_particular_get_attributes () {
    return ATTRIBUTES;
}


/* FIXME: Compatibility function */
/**
 * d_particular_create_partial:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Ahora mismo crea un participante completo. !!OJO NO UTILIZAR.
 *
 * Returns:
 * Un objeto #Particular con los campos rellenados con los datos de la base de datos.
 */
ParticularObject * d_particular_create_partial (GdaRecordset *rs) {
   
    return d_particular_create (rs);

}

/**
 * d_particular_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Particular con los campos rellenados con los datos de la base de datos.
 */
ParticularObject * d_particular_create (GdaRecordset *rs) {

    gchar  *temp;
    GdaField * field;
    ParticularObject * particular;

    particular = d_particular_create_empty();

    particular->participant = m_participant_create (rs,"partic");

    if (d_participant_get_code (d_particular_get_participant (particular)) != ULONG_MAX) {

	field = gda_recordset_field_name(rs,"primerapellidopartic");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		temp = gda_stringify_value(NULL, 0, field);
		particular->first_surname = g_strdup(temp);
                g_free(temp);
	    }
	    gda_field_free(field);
	}


	field = gda_recordset_field_name(rs,"segundoapellidopartic");
	if (field!=NULL) {
	    if (!gda_field_is_null(field)) {
		temp = gda_stringify_value(NULL, 0, field);
		particular->second_surname = g_strdup(temp);
                g_free(temp);
	    }
	    gda_field_free(field);
	}
    }

    return particular;
}

