
/* policy_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#define M_FIRST_ERROR_MESSAGE "Error en los datos de la POLIZA"

#define TABLE         "Poliza"
#define ATTRIBUTES    "codigoCompania as codigocompaniapolicy , \
                       numeroPoliza as numeropolizapolicy , \
                       codigoTitular as codigotitularpolicy"

#include "policy_model.h"
#include "util.h"


/**
 * m_policy_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_policy_get_table () {
    return TABLE;
}

/**
 * m_policy_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_policy_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_policy_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Policy con los campos rellenados con los datos de la base de datos.
 */
Policy *m_policy_create (GdaRecordset *rs) {
    GdaField *field;
    gchar    *temp;
    Policy *policy = (Policy *) g_malloc (sizeof(Policy));

    d_policy_init (policy);

    field = gda_recordset_field_name(rs,"codigocompanypolicy");
    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    policy->company_code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"numeropolizapolicy");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    policy->policy_number = g_strdup(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigotitularpolicy");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    policy->holder_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigocompanyapolicy");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    policy->old_company_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"numeropolizaapolicy");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    policy->old_policy_number = g_strdup(temp);
                    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"codigotitularapolicy");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    policy->old_holder_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	} else {
	    gda_field_free(field);
	}
    }

    return policy;
}


/**
 * m_policy_find_by_name:
 * @policy_code: Cdigo de la pliza que se desea buscar.
 * @party_code:  Cdigo del participante del que se pretende buscar la pliza.
 * @client: Cdigo del titular de la pliza.
 *
 * Busca las plizas que pertenezcan a un cliente o con el mismo nmero de pliza, o
 * por los dos atributos simultneamente.
 *
 * Returns:
 * La lista de plizas de la base de datos.
 */
Policy ** m_policy_find_by_name (gchar *policy_code, gchar *party_code, gchar *client) {

  Policy ** policies_list = NULL; 
  GdaCommand         * my_command;
  GdaConnection      * my_connection = m_model_get_connection ();
  GdaRecordset       * record_set = NULL;
  Policy * policy;
  gchar * gchar_query = NULL;
  gulong * tuples;
  gint * column;
  gint index;
  gint length_of_list;
  
  my_command = gda_command_new();
  gda_command_set_connection(my_command,my_connection);
    
  if (policy_code == NULL){
    gchar_query = g_strconcat("SELECT ",
			      "poliza.codigocompania as codigocompanypolicy,",
			      "poliza.numeropoliza as numeropolizapolicy,",
			      "poliza.codigotitular as codigotitularpolicy,",
			      "poliza.codigocompania as codigocompanyapolicy,",
			      "poliza.numeropoliza as numeropolizaapolicy,",
			      "poliza.codigotitular as codigotitularapolicy",
			      " FROM poliza",
			      ", participante WHERE participante.codigoparty=poliza.codigotitular AND participante.codigoparty='",party_code,"'",NULL);    
  }else {
    gchar_query = g_strconcat("SELECT ",
			      "poliza.codigocompania as codigocompanypolicy,",
			      "poliza.numeropoliza as numeropolizapolicy,",
			      "poliza.codigotitular as codigotitularpolicy,",
			      "poliza.codigocompania as codigocompanyapolicy,",
			      "poliza.numeropoliza as numeropolizaapolicy,",
			      "poliza.codigotitular as codigotitularapolicy",
			      " FROM poliza",
			      NULL);


    if (policy_code != NULL) {
      gchar *gchar_query_aux = NULL;
      
      gchar_query_aux = g_strconcat(gchar_query," WHERE numeropoliza='",policy_code,"'",NULL);
      g_free(gchar_query);
      gchar_query = gchar_query_aux;
      gchar_query_aux = NULL;
    }

    if (client != NULL) {
      gchar *gchar_query_aux = NULL;
      
      gchar_query_aux = g_strconcat(gchar_query," AND codigotitular='",client,"'",NULL);
      g_free(gchar_query);
      gchar_query = gchar_query_aux;
      gchar_query_aux = NULL;
    }
    
  }
  
  tuples = (gulong *) g_malloc0(sizeof(gulong));
  m_model_command_execute (&record_set, gchar_query, my_connection,
			my_command,tuples,0, TRUE);

  g_print(gchar_query);
  g_free(gchar_query);
  
  length_of_list = *tuples;
  column = (gint *) g_malloc0(sizeof(gint));
  
  if (record_set!=NULL) {
    if (*tuples!=0) {
      policies_list = (Policy **) g_malloc0(sizeof(Policy*)*(*tuples+1));
      index = gda_recordset_move_first(record_set);
      while (!gda_recordset_eof(record_set)) {
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	(*column)=0;
	policy = m_policy_create (record_set);
	policies_list[index-1] = policy;
	index = gda_recordset_move_next(record_set);
      }
      policies_list[length_of_list]=NULL;
      
    } else {
      policies_list=NULL;
    }
    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
  } else {
    policies_list=NULL;
  }
  
  igalia_command_free(my_command);
  
  g_free(tuples);
  g_free(column);
  
  return policies_list;
}

/**
 * m_policy_insert:
 * @policy: Pliza de seguros a insertar (o modificar) en la base de datos.
 * @participant_code: Cdigo del cliente al que se le asignar la pliza.
 * @update: Si est a %TRUE modifica la pliza en la base de datos.
 * @command: Comando en el que se ejetutar la sentencia sql.
 * @connection: Conexin a la BD.
 * @exception: Objeto que indicar un error si se produce.
 *
 * Inserta una pliza en la base de datos, o la modifica si update=%TRUE, dentro de
 * una transaccin.
 *
 * Returns:
 * 
 */
gboolean m_policy_insert (Policy *policy, gchar *participant_code, gboolean update, GdaCommand * command, GdaConnection *connection, Exception *exception) {
  
  gboolean result = FALSE;
  
  if(policy != NULL) {
    gchar * query = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    gchar *policy_number = d_policy_get_policy_number (policy);
    gulong company_code_gulong = d_policy_get_company_code (policy);
    gchar *company_code = NULL;   
    gchar *local_participant_code = g_strdup(participant_code);
    
    if (company_code_gulong!=ULONG_MAX)
      company_code = g_strdup_printf("%lu",company_code_gulong);
    else company_code = g_strdup("NULL");
    
    if (policy_number == NULL) policy_number = g_strdup("NULL");
    else{
      gchar *policy_number_aux = NULL;

      policy_number_aux = g_strconcat("'",policy_number,"'",NULL);
      g_free(policy_number);
      policy_number = policy_number_aux;
      policy_number_aux = NULL;
    }

    if (local_participant_code == NULL) local_participant_code = g_strdup("NULL");
    else{
      gchar *local_participant_code_aux = NULL;

      local_participant_code_aux = g_strconcat("'",local_participant_code,"'",NULL);
      g_free(local_participant_code);
      local_participant_code = local_participant_code_aux;
      local_participant_code_aux = NULL;
    }

    if (!update) {
      query = g_strconcat("INSERT INTO ",m_policy_get_table ()," VALUES (",
			  company_code,",",
			  policy_number,",",
			  local_participant_code,
			  ")",
			  NULL);

      result = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);

      if (!result) d_exception_set_message (exception, M_FIRST_ERROR_MESSAGE);

      g_print(query);
    } else {
      gchar *old_policy_number = d_policy_get_old_policy_number (policy);
      gulong old_company_code_gulong = d_policy_get_old_company_code (policy);
      gulong old_participant_gulong = d_policy_get_old_holder_code (policy);
      gchar * old_company_code = NULL;
      gchar * old_participant = NULL;
      
      if (old_company_code_gulong!=ULONG_MAX)
	old_company_code = g_strdup_printf("%lu",old_company_code_gulong);
      else old_company_code = g_strdup("NULL");
      
      if (old_participant_gulong!=ULONG_MAX)
	old_participant = g_strdup_printf("%lu",old_participant_gulong);
      else old_participant = g_strdup("NULL");
    
      if (old_policy_number == NULL) old_policy_number = g_strdup("NULL");
      else {
	gchar *old_policy_number_aux = NULL;
	
	old_policy_number_aux = g_strconcat("'",old_policy_number,"'",NULL);
	g_free(old_policy_number);
	old_policy_number = old_policy_number_aux;
	old_policy_number_aux = NULL;
      }
      
      query = g_strconcat("UPDATE ",m_policy_get_table (),
			  " set codigoCompania=",company_code,
			  ", codigotitular = ",local_participant_code,
			  ", numeroPoliza=", policy_number,
			  " where numeroPoliza=", old_policy_number,
			  " and codigoCompania= ",old_company_code,
			  " and codigotitular = ",old_participant,
			  NULL);


      result = m_model_command_execute (&record_set, query, connection,command,tuples,0,FALSE);

      if (!result) d_exception_set_message (exception, M_FIRST_ERROR_MESSAGE);

      g_print(query);
    }
        
    if (record_set != NULL) gda_recordset_free(record_set);

    if (query != NULL) g_free(query);

    g_free(local_participant_code);
    g_free(tuples);
    g_free(policy_number);
    g_free(company_code);
  }

  return result;
}

/**
 * m_policy_clear_array: 
 * @policies_list: Array de plizas que se desea liberar.
 *
 * Libera la memoria utilizada por un array de plizas.
 *
 */
void m_policy_clear_array (Policy **policies_list) {
  
    if (policies_list!=NULL) {
	gint index = 0;
	while (policies_list[index]!=NULL) {
	  d_policy_clear (policies_list[index]);
	  index++;
	}
    }

    g_free(policies_list);
    
    policies_list = NULL;
    
}

/*Funcion para sair do paso que busca a poliza de un tio por numero de poliza e por codigoParty e
  devolve o codigo da compania*/

/**
 * m_policy_get_company_code_by_partial_key:
 * @participant_code: Cdigo del participante al que pertenece la pliza.
 * @policy_number: Numero de pliza.
 * @connection: Conexin a la base de datos.
 * @command: Comando en el que se ejecutar la sentencia sql.
 *
 * Obtiene el cdigo de la compaa de seguros a la que pertenece la pliza con
 * nmero de pliza %policy_number y perteneciente al cliente con cdigo %participant_code.
 *
 * Returns:
 * El cdigo de la compaa de seguros si la encuentra, %ULONG_MAX en otro caso.
 */
gulong m_policy_get_company_code_by_partial_key (gulong participant_code,gchar *policy_number,GdaConnection *connection,GdaCommand *command) {
    gchar * query;
    gchar * participant_code_aux=g_strdup_printf("%lu",participant_code);
    GdaRecordset *record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gulong result=ULONG_MAX;


    query = g_strconcat(" select codigoCompania from poliza where codigoParty=",participant_code_aux,
			 " and numeroPoliza=",policy_number,NULL);
    if (m_model_command_execute (&record_set, query, connection,command,tuples,0,TRUE)==FALSE) {
	result = ULONG_MAX;
    }

    if (record_set!=NULL) {
	if (*tuples!=0) {
	    GdaField * field;
	    gda_recordset_move_first(record_set);
	    field  = gda_recordset_field_idx(record_set,0);
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
                    gchar * company_code;
		    gchar ** stop = (gchar **)g_malloc0(sizeof(gchar *));
		    company_code = gda_stringify_value(NULL, 0, field);
		    result = strtoul(company_code,stop,10);
		    g_free(stop);
		}
		gda_field_free(field);
	    }

	} else {
	    result = ULONG_MAX;
	}
    } else {
	result = ULONG_MAX;
    }

    if (record_set!=NULL)
	gda_recordset_free(record_set);
    g_free(tuples);
    g_free(participant_code_aux);
    return result;


}

