/* post_address_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "post_address_model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"


#define TABLE         "Participante"
#define IDENTIFICATOR "codigoParty"
#define ATTRIBUTES    "codigoParty as codigopartypost_address, \
    codigo as codigopost_address, \
    codigoParty as codigoPartypost_address, \
    calle as callepost_address, \
    numero as numeropost_address, \
    piso as pisopost_address, \
    CP as cppost_address, \
    municipio as municipiopost_address, \
    provincia as provinciapost_address"

/**
 * m_post_address_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_post_address_get_table () {
    return TABLE;
}

/**
 * m_post_address_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_post_address_get_attributes () {
    return ATTRIBUTES;
}


/**
 * m_post_address_create_partial:
 * @rs: Record set que contiene los campos de la BD.
 * @rol: EL rol que desempea el cliente para el que obtendremos la direccin de envo.
 *
 * Crea un objeto direccin con solo el cdigo. NO UTILIZAR.
 *
 * Returns:
 * Un objeto #PostAddress con solo el cdigo.
 */
PostAddress * m_post_address_create_partial (GdaRecordset *rs, gchar *rol)
{
    GdaField * field;
    gchar *plusrol;
    PostAddress * post_address = d_post_address_create_empty();

    plusrol=g_strconcat("codigopost_address",rol,NULL);
    field = gda_recordset_field_name(rs,plusrol);
    if ((field!=NULL) && (!gda_field_is_null(field))) {
	post_address->code = igalia_field_get_bigint_value (field);
    }

    return post_address;
}


/**
 * m_post_address_create:
 * @rs: Record set que contiene los campos de la BD.
 * @rol: EL rol que desempea el cliente para el que obtendremos la direccin de envo.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #PostAddress con los campos rellenados con los datos de la base de datos.
 */
PostAddress * m_post_address_create (GdaRecordset *rs, gchar *rol)
{
    gchar *temp;
    gchar *plusrol;
    GdaField * field;
    PostAddress * post_address = d_post_address_create_empty();


    plusrol=g_strconcat("codigo",rol,NULL);

    field = gda_recordset_field_name(rs,plusrol);

    g_free(plusrol);
    if (field!=NULL){
	if (!gda_field_is_null(field)) {
	    post_address->code = igalia_field_get_bigint_value (field);
	    gda_field_free(field);

 	    plusrol=g_strconcat("codigoparty",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);

	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    post_address->participant_code = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("calle",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    /* 	      post_address->street = (gchar *) g_malloc (strlen(temp)+1); */
		    /* 	      strcpy(post_address->street, temp); */
		    post_address->street = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("numero",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    /* 	      post_address->number = (gchar *) g_malloc (strlen(temp)+1); */
		    /* 	      strcpy(post_address->number, temp); */
		    post_address->number = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("piso",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    /* 	      post_address->floor = (gchar *) g_malloc (strlen(temp)+1); */
		    /* 	      strcpy(post_address->floor, temp); */
		    post_address->floor = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("cp",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field))
		    post_address->postal_code = igalia_field_get_bigint_value (field);
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("municipio",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    /* 	      post_address->town = (gchar *) g_malloc (strlen(temp)+1); */
		    /* 	      strcpy(post_address->town, temp); */
		    post_address->town = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    plusrol=g_strconcat("provincia",rol,NULL);
	    field = gda_recordset_field_name(rs,plusrol);
	    g_free(plusrol);
	    if (field!=NULL){
		if (!gda_field_is_null(field)) {
		    temp = gda_stringify_value(NULL, 0, field);
		    /* 	      post_address->province = (gchar *) g_malloc (strlen(temp)+1); */
		    /* 	      strcpy(post_address->province, temp); */
		    post_address->province = g_strdup(temp);
		    g_free(temp);
		};
		gda_field_free(field);
	    }


	} else {
	    gda_field_free(field);
	}
    }
    return post_address;
}


/**
 * m_post_address_insert:
 * @post_address: Direccin de envo de un participante.
 *
 * Inserta una direccin de envo de un cliente en la base de datos.
 *
 * Returns:
 * %TRUE si se ha realizado la insercin en la base de datos, %FALSE en otro caso.
 * 
 */
gboolean m_post_address_insert (PostAddress * post_address) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong party_code = d_post_address_get_participant_code (post_address);
    gchar * street=NULL;
    gchar * party_code_char=NULL;
    gchar * number=NULL;
    gchar * floor=NULL;
    gchar * postal_code=NULL;
    gchar * town=NULL;
    gchar * province=NULL;
    gchar * temp_nextval=NULL;
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    if (party_code!=ULONG_MAX) {
        party_code_char = g_strdup_printf("%lu",party_code);
	street = d_post_address_get_street (post_address);
	if (street==NULL) {
            street = g_strdup("NULL");
	}

	number = d_post_address_get_number (post_address);
	if (number==NULL) {
            number = g_strdup("NULL");
	}

	floor = d_post_address_get_floor (post_address);
	if (floor==NULL) {
            floor = g_strdup("NULL");
	}

	town = d_post_address_get_town (post_address);
	if (town==NULL) {
            town = g_strdup("NULL");
	}

	province = d_post_address_get_province (post_address);
	if (province==NULL) {
            province = g_strdup("NULL");
	}

	if ((d_post_address_get_postal_code (post_address))!=ULONG_MAX) {
            postal_code = g_strdup_printf("%lu",(d_post_address_get_postal_code (post_address)));
	} else {
            postal_code = g_strdup("NULL");
	}
	temp_nextval = m_model_nextval_sequence ("seqdireccionenvio",connection);

	query = g_strconcat(" INSERT INTO DIRECCIONENVIO VALUES (",
			    temp_nextval,",",
                            party_code_char,",'",
                            street,"','",
                            number,"','",
                            floor,"','",
                            postal_code,"','",
                            town,"','",
                            province,"'",
			    ")",NULL);

        g_free(temp_nextval);
	result = m_model_command_execute (&record_set, query, connection,
				       command,tuples,0, FALSE);

    }

    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (party_code_char!=NULL) g_free(party_code_char);
    if (street!=NULL) g_free(street);
    if (number!=NULL) g_free(number);
    if (floor!=NULL) g_free(floor);
    if (postal_code!=NULL) g_free(postal_code);
    if (town!=NULL) g_free(town);
    if (province!=NULL) g_free(province);

    if (record_set!=NULL)
	gda_recordset_free(record_set);

    igalia_command_free(command);

    return result;
}



/**
 * m_post_address_find_by_participant_code:
 * @party_code: Cdigo del participante para el que se busca la lista de cuentas.
 *
 * Busca las cuentas corrientes de un participante.
 *
 * Returns:
 * Un array con las cuentas corrientes.
 */
PostAddress ** m_post_address_find_by_participant_code (gulong party_code) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_from=NULL;
    gchar              *gchar_where=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    gchar * party_code_char = g_strdup_printf("%lu",party_code);
    PostAddress ** post_addresses=NULL;
    PostAddress * post_address;
    gint index;
    gulong * tuples;

    my_command = gda_command_new();
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_select = g_strconcat(" SELECT  ",
			       " codigoParty as codigopartypost_address," ,
			       " codigo as codigopost_address," ,
			       " codigoParty as codigoPartypost_address," ,
			       " calle as callepost_address," ,
			       " numero as numeropost_address," ,
			       " piso as pisopost_address," ,
			       " CP as cppost_address," ,
			       " municipio as municipiopost_address," ,
			       " provincia as provinciapost_address ",
			       NULL );


    gchar_from = g_strconcat(" from direccionenvio ",NULL);
    gchar_where = g_strconcat(" where codigoParty=",party_code_char,
			      NULL);

    g_free(party_code_char);

    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where,NULL);


    g_print(_("\nQuery: %s\n"),gchar_query);

    m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0, TRUE);

    if (record_set!=NULL) {
	if (*tuples!=0) {
	    post_addresses = (PostAddress **) g_malloc0(sizeof(PostAddress*)*(*tuples+1));
		index = gda_recordset_move_first(record_set);
		while (!gda_recordset_eof(record_set)) {
		    if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
		    post_address = m_post_address_create (record_set,"post_address");
		    post_addresses[index-1] = post_address;
		    index = gda_recordset_move_next(record_set);
		}
	    } else {
		post_addresses=NULL;
	    }
	    gda_recordset_close(record_set);
	} else {
	    post_addresses=NULL;
	}

	if (post_addresses!=NULL) {
	    post_addresses[*tuples]= NULL;
	}

	g_free(gchar_from);
	g_free(gchar_query);
	g_free(gchar_where);
	g_free(tuples);


    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));

    return post_addresses;

}

/**
 * m_post_address_delete:
 * @post_address: Direccin de envo que se desea borrar.
 *
 * Borra una cuenta corriente de la base de datos.
 *
 * Returns:
 * %TRUE si se ha podido borrar correctamente, %FALSE si no.
 */
gboolean m_post_address_delete (PostAddress * post_address) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong party_code = d_post_address_get_participant_code (post_address);
    gchar * party_code_char=NULL;
    gulong code= d_post_address_get_code (post_address);
    gchar * code_char=NULL;
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    if ((party_code!=ULONG_MAX) && (party_code!=ULONG_MAX)){
        party_code_char = g_strdup_printf("%lu",party_code);
        code_char = g_strdup_printf("%lu",code);

	query = g_strconcat(" DELETE FROM DIRECCIONENVIO ",
			    " WHERE CODIGO=",
			    code_char," AND CODIGOPARTY",
                            party_code_char,
			    ")",NULL);


	result = m_model_command_execute (&record_set, query, connection,
				       command,tuples,0, FALSE);

    }

    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (party_code_char!=NULL) g_free(party_code_char);
    if (code_char!=NULL) g_free(code_char);

    if (record_set!=NULL)
	gda_recordset_free(record_set);

    igalia_command_free(command);

    return result;

}

/**
 * m_post_address_update:
 * @post_address: Direccin de envo que se desea modificar.
 *
 * Modifica una direccin de envo en la base de datos.
 *
 * Returns:
 * %TRUE si se ha realizado correctamente, %FALSE en otro caso.
 */
gboolean m_post_address_update (PostAddress * post_address) {
    gboolean result=TRUE;
    gchar * query=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong party_code = d_post_address_get_participant_code (post_address);
    gulong code = d_post_address_get_code (post_address);
    gchar * street=NULL;
    gchar * party_code_char=NULL;
    gchar * code_char=NULL;
    gchar * number=NULL;
    gchar * floor=NULL;
    gchar * postal_code=NULL;
    gchar * town=NULL;
    gchar * province=NULL;
    gulong logged_shop_code;
    gulong logged_enterprise_code;
    gulong logged_warehouse_code;
    gchar * temp_name;

    temp_name = u_parameter_manager_get_parameter ("empresa");
    logged_enterprise_code = m_enterprise_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("tienda");
    logged_shop_code = m_shop_get_code_from_name (temp_name);
    g_free(temp_name);
    temp_name = u_parameter_manager_get_parameter ("almacen");
    logged_warehouse_code = m_warehouse_get_code_from_name (temp_name);

    g_free(temp_name);

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,logged_enterprise_code,logged_shop_code,logged_warehouse_code);

    if (party_code!=ULONG_MAX) {
        party_code_char = g_strdup_printf("%lu",party_code);
	code_char = g_strdup_printf("%lu",code);
	street = d_post_address_get_street (post_address);
	if (street==NULL) {
            street = g_strdup("");
	}

	number = d_post_address_get_number (post_address);
	if (number==NULL) {
            number = g_strdup("");
	}

	floor = d_post_address_get_floor (post_address);
	if (floor==NULL) {
            floor = g_strdup("");
	}

	town = d_post_address_get_town (post_address);
	if (town==NULL) {
            town = g_strdup("");
	}

	province = d_post_address_get_province (post_address);
	if (province==NULL) {
            province = g_strdup("");
	}

	if ((d_post_address_get_postal_code (post_address))!=ULONG_MAX) {
            postal_code = g_strdup_printf("%lu",d_post_address_get_postal_code (post_address));
	} else {
            postal_code = g_strdup("0");
	}

	query = g_strconcat(" UPDATE DIRECCIONENVIO SET ",
                            "calle='",street,"',",
                            "numero='",number,"',",
                            "piso='",floor,"',",
                            "cp='",postal_code,"',",
                            "municipio='",town,"',",
			    "provincia='",province,"'",
			    " where codigo=",code_char,
                            " and codigoParty=",party_code_char,
			    NULL);


	result = m_model_command_execute (&record_set, query, connection,
				       command,tuples,0, FALSE);

    }

    result = m_model_execute_mutex_modification_logged (connection) && result;

    if (query!=NULL) g_free(query);
    if (tuples!=NULL) g_free(tuples);
    if (party_code_char!=NULL) g_free(party_code_char);
    if (code_char!=NULL) g_free(code_char);
    if (street!=NULL) g_free(street);
    if (number!=NULL) g_free(number);
    if (floor!=NULL) g_free(floor);
    if (postal_code!=NULL) g_free(postal_code);
    if (town!=NULL) g_free(town);
    if (province!=NULL) g_free(province);

    if (record_set!=NULL)
	gda_recordset_free(record_set);

    igalia_command_free(command);

    return result;

}

/**
 * m_post_address_clear_array:
 * @post_addresses: Array con las direcciones de envo.
 *
 * Libera la memoria utilizada por el array.
 *
 */
void m_post_address_clear_array (PostAddress ** post_addresses) {
    if (post_addresses!=NULL) {
	gint index = 0;
	while (post_addresses[index]!=NULL) {
	    d_post_address_clear (post_addresses[index]);
	    index++;
	}
    }
    g_free(post_addresses);

}

/**
 * m_post_address_show_on_one_line:
 * @post_address: Array con las direcciones de envo.
 *
 * Une en una lnea la direccin.
 *
 * Returns:
 * Devuelve una lnea (gchar *) con la direccin.
 */
gchar * m_post_address_show_on_one_line (PostAddress * post_address) {

    gchar * post_address_gchar;
    gchar * street = d_post_address_get_street (post_address);
    gchar * number = d_post_address_get_number (post_address);
    gchar * floor = d_post_address_get_floor (post_address);
    gchar * town = d_post_address_get_town (post_address);
    gchar * province = d_post_address_get_province (post_address);

    if (street ==NULL) street = g_strdup("");
    if (number ==NULL) number = g_strdup("");
    if (floor ==NULL) floor = g_strdup("");
    if (town ==NULL) town = g_strdup("");
    if (province ==NULL) province = g_strdup("");

    post_address_gchar = g_strconcat(street," ",number," ",floor," ",town," ",province,NULL);

    g_free(street);
    g_free(number);
    g_free(floor);
    g_free(town);
    g_free(province);

    return post_address_gchar;
}
