
/* repair_object_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "repair_object_model.h"
#include "repair_line_model.h"
#include "vehicle_model.h"
#include "policy_model.h"
#include "receipt_model.h"
#include "repair_line_object.h"
#include "vehicle.h"
#include "company_model.h"
#include "policy.h"
#include "receipt.h"
#include "accident.h"
#include "shop_model.h"
#include "enterprise_model.h"
#include "client_model.h"
#include "garage_model.h"
#include "agent_model.h"
#include "receipt.h"
#include "participant.h"

#define M_FIRST_ERROR_MESSAGE N_("Error in accident data")
#define M_SECOND_ERROR_MESSAGE N_("Error in REPAIR DATES or in MOBILE UNIT data.")

#define TABLE      "Reparacion"
#define ATTRIBUTES "codigoReparacion as codigoreparacionrepair, \
                    codigoEmpresa as codigoempresarepair, \
                    codigoTienda as codigotienda, \
                    codigoTaller as codigotallerrepair, \
                    codigoCliente as codigoclienterepair, \
                    codigoCompania as codigocompaniarepair, \
                    numeroPoliza as numeropolizarepair, \
                    numeroRecibo as numeroreciborepair, \
                    fechaEntrada as fechaentradarepair, \
                    fechaSalida as fechasalidarepair, \
                    observaciones as observacionesrepair, \
                    lugarUnidadMovil as lugarUnidadMovilrepair, \
                    distanciaUnidadMovil as distanciaUnidadMovilrepair, \
                    importeTotal as importeTotalrepair, \
                    codigoAgente as codigoAgenterepair, \
                    codigoVehiculo as codigovehiculorepair"                      




static gboolean insert_accident(Accident *accident, gulong repair_code_gulong, gchar *shop_code, gchar *enterprise_code, GdaCommand * command, GdaConnection *connection, gboolean update, Exception *exception);
static gboolean is_new_accident(gchar *repair_code, gchar *shop_code, gchar *enterprise_code);



/**
 * m_repair_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_repair_get_table () {
    return TABLE;
}

/**
 * m_repair_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_repair_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_repair_create:
 * @rs: Record set que contiene los campos de la BD.
 * @repair_line: Lneas de la reparacin que se crear.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Repair con los campos rellenados con los datos de la base de datos.
 */
Repair *m_repair_create (GdaRecordset *rs, Repair_line **repair_line) {
    GdaField *field;
//    Repair *repair = (Repair *) g_malloc (sizeof(Repair));

//    repair_init(repair);
    Repair * repair = d_repair_create_empty();



    field = gda_recordset_field_name(rs,"codigoreparacionrepair");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    repair->repair_code = igalia_field_get_bigint_value (field);

	    gda_field_free(field);

	    field = gda_recordset_field_name(rs, "fechaentradarepair");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    repair->in_date = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "fechasalidarepair");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    time_t * my_time;
		    temp=gda_stringify_value(NULL, 0, field);
		    my_time = u_parse_date (temp);
		    repair->out_date = (*my_time);
		    g_free(my_time);
		    my_time = NULL;
		    g_free(temp);

		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "observacionesrepair");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;

		    temp=gda_stringify_value(NULL, 0, field);
		    repair->observations = (gchar *) g_malloc0 (strlen(temp)+1);
		    strcpy(repair->observations, temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "importeTotalRepair");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar *temp;

		    temp=gda_stringify_value(NULL, 0, field);
		    //	temp = g_strdup(field->real_value->_u.v._u.lvc);
		    //	temp=gda_field_get_string_value(field);
		    repair->total = atof(temp);

		}
		gda_field_free(field);
	    }


	    //	repair->total = gda_field_get_dec_value(field);
	    //          repair->total = field->real_value->_u.v._u.dp;

	    field = gda_recordset_field_name(rs, "lugarUnidadMovilrepair");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    gchar * temp;
		    temp=gda_stringify_value(NULL, 0, field);
		    repair->mobile_unit_place = (gchar *) g_malloc0 (strlen(temp)+1);
		    strcpy(repair->mobile_unit_place, temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs, "distanciaUnidadMovilrepair");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    repair->mobile_unit_distance = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }


	    repair->enterprise = m_enterprise_create (rs);
	    repair->shop = m_shop_create (rs);
	    repair->garage = m_garage_create (rs);
	    repair->company = m_company_create (rs);
	    repair->vehicle = m_vehicle_create (rs);
	    repair->receipt = m_receipt_create (rs);
	    repair->policy = m_policy_create (rs);
	    repair->accident = m_accident_create (rs);
	    repair->agent = m_agent_create (rs);
	    repair->client = m_client_create (rs);
	    repair->repair_line = repair_line;

	} else {
	    gda_field_free(field);
	}
    }
    return repair;
}


/**
 * m_repair_find_by_id:
 * @condition: Identificador de la reparacin de la que se quiere obtener los datos.
 * @repair_lines: Lineas de reparacin de la reparacin en cuestin.
 *
 * Busca los datos de una reparacin a partir de su identificador.
 *
 * Returns:
 * Reparacin con sus lneas en caso de que se haya encnotrado, %NULL en otro caso.
 */
Repair * m_repair_find_by_id (RepairId * condition,Repair_line ** repair_lines) {
    GdaCommand         *my_command;
    gchar              *gchar_query=NULL;
    gchar              *gchar_select=NULL;
    gchar              *gchar_where=NULL;
    gchar              *gchar_where_aux=NULL;
    gchar              *gchar_from=NULL;
    GdaConnection      *my_connection = m_model_get_connection ();
    GdaRecordset       *record_set = NULL;
    Repair * repair;
    gint index;
    gulong * tuples;


    my_command = gda_command_new();

    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(my_command,my_connection);

    gchar_select = g_strconcat("select ",
			       "reparacion.codigoReparacion as codigoReparacionRepair,",
			       "reparacion.fechaEntrada as fechaEntradaRepair,",
			       "reparacion.fechaSalida as fechaSalidaRepair,",
			       "reparacion.observaciones as observacionesRepair,",
			       "reparacion.distanciaUnidadMovil as distanciaUnidadMovilRepair,",
			       "reparacion.lugarunidadmovil as lugarunidadmovilRepair,",
			       "reparacion.matricula as matriculavehicle,",
			       "reparacion.kilometros as kilometrosvehicle,",
			       "reparacion.color as colorvehicle,",
			       "participanteAgent.codigoParty as codigoPartyParticipantAgent,",
			       "participanteAgent.nick as nickParticipantAgent,",
			       "participanteAgent.nombre as nombreParticipantAgent,",
			       "participanteAgent.cif as cifParticipantAgent,",
			       "participanteAgent.telefono as telefonoParticipantAgent,",
			       "participanteAgent.telefono2 as telefono2ParticipantAgent,",
			       "participanteAgent.fax as faxParticipantAgent,",
			       "participanteAgent.email as emailParticipantAgent,",
			       "participanteAgent.calle as calleParticipantAgent,",
			       "participanteAgent.numero as numeroParticipantAgent,",
			       "participanteAgent.piso as pisoParticipantAgent,",
			       "participanteAgent.cp as cpParticipantAgent,",
			       "participanteAgent.municipio as municipioParticipantAgent,",
			       "participanteAgent.provincia as provinciaParticipantAgent,",
			       "to_char(Reparacion.importeTotal,'99999999D99') as importeTotalrepair, ",
			       "vehiculo.codigovehiculo as codigovehiculovehicle,",
			       "vehiculo.numerobastidor as numerobastidorvehicle,",
			       "vehiculo.modelo as modelovehicle,",
			       "vehiculo.marca as marcavehicle,",
			       "siniestro.codigoReparacion as codigoReparacionAccident,",
			       "siniestro.codigoEmpresa as codigoEmpresaAccident,",
			       "siniestro.codigoTienda as codigoTiendaAccident,",
			       "siniestro.referenciaSiniestro as referenciaSiniestroAccident,",
			       "siniestro.fecha as fechaAccident,",
			       "siniestro.lugar as lugarAccident,",
			       "siniestro.danos as danosAccident,",
			       "siniestro.codprov as codprovaccident,",
			       "siniestro.codcausa as codcausaaccident,",
			       "siniestro.foto1 as foto1accident,",
			       "siniestro.descripcion as descripcionaccident,",
			       "siniestro.foto2 as foto2accident,",
			       "siniestro.modo as modoAccident,",
			       "siniestro.descripcion as descripcionAccident,",
			       "siniestro.ramo as ramoAccident,",
			       "siniestro.fechaApertura as fechaAperturaAccident,",
			       "participanteEnter.codigoParty as codigoPartyParticipantEnter,",
			       "participanteEnter.nombre as nombreParticipantEnter,",
			       "participanteEnter.cif as cifParticipantEnter,",
			       "participanteEnter.municipio as municipioParticipantEnter,",
			       "empresa.perteneceGrupo as perteneceGrupoEnterprise,",
			       "participanteShop.codigoParty as codigoPartyParticipantShop,",
			       "participanteShop.nombre as nombreParticipantShop,",
			       "participanteShop.nick as nickParticipantShop,",
        		       "participanteShop.telefono as telefonoParticipantShop,",
        		       "participanteShop.fax as faxParticipantShop,",
        		       "participanteShop.calle as calleParticipantShop,",
        		       "participanteShop.numero as numeroParticipantShop,",
        		       "participanteShop.piso as pisoParticipantShop,",
        		       "participanteShop.cp as cpParticipantShop,",
        		       "participanteShop.email as emailParticipantShop,",
        		       "participanteShop.municipio as municipioParticipantShop,",
			       "tienda.codigoPertenenciaGrupo as codigoPertenenciaGrupoShop,",
			       "tienda.codigoReparador as codigoReparadorShop,",
			       "participanteGarage.codigoParty as codigoPartyParticipantGarage,",
			       "participanteGarage.nombre as nombreParticipantGarage,",
			       "participanteGarage.nick as nickParticipantGarage,",
			       "participanteGarage.nombreCorto as shortNameParticipantGarage,",
			       "participanteGarage.cif as cifParticipantGarage,",
			       "participanteGarage.telefono as telefonoParticipantGarage,",
			       "participanteGarage.telefono2 as telefono2ParticipantGarage,",
			       "participanteGarage.fax as faxParticipantGarage,",
			       "participanteGarage.email as emailParticipantGarage,",
			       "participanteGarage.calle as calleParticipantGarage,",
			       "participanteGarage.numero as numeroParticipantGarage,",
			       "participanteGarage.piso as pisoParticipantGarage,",
			       "participanteGarage.cp as cpParticipantGarage,",
			       "participanteGarage.municipio as municipioParticipantGarage,",
			       "participanteGarage.provincia as provinciaParticipantGarage,",
			       "taller.comision as comisionGarage,",
			       "to_char(cliente.dtocliente,'999D99') as dtoclienteclient, ",
			       "participanteCliente.codigoParty as codigoPartyParticipantClient,",
			       "participanteCliente.nick as nickParticipantClient,",
			       "participanteCliente.nombre as nombreParticipantClient,",
			       "participanteCliente.nombreCorto as shortNameParticipantClient,",
			       "participanteCliente.cif as cifParticipantClient,",
			       "participanteCliente.telefono as telefonoParticipantClient,",
			       "participanteCliente.telefono2 as telefono2ParticipantClient,",
			       "participanteCliente.fax as faxParticipantClient,",
			       "participanteCliente.email as emailParticipantClient,",
			       "participanteCliente.calle as calleParticipantClient,",
			       "participanteCliente.numero as numeroParticipantClient,",
			       "participanteCliente.piso as pisoParticipantClient,",
			       "participanteCliente.cp as cpParticipantClient,",
			       "participanteCliente.municipio as municipioParticipantClient,",
			       "participanteCliente.provincia as provinciaParticipantClient,",
			       "participanteCompany.codigoParty as codigoPartyParticipantCompany,",
			       "participanteCompany.nombre as nombreParticipantCompany,",
			       "participanteCompany.nick as nickParticipantCompany,",
			       "participanteCompany.nombreCorto as shortNameParticipantCompany,",
			       "participanteCompany.cif as cifParticipantCompany,",
			       "participanteCompany.calle as calleParticipantCompany,",
			       "participanteCompany.numero as numeroParticipantCompany,",
			       "participanteCompany.piso as pisoParticipantCompany,",
			       "participanteCompany.cp as cpParticipantCompany,",
			       "participanteCompany.municipio as municipioParticipantCompany,",
			       "participanteCompany.provincia as provinciaParticipantCompany,",
			       "compania.codigoConsorcio as codigoConsorcioCompany,",
			       "compania.plantilla as plantillaCompany,",
			       "to_char(compania.dtocompania,'999D99') as dtoCompany,",
			       "participanteConsorcio.nombre as nombreParticipantConsortium,",
			       "poliza.codigoCompania as codigoCompanypolicy,",
			       "poliza.numeroPoliza as numeroPolizapolicy,",
			       "poliza.codigoTitular as codigoTitularpolicy,",
			       "poliza.codigoCompania as codigoCompanyapolicy,",
			       "poliza.numeroPoliza as numeroPolizaapolicy,",
			       "poliza.codigoTitular as codigoTitularapolicy,",
			       "recibo.codigoCompania as numerocompaniareceipt,",
			       "recibo.numeroPoliza as numeroPolizareceipt,",
			       "recibo.codigoTitular as codigoTitularreceipt,",
			       "recibo.numeroRecibo as numeroReciboreceipt,",
			       "recibo.sucursalMediador as sucursalMediadorreceipt,",
			       "recibo.iniciovalidez as inicioValidezreceipt,",
			       "recibo.finValidez as finValidezreceipt",
			       NULL );
    gchar_from = g_strconcat(" from tienda,",
			     " reparacion",
			     " left outer join taller on (reparacion.codigoReparacion=taller.codigoParty)",
			     " left outer join participante participanteGarage on (reparacion.codigoTaller=participanteGarage.codigoParty)",
			     " left outer join participante participanteAgent on (reparacion.codigoagente = participanteAgent.codigoParty)",
			     " left outer join compania on (reparacion.codigocompania =compania.codigoparty)",
			     " left outer join participante participanteCompany on (reparacion.codigocompania = participantecompany.codigoparty)",
			     " left outer join participante participanteConsorcio on (participanteConsorcio.codigoParty=compania.codigoConsorcio)",
			     " left outer join recibo on ((recibo.numeroRecibo=reparacion.numeroRecibo) and (recibo.numeroPoliza=reparacion.numeroPoliza) and (recibo.codigocompania = reparacion.codigocompania) and (reparacion.codigocliente = recibo.codigotitular))",
			     " left outer join poliza on ((poliza.numeroPoliza=recibo.numeroPoliza) and (poliza.codigotitular=recibo.codigotitular) and (poliza.codigocompania = recibo.codigocompania))",
			     " left outer join siniestro on ((siniestro.codigoReparacion=reparacion.codigoReparacion) and (siniestro.codigoEmpresa=reparacion.codigoEmpresa) and (siniestro.codigoTienda=reparacion.codigoTienda))",
			     " left outer join vehiculo on (vehiculo.codigoVehiculo=reparacion.codigoVehiculo)",
			     " ,participante participanteEnter",
			     " ,empresa",
			     " ,participante participanteShop,",
			     " participante participanteCliente,",
			     " cliente",
			     NULL);
    
    gchar_where=g_strconcat(" where "," (participanteShop.codigoParty=tienda.codigoTienda) and (tienda.codigoTienda=reparacion.codigoTienda) and ",
			    " (participanteCliente.codigoParty=reparacion.codigoCliente) and (cliente.codigoparty = reparacion.codigocliente)",NULL);
    
    if (((condition->repair_code) != NULL) && (strlen(condition->repair_code)!=0)) {
      gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoReparacion=",
				    condition->repair_code,")",NULL);
      g_free(gchar_where);
      gchar_where = gchar_where_aux;
      gchar_where_aux=NULL;
    }
    
    
    gchar_where_aux=g_strconcat(gchar_where,
				" and participanteEnter.codigoParty=empresa.codigoParty ",
				" and empresa.codigoParty=reparacion.codigoEmpresa ",
				" and participanteShop.codigoParty=reparacion.codigoTienda ",
// 				" and vehiculo.codigovehiculo=reparacion.codigoVehiculo ",
				" and cliente.codigoParty=reparacion.codigoCliente ",
				NULL);
    g_free(gchar_where);
    gchar_where = gchar_where_aux;
    gchar_where_aux=NULL;
    
    if (((condition->enterprise_code) != NULL) && (strlen(condition->enterprise_code)!=0)) {
      gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoEmpresa=",
				    condition->enterprise_code,")",NULL);
      g_free(gchar_where);
      gchar_where = gchar_where_aux;
      gchar_where_aux=NULL;
    }
    
    if (((condition->shop_code) != NULL) && (strlen(condition->shop_code)!=0)) {
      gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoTienda=",
				    condition->shop_code,")",NULL);
      g_free(gchar_where);
      gchar_where = gchar_where_aux;
      gchar_where_aux=NULL;
    }
    
    gchar_query = g_strconcat(gchar_select,gchar_from,gchar_where, NULL);
    
    
    g_print(_("\nQuery: %s\n"),gchar_query);
    
    m_model_command_execute (&record_set, gchar_query, my_connection,
			      my_command,tuples,0,TRUE);

    
    if (record_set!=NULL) {
	if (*tuples==1) {
	  index = gda_recordset_move_first(record_set);
	  if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	  repair = m_repair_create (record_set,repair_lines);
	}
	else {
	  repair=d_repair_create_empty ();
	}
	gda_recordset_close(record_set);
	gda_recordset_free(record_set);
    } else {
      repair=NULL;
    }
    
    igalia_command_free(my_command);
    g_print(_("Freeing database completed\n"));
    
    
    g_free(gchar_where);
    g_free(gchar_select);
    g_free(gchar_query);
    g_free(gchar_from);
    g_free(tuples);
    
    return repair;
    
}

/**
 * m_repair_find_by_filter:
 * @filter: Condicin de bsqueda. Permite realiza la busqueda por matrcula de coche,
 * cif y nombre del cliente, nombre del agente de la reparacin, nombre de la compaa
 * de seguros, fecha de entrada y salida de la reparacin, cdigo de la tienda y cdigo
 * de la empresa de la reparacin.
 *
 * Busca las reparaciones que cumplen unos filtros de bsqueda.
 *
 * Returns:
 * La lista de reparaciones que cumplen la condicin de bsqeda.
 */
Repair ** m_repair_find_by_filter (Repair_filter *filter) {

  Repair ** repairs_list = NULL; 
  GdaCommand         * my_command;
  GdaConnection      * my_connection = m_model_get_connection ();
  GdaRecordset       * record_set = NULL;
  Repair * repair;
  gchar * gchar_query = NULL;
  gchar * gchar_query_aux = NULL;
  gchar * gchar_where = NULL;
  gchar * gchar_order = NULL;
  gchar * gchar_where_aux = NULL;
  gulong * tuples;
  gint * column;
  gint index;
  gint length_of_list;
  
  my_command = gda_command_new();
  gda_command_set_connection(my_command,my_connection);
    
  gchar_query_aux = g_strconcat("SELECT ",
			    "vehiculo.codigovehiculo as codigovehiculovehicle,",
			    "vehiculo.numerobastidor as numerobastidorvehicle,",
			    "reparacion.matricula as matriculavehicle,",
			    "participantecliente.codigoparty as codigopartyparticipantclient,",
			    "participantecliente.cif as cifparticipantclient,",
			    "participantecliente.nick as nickparticipantclient,",
			    "participantecliente.nombreCorto as shortNameparticipantclient,",
			    "participantecompania.codigoparty as codigopartyparticipantcompany,",
			    "participantecompania.nick as nickparticipantcompany,",
			    "participantecompania.nombreCorto as shortNameparticipantcompany,",
			    "reparacion.codigoreparacion as codigoreparacionrepair,", 
			    "reparacion.codigoempresa as codigoPartyParticipantEnter,", 
			    "reparacion.codigotienda as codigoPartyparticipantShop,", 
			    "reparacion.fechaentrada as fechaentradarepair,", 
			    "participantetaller.codigoparty as codigopartyparticipantgarage,", 
			    "participantetaller.nick as nickparticipantgarage,",
			    "participantetaller.nombreCorto as shortNameparticipantgarage",
			    " FROM vehiculo, participante participantecliente, reparacion",
			    " left outer join participante participantecompania on (reparacion.codigocompania = participantecompania.codigoParty)",
			    " left outer join participante participantetaller on (reparacion.codigotaller = participantetaller.codigoParty)",
			    " left outer join participante participanteagente on (reparacion.codigoagente = participanteagente.codigoParty)",
			    NULL);
  gchar_where = g_strconcat(" WHERE vehiculo.codigoVehiculo = reparacion.codigovehiculo",
			    " AND participantecliente.codigoparty = reparacion.codigocliente "
			    ,NULL);

  gchar_order = g_strdup(" order by participantecliente.nick,participantecompania.nick");
  if (filter -> license_number != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (vehiculo.matricula ilike '%",
		     filter->license_number,"%')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> legal_id != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (participantecliente.cif ilike '",
		     filter->legal_id,"%')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> name != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (participantecliente.nick ilike '%",
		     filter->name,"%')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> agent_name != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (participanteagente.nick ilike '%",
		     filter->agent_name,"%')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> company != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (participantecompania.nick ilike '%",
		     filter->company,"%')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> garage != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (participantetaller.nick ilike '%",
		     filter->garage,"%')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> initial_date != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.fechaentrada>'",
		     filter->initial_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> ending_date != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.fechasalida<'",
		     filter->ending_date,"')",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> shop_code != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoTienda=",
		     filter->shop_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  if (filter -> enterprise_code != NULL){
	gchar_where_aux = g_strconcat(gchar_where, " and (reparacion.codigoEmpresa = ",
		     filter->enterprise_code,")",NULL);
	g_free(gchar_where);
	gchar_where = gchar_where_aux;
	gchar_where_aux=NULL;
  }

  gchar_query = g_strconcat(gchar_query_aux,gchar_where,gchar_order,NULL);

  g_print(_("\nQuery: %s\n"),gchar_query);

  tuples = (gulong *) g_malloc0(sizeof(gulong));
  m_model_command_execute (&record_set, gchar_query, my_connection,
			my_command,tuples,0,TRUE);


  
  length_of_list = *tuples;
  column = (gint *) g_malloc0(sizeof(gint));
  
  if (record_set!=NULL) {
    if (*tuples!=0) {
      repairs_list = (Repair **) g_malloc0(sizeof(Repair*)*(*tuples+1));
      index = gda_recordset_move_first(record_set);
      while (!gda_recordset_eof(record_set)) {
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	(*column)=0;
	repair = m_repair_create (record_set, NULL);
	repairs_list[index-1] = repair;
	index = gda_recordset_move_next(record_set);
      }
      repairs_list[length_of_list]=NULL;
      
    } else {
      repairs_list=NULL;
    }
    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
  } else {
    repairs_list=NULL;
  }
  
  igalia_command_free(my_command);

  if (gchar_query_aux!=NULL) g_free(gchar_query_aux);
  if (gchar_order!=NULL) g_free(gchar_order);
  if (gchar_where!=NULL) g_free(gchar_where);

  g_free(gchar_query);
  
  g_free(tuples);
  g_free(column);
  
  return repairs_list;
}


/**
 * model_delete_repair:

 *
 * Borra los movimientos de caja 
 *
 * Returns:
 * %TRUE si se modific correctamente, %FALSE en otro caso.
 */
gboolean m_repair_delete (GArray ** selected, Repair ** repairs) {

    GdaConnection * connection = m_model_get_connection ();

    return m_repair_delete_in_trans (selected, repairs, connection);
}	


/**
 * model_delete_repair_in_trans:
 * @codigoPago: Pago al que se refiere este movimiento.
 * @shop_code: Cdigo de la tienda en la que se realiz en movimiento.
 * @enterprise_code: Cdigo de la empresa a la que pertenece la tienda.
 * @connection: Conexin a la base de datos.
 *
 * Borra las reparaciones
 *
 * Returns:
 * %TRUE si se modific correctamente, %FALSE en otro caso.
 */
gboolean m_repair_delete_in_trans (GArray ** selected, Repair ** repairs, GdaConnection * connection) {
    gboolean result=TRUE;
    GdaCommand *command  = gda_command_new();
    gchar * query = NULL;
    GdaRecordset * record_set=NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));    
    Enterprise *enterprise;
    gulong enterprise_code_gulong;    
    gchar *enterprise_code;    
    Shop *shop;
    gulong shop_code_gulong;
    gchar *shop_code;    
    Repair * repair = repairs[0];    
    gint i, index, limit;   
    gulong repair_code_gulong;
    gchar * repair_code_char;
    
      
    enterprise = d_repair_get_enterprise (repair);
    enterprise_code_gulong = d_participant_get_code (d_enterprise_get_participant (enterprise));    
    enterprise_code = g_strdup_printf("%lu",enterprise_code_gulong);    
    shop = d_repair_get_shop (repair);
    shop_code_gulong = d_participant_get_code (d_shop_get_participant (shop));
    shop_code = g_strdup_printf("%lu",shop_code_gulong);            
    
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code_gulong,shop_code_gulong,ULONG_MAX);        	
    gda_command_set_connection(command,connection);
    
    limit = (*selected)->len;
    i = limit-1;
    while (i >= 0) {

	index = g_array_index(*selected, gint , i); 
	repair = repairs[index];		
	g_print(_("\nIndex: %d | i : %d\n"),index,i);
		 	
	if (!m_repair_is_managed (repair)) {
      		
		repair_code_gulong = d_repair_get_repair_code (repair);			
		repair_code_char =  g_strdup_printf("%lu",repair_code_gulong);    		
		
		query = g_strconcat(" DELETE FROM Reparacion WHERE ",
					" codigoReparacion = ", repair_code_char, NULL);

	    	g_print(_("\nQuery: %s\n"),query);
    
    		result = m_model_command_execute (&record_set, query, connection, command,tuples,0, FALSE);
		
		g_array_remove_index(*selected, i);
		
		g_free(repair_code_char);
    	}	
	    	
	i--;
	g_free(query);
    
    	if (record_set!=NULL) {
		if (GDA_IS_RECORDSET(record_set)) {
	    		gda_recordset_close(record_set);
		}
    	}
    }
            
    g_free(enterprise_code);
    g_free(shop_code);
    g_free(tuples);
    igalia_command_free(command);
  

    if (m_model_execute_mutex_modification_logged (connection)==FALSE) {
	   
	    result = FALSE;
    }  
    

    return result;
}


/**
 * m_repair_clear_array:
 * @repairs: Array con las reparacines que se desea liberar.
 *
 * Libera la memoria utilizada por el array de reparaciones.
 *
 */
void m_repair_clear_array (Repair ** repairs) {

  if (repairs!=NULL) {
    gint index = 0;
    
    while (repairs[index]!=NULL) {
      d_repair_clear (repairs[index]);
      index++;
    }
  }
  
  g_free(repairs);
  
}

/**
 * insert_accident:
 * @accident: Siniestro que se desea insertar / modificar, dentro de una transaccin.
 * @repair_code_gulong: Cdigo de la reparacin a la que se asocia el siniestro
 * @shop_code: Cdigo de la tienda donde se realiz la reparacin.
 * @enterprise_code: Cdigo de la empresa donde se realiz la reparacin.
 * @command: Comando en el que se ejecuta la sentencia sql.
 * @connection: Conexin a la base de datos.
 * @update: Si update es %TRUE se modifica, en otro caso se inserta.
 * @exception: Objeto que indica el tipo de error que se haya producido.
 *
 * Inserta un siniestro en la base de datos, relacionado con una reparacin.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
static gboolean insert_accident(Accident *accident, gulong repair_code_gulong, gchar *shop_code, gchar *enterprise_code, GdaCommand * command, GdaConnection *connection, gboolean update, Exception *exception) {

  gboolean result = FALSE;

  if(accident != NULL) {
    gchar * query = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    time_t accident_date = d_accident_get_date (accident);
    time_t accident_open_date = d_accident_get_open_date (accident);
    gchar *date = NULL;
    gchar *open_date = NULL;
    gchar *date_aux = NULL;
    gchar *accident_reference = d_accident_get_accident_reference (accident);
    gchar *place = d_accident_get_place (accident);
    gchar *field = d_accident_get_field (accident);
    gchar *damages = d_accident_get_damages (accident);
    gchar *way = d_accident_get_way (accident);
    gchar *province_code = d_accident_get_province_code (accident);
    gchar *reason_code = d_accident_get_reason_code (accident);
    gchar *photo1 = d_accident_get_photo1 (accident);
    gchar *photo2 = d_accident_get_photo2 (accident);
    gchar *description = d_accident_get_description (accident);
    gchar *shop_code_local = g_strdup(shop_code);
    gchar *enterprise_code_local = g_strdup(enterprise_code);
    gchar *repair_code = NULL;

    if (shop_code_local == NULL) shop_code_local = g_strdup("NULL");
    else{
      gchar *shop_code_local_aux = NULL;

      shop_code_local_aux = g_strconcat("'",shop_code_local,"'",NULL);
      g_free(shop_code_local);
      shop_code_local = shop_code_local_aux;
      shop_code_local_aux = NULL;
    }


    if (enterprise_code_local == NULL) enterprise_code_local = g_strdup("NULL");
    else{
      gchar *enterprise_code_local_aux = NULL;

      enterprise_code_local_aux = g_strconcat("'",enterprise_code_local,"'",NULL);
      g_free(enterprise_code_local);
      enterprise_code_local = enterprise_code_local_aux;
      enterprise_code_local_aux = NULL;
    }


    
    if (accident_reference == NULL) accident_reference = g_strdup("NULL");
    else{
      gchar *accident_reference_aux = NULL;

      accident_reference_aux = g_strconcat("'",accident_reference,"'",NULL);
      g_free(accident_reference);
      accident_reference = accident_reference_aux;
      accident_reference_aux = NULL;
    }


    date = ctime(&accident_date);
    date_aux = g_strconcat("'",date,"'",NULL);
    date = date_aux;

    open_date = ctime(&accident_open_date);
    date_aux = g_strconcat("'",open_date,"'",NULL);
    open_date = date_aux;
    date_aux = NULL;

    if (place == NULL) place = g_strdup("NULL");
    else{
      gchar *place_aux = NULL;

      place_aux = g_strconcat("'",place,"'",NULL);
      g_free(place);
      place = place_aux;
      place_aux = NULL;
    }

    if (way == NULL) way = g_strdup("NULL");
    else{
      gchar *way_aux = NULL;

      way_aux = g_strconcat("'",way,"'",NULL);
      g_free(way);
      way = way_aux;
      way_aux = NULL;
    }

    if (field == NULL) field = g_strdup("NULL");
    else{
      gchar *field_aux = NULL;

      field_aux = g_strconcat("'",field,"'",NULL);
      g_free(field);
      field = field_aux;
      field_aux = NULL;
    }

    if (damages == NULL) damages = g_strdup("NULL");
    else{
      gchar *damages_aux = NULL;

      damages_aux = g_strconcat("'",damages,"'",NULL);
      g_free(damages);
      damages = damages_aux;
      damages_aux = NULL;
    }

    if (province_code == NULL) province_code = g_strdup("NULL");
    else{
      gchar *province_code_aux = NULL;

      province_code_aux = g_strconcat("'",province_code,"'",NULL);
      g_free(province_code);
      province_code = province_code_aux;
      province_code_aux = NULL;
    }

    if (reason_code == NULL) reason_code = g_strdup("NULL");
    else{
      gchar *reason_code_aux = NULL;

      reason_code_aux = g_strconcat("'",reason_code,"'",NULL);
      g_free(reason_code);
      reason_code = reason_code_aux;
      reason_code_aux = NULL;
    }

    if (description == NULL) description = g_strdup("NULL");
    else{
      gchar *description_aux = NULL;

      description_aux = g_strconcat("'",description,"'",NULL);
      g_free(description);
      description = description_aux;
      description_aux = NULL;
    }

    if (photo1 == NULL) photo1 = g_strdup("NULL");
    else{
      gchar *photo1_aux = NULL;

      photo1_aux = g_strconcat("'",photo1,"'",NULL);
      g_free(photo1);
      photo1 = photo1_aux;
      photo1_aux = NULL;
    }

    if (photo2 == NULL) photo2 = g_strdup("NULL");
    else{
      gchar *photo2_aux = NULL;

      photo2_aux = g_strconcat("'",photo2,"'",NULL);
      g_free(photo2);
      photo2 = photo2_aux;
      photo2_aux = NULL;
    }
    
    if (update) {

      repair_code = g_strdup_printf("%lu",repair_code_gulong);

      query = g_strconcat("UPDATE ",m_accident_get_table ()," SET ",
			  " referenciasiniestro = ",accident_reference,",",
			  " fecha = ",date,",",
			  " lugar = ",place,",",
			  " ramo = ",field,",",
			  " danos = ",damages,",",
			  " modo = ",way,",",
			  " codprov = ",province_code,",",
			  " codcausa = ",reason_code,",",
			  " descripcion = ",description,",",
			  " foto1 = ",photo1,",",
			  " foto2 = ",photo2,",",
			  " fechaApertura = ",open_date,
			  " WHERE codigoreparacion = ",repair_code,
			  " AND codigoempresa = ",enterprise_code,
			  " AND codigotienda = ",shop_code,
			  NULL);        

    }
   else {
     
     if (repair_code_gulong != ULONG_MAX) repair_code = g_strdup_printf("%lu",repair_code_gulong);
     else repair_code = m_model_currval_sequence ("seqreparacion",connection); 
     
     query = g_strconcat("INSERT INTO ",m_accident_get_table ()," VALUES (",
			  repair_code,",",
			  enterprise_code,",",
			  shop_code,",",
			  accident_reference,",",
			  date,",",
			  place,",",
			  field,",",
			  damages,",",
			  way,",",
			  province_code,",",
			  reason_code,",",
			  description,",",
			  photo1,",",
			  photo2,",",
			  open_date,")",
			  NULL);  
    }    

    result = m_model_command_execute (&record_set, query, connection,
				   command,tuples,0,FALSE);
    
    if (!result) d_exception_set_message (exception, _(M_FIRST_ERROR_MESSAGE));

    g_print(_("\nAccident: %s\n"),query);

    if (record_set != NULL) gda_recordset_free(record_set);

    g_free(shop_code_local);
    g_free(enterprise_code_local);
    g_free(accident_reference);
    g_free(date);
    g_free(open_date);
    g_free(place);
    g_free(field);
    g_free(damages);
    g_free(way);
    g_free(province_code);
    g_free(reason_code);
    g_free(description);
    g_free(photo1);
    g_free(photo2);
    g_free(query);
    g_free(tuples);
    g_free(repair_code);

  }

  return result;
}

/**
 * m_repair_insert:
 * @repair: Reparacin que se desea inesrtar/modificar.
 * @new_policy: Indica si la pliza es nueva o ya existe en la base de datos.
 * @new_receipt: Indica si el recibo es nuevo o ya existe en la base de datos.
 * @new_vehicle: Indica si el vehculo es nuevo o ya existe en la base de datos.
 * @exception: Indica el tipo de error que se haya producido.
 *
 * Inserta  una reparacin en la base de datos si el cdigo de la reparacin es igual a %ULONG_MAX, o la modifica
 * si es distinto. Adems incluye la insercin/modificacin de la pliza de seguros, el recibo de la pliza,
 * el vehculo, el siniestro y las lneas de reparacin dentro de la misma transaccin.
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 * 
 */
gboolean m_repair_insert (Repair *repair, gboolean new_policy, gboolean new_receipt, gboolean new_vehicle, Exception *exception) {
 
  gboolean result = FALSE;

  if (repair != NULL) {
    gchar *query = NULL;
    time_t in_date_time = d_repair_get_in_date (repair);
    gchar *in_date = NULL;
    gchar *in_date_aux = NULL;
    time_t out_date_time = d_repair_get_out_date (repair);
    gchar *out_date = NULL;
    gchar *out_date_aux= NULL;
    gchar *observations = d_repair_get_observations (repair);
    gulong mobile_unit_distance_gulong = d_repair_get_mobile_unit_distance (repair);
    gchar *mobile_unit_distance = NULL;
    gchar *mobile_unit_place = d_repair_get_mobile_unit_place (repair);
    Enterprise *enterprise = d_repair_get_enterprise (repair);
    gulong enterprise_code_gulong = ULONG_MAX ;
    gchar *enterprise_code = NULL;
    Shop *shop = d_repair_get_shop (repair);
    gulong shop_code_gulong = ULONG_MAX;
    gchar *shop_code = NULL;
    Garage *garage = d_repair_get_garage (repair);
    gulong garage_code_gulong = ULONG_MAX;
    gchar *garage_code = NULL;
    Company *company = d_repair_get_company (repair);
    gulong company_code_gulong = ULONG_MAX;
    gchar *company_code = NULL;
    Client *client = d_repair_get_client (repair);
    gulong client_code_gulong = ULONG_MAX;
    gchar *client_code = NULL;
    Agent *agent = d_repair_get_agent (repair);
    gulong agent_code_gulong = ULONG_MAX;
    gchar *agent_code = NULL;    
    Repair_line **repair_lines = d_repair_get_repair_lines (repair);
    Policy *policy = d_repair_get_policy (repair);
    gchar *policy_number = NULL;
    Receipt *receipt = d_repair_get_receipt (repair);
    gchar *receipt_number = NULL;
    Vehicle *vehicle = d_repair_get_vehicle (repair);
    Accident *accident = d_repair_get_accident (repair);
    gdouble subtotal_gdouble =  d_repair_get_total (repair);
    gchar *subtotal = NULL;
    GdaRecordset *record_set = NULL;
    GdaRecordset *repair_code_rs = NULL;
    GdaField * field;
    gulong repair_code_gulong = ULONG_MAX;    
    gulong vehicle_code_gulong = d_vehicle_get_code (vehicle);
    gchar *vehicle_code = NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    gchar *license_number = d_vehicle_get_license_number (vehicle);
    gchar *colour = d_vehicle_get_colour (vehicle);
    gulong kms_gulong = d_vehicle_get_kilometres (vehicle);
    gchar *kms = NULL;   
    gchar * repair_code = NULL;
       
    if (kms_gulong != ULONG_MAX)
      kms = g_strdup_printf("%lu",kms_gulong);
    else kms = g_strdup("NULL");

    if (policy != NULL)
      policy_number = d_policy_get_policy_number (policy);
    if (receipt != NULL)
      receipt_number = d_receipt_get_receipt_number (receipt);
    if (enterprise != NULL)
      enterprise_code_gulong = d_participant_get_code (d_enterprise_get_participant (enterprise));
    if (shop != NULL)
      shop_code_gulong = d_participant_get_code (d_shop_get_participant (shop));
    if (garage != NULL)
      garage_code_gulong = d_participant_get_code (d_garage_get_participant (garage));
    if (company != NULL)
      company_code_gulong = d_participant_get_code (d_company_get_participant (company));
    if (client != NULL)
      client_code_gulong = d_participant_get_code (d_client_get_participant (client));
    if (agent != NULL)
      agent_code_gulong = d_participant_get_code (d_agent_get_participant (agent));

    if (mobile_unit_distance_gulong!=ULONG_MAX)
      mobile_unit_distance = g_strdup_printf("%lu",mobile_unit_distance_gulong);
    else mobile_unit_distance = g_strdup("NULL");
    
    if (client_code_gulong!=ULONG_MAX)
      client_code = g_strdup_printf("%lu",client_code_gulong);
    else client_code = g_strdup("NULL");

    if (enterprise_code_gulong!=ULONG_MAX)
      enterprise_code = g_strdup_printf("%lu",enterprise_code_gulong);
    else enterprise_code = g_strdup("NULL");

    if (shop_code_gulong!=ULONG_MAX)
      shop_code = g_strdup_printf("%lu",shop_code_gulong);
    else shop_code = g_strdup("NULL");

    if (garage_code_gulong!=ULONG_MAX)
      garage_code = g_strdup_printf("%lu",garage_code_gulong);
    else garage_code = g_strdup("NULL");

    if (company_code_gulong!=ULONG_MAX)
      company_code = g_strdup_printf("%lu",company_code_gulong);
    else company_code = g_strdup("NULL");

    if (agent_code_gulong!=ULONG_MAX)
      agent_code = g_strdup_printf("%lu",agent_code_gulong);
    else agent_code = g_strdup("NULL");

    in_date = ctime(&in_date_time); 
    in_date_aux = g_strconcat("'",in_date,"'",NULL);
    in_date = in_date_aux;
    in_date_aux = NULL;

    out_date = ctime(&out_date_time);
    out_date_aux = g_strconcat("'",out_date,"'",NULL);
    out_date = out_date_aux;
    out_date_aux = NULL;

    if (license_number == NULL) license_number = g_strdup("NULL");
    else{
      gchar *license_number_aux = NULL;

      license_number_aux = g_strconcat("'",license_number,"'",NULL);
      g_free(license_number);
      license_number = license_number_aux;
      license_number_aux = NULL;
    }

    if (colour == NULL) colour = g_strdup("NULL");
    else{
      gchar *colour_aux = NULL;

      colour_aux = g_strconcat("'",colour,"'",NULL);
      g_free(colour);
      colour = colour_aux;
      colour_aux = NULL;
    }

    if (policy_number == NULL) policy_number = g_strdup("NULL");
    else{
      gchar *policy_number_aux = NULL;

      policy_number_aux = g_strconcat("'",policy_number,"'",NULL);
      g_free(policy_number);
      policy_number = policy_number_aux;
      policy_number_aux = NULL;
    }

    if (receipt_number == NULL) receipt_number = g_strdup("NULL");
    else{
      gchar *receipt_number_aux = NULL;
    
      receipt_number_aux = g_strconcat("'",receipt_number,"'",NULL);
      g_free(receipt_number);
      receipt_number = receipt_number_aux;
      receipt_number_aux = NULL;
    }

    if (observations == NULL) observations = g_strdup("NULL");
    else{
      gchar *observations_aux = NULL;
    
      observations_aux = g_strconcat("'",observations,"'",NULL);
      g_free(observations);
      observations = observations_aux;
      observations_aux = NULL;
    }

    if (mobile_unit_place == NULL) mobile_unit_place = g_strdup("NULL");
    else{
      gchar *mobile_unit_place_aux = NULL;
    
      mobile_unit_place_aux = g_strconcat("'",mobile_unit_place,"'",NULL);
      g_free(mobile_unit_place);
      mobile_unit_place = mobile_unit_place_aux;
      mobile_unit_place_aux = NULL;
    }

    if (subtotal_gdouble != G_MINDOUBLE)
      subtotal = g_strdup_printf("to_number('%.2f','99999999D99')",subtotal_gdouble);
    else subtotal = g_strdup("NULL");

    gda_command_set_connection(command,connection);
    m_model_connection_begin_mutex_transaction_logged (connection,enterprise_code_gulong,shop_code_gulong,ULONG_MAX);

    repair_code_gulong = d_repair_get_repair_code (repair);

    if (new_vehicle)
      result = m_vehicle_insert (vehicle, &vehicle_code_gulong, command, connection, FALSE, exception);
    else result = m_vehicle_insert (vehicle, &vehicle_code_gulong, command, connection, TRUE, exception);

    if (vehicle_code_gulong != ULONG_MAX)
      vehicle_code = g_strdup_printf("%lu",vehicle_code_gulong);
    else vehicle_code = g_strdup("NULL");
    
    if (repair_code_gulong == ULONG_MAX) {
      repair_code =  m_model_nextval_sequence ("seqreparacion",connection);
      
      query = g_strconcat("INSERT INTO ",m_repair_get_table ()," VALUES (",
			  repair_code,",",
			  enterprise_code,",",
			  shop_code,",",
			  garage_code,",",
			  in_date,",",
			  out_date,",",
			  client_code,",",
			  company_code,",",
			  policy_number,",",
			  receipt_number,",",
			  vehicle_code,",",
			  observations,",",
			  mobile_unit_distance,",",
			  mobile_unit_place,",",
			  subtotal,",",
			  agent_code,",",
			  license_number,",",
			  kms,",",
			  colour,
			  ")",NULL);	
      
      
    } else {
      repair_code = g_strdup_printf("%lu",repair_code_gulong);	  
      
      query = g_strconcat("UPDATE ",m_repair_get_table ()," SET ",
			  " codigotaller = ",garage_code,",",
			  " fechaentrada = ",in_date,",",
			  " fechasalida = ",out_date,",",
			  " codigocliente = ",client_code,",",
			  " codigocompania = ",company_code,",",
			  " numeropoliza = ",policy_number,",",
			  " numerorecibo = ",receipt_number,",",
			  " codigovehiculo = ",vehicle_code,",",
			  " observaciones = ",observations,",",
			  " distanciaunidadmovil = ",mobile_unit_distance,",",
			  " lugarunidadmovil = ",mobile_unit_place,",",
			  " importetotal = ",subtotal,",",
			  " codigoagente = ",agent_code,",",
			  " matricula = ",license_number,",",
			  " kilometros = ",kms,",",
			  " color = ",colour,
			  " WHERE codigoreparacion = ",repair_code,
			  " AND codigoempresa = ", enterprise_code,
			  " AND codigotienda = ",shop_code,
			  NULL);	
      
    }
    
    if (policy != NULL) {
      if (new_policy) {
	result = result && m_policy_insert (policy, client_code, FALSE, command, connection, exception);
      }
      else {
	result = result && m_policy_insert (policy, client_code, TRUE, command, connection, exception);
      }
    }

    if (receipt != NULL) {
      if (new_receipt)
	result = result && m_receipts_insert (receipt, client_code, FALSE,command, connection, exception);
      else result = result && m_receipts_insert (receipt, client_code, TRUE,command, connection, exception);
    }
    
    if (!(result && m_model_command_execute (&record_set, query, connection,
					  command,tuples,0, FALSE))) {
      if (result) {
	d_exception_set_message (exception, _(M_SECOND_ERROR_MESSAGE));
	result = FALSE;      
      }
    }
    
    g_print(_("\nRepair: %s\n"),query);

    if (policy != NULL){
      if (is_new_accident(repair_code, shop_code, enterprise_code)) {
	result = result && insert_accident(accident, repair_code_gulong, shop_code, enterprise_code, command, connection, FALSE, exception);
      }
      else {
	result = result && insert_accident(accident, repair_code_gulong, shop_code, enterprise_code, command, connection, TRUE, exception);       
      }
    }

    if (repair_lines!=NULL) {
      gint index = 0;

/*       repair_code = g_strdup_printf("%lu",repair_code_gulong); */

      while (repair_lines[index]!=NULL) {
	if (repair_code_gulong == ULONG_MAX) 
	  result = result && m_repair_line_insert (repair_lines[index], m_model_currval_sequence ("seqreparacion",connection), enterprise_code, shop_code, FALSE, command, connection, exception);
	else {
	  gulong line_number_gulong = d_repair_line_get_line_number (repair_lines[index]);
	  gchar * line_number = g_strdup_printf("%lu",line_number_gulong);
	  
	  if (m_repair_line_is_new (repair_code, shop_code, enterprise_code, line_number))
	    result = result && m_repair_line_insert (repair_lines[index],repair_code, enterprise_code, shop_code, FALSE, command, connection, exception);
	  else result = result && m_repair_line_insert (repair_lines[index],repair_code, enterprise_code, shop_code, TRUE, command, connection, exception);
	  
	  g_free(line_number);
	}
	index++;
      }
      
/*       g_free(repair_code); */
    }

    g_free(query);

    
    if (repair_code_gulong == ULONG_MAX) {
      query =g_strdup("select currval('seqreparacion')");

      result = result && m_model_command_execute (&repair_code_rs, query, connection,
					      command,tuples,0, TRUE);
      g_print(query);
      
      g_free(query);
    }


    result = m_model_execute_mutex_modification_logged (connection) && result;

    if ((result !=FALSE) && (repair_lines != NULL)) { 
	gint index=0;

	while (repair_lines[index]!=NULL) {
	    d_repair_line_set_earlier_quantity(repair_lines[index],d_repair_line_get_quantity(repair_lines[index]));

	    if (d_repair_line_get_stock(repair_lines[index])!=NULL) {
		d_stock_set_previous_quantity(d_repair_line_get_stock(repair_lines[index]),d_stock_get_quantity(d_repair_line_get_stock(repair_lines[index])));
	    }

            index++;
	}
    }

    if (repair_code_rs != NULL) {
      gda_recordset_move_first(repair_code_rs);

      field = gda_recordset_field_name(repair_code_rs,"currval");
      
      if (field!=NULL) {
	if (!gda_field_is_null(field)){
	  gchar **stop = (gchar **) g_malloc0(sizeof(gchar *));
	  gchar *temp = gda_stringify_value(NULL, 0, field);
	  
	  repair_code_gulong = strtoul(temp,stop,10);	
	  
	  g_free(temp);
	  g_free(stop);
	  
	}
      
	d_repair_set_repair_code (repair, repair_code_gulong);

	gda_field_free(field);
      }
    }
      
    if (record_set != NULL) gda_recordset_free(record_set);
    if (repair_code_rs != NULL) gda_recordset_free(repair_code_rs);  

    igalia_command_free(command);

    g_free(tuples);
    g_free(vehicle_code);
    g_free(subtotal);
    g_free(in_date);
    g_free(out_date);
    g_free(observations);
    g_free(mobile_unit_distance);
    g_free(mobile_unit_place);
    g_free(enterprise_code);
    g_free(shop_code);
    g_free(garage_code);
    g_free(company_code);
    g_free(client_code);
    g_free(agent_code);
    g_free(policy_number);
    g_free(receipt_number);
    g_free(kms);
    g_free(colour);
    g_free(license_number);
    g_free(repair_code);
    
  }

  if (result) m_repair_line_get_changed (d_repair_get_repair_lines (repair));

  return result;
}

/**
 * m_repair_is_managed:
 * @repair: Reparacin que se comprueba.
 *
 * Comprueba si una reparacin ya est siendo gestionada, es decir, ya existe un albarn de la
 * reparacin o una factura.
 *
 * Returns:
 * %TRUE si la reparacin esta siendo gestionada, %FALSE en otro caso.
 */
gboolean m_repair_is_managed (Repair * repair) {
    gchar * query1 = NULL;
    gchar * query2 = NULL;
    gchar * query3 = NULL;
    GdaRecordset *record_set1=NULL;
    GdaRecordset *record_set2=NULL;
    GdaRecordset *record_set3=NULL;
    GdaConnection * connection = m_model_get_connection ();
    GdaCommand * command = gda_command_new();
    gulong repair_code = d_repair_get_repair_code (repair);
    gulong enterprise_code = ULONG_MAX;
    gulong shop_code = ULONG_MAX;
    gboolean result= FALSE;
    gulong * tuples;

    if (d_repair_get_enterprise (repair)!=NULL) {
        enterprise_code = d_participant_get_code (d_enterprise_get_participant (d_repair_get_enterprise (repair)));
    }
    if (d_repair_get_shop (repair)!=NULL) {
	shop_code =  d_participant_get_code (d_shop_get_participant (d_repair_get_shop (repair)));;
    }
    tuples = (gulong *) g_malloc0(sizeof(gulong));
    gda_command_set_connection(command,connection);

    if ((shop_code!=ULONG_MAX) && (enterprise_code!=ULONG_MAX) && (repair_code!=ULONG_MAX)) {
	gchar * repair_code_char=NULL;
	gchar * enterprise_code_char=NULL;
	gchar * shop_code_char=NULL;

	repair_code_char =g_strdup_printf("%lu",repair_code);
	shop_code_char=g_strdup_printf("%lu",shop_code);
        enterprise_code_char=g_strdup_printf("%lu",enterprise_code);


        /*Se comprueba si ya existe un albarn de la reparacin*/
	query1 = g_strconcat("select codigoReparacion from albaranabono where codigoReparacion=",
			     repair_code_char," and codigoEmpresa=",enterprise_code_char,
			     " and codigoTienda=",shop_code_char,NULL);

	g_print(_("\nQuery: %s\n"),query1);

	m_model_command_execute (&record_set1, query1, connection,
			      command,tuples,0,TRUE);

	if (record_set1!=NULL) {
            if (*tuples>0) result = TRUE;
	}
	if (query1!=NULL) g_free(query1);
	if (record_set1 != NULL) gda_recordset_free(record_set1);

	if (result ==FALSE) {
        /*Se comprueba si ya existe una factura de la reparacin si no existe albarn*/
	    query2 = g_strconcat("select codigoReparacion from facturaabono where "
				 ," codigoReparacion=",repair_code_char
				 ," and codigoTienda=",shop_code_char
				 ," and codigoEmpresa=",enterprise_code_char
				 ,NULL);
	    g_print(_("\nQuery: %s\n"),query2);

	    m_model_command_execute (&record_set2, query2, connection,
				  command,tuples,0,TRUE);
	    if (record_set2!=NULL) {
		if (*tuples>0) result = TRUE;
	    }
	    if (result == FALSE) {
		/*Se comprueba si ya existe una nota de la reparacin si no existe albarn ni factura*/
		query3 = g_strconcat("select codigoReparacion from nota where "
				     ," codigoReparacion=",repair_code_char
				     ," and codigoTienda=",shop_code_char
				     ," and codigoEmpresa=",enterprise_code_char
				     ,NULL);

		m_model_command_execute (&record_set3, query3, connection,
				      command,tuples,0,TRUE);
		if (record_set3!=NULL) {
		    if (*tuples>0) result = TRUE;
		}

		if (query3!=NULL) g_free(query3);
		if (record_set3 != NULL) gda_recordset_free(record_set3);

	    }
	    if (query2!=NULL) g_free(query2);
	    if (record_set2 != NULL) gda_recordset_free(record_set2);
      	}
        if (repair_code_char!=NULL) g_free(repair_code_char);
        if (shop_code_char!=NULL) g_free(shop_code_char);
        if (enterprise_code_char!=NULL) g_free(enterprise_code_char);
    } else {
	result = TRUE;
    }
    igalia_command_free(command);
    return result;
}

/**
 * m_repair_clear_filter:
 * @filter: Filtro de bsqueda.
 *
 * Libera la memoria utilizada por el filtro de bsqueda.
 *
 */
void m_repair_clear_filter (Repair_filter * filter) {
    if (filter !=NULL) {
	if (filter->license_number!=NULL) g_free(filter->license_number);
	if (filter->legal_id!=NULL) g_free(filter->legal_id);
	if (filter->name!=NULL) g_free(filter->name);
	if (filter->agent_name!=NULL) g_free(filter->agent_name);
	if (filter->company!=NULL) g_free(filter->company);
	if (filter->garage!=NULL) g_free(filter->garage);
	if (filter->initial_date!=NULL) g_free(filter->initial_date);
	if (filter->ending_date!=NULL) g_free(filter->ending_date);
	if (filter->shop_code!=NULL) g_free(filter->shop_code);
	if (filter->enterprise_code!=NULL) g_free(filter->enterprise_code);

        g_free(filter);
    }

}

/**
 * is_new_accident:
 * @repair_code: Cdigo de la reparacin para la que se comprobar la existencia del siniestro.
 * @shop_code: Cdigo de la tienda en la que se realiz la reparacin.
 * @enterprise_code: Cdigo de la empresa en la que se realiz la reparacin.
 *
 * Comprueba si el siniestro ya existe en la base de datos. Solo compueba si una reparacin tiene
 * un siniestro asociado ya que solo puede existir un siniestro asociado a una reparacin.
 *
 * Returns:
 * %TRUE si ya existe el siniestro en la base de datos, %FALSE en otro caso.
 */
static gboolean is_new_accident(gchar *repair_code, gchar *shop_code, gchar *enterprise_code){

  gboolean result = TRUE;
  GdaCommand *command = gda_command_new();
  gchar *gchar_query = NULL;
  GdaConnection *my_connection = m_model_get_connection ();
  GdaRecordset *record_set = NULL;
  gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));

  gda_command_set_connection(command,my_connection);

  gchar_query = g_strconcat("SELECT referenciaSiniestro FROM siniestro",
			    " WHERE codigoreparacion = ", repair_code,			      
			    " AND codigoempresa = ", enterprise_code,
			    " AND codigotienda = ", shop_code,
			    NULL);

  g_print(_("\nQuery: %s\n"),gchar_query);

  m_model_command_execute (&record_set, gchar_query, my_connection,
			command,tuples,0,TRUE);

  igalia_command_free(command);

  result = !(*tuples);

  if (record_set != NULL) 
    gda_recordset_free(record_set);

  g_free(gchar_query);
  g_free(tuples);
    
  return result;

}
