
/* vehicle_model.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#define M_FIRST_ERROR_MESSAGE "Error en los datos del VEHICULO"
#define TABLE         "Vehiculo"
#define IDENTIFICATOR "numeroBastidor"
#define ATTRIBUTES    "matricula as matriculavehicle, \
    codigovehiculo as codigovehiculovehicle, \
    numeroBastidor as numerobastidorvehicle, \
    kilometros as kilometrosvehicle, \
    modelo as modelovehicle, \
    marca as marcavehicle, \
    color as colorvehicle"

#include "vehicle_model.h"

#include "util.h"



/**
 * m_vehicle_get_table:
 *
 * Devuelve un string con la tabla correspondiente al objeto de la base de datos. Si el
 * objeto se corresponde con varias tablas se devolveran los nombres separados por comas.
 *
 * Returns:
 * Cadena de caracteres con la tabla o lista de tablas a las que se refiere el objeto.
 */
gchar *m_vehicle_get_table () {
    return TABLE;
}

/**
 * m_vehicle_get_attributes:
 *
 * Funcin que obtiene los atributos de un objeto, estos atributos son los que corresponden al
 * objeto en la base de datos.
 *
 * Returns:
 * Cadena de caracteres con los atributos a los que se refiere el objeto con un alias.
 */
gchar *m_vehicle_get_attributes () {
    return ATTRIBUTES;
}

/**
 * m_vehicle_create:
 * @rs: Record set que contiene los campos de la BD.
 *
 * Crea un objeto a partir de los valores de un record set de la base de datos.
 *
 * Returns:
 * Un objeto #Vehicle con los campos rellenados con los datos de la base de datos.
 */
Vehicle *m_vehicle_create (GdaRecordset *rs) {

    gchar  *temp;
    GdaField * field;
    Vehicle * vehicle = d_vehicle_create_empty();

    field = gda_recordset_field_name(rs,"codigovehiculovehicle");

    if (field!=NULL) {
	if (!gda_field_is_null(field)) {
	    vehicle->vehicle_code = igalia_field_get_bigint_value (field);

	    gda_field_free(field);

	    field = gda_recordset_field_name(rs,"numerobastidorvehicle");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    vehicle->chassis_number = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }


	    field = gda_recordset_field_name(rs,"matriculavehicle");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    vehicle->license_number = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"kilometrosvehicle");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    vehicle->kilometres = igalia_field_get_bigint_value (field);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"modelovehicle");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    vehicle->model = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"marcavehicle");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    vehicle->make   =  g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }

	    field = gda_recordset_field_name(rs,"colorvehicle");
	    if (field!=NULL) {
		if (!gda_field_is_null(field)) {
		    temp=gda_stringify_value(NULL, 0, field);
		    vehicle->colour = g_strdup(temp);
		    g_free(temp);
		}
		gda_field_free(field);
	    }
	} else {
	    gda_field_free(field);
	}

    }

    return vehicle;
}


/**
 * m_vehicle_find_by_name:
 * @vehicle_code: Cdigo del vehculo.
 * @participant_code: Cdigo del participante del que se desean obtener los vehculos
 *
 * Busca los vehculos que cumplan la condicin. Si @cdigo_vehiculo es %NULL se buscan
 * todos los vehculos de un participante.
 *
 * Returns:
 * Lista de vehculos que cumplan la condicin.
 */
Vehicle ** m_vehicle_find_by_name (gchar *vehicle_code, gchar *participant_code) {

  Vehicle ** vehicles_list = NULL; 
  GdaCommand         * my_command;
  GdaConnection      * my_connection = m_model_get_connection ();
  GdaRecordset       * record_set = NULL;
  Vehicle * vehicle;
  gchar * gchar_query = NULL;
  gulong * tuples;
  gint * column;
  gint index;
  gint length_of_list;
  
  my_command = gda_command_new();
  gda_command_set_connection(my_command,my_connection);
    
  if (participant_code != NULL){
    gchar_query = g_strconcat("SELECT DISTINCT ",
			      "vehiculo.codigovehiculo as codigovehiculovehicle,"
			      "vehiculo.matricula as matriculavehicle,",
			      "vehiculo.numeroBastidor as numerobastidorvehicle,",
			      "vehiculo.kilometros as kilometrosvehicle,",
			      "vehiculo.modelo as modelovehicle,",
			      "vehiculo.marca as marcavehicle,",
			      "vehiculo.color as colorvehicle",
			      " FROM vehiculo,",
			      "reparacion WHERE reparacion.codigovehiculo=vehiculo.codigovehiculo AND reparacion.codigoCliente='",participant_code,"'",NULL);
    
  }else{
    gchar_query = g_strconcat("SELECT ",
			      "vehiculo.codigovehiculo as codigovehiculovehicle,"
			      "vehiculo.matricula as matriculavehicle,",
			      "vehiculo.numeroBastidor as numerobastidorvehicle,",
			      "vehiculo.kilometros as kilometrosvehicle,",
			      "vehiculo.modelo as modelovehicle,",
			      "vehiculo.marca as marcavehicle,",
			      "vehiculo.color as colorvehicle",
			      " FROM vehiculo",
			      NULL);    

    if (vehicle_code != NULL){
      gchar *gchar_query_aux = NULL;
      
      gchar_query_aux = g_strconcat(gchar_query," WHERE codigovehiculo=",vehicle_code,NULL);
      g_free(gchar_query);
      gchar_query = gchar_query_aux;
      gchar_query_aux = NULL;
    }
    
  }
  
  tuples = (gulong *) g_malloc0(sizeof(gulong));
  m_model_command_execute (&record_set, gchar_query, my_connection, my_command,tuples,0,TRUE);
  g_print(gchar_query);
  g_free(gchar_query);
  
  length_of_list = *tuples;
  column = (gint *) g_malloc0(sizeof(gint));
  
  if (record_set!=NULL) {
    if (*tuples!=0) {
      vehicles_list = (Vehicle **) g_malloc0(sizeof(Vehicle*)*(*tuples+1));
      index = gda_recordset_move_first(record_set);
      while (!gda_recordset_eof(record_set)) {
	if (index==GDA_RECORDSET_INVALID_POSITION) return NULL;
	(*column)=0;
	vehicle = m_vehicle_create (record_set);
	vehicles_list[index-1] = vehicle;
	index = gda_recordset_move_next(record_set);
      }
      vehicles_list[length_of_list]=NULL;
      
    } else {
      vehicles_list=NULL;
    }
    gda_recordset_close(record_set);
    gda_recordset_free(record_set);
  } else {
    vehicles_list=NULL;
  }
  
  igalia_command_free(my_command);
  
  g_free(tuples);
  g_free(column);
  

  return vehicles_list;
}

/**
 * m_vehicle_clear_array:
 * @vehicles_list: Lista de vehiculos a liberar.
 *
 * Libera la memoria utilizada por el array de vehculos.
 *
 */
void m_vehicle_clear_array (Vehicle **vehicles_list) {
  
    if (vehicles_list!=NULL) {
	gint index = 0;
	while (vehicles_list[index]!=NULL) {
	  d_vehicle_clear (vehicles_list[index]);
	  index++;
	}
    }

    g_free(vehicles_list);
    
    vehicles_list = NULL;
    
}

/**
 * m_vehicle_insert:
 * @vehicle: Vehculo que se desea insertar.
 * @vehicle_code_gulong: Puntero a un gulong que se utilizar para rellenar con el cdigo de vehculo obtenido
 * al insertar.
 * @command: Comando en el que se ejecuta la sentencia sql.
 * @connection: Conexin a la base de datos.
 * @update: Indica si se modifica o se inserta.
 * @exception: Objeto que indica el tipo de error que se haya producido.
 *
 * Inserta o modifica un vehculo en la base de datos dentro de una transaccin
 *
 * Returns:
 * %TRUE si se ha insertado correctamente, %FALSE en otro caso.
 */
gboolean m_vehicle_insert (Vehicle *vehicle, gulong *vehicle_code_gulong, GdaCommand * command, GdaConnection *connection, gboolean update, Exception *exception) {

  gboolean result = FALSE;

  if(vehicle != NULL) {
    gchar * query = NULL;
    gulong * tuples = (gulong *) g_malloc0(sizeof(gulong));
    GdaRecordset *record_set = NULL;
    gchar *chassis_number = d_vehicle_get_chassis_number (vehicle);
    gchar *model = d_vehicle_get_model (vehicle);
    gchar *make = d_vehicle_get_make (vehicle);
    gchar *license_number = d_vehicle_get_license_number (vehicle);
    gchar *colour = d_vehicle_get_colour (vehicle);
    gulong kms_gulong = d_vehicle_get_kilometres (vehicle);
    gchar *kms = NULL;   
       
    if (kms_gulong != ULONG_MAX)
      kms = g_strdup_printf("%lu",kms_gulong);
    else kms = g_strdup("NULL");
    
    if (license_number == NULL) license_number = g_strdup("NULL");
    else{
      gchar *license_number_aux = NULL;

      license_number_aux = g_strconcat("'",license_number,"'",NULL);
      g_free(license_number);
      license_number = license_number_aux;
      license_number_aux = NULL;
    }

    if (chassis_number == NULL) chassis_number = g_strdup("NULL");
    else{
      gchar *chassis_number_aux = NULL;

      chassis_number_aux = g_strconcat("'",chassis_number,"'",NULL);
      g_free(chassis_number);
      chassis_number = chassis_number_aux;
      chassis_number_aux = NULL;
    }

    if (model == NULL) model = g_strdup("NULL");
    else{
      gchar *model_aux = NULL;

      model_aux = g_strconcat("'",model,"'",NULL);
      g_free(model);
      model = model_aux;
      model_aux = NULL;
    }

    if (make == NULL) make = g_strdup("NULL");
    else{
      gchar *make_aux = NULL;

      make_aux = g_strconcat("'",make,"'",NULL);
      g_free(make);
      make = make_aux;
      make_aux = NULL;
    }

    if (colour == NULL) colour = g_strdup("NULL");
    else{
      gchar *colour_aux = NULL;

      colour_aux = g_strconcat("'",colour,"'",NULL);
      g_free(colour);
      colour = colour_aux;
      colour_aux = NULL;
    }

    if (!update) {
      gchar * temp_nextval = m_model_nextval_sequence ("seqvehiculo",connection);
      query = g_strconcat("INSERT INTO ",m_vehicle_get_table ()," VALUES (",
			  temp_nextval,",",
			  license_number,",",
			  chassis_number,",",
			  kms,",",
			  model,",",
			  make,",",
			  colour,")",
			  NULL);  
      g_free(temp_nextval);

    } else  {

      gchar *vehicle_code = g_strdup_printf("%lu",*vehicle_code_gulong);

      query = g_strconcat("UPDATE ",m_vehicle_get_table ()," SET ",
			  "matricula = ", license_number,",",
			  "numerobastidor = ", chassis_number,",",
			  "kilometros = ", kms,",",
			  "modelo = ", model,",",
			  "marca = ", make,",",
			  "color = ", colour," ",
			  "WHERE codigovehiculo= ",vehicle_code,
			  NULL);
      
      g_free(vehicle_code);
    }

    result = m_model_command_execute (&record_set, query, connection,command,tuples,0, FALSE);

    if (!result) d_exception_set_message (exception, M_FIRST_ERROR_MESSAGE);

    g_print(query);

    if (!update) {

      if (record_set != NULL) {
	gda_recordset_free(record_set);
	record_set = NULL;
      }            
      
      g_free (query);

      query =g_strdup("select currval('seqvehiculo')");
      
      result = result && m_model_command_execute (&record_set, query, connection,
					      command,tuples,0, TRUE);
      g_print(query);
          
      if (record_set != NULL) {
	GdaField *field = NULL;

	gda_recordset_move_first(record_set);
	
	field = gda_recordset_field_name(record_set,"currval");
	
	if (field!=NULL) {
	  if (!gda_field_is_null(field)){
	    gchar **stop = (gchar **) g_malloc0(sizeof(gchar *));
	    gchar *temp = gda_stringify_value(NULL, 0, field);
	  
	    *vehicle_code_gulong = strtoul(temp,stop,10);	
	    
	    g_free(temp);
	    g_free(stop);
	    
	  }
	  
	  d_vehicle_set_code (vehicle, *vehicle_code_gulong);
	  
	  gda_field_free(field);
	}
      }

    }

    gda_recordset_free(record_set);

    g_free(license_number);
    g_free(chassis_number);
    g_free(kms);
    g_free(model);
    g_free(make);
    g_free(colour);
    g_free(query);
    g_free(tuples);
  }

  return result;
}
