/* execute_query.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "model.h"

static gint execute_query(gchar *query);
static gchar *help(gchar *executable);

static gint execute_query(gchar *query) {
    GdaCommand    *command;
    GdaConnection *connection = m_model_get_connection ();
    GdaRecordset  *record_set = NULL;
    gboolean       correct_execute = FALSE;
    gulong         tuples;

    command = gda_command_new();
    gda_command_set_connection(command, connection);
    gda_command_set_text(command, query);

    gda_connection_begin_transaction(connection);
    record_set = gda_command_execute(command, &tuples, 0);
    correct_execute = (record_set!=NULL)
	&& (gda_connection_get_errors(connection)==NULL)
	&&(gda_connection_commit_transaction(connection)==0);

    gda_recordset_free(record_set);
    gda_command_free(command);
    m_model_destroy_connection ();

    if (correct_execute) {
	return 0;
    } else {
	return -1;
    }
}

static gchar *help(gchar *executable) {
    gchar *help_text = NULL;

    help_text = g_strconcat(_("Use: "), executable, _(" query\nReturns 0 if all OK, 0 or -1 otherwise\n"), NULL);
    return help_text;
}

int main(int argc, char *argv[]) {

    gint result;
    if (argc!=2) {
	g_print("\n%s\n", help(argv[0]));
        m_model_destroy_connection ();
	exit(-1);
    }
    m_model_init (argc, argv);

    result = execute_query(argv[1]);
    m_model_destroy_connection ();
    return result;
}
