
/* generate_todos.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "config.h"
#include "shoplogs2todos.h"
#include "centrallog2todos.h"
#include "model.h"
#include "enterprise_model.h"
#include "shop_model.h"
#include "warehouse_model.h"
#include "parameter_manager.h"
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <unistd.h>

static gchar *help(gchar *executable);
static void   create_todo_files(gchar *directory);
static gchar *get_log_filename(gchar *enterprise, gchar *shop, gchar *warehouse);
static gchar *get_identificator(gchar *enterprise, gchar *shop, gchar *warehouse);
static void signal_action(int senhal,siginfo_t * info, void * user_data);

// Generate the log file identificator with the parameters enterprise,
// shop and warehouse
static gchar *get_identificator(gchar *enterprise, gchar *shop,
				gchar *warehouse) {
    gchar *identificator = " ";
    gchar *identificator_aux = NULL;

    if (enterprise!=NULL) {
	identificator_aux = g_strconcat(identificator, enterprise, ".", NULL);
    } else {
	identificator_aux = g_strconcat(identificator, "null.", NULL);
    }
    identificator = identificator_aux;
    identificator_aux = NULL;

    if (shop!=NULL) {
	identificator_aux = g_strconcat(identificator, shop, ".", NULL);
    } else {
	identificator_aux = g_strconcat(identificator, "null.", NULL);
    }
    g_free(identificator);
    identificator = identificator_aux;
    identificator_aux = NULL;

    /* Ahora no tenemos en cuenta para nada el almacn*/
    /*XAVI
     if (warehouse!=NULL) {
	identificator_aux = g_strconcat(identificator, warehouse, ".", NULL);
    } else {
	identificator_aux = g_strconcat(identificator, "null.", NULL);
    }
    g_free(identificator);
    identificator = identificator_aux;
    identificator_aux = NULL;
    */
    return identificator;
}

// Generate the log file name with the parameters enterprise, shop
// and warehouse
static gchar *get_log_filename(gchar *enterprise, gchar *shop,
			       gchar *warehouse) {
    gchar *temp = NULL;
    gchar *path = NULL;
    gchar *path_aux = NULL;

    path_aux = g_strconcat("bd_modifications.",
			   NULL);
    g_free(temp);
    path = path_aux;
    path_aux = NULL;

    temp =  get_identificator(enterprise, shop, warehouse);

    path_aux = g_strconcat(path, temp, NULL);
    g_free(temp);
    g_free(path);
    path = path_aux;
    path_aux = NULL;

    path_aux = g_strconcat(path, "log", NULL);
    g_free(path);
    path = path_aux;
    path_aux = NULL;

    return path;

}

// Creates all the possible combinations of todo filenames according
// to the posible parameters (enterprise, shop and warehouse).
// Asume shop=warhouse
static void create_todo_files(gchar *directory) {
    Enterprise ** enterprises = m_enterprise_find_all ();
    Shop       ** shops       = m_shop_find_all ();
    gint          enterprise_index = 0;
    gint          shop_index       = 0;
    gint          file             = -1;
    gulong        enterprise_code  = ULONG_MAX;
    gulong        shop_code        = ULONG_MAX;
    gchar        *enterprise_name  = NULL;
    gchar        *shop_name        = NULL;
    gchar        *path             = NULL;
    gchar        *path_aux         = NULL;
    mode_t        mask;


    while(enterprises[enterprise_index]!=NULL) {
	enterprise_code =
	    d_participant_get_code (d_enterprise_get_participant (enterprises[enterprise_index]));
        enterprise_index++;

	shop_index = 0;
	while(shops[shop_index]!=NULL) {
	    if (enterprise_code==d_shop_get_enterprise_code (shops[shop_index])) {
		shop_code = d_participant_get_code (d_shop_get_participant (shops[shop_index]));

		    if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)) {
			enterprise_name = m_enterprise_get_name_from_code (enterprise_code);
			shop_name       = m_shop_get_name_from_code (shop_code);

			path = get_log_filename(enterprise_name, shop_name,
						NULL);
			g_print(_("\nThe log path is: %s\n"),path);


			path_aux = g_strconcat(directory, "/", path,
					       R_QUERY_EXTENSION, R_TODO_EXTENSION,
					       NULL);
			g_free(path);
			path = path_aux;
			path_aux = NULL;

			mask = umask(0);
			file = open(path, O_SYNC|O_CREAT|O_RDWR,
				    S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
			umask(mask);
			close(file);

			g_free(path);
			g_free(enterprise_name);
			g_free(shop_name);
                        path            = NULL;
			enterprise_name = NULL;
			shop_name       = NULL;
		}
	    } else {
                shop_code = ULONG_MAX;
	    }
            shop_index++;
	}
    }

}

/*
 static void create_todo_files(gchar *directory) {
    Enterprise ** enterprises = m_enterprise_find_all ();
    Shop       ** shops       = m_shop_find_all ();
    Warehouse  ** warehouses  = m_warehouse_find_all ();
    gint          enterprise_index = 0;
    gint          shop_index       = 0;
    gint          warehouse_index  = 0;
    gint          file             = -1;
    gulong        enterprise_code  = ULONG_MAX;
    gulong        shop_code        = ULONG_MAX;
    gulong        warehouse_code   = ULONG_MAX;
    gchar        *enterprise_name  = NULL;
    gchar        *shop_name        = NULL;
    gchar        *warehouse_name   = NULL;
    gchar        *path             = NULL;
    gchar        *path_aux         = NULL;
    mode_t        mask;


    while(enterprises[enterprise_index]!=NULL) {
	enterprise_code =
	    d_participant_get_code (d_enterprise_get_participant (enterprises[enterprise_index]));
        enterprise_index++;

	shop_index = 0;
	while(shops[shop_index]!=NULL) {
	    if (enterprise_code==d_shop_get_enterprise_code (shops[shop_index])) {
		shop_code = d_participant_get_code (d_shop_get_participant (shops[shop_index]));

		warehouse_index = 0;
		while(warehouses[warehouse_index]!=NULL) {
		    if (enterprise_code==d_warehouse_get_enterprise_code (warehouses[warehouse_index])) {
			warehouse_code = d_warehouse_get_code (warehouses[warehouse_index]);
		    } else {
                        warehouse_code = ULONG_MAX;
		    }

		    if ((enterprise_code!=ULONG_MAX) && (shop_code!=ULONG_MAX)
			&& (warehouse_code!=ULONG_MAX)) {
			enterprise_name = m_enterprise_get_name_from_code (enterprise_code);
			shop_name       = m_shop_get_name_from_code (shop_code);
			warehouse_name  = m_warehouse_get_name_from_code (warehouse_code);

			path = get_log_filename(enterprise_name, shop_name,
						warehouse_name);
			path_aux = g_strconcat(directory, "/", path,
					       R_QUERY_EXTENSION, R_TODO_EXTENSION,
					       NULL);
			g_free(path);
			path = path_aux;
			path_aux = NULL;

			mask = umask(0);
			file = open(path, O_SYNC|O_CREAT|O_RDWR,
				    S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH|S_IWOTH);
			umask(mask);
			close(file);

			g_free(path);
			g_free(enterprise_name);
			g_free(shop_name);
			g_free(warehouse_name);
                        path            = NULL;
			enterprise_name = NULL;
			shop_name       = NULL;
			warehouse_name  = NULL;
		    }
		    warehouse_index++;
		}
	    } else {
                shop_code = ULONG_MAX;
	    }
            shop_index++;
	}
    }

}
*/

static gchar *help(gchar *executable) {
    gchar *help_text = NULL;

    help_text = g_strconcat(_("Use: "), executable,
			    " (log_shop_directory) (todo_shop_directory) ",
			    " (log_central_filename) (todo_central_filename)\n", NULL);
    return help_text;
}

static void signal_action(int senhal,siginfo_t * info, void * user_data) {
    g_print(_("\nNormal exit with error\n"));
    m_model_destroy_connection ();
    exit(-1);
}


int main(int argc, char *argv[]) {
    struct sigaction my_sigaction;
    sigset_t * my_conjunto = (sigset_t *) g_malloc0(sizeof(sigset_t));


    sigemptyset(my_conjunto);
    sigaddset(my_conjunto,SIGSEGV);
    sigaddset(my_conjunto,SIGINT);
    sigaddset(my_conjunto,SIGILL);
    sigaddset(my_conjunto,SIGFPE);
    sigaddset(my_conjunto,SIGABRT);
//    sigprocmask(SIG_BLOCK,my_conjunto,NULL);
//    my_sigaction.sa_handler = &manejador_senhales;
    my_sigaction.sa_sigaction=signal_action;
    my_sigaction.sa_mask=(*my_conjunto);
    my_sigaction.sa_flags = SA_RESTART;
    my_sigaction.sa_restorer = NULL;

    if (!sigaction(SIGFPE,&my_sigaction,NULL)) g_print(_("\nSuccess\n"));
    sigaction(SIGILL,&my_sigaction,NULL);
    if (sigaction(SIGSEGV,&my_sigaction,NULL)==0) g_print(_("\nSuccess\n"));
    sigaction(SIGINT,&my_sigaction,NULL);


    if (argc!=5) {
	g_print("\n%s\n%lu\n", help(argv[0]), ULONG_MAX);
	m_model_destroy_connection ();
	exit(-1);
    }

    m_model_init (argc, argv);
    u_parameter_manager_init ();
    fflush(stdout);

    create_todo_files(argv[2]);
    fflush(stdout);

    r_central_log_to_todos (argv[2], argv[3]);
    r_shop_logs_to_todos (argv[1], argv[2], argv[4]);

    m_model_destroy_connection ();

    return 0;

}

