/* log2todo.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include "log2todo.h"

#define R_INSERT_CLAUSE "INSERT INTO "
#define R_UPDATE_CLAUSE "UPDATE "
#define R_DELETE_CLAUSE "DELETE FROM "
#define R_INSERT_FINISH "VALUES"
#define R_UPDATE_FINISH "SET"
#define R_DELETE_FINISH "WHERE"

#define R_SIZE_QUERY 6

static void parse_transaction(FILE *in_file, FILE *out_file);

static const gchar *spreadables[] = {
     "FAMILIA", "PRODUCTO", "MANODEOBRA", "PIEZA", "PARTICIPANTE",
     "DIRECCIONENVIO", "CLIENTE", "EMPRESA", "TIENDA", "BANCO",
     "OFICINA", "CUENTACORRIENTE", "MOVIMIENTO", "CCPARTY",
     "PROVEEDOR", "TALLER", "PARTICULAR", "CONSORCIO", "COMPANIA",
     "POLIZA", "RECIBO", "TASACION", "VEHICULO", "ALMACEN",
     "STOCK", "DOCUMENTO", "NECESITADOCUMENTO", "MEDIDA" , "PRECIOSCOSTE",
     NULL};

static gchar *get_table_name(gchar *query);
static gchar *extract_table_name(gchar *query, const gchar *begin, const gchar *finish);

static gchar *extract_table_name(gchar *query, const gchar *begin, const gchar *finish) {
    gchar * temp;
    gchar * table_name;
    gchar * result;

    temp = strstr(query+strlen(begin),finish);
    if (temp==NULL)
        temp = query+strlen(query);
    table_name = g_strndup(query+strlen(begin),temp-(query+strlen(begin)));

    result = g_strstrip(table_name);
    return result;
}

static gchar *get_table_name(gchar *query) {
    if (!strncmp(query, R_QUERY_UPDATE, R_SIZE_QUERY)) {
        return extract_table_name(query, R_UPDATE_CLAUSE, R_UPDATE_FINISH);
    } else if (!strncmp(query, R_QUERY_INSERT, R_SIZE_QUERY)) {
        return extract_table_name(query, R_INSERT_CLAUSE, R_INSERT_FINISH);
    } else if (!strncmp(query, R_QUERY_DELETE, R_SIZE_QUERY)) {
        return extract_table_name(query, R_DELETE_CLAUSE, R_DELETE_FINISH);
    }
    // A update, delete or insert query with a correct begining but
    // a not correted finish.
    exit(-1);
    m_model_destroy_connection ();
    return NULL;
}

// returns TRUE if the query have to be written in the other shops,
// FALSE otherwise.
/**
 * r_is_spreadable:
 * @query: Nombre de tabla a comprobar.
 *
 * Comprueba si una tabla es dispersable a la base de datos de todas las tiendas.
 *
 * Returns:
 * 1 si es dispersable, 0 en otro caso.
 */
gint r_is_spreadable (const gchar *query) {
    if (query!=NULL) {
        gchar *query_aux = g_strdup(query);
	gchar *query_up;
	gchar *table_name;
	gint  index = 0;

	g_strup(query_aux);
	query_up = g_strstrip(query_aux);
	table_name = get_table_name(query_up);

	g_strup(table_name);
	while(spreadables[index]!=NULL) {
	    if(!strcmp(table_name, spreadables[index])) {
		g_free(table_name);
		return 1;
	    }
	    index++;
	}
	g_free(table_name);
	g_free(query_aux);
	return 0;
    }
    return 0;
}

// returns TRUE if the argument is a update, insert, or delete query.
/**
 * r_is_query:
 * @query_const: Cadena a comprobar.
 * 
 * Comprueba si una lea de un fichero de log es una sentencia vlida.
 *
 * Returns:
 * 1 si es vlida, 0 en otro caso.
 *
 */
gint r_is_query (const gchar *query_const) {
    if (query_const!=NULL) {
	char *query = g_strdup(query_const);

	g_strup(query);
	if ((strstr(query, R_QUERY_UPDATE)
	     ||strstr(query, R_QUERY_INSERT)
	     ||strstr(query, R_QUERY_DELETE))
	    &&((strstr(query,R_BEGIN_TRANSACTION)==NULL)
	       &&(strstr(query,R_END_TRANSACTION)==NULL))) {
	    g_free(query);
	    return 1;
	}
	g_free(query);
    }
    return 0;
}

// returns TRUE if the argument is a value of correctness of a query execution
/**
 * r_is_value:
 * @query_const: Cadena de la que se quiere comprobar la correccin.
 *
 * Comprueba si una transaccin ha finalizado correctamente.
 *
 * Returns:
 * 1 si ha terminado correctamente la transaccin, 0 en otro caso.
 */
gint r_is_value (const gchar *query_const) {
    if (query_const !=NULL) {

	char *query = g_strdup(query_const);
	g_strup(query);
	if ((strstr(query, R_CORRECT)
	     ||strstr(query, R_INR_CORRECT))
	    &&((strstr(query,R_BEGIN_TRANSACTION)==NULL)
	       &&(strstr(query,R_END_TRANSACTION)==NULL))) {

	    g_free(query);
	    return 1;
	}
	g_free(query);
    }
    return 0;
}

// this function read the next line of a file, and write it
// in a concrete position in the buffer passed
// (allocating more memory if is necesary)
// return the last position where it writed
/**
 * r_read_line:
 * @buffer: Buffer a leer del fichero.
 * @position_to_write: posicin a escribir del buffer.
 * @file: Puntero al descriptor de fichero que se desea leer.
 *
 * Lee una lnea completa de un fichero y lo mete en el #buffer
 *
 * Returns:
 * -1 si se ha producido error.
 *
 */
gint r_read_line (gchar **buffer, gint position_to_write, FILE *file) {
    gint index=0;
    gchar *temp_aux = (gchar *) g_malloc(sizeof(gchar)*R_LIMIT_SIZE);
    gchar *temp =NULL;
    gchar *buffer_aux = NULL;
    gint returned = position_to_write;

    temp = fgets(temp_aux, R_LIMIT_SIZE, file);
    if(temp!=NULL) {
	while (temp[index++]!=0);

	buffer_aux = *buffer;
	*buffer = (gchar *)realloc(buffer_aux, sizeof(gchar)*(position_to_write+R_LIMIT_SIZE));

	if (*buffer==NULL) {
	    perror("replication/log2todo.c/r_read_line:realloc");
	    g_free(buffer_aux);
            m_model_destroy_connection ();
	    exit(-1);
	}

	memcpy(*buffer+position_to_write,temp,index);

	buffer_aux = *buffer;
	*buffer = (gchar *)realloc(buffer_aux, sizeof(gchar)*(position_to_write+index));

	if (*buffer==NULL) {
	    perror("replication/log2todo.c/r_read_line:realloc");
	    g_free(buffer_aux);
            m_model_destroy_connection ();
	    exit(-1);
	}

	returned = (position_to_write+index);
    }

    g_free(temp_aux);
    temp_aux = NULL;
    return returned;
}

// copy a file in other file line by line.
/**
 * r_copy:
 * @in_file_name: Fichero que se copiar.
 * @out_file_name: Fichero destino.
 *
 * Copia un fichero en otro. Si el segundo exisa se sobreescribir.
 *
 */
void r_copy (const gchar* in_file_name, const gchar *out_file_name) {
    FILE  *in_file  = fopen(in_file_name, "r");
    FILE  *out_file = fopen(out_file_name, "a");
    gchar *buffer = NULL;
    gint   old_position = 0;
    gint   new_position = 0;

    buffer = (gchar *) g_malloc0(sizeof(gchar *)*R_BLOCK_SIZE);
    new_position = r_read_line (&buffer, old_position, in_file);

    while((new_position!=old_position) && (buffer!=NULL)) {

	if (buffer!=NULL) {
	    fwrite(buffer, strlen(buffer), 1, out_file);
	}
	new_position = r_read_line (&buffer, old_position, in_file);
    }

    if(buffer!=NULL) {
	g_free(buffer);
    }

    fclose(in_file);
    fclose(out_file);
}

// copy a file in other file line by line, but only spreadable query lines.
/**
 * r_copy_spreadable:
 * @in_file_name: Fichero del que se copiarn las sentencias dispersables.
 * @out_file_name: Fichero destino.
 *
 * Copia las sentencias dispersables de un fichero a otro.
 *
 */
void r_copy_spreadable (const gchar* in_file_name, const gchar *out_file_name) {
    FILE  *in_file  = fopen(in_file_name, "r");
    FILE  *out_file = fopen(out_file_name, "a");
    gchar *buffer = NULL;
    gint   old_position = 0;
    gint   new_position = 0;

    buffer = (gchar *) g_malloc0(sizeof(gchar *)*R_BLOCK_SIZE);
    new_position = r_read_line (&buffer, old_position, in_file);

    while((new_position!=old_position) && (buffer!=NULL)) {

	if (buffer!=NULL) {
 	    if (r_is_spreadable (buffer)) { 
		fwrite(buffer, strlen(buffer), 1, out_file);
 	    } 
	}
	new_position = r_read_line (&buffer, old_position, in_file);
    }

    if(buffer!=NULL) {
	g_free(buffer);
    }

    fclose(in_file);
    fclose(out_file);
}

// Parse a transaction writted in the in_file file and
// write it in out_file if all queries of a transaction
// was perfomed and if the transaction was correctly closed.
/**
 * parse_transaction:
 * @in_file:
 * @out_file:
 *
 * Parsea una transaccin escrita en el fichero #in_file y la escribe en el
 * fichero de salida #out_file en el caso de que todas las sentencias sql
 * del fichero #in_file se hayan ejecutado correctamente.
 *
 *
 */
static void parse_transaction(FILE *in_file, FILE *out_file) {
    gint      old_position = 0;
    gint      new_position = 0;
    gint      write_position = 0;
    gchar    *buffer = (gchar *) g_malloc(sizeof(gchar)*R_LIMIT_SIZE);
    gchar    *buffer_to_print = NULL;
    gchar    *buffer_to_print_aux = NULL;
    gboolean  correct_execute = TRUE;

    new_position = r_read_line (&buffer, old_position, in_file);

    while((new_position!=old_position)
	  && !strstr(buffer,R_END_TRANSACTION)
	  && (buffer!=NULL)) {

	while((new_position!=old_position)
	      && !r_is_query (buffer)
 	      && !r_is_value (buffer)
	      && !strstr(buffer,R_END_TRANSACTION)
	      && (buffer!=NULL)) {
	    new_position = r_read_line (&buffer, old_position, in_file);
	}

	if (!strstr(buffer,R_END_TRANSACTION) &&
	    !strstr(buffer,R_BEGIN_TRANSACTION)) {
	    if (r_is_query (buffer)) {
		// Fix me!!! Ok, we insert this "if" because there are queries
		// build with this form and they cause a glib error.
		// When this queries do not exist remove this if.
		if (!strstr(buffer, "INSERT INTO FALSE")) {
		    buffer_to_print_aux = buffer_to_print;
		    buffer_to_print = (gchar *)
			realloc(buffer_to_print_aux,
			    sizeof(gchar)*(write_position+new_position));
		    memcpy(buffer_to_print+write_position,buffer,(new_position-old_position));
		    write_position += (new_position-old_position)-1;
		}
	    } else {
		if (r_is_value (buffer)) {
		    if(!strstr(buffer,R_CORRECT)) {
			//The execution within de transaction was not performed
			correct_execute = FALSE;
		    }
		}
	    }
	    new_position = r_read_line (&buffer, old_position, in_file);
	}
    }

    if (correct_execute && strstr(buffer,R_CORRECT)) { //Transaction is OK
	if(buffer_to_print!=NULL) {
	    fprintf(out_file, "%s", buffer_to_print);
	}
    }

    if (buffer!=NULL) {
	g_free(buffer);
    }

    if (buffer_to_print!=NULL) {
	g_free(buffer_to_print);
    }
}

// convert a log file y a todo file.
// Writes in the todo file all correct transaction
// that find in the log file.
/**
 * r_log_to_todo:
 * @in_file_name: Fichero de entrada del que se extraern las sentencias sql.
 * @out_file_name: Fichero "todo" destino en el que se escribir.
 *
 * Transforma un log en un fichero "todo".
 *
 * Returns:
 * 0 si ha terminado correctamente.
 *
 */
gint r_log_to_todo (const gchar *in_file_name, const gchar *out_file_name) {
    FILE   *in_file  = NULL;
    FILE   *out_file = NULL;
    gint    old_position = 0;
    gint    new_position = 0;
    gchar  *buffer = (gchar *) g_malloc(sizeof(gchar)*R_LIMIT_SIZE);
    gchar  *buffer_to_print;


    in_file  = fopen(in_file_name, "r");
    out_file = fopen(out_file_name, "w");

    if(in_file==NULL) {
	perror("replication/log2todo.c/r_log_to_todo:fopen");
	printf(_("\tTrying open: %s\n"), in_file_name);
        m_model_destroy_connection ();
	exit(-1);
    }

    if(out_file==NULL) {
	perror("replication/log2todo.c/r_log_to_todo:fopen");
	printf(_("\tTrying open: %s\n"), out_file_name);
        m_model_destroy_connection ();
	exit(-1);
    }

    do {
	new_position = r_read_line (&buffer, old_position, in_file);

	while((new_position!=old_position)
	      && !r_is_query (buffer)
	      && !strstr(buffer, R_BEGIN_TRANSACTION)
	      && (buffer!=NULL)) {

	    new_position = r_read_line (&buffer, old_position, in_file);
	}

	if (buffer!=NULL) {
	    if (strstr(buffer, R_BEGIN_TRANSACTION)) {
		parse_transaction(in_file, out_file);
	    } else {
		if (new_position!=old_position) {
		    buffer_to_print = g_strdup(buffer);
		    new_position = r_read_line (&buffer, old_position, in_file);
		    while((new_position!=old_position)
			  && (!r_is_value (buffer))
			  && (buffer!=NULL)) {
			new_position = r_read_line (&buffer, old_position, in_file);
		    }

		    if((new_position!=old_position)
		       && strstr(buffer, R_CORRECT)
		       && (buffer!=NULL)
		       && (buffer_to_print!=NULL)) {
			fprintf(out_file, "%s", buffer_to_print);
		    }
		}
	    }
	}
    } while(new_position!=old_position);

    g_free(buffer);
    fclose(in_file);
    fclose(out_file);

    return 0;
}

