
/* log2todo.h
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#ifndef _LOG_2_TODO_
#define _LOG_2_TODO_

#include "config.h"
#include "util.h"
#include "model.h"

/**
 * R_LIMIT_SIZE:
 *
 * Tamao lmite para una lnea.
 *
 */
#define R_LIMIT_SIZE     10240
/**
 * R_BLOCK_SIZE:
 *
 * Tamao de un bloque que se copiar de un fichero a otro.
 *
 */
#define R_BLOCK_SIZE     4*1024
/**
 * R_CORRECT:
 *
 * Texto que indica que una transaccin en el fichero ha terminado correctamente.
 *
 */
#define R_CORRECT           "1"
/**
 * R_INR_CORRECT:
 *
 * Texto que indica que una transaccin en el fichero ha terminado de modo incorrecto.
 *
 */
#define R_INR_CORRECT         "0"
/**
 * R_BEGIN_TRANSACTION:
 *
 * Texto que indica el comienzo de una transaccin en el fichero.
 *
 */
#define R_BEGIN_TRANSACTION "Begin Transaction"
/**
 * R_END_TRANSACTION:
 *
 * Texto que indica la finalizacin de una transaccin.
 *
 */
#define R_END_TRANSACTION   "End Transaction"
/**
 * R_QUERY_UPDATE:
 *
 * Texto que indica el comienzo de una sentencia UPDATE (de actualizacin).
 *
 */
#define R_QUERY_UPDATE      "UPDATE"
/**
 * R_QUERY_INSERT:
 *
 * Texto que indica el comienzo de una sentencia INSERT (de insercin).
 *
 */
#define R_QUERY_INSERT      "INSERT"
/**
 * R_QUERY_DELETE:
 *
 * Texto que indica el comienzo de una sentencia DELETE (de borrado).
 *
 */
#define R_QUERY_DELETE      "DELETE"

gint r_log_to_todo (const gchar *in_file_name, const gchar *out_file_name);
gint r_read_line (gchar **buffer, gint position_to_write, FILE *file);
void r_copy (const gchar* in_file_name, const gchar *out_file_name);
void r_copy_spreadable (const gchar* in_file_name, const gchar *out_file_name);
gint r_is_query (const gchar *query_const);
gint r_is_value (const gchar *query_const);
gint r_is_spreadable (const gchar *query);

#endif


