
/* parameter_manager.c
 *
 * Copyright (C) 2001, 2002 Igalia, S.L..
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of version 2 of the GNU General Public
 * License as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 * Author: Igalia S.L.
 */

#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <gda-client.h>
#include <gnome.h>
#include <gtk/gtk.h>
#include <glib.h>
#include "parameter_manager.h"

#define U_PARAMETER_PATH "/parameters.xml"

static GHashTable *hash_table = NULL;

/**
 * u_parameter_manager_get_parameter: 
 * @key: Clave de la cual se desea obtener el valor en el fichero de configuracin en xml.
 *
 * Obtiene un valor a partir de una clave en el fichero de configuracin del programa.
 * El formato en dicho fichero ser: <#key>Valor a obtener</#key>.
 *
 * Returns:
 * Una cadena de caracteres con el valor en el xml.
 */
gchar * u_parameter_manager_get_parameter (gchar *key) {
    if (hash_table == NULL) {
	u_parameter_manager_init ();
    }
    return g_strdup(g_hash_table_lookup(hash_table, key));
}

/**
 * u_parameter_manager_init: 
 *
 * Carga en memoria el rbol xml del fichero de configuracin de la
 * aplicacin. El nombre del fichero es "parameters.xml". El fichero de configuracin
 * se guarda en memoria en una tabla #hash.
 *
 */
void u_parameter_manager_init (void) {
    if (hash_table == NULL) {
	xmlDocPtr  doc;
	xmlNodePtr cur;
	xmlNodePtr items;
	gchar     *content;
	gchar     *name;
	gchar     *path = g_strconcat(DIR_CONFIG, U_PARAMETER_PATH, NULL);
	if(u_exists (path)==0) {
	    g_print(_("Runing u_parameter_manager_init (%s)"), path);

            /*Parsea el fichero xml.*/
	    doc = xmlParseFile(path);
            /*Coloca el cursor en el elemento raz*/
	    cur = xmlDocGetRootElement(doc);
	    hash_table = g_hash_table_new(g_str_hash, g_str_equal);

            /*Busca el elemento con nombre "parameters"*/
	    while ((cur!=NULL)&&(strcmp(cur->name, "parameters"))) {
		cur = cur ->  childs;
	    }

            /*Recorre cada parmetro y lo introduce en la tabla hash.*/
	    while (cur !=NULL) {
		items = cur -> childs;
		while ((items != NULL)) {
		    g_print(".");
		    content = g_strstrip(xmlNodeGetContent ((xmlNodePtr) items));
		    name = g_strstrip(g_strdup(items->name));
		    g_hash_table_insert(hash_table, name, content);
		    items = items -> next;
		}
		cur = cur -> next;
	    }
	    g_free(path);

	    xmlFreeDoc(doc);

	    g_print(_(" OK\n"));
	}
    }
}


